<?php

namespace App\Console\Commands;

use App\Helpers\Helper;
use App\Model\PropertyHistory;
use App\Model\PropertyHouseNumber;
use App\Model\UserNotification;
use App\Model\UserOutcode;
use App\Model\UserValuation;
use Carbon\Carbon;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;

class PropertyUpdates extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'cron:property-updates';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Property updates notification about subscribed outcodes.';

    /**
     * Create a new command instance.
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        $now = Carbon::now();
        // $now = Carbon::yesterday();
        // $now = Carbon::createFromFormat('d/m/Y H:i:s', '27/09/2020 07:20:19');
        Log::channel('cron')->info('Cron start: Property Updates @'.$now->format('Y-m-d H:i:s a'));

        $dt = $now->copy()->startOfDay();
        if ('am' == $now->format('a')) {
            $startTime = $dt->copy()->subHours(12);
            $endTime = $dt->copy()->subSeconds(1);
        } else {
            $startTime = $dt->copy();
            $endTime = $dt->copy()->addHours(12)->subSeconds(1);
        }

        // Log::channel('cron')->info('start: '.$startTime->format('l jS \\of F Y h:i:s A').'// end: '.$endTime->format('l jS \\of F Y h:i:s A'));

        $properties = PropertyHistory::whereBetween('updated_at', [$startTime, $endTime])->with('propertyHouseNumber')->get();
        foreach ($properties as $key => $property) {
            $outcodeId = $property->postcode->outcode_id;
            $houseNumber = PropertyHouseNumber::where('property_history_id', $property->id)->first();
            $notificationStatus = Helper::getNotificationStatus($property->id);

            if ($houseNumber) {
                // Notifucation for the users who have subscribed the outcode
                $userOutcodes = UserOutcode::select(['user_id'])->where('outcode_id', $outcodeId)->whereStatus(1)->get();
                if ($userOutcodes->count()) {
                    foreach ($userOutcodes as $index => $userOutcode) {
                        UserNotification::create([
                            'property_history_id' => $property->id,
                            'house_number_id' => $houseNumber->house_number_id,
                            'notify_status' => $notificationStatus,
                            'outcode_id' => $outcodeId,
                            'type' => 1,
                            'user_id' => $userOutcode->user_id,
                        ]);

                        // Agency Notification //here//pending
                    }
                }

                // Notification for the users who have valuated the proeprty by address
                $userValuations = UserValuation::select(['user_id', 'id'])->where('address', $property->address)->get();
                if ($userValuations->count()) {
                    foreach ($userValuations as $index => $valuation) {
                        UserNotification::create([
                            'property_history_id' => $property->id,
                            'house_number_id' => $houseNumber->house_number_id,
                            'notify_status' => $notificationStatus,
                            'outcode_id' => $outcodeId,
                            'valuation_id' => $valuation->id,
                            'type' => 3,
                            'user_id' => $valuation->user_id,
                        ]);
                    }
                }
            }
        }

        $this->info('The property updates notification successfully executed!');

        return true;
    }
}
