<?php

namespace App\Helpers;

use App\Model\Agency;
use App\Model\HouseNumber;
use App\Model\ProbatePropertyFlag;
use App\Model\PropertyComment;
use App\Model\PropertyFlag;
use App\Model\PropertyHistory;
use App\Model\PropertyPriceHistory;
use App\Model\ProbatePropertyCampaign;
use App\Model\UserNotification;
use App\Model\UserOutcode;
use Carbon\Carbon;
use DB;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Session;

class Helper
{
    //Decypt
    public static function decryptData($data)
    {
        return Crypt::decryptString($data);
    }

    //Encrypt
    public static function encryptData($data)
    {
        return Crypt::encryptString($data);
    }

    public static function propertyFlag($house_number_id)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();
        if ($exist_data) {
            return $exist_data['flag'];
        }

        return 0;
    }

    public static function probatePropertyStatus($probate_id)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $exist_data = ProbatePropertyFlag::where('user_id', $user_id)->where('probate_id', $probate_id)->first();
        if ($exist_data) {
            return $exist_data['flag'];
        }

        return 0;
    }

    public static function propertyComment($house_number_id)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $exist_data = PropertyComment::where('user_id', $user_id)->where('house_number_id', $house_number_id)->orderBy('id', 'desc')->get();
        if ($exist_data) {
            return $exist_data;
        }

        return [];
    }

    public static function selectTrigger($item, $properties)
    {
        $listed_date = $item->listed_date;
//        dd($listed_date);
        $status = [
            1 => 'New to Market',
            2 => 'For sale',
            3 => 'Sold',
            4 => 'Withdrawn',
            5 => 'Price drop',
            6 => 'Fallen Through',
        ];
        $triggers = [
            0 => 'General Letters',
            1 => 'New to Market',
            2 => 'For sale',
            3 => 'Sold',
            4 => 'Withdrawn',
            5 => 'Price drop',
            6 => 'Fallen Through',
            7 => '8 Week',
            8 => '16 Week',
            9 => '22 Week',
        ];
        $select = [];

        if (count($triggers) > 0) {
            for ($i = 0; $i <= 6; ++$i) {
                $class = '';
                foreach ($properties as $property) {
                    if ($i == $property->for_sale && '' == $class) {
                        $class = 'style="font-weight:bolder"';
                    }
                }
                $select[] = '<option value="'.$i.'" '.$class.'>'.$triggers[$i].'</option>';
            }
            for ($i = 7; $i <= 9; ++$i) {
                $class = '';
                foreach ($properties as $property) {
                    if (in_array($property->for_sale, [1, 2, 5, 6], true)) {
                        $event_date = Carbon::parse($listed_date);
                        $diff = $event_date->diffInWeeks(Carbon::now());

                        if (7 == $i && $diff >= 8 && $diff < 16) {
                            $class = 'style="font-weight:bolder"';
                        } elseif (8 == $i && $diff >= 16 && $diff < 22) {
                            $class = 'style="font-weight:bolder"';
                        } elseif (9 == $i && $diff >= 22) {
                            $class = 'style="font-weight:bolder"';
                        } else {
                            $class = '';
                        }
                    }
                }
                $select[] = '<option value="'.$i.'" '.$class.'>'.$triggers[$i].'</option>';
            }
//            foreach ($triggers as $key=>$trigger) {
            //                $class='';
            //                foreach ($properties as $property) {
            //
            ////                    dump('basic == '.$class);
            //                    if (in_array($property->for_sale, [1, 2, 5, 6], true)) {
            //                        $event_date = Carbon::parse($property->listed_date);
            //                        $diff = $event_date->diffInWeeks(Carbon::now());
            //                        if ($diff >= 8 && $diff < 16) {
            //                            $class='style="font-weight:bolder"';
            //                        } elseif ($diff >= 16 && $diff < 22) {
            //                            $class='style="font-weight:bolder"';
            //                        } elseif ($diff >= 22) {
            //                            $class='style="font-weight:bolder"';
            //                        } else {
            //                            $class='';
            //                        }
            //                    }
            ////                    dump('date == '.$class);
            //                }
            //                $select[] = '<option value="' . $key . '" '.$class.'>' . $trigger . '</option>';
            //            }
        }
//        dd($select);
        $selects = '<select name="trigger['.$item->id.']" class="form-control select-trigger"><option value="">Select Trigger</option>';
        $selects .= implode('', $select);
        $selects .= '</select>';
        $selects .= '<span class="error" style="color: #ff0000"></span>';

        return $selects;
    }

    public static function editButton($path)
    {
        $button = '';
        if ('' != $path) {
            $button = '<a class="btn btn-primary btn-sm mr-2" href="'.$path.'">Edit</a>';
        }

        return $button;
    }
    public static function viewButton($path)
    {
        $button = '';
        if ('' != $path) {
            $button = '<a class="btn btn-info btn-sm mr-2" href="'.$path.'">View</a>';
        }

        return $button;
    }
   

    public static function deleteButton($id)
    {
        $button = '';
        if ('' != $id) {
            $button = '<a onclick="ajaxDelete('.$id.')" class="btn btn-danger btn-sm mr-2" data-toggle="confirmation" data-original-title="Are you sure ?">Delete</a>';
        }

        return $button;
    }

    // UserNotification
    public static function notifications()
    {
        $property = [];
        $recent_valuation = [];
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $user_outcode = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.user_id', $user_id)->where('user_outcode.status', 1)->where('outcode.archive', 0)->pluck('user_outcode.outcode_id');

        // property notification
        $property = UserNotification::join('property_history', 'property_history.id', 'user_notification.property_history_id')
            ->join('outcode', 'outcode.id', 'user_notification.outcode_id')
            ->where('user_notification.type', 1)
            ->where('user_notification.user_id', $user_id)
            ->whereIn('outcode.id', $user_outcode)
            ->where('user_notification.viewed', 0)
            ->select(DB::raw("GROUP_CONCAT(DISTINCT outcode.outcode SEPARATOR ', ') AS outcode_name"), 'user_notification.type', 'user_notification.notify_status', DB::raw('count(user_notification.property_history_id) as prop_count'))
            ->groupBy('user_notification.notify_status', 'user_notification.type')
            ->get()
        ;

        // recent valuation notification
        $recent_valuation = UserNotification::join('user_valuations', 'user_valuations.id', 'user_notification.valuation_id')
            ->join('outcode', 'outcode.id', 'user_notification.outcode_id')
            ->where('user_notification.type', 3)
            ->select('user_valuations.address', 'user_notification.*')
            ->where('user_notification.user_id', $user_id)
            ->whereIn('outcode.id', $user_outcode)
            ->where('user_notification.viewed', 0)
            ->get()
        ;

        //agent status notification
        $agent_status = UserNotification::join('property_history', 'property_history.id', 'user_notification.property_history_id')
            ->join('agency', 'agency.id', 'user_notification.agent_id')
            ->join('outcode', 'outcode.id', 'user_notification.outcode_id')
            ->whereIn('outcode.id', $user_outcode)
            ->where('user_notification.type', 4)
            ->where('user_notification.user_id', $user_id)
            ->where('user_notification.viewed', 0)
            ->select('property_history.address', 'user_notification.*', 'agency.name')
            ->get()
        ;

        $recent_count = count($recent_valuation);
        $prop_count = count($property);
        $agent_status_count = count($agent_status);

        $count = $recent_count + $prop_count + $agent_status_count;
        $data['recent_valuation'] = $recent_valuation;
        $data['property'] = $property;
        $data['agent_status'] = $agent_status;
        $data['count'] = $count;

        return $data;
    }

    //property status
    public static function propertyStatus($house_number_id)
    {
        // $activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')->join("property_house_number", "property_history.id", "property_house_number.property_history_id")
        //     ->join("house_number", "house_number.id", "property_house_number.house_number_id")
        //     ->where('house_number_id', $house_number_id)
        //     ->select('property_price_history.property_history_id')
        //     // ->select('property_price_history.type','property_price_history.event_date')
        //     ->groupBy('property_price_history.property_history_id')
        //     ->get();
        // // dd($activity);
        // $status = PropertyPriceHistory::whereIn('property_price_history.id', $activity)->orderBy('property_price_history.event_date', 'desc')
        //     ->pluck('type');

        // if ($status) {
        //     $i = 1;
        //     foreach ($status as $item) {
        //         //check the type string have price keyword
        //         $exist = 0;
        //         if (strpos($item, 'Price') !== false) {
        //             return "Forsale";
        //         } else if ($item == "First listed" || $item == "Last sold") {
        //             return "Forsale";
        //         } else if ($item == "Sold") {
        //             return "Sold";
        //         } else if (count($status) == $i) {
        //             return "Withdrawn";
        //         }
        //         $i++;
        //     }
        // } else {
        //     $item = HouseNumber::where('id', $house_number_id)->first();
        //     if ($item['latest_status'] == 1) {
        //         return "Forsale";
        //     } elseif ($item['latest_status'] == 2) {
        //         return "Sold";
        //     } else {
        //         return "Withdrawn";
        //     }
        // }

        $item = HouseNumber::where('id', $house_number_id)->first();

        if (1 == $item['latest_status']) {
            return 'For Sale';
        }
        if (2 == $item['latest_status']) {
            return 'Sold';
        }
        if (3 == $item['latest_status']) {
            return 'Withdrawn';
        }
        if (6 == $item['latest_status']) {
            return 'STC';
        }

        return 'Offer';
    }

    //property status
    public static function propertyHistoryStatus($history_id)
    {
        $activity = PropertyPriceHistory::where('property_history_id', $history_id)
            ->pluck('property_price_history.id')
        ;

        $status = PropertyPriceHistory::whereIn('property_price_history.id', $activity)->orderBy('property_price_history.event_date', 'desc')
            ->orderBy('property_price_history.id', 'desc')
            ->pluck('type')
        ;

        if ($status) {
            $i = 1;
            foreach ($status as $item) {
                //check the type string have price keyword
                $exist = 0;
                // stc -> stc
                // under offer ->under offer
                // stc removed  ->forsale
                // offer removed ->forsale
                if ('offer removed' == strtolower($item)) {
                    return 'Forsale';
                }
                if ('stc removed' == strtolower($item)) {
                    return 'Forsale';
                }
                if ('stc' == strtolower($item)) {
                    return 'STC';
                }
                if ('offer' == strtolower($item)) {
                    return 'Offer';
                }
                if (false !== strpos($item, 'Price')) {
                    return 'Forsale';
                }
                if ('First listed' == $item || 'Last sold' == $item) {
                    return 'Forsale';
                }
                if ('Sold' == $item) {
                    return 'Sold';
                }
                if (count($status) == $i) {
                    return 'Withdrawn';
                }
                ++$i;
            }
        } else {
            $item = PropertyHistory::where('id', $history_id)->first();
            if (1 == $item['property_current_status']) {
                return 'Forsale';
            }
            if (2 == $item['property_current_status']) {
                return 'Sold';
            }
            if (3 == $item['property_current_status']) {
                return 'Withdrawn';
            }
            if (6 == $item['property_current_status']) {
                return 'STC';
            }

            return 'Offer';
        }
    }

    //property url
    public static function agentUrl($id)
    {
        if ($id) {
            return Agency::where('id', $id)->first();
        }

        return '';
    }

    // house number agencies
    public static function agencyName($house_number_id)
    {
        $agencyName = PropertyHistory::join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
            ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
            ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
            ->whereNull('property_house_number.deleted_at')
            ->where('house_number.id', $house_number_id)
            ->select(DB::raw("GROUP_CONCAT(DISTINCT agency.name SEPARATOR ', ') AS agency_name"))
            ->pluck('agency_name')
        ;
        if ($agencyName) {
            return $agencyName[0];
        }

        return '';
    }

    public static function getNotificationStatus($historyId)
    {
        $propertyHistory = PropertyHistory::find($historyId);
        $status = $propertyHistory->property_current_status;

        $priceHistory = PropertyPriceHistory::where('property_history_id', $historyId)
            ->getQuery()
            ->select(['*', DB::raw('coalesce(`price` - @last, 0) as diff'), DB::raw('@last := `price`')])
            ->get()
            ->last()
        ;

        if ($priceHistory) {
            if (0 != $priceHistory->diff) {
                $status = 4;
            }
        }

        return $status;
    }

    public static function getLastWords($text, $numWords = 2)
    {
        $nonWordChars = ':;,.?![](){}*';
        $result = '';
        $words = explode(' ', $text);
        $wordCount = count($words);
        if ($numWords > $wordCount) {
            $numWords = $wordCount;
        }
        for ($w = $numWords; $w > 0; --$w) {
            if (!empty($result)) {
                $result .= ' ';
            }
            $result .= trim($words[$wordCount - $w], $nonWordChars);
        }

        return $result;
    }

    public static function parseProbateAddress($str = '', $name = true, $outact = true, $sep = "\n")
    {
        $fl = false;
        if (false !== strpos($str, 'St.')) {
            $str = str_replace('St.', '+St+', $str);
            $fl = true;
        }

        if (false !== strpos($str, ';')) {
            $str = explode(';', $str)[0];
        }

        $main = explode('.', $str)[0];
        $rest = '';

        if (false !== strpos($str, '.')) {
            $rest = str_replace($main.'.', '', $str);
        }

        $exe = explode('(', $rest);
        $exe = end($exe);
        $exe = substr($exe, 0, strpos($exe, ')'));
        $exe = str_replace('.', '', $exe);

        $out = explode(',', $main);
        $out = end($out);

        if ($outact && str_word_count($out) >= 3) {
            $oc = self::getLastWords($out);
            $main = str_replace($oc, ', '.$oc, $main);
        }

        $res = '';
        if ($name && $exe) {
            $res .= str_replace('.', '', $exe).$sep;
        }
        $res .= str_replace(',', $sep, $main);

        if ($fl) {
            $res = str_replace('+St+', 'St.', $res);
        }

        return str_replace($sep.' ', $sep, preg_replace('/\s+/', ' ', $res));
    }


    public static function existCampaign($probatePropertyid)
    {
    
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
         
        if($user_id && $probatePropertyid)
        {
            $exist_data = \App\Model\ProbatePropertyCampaign::where('user_id', $user_id)->where('property_id', $probatePropertyid)->first();
            return $exist_data;
        }
    }

}
