<?php

namespace App\Http\Controllers\Admin;

use App\Helpers\Helper;
use App\Http\Controllers\Controller;
use App\Model\Outcode;
use App\Model\User;
use App\Model\UserOutcode;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Session;
use Redirect;
use Validator;

class AgentController extends Controller
{
    public function AllAgent()
    {
        $page_title = 'All Agents';
        $query = User::select('id', 'first_name', 'last_name', 'email', 'contact_number', 'is_active')->where('user_type', 2);
        // dd($query->get());

        return view('admin.agent', compact('page_title'));
    }

    public function allAgentArray(Request $request)
    {
        $authUser = json_decode(Session::get(config('propai.authSessionKey')));
        $user_id = $id = $authUser->id;
        $user_type = $id = $authUser->user_type;

        $query = User::select('id', 'first_name', 'last_name', 'email', 'contact_number', 'is_active', 'enable_export')->where('user_type', 2)->where('organisation_id', $authUser->organisation_id);

        $primaryKey = 'id';

        $columns = [
            0 => 'id',
            1 => 'first_name',
            2 => 'last_name',
            3 => 'email',
            4 => 'contact_number',
            5 => 'is_active',
            6 => 'enable_export',
            7 => 'action',
        ];
        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;
        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $list = $query->where(function ($q) use ($search) {
                $q->Where('users.first_name', 'like', '%'.$search.'%')
                    ->orWhere('users.last_name', 'like', '%'.$search.'%')
                    ->orWhere('users.email', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
                ->offset($start)
                ->limit($limit)
                ->get()
            ;

            $totalFiltered = $query->Where('users.first_name', 'like', '%'.$search.'%')
                ->orWhere('users.last_name', 'like', '%'.$search.'%')
                ->orWhere('users.email', 'like', '%'.$search.'%')
                ->count()
            ;
        }
        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                //$url = route('agentDetails',['agent_id'=>$encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->id;
                $nestedData['sl'] = $i;
                $nestedData['first_name'] = $item->first_name;
                $nestedData['last_name'] = $item->last_name;
                $nestedData['email'] = $item->email;
                $nestedData['contact_number'] = $item->contact_number;
                $nestedData['flag'] = $item->is_active;
                $nestedData['enable_export'] = 0 == $item->enable_export ? '<span class="font-weight-bold btn btn-danger label-inline export-tog" data-id="'.$item->id.'" data-val="'.$item->enable_export.'"><i class="fas fa-toggle-off"></i></span>' : '<span class="font-weight-bold btn btn-success label-inline export-tog" data-id="'.$item->id.'" data-val="'.$item->enable_export.'"><i class="fas fa-toggle-on"></i></span>';
                $nestedData['action'] = $item->id;
                $nestedData['login_as'] = '<button type="button" data-id="'.$item->id.'" class="btn btn-danger font-weight-bold login_as">
                <i class="flaticon2-user "></i> </button>';

                $dataArr[] = $nestedData;
            }
        }
        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    public function addAgent()
    {
        $outcode = Outcode::get();

        return view('admin.add-agent', compact('outcode'));
    }

    public function saveAgent(Request $request)
    {
        // dd($request);
        $authUser = json_decode(Session::get(config('propai.authSessionKey')));

        $id = $authUser->id;

        $rules = [
            'first_name' => 'required',
            'last_name' => 'required',
            'contact_number' => ['required', 'regex:/^(((\+44\s?\d{4}|\(?0\d{4}\)?)\s?\d{3}\s?\d{3})|((\+44\s?\d{3}|\(?0\d{3}\)?)\s?\d{3}\s?\d{4})|((\+44\s?\d{2}|\(?0\d{2}\)?)\s?\d{4}\s?\d{4}))(\s?\#(\d{4}|\d{3}))?$/'],
            'company_name' => 'required',
            'address' => 'required',
            'email' => 'required|email|unique:users,email',
            'password' => 'required|min:6',
            'out_code' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);
        if ($validator->fails()) {
            return Redirect::back()->with('msg_error', $validator->errors()->first())->withInput();
        }
        $agent = new User();

        $agent->first_name = ucfirst($request->first_name);
        $agent->last_name = ucfirst($request->last_name);
        $agent->contact_number = $request->contact_number;
        $agent->email = $request->email;
        $agent->company_name = $request->company_name;
        $agent->address = $request->address;
        $agent->is_active = 2;
        $agent->password = Hash::make($request->password);
        $agent->user_type = 2;
        $agent->organisation_id = $authUser->organisation_id;
        $agent->enable_export = isset($request->enable_export) ? 1 : 0;
        if ($agent->save()) {
            $user_id = $agent->id;
            if ($request->out_code) {
                foreach ($request->out_code as $key => $value) {
                    $userOutcode = new UserOutcode();
                    $userOutcode->user_id = $user_id;
                    $userOutcode->outcode_id = $value;
                    $userOutcode->status = 1;
                    $userOutcode->save();
                }
            }

            return Redirect::route('allAgent')->with('msg_success', 'Agent has been created successfully...!');
        }

        return $this->simpleReturn('error', 'Something Went Wrong...!');
    }

    //property flag change
    public function flagChange(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        foreach ($request->property as $agent_id) {
            $exist_data = User::where('id', $agent_id)->first();

            if ($exist_data) { //update
                $exist_data->is_active = $request->flag;
                $exist_data->save();
            } else { //insert
                return $this->simpleReturn('error', 'Something went wrong...!');
            }
        }

        return $this->simpleReturn('success', 'Successfully updated status...!');
    }

    public function deleteAgent(Request $request)
    {
        foreach ($request->property as $agent_id) {
            $exist_data = User::where('id', $agent_id)->first();

            if ($exist_data) {
                $exist_data->delete();
            } else { //insert
                return $this->simpleReturn('error', 'Something went wrong...!');
            }
        }

        return $this->simpleReturn('success', 'Successfully agent deleted...!');
    }

    public function editAgent($id)
    {
        $page_title = 'Edit Agent';
        $authUser = json_decode(Session::get(config('propai.authSessionKey')));

        if ($id) {
            $agent = User::find($id);

            if ($agent) {
                if ($agent->organisation_id != $authUser->organisation_id) {
                    return redirect()->back()->with('msg_error', 'Cannot find agent.');
                }

                $outcode = Outcode::get();
                $user_outcode = UserOutcode::where('user_id', $id)->where('status', 1)->pluck('outcode_id')->toArray();

                return view('admin.edit-agent', compact('page_title', 'agent', 'outcode', 'user_outcode'));
            }

            return redirect()->back()->with('msg_error', 'Cannot find agent.');
        }

        return $this->simpleReturn('error', 'Something Went Wrong...!');
    }

    public function updateAgent(Request $request, $id)
    {
        $rules = [
            'first_name' => 'required',
            'last_name' => 'required',
            'contact_number' => ['required', 'regex:/^(((\+44\s?\d{4}|\(?0\d{4}\)?)\s?\d{3}\s?\d{3})|((\+44\s?\d{3}|\(?0\d{3}\)?)\s?\d{3}\s?\d{4})|((\+44\s?\d{2}|\(?0\d{2}\)?)\s?\d{4}\s?\d{4}))(\s?\#(\d{4}|\d{3}))?$/'],
            'email' => 'required|email',
            'company_name' => 'required',
            'address' => 'required',
            'password' => 'sometimes|nullable',
            'out_code' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);
        if ($validator->fails()) {
            return Redirect::back()->with('msg_error', $validator->errors()->first())->withInput();
        }
        if ($id) {
            $authUser = json_decode(Session::get(config('propai.authSessionKey')));

            $agent = User::find($id);
            if ($agent) {
                if ($agent->organisation_id != $authUser->organisation_id) {
                    return redirect()->back()->with('msg_error', 'Cannot find agent.');
                }

                $agent->first_name = $request->first_name;
                $agent->last_name = $request->last_name;
                $agent->contact_number = $request->contact_number;
                $agent->email = $request->email;
                $agent->company_name = $request->company_name;
                $agent->address = $request->address;
                $agent->enable_export = isset($request->enable_export) ? 1 : 0;
                if ($request->has('password') && '' != $request->password) {
                    $agent->password = Hash::make($request->password);
                }

                if ($agent->save()) {
                    $user_id = $agent->id;
                    if ($request->out_code) {
                        foreach ($request->out_code as $key => $value) {
                            $userOutcode = UserOutcode::where('user_id', $user_id)->where('status', 1)->where('outcode_id', $value)->first();
                            if (!$userOutcode) {
                                $userOutcode = new UserOutcode();
                                $userOutcode->user_id = $user_id;
                                $userOutcode->outcode_id = $value;
                                $userOutcode->status = 1;
                                $userOutcode->save();
                            }
                        }
                        $user_outcode = UserOutcode::where('user_id', $user_id)->where('status', 1)->pluck('outcode_id')->toArray();
                        foreach ($user_outcode as $key => $value) {
                            if (!in_array($value, $request->out_code)) {
                                UserOutcode::where('user_id', $user_id)->where('outcode_id', $value)->delete();
                            }
                        }
                    }

                    return Redirect::route('allAgent')->with('msg_success', 'Agent has been updated successfully...!');
                }

                return $this->simpleReturn('error', 'Something Went Wrong...!');
            }

            return $this->simpleReturn('error', 'Something Went Wrong...!');
        }

        return $this->simpleReturn('error', 'Something Went Wrong...!');
    }

    public function outcodeApproval()
    {
        $page_title = 'Pending Approvals';

        return view('admin.outcode-approval', compact('page_title'));
    }

    public function outcodeApprovalList(Request $request)
    {
        $authUser = json_decode(Session::get(config('propai.authSessionKey')));
        $user_id = $id = $authUser->id;
        $user_type = $id = $authUser->user_type;
        $orgId = $authUser->organisation_id;

        $query = UserOutCode::where('status', 0)->whereHas('agent', function ($qry) use ($orgId) {
            $qry->where('organisation_id', $orgId);
        });
        $primaryKey = 'id';

        $columns = [
            0 => 'id',
            1 => 'name',
            2 => 'email',
            3 => 'outcode',
            4 => 'date',
            5 => 'action',
        ];

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;
        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->has('agent')->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $list = $query->where(function ($q) use ($search) {
                $q->whereHas('outcode', function ($qry) use ($search) {
                    $qry->where('outcode', 'like', '%'.$search.'%');
                })->orWhereHas('agent', function ($qry) use ($search) {
                    $qry->where('first_name', 'like', '%'.$search.'%');
                });
            })->orderBy($order, $dir)
                ->offset($start)
                ->limit($limit)
                ->get()
            ;

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->whereHas('outcode', function ($qry) use ($search) {
                    $qry->where('outcode', 'like', '%'.$search.'%');
                })->orWhereHas('agent', function ($qry) use ($search) {
                    $qry->where('first_name', 'like', '%'.$search.'%')
                        ->orWhere('last_name', 'like', '%'.$search.'%')
                    ;
                });
            })->count();
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                //$url = route('agentDetails',['agent_id'=>$encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->id;
                $nestedData['sl'] = $i;
                $nestedData['name'] = $item->agent->first_name.' '.$item->agent->last_name;
                $nestedData['email'] = $item->agent->email;
                $nestedData['outcode'] = $item->outcode->outcode;
                $nestedData['date'] = $item->created_at ? Carbon::parse($item->created_at)->format('d/m/Y') : '';
                $nestedData['action'] = $item->id;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    public function outcodeApprovalDelete(Request $request)
    {
        $approvalRequest = UserOutCode::find($request->id);

        if ($approvalRequest) {
            $approvalRequest->delete();

            return $this->simpleReturn('success', 'Outcode request deleted.');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    public function outcodeApprovalsDelete(Request $request)
    {
        foreach ($request->ids as $id) {
            $exist_data = UserOutCode::find($id);

            if ($exist_data) {
                $exist_data->delete();
            } else {
                return $this->simpleReturn('error', 'Something went wrong...!');
            }
        }

        return $this->simpleReturn('success', 'Outcode requests deleted.');
    }

    public function outcodeApprovalApprove(Request $request)
    {
        $approvalRequest = UserOutCode::find($request->id);

        if ($approvalRequest) {
            $approvalRequest->status = 1;
            $approvalRequest->save();

            return $this->simpleReturn('success', 'Outcode request approved.');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    public function outcodeApprovalsApprove(Request $request)
    {
        foreach ($request->ids as $id) {
            $exist_data = UserOutCode::find($id);

            if ($exist_data) {
                $exist_data->status = 1;
                $exist_data->save();
            } else {
                return $this->simpleReturn('error', 'Something went wrong...!');
            }
        }

        return $this->simpleReturn('success', 'Outcode requests deleted.');
    }

    public function exportstatus(Request $request)
    {
        foreach ($request->property as $agent_id) {
            $exist_data = User::where('id', $agent_id)->first();

            if ($exist_data) {
                $exist_data->enable_export = $request->status;
                $exist_data->save();
            } else { //insert
                return $this->simpleReturn('error', 'Something went wrong...!');
            }
        }

        return $this->simpleReturn('success', 'Successfully agent updated...!');
    }
}
