<?php

// <summary>
// File : CampaignController
// File Type : Controller
// Creator : Susmi Sidharthan
// Email: susmi@designdirect@gmail.com
// Company: Design Direct UK
// Description ; This is used as web controller for all estate agent campaign controller.
// </summary>

namespace App\Http\Controllers\Admin;

use App\Helpers\Helper;
use App\Http\Controllers\Controller;
use App\Model\Campaign;
use App\Model\Letter;
use App\Model\PropertyCampaign;
use App\Model\PropertyHistory;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Session;
use Redirect;
use Validator;
use Yajra\DataTables\DataTables;

class CampaignController extends Controller
{
    public $status = [
        1 => 'New to Market',
        2 => 'For sale',
        3 => 'Sold',
        4 => 'Withdrawn',
        5 => 'Price drop',
        6 => 'Fallen Through',
    ];
    public $trigger = [
        0 => 'General Letters',
        1 => 'New to Market',
        2 => 'For sale',
        3 => 'Sold',
        4 => 'Withdrawn',
        5 => 'Price drop',
        6 => 'Fallen Through',
        7 => '8 Week',
        8 => '16 Week',
        9 => '22 Week',
    ];

    public function campaignHistory(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $campaign = Campaign::where('type',1)->get();
        $admin_campaign_merge = 1;
        $my_class = "active";
        $prob_class = "add-un";
        return view('admin.campaign_history', compact('campaign','admin_campaign_merge','prob_class','my_class'));
    }

    public function lists(Request $request)
    {
        $authUser = json_decode(Session::get(config('propai.authSessionKey')));
        $orgId = $authUser->organisation_id;

        $detail = Campaign::where('type',1)->select(['name', 'sent_date', 'campaign_status', 'id'])->whereHas('user', function ($qry) use ($orgId) {
            $qry->where('organisation_id', $orgId);
        })->orderBy('campaigns.id', 'desc');

        return DataTables::of($detail)
            ->editColumn('sent_date', function ($detail) {
                return '' != $detail->sent_date ? Carbon::parse($detail->sent_date)->format('d/m/Y') : '';
            })
            ->editColumn('campaign_status', function ($detail) {
                return 1 == $detail->campaign_status ? 'Send' : 'Sending';
            })
            ->addColumn('action', function ($detail) {
                $print_url = url('admin/campaign-history/'.$detail->id.'/print');
                $action = '<a class="btn btn-primary btn-sm mr-2" target="_blank" href="'.$print_url.'">Print</a>';
                $action .= Helper::deleteButton($detail->id);
                $action .= '<button class="btn btn-success btn-sm mr-2 editCampaign" data-id="'.$detail->id.'" data-name="'.$detail->name.'">Edit</button>';
                $action .= '<a class="btn btn-info btn-sm mr-2" href=" '.route('adminCampaignHistoryView', $detail->id).'">View</a>';

                return $action;
            })
            ->removeColumn(['id'])
            ->escapeColumns([])
            ->make(false)
        ;
    }

    public function destroy($id): array
    {
        Campaign::destroy($id);
        $error = false;
        $message = 'Campaign deleted successfully';

        return compact('error', 'message');
    }

    public function campaignPrint($campaign_id)
    {
       
        $authUser = json_decode(Session::get(config('propai.authSessionKey')));
        $orgId = $authUser->organisation_id;


        $user_id = $authUser->id;
        $campaign = Campaign::whereHas('user', function ($qry) use ($orgId) {
            $qry->where('organisation_id', $orgId);
        })->find($campaign_id);

        if ($campaign) {
            $campaign->campaign_status = 1;
            $campaign->sent_date = Carbon::now();
            $campaign->save();
            $properties = PropertyCampaign::where('campaign_id', $campaign_id)
                ->onlyTrashed()->get() ;
        // dd($properties);
            if (count($properties) > 0) {
                foreach ($properties as $property) {
                    ini_set('memory_limit', '-1');
                    $property->address = PropertyHistory::join('property_house_number as phn', 'property_history.id', 'phn.property_history_id')
                        ->join('house_number as hn', 'hn.id', 'phn.house_number_id')
                        ->join('postcode as pc', 'pc.id', 'hn.postcode_id')
                        ->join('outcode as oc', 'oc.id', 'pc.outcode_id')
                        ->where('hn.id', $property->house_number_id)
                        ->select('hn.house_number', 'property_history.address', DB::raw('CONCAT(oc.outcode,pc.postcode) as pincode'))
                        ->first()
                ;
                $property->letter = Letter::where('triggers', $property->trigger)->where('user_id', $property->user_id)->first();

                }
//            return view('campaign.print', compact('properties'));
                $pdf = App::make('dompdf.wrapper');
                $pdf->loadView('campaign.print', compact('properties'));

                return $pdf->stream($campaign->name.'.pdf','I');
            }
        }
        return redirect()->route('campaignHistory');
    }

    public function campaignDetail(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));

        $campaign = Campaign::where('id', $id)
            ->with(
                'propertyCampaign',
                'propertyCampaign.houseNumber',
                'propertyCampaign.houseNumber.propertyHouseNumber',
                'propertyCampaign.houseNumber.propertyHouseNumber.propertyHistory',
                'propertyCampaign.houseNumber.propertyHouseNumber.propertyLetter',
                'propertyCampaign.houseNumber.propertyHouseNumber.propertyLetter.letter'
            )
            ->first()
        ;
        // dd($campaign->propertyCampaign[0]['houseNumber']['propertyHouseNumber'][0]['propertyHistory']['address']);die;

        return view('campaign.detail', compact('campaign'));
    }

    public function searchHistory(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $letter = Letter::where('user_id', $user->id)->get();
        $campaign = Campaign::where('user_id', $user->id)
            ->where('name', 'LIKE', '%'.$request->search.'%')
            ->get()
        ;

        return view('campaign.history', compact('campaign', 'letter'));
    }

    public function removeCampaign(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $campaign = PropertyCampaign::where('user_id', $user->id)->delete();

        return view('campaign.empty');
    }

    public function removeCampaignId(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $delete = PropertyCampaign::where('id', $id)->delete();
        $callback = function ($query) {
            $query->where('campaign_status', '=', 1);
        };

        $campaign = PropertyCampaign::where('user_id', $user->id)
            ->with(
                'propertyHistory',
                'campaign',
                'user',
                'propertyHistory.agency',
                'propertyHistory.propertyLetter',
                'propertyHistory.propertyLetter.letter'
            )
            ->whereHas('campaign', $callback)
            ->with(['campaign' => $callback])
            ->get()
        ;
        $letter = Letter::where('user_id', $user->id)->get();

        return view('campaign.list', compact('campaign', 'letter'));
    }

    public function createLetter(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        if (!empty($request->file('file'))) {
            $path = $request->file('file')->store('public/letters');
        } else {
            $path = '';
        }

        Letter::create([
            'name' => $request->name,
            'user_id' => $user->id,
            'file' => $path,
            'trigger' => $request->trigger,
        ]);
        $letter = Letter::where('user_id', $user->id)->orderby('created_at', 'DESC')->get();

        return view('campaign.edit-letter', compact('letter'));
    }

    public function updateLetter(Request $request, $id)
    {
        $rules = [
            'name' => 'required',
            'trigger' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('msg_error', $validator->errors()->first());
        }
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        if (!empty($request->file('file'))) {
            $path = $request->file('file')->store('public/letters');
            Letter::where('id', $id)->update([
                'file' => $path,
            ]);
        }

        Letter::where('id', $id)->update([
            'name' => $request->name,
            'trigger' => $request->trigger,
        ]);
        $letter = Letter::where('user_id', $user->id)->orderby('created_at', 'DESC')->get();

        return view('campaign.edit-letter', compact('letter'));
    }

    public function editLetter(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $letter = Letter::where('user_id', $user->id)->orderby('created_at', 'DESC')->get();

        return view('campaign.edit-letter', compact('letter'));
    }

    public function viewLetter(Request $request)
    {
        return view('campaign.view-letter');
    }

    public function manageLetter(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $letter = Letter::where('id', $id)->first();

        return view('campaign.manage-letter', compact('letter'));
    }

    public function deleteLetterId(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $delete = Letter::where('id', $id)->delete();
        $letter = Letter::where('user_id', $user->id)->orderby('created_at', 'DESC')->get();

        return view('campaign.edit-letter', compact('letter'));
    }

    public function update(Request $request, $id)
    {
        $campaign = Campaign::find($id);
        $campaign->name = $request->name;
        $campaign->save();

        $error = false;
        $message = 'Updated successfully';

        return compact('error', 'message');
    }

    public function show(Request $request, $id)
    {
        $authUser = json_decode(Session::get(config('propai.authSessionKey')));
        $orgId = $authUser->organisation_id;

        $campaign = Campaign::whereHas('user', function ($qry) use ($orgId) {
            $qry->where('organisation_id', $orgId);
        })->find($id);

        if ($campaign) {
            $user = json_decode(Session::get(config('propai.authSessionKey')));
            $sendDate = '' != $campaign->sent_date ? Carbon::parse($campaign->sent_date)->format('d/m/Y') : '';

            $lists = PropertyCampaign::withTrashed()->join('house_number as h', 'property_campaigns.house_number_id', 'h.id')
                ->join('postcode as pc', 'pc.id', 'h.postcode_id')
                ->join('outcode as oc', 'oc.id', 'pc.outcode_id')
                ->select('property_campaigns.*', 'h.house_number', DB::raw('CONCAT(oc.outcode,pc.postcode) as pincode'), 'h.listed_date')
                ->where('property_campaigns.campaign_id', $campaign->id)
                ->get()
            ;

            if (count($lists) > 0) {
                $i = 0;
                foreach ($lists as $item) {
                    $properties = PropertyHistory::select('property_history.*', 'a.name as agency')
                        ->join('property_house_number as phn', 'phn.property_history_id', 'property_history.id')
                        ->join('agency as a', 'a.id', 'property_history.agency_id')
                        ->where('phn.house_number_id', $item->house_number_id)
                        ->get()
                    ;
                    $full_address = $item->house_number.',<br/> ';
                    $full_address .= (count($properties) > 0 && '' != $properties[0]->address) ? $properties[0]->address.',<br/>' : '';
                    // $full_address .= $item->pincode;
                    $agency = count($properties->pluck('agency')) > 0 ? implode(',', $properties->pluck('agency')->toArray()) : '';

                    // $campaign = Campaign::where('id', $item->campaign_id)
                    //     ->where('campaign_status', 1)
                    //     ->latest()
                    //     ->first();
                    $encrpt_id = Helper::encryptData($item->house_number_id);
                    
                    $lastLetter = PropertyCampaign::withTrashed()
                        ->whereHas('campaign')
                        ->where('house_number_id', $item->house_number_id)
                        ->where('user_id', $campaign->user_id)
                        ->whereNotNull('campaign_id')
                        ->latest()->first();

                    ++$i;
                    $item->sl = $i;
                    $item->address = $full_address;
                    $item->property_url = route('admPropertyDetails', ['property_id' => $encrpt_id]);
                    $item->agency = $agency;
                    $item->trigger = $this->trigger[$item->trigger];
                    $item->listed_date = Carbon::parse($item->listed_date)->format('d/m/Y');
                    $item->last_letter_sent = $lastLetter ? $lastLetter->campaign->sent_date->format('d/m/Y') : '';
                }
            }

            return view('admin.campaign-history-view', compact('campaign', 'sendDate', 'lists'));
        }

        return redirect()->route('campaignHistory')->with('msg_error', 'Cannot find any campaign with this ID.');
    }
}
