<?php

namespace App\Http\Controllers\Admin;

use App\Helpers\Helper;
use App\Http\Controllers\Controller;
use App\Model\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Session;
use Redirect;
use Validator;

class DataEntryOperatorController extends Controller
{
    public function allDataOperator(Request $request)
    {
        $operators = User::where('user_type', 3)->pluck('first_name', 'id');
        $page_title = 'All Data Entry Operator';

        return view('admin.data-operator', compact('page_title', 'operators'));
    }

    public function allDataOperatorArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $user_outcode = $this->userOutcode();

        $query = User::select('id', 'first_name', 'last_name', 'email', 'contact_number', 'is_active')->where('user_type', 3);

        $primaryKey = 'id';

        $columns = [
            0 => 'id',
            1 => 'first_name',
            2 => 'last_name',
            3 => 'email',
            4 => 'contact_number',
            5 => 'is_active',
            6 => 'action',
        ];

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;
        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $list = $query->where(function ($q) use ($search) {
                $q->Where('users.first_name', 'like', '%'.$search.'%')
                    ->orWhere('users.last_name', 'like', '%'.$search.'%')
                    ->orWhere('users.email', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
                ->offset($start)
                ->limit($limit)
                ->get()
            ;

            $totalFiltered = $query->Where('users.first_name', 'like', '%'.$search.'%')
                ->orWhere('users.last_name', 'like', '%'.$search.'%')
                ->orWhere('users.email', 'like', '%'.$search.'%')
                ->count()
            ;
        }
        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                // $url = route('data-operator-details',['id'=>$encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->id;
                $nestedData['sl'] = $i;
                $nestedData['first_name'] = $item->first_name;
                $nestedData['last_name'] = $item->last_name;
                $nestedData['email'] = $item->email;
                $nestedData['contact_number'] = $item->contact_number;
                $nestedData['flag'] = $item->is_active;
                $nestedData['action'] = $item->id;
                $url = route('DEOSubmittedProperty', ['user_id' => $item->id]);
                $nestedData['submitted'] = '<a  href="'.$url.'" class="btn btn-danger font-weight-bold ">
            Submitted Property </a>';

                $dataArr[] = $nestedData;
            }
        }
        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    public function addDataOperator()
    {
        return view('admin.add-data-operator');
    }

    public function saveDataOperator(Request $request)
    {
        $authUser = json_decode(Session::get(config('propai.authSessionKey')));

        $id = $authUser->id;

        $rules = [
            'first_name' => 'required',
            'last_name' => 'required',
            'contact_number' => ['required', 'regex:/^(((\+44\s?\d{4}|\(?0\d{4}\)?)\s?\d{3}\s?\d{3})|((\+44\s?\d{3}|\(?0\d{3}\)?)\s?\d{3}\s?\d{4})|((\+44\s?\d{2}|\(?0\d{2}\)?)\s?\d{4}\s?\d{4}))(\s?\#(\d{4}|\d{3}))?$/'],
            'email' => 'required|email|unique:users,email',
            'password' => 'required|min:6',
        ];

        $validator = validator::make($request->all(), $rules);
        if ($validator->fails()) {
            return Redirect::back()->with('msg_error', $validator->errors()->first())->withInput();
        }
        $dataOperator = new User();

        $dataOperator->first_name = ucfirst($request->first_name);
        $dataOperator->last_name = ucfirst($request->last_name);
        $dataOperator->contact_number = $request->contact_number;
        $dataOperator->email = $request->email;
        $dataOperator->is_active = 2;
        $dataOperator->password = Hash::make($request->password);
        $dataOperator->user_type = 3;
        $dataOperator->organisation_id = $authUser->organisation_id;

        if ($dataOperator->save()) {
            return Redirect::route('allDataOperator')->with('msg_success', 'Data operator has been created successfully...!');
        }

        return $this->simpleReturn('error', 'Something Went Wrong...!');
    }

    public function flagChange(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        foreach ($request->property as $operator_id) {
            $exist_data = User::where('id', $operator_id)->first();

            if ($exist_data) { //update
                $exist_data->is_active = $request->flag;
                $exist_data->save();
            } else {//insert
                return $this->simpleReturn('error', 'Something went wrong...!');
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the status...!');
    }

    public function editDataOperator($id)
    {
        $page_title = 'Edit Data Operator';
        if ($id) {
            $operator = User::find($id);
            if ($operator) {
                return view('admin.edit-data-operator', compact('page_title', 'operator'));
            }

            return $this->simpleReturn('error', 'Something Went Wrong...!');
        }

        return $this->simpleReturn('error', 'Something Went Wrong...!');
    }

    public function updateDataOperator(Request $request, $id)
    {
        $rules = [
            'first_name' => 'required',
            'last_name' => 'required',
            'contact_number' => ['required', 'regex:/^(((\+44\s?\d{4}|\(?0\d{4}\)?)\s?\d{3}\s?\d{3})|((\+44\s?\d{3}|\(?0\d{3}\)?)\s?\d{3}\s?\d{4})|((\+44\s?\d{2}|\(?0\d{2}\)?)\s?\d{4}\s?\d{4}))(\s?\#(\d{4}|\d{3}))?$/'],
            'email' => 'required|email',
            'password' => 'sometimes|nullable',
        ];

        $validator = validator::make($request->all(), $rules);
        if ($validator->fails()) {
            return Redirect::back()->with('msg_error', $validator->errors()->first())->withInput();
        }
        if ($id) {
            $dataOperator = User::find($id);
            if ($dataOperator) {
                $dataOperator->first_name = $request->first_name;
                $dataOperator->last_name = $request->last_name;
                $dataOperator->contact_number = $request->contact_number;
                $dataOperator->email = $request->email;
                if ($request->has('password') && '' != $request->password) {
                    $dataOperator->password = Hash::make($request->password);
                }
                if ($dataOperator->save()) {
                    $user_id = $dataOperator->id;

                    return Redirect::route('allDataOperator')->with('msg_success', 'Data Operator has been updated successfully...!');
                }

                return $this->simpleReturn('error', 'Something Went Wrong...!');
            }

            return $this->simpleReturn('error', 'Something Went Wrong...!');
        }

        return $this->simpleReturn('error', 'Something Went Wrong...!');
    }

    public function deleteDataOperator(Request $request)
    {
        foreach ($request->property as $id) {
            $exist_data = User::where('id', $id)->first();

            if ($exist_data) {
                $exist_data->delete();
            } else {//insert
                return $this->simpleReturn('error', 'Something went wrong...!');
            }
        }

        return $this->simpleReturn('success', 'Successfully data operator deleted...!');
    }
}
