<?php


namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Redirect, Validator;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Storage;
use App\Model\User;
use App\Model\Outcode;
use App\Model\UserOutcode;
use App\Helpers\Helper;
use Illuminate\Support\Facades\Hash;

class OutCodeController extends Controller
{
    public function AllOutCode()
    {
        $page_title = "All OutCodes";

       
        return view('admin.outcode', compact('page_title'));
    }

    public function outcodeArray(Request $request){
       
        // Table's primary key
        $primaryKey = 'id';

        $columns = array(
            0 =>'outcode',
            1 =>'price',          
            2 =>'updated_date',          
        );

        $data = Outcode::where('archive',0)
        ->orderBy('updated_at','desc')
        ->get();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
       // $query = $query->orderBy('house_number.listed_date','desc');
        if(empty( $request->input('search.value')))
        {
            $list = Outcode::where('archive',0)->orderBy('updated_at','desc')->offset($start)->limit($limit)->get();
        }else {

            $search =  $request->input('search.value');

            $count = Outcode::where('archive',0)
            ->orderBy('updated_at','desc')
            ->Where('outcode', 'like', '%' . $search . '%')->get();

            $list =  Outcode::where('archive',0)
                ->orderBy('updated_at','desc')
                ->Where('outcode', 'like', '%' . $search . '%')
                ->offset($start)
                ->limit($limit)
                ->get();

           
            $totalFiltered = count($count);
        }

        $dataArr = array();
        if(!empty($list))
        {
            $i =0;
            foreach ($list as $item)
            {   
                $nestedData['price'] = $item->price;
                $nestedData['outcode'] = $item->outcode;
                $nestedData['updated_date'] = date('d-m-Y H:i',strtotime($item->updated_at));
                
                $edit_url = route('editOutCode',['outcode' => $item->id]);
                $nestedData['edit'] = "<a href='$edit_url' class='btn btn-primary delete-user' title='Edit Outcode'><i class='flaticon-edit'></i>";
                
                $nestedData['archive'] = "<button type='button' data-id='$item->id' class='btn btn-danger move-archive' title='Move to archive'><i class='flaticon2-arrow-down'></i>";
                

                $dataArr[] = $nestedData;

            
            
            }
        }

        $json_data = array(
                    "draw"            => intval($request->input('draw')),
                    "recordsTotal"    => intval($totalData),
                    "recordsFiltered" => intval($totalFiltered),
                    "data"            => $dataArr
                    );

        return json_encode($json_data);
    }
    public function addOutCode()
    {
        $outcode = Outcode::get();
        return view('admin.add-outcode', compact('outcode'));
    }
    public function saveOutCode(Request $request)
    {
        $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $rules = array(
            'outcode' => 'required|unique:outcode,outcode',
            'price' => 'required|integer',
        );

        $validator = validator::make($request->all(), $rules);
        if ($validator->fails()) {
            return Redirect::back()->with('msg_error', $validator->errors()->first())->withInput();
        } else {
            $outcode = new Outcode();

            $outcode->outcode = ($request->outcode);
            $outcode->price = ($request->price);
            if ($outcode->save()) {


                return Redirect::route('allOutCode')->with('msg_success', "OutCode has been created successfully...!");
            } else {
                return $this->simpleReturn('error', 'Something Went Wrong...!');
            }
        }
    }
    
    public function editOutCode(Request $request)
    {

        $id = $request->outcode;
        $page_title = "Edit OutCode";
        if ($id) {
            $outcode = Outcode::find($id);

            if ($outcode) {
                return view('admin.edit-outcode', compact('page_title', 'outcode'));
            }
            return $this->simpleReturn('error', 'Something Went Wrong...!');
        }
        return $this->simpleReturn('error', 'Something Went Wrong...!');
    }
    public function updateOutCode(Request $request)
    {

        $rules = array(
            'outcode' => 'required|unique:outcode,outcode,' . $request->outcode_id,
            'price' => 'required|integer',
        );

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('msg_error', $validator->errors()->first())->withInput();
        } else {
            $outcode = Outcode::where('id', $request->outcode_id)->update(
                [
                    'outcode' => $request->outcode,
                    'price' => $request->price
                ]
            );
            return Redirect::route('allOutCode')->with('msg_success', "OutCode has been updated successfully...!");
        }
    }

    //move to archive and remove from archive
    public function moveArchive(Request $request){
        $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $rules = array(
            'outcode' => 'required',
        );

        $validator = validator::make($request->all(), $rules);
        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first())->withInput();
        } else {
            $outcode = Outcode::where('id',$request->outcode)->first();

            if($outcode->archive==1){
                $archive = 0;
            }else{
                $archive = 1;
            }

            $update = Outcode::where('id', $request->outcode)->update(['archive' => $archive]);

            if($update){

                if($archive == 1){
                    return $this->simpleReturn('success', 'Sucessfully moved to archive');
                }else{
                    return $this->simpleReturn('success', 'Sucessfully removed from archive');
                }

            }else{
                return $this->simpleReturn('error', 'Something went wrong...!');
            }
        }
    }

    public function archiveOutCode()
    {
        $page_title = "All OutCodes";

        $query = Outcode::where('archive',1)->select('id', 'outcode', 'price')->orderby('created_at', 'DESC')->get();

        return view('admin.archive-outcode', compact('page_title', 'query'));
    }
}
