<?php

// <summary>
// File : PropertyController
// File Type : Controller
// Creator : Susmi Sidharthan
// Email: susmi@designdirect@gmail.com
// Company: Design Direct UK
// Description ; This is used as web controller for all estate agent Property controller.
// </summary>

namespace App\Http\Controllers\Admin;

use App\Helpers\Helper;
use App\Http\Controllers\Controller;
use App\Model\Agency;
use App\Model\Outcode;
use App\Model\PostCode;
use App\Model\PropertyHistory;
use DB;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Session;

class PropertyController extends Controller
{
    //All property
    public function allProperty()
    {
        $page_title = 'Full Address Property';
        $post_code = PostCode::orderBy('postcode', 'asc')->get()->pluck('postcode', 'id');
        $active = 2;
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $filter = true;
        $agency = Agency::get();
        $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');
        $outcode = Outcode::where('archive', 0)
            ->orderBy('updated_at', 'desc')
            ->get()
        ;
        $totalProp = true;

        return view('admin.all-property', compact('page_title', 'post_code', 'active', 'user_type', 'filter', 'agency', 'property', 'outcode', 'totalProp'));
    }

    public function allPropertyArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        $query = $this->commonPropQuery();

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }
        // $query = $query->whereIn('outcode.id',$user_outcode);
        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('platform')) {
            if (('-1' != $request->platform && 'undefined' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        //    if ($request->has('outcode')) {
        //         if(($request->outcode != '0' && $request->outcode != 'undefined')){
        //            $query->where('outcode.id',$request->input('outcode'));
        //         }
        //    }
        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'latest_price',
            3 => 'house_number.listed_date',
            4 => 'agency.name',
            5 => 'property_type',
            6 => 'portal',
        ];

        $data = $query->get()->unique('house_number_id');
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = Outcode::
            join('postcode', 'postcode.outcode_id', 'outcode.id')->where('archive', 0)
                ->join('house_number', 'house_number.postcode_id', 'postcode.id')
                ->orderBy('outcode.outcode', 'asc')
                ->distinct('outcode.id')
                ->pluck('outcode.outcode')
            ;

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                    ->orWhere('agency.name', 'like', '%'.$search.'%')
                    ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                    ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                    ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                    ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            // $totalFiltered =  $query->where(function($q) use ($search) {
            //     $q->Where('property_history.address', 'like', '%' . $search . '%')
            //         ->orWhere('agency.name', 'like', '%' . $search . '%')
            //     ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            //     ->orWhere('outcode.outcode', 'like', '%' . $search . '%')
            //     ->orWhere('postcode.postcode', 'like', '%' . $search . '%')
            //     ->orWhere('house_number.house_number', 'like', '%' . $search . '%');

            // })->orderBy($order, $dir)->get()->unique('house_number_id');

            // $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                    ->orWhere('agency.name', 'like', '%'.$search.'%')
                    ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                    ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                    ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                    ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
                ->offset($start)
                ->limit($limit)
                ->get()->unique('house_number_id');
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('admPropertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = "https://www.google.com/maps?q={$item->postcode}";
                }
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;

                if (null == $item->created_by) {
                    $st = 'By System';
                } else {
                    $st = 'By Manual';
                }

                $nestedData['address'] = "<a href='{$url}'>{$item->house_number} ,{$item->address},  {$item->postcode}</a><div>".$st.'</div>';

                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days : '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = ucfirst($item->property_type);
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                $nestedData['portal_url'] = $platform;

                $nestedData['street_view'] = "<a target='street_view' title='Access Street View' class='btn btn-danger btn-sm ml-2' href='{$map_url}'><i class='flaticon2-map icon-lg'></i></a>";

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    public function propertyDetails(Request $request)
    {
        $propert_id = Helper::decryptData($request->property_id);
        $page_title = 'Property Details';
        $query = $this->commonPropQuery();
        $details = $query->where('house_number.id', $propert_id)->first();
        $house_number_id = Helper::decryptData($request->property_id);
        $portal_url = $this->houseNumberPortal($house_number_id);
        // dd($details);
        return view('admin.property-details', compact('page_title', 'details', 'portal_url', 'house_number_id'));
    }

    public function propertyTrigger($tab)
    {
        $page_title = 'Property';
        $post_code = PostCode::orderBy('postcode', 'asc')->get();
        $active = 2;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        if (1 == $tab) {
            $active = 1;

            return view('admin.property-all-type', compact('page_title', 'post_code', 'active', 'user_type'));
        }
        if (2 == $tab) {
            $active = 2;

            return view('admin.all-property', compact('page_title', 'post_code', 'active', 'user_type'));
        }
        if (3 == $tab) {
            $active = 3;

            if (2 == $user_type) {
                $userOutcodes = $this->userOutcodes();
            } else {
                $userOutcodes = Outcode::where('archive', 0)
                    ->orderBy('updated_at', 'desc')
                    ->pluck('outcode.outcode')
                ;
            }

            return view('common.property.property-without-address', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active'));
        }

        return view('admin.all-property', compact('page_title', 'post_code', 'active', 'user_type'));
    }

    public function allTypeProperty()
    {
        $page_title = 'Property';
        $post_code = PostCode::orderBy('postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        return view('admin.property-all-type', compact('page_title', 'post_code', 'user_type'));
    }

    public function admAllTypePropertyArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        // $query = $this->commonPropQuery();

        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
            ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
            ->leftjoin('house_number', 'house_number.id', 'property_house_number.house_number_id')
        // ->where('property_house_number.house_number_id', NULL)
        // ->where('property_house_number.property_history_id', NULL)
            ->select('property_history.id', 'outcode.outcode', 'postcode.postcode', 'property_history.address', DB::raw("GROUP_CONCAT(DISTINCT agency.name SEPARATOR ', ') AS agency_name"), 'property_history.property_type', 'property_history.price', 'property_history.portal_url', 'property_history.latitude', 'property_history.longitude', 'property_history.portal', 'property_history.title', 'property_house_number.house_number_id', 'house_number.latest_price', 'house_number.listed_date', 'house_number.created_by')
            ->groupBy('property_history.id', 'outcode.outcode', 'postcode.postcode', 'property_history.address', 'property_history.property_type', 'property_history.price', 'property_history.portal_url', 'property_history.latitude', 'property_history.longitude', 'property_history.portal', 'property_history.title', 'property_house_number.house_number_id', 'house_number.latest_price', 'house_number.listed_date')
        ;

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('platform')) {
            if (('' != $request->platform && 'undefined' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
            }
        }
        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'latest_price',
            3 => 'house_number.listed_date',
            4 => 'agency.name',
            5 => 'property_type',
            6 => 'portal',
        ];

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                    ->orWhere('agency.name', 'like', '%'.$search.'%')
                    ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                    ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                    ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
                ->offset($start)
                ->limit($limit)
                ->get()
            ;

            // $totalFiltered  =  $query->where(function($q) use ($search) {
            //     $q->Where('property_history.address', 'like', '%' . $search . '%')
            //         ->orWhere('agency.name', 'like', '%' . $search . '%')
            //     ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            //     ->orWhere('outcode.outcode', 'like', '%' . $search . '%')
            //     ->orWhere('postcode.postcode', 'like', '%' . $search . '%');

            // })->count();
            $totalFiltered = count($list);
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                if (null != $item->house_number_id) {
                    $encrpt_id = Helper::encryptData($item->house_number_id);
                    $url = route('admPropertyDetails', ['property_id' => $encrpt_id]);
                    ++$i;
                    // $map_url = "https://www.google.com/maps?q=$item->latitude,$item->longitude";
                    if ('' != $item->latitude && '' != $item->longitude) {
                        $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                    } else {
                        $map_url = "https://www.google.com/maps?q={$item->postcode}";
                    }

                    $nestedData['id'] = $item->house_number_id;
                    $nestedData['sl'] = $i;
                    $nestedData['outcode'] = $item->outcode;
                    if (null == $item->updated_by) {
                        $st = 'By System';
                    } else {
                        $st = 'By Data Entry';
                    }
                    $nestedData['address'] = "<a href='{$url}'>{$item->house_number} ,{$item->address},  {$item->postcode}</a><div>".$st.'</div>';

                    if (0 == $item->latest_price) {
                        $nestedData['price'] = 'POA';
                    } else {
                        $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                    }

                    if ($item->listed_date) {
                        $listed_date = date('Y-m-d', strtotime($item->listed_date));
                        $today = date('Y-m-d');
                        $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                        $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Number of days : '.$days.'</div>';
                    } else {
                        $nestedData['listed_date'] = 'Hidden from public';
                    }

                    $nestedData['agency'] = $item->agency_name;
                    $nestedData['type'] = ucfirst($item->property_type);
                    //portal url
                    $portal_url = $this->houseNumberPortal($item->house_number_id);
                    $urls = '';
                    if (count($portal_url)) {
                        foreach ($portal_url as $url) {
                            if (0 == $url->portal) {
                                $urls .= "<a title='Click here to see the listing' target=rightmovel' class='btn btn-rightmove  btn-sm' title='Rightmove' href='{$url->portal_url}'><i class='icon-lg'>R</i></a>";
                            } else {
                                $urls .= "<a title='Click here to see the listing' target='portal' title='Zoopla' class='btn btn-zoopla  btn-sm ml-2' href='{$url->portal_url}'><i class='icon-lg'>Z</i></a>";
                            }
                        }
                    }

                    $nestedData['porta_url'] = $url;
                    $nestedData['street_view'] = "<a target='street_view' title='Access Street View' class='btn btn-danger btn-sm ml-2' href='{$map_url}'><i class='flaticon2-map icon-lg'></i></a>";

                    $dataArr[] = $nestedData;
                } else {
                    $encrpt_id = Helper::encryptData($item->id);
                    //    $url = route('DEOpropertyDetails',['property_id'=>$encrpt_id]);
                    // $map_url = "https://www.google.com/maps?q=$item->latitude,$item->longitude";

                    if ('' != $item->latitude && '' != $item->longitude) {
                        $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                    } else {
                        $map_url = "https://www.google.com/maps?q={$item->postcode}";
                    }

                    $url = route('propertyHistoryDetails', ['property_id' => $encrpt_id]);

                    $nestedData['property_number'] = '<div class="input-group">
                    <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                    <div class="input-group-append">
                        <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                    </div>
                </div>';

                    if (!empty($item->house_number)) {
                        $address2 = $item->address;
                    } else {
                        $address2 = $item->address.',<br>  '.$item->postcode;
                    }
                    if (!empty($item->created_by)) {
                        $address2 = $item->address.',<br>  '.$item->postcode;
                    }
                    $nestedData['address'] = "<a href='{$url}'>{$address2}</a>";
                    $nestedData['outcode'] = '';
                    $nestedData['listed_date'] = '';
                    $nestedData['agency'] = '';

                    // $nestedData['price'] = "&pound;" . number_format($item->latest_price, 2);
                    if (0 == $item->latest_price) {
                        $nestedData['price'] = 'POA';
                    } else {
                        $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                    }

                    $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                    if (0 == $item->portal) {
                        $nestedData['portal_url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove  btn-sm' href='{$url->portal_url}'><i class='icon-lg'>R</i></a>";
                    } else {
                        $nestedData['portal_url'] = "<a title='Click here to see the listing' target='zoopla'  class='btn btn-zoopla  btn-sm ml-2' href='{$url->portal_url}'><i class='icon-lg'>Z</i></a>";
                    }
                    $nestedData['street_view'] = "<a target='street_view' title='Access Street View' class='btn btn-danger btn-sm ml-2' href='{$map_url}'><i class='flaticon2-map icon-lg'></i></a>";

                    ++$i;

                    $dataArr[] = $nestedData;
                }
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }
}
