<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Model\Outcode;
use App\Model\User;
use App\Model\UserOutcode;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Str;
use Redirect;
use Validator;

class AgentController extends Controller
{
    public function saveAgent(Request $request)
    {
        $rules = [
            'first_name' => 'required',
            'last_name' => 'required',
            'contact_number' => ['required', 'regex:/^(((\+44\s?\d{4}|\(?0\d{4}\)?)\s?\d{3}\s?\d{3})|((\+44\s?\d{3}|\(?0\d{3}\)?)\s?\d{3}\s?\d{4})|((\+44\s?\d{2}|\(?0\d{2}\)?)\s?\d{4}\s?\d{4}))(\s?\#(\d{4}|\d{3}))?$/'],
            'company_name' => 'required',
            'address' => 'required',
            'email' => 'required|email|unique:users,email',
            'password' => 'required|min:6',
            'out_code' => 'required|array|min:1',
        ];
        $validator = validator::make($request->all(), $rules);
        if ($validator->fails()) {
            if ($validator->fails()) {
                return response()->json($validator->messages(), 401);
            }
        } else {
            $agent = new User();
            $token = Str::random(60);
            $agent->first_name = ucfirst($request->first_name);
            $agent->last_name = ucfirst($request->last_name);
            $agent->contact_number = $request->contact_number;
            $agent->email = $request->email;
            $agent->company_name = $request->company_name;
            $agent->address = $request->address;
            $agent->is_active = 1;
            //$agent->status = 1;
            $agent->password = Hash::make($request->password);
            $agent->user_type = 2;
            $agent->api_token = $token;
            $agent->organisation_id = config('propai.organisation');
            if ($agent->save()) {
                $user_id = $agent->id;
                if ($request->out_code) {
                    foreach ($request->out_code as $key => $value) {
                        $userOutcode = new UserOutcode();
                        $userOutcode->user_id = $user_id;
                        $userOutcode->outcode_id = $value;
                        $userOutcode->save();
                    }
                }

                return response()->json([
                    'success' => true,
                    'data' => ['token' => $token],
                ], 200);
            }

            return response()->json([
                'success' => false,
                'message' => 'Sorry, something went wrong.',
            ], 401);
        }
    }

    public function login(Request $request)
    {
        if (!($request->token)) {
            return response()->json([
                'success' => false,
                'message' => 'Sorry, something went wrong.',
            ], 401);
        }
        $data = User::where('api_token', $request->token)->where('user_type', 2)->first();
        if ($data) {
            if (1 == $data['is_active']) {
                SESSION::put(config('propai.authSessionKey'), json_encode($data->toArray()));

                return Redirect::route('allProperty');
            }

            return response()->json([
                'success' => false,
                'message' => 'Sorry, The account not active, Please contact administrator.',
            ], 401);
        }

        return response()->json([
            'success' => false,
            'message' => 'Sorry, agent is not found.',
        ], 401);
    }

    public function getOutcode()
    {
        $outcode = Outcode::get();

        return response()->json([
            'success' => true,
            'data' => ['outcode' => $outcode],
        ], 200);
    }

    public function getData($token)
    {
        if ($token) {
            $data = User::where('api_token', $token)->where('user_type', 2)->first();
            if ($data) {
                $user_outcode = UserOutcode::where('user_id', $data->id)->where('status', 1)->get();

                return response()->json([
                    'success' => true,
                    'data' => ['agent' => $data, 'user_outcodes' => $user_outcode],
                ], 200);
            }

            return response()->json([
                'success' => false,
                'message' => 'Sorry, agent is not found.',
            ], 401);
        }

        return response()->json([
            'success' => false,
            'message' => 'Sorry, something went wrong.',
        ], 401);
    }

    public function update(Request $request, $token)
    {
        $rules = [
            'first_name' => 'required',
            'last_name' => 'required',
            'contact_number' => ['required', 'regex:/^(((\+44\s?\d{4}|\(?0\d{4}\)?)\s?\d{3}\s?\d{3})|((\+44\s?\d{3}|\(?0\d{3}\)?)\s?\d{3}\s?\d{4})|((\+44\s?\d{2}|\(?0\d{2}\)?)\s?\d{4}\s?\d{4}))(\s?\#(\d{4}|\d{3}))?$/'],
            'company_name' => 'required',
            'address' => 'required',
            'email' => 'required|email',
            'out_code' => 'required|array|min:1',
        ];
        $validator = validator::make($request->all(), $rules);
        if ($validator->fails()) {
            if ($validator->fails()) {
                return response()->json($validator->messages(), 401);
            }
        }

        if ($token) {
            $agent = User::where('api_token', $token)->where('user_type', 2)->first();
            if ($agent) {
                $agent->first_name = $request->first_name;
                $agent->last_name = $request->last_name;
                $agent->contact_number = $request->contact_number;
                $agent->email = $request->email;
                $agent->company_name = $request->company_name;
                $agent->address = $request->address;
                // if($request->has('password') && $request->password != ''){
                //      $agent->password = Hash::make($request->password);
                //  }
                if ($request->has('email') && '' != $request->email) {
                    $is_exist = User::where('email', $request->email)->where('id', '<>', $agent->id)->first();
                    if ($is_exist) {
                        return response()->json([
                            'success' => false,
                            'message' => 'Sorry, Someone has already used this email.',
                        ], 401);
                    }
                }
                if ($agent->save()) {
                    $user_id = $agent->id;
                    if ($request->out_code) {
                        foreach ($request->out_code as $key => $value) {
                            $userOutcode = UserOutcode::where('user_id', $user_id)->where('status', 1)->where('outcode_id', $value)->first();
                            if (!$userOutcode) {
                                $userOutcode = new UserOutcode();
                                $userOutcode->user_id = $user_id;
                                $userOutcode->outcode_id = $value;
                                // $userOutcode->status = 1;
                                $userOutcode->save();
                            }
                        }
                        $user_outcode = UserOutcode::where('user_id', $user_id)->where('status', 1)->pluck('outcode_id')->toArray();
                        foreach ($user_outcode as $key => $value) {
                            if (!in_array($value, $request->out_code)) {
                                UserOutcode::where('user_id', $user_id)->where('outcode_id', $value)->delete();
                            }
                        }
                    }

                    return response()->json([
                        'success' => true,
                        'message' => 'Agent has been updated successfully',
                    ], 200);
                }

                return response()->json([
                    'success' => false,
                    'message' => 'Sorry, Something Went Wrong.',
                ], 401);
            }

            return response()->json([
                'success' => false,
                'message' => 'Sorry, agent is not found.',
            ], 401);
        }
    }
}
