<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Model\HouseNumber;
use App\Model\PropertyHistory;
use App\Model\PropertyHouseNumber;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Validator;

class HouseNumberController extends Controller
{
    public function getHouseNumbers(Request $request)
    {
        $data = PropertyHistory::join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
            ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
            ->whereNull('property_house_number.deleted_at')
            ->whereNotNull('house_number.created_by')
            ->select(['property_history.pid', 'house_number.house_number', 'property_history.address', 'house_number.created_by'])
            ->get()
        ;

        if ($data->count()) {
            Storage::put('house-number/'.date('Ymd-His').'.json', $data);

            return response()->json([
                'success' => true,
                'message' => 'Please find the file in storage > house-number.',
                'count' => $data->count(),
                'data' => $data,
            ], 200);
        }

        return response()->json([
            'success' => false,
            'message' => 'Cannot find any entries',
            'data' => [],
        ], 400);
    }

    public function addHouseNumbers(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'upload_file' => 'required|file',
        ]);

        if ($validator->fails()) {
            $error_messages = implode(', ', $validator->messages()->all());

            return response()->json([
                'success' => false,
                'error_messages' => $error_messages,
            ], 422);
        }

        try {
            $path = $request->file('upload_file')->getRealPath();
            $getData = json_decode(file_get_contents($path));
            $data = json_decode(json_encode($getData), true);
            // $data = array_slice($data, 4000, 1000);
        } catch (\Exception $e) {
            return response()->json([
                'msg' => 'Invalid file',
                'error' => $e->getMessage(),
            ]);
        }
        $retrieved = 0;

        foreach ($data as $key => $house) {
            $history = PropertyHistory::where('pid', $house['pid'])->first();
            Log::info('key: '.$key.', pid: '.$house['pid']);
            if ($history) {
                $propertyHouseNumber = PropertyHouseNumber::where('property_history_id', $history->id);
                if (!$propertyHouseNumber->exists()) {
                    ++$retrieved;

                    $add = new HouseNumber();
                    $add->house_number = $house['house_number'];
                    $add->listed_date = $history->listed_date;
                    $add->sold_date = $history->sold_date;
                    $add->price_drop_date = $history->price_drop_date;
                    $add->withdrawn_date = $history->withdrawn_date;
                    $add->postcode_id = $history->postcode_id;
                    $add->latest_price = $history->price;
                    $add->latest_status = $history->property_current_status;
                    $add->price_change_diff = $history->price_drop;
                    $add->latitude = $history->latitude;
                    $add->longitude = $history->longitude;
                    $add->created_by = $house['created_by'];

                    if ($add->save()) {
                        $addHouseNum = new PropertyHouseNumber();
                        $addHouseNum->property_history_id = $history->id;
                        $addHouseNum->house_number_id = $add->id;
                        $addHouseNum->created_by = $house['created_by'];
                        $addHouseNum->save();
                    }
                }
            }
        }

        return response()->json([
            'success' => true,
            'retrieved' => $retrieved,
        ], 200);
    }

    public function getHouseNumbersByOutcode(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'outcode' => 'required',
        ]);

        if ($validator->fails()) {
            $error_messages = implode(', ', $validator->messages()->all());

            return response()->json([
                'success' => false,
                'error_messages' => $error_messages,
            ], 422);
        }

        $data = PropertyHistory::join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
            ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
            ->leftjoin('postcode', 'postcode.id', 'property_history.postcode_id')
            ->leftjoin('outcode', 'outcode.id', 'postcode.outcode_id')
            ->leftjoin('users', 'users.id', 'house_number.created_by')
            ->where('outcode.outcode', $request->outcode)
            ->whereNull('property_house_number.deleted_at')
            ->whereNotNull('house_number.created_by')
            ->select(['property_history.pid', 'house_number.house_number', 'property_history.address', 'postcode.postcode', 'house_number.created_by', 'users.first_name as created_user'])
            ->get()
        ;

        if ($data->count()) {
            $fileName = strtoupper($request->outcode).'-'.date('Ymd-His').'-'.$data->count().'.json';
            Storage::put('house-number/'.$fileName, $data);
            $headers = ['Content-Type: application/json'];
            $pathToFile = storage_path('app/house-number/'.$fileName);

            return response()->download($pathToFile, $fileName, $headers);

            return response()->json([
                'success' => true,
                'message' => 'Please find the file in storage > house-number.',
                'count' => $data->count(),
                'data' => $data,
            ], 200);
        }

        return response()->json([
            'success' => false,
            'message' => 'Cannot find any entries',
            'data' => [],
        ], 400);
    }
}
