<?php

// <summary>
// File : StannpAPIController
// File Type : API Controller
// Creator : Rijo Kurian
// Email: rijodesigndirect@gmail.com
// Company: Design Direct UK
// Description ; This is intergrate mass mailing of Stannp Service and postcodes.io api service calls with the system
// </summary>

// Curl call postcodeio param defenitions
/*
quality : 
Positional Quality. Shows the status of the assigned grid reference.
1 = within the building of the matched address closest to the postcode mean
2 = as for status value 1, except by visual inspection of Landline maps (Scotland only)
3 = approximate to within 50m
4 = postcode unit mean (mean of matched addresses with the same postcode, but not snapped to a building)
5 = imputed by ONS, by reference to surrounding postcode grid references
6 = postcode sector mean, (mainly | PO Boxes)
8 = postcode terminated prior to Gridlink® initiative, last known ONS postcode grid reference1
9 = no grid reference available
*/

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Model\Campaign;
use App\Model\Letter;
use App\Model\LetterHead;
use App\Model\Outcode;
use App\Model\PropertyCampaign;
use App\Model\User;
use App\Model\UserOutcode;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Str;
use Redirect;
use Validator;

class StannpAPIController extends Controller
{
    public function stannpAccountInfo() // get stannp account info
    {
        $func_name          = 'stannpAccountInfo';
        $func_url           = 'users/me';
        $trans_array        = [];
        $trans_array        = array_filter($trans_array);
        $trans_data         = [
                                    'func_name' => $func_name,
                                    'func_url'  => $func_url,
                                    'func_data' => $trans_array,
                                ];
        $result             = $this->stannpTransmission($trans_data);
        $account_response   = json_decode($result, true);
        if (isset($account_response) && $account_response['success'] === true) {
            return [
                        'success'   => true,
                        'message'   => 'account data fetch successful',
                        'data'      => $account_response['data'],
                    ];
        } else {
            return [
                        'success'   => false,
                        'message'   => 'account data fetch failed',
                        'error'     => $account_response['error'],
                    ];
        }
    }

    public function stannpAccountBalance() // get stannp account info
    {
        $func_name          = 'stannpAccountBalance';
        $func_url           = 'accounts/balance';
        $trans_array        = [];
        $trans_array        = array_filter($trans_array);
        $trans_data         = [
                                    'func_name' => $func_name,
                                    'func_url'  => $func_url,
                                    'func_data' => $trans_array,
                                ];
        $result             = $this->stannpTransmission($trans_data);
        $account_response   = json_decode($result, true);
        if (isset($account_response) && $account_response['success'] === true) {
            return [
                        'success'   => true,
                        'message'   => 'account data fetch successful',
                        'data'      => $account_response['data'],
                    ];
        } else {
            return [
                        'success'   => false,
                        'message'   => 'account data fetch failed',
                        'error'     => $account_response['error'],
                    ];
        }
    }

    public function addRecipient($data) // add recipient call to stannp API
    {
        $func_name      = 'addRecipient';
        $func_url       = 'recipients/new';
        $group_id       = isset($data['group_id']) ? $data['group_id'] : '';
        $on_duplicate   = isset($data['on_duplicate']) ? $data['on_duplicate'] : '';
        $title          = isset($data['title']) ? $data['title'] : '';
        $firstname      = isset($data['firstname']) ? $data['firstname'] : '';
        $lastname       = isset($data['lastname']) ? $data['lastname'] : '';
        $job_title      = isset($data['job_title']) ? $data['job_title'] : '';
        $address1       = isset($data['address1']) ? $data['address1'] : '';
        $address2       = isset($data['address2']) ? $data['address2'] : '';
        $address3       = isset($data['address3']) ? $data['address3'] : '';
        $city           = isset($data['city']) ? $data['city'] : '';
        $postcode       = isset($data['postcode']) ? $data['postcode'] : '';
        $country        = isset($data['country']) ? $data['country'] : '';
        $voucher        = isset($data['voucher']) ? $data['voucher'] : '';
        if ($postcode != '' && $address1 != '') { // proceed if importand fields passed
            $trans_array        = [
                                        'test'          => 'true',
                                        'group_id'      => $group_id,     // recipient - group id
                                        'on_duplicate'  => $on_duplicate, // when dupplicate data is sent, what needs to be done, sample value: update
                                        'title'         => $title,        // recipient - title
                                        'firstname'     => $firstname,    // recipient - firstname
                                        'lastname'      => $lastname,     // recipient - lastname
                                        'job_title'     => $job_title,    // recipient - job_title
                                        'address1'      => $address1,     // recipient - road house name
                                        'address2'      => $address2,     // recipient - other specific address related data
                                        'address3'      => $address3,     // recipient - address3 specifics
                                        'city'          => $city,         // recipient - city info
                                        'postcode'      => $postcode,     // recipient - postcode data
                                        'country'       => $country,      // recipient - country data
                                        'voucher'       => $voucher,      // stanpp user - voucher data
                                    ];
            $trans_array        = array_filter($trans_array);
            $trans_data         = [
                                        'func_name' => $func_name,
                                        'func_url'  => $func_url,
                                        'func_data' => $trans_array,
                                    ];
            $result             = $this->stannpTransmission($trans_data);
            $recipient_response = json_decode($result, true);

            return [
                        'success'   => true,
                        'message'   => 'new recipient created',
                        'data'      => $recipient_response['data'],
                    ];
        } else {
            return ['success' => false, 'message' => 'recipient params empty'];
        }
    }

    public function getRecipient($data) // get specific recipient call to stannp API
    {
        $func_name      = 'getRecipient';
        $func_url       = 'recipient/get/';
        $recipient_id   = isset($data['id']) ? $data['id'] : '';
        if (isset($recipient_id) && $recipient_id != '') {
            $trans_array        = ['recipient_id' => $recipient_id,];
            $trans_data         = [
                                        'func_name' => $func_name,
                                        'func_url'  => $func_url,
                                        'func_data' => $trans_array,
                                    ];
            $result             = $this->stannpTransmission($trans_data);
            $recipient_response = json_decode($result, true);
            if (isset($recipient_response) && $recipient_response['success'] === true) {
                return ['success' => true, 'data' => $recipient_response];
            } else {
                return [
                            'success'   => false,
                            'message'   => 'recipient not fetched',
                            'error'     => $recipient_response['error'],
                        ];
            }
        } else {
            return ['success' => false, 'message' => 'recipient id params empty'];
        }
    }

    public function getRecipientlist($data) // get all recipients in stannp account
    {
        $func_name          = 'getRecipientlist';
        $func_url           = 'recipients/list/';
        $group_id           = isset($data['group_id']) ? $data['group_id'] : '';
        $trans_array        = ['group_id' => $group_id,];
        $trans_data         = [
                                    'func_name' => $func_name,
                                    'func_url'  => $func_url,
                                    'func_data' => $trans_array,
                                ];
        $result             = $this->stannpTransmission($trans_data);
        $recipient_response = json_decode($result, true);
        if (isset($recipient_response) && $recipient_response['success'] === true) {
            return [
                        'success'   => true,
                        'group_id'  => $group_id,
                        'data'      => $recipient_response
                    ];
        } else {
            return [
                        'success'   => false,
                        'message'   => 'recipients not fetched',
                        'error'     => $recipient_response['error'],
                    ];
        }
    }

    public function deleteRecipient($data) // delete specific recipient from stannp API
    {
        $func_name      = 'deleteRecipient';
        $func_url       = 'recipients/delete';
        $recipient_id   = isset($data['id']) ? $data['id'] : '';
        if (isset($recipient_id) && $recipient_id != '') {
            $trans_array        = ['id' => $recipient_id,];
            $trans_data         = [
                                        'func_name' => $func_name,
                                        'func_url'  => $func_url,
                                        'func_data' => $trans_array,
                                    ];
            $result             = $this->stannpTransmission($trans_data);
            $recipient_response = json_decode($result, true);
            if (isset($recipient_response) && $recipient_response['success'] === true) {
                return ['success' => true, 'data' => $recipient_response];
            } else {
                return [
                            'success'   => false,
                            'message'   => 'recipient not deleted',
                            'error'     => $recipient_response['error'],
                        ];
            }
        } else {
            return ['success' => false, 'message' => 'recipient id params empty'];
        }
    }

    public function deleteAllRecipient() // delete all recipients from stannp API
    {
        $func_name          = 'deleteAllRecipients';
        $func_url           = 'recipients/deleteAll';
        $trans_array        = ['delete_all' => 1,];
        $trans_data         = [
                                    'func_name' => $func_name,
                                    'func_url'  => $func_url,
                                    'func_data' => $trans_array,
                                ];
        $result             = $this->stannpTransmission($trans_data);
        $recipient_response = json_decode($result, true);
        if (isset($recipient_response) && $recipient_response['success'] === true) {
            return ['success' => true, 'data' => $recipient_response];
        } else {
            return [
                        'success'   => false,
                        'message'   => 'all recipient deletion failed',
                        'error'     => $recipient_response['error'],
                    ];
        }
    }

    public function postPostcardToStannp($data) // print and post postcard call to stannp API
    {
        $func_name      = 'postPostcardToStannp';
        $func_url       = 'postcards/create';
        $campaign_id    = isset($data['campaign_id']) ? $data['campaign_id'] : '';
        $print_type     = isset($data['print_type']) ? $data['print_type'] : '';
        $print_size     = isset($data['print_size']) ? $data['print_size'] : '';
        $properties     = PropertyCampaign::where('campaign_id', $campaign_id)->get();
        $user_data      = $this->authUserData();
        if (isset($user_data) && $user_data['success'] === true) {
            $user_id = $user_data['user'];
        } else {
            return ['success' => false, 'message' => 'session expired login again'];
        }
        if (count($properties) > 0 && $print_type != '') {
                $response = [];
                foreach ($properties as $property) {
                    // set postcard size to send
                    $postcard_print_size = '';
                    $postcard_size       = [
                                                '1' => 'A6',
                                                '2' => 'A5',
                                                '3' => 'A5-ENV',
                                            ];
                    if (in_array($print_size, array_keys($postcard_size)) && $print_size != '') {
                        $postcard_print_size = $postcard_size[$print_size];
                    }
                    // fetch postcard data
                    $property->postcard = Letter::where('triggers', $property->trigger)
                                            ->where('user_id', $user_id)
                                            ->first();
                    if ($print_type === 2) {
                        $source         = public_path('letter-heads/blank_letter.pdf');
                    } else {
                        $source_data    = LetterHead::where('user_id', $user_id)->first();
                        $source         = storage_path('app/public/letter-heads/'.$source_data['file']);
                    }
                    // convert pdf to image using imagic or ghostscript to use as only png and jpg are supported
                    /*$im = new imagick('file.pdf[0]');
                    $im->setImageFormat('jpg');
                    header('Content-Type: image/jpeg');
                    echo $im;*/
                    $content            = '';
                    if ($property->postcard) {
                        $content        = $property->postcard->content;
                        $content        = str_replace('&nbsp;', '', $content);
                        // $content        = str_replace('&nbsp;', ' ', $content);
                        // $content        = str_replace('/n', '<br>', $content);
                    }
                    $postcode           = isset($property->edit_property_address) ? $property->edit_property_address : '';
                    $post_code          = [];
                    if ($postcode != '') {
                        $post_code          = explode("\n", $postcode);
                        // Test other custom postcodes
                        // $post_code          = [
                        //                             '0' => 'Lyncroft',
                        //                             '1' => 'Walnut Tree Avenue',
                        //                             '2' => 'Maidstone',
                        //                             '3' => '',
                        //                             '4' => 'ME15 9UU',
                        //                         ];
                        $valid_response     = $this->validatePostCodeStannp($post_code);
                        if (isset($valid_response['success']) && $valid_response['success'] === true && !empty($valid_response['data'])) { // If postcode valid, proceed with postcard posting to stannp api
                            $address_array      = [ 
                                                        $valid_response['data']->admin_ward,
                                                        $valid_response['data']->admin_district,
                                                        $valid_response['data']->admin_county,
                                                    ];
                            $address_string     = implode(', <br>', $address_array);
                            $trans_array        = [
                                                        'size'                  => $postcard_print_size,
                                                        'test'                  => true,
                                                        # 'template'              => '',
                                                        # 'recipient'             => '',
                                                        'recipient[title]'      => 'Test title',
                                                        'recipient[firstname]'  => 'Test First Name',
                                                        'recipient[lastname]'   => 'Test Last Name',
                                                        'recipient[address1]'   => $post_code[0],
                                                        'recipient[address2]'   => $address_string,
                                                        # 'recipient[address2]'   => 'Sampleland',
                                                        # 'recipient[city]'       => 'Barnstaple',
                                                        'recipient[country]'    => ($valid_response['data']->country == 'England') ? 'GB' : '',
                                                        'recipient[postcode]'   => $valid_response['data']->postcode,
                                                        # 'front'                 => 'https://www.stannp.com/assets/samples/a6-postcard-front.jpg',
                                                        # 'front'                 => public_path('letter-heads/a6-postcard-front-dev-edited.jpg'),
                                                        'front'                 => 'https://st.depositphotos.com/2274151/3518/i/950/depositphotos_35186549-stock-photo-sample-grunge-red-round-stamp.jpg',
                                                        # 'front'                 => "{$source}", // error
                                                        // 'back'                  => 'https://iacoa.com/wp-content/uploads/2018/01/sample-company-logo.jpg',
                                                        'message'               => "Hello Property Owner,<br> {$content}",
                                                        'signature'             => 'https://iacoa.com/wp-content/uploads/2018/01/sample-company-logo.jpg',
                                                        # 'padding'             => '',
                                                        # 'post_unverified'     => '',
                                                        # 'clearzone'           => '',
                                                        # 'addons'              => '',
                                                        # 'tags'                => '',
                                                    ];
                            $trans_data         = [
                                                        'func_name' => $func_name,
                                                        'func_url'  => $func_url,
                                                        'func_data' => $trans_array,
                                                        'user_id'   => $user_id,
                                                    ];
                            $result             = $this->stannpTransmission($trans_data);
                            $print_response     = json_decode($result, true);
                            if (isset($print_response) && $print_response['success'] === true) {
                                // Display anchor tag for postcard quick look
                                // echo "<embed src='{$print_response['data']['pdf']}'' width='800px' height='2100px' />";
                                // echo "<iframe src='{$print_response['data']['pdf']}' width='100%' height='500px'>";
                                echo "<a href='{$print_response['data']['pdf']}' target='_blank'> Preview Generated Postcard > <a>";
                                $property_response =  [
                                                            'success'   => true,
                                                            'message'   => 'postcard generated',
                                                            'data'      => $print_response['data'],
                                                        ];

                                // return $property_response;
                            } else {
                                $property_response =  [
                                                            'success'   => false,
                                                            'message'   => 'postcard not generated',
                                                            'error'     => $print_response['error']
                                                        ];

                                // return $property_response;
                            }
                            $response          += [$property->id => $property_response];
                        } else {
                            return ['success' => false, 'message' => 'invalid postcode', 'data' => $post_code];
                        }
                    }
                }
                return $response;
        } else {
            return ['success' => false, 'message' => 'campaign params empty'];
        }
    }

    public function getPostcard($data) // get a specific postcard from stannp API
    {
        $func_name      = 'getPostcard';
        $func_url       = 'postcards/get/';
        $postcard_id    = isset($data['postcard_id']) ? $data['postcard_id'] : '';
        if (isset($postcard_id) && $postcard_id != '') {
            $trans_array        = ['postcard_id'    => $postcard_id,];
            $trans_data         = [
                                        'func_name' => $func_name,
                                        'func_url'  => $func_url,
                                        'func_data' => $trans_array,
                                    ];
            $result             = $this->stannpTransmission($trans_data);
            $postcard_response  = json_decode($result, true);
            if (isset($postcard_response) && $postcard_response['success'] === true) {
                return ['success' => true, 'data' => $postcard_response['data']];
            } else {
                return [
                            'success'   => false,
                            'message'   => 'postcard fetch failed',
                            'error'     => $postcard_response['error'],
                        ];
            }
        } else {
            return ['success' => false, 'message' => 'postcard id params empty'];
        }
    }

    public function postLetterToStannp($data) // print and post letter call to stannp API
    {
        $func_name      = 'postLetterToStannp';
        $func_url       = 'letters/create';
        $campaign_id    = isset($data['campaign_id']) ? $data['campaign_id'] : '';
        $print_type     = isset($data['print_type']) ? $data['print_type'] : '';
        $print_size     = isset($data['print_size']) ? $data['print_size'] : '';
        $properties     = PropertyCampaign::where('campaign_id', $campaign_id)->get();
        $session_key    = config('propai.authSessionKey') ? config('propai.authSessionKey') : '';
        $user_data      = $this->authUserData();
        if (isset($user_data) && $user_data['success'] === true) {
            $user_id = $user_data['user'];
        } else {
            return ['success' => false, 'message' => 'session expired login again'];
        }
        if (count($properties) > 0 && $print_type != '') {
                $response = [];
                foreach ($properties as $property) {
                    // Set Letter size to send
                    $letter_print_size  = '';
                    $letter_size        = [
                                                '1' => 'A6',
                                                '2' => 'A5',
                                                '3' => 'A5-ENV',
                                            ];
                    if (in_array($print_size, array_keys($letter_size)) && $print_size != '') {
                        $letter_print_size = $letter_size[$print_size];
                    }
                    // fetch letter data
                    $property->letter   = Letter::where('triggers', $property->trigger)
                                            ->where('user_id', $user_id)
                                            ->first();
                    if ($print_type === 2) {
                        $source         = public_path('letter-heads/blank_letter.pdf');
                    } else {
                        $source_data    = LetterHead::where('user_id', $user_id)->first();
                        $source         = storage_path('app/public/letter-heads/' . $source_data['file']);
                    }
                    // convert pdf to image using imagic or ghostscript to use as only png and jpg are supported
                    /*$im = new imagick('file.pdf[0]');
                    $im->setImageFormat('jpg');
                    header('Content-Type: image/jpeg');
                    echo $im;*/
                    $content            = '';
                    if ($property->letter) {
                        $content        = $property->letter->content;
                        $content        = str_replace('&nbsp;', '', $content);
                        // $content        = str_replace('&nbsp;', ' ', $content);
                        // $content        = str_replace('/n', '<br>', $content);
                    }
                    $postcode           = isset($property->edit_property_address) ? $property->edit_property_address : '';
                    $post_code          = [];
                    if ($postcode != '') {
                        $post_code          = explode("\n", $postcode);
                        // Test other custom postcodes
                        // $post_code          = [
                        //                             '0' => 'Lyncroft',
                        //                             '1' => 'Walnut Tree Avenue',
                        //                             '2' => 'Maidstone',
                        //                             '3' => '',
                        //                             '4' => 'ME15 9UU',
                        //                         ];
                        $valid_response     = $this->validatePostCodeStannp($post_code);
                        if (isset($valid_response['success']) && $valid_response['success'] === true && !empty($valid_response['data'])) { // If postcode valid - proceed with letter posting to stannp api
                            $address_array      = [ 
                                                        $valid_response['data']->admin_ward,
                                                        $valid_response['data']->admin_district,
                                                        $valid_response['data']->admin_county,
                                                    ];
                            $address_string     = implode(', <br>', $address_array);
                            $trans_array        = [
                                                        'test'                  => true,
                                                        # 'recipient[title]'      => 'Test title',
                                                        # 'recipient[firstname]'  => 'Test First Name',
                                                        # 'recipient[lastname]'   => 'Test Last Name',
                                                        'recipient[address1]'   => $post_code[0],
                                                        'recipient[address2]'   => $address_string,
                                                        'recipient[country]'    => ($valid_response['data']->country == 'England') ? 'GB' : '',
                                                        'recipient[postcode]'   => $valid_response['data']->postcode,
                                                        # 'recipient'         => '', // when id is passed, that has been created in system beforehand
                                                        'background'            => 'https://st2.depositphotos.com/3440785/7566/v/950/depositphotos_75668165-stock-illustration-logo-small-green-tree.jpg',
                                                        'template'              => '',
                                                        'pages'                 => ['page0' => "Hello Property Owner,<br> {$content}"],
                                                        # 'file'                  => '', // Alternatively to using the template or pages parameters you could send a PDF / DOC file directly. This value can be either a binary file, a URL to a file or a base64 encoded string. Maximum of 10 pages.
                                                        # 'duplex'                => '', // True or False. Duplex means we print on both sides of the letter. Set this to false if you want to only print on the front of each page. Defaults to true.
                                                        'transactional'         => false, // True or False. Transactional mail is handled separatley from advertising mail and is not entilted to same discounts. Use this if your mail contains sensitive data. Defaults to false.
                                                        # 'clearzone'             => '', // True or False. If true we will overlay clear zones with a white background. Clearzones must be clear to keep a mailpiece machine readable. Defaults to true.
                                                        'post_unverified'       => '',
                                                        'tags'                  => '',
                                                        'addons'                => '',
                                                    ];
                            $trans_data         = [
                                                        'func_name' => $func_name,
                                                        'func_url'  => $func_url,
                                                        'func_data' => $trans_array,
                                                        'user_id'   => $user_id,
                                                    ];
                            $result             = $this->stannpTransmission($trans_data);
                            $print_response     = json_decode($result, true);
                            if (isset($print_response) && $print_response['success'] === true) {
                                // Display anchor tag for Post Letter quick look
                                // echo "<embed src='{$print_response['data']['pdf']}'' width='800px' height='2100px' />";
                                // echo "<iframe src='{$print_response['data']['pdf']}' width='100%' height='500px'>";
                                echo "<a href='{$print_response['data']['pdf']}' target='_blank'> Preview Generated Post Letter > <a>";
                                $property_response  = [
                                                            'success'   => true,
                                                            'message'   => 'post letter generated',
                                                            'data'      => $print_response['data'],
                                                        ];
                            } else {
                                $property_response  = [
                                                            'success'   => false,
                                                            'message'   => 'post letter not generated',
                                                            'error'     => $print_response['error'],
                                                        ];
                            }
                            $response[]         = [$property_response];
                        } else {
                            return ['success' => false, 'message' => 'invalid postcode', 'data' => $post_code];
                        }
                    }
                }
                return $response;
        } else {
            return ['success' => false, 'message' => 'campaign params empty'];
        }
    }

    public function getLetter($data) // get specific letter from stannp API
    {
        $func_name  = 'getLetter';
        $func_url   = 'letters/get/';
        $letter_id  = isset($data['id']) ? $data['id'] : '';
        if (isset($letter_id) && $letter_id != '') {
            $trans_array        = ['letter_id' => $letter_id,];
            $trans_data         = [
                                        'func_name' => $func_name,
                                        'func_url'  => $func_url,
                                        'func_data' => $trans_array,
                                    ];
            $result             = $this->stannpTransmission($trans_data);
            $letter_response    = json_decode($result, true);
            if (isset($letter_response) && $letter_response['success'] === true) {
                return ['success' => true, 'data' => $letter_response];
            } else {
                return [
                            'success'   => false,
                            'message'   => 'letter not fetched',
                            'error'     => $letter_response['error'],
                        ];
            }
        } else {
            return ['success' => false, 'message' => 'letter id params empty'];
        }
    }

    public function cancelLetter($data) // cancel specific letter from stannp API
    {
        $func_name  = 'cancelLetter';
        $func_url   = 'letters/cancel';
        $letter_id  = isset($data['id']) ? $data['id'] : '';
        if (isset($letter_id) && $letter_id != '') {
            $trans_array        = ['id' => $letter_id,];
            $trans_data         = [
                                        'func_name' => $func_name,
                                        'func_url'  => $func_url,
                                        'func_data' => $trans_array,
                                    ];
            $result             = $this->stannpTransmission($trans_data);
            $letter_response    = json_decode($result, true);
            if (isset($letter_response) && $letter_response['success'] === true) {
                return ['success' => true, 'data' => $letter_response];
            } else {
                return [
                            'success'   => false,
                            'message'   => 'letter could not be canceled',
                            'error'     => $letter_response['error'],
                        ];
            }
        } else {
            return ['success' => false, 'message' => 'letter id params empty'];
        }
    }

    public function validatePostCodeStannp($data) // validate postcode using stannp postcode validation service and retrieve using opensource postcode fetch
    {
        $func_name  = 'validatePostCodeStannp';
        $func_url   = 'addresses/validate';
        if (isset($data)) {
            $post_code_count    = count($data);
            $postcode           = end($data);
            $postcode_test      = strtoupper(str_replace(' ', '', $postcode));
            // Check Postcode Pattern
            if (preg_match('/(^[A-Z]{1,2}[0-9R][0-9A-Z]?[\s]?[0-9][ABD-HJLNP-UW-Z]{2}$)/i',$postcode_test) || preg_match('/(^[A-Z]{1,2}[0-9R][0-9A-Z]$)/i',$postcode_test)) {
                // Check with API Postcode validator
                $trans_array        = [
                                            'address1'  => $data[0] !== '' ? $data[0] : '',
                                            'postcode'  => $postcode !== '' ? $postcode : '',
                                        ];
                $trans_data         = [
                                            'func_name' => 'validatePostCodeStannp',
                                            'func_url'  => $func_url,
                                            'func_data' => $trans_array,
                                        ];
                $result             = $this->stannpTransmission($trans_data);
                $property_response  = json_decode($result, true);
                if (isset($property_response['data']['is_valid']) && $property_response['data']['is_valid'] === false) {
                    return ['success' => false, 'message' => 'invalid postcode', 'data' => $postcode];
                } else {
                    // Provide all postcode data, when postcode is valid
                    $jsonurl    = 'https://api.postcodes.io/postcodes/'.$postcode;
                    $json       = $this->curl_request($jsonurl);
                    $decoded    = json_decode($json);
                    if ($decoded->status === 200) {
                        return ['success' => true, 'data' => $decoded->result];
                    } else {
                        return [
                                    'success' => false,
                                    'message' => 'fetching postcode details failed',
                                    'error' => $decoded->error
                                ];
                    }
                    return false;
                }
            } else {
                return ['success' => false, 'message' => 'invalid postcode pattern'];
            }
        } else {
            return ['success' => false, 'message' => 'postcode param empty'];
        }
    }

    public function validatePostcode($data) // validate postcode - cURL request
    {
        $postcode = isset($data['postcode']) ? $data['postcode'] : '';
        if (isset($postcode) && $postcode != '') {
            $postcode_test = strtoupper(str_replace(' ', '', $postcode));
            if (preg_match('/(^[A-Z]{1,2}[0-9R][0-9A-Z]?[\s]?[0-9][ABD-HJLNP-UW-Z]{2}$)/i',$postcode_test) || preg_match('/(^[A-Z]{1,2}[0-9R][0-9A-Z]$)/i',$postcode_test)) {
                // Provide validation for the postcode data, when postcode is valid
                $jsonurl    = 'https://api.postcodes.io/postcodes/' . $postcode;
                $json       = $this->curl_request($jsonurl);
                $decoded    = json_decode($json);
                if ($decoded->status === 200) {
                    return ['success' => true, 'data' => $decoded->result];
                } else {
                    return [
                                'success'   => false,
                                'message'   => 'postcode invalid',
                                'error'     => $decoded->error
                            ];
                }
            } else {
                return ['success' => false, 'message' => 'invalid postcode pattern'];
            }
        } else {
            return ['success' => false, 'message' => 'postcode param not provided'];
        }
    }

    public function getNearestPostcodes($data) // get nearest postcodes - cURL request
    {
        $postcode = isset($data['postcode']) ? $data['postcode'] : '';
        if (isset($postcode) && $postcode != '') {
            $postcode_test = strtoupper(str_replace(' ', '', $postcode));
            if (preg_match('/(^[A-Z]{1,2}[0-9R][0-9A-Z]?[\s]?[0-9][ABD-HJLNP-UW-Z]{2}$)/i',$postcode_test) || preg_match('/(^[A-Z]{1,2}[0-9R][0-9A-Z]$)/i',$postcode_test)) {
                // Provide all nearest postcode data, when postcode is valid
                $jsonurl    = 'https://api.postcodes.io/postcodes/' . $postcode . '/nearest';
                $json       = $this->curl_request($jsonurl);
                $decoded    = json_decode($json);
                if ($decoded->status === 200) {
                    array_shift($decoded->result);
                    return ['success' => true, 'data' => $decoded->result];
                } else {
                    return [
                                'success'   => false,
                                'message'   => 'fetching nearest postcode data failed',
                                'error'     => $decoded->error
                            ];
                }
            } else {
                return ['success' => false, 'message' => 'invalid postcode pattern'];
            }
        } else {
            return ['success' => false, 'message' => 'postcode param not provided'];
        }
    }

    public function getNearestPostcodesWithCoordinates($data) // get nearest postcodes using provided coordinates - cURL request
    {
        $lon = isset($data['lon']) ? $data['lon'] : '';
        $lat = isset($data['lat']) ? $data['lat'] : '';
        if (isset($lon) && $lon != '' && isset($lat) && $lat != '') {
            $jsonurl    = 'https://api.postcodes.io/postcodes?lon=' . $lon . '&lat=' . $lat;
            $json       = $this->curl_request($jsonurl);
            $decoded    = json_decode($json);
            if ($decoded->status === 200) {
                return ['success' => true, 'data' => $decoded->result];
            } else {
                return [
                            'success'   => false,
                            'message'   => 'fetching nearest postcode data failed',
                            'error'     => $decoded->error
                        ];
            }
        } else {
            return ['success' => false, 'message' => 'latitude-longitude coordinates not provided'];
        }
    }

    public function autoCompletePostcode($data) // validate postcode - cURL request
    {
        $postcode = isset($data['postcode']) ? $data['postcode'] : '';
        if (isset($postcode) && $postcode != '') {
            $postcode_test = strtoupper(str_replace(' ', '', $postcode));
            // Provide auto completion for the postcode string passed, max limit 10 postcodes
            $jsonurl    = 'https://api.postcodes.io/postcodes/' . $postcode . '/autocomplete';
            $json       = $this->curl_request($jsonurl);
            $decoded    = json_decode($json);
            if ($decoded->status === 200 && $decoded->result === null) {
                return [
                            'success'   => true,
                            'data'      => $decoded->result,
                            'message'   => 'no data in server about the postcode passed'
                        ];
            } else if ($decoded->status === 200) {
                return ['success' => true, 'data' => $decoded->result];
            } else {
                return [
                            'success'   => false,
                            'message'   => 'postcode invalid',
                            'error'     => $decoded->error
                        ];
            }
        } else {
            return ['success' => false, 'message' => 'postcode param not provided'];
        }
    }

    public function stannpTransmission($data) // All api call and responce method towards stannp api service
    {
        // curl API with passed parameters of diffrent functions
        $func_name  = isset($data['func_name']) ? $data['func_name'] : '';
        $func_data  = isset($data['func_data']) ? $data['func_data'] : '';
        $func_url   = isset($data['func_url']) ? $data['func_url'] : '';
        $user_id    = isset($data['user_id']) ? $data['user_id'] : '';
        // call sanitize function data variable
        $func_data  = $this->sanitize_arrays($func_data);
        if ($func_name != '' && $func_url != '') {
            try {
                // provide initial data wrt function call name
                switch ($func_name) {
                    case 'stannpAccountInfo':
                        break;
                    case 'addRecipient':
                        break;
                    case 'getRecipient':
                        $func_url   = $func_url . $func_data['recipient_id'];
                        $func_data  = [];
                        break;
                    case 'getRecipientlist':
                        $func_url   = $func_url . $func_data['group_id'];
                        $func_data  = [];
                        break;
                    case 'deleteRecipient':
                        break;
                    case 'postPostcardToStannp':
                        break;
                    case 'getPostcard':
                        $func_url   = $func_url . $func_data['postcard_id'];
                        $func_data  = [];
                        break;
                    case 'postLetterToStannp':
                        break;
                    case 'getLetter':
                        $func_url   = $func_url . $func_data['letter_id'];
                        $func_data  = [];
                        break;
                    case 'cancelLetter':
                        break;
                    default:
                        break;
                }
                $api_key = config('stannp_api.appKey') ? config('stannp_api.appKey') : '';
                $api_url = config('stannp_api.appUrl') ? config('stannp_api.appUrl') : '';
                if ($api_key != '' && $api_url != '') {
                    $postdata   = http_build_query($func_data);
                    $opts       = [
                                        'http' => [
                                                        'method'        => 'POST',
                                                        'header'        => 'Content-type: application/x-www-form-urlencoded',
                                                        'content'       => $postdata,
                                                        'ignore_errors' => true,
                                                    ]
                                    ];
                    $context    = stream_context_create($opts);
                    $trans_url  = $api_url . $func_url . '?api_key=' . $api_key;
                    if ($func_name === 'cancelLetter') { // provide specific params wrt function call
                        // dd($context);
                        // dd($trans_url);
                    }
                    $result     = file_get_contents($trans_url, false, $context);
                    if ($func_name === '') { // provide specific result wrt function call
                        // dd($result);
                    }
                    return $result;
                } else {
                    return ['success' => false, 'message' => 'api credentials empty'];
                }
            } catch(Exception $e) {
                 return [
                            'success'   => false,
                            'message'   => 'api transmission failed',
                            'error'     => $e->getMessage()
                        ];
            }
        } else {
            return ['success' => false, 'message' => 'function params empty'];
        }
    }

    public function curl_request($jsonurl) // cURL calling method
    {
        $ch         = curl_init();
        curl_setopt($ch, CURLOPT_URL, str_replace(' ', '%20', $jsonurl));
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
        curl_setopt($ch, CURLOPT_HTTPHEADER, ['Content-Type: application/json']);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $response   = curl_exec($ch);
        curl_close($ch);

        return $response;
    }

    public function authUserData()
    {
        $session_key    = config('propai.authSessionKey') ? config('propai.authSessionKey') : '';
        $user           = json_decode(Session::get($session_key));
        if (isset($user) && $user->id !== '') {
            return ['success' => true, 'user' => $user->id];
        } else {
            return ['success' => false, 'message' => 'session expired login again'];
        }
    }

    public function generateRandomString($data)
    {
        $length             = isset($data['length']) ? $data['length'] : 10;
        $characters         = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength   = strlen($characters);
        $randomString       = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[mt_rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }

    public function sanitize_arrays($data)
    {
        $sanitizedInput = []; 
        foreach ($data as $key => $value) {
            if (is_string($key)) {
                $key1   = strip_tags($key); 
                $key2   = htmlentities($key1);
                $key    = stripslashes($key2);
                // $key    = mysqli_real_escape_string($link,$key);
            }
            if (is_string($value)) {
                $value1 = strip_tags($value);
                $value2 = htmlentities($value1);
                $value  = stripslashes($value2);
                // $value  = mysqli_real_escape_string($link,$value);
            }
            $sanitizedInput[$key] = $value;
        }
        return $sanitizedInput;
    }
}
