<?php

namespace App\Http\Controllers;

use App\Model\Agency;
use App\Model\ErrorLog;
use App\Model\HouseNumber;
use App\Model\Outcode;
use App\Model\PostCode;
use App\Model\PropertyHistory;
use App\Model\PropertyHouseNumber;
use App\Model\PropertyPriceHistory;
use App\Model\PropertySaleHistory;
use App\Model\Withdrawn;
use Carbon\Carbon;
use File;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Validator;

class ApiPropertyController extends Controller
{
    public function importOutcode(Request $request)
    {
        $path = $request->file('file')->getRealPath();
        $dataArr = array_map('str_getcsv', file($path));
        foreach ($dataArr as $key => $val) {
            foreach ($val as $k => $v) {
                if ($key >= 1) {
                    Outcode::create([
                        'outcode' => $v,
                        'price' => 0,
                    ]);
                }
            }
        }

        return response()->json(['msg' => 'successfully outcode inserted']);
    }

    public function postcode(Request $request)
    {
        $list = Agency::get();
        foreach ($list as $k => $v) {
            $newstring = substr($v->address, -8);

            $agentPostCode = preg_replace('/[^A-Z0-9\-]/', ' ', $newstring);
            $str = ltrim($agentPostCode);
            Agency::where('id', $v->id)->update([
                'postcode' => $str,
            ]);
        }

        return response()->json(['msg' => 'successfully postcode inserted in agent']);
    }

    public function updateHouseNo(Request $request)
    {
        
        $list = Agency::get();
        foreach ($list as $k => $v) {
            $arr = explode(',', $v['address']);
            $first = $arr[0];
            $string = str_replace(' ', '', $first);
            Agency::where('id', $v->id)->update([
                'house_no' => $string,
            ]);
        }

        return response()->json(['msg' => 'successfully house no inserted in agent']);
    }

    public function updateAgentProperty(Request $request)
    {
        $list = Agency::get(['postcode', 'house_no', 'id'])->unique(['postcode'])->toArray();
        $list2 = Agency::whereIn('postcode', $list)->get(['postcode', 'house_no', 'id'])->unique(['house_no'])->toArray();
       
        foreach ($list2 as $k => $v) {
            $a = Agency::where('postcode', $v['postcode'])
                ->where('house_no', $v['house_no'])->get(['id'])->toArray();

            PropertyHistory::whereIn('agency_id', $a)->update([
                'agency_id' => $v['id'],
            ]);
        }

        return response()->json(['msg' => 'successfully agent id inserted in property']);
    }

    public function importProperty(Request $request)
    {
        @ini_set('max_execution_time',0);
        @set_time_limit(-1);
        @ini_set('memory_limit', '-1');
        try {
            $validator = Validator::make($request->all(), [
                'upload_file' => 'required',
                'site' => 'required',
                'outcode' => 'required',
            ]);

            if ($validator->fails()) {
                $error_messages = implode(',', $validator->messages()->all());

                $response_array = ['success' => false, 'error' => ('invalid_input'), 'error_code' => 101, 'error_messages' => $error_messages];

                return $response = response()->json($response_array, 200);
            }
            $sitename = $request->site;

            if (!empty($request->file('upload_file'))) {
                $path = $request->file('upload_file')->getRealPath();

                try {
                    $getData = json_decode(file_get_contents($path));
                    $propertyArray = json_decode(json_encode($getData), true);
                } catch (\Exception $e) {
                    return response()->json([
                        'msg' => 'Invalid file',
                        'error' => $e->getMessage(),
                    ]);
                }
                if (empty($propertyArray)) {
                    return response()->json([
                        'msg' => 'Invalid file',
                    ]);
                }
            }

            $filename = $request->file('upload_file')->getClientOriginalName();

            $explode = explode('-', $filename);
            if ($request->outcode != $explode[0]) {
                return response()->json([
                    'msg' => 'Please enter valid outcode',
                ]);
            }
            $explode2 = explode('.', $explode[1]);
            if ($request->site != $explode2[0]) {
                return response()->json([
                    'msg' => 'Please enter valid site name',
                ]);
            }

            $updatedProp = 0;
            $insertCount = 0;
            $withoutPostCode = 0;
            $withoutPostCodeCount = 0;
            $stcCount=0;
            $offerCount=0;
            $withoutAgent=0;
            $withoutHouseNrCount=0;
            
            //ErrorLog Create

            $log = ErrorLog::create([
                'outcode' => $request->outcode,
                'portal' => $request->site,
            ]);
            $uploadStartTime = time();
            //checking site name
            if ('zoopla' == $sitename) {
                $portal = 1;
            } else {
                $portal = 0;
            }
            $oldProperties = PropertyHistory::leftjoin('postcode', 'postcode.id', 'property_history.postcode_id')
                ->leftjoin('outcode', 'outcode.id', 'postcode.outcode_id')
                ->where('outcode.outcode', $request->outcode)
                ->where('property_history.portal', $portal)
                ->pluck('property_history.pid')->toArray();
            $property = '';
            $houseNumber = '';
            $propertyExists = '';
            $data1 = [];
            $Data1 = [];
            $data2 = [];
            $Data2 = [];
            $propertyPriceHistoryGet = '';
            $stc = '';
            $outcode = '';
            $postcode = '';
            $agencyGet = '';
            $pid = [];
            $zooplaUrl = '';
            $rightmoveUrl = '';
            $status = '';
            $newMarket = '';
            $soldDate = '';
            $priceDropDate = '';
            $extractPrice = '';
            $houseNumberGet = '';
            $newCheck1 = '';
            $newCheck2 = '';
            $newCheck3 = '';
            $newCheck4 = '';
            $newCheck5 = '';
            $stcData1 = '';
            $zooplaPropertyGet='';
            $zooplaPropertyGetPrice ='';
            $propertyRM='';
            $soldRMPriceDifference ='';
            $negativePrice='';
            $rightMovePropertyGetPrice='';
            $rightMovePropertyGet='';
            $priceType = '';
            $priceDropDate = '';
            $getHouseNumber='';
            $propertyHouseNumberData='';
            $deleteWithdrawn='';
            $priceDrop='';
            $allPropertyHouseNumber='';
            $propertyGetID ='';
            foreach ($propertyArray as $k => $v) {
                //Outcode Insertion  extracting from postcode .

                if (preg_match('/^[0-9]+$/', $v['pid'])) {
                    // if (!empty($v['postcode'])) {
                        // $explode = explode(' ', $v['postcode']);
                        if (Outcode::where('outcode', $request->outcode)->exists()) {
                            $Outcode = Outcode::where('outcode', $request->outcode)->first();
                            Outcode::where('id', $Outcode->id)->update([
                                'price' => 0,
                            ]);
                            $outcode = Outcode::where('id', $Outcode->id)->first();
                        } else {
                            $outcode = Outcode::create([
                                'outcode' => strtoupper($request->outcode),
                                'price' => 0,
                            ]);
                        }
                        //Checking length of postcode > 5  if not property wont insert.
                        $length = strlen($v['postcode']);
                    // }

                    if (!empty($v['postcode'] && $length > 5)) {
                        if (PostCode::where('postcode', $v['postcode'])->where('outcode_id', $outcode->id)->exists()) {
                            $postcode = PostCode::where('postcode', $v['postcode'])->first();
                        } else {
                            $postcode = PostCode::create([
                                'outcode_id' => $outcode->id,
                                'postcode' => $v['postcode'],
                            ]);
                        }
                    }
                    else{

                        if (PostCode::where('postcode', 'Unknown')->where('outcode_id', $outcode->id)->exists()) {
                            $postcode = PostCode::where('postcode', 'Unknown')->where('outcode_id', $outcode->id)->first();
                        } else {
                            $postcode = PostCode::create([
                                'outcode_id' => $outcode->id,
                                'postcode' => 'Unknown',
                            ]);
                        }
                            $withoutPostCode = $withoutPostCode + 1;
                        
                    }
                        //checking site name
                        if ('zoopla' == $sitename) {
                            $portal = 1;
                        } else {
                            $portal = 0;
                        }

                        //Agency -> additionally adding house no and postcode , that extracted from agency address
                        // if agent name null no need to insert anything
                        if (!empty($v['agent_name'])) {
                            $newstring = substr($v['agent_address'], -8);
                            $agentPostCode = preg_replace('/[^A-Z0-9\-]/', ' ', $newstring);
                            $str = ltrim($agentPostCode);

                            $arr = explode(',', $v['agent_address']);
                            $first = $arr[0];
                            $string = str_replace(' ', '', $first);

                            $agencyGet = Agency::where('postcode', $str)
                                ->where('house_no', $string)
                            ;

                            if ($agencyGet->exists()) {
                                if (!empty($v['agent_url'])) {
                                    $agency = $agencyGet->first();
                                    if (1 == $portal) {
                                        if (null == $agency->zoopla_url) {
                                            $agencyGet->update([
                                                'zoopla_url' => $v['agent_url'],
                                            ]);
                                        }
                                    } else {
                                        if (null == $agency->rightmove_url) {
                                            $agencyGet->update([
                                                'rightmove_url' => $v['agent_url'],
                                            ]);
                                        }
                                    }
                                }
                            } else {
                                if (!empty($v['agent_url'])) {
                                    if (1 == $portal) {
                                        $zooplaUrl = $v['agent_url'];
                                        $rightmoveUrl = null;
                                    } else {
                                        $rightmoveUrl = $v['agent_url'];
                                        $zooplaUrl = null;
                                    }
                                } else {
                                    $zooplaUrl = null;
                                    $rightmoveUrl = null;
                                }

                                $agency = Agency::create([
                                    'name' => !empty($v['agent_name']) ? $v['agent_name'] : null,
                                    'address' => !empty($v['agent_address']) ? $v['agent_address'] : null,
                                    // 'agent_url' => !empty($v['agent_url']) ? $v['agent_url'] : null,
                                    'zoopla_url' => $zooplaUrl,
                                    'rightmove_url' => $rightmoveUrl,
                                    'portal' => $portal,
                                    'postcode' => $str,
                                    'house_no' => $string,
                                ]);
                            }
                        }else{

                            if(Agency::where('name','Unknown')->exists()){
                                $agency=Agency::where('name','Unknown')->first();
                            }else{
                            $agency = Agency::create([
                                'name' =>  'Unknown',
                                'address' =>  null,
                                // 'agent_url' => !empty($v['agent_url']) ? $v['agent_url'] : null,
                                'zoopla_url' => NULL,
                                'rightmove_url' => NULL,
                                'portal' => $portal,
                                'postcode' => NULL,
                                'house_no' => NULL,
                            ]);
                            }
                            $withoutAgent = $withoutAgent+ 1;
                        }
                        //Checking price
                        if ('OA' == $v['price'] || 'POA' == $v['price'] || 0 == $v['price']) {
                            $price = 0;
                            $poa = 1;
                        } else {
                            $price = $v['price'];
                            $poa = 0;
                        }

                        ///////////////// RIGHTMOVE OR ZOOPLA//////////////////////////////////

                        if ('zoopla' == $sitename) {
                            $priceDropDate = null;
                            $extractPrice = null;
                            // Setting Portal URL
                            // if (null != $v['link']) {
                            //     $zoopla_url = 'https://www.zoopla.co.uk';
                            //     $portalUrl = $zoopla_url.''.$v['link'];
                            // } else {
                            //     $portalUrl = null;
                            // }
                            if (null != $v['link']) {
                                if (str_contains($v['link'], 'https://www.zoopla.co.uk')) {
                                    $portalUrl=$v['link'];
                                }
                                else{                   
                                    $zoopla_url = 'https://www.zoopla.co.uk';
                                    $portalUrl = $zoopla_url.''.$v['link'];
                                }
                             } else {
                                    $portalUrl = null;
                                }
                               
                            //Pid exists Update
                            if (PropertyHistory::where('pid', $v['pid'])->exists()) {
                                // Price drop manually for update properties

                                // die('exists');
                                $exists = 1;
                                $zooplaPropertyGet = PropertyHistory::where('pid', $v['pid'])->first();
                                $propertyRM = PropertyPriceHistory::where('property_history_id', $zooplaPropertyGet->id);
                                if ($propertyRM->exists()) {
                                    $zooplaPropertyGetPrice = $zooplaPropertyGet->price;
                                } else {
                                    $zooplaPropertyGetPrice = 0;
                                }

                                PropertyHistory::where('pid', $v['pid'])->update([
                                    'sold_stc' => isset($v['sold_stc']) ? $v['sold_stc'] : 0,
                                    'under_offer' => isset($v['under_offer']) ? $v['under_offer'] : 0,
                                    'price' => $price,
                                    'poa' => $poa,
                                    'unconfirmed' => $v['unconfirmed'],
                                ]);

                                $property = PropertyHistory::where('pid', $v['pid'])->first();

         

                                $status = 1;
                                $newMarket = 0;
                                $soldDate = null;
                                $priceDropDate = null;
                                $extractPrice = null;

                                //Price drop in Conditions checking manualy by getting price from previus property and current price
                                if (PropertyPriceHistory::where('property_history_id', $zooplaPropertyGet->id)
                                    ->exists()
                                ) {
                                    if (0 == $poa) {
                                        $soldRMPriceDifference = $v['price'] - $zooplaPropertyGetPrice;
                                        if (($soldRMPriceDifference < 0)) {
                                            $negativePrice = abs($soldRMPriceDifference);
                                            $priceType = 'Price reduced by £'.$negativePrice;
                                            $extractPrice = $soldRMPriceDifference;

                                            $priceDropDate = Carbon::now()->toDateTimeString();
                                        } elseif (($soldRMPriceDifference > 0)) {
                                            $priceType = 'Price increased by £'.$soldRMPriceDifference;
                                            $extractPrice = $soldRMPriceDifference;

                                            $priceDropDate = Carbon::now()->toDateTimeString();
                                        } elseif ((0 == $soldRMPriceDifference)) {
                                            $extractPrice = null;
                                            $priceType = 'Last sold';
                                            $priceDropDate = null;
                                        }
                                    }
                                }

                                // Adding price drop manually in price histry tabel
                                if (1 == $exists) {
                                    if (null != $extractPrice) {
                                        PropertyPriceHistory::create([
                                            'property_history_id' => $property->id,
                                            'event_date' => $priceDropDate,
                                            'type' => $priceType,
                                            'price' => $price,
                                        ]);
                                    }
                                }
                                // Checking house number not null and updateOrCreate
                                if(PropertyHouseNumber::where('property_history_id',$property->id)->exists()){
                                   
                                    if (null != $v['house_no']) {
                                      
                                    $propertyHouseNumberData =PropertyHouseNumber::where('property_history_id',$property->id)->first();
                                    HouseNumber::where('id', $propertyHouseNumberData->house_number_id)->update([
                                        'house_number' => !empty($v['house_no']) ? $v['house_no'] : null,
                                        'latest_price' => !empty($price) ? $price : null,
                                        'sold_date' => $soldDate,
                                        'new_to_market' => $newMarket,
                                        
                                    ]);
                                    $houseNumber=HouseNumber::where('id', $propertyHouseNumberData->house_number_id)->first();
                               } else{
                                
                                $propertyHouseNumberData =PropertyHouseNumber::where('property_history_id',$property->id)->first();
                                HouseNumber::where('id', $propertyHouseNumberData->house_number_id)->update([
                                   
                                    'latest_price' => !empty($price) ? $price : null,
                                    'sold_date' => $soldDate,
                                    'new_to_market' => $newMarket,
                                    
                                ]);
                                $houseNumber=HouseNumber::where('id', $propertyHouseNumberData->house_number_id)->first();
                               }
                            }
                                else{
                                
                                if (null != $v['house_no']) {
                                    
                                    $housenumber = HouseNumber::where('house_number', $v['house_no'])->where('postcode_id', $postcode->id);
                                    if ($housenumber->exists()) {
                                        $houseNumber = $housenumber->first();

                                        HouseNumber::where('id', $houseNumber->id)->update([
                                            'latest_price' => !empty($price) ? $price : null,
                                            
                                        ]);
                                    // $houseNumber = $housenumber->first();
                                    } else {
                                        $houseNumber = HouseNumber::create([
                                            'house_number' => !empty($v['house_no']) ? $v['house_no'] : null,
                                            'latest_price' => !empty($price) ? $price : null,
                                            'sold_date' => $soldDate,
                                            'postcode_id' => $postcode->id,
                                            'new_to_market' => $newMarket,
                                            
                                        ]);
                                    }
                                    // market status house number doubt
                                    $houseNumberGet = HouseNumber::where('id', $houseNumber->id);

                                    // Updating house number and property id in house number table
                                    $allPropertyHouseNumber = PropertyHouseNumber::where('house_number_id', $houseNumber->id);
                                    if (!$allPropertyHouseNumber
                                        ->where('property_history_id', $property->id)->exists()) {
                                        $propertyHouseNumber = PropertyHouseNumber::create([
                                            'house_number_id' => $houseNumber->id,
                                            'property_history_id' => $property->id,
                                        ]);
                                    }
                                } 
                                // else {
                                    
                                //     // $houseNumber = null;
                                //     $getHouseNumber=PropertyHouseNumber::where('property_history_id',$property->id);
                                //     if($getHouseNumber->exists()){
                                //         $propertyHouseNumberData =$getHouseNumber->first();
                                //         HouseNumber::where('id', $propertyHouseNumberData->house_number_id)->update([
                                           
                                           
                                //             'latest_price' => !empty($price) ? $price : null,
                                //             'sold_date' => $soldDate,
                                //             'new_to_market' => $newMarket,
                                //         ]);
                                //         $houseNumber=HouseNumber::where('id', $propertyHouseNumberData->house_number_id)->first();
                                //     }
                                // }
                            }
                        //if withdrawn exists it removes from price history table 
                            $deleteWithdrawn = PropertyPriceHistory::where('property_history_id', $property->id)
                            ->where('type', 'Withdrawn')
                            ->orderby('created_at', 'desc')
                            ->orderby('id', 'desc');

                            if ($deleteWithdrawn->exists()) {
                                if(!empty($houseNumber)){
                                $houseNumberGet = HouseNumber::where('id', $houseNumber->id);
                                 $houseNumberGet->update([
                                    'latest_status' => 1,
                                    'withdrawn_date' => null,
                                    'price_drop_date' => null,
                                    'price_change_diff' => null,
                                ]);
                                 }
                                 if ($property) {
                                    $propertyGetID = PropertyHistory::where('id', $property->id);
                                    $propertyGetID->update([
                                        'property_current_status' =>1,
                                        'for_sale' => 1,
                                        'withdrawn_date' => null,
                                        'price_drop_date' => null,
                                        'price_drop' => null,
                                    ]);
                                }
                            $deleteWithdrawn->delete();
                            }
                                //For count of property gets updated in db
                                $propertyExists = 1;
                            } else {
                                
                                $newMarket = 1;
                                $priceDropDate = null;
                                $extractPrice = null;
                                // if (!empty($v['agent_name'])) {
                                // if (!empty($agency)) {
                                    // $priceDropDate = null;
                                    // $extractPrice = null;
                                    //For count of properties inserted in db
                                    $propertyExists = 0;
                                    //Price drop Condition
                                    // $zooplaPropertyGet = (object) array();

                                    // $zooplaPropertyGet->id = null;

                                    $exists = 0;
                                    //Create new property if have new pid
                                    $property = PropertyHistory::create([
                                        'portal' => 1,
                                        'pid' => $v['pid'],
                                        'for_sale' => 1,
                                        'agency_id' => $agency->id,
                                        'postcode_id' => $postcode->id,
                                        'title' => $v['title'],
                                        'property_type' => $v['type'],
                                        'bed_rooms' => !empty($v['bedroom']) ? $v['bedroom'] : null,
                                        'sold_stc' => isset($v['sold_stc']) ? $v['sold_stc'] : 0,
                                        'under_offer' => isset($v['under_offer']) ? $v['under_offer'] : 0,
                                        'price' => $price,
                                        'poa' => $poa,
                                        'portal_url' => $portalUrl,
                                        'address' => $v['address'],
                                        // 'description' => $v['description'],
                                        'latitude' => isset($v['latitude']) ? $v['latitude'] : null,
                                        'longitude' => isset($v['longitude']) ? $v['longitude'] : null,
                                        'listed_date' => !empty($v['first_listed']) ? $v['first_listed'] : Carbon::now()->toDateString(),
                                        'event_date' => Carbon::now()->toDateString(),
                                        'unconfirmed' => $v['unconfirmed'],
                                    ]);
                                    // Updating  sale history
                                    if (!empty($v['last_sold_price']) && !empty($v['last_sold'])) {
                                        if (!PropertySaleHistory::where('property_history_id', $property->id)
                                            ->where('price', $v['last_sold_price'])
                                            ->where('sale_date', $v['last_sold'])
                                            ->exists()) {
                                            $saleHistory = PropertySaleHistory::create([
                                                'property_history_id' => $property->id,
                                                'price' => !empty($v['last_sold_price']) ? $v['last_sold_price'] : 0,
                                                'sale_date' => !empty($v['last_sold']) ? $v['last_sold'] : null,
                                            ]);
                                        }
                                    }

                                    //Price history array based on that status
                                    if (!empty($v['price_history'])) {
                                        if ('First listed' == $v['price_history'][0]['type']) {
                                            $status = 1;
                                            $newMarket = 1;
                                            $soldDate = null;
                                        } elseif ('Sold' == $v['price_history'][0]['type']) {
                                            // $status = 2;
                                            $newMarket = 0;
                                            $soldDate = $v['price_history'][0]['date'];
                                        } elseif ('Withdrawn' == $v['price_history'][0]['type']) {
                                            // $status = 3;
                                            $newMarket = 0;

                                            $soldDate = null;
                                        } else {
                                            $status = 1;
                                            $newMarket = 0;
                                            $soldDate = null;
                                        }
                                        // Inserting Price History

                                        $propertyPriceHistoryGet = PropertyPriceHistory::where('property_history_id', $property->id);
                                        foreach (array_reverse($v['price_history']) as $Key => $Value) {
                                            if (!$propertyPriceHistoryGet
                                                ->where('type', $Value['type'])
                                                ->where('event_date', $Value['date'])
                                                ->where('price', $Value['price'])
                                            // ->orderby('event_date', $Value['date'])
                                                ->exists()) {
                                                $priceHistory = PropertyPriceHistory::create([
                                                    'property_history_id' => $property->id,
                                                    'type' => !empty($Value['type']) ? $Value['type'] : null,
                                                    'event_date' => !empty($Value['date']) ? $Value['date'] : null,
                                                    'price' => !empty($Value['price']) ? $Value['price'] : null,
                                                ]);
                                            }
                                        }
                                        //Checking price drop in price history if there is price string , takes only number from that string and insert in house number table
                                        $propertyPriceHistoryGet = PropertyPriceHistory::where('property_history_id', $property->id);
                                        $newCheck1 = $propertyPriceHistoryGet
                                            ->where('type', 'LIKE', '%Price%')
                                            ->orderby('created_at', 'desc')
                                            ->orderby('id', 'desc')
                                        ;

                                        if ($newCheck1->exists()) {
                                            $priceDrop = $newCheck1->first();
                                            if (str_contains($priceDrop->type, 'reduced')) {
                                                $extractPrice = '-'.preg_replace('/[^0-9]/', '', $priceDrop->type);

                                                $priceDropDate = $priceDrop->event_date;
                                            } elseif (str_contains($priceDrop->type, 'increased')) {
                                                $extractPrice = preg_replace('/[^0-9]/', '', $priceDrop->type);

                                                $priceDropDate = $priceDrop->event_date;
                                            }
                                        } else {
                                            $priceDropDate = null;
                                            $extractPrice = null;
                                        }
                                    } else {
                                        //Also adding  First Listed manually because price history array is empty
                                        $propertyPriceHistoryGet = PropertyPriceHistory::where('property_history_id', $property->id);
                                        if (!$propertyPriceHistoryGet
                                            ->where('type', 'First listed')->exists()) {
                                            $priceHistory = PropertyPriceHistory::create([
                                                'property_history_id' => $property->id,
                                                'type' => 'First listed',
                                                'event_date' => $property->listed_date,
                                                'price' => $price,
                                            ]);
                                        }
                                        $status = 1;
                                        $newMarket = 1;
                                        $soldDate = null;
                                        $priceDropDate = null;
                                        $extractPrice = null;
                                    }

                                    // Checking house number not null and updateOrCreate
                                    if (null != $v['house_no']) {
                                        $housenumber = HouseNumber::where('house_number', $v['house_no'])->where('postcode_id', $postcode->id);
                                        if ($housenumber->exists()) {
                                            $houseNumber = $housenumber->first();

                                            HouseNumber::where('id', $houseNumber->id)->update([
                                                'listed_date' => !empty($v['first_listed']) ? $v['first_listed'] : Carbon::now()->toDateString(),
                                                'latest_price' => !empty($price) ? $price : null,
                                                'sold_date' => $soldDate,
                                                'last_sold_price' => !empty($v['last_sold_price']) ? $v['last_sold_price'] : 0,
                                                'last_sold_date' => !empty($v['last_sold']) ? $v['last_sold'] : null,
                                                
                                            ]);
                                        // $houseNumber = $housenumber->first();
                                        } else {
                                            $houseNumber = HouseNumber::create([
                                                'house_number' => !empty($v['house_no']) ? $v['house_no'] : null,
                                                'listed_date' => !empty($v['first_listed']) ? $v['first_listed'] : Carbon::now()->toDateString(),
                                                'latest_price' => !empty($price) ? $price : null,
                                                'sold_date' => $soldDate,
                                                'postcode_id' => $postcode->id,
                                                'last_sold_price' => !empty($v['last_sold_price']) ? $v['last_sold_price'] : 0,
                                                'last_sold_date' => !empty($v['last_sold']) ? $v['last_sold'] : null,
                                                'new_to_market' => $newMarket,
                                                
                                            ]);
                                        }
                                        // market status house number doubt
                                        $houseNumberGet = HouseNumber::where('id', $houseNumber->id);

                                        // Updating house number and property id in house number table
                                        $allPropertyHouseNumber = PropertyHouseNumber::where('house_number_id', $houseNumber->id);
                                        if (!$allPropertyHouseNumber
                                            ->where('property_history_id', $property->id)->exists()) {
                                            $propertyHouseNumber = PropertyHouseNumber::create([
                                                'house_number_id' => $houseNumber->id,
                                                'property_history_id' => $property->id,
                                            ]);
                                        }
                                    } else {

                                        $withoutHouseNrCount = $withoutHouseNrCount + 1;
                                        $houseNumber = null;
                                    }
                               
                            }
                 
                            if ($property) {
                                $propertyPriceHistoryGet = PropertyPriceHistory::where('property_history_id', $property->id);

                                //Checking  sold_stc=1 if the price history table have the latest record as STC Removed means add STC , if both are not there means will add STC
                                // Same for sold_stc =0
                                if (isset($v['sold_stc']) && 1 == $v['sold_stc']) {
                            /*          $newCheck2 = $propertyPriceHistoryGet
                                        ->where('type', 'STC')
                                        ->where('price', $price)
                                ; */
                                    if (!$propertyPriceHistoryGet
                                        ->where('type', 'STC')
                                        ->where('price', $price)->exists()) {
                                        //   $propertyPriceHistoryGett = PropertyPriceHistory::where('property_history_id', $property->id); 
                                        $stc = PropertyPriceHistory::where('property_history_id', $property->id)
                                            // ->where('type', 'STC')->orWhere('type', 'STC Removed')
                                            ->orderby('created_at', 'DESC')
                                        ;

                                        if ($stc->exists()) {
                                            $stcData = $stc->first();
                                            if ('STC' != $stcData->type) {
                                                PropertyPriceHistory::create([
                                                    'property_history_id' => $property->id,
                                                    'event_date' => Carbon::now()->toDateString(),
                                                    'type' => 'STC',
                                                    'price' => $price,
                                                ]);
                                            }
                                        } 
                                        // else {
                                        //     PropertyPriceHistory::create([
                                        //         'property_history_id' => $property->id,
                                        //         'event_date' => Carbon::now()->toDateString(),
                                        //         'type' => 'STC',
                                        //         'price' => $price,
                                        //     ]);
                                        // }
                                    }
                                    $stcCount = $stcCount + 1;
                                }

                                if (isset($v['sold_stc']) && 0 == $v['sold_stc']) {
                                    // $propertyPriceHistoryGet = PropertyPriceHistory::where('property_history_id', $property->id);
                            /*          $newCheck3 = $propertyPriceHistoryGet
                                        ->where('type', 'STC Removed')
                                        ->where('price', $price)
                                    ; */
                                    if (!$propertyPriceHistoryGet
                                        ->where('type', 'STC Removed')
                                        ->where('price', $price)->exists()) {
                                        //   $propertyPriceHistoryGett = PropertyPriceHistory::where('property_history_id', $property->id); 
                                        $stc = PropertyPriceHistory::where('property_history_id', $property->id)
                                            // ->where('type', 'STC')->orWhere('type', 'STC Removed')
                                            ->orderby('created_at', 'DESC')
                                        ;

                                        if ($stc->exists()) {
                                            $stcData1 = $stc->first();
                                            if ('STC' == $stcData1->type) {
                                                // if($stc->type=='STC'){
                                                PropertyPriceHistory::create([
                                                    'property_history_id' => $property->id,
                                                    'event_date' => Carbon::now()->toDateString(),
                                                    'type' => 'STC Removed',
                                                    'price' => $price,
                                                ]);
                                            }
                                        }
                                    }
                                }

                                if (isset($v['under_offer']) && 1 == $v['under_offer']) {
                                    // $propertyPriceHistoryGet = PropertyPriceHistory::where('property_history_id', $property->id);
                            /*          $newCheck4 = $propertyPriceHistoryGet
                                        ->where('type', 'Offer')
                                        ->where('price', $price)
                                    ; */
                                    if (!$propertyPriceHistoryGet
                                        ->where('type', 'Offer')
                                        ->where('price', $price)->exists()) {
                                        //   $propertyPriceHistoryGett = PropertyPriceHistory::where('property_history_id', $property->id); 
                                        $stc = PropertyPriceHistory::where('property_history_id', $property->id)
                                            // ->where('type', 'Offer')->orWhere('type', 'Offer Removed')
                                            ->orderby('created_at', 'DESC')
                                        ;

                                        if ($stc->exists()) {
                                            $stcData = $stc->first();
                                            if ('Offer' != $stcData->type) {
                                                PropertyPriceHistory::create([
                                                    'property_history_id' => $property->id,
                                                    'event_date' => Carbon::now()->toDateString(),
                                                    'type' => 'Offer',
                                                    'price' => $price,
                                                ]);
                                            }
                                        } 
                                        // else {
                                        //     PropertyPriceHistory::create([
                                        //         'property_history_id' => $property->id,
                                        //         'event_date' => Carbon::now()->toDateString(),
                                        //         'type' => 'Offer',
                                        //         'price' => $price,
                                        //     ]);
                                        // }
                                    }
                                    $offerCount = $offerCount + 1;
                                }
                                if (isset($v['under_offer']) && 0 == $v['under_offer']) {
                                    // $propertyPriceHistoryGet = PropertyPriceHistory::where('property_history_id', $property->id);
                            /*          $newCheck5 = $propertyPriceHistoryGet
                                        ->where('type', 'Offer Removed')
                                        ->where('price', $price)
                                ; */
                                    if (!$propertyPriceHistoryGet
                                        ->where('type', 'Offer Removed')
                                        ->where('price', $price)->exists()) {
                                    //    $propertyPriceHistoryGett = PropertyPriceHistory::where('property_history_id', $property->id); 
                                        $stc = PropertyPriceHistory::where('property_history_id', $property->id)
                                            // ->where('type', 'Offer')->orWhere('type', 'Offer Removed')
                                            ->orderby('created_at', 'DESC')
                                    ;

                                        if ($stc->exists()) {
                                            $stcData1 = $stc->first();
                                            if ('Offer' == $stcData1->type) {
                                                PropertyPriceHistory::create([
                                                    'property_history_id' => $property->id,
                                                    'event_date' => Carbon::now()->toDateString(),
                                                    'type' => 'Offer Removed',
                                                    'price' => $price,
                                                ]);
                                            }
                                        }
                                    }
                                }
                            }
                            if (($houseNumber)) {
                                $allPropertyHouseNumber = PropertyHouseNumber::where('house_number_id', $houseNumber->id);
                                $propertiesGet = $allPropertyHouseNumber->get();

                                  
                                // To add sold stc /under offer status . house number -> properties -> price histories except withdrawn . if the latest one for all  STC means  status =6
                                // same for under offer = 7

                                $data1 = [];
                                foreach ($propertiesGet as $keyGet => $valueGet) {
                                    $data1[] = PropertyPriceHistory::where('property_history_id', $valueGet->property_history_id)
                                        ->where('type', '<>', 'Withdrawn')
                                        ->orderby('created_at', 'desc')
                                        ->orderby('id', 'desc')
                                        ->first()
                                    ;
                                }
                                
                                if (count($data1) >= 1) {
                                   
                                    $Data1 = [];
                                    foreach ($data1 as $kk => $s) {
                                        if ($s) {
                                           
                                            $Data1[] = $s['type'];
                                        }
                                    }

                                    if (1 == count(array_unique($Data1))) {
                                        if ('STC' == array_unique($Data1)[0]) {
                                            $status = 6;
                                            // $stcCount = $stcCount + 1;
                                        } elseif ('Offer' == array_unique($Data1)[0]) {
                                            $status = 7;
                                            // $offerCount = $offerCount + 1;
                                        }
                                    }
                                }
                                // Update in house number status by checking all conditons price drop , sold stc etc ..
                                $houseNumberGet = HouseNumber::where('id', $houseNumber->id);
                                $updateHouseNumberStatus = $houseNumberGet->update([
                                    'latest_status' => $status,
                                    'price_drop_date' => $priceDropDate,
                                    'price_change_diff' => $extractPrice,
                                ]);
                            }
                            //Update in  property status
                            if ($property) {
                                $propertyGetID = PropertyHistory::where('id', $property->id);
                                $updatePropertyStatus = $propertyGetID->update([
                                    'property_current_status' => $status,
                                    'for_sale' => $status,
                                    'price_drop' => $extractPrice,
                                    'price_drop_date' => $priceDropDate,
                                ]);
                            }
                
                            // Total count of property inserted and updated in db
                            if (0 == $propertyExists) {
                                $insertCount = $insertCount + 1;
                            } elseif (1 == $propertyExists) {
                                $updatedProp = $updatedProp + 1;
                            }
                        }
                        ///////////////////////////////RIGHT MOVE ////////////////////////////////////////////////////////////////////
                        else {
                            // die('rightmove');
                            //URL Checking
                            // if (null != $v['link']) {
                            //     $rightmove_url = 'https://www.rightmove.co.uk/';
                            //     $portalUrl = $rightmove_url.''.$v['link'];
                            // } else {
                            //     $portalUrl = null;
                            // }
                            if (null != $v['link']) {
                                if (str_contains($v['link'], 'https://www.rightmove.co.uk')) {
                                    $portalUrl=$v['link'];
                                }
                                else{                   
                                    $rightmove_url = 'https://www.rightmove.co.uk';
                                    $portalUrl = $rightmove_url.''.$v['link'];
                                }
                             } else {
                                    $portalUrl = null;
                                }
                            //Update if pid exists
                            if (PropertyHistory::where('pid', $v['pid'])->exists()) {
                                //For Price drop manually checking
                                $exists = 1;
                                $rightMovePropertyGet = PropertyHistory::where('pid', $v['pid'])->first();
                                $propertyRM = PropertyPriceHistory::where('property_history_id', $rightMovePropertyGet->id);
                                // ->where('sold_year', '<>', null);
                                if ($propertyRM->exists()) {
                                    $rightMovePropertyGetPrice = $rightMovePropertyGet->price;
                                } else {
                                    $rightMovePropertyGetPrice = 0;
                                }
                                //Updating pid
                                PropertyHistory::where('pid', $v['pid'])->update([
                                    'sold_stc' => isset($v['sold_stc']) ? $v['sold_stc'] : 0,
                                    'under_offer' => isset($v['under_offer']) ? $v['under_offer'] : 0,
                                    'price' => $price,
                                    'poa' => $poa,
                                    'unconfirmed' => $v['unconfirmed'],
                                ]);

                                $property = PropertyHistory::where('pid', $v['pid'])->first();
                            
                                $status = 1;
                                $newMarket = 0;
                                $soldDate = null;
                                $priceDropDate = null;
                                $extractPrice = null;
                                //Also adding  First Listed manually because in rightmove we dont get firest listed in price history array
                                $propertyPriceHistoryGet = PropertyPriceHistory::where('property_history_id', $property->id);
                                if (!$propertyPriceHistoryGet
                                    ->where('type', 'First listed')->exists()) {
                                    $priceHistory = PropertyPriceHistory::create([
                                        'property_history_id' => $property->id,
                                        'type' => 'First listed',
                                        'event_date' => $property->listed_date,
                                        'price' => $price,
                                    ]);
                                }
                                //Price drop in Conditions checking manualy by getting price from previus property and current price
                                if (PropertyPriceHistory::where('property_history_id', $rightMovePropertyGet->id)
                                    // ->where('sold_year', '<>', null)
                                    ->exists()
                                ) {
                                    if (0 == $poa) {
                                        $soldRMPriceDifference = $v['price'] - $rightMovePropertyGetPrice;
                                        if (($soldRMPriceDifference < 0)) {
                                            $negativePrice = abs($soldRMPriceDifference);
                                            $priceType = 'Price reduced by £'.$negativePrice;
                                            $extractPrice = $soldRMPriceDifference;

                                            $priceDropDate = Carbon::now()->toDateTimeString();
                                        } elseif (($soldRMPriceDifference > 0)) {
                                            $priceType = 'Price increased by £'.$soldRMPriceDifference;
                                            $extractPrice = $soldRMPriceDifference;
                                            $priceDropDate = Carbon::now()->toDateTimeString();
                                        } elseif ((0 == $soldRMPriceDifference)) {
                                            $extractPrice = null;
                                            $priceType = 'Last sold';
                                            $priceDropDate = null;
                                        }
                                    }
                                }

                                // Adding price drop manually in price histry tabel
                                if (1 == $exists) {
                                    if (null != $extractPrice) {
                                        PropertyPriceHistory::create([
                                            'property_history_id' => $property->id,
                                            'event_date' => $priceDropDate,
                                            'type' => $priceType,
                                            'price' => $price,
                                        ]);
                                    }
                                }

                                // Checking house number not null and updateOrCreate
                                 // Checking house number not null and updateOrCreate
                                 if(PropertyHouseNumber::where('property_history_id',$property->id)->exists()){
                                   
                                    if (null != $v['house_no']) {
                                        
                                    $propertyHouseNumberData =PropertyHouseNumber::where('property_history_id',$property->id)->first();
                                    HouseNumber::where('id', $propertyHouseNumberData->house_number_id)->update([
                                        'house_number' => !empty($v['house_no']) ? $v['house_no'] : null,
                                        'latest_price' => !empty($price) ? $price : null,
                                        'sold_date' => $soldDate,
                                        'new_to_market' => $newMarket,
                                        
                                    ]);
                                    $houseNumber=HouseNumber::where('id', $propertyHouseNumberData->house_number_id)->first();
                               } else{
                               
                                $propertyHouseNumberData =PropertyHouseNumber::where('property_history_id',$property->id)->first();
                                HouseNumber::where('id', $propertyHouseNumberData->house_number_id)->update([
                                   
                                    'latest_price' => !empty($price) ? $price : null,
                                    'sold_date' => $soldDate,
                                    'new_to_market' => $newMarket,
                                    
                                ]);
                                $houseNumber=HouseNumber::where('id', $propertyHouseNumberData->house_number_id)->first();
                               }
                            }else{
                                if (null != $v['house_no']) {
                                    $housenumber = HouseNumber::where('house_number', $v['house_no'])->where('postcode_id', $postcode->id);
                                    if ($housenumber->exists()) {
                                        $houseNumber = $housenumber->first();

                                        HouseNumber::where('id', $houseNumber->id)->update([
                                            'latest_price' => !empty($price) ? $price : null,
                                            
                                        ]);
                                    // $houseNumber = $housenumber->first();
                                    } else {
                                        $houseNumber = HouseNumber::create([
                                            'house_number' => !empty($v['house_no']) ? $v['house_no'] : null,
                                            'latest_price' => !empty($price) ? $price : null,
                                            'sold_date' => $soldDate,
                                            'postcode_id' => $postcode->id,
                                            'new_to_market' => $newMarket,
                                            
                                        ]);
                                    }
                                    // market status house number doubt
                                    $houseNumberGet = HouseNumber::where('id', $houseNumber->id);

                                    // Updating house number and property id in house number table
                                    $allPropertyHouseNumber = PropertyHouseNumber::where('house_number_id', $houseNumber->id);
                                    if (!$allPropertyHouseNumber
                                        ->where('property_history_id', $property->id)->exists()) {
                                        $propertyHouseNumber = PropertyHouseNumber::create([
                                            'house_number_id' => $houseNumber->id,
                                            'property_history_id' => $property->id,
                                        ]);
                                    }
                                } 
                            }
                                // else {
                                  
                                      
                                //     // $houseNumber = null;
                                //     $getHouseNumber=PropertyHouseNumber::where('property_history_id',$property->id);
                                //     if($getHouseNumber->exists()){
                                //         $propertyHouseNumberData =$getHouseNumber->first();
                                //         HouseNumber::where('id', $propertyHouseNumberData->house_number_id)->update([
                                //             'latest_price' => !empty($price) ? $price : null,
                                //         ]);
                                //         $houseNumber=HouseNumber::where('id', $propertyHouseNumberData->house_number_id)->first();
                                //     }
                                // }
                                            //if withdrawn exists it removes from price history table 
                            $deleteWithdrawn = PropertyPriceHistory::where('property_history_id', $property->id)
                            ->where('type', 'Withdrawn')
                            ->orderby('created_at', 'desc')
                            ->orderby('id', 'desc');

                            if ($deleteWithdrawn->exists()) {
                                if(!empty($houseNumber)){
                                $houseNumberGet = HouseNumber::where('id', $houseNumber->id);
                                $houseNumberGet->update([
                                   'latest_status' => 1,
                                   'withdrawn_date' => null,
                                   'price_drop_date' => null,
                                   'price_change_diff' => null,
                               ]);
                                }
                                if ($property) {
                                    $propertyGetID = PropertyHistory::where('id', $property->id);
                                    $propertyGetID->update([
                                        'property_current_status' =>1,
                                        'for_sale' => 1,
                                        'withdrawn_date' => null,
                                        'price_drop_date' => null,
                                        'price_drop' => null,
                                    ]);
                                }
                            $deleteWithdrawn->delete();
                            }
                                // for count (update )
                                $propertyExists = 1;
                            } else {
                                
                                $newMarket = 1;
                                $priceDropDate = null;
                                $extractPrice = null;
                                // if (!empty($agency)) {
                                    // for count insert
                                    $propertyExists = 0;
                                    //Price drop Condition
                                    $rightMovePropertyGet = (object) [];

                                    $rightMovePropertyGet->id = null;

                                    $exists = 0;

                                    $property = PropertyHistory::create([
                                        'portal' => 0,
                                        'pid' => $v['pid'],
                                        'for_sale' => 1,
                                        'agency_id' => $agency->id,
                                        'postcode_id' => $postcode->id,
                                        'title' => $v['title'],
                                        'property_type' => $v['type'],
                                        'bed_rooms' => !empty($v['bedroom']) ? $v['bedroom'] : null,
                                        'sold_stc' => isset($v['sold_stc']) ? $v['sold_stc'] : 0,
                                        'under_offer' => isset($v['under_offer']) ? $v['under_offer'] : 0,
                                        'price' => $price,
                                        'poa' => $poa,
                                        'portal_url' => $portalUrl,
                                        'address' => $v['address'],
                                        // 'description' => $v['description'],
                                        'latitude' => isset($v['latitude']) ? $v['latitude'] : null,
                                        'longitude' => isset($v['longitude']) ? $v['longitude'] : null,
                                        'listed_date' => !empty($v['first_listed']) ? $v['first_listed'] : Carbon::now()->toDateString(),
                                        'event_date' => Carbon::now()->toDateString(),
                                        'unconfirmed' => $v['unconfirmed'],
                                    ]);
                                    // Sale history on year comes
                                    if (!empty($v['last_sold_price']) && !empty($v['last_sold'])) {
                                        if (!PropertySaleHistory::where('property_history_id', $property->id)
                                            ->where('price', $v['last_sold_price'])
                                            ->where('sold_year', $v['last_sold'])
                                            ->exists()) {
                                            $saleHistory = PropertySaleHistory::create([
                                                'property_history_id' => $property->id,
                                                'price' => !empty($v['last_sold_price']) ? $v['last_sold_price'] : 0,
                                                'sold_year' => !empty($v['last_sold']) ? $v['last_sold'] : null,
                                            ]);
                                        }
                                    }
                                    $propertyPriceHistoryGet = PropertyPriceHistory::where('property_history_id', $property->id);
                                    // Status from  price history array
                                    if (!empty($v['price_history'])) {
                                        if ('First listed' == $v['price_history'][0]['type']) {
                                            $status = 1;

                                            $newMarket = 1;
                                            $soldDate = null;
                                        } elseif ('Sold' == $v['price_history'][0]['type']) {
                                            // $status = 2;
                                            $newMarket = 0;
                                            $soldDate = $v['price_history'][0]['date'];
                                        } elseif ('Withdrawn' == $v['price_history'][0]['type']) {
                                            // $status = 3;

                                            $newMarket = 0;
                                            $soldDate = null;
                                        } else {
                                            $status = 1;

                                            $newMarket = 0;
                                            $soldDate = null;
                                        }
                                        //Price history Insertion
                                        foreach ($v['price_history'] as $Key => $Value) {
                                            if (!$propertyPriceHistoryGet
                                                ->where('type', $Value['type'])
                                                ->where('sold_year', $Value['date'])
                                                ->where('price', $Value['price'])
                                            // ->orderby('event_date', $Value['date'])
                                                ->exists()) {
                                                $priceHistory = PropertyPriceHistory::create([
                                                    'property_history_id' => $property->id,
                                                    'type' => !empty($Value['type']) ? $Value['type'] : null,
                                                    'sold_year' => !empty($Value['date']) ? $Value['date'] : null,
                                                    'price' => !empty($Value['price']) ? $Value['price'] : null,
                                                ]);
                                            }
                                        }
                                    } else {
                                        $status = 1;
                                        $newMarket = 1;
                                        $soldDate = null;
                                    }
                                    //Also adding  First Listed manually because in rightmove we dont get firest listed in price history array
                                    $propertyPriceHistoryGet = PropertyPriceHistory::where('property_history_id', $property->id);
                                    if (!$propertyPriceHistoryGet
                                        ->where('type', 'First listed')->exists()) {
                                        $priceHistory = PropertyPriceHistory::create([
                                            'property_history_id' => $property->id,
                                            'type' => 'First listed',
                                            'event_date' => !empty($v['first_listed']) ? $v['first_listed'] : Carbon::now()->toDateString(),
                                            'price' => $price,
                                        ]);
                                    }
                                    if (null != $v['house_no']) {
                                        $housenumber = HouseNumber::where('house_number', $v['house_no'])->where('postcode_id', $postcode->id);
                                        if ($housenumber->exists()) {
                                            $houseNumber = $housenumber->first();

                                            HouseNumber::where('id', $houseNumber->id)->update([
                                                'listed_date' => !empty($v['first_listed']) ? $v['first_listed'] : Carbon::now()->toDateString(),
                                                'latest_price' => !empty($price) ? $price : null,
                                                // 'sold_year' => !empty($v['last_sold']) ? $v['last_sold'] : NULL,
                                                'sold_year' => $soldDate,
                                                'last_sold_price' => !empty($v['last_sold_price']) ? $v['last_sold_price'] : 0,
                                                'last_sold_date' => !empty($v['last_sold']) ? $v['last_sold'] : null,
                                                
                                            ]);
                                        // $houseNumber = $housenumber->first();
                                        } else {
                                            $houseNumber = HouseNumber::create([
                                                'house_number' => !empty($v['house_no']) ? $v['house_no'] : null,
                                                'listed_date' => !empty($v['first_listed']) ? $v['first_listed'] : Carbon::now()->toDateString(),
                                                'latest_price' => !empty($price) ? $price : null,
                                                'sold_year' => $soldDate,
                                                'postcode_id' => $postcode->id,
                                                'last_sold_price' => !empty($v['last_sold_price']) ? $v['last_sold_price'] : 0,
                                                'last_sold_date' => !empty($v['last_sold']) ? $v['last_sold'] : null,
                                                'new_to_market' => $newMarket,
                                                
                                            ]);
                                        }
                                        // market status house number
                                        $houseNumberGet = HouseNumber::where('id', $houseNumber->id);

                                        //  Update in property hosue number table
                                        $allPropertyHouseNumber = PropertyHouseNumber::where('house_number_id', $houseNumber->id);
                                        if (!$allPropertyHouseNumber
                                            ->where('property_history_id', $property->id)->exists()) {
                                            $propertyHouseNumber = PropertyHouseNumber::create([
                                                'house_number_id' => $houseNumber->id,
                                                'property_history_id' => $property->id,
                                            ]);
                                        }
                                    } else {
                                        $withoutHouseNrCount = $withoutHouseNrCount + 1;
                                        $houseNumber = null;
                                    }
                                
                            }
                      
                            if ($property) {
                                $propertyPriceHistoryGet = PropertyPriceHistory::where('property_history_id', $property->id);

                                //Checking  sold_stc=1 if the price history table have the latest record as STC Removed means add STC , if both are not there means will add STC
                                // Same for sold_stc =0
                                if (isset($v['sold_stc']) && 1 == $v['sold_stc']) {
                                    // $newCheck2 = $propertyPriceHistoryGet
                                    //     ->where('type', 'STC')
                                    //     ->where('price', $price)
                                    // ;
                                    if (!$propertyPriceHistoryGet
                                    ->where('type', 'STC')
                                    ->where('price', $price)->exists()) {
                                        // $propertyPriceHistoryGett = PropertyPriceHistory::where('property_history_id', $property->id);
                                        $stc = PropertyPriceHistory::where('property_history_id', $property->id)
                                            // ->where('type', 'STC')->orWhere('type', 'STC Removed')
                                            ->orderby('created_at', 'DESC')
                                        ;

                                        if ($stc->exists()) {
                                            $stcData = $stc->first();
                                            if ('STC' != $stcData->type) {
                                                PropertyPriceHistory::create([
                                                    'property_history_id' => $property->id,
                                                    'event_date' => Carbon::now()->toDateString(),
                                                    'type' => 'STC',
                                                    'price' => $price,
                                                ]);
                                            }
                                        } 
                                        // else {
                                        //     PropertyPriceHistory::create([
                                        //         'property_history_id' => $property->id,
                                        //         'event_date' => Carbon::now()->toDateString(),
                                        //         'type' => 'STC',
                                        //         'price' => $price,
                                        //     ]);
                                        // }
                                    }
                                    $stcCount = $stcCount + 1;
                                }

                                if (isset($v['sold_stc']) && 0 == $v['sold_stc']) {
                                    // $propertyPriceHistoryGet = PropertyPriceHistory::where('property_history_id', $property->id);
                                    // $newCheck3 = $propertyPriceHistoryGet
                                    //     ->where('type', 'STC Removed')
                                    //     ->where('price', $price)
                                    // ;
                                    if (!$propertyPriceHistoryGet
                                    ->where('type', 'STC Removed')
                                    ->where('price', $price)->exists()) {


                                        // $propertyPriceHistoryGett = PropertyPriceHistory::where('property_history_id', $property->id);
                                        $stc = PropertyPriceHistory::where('property_history_id', $property->id)
                                            // ->where('type', 'STC')->orWhere('type', 'STC Removed')
                                            ->orderby('created_at', 'DESC')
                                        ;

                                        if ($stc->exists()) {
                                            $stcData1 = $stc->first();
                                            if ('STC' == $stcData1->type) {
                                                // if($stc->type=='STC'){
                                                PropertyPriceHistory::create([
                                                    'property_history_id' => $property->id,
                                                    'event_date' => Carbon::now()->toDateString(),
                                                    'type' => 'STC Removed',
                                                    'price' => $price,
                                                ]);
                                            }
                                        }
                                    }
                                }

                                if (isset($v['under_offer']) && 1 == $v['under_offer']) {
                                    // $propertyPriceHistoryGet = PropertyPriceHistory::where('property_history_id', $property->id);
                                    // $newCheck4 = $propertyPriceHistoryGet
                                    //     ->where('type', 'Offer')
                                    //     ->where('price', $price)
                                    // ;
                                    if (!$propertyPriceHistoryGet
                                    ->where('type', 'Offer')
                                    ->where('price', $price)->exists()) {
                                        // $propertyPriceHistoryGett = PropertyPriceHistory::where('property_history_id', $property->id);
                                        $stc = PropertyPriceHistory::where('property_history_id', $property->id)
                                            // ->where('type', 'Offer')->orWhere('type', 'Offer Removed')
                                            ->orderby('created_at', 'DESC')
                                        ;

                                        if ($stc->exists()) {
                                            $stcData = $stc->first();
                                            if ('Offer' != $stcData->type) {
                                                PropertyPriceHistory::create([
                                                    'property_history_id' => $property->id,
                                                    'event_date' => Carbon::now()->toDateString(),
                                                    'type' => 'Offer',
                                                    'price' => $price,
                                                ]);
                                            }
                                        } 
                                        // else {
                                        //     PropertyPriceHistory::create([
                                        //         'property_history_id' => $property->id,
                                        //         'event_date' => Carbon::now()->toDateString(),
                                        //         'type' => 'Offer',
                                        //         'price' => $price,
                                        //     ]);
                                        // }
                                    }
                                    $offerCount = $offerCount + 1;
                                }
                                if (isset($v['under_offer']) && 0 == $v['under_offer']) {
                                    // $propertyPriceHistoryGet = PropertyPriceHistory::where('property_history_id', $property->id);
                                    // $newCheck5 = $propertyPriceHistoryGet
                                    //     ->where('type', 'Offer Removed')
                                    //     ->where('price', $price)
                                    // ;
                                    if (!$propertyPriceHistoryGet
                                    ->where('type', 'Offer Removed')
                                    ->where('price', $price)->exists()) {
                                        // $propertyPriceHistoryGett = PropertyPriceHistory::where('property_history_id', $property->id);
                                        $stc = PropertyPriceHistory::where('property_history_id', $property->id)
                                            // ->where('type', 'Offer')->orWhere('type', 'Offer Removed')
                                            ->orderby('created_at', 'DESC')
                                        ;

                                        if ($stc->exists()) {
                                            $stcData1 = $stc->first();
                                            if ('Offer' == $stcData1->type) {
                                                PropertyPriceHistory::create([
                                                    'property_history_id' => $property->id,
                                                    'event_date' => Carbon::now()->toDateString(),
                                                    'type' => 'Offer Removed',
                                                    'price' => $price,
                                                ]);
                                            }
                                        }
                                    }
                                }
                            }
                            if (!empty($houseNumber)) {
                                $allPropertyHouseNumber = PropertyHouseNumber::where('house_number_id', $houseNumber->id);

                               
                                $propertiesGet = $allPropertyHouseNumber->get();

                               
                                   //check all price history latest are STC or OFFER ,Same as zoopla
                                $data2 = [];
                                foreach ($propertiesGet as $KEY => $VAL) {
                                    $data2[] = PropertyPriceHistory::where('property_history_id', $VAL->property_history_id)
                                        ->where('type', '<>', 'Withdrawn')->orderby('created_at', 'desc')->orderby('id', 'desc')->first();
                                }

                                if (count($data2) >= 1) {
                                    $Data2 = [];
                                    foreach ($data2 as $kk => $vv) {
                                        if ($vv) {
                                            $Data2[] = $vv['type'];
                                        }
                                    }
                   
                                    if (1 == count(array_unique($Data2))) {
                                        if ('STC' == array_unique($Data2)[0]) {
                                            $status = 6;
                                            // $stcCount = $stcCount + 1;
                                        } elseif ('Offer' == array_unique($Data2)[0]) {
                                            // print_r($data1);die;
                                            $status = 7;
                                            // $offerCount = $offerCount + 1;
                                        } elseif ('First listed' == array_unique($Data2)[0]) {
                                            $newMarket = 1;
                                        } else {
                                            $newMarket = 0;
                                        }
                                    }
                                }
                                // Updating status in house nr table
                                $houseNumberGet = HouseNumber::where('id', $houseNumber->id);
                                $updateHouseNumberStatus = $houseNumberGet->update([
                                    'latest_status' => $status,
                                    'new_to_market' => $newMarket,
                                    'price_drop_date' => $priceDropDate,
                                    'price_change_diff' => $extractPrice,
                                ]);
                            }
                            //updating  property status
                            if ($property) {
                                $propertyGetID = PropertyHistory::where('id', $property->id);
                                $updatePropertyStatus = $propertyGetID->update([
                                    'property_current_status' => $status,
                                    'for_sale' => $status,
                                    'price_drop' => $extractPrice,
                                    'price_drop_date' => $priceDropDate,
                                ]);
                            }
               
                            // Taking count of properties insert / updated
                            if (0 == $propertyExists) {
                                $insertCount = $insertCount + 1;
                            } elseif (1 == $propertyExists) {
                                $updatedProp = $updatedProp + 1;
                            }
                        }
                    // }
                     $pid[] = $v['pid'];
                }
               
            }
            //To check withdrwan , old pid exists in new file if not the pid will be withdrwan
            // $pid = [];
            // foreach ($propertyArray as $k1 => $v1) {
                // $pid[] = $v1['pid'];
            // }
            // return $pid;
            if (!empty($pid)) {
            if (!empty($oldProperties)) {
                $result = array_diff($oldProperties, $pid);
            }
            }
            // if (!empty($result)) {
            //     foreach ($result as $withdrawn) {
            //         Withdrawn::insert([
            //             'pid' => $withdrawn,
            //         ]);
            //     }
            // }
            if (!empty($result)) {
                $withdrawnArray = [];
                foreach ($result as $withdrawn) {
                    $withdrawnArray[]=['pid'=>$withdrawn];
                 }
                  $withdrawnDetailsCollection = collect($withdrawnArray)->chunk(100)->toArray();
            foreach($withdrawnDetailsCollection as $chunk){
                Withdrawn::insert($chunk);
            }
                //  Withdrawn::insert($withdrawnArray);
            }

            
           
        } catch (\Exception $e) {
            $recievedCount = count($propertyArray);
            $updatedCount = ($updatedProp);
            $withoutPostCodeCount = ($withoutPostCode);

            if (!empty($insertCount)) {
                $insertedCount = ($insertCount);
            } else {
                $insertedCount = 0;
            }
            $total = $insertedCount + $updatedCount;

            ErrorLog::where('id', $log->id)->update([
                'outcode' => $request->outcode,
                'message' => $e->getMessage(),
                'properties_received' => $recievedCount,
                'properties_inserted' => $insertedCount,
                'properties_updated' => $updatedCount,
                'without_postcode' => $withoutPostCodeCount,
                'stc' => $stcCount,
                'offer' => $offerCount,
                'without_agent'=>$withoutAgent,
                'without_house_nr'=>$withoutHouseNrCount,
               
            ]);

            return response()->json(['success' => false, 'properties_received' => $recievedCount, 'properties_inserted' => $total, 'error' => $e->getMessage().' line '.$e->getLine()]);
        }

        $recievedCount = count($propertyArray);
        $updatedCount = ($updatedProp);
        $withoutPostCodeCount = ($withoutPostCode);
        if (!empty($insertCount)) {
            $insertedCount = ($insertCount);
        } else {
            $insertedCount = 0;
        }
        $total = $insertedCount + $updatedCount;

        ErrorLog::where('id', $log->id)->update([
            'outcode' => $request->outcode,
            'properties_received' => $recievedCount,
            'properties_inserted' => $insertedCount,
            'properties_updated' => $updatedCount,
            'without_postcode' => $withoutPostCodeCount,
            'stc' => $stcCount,
            'offer' => $offerCount,
            'without_agent'=>$withoutAgent,
            'without_house_nr'=>$withoutHouseNrCount,
           
        ]);

        // moving file to public storage in property/upload_file

        Storage::putFileAs('property/uploads', $request->file('upload_file'), date('Y-m-d.H-i-s.', $uploadStartTime).$request->file('upload_file')->getClientOriginalName());


        return response()->json(['success' => true, 'properties_received' => $recievedCount, 'properties_inserted' => $total, 'error' => '']);
    }
}
