<?php

// <summary>
// File : CampaignController
// File Type : Controller
// Creator : Susmi Sidharthan, Sandeep Kumar
// Email: susmi@designdirect@gmail.com
// Company: Design Direct UK
// Description ; 1.This is used as web controller for all estate agent campaign controller. 
//   2. Function used to print letter
// </summary>

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Http\Controllers\PaymentController;
use App\Http\Controllers\Api\getAddressAPIController;
use App\Http\Controllers\Api\StannpAPIController;
use App\Mail\ContactCampaign;
use App\Model\Campaign;
use App\Model\CampaignLetterHistory;
use App\Model\Letter;
use App\Model\LetterHead;
use App\Model\Payment;
use App\Model\Property;
use App\Model\PropertyCampaign;
use App\Model\PropertyFlag;
use App\Model\PropertyHistory;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Session;
use File;
use Redirect;
use Validator;
use Response;
use Yajra\DataTables\DataTables;
use Symfony\Component\Process\Process;
//use Spatie\PdfToText\Pdf;
//use Org_Heigl\Ghostscript\Ghostscript;
// use Xthiago\PDFVersionConverter\Guesser\RegexGuesser;
// use Symfony\Component\Filesystem\Filesystem;
// use Xthiago\PDFVersionConverter\Converter\GhostscriptConverterCommand;
// use Xthiago\PDFVersionConverter\Guesser\RegexGuesser;
// use Xthiago\PDFVersionConverter\Converter\GhostscriptConverter;
//use Org_Heigl\Ghostscript\Ghostscript;



class CampaignController extends Controller
{
    public $status = [
        1 => 'New to Market',
        2 => 'For sale',
        3 => 'Sold',
        4 => 'Withdrawn',
        5 => 'Price drop',
        6 => 'Fallen Through',
    ];
    public $trigger = [
        0 => 'General Letters',
        1 => 'New to Market',
        2 => 'For sale',
        3 => 'Sold',
        4 => 'Withdrawn',
        5 => 'Price drop',
        6 => 'Fallen Through',
        7 => '8 Week',
        8 => '16 Week',
        9 => '22 Week',
    ];

    protected $B;
    protected $I;
    protected $U;
    protected $HREF;
    protected $fontList;
    protected $issetfont;
    protected $issetcolor;

    public function __construct($orientation = 'P', $unit = 'mm', $format = 'A4')
    {
        //Call parent constructor
        // parent::__construct($orientation,$unit,$format);
        //Initialization
        $this->B = 0;
        $this->I = 0;
        $this->U = 0;
        $this->HREF = '';
        $this->fontlist = ['arial', 'times', 'courier', 'helvetica', 'symbol'];
        $this->issetfont = false;
        $this->issetcolor = false;

        $pdf = new \setasign\Fpdi\Fpdi('Portrait', 'mm', [210, 297], true, 'UTF-8', true);

        $pdf->SetFont('Helvetica');
    }

    public function index()
    {
        $page = collect();
        $page->title = 'Campaign History';
        $page->link = url('campaign-history');
        $page_title = 'Campaign History';

        $cam_history_merge = 1;
        $prob_class = 'add-un';
        $my_class = 'active';

        return view('campaign/histories', compact('page', 'cam_history_merge', 'my_class', 'prob_class'));
    }

    public function lists(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $detail = Campaign::where('type', 1)->select(['name', 'sent_date', 'campaign_status', 'id', 'created_at'])
            ->where('user_id', $user_id)
            ->orderBy('campaigns.id', 'desc')
        ;

        return DataTables::of($detail)
            ->editColumn('sent_date', function ($detail) {
                return '' != $detail->sent_date ? Carbon::parse($detail->sent_date)->format('d/m/Y').Carbon::parse($detail->created_at)->format(' h:i a') : '';
            })
            ->editColumn('campaign_status', function ($detail) {
                return 1 == $detail->campaign_status ? 'Sent' : 'Sending';
            })
            ->addColumn('action', function ($detail) {
                $print_url = url('campaigns/print/'.$detail->id);
                $action = '';
                // $action .= '<a class="btn btn-primary btn-sm mr-2" target="_blank" href="' . $print_url . '">Print</a>';
                $action .= Helper::deleteButton($detail->id);
                $action .= '<button class="btn btn-success btn-sm mr-2 editCampaign" data-id="'.$detail->id.'" data-name="'.$detail->name.'">Edit</button>';
                $action .= '<a class="btn btn-info btn-sm mr-2" href=" '.route('campaign-history.show', $detail->id).'">View</a>';

                return $action;
            })
            ->removeColumn(['id', 'created_at'])
            ->escapeColumns([])
            ->make(false)
        ;
    }

    public function destroy($id): array
    {
        Campaign::destroy($id);
        $error = false;
        $message = 'Campaign deleted successfully';

        return compact('error', 'message');
    }

    //property add to campaign
    public function addToCampaignProperty(Request $request)
    {
        $rules = [
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }

        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        foreach ($request->property as $property_id) {
            $exist_data = PropertyCampaign::where('user_id', $user_id)->where('house_number_id', $property_id)->where('campaign_id', null)->first();

            if (!$exist_data) {
                $add = new PropertyCampaign();
                $add->house_number_id = $property_id;
                $add->user_id = $user_id;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully added to campaign...!');
    }

    //property add to campaign
    public function createCampaign(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $campaign = new Campaign();
        $campaign->name = 'Campaign-'.Carbon::now()->format('d-m-Y h:i');
        $campaign->campaign_status = 0;
        $campaign->user_id = $user_id;
        $campaign->save();
        foreach ($request->property_id as $property_id) {
            $trigger = 'trigger_'.$property_id;
            PropertyCampaign::where('id', $property_id)
                ->update([
                    'campaign_id' => $campaign->id,
                    'trigger' => $request->{$trigger},
                ])
            ;
        }

        return redirect('campaign-history');
    }

    //property add to campaign
    public function printCampaign(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if (1 == $request->print_type || 3 == $request->print_type) {
            $source_data = LetterHead::where('user_id', $user_id)->first();
            if (!$source_data) {
                return -1;
            }
        }
        $campaign = new Campaign();
        $campaign->name = 'Campaign-'.Carbon::now()->format('d-m-Y h:i');
        $campaign->campaign_status = 0;
        $campaign->user_id = $user_id;
        $campaign->save();
        foreach ($request->data as $data) {
            PropertyCampaign::where('id', $data[0])
                ->update([
                    'campaign_id' => $campaign->id,
                    'trigger' => $data[1],
                ])
            ;

            $propertyCampaign = PropertyCampaign::where('id', $data[0])->first();
            if ($propertyCampaign) {
                $flag = PropertyFlag::updateOrCreate(
                    ['house_number_id' => $propertyCampaign->house_number_id, 'user_id' => $user_id],
                    ['house_number_id' => $propertyCampaign->house_number_id, 'user_id' => $user_id, 'flag' => 3]
                );

                $letterHistory = new CampaignLetterHistory();
                $letterHistory->house_number_id = $propertyCampaign->house_number_id;
                $letterHistory->user_id = $user_id;
                $letterHistory->letter_type = $data[1];
                $letterHistory->save();
            }
        }

        $url = url('campaigns/print/'.$campaign->id.'/'.$request->print_type);
        $error = false;
        $message = 'campaign submitted successfully';

        return compact('message', 'url', 'error');
    }

    public function printLetter($campaign_id, $print_type, $print_method = null)
    { // printLetter functionality is shared for local printing, bulk posting and also preview
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        if ($print_method !== 'preview') {
            $campaign                   = Campaign::find($campaign_id);
            $campaign->campaign_status  = 1;
            $campaign->sent_date        = Carbon::now();
            $campaign->save();
            $properties                 = PropertyCampaign::where('campaign_id', $campaign_id)->get();
        } else {
            $properties                 = PropertyCampaign::where('id', $campaign_id[0])->get();
        }
        $user   = json_decode(Session::get(config('propai.authSessionKey')));
        if ($print_method === null) {
            $pdf    = new \setasign\Fpdi\Fpdi('Portrait', 'mm', [210, 297], true, 'UTF-8', true);
        }
        // using FDPI class to generate pdf
        if (count($properties) > 0) {
            $file_send_return_responce  = [];
            $i                          = 1;
            foreach ($properties as $property) {
                // if ((!isset($property->edit_property_address) || $property->edit_property_address === null) && $print_method !== null) {
                if ($print_method !== null) { // if not local printing
                    // fetch address using campaign id
                    $property_house_number_id           = [$property->house_number_id];
                    $edit_address_response              = $this->fetchPropertyAddress($property_house_number_id);
                    $property->edit_property_address    = $edit_address_response[0]['address'];
                }
                if (isset($property->edit_property_address) && $property->edit_property_address !== null  && $print_method !== null) { // if not local printing and edit address present through above fetch
                    // correct the address through getaddress API
                    $getAddressAPIController    = new getAddressAPIController();
                    $addressCorrectionResult    = $getAddressAPIController->validatePostcodeScrapSource($property->edit_property_address);
                    if (isset($addressCorrectionResult['success']) && $addressCorrectionResult['success'] === true) {
                        $property->edit_property_address = $addressCorrectionResult['data'];
                    } else if (isset($addressCorrectionResult['success']) && $addressCorrectionResult['success'] === false) {
                        // address not correct and requires manual correction
                    }
                }
                ini_set('memory_limit', '-1');
                if ($print_method !== null) {
                    // $pdf    = new \setasign\Fpdi\Fpdi('Portrait', 'mm', [215.9, 279.4], true, 'UTF-8', true);
                    $pdf    = new \setasign\Fpdi\Fpdi('Portrait', 'mm', [210, 297], true, 'UTF-8', true); // A4 Size
                }
                $property->letter   = Letter::where('triggers', $property->trigger)->where('user_id', $user_id)->first();
                if ($print_method === 'preview') {
                    $property->letter   = Letter::where('triggers', $campaign_id[1])->where('user_id', $user_id)->first();
                }
                if ($print_type == 2) {
                    $source         = public_path('letter-heads/blank_letter.pdf');
                } else {
                    $source_data    = LetterHead::where('user_id', $user->id)->first();
                    $source         = storage_path('app/public/letter-heads/'.$source_data['file']);
                    $filepdf        = fopen($source,"r");
                    if ($filepdf) {
                     $line_first = fgets($filepdf);
                     fclose($filepdf);
                    } else {
                        echo "error opening the file.";
                    }
                    preg_match_all('!\d+!', $line_first, $matches);
                    $pdfversion     = implode('.', $matches[0]);  
                    if ($pdfversion > "1.4") {         
                    } else {
                        $pdfVersion     = "1.7";   //The path that you want to save the new file to
                        $newFile        = storage_path('app/public/letter-heads/1-'.$source_data['file']); //The path of the file that you want to convert
                        $currentFile    = $source;
                        $imgPath        = storage_path('app/public/letter-heads/');
                        ////////////////////// LOCAL WORKING /////////////////////////
                        if (env('APP_ENV') == 'local') {
                            $gslocal    = public_path('mygs/bin/gswin64c.exe');
                            $gsPath     = '"'.$gslocal.'"';
                            $gsCmd      = $gsPath.' -dSAFER -dBATCH -dNOPAUSE -dCompatibilityLevel='.$pdfVersion.' -sDEVICE=pdfwrite -sOutputFile='.$newFile .' '.$currentFile;     
                        }
                        /////////////////////// DEMO TESTING ////////////////////////
                        if (env('APP_ENV') == 'demo' || env('APP_ENV') == 'live' ) {
                            $gsCmd = 'gs -dSAFER -dBATCH -dNOPAUSE -dCompatibilityLevel='.$pdfVersion.' -sDEVICE=pdfwrite -sOutputFile='.$newFile .' '.$currentFile;
                        }
                        //Run it using PHP's exec function.
                        $result = exec($gsCmd);
                        $source = $newFile;
                    }
                }
                $pagecount      = $pdf->setSourceFile($source);
                $tpl            = $pdf->importPage(1);
                if ($print_method === 'preview') {
                    $stannp_source  = storage_path('app/public/stannp/stannp.pdf');
                    $pagecount      = $pdf->setSourceFile($stannp_source);
                    $stannp_tpl     = $pdf->importPage(1);
                }
                $pdf->AddPage();
                // $pdf->useTemplate($tpl);
                if ($print_method === 'preview') {
                    $pdf->useImportedPage($stannp_tpl, null, null, 210, 297, FALSE);
                }
                $pdf->useTemplate($tpl, null, null, 210, 297, FALSE);
                /*if ($print_method === 'preview') {
                }*/
                $pdf->SetFont('Helvetica');
                ini_set('memory_limit', '-1');
                $pdf->SetFontSize('10');
                if ($print_method === 'preview') {
                    $pdf->SetXY(30, 50);
                } else {
                    $pdf->SetXY(19.25, 52);
                }
                // $pdf->Write(0, 'The Home Owner');
                if ($print_method === 'bulk_send') {
                    $pdf->MultiCell(0, 4.5, "", 0, 1); // blank address for bulk mails
                    $offset     = 92;
                    $space      = 5;
                } else {
                    if ($print_method === 'preview') {
                        $property->edit_property_address = " " . $property->edit_property_address;
                        $property->edit_property_address = str_replace(',', ",\n", $property->edit_property_address);
                        $pdf->MultiCell(0, 4.5, $property->edit_property_address, 0, 1);
                        $offset     = 92;
                        $space      = 5;
                    } else {
                        $pdf->MultiCell(0, 4.5, $property->edit_property_address, 0, 1);
                        $offset     = 78;
                        $space      = 5;
                    }
                }
                $addrLines  = preg_match_all('/\n/', $property->edit_property_address);
                if ($addrLines > 4) {
                    $offset += ($addrLines - 4) * $space;
                }
                if ($property->letter) {
                    $pdf->SetXY(19.25, $offset); 
                    $content = strip_tags($property->letter->content); 
                    $content = $this->normalize($content);
                    $content = iconv('UTF-8', 'windows-1252', $content);                
                    $pdf->MultiCell(0, 4.5, $content, 0, 1);
                }
                if ($print_method === null) {
                    // delete specific campaign when mail send
                    PropertyCampaign::find($property->id)->delete();
                }
                if ($print_method === 'preview') {
                    ob_get_clean();
                    // generate a preview pdf
                    $campaign_path          = public_path() . "/storage/generated-letters/";
                    $filename               = 'preview_'.$campaign_id[0] . "_" . $i . rand(5000,9999) .".pdf";
                    $generated_pdf_filepath = $campaign_path . $filename;
                    $pdf->Output("$generated_pdf_filepath", 'F');
                    $filepath_pdf           = asset('storage/generated-letters/'.$filename);

                    return $filepath_pdf;
                }
                if ($print_method === 'bulk_send') {
                    ob_get_clean();
                    $postcode               = isset($property->edit_property_address) ? $property->edit_property_address : '';
                    $campaign_path          = public_path() . "/storage/generated-letters/";
                    $filename               = $campaign_id . "_" . $i . ".pdf";
                    $generated_pdf_filepath = $campaign_path . $filename;
                    $filepath_pdf           = asset('storage/generated-letters/'.$filename);
                    $pdf->Output("$generated_pdf_filepath", 'F');
                    unset($pdf);
                    if (file_exists($generated_pdf_filepath)) {
                        // call stannp function to transfer the pdf
                        $request_params             = ['pdf_path' => $filepath_pdf, "postcode" => $postcode];
                        // log all stannp post requests
                        Log::info($request_params);
                        $stannpAPIController        = new StannpAPIController();
                        $file_send_responce         = $stannpAPIController->postPDFLetterToStannp($request_params);
                        // log file send responces
                        Log::info($file_send_responce);
                        if (isset($file_send_responce['success']) && $file_send_responce['success'] === true ) {
                            // delete files when transfered to stannp api
                            unlink($generated_pdf_filepath);
                            // delete specific campaign when mail send
                            PropertyCampaign::find($property->id)->delete(); // delete the campaign that has send mail correct
                            $file_send_return_responce[$i] = [
                                                                "property_id"   => $property->id,
                                                                "success"       => true,
                                                                "message"       => "letter send"
                                                            ];
                        } else {
                            // delete files when not transfered to stannp api failes
                            unlink($generated_pdf_filepath);
                            $file_send_return_responce[$i] = [
                                                                "property_id"   => $property->id,
                                                                 "success"      => false,
                                                                 "message"      => "letter not send"
                                                             ];
                        }
                    }
                    $i++;
                }
            }
            if ($print_method === null) {
                ob_get_clean();
                $pdf->Output($campaign->name.'.pdf', 'I');
            }
            if ($print_method === 'bulk_send') {
                // calculate and record payment debit
                if (isset($file_send_return_responce) && count($file_send_return_responce) > 0) {
                    $i = $j = 0;
                    // log all bulksend responce
                    Log::info($file_send_return_responce);
                    foreach ($file_send_return_responce as $property_send_responce) {
                        if ($property_send_responce['success'] === true) {
                            $j++; // total success send count
                        }
                    }
                    if ($j > 0) { // there is some success in sending bulkmail
                        $single_post_cost   = config('stannp_api.single_post_cost')/100;
                        $percentage_of_vat  = config('stannp_api.vat_percentage')/100;
                        $amount             = $j * $single_post_cost;
                        $vat_amount         = $amount * $percentage_of_vat;
                        $total_amount       = $vat_amount + $amount;
                        $stannpAPIController= new StannpAPIController();
                        $randString         = $stannpAPIController->generateRandomString(6);
                        $transaction_id     = "BULK-$randString-$j";
                        $is_unique          = false;
                        while (!$is_unique) {
                            $result = Payment::where('transaction_id', $transaction_id)->count();
                            if ($result === 0) { // if you don't get a result, then you're good
                                $is_unique = true;
                            } else { // if you DO get a result, keep generating new transID
                                $randString     = $stannpAPIController->generateRandomString(6);
                                $transaction_id = "BULK-$randString-$j";
                            }
                        }
                        $PaymentController = new PaymentController;
                        $PaymentController->recordPaymentDetails([
                                                                    'transaction_id'    => $transaction_id,
                                                                    'user_id'           => $user_id,
                                                                    'action'            => 'debit',
                                                                    'action_amount'     => $total_amount,
                                                                ]);
                    }
                }
                return $file_send_return_responce;
            }
        }
    }

    // function to convert characters
    protected function normalize($word)
    {
        $word = str_replace("&quot;", '"', $word);
        $word = str_replace("&amp;", "&", $word);
        $word = str_replace("&lt;", "<", $word);
        $word = str_replace("&gt;", ">", $word);
        $word = str_replace("&nbsp;", " ", $word);
        $word = str_replace("&Agrave;", "À", $word);
        $word = str_replace("&Aacute;", "Á", $word);
        $word = str_replace("&Acirc;", "Â", $word);
        $word = str_replace("&Atilde;", "Ã", $word);
        $word = str_replace("&Auml;", "Ä", $word);
        $word = str_replace("&Aring;", "Å", $word);
        $word = str_replace("&AElig;", "Æ", $word);
        $word = str_replace("&Ccedil;", "Ç", $word);
        $word = str_replace("&Egrave;", "È", $word);
        $word = str_replace("&Eacute;", "É", $word);
        $word = str_replace("&Ecirc;", "Ê", $word);
        $word = str_replace("&Euml;", "Ë", $word);
        $word = str_replace("&Igrave;", "Ì", $word);
        $word = str_replace("&Iacute;", "Í", $word);
        $word = str_replace("&Icirc;", "Î", $word);
        $word = str_replace("&Iuml;", "Ï", $word);
        $word = str_replace("&ETH;", "Ð", $word);
        $word = str_replace("&Ntilde;", "Ñ", $word);
        $word = str_replace("&Ograve;", "Ò", $word);
        $word = str_replace("&Oacute;", "Ó", $word);
        $word = str_replace("&Ocirc;", "Ô", $word);
        $word = str_replace("&Otilde;", "Õ", $word);
        $word = str_replace("&Ouml;", "Ö", $word);
        $word = str_replace("&times;", "×", $word);
        $word = str_replace("&Oslash;", "Ø", $word);
        $word = str_replace("&Ugrave;", "Ù", $word);
        $word = str_replace("&Uacute;", "Ú", $word);
        $word = str_replace("&Ucirc;", "Û", $word);
        $word = str_replace("&Uuml;", "Ü", $word);
        $word = str_replace("&Yacute;", "Ý", $word);
        $word = str_replace("&THORN;", "Þ", $word);
        $word = str_replace("&szlig;", "ß", $word);
        $word = str_replace("&agrave;", "à", $word);
        $word = str_replace("&aacute;", "á", $word);
        $word = str_replace("&acirc;", "â", $word);
        $word = str_replace("&atilde;", "ã", $word);
        $word = str_replace("&auml;", "ä", $word);
        $word = str_replace("&aring;", "å", $word);
        $word = str_replace("&aelig;", "æ", $word);
        $word = str_replace("&ccedil;", "ç", $word);
        $word = str_replace("&egrave;", "è", $word);
        $word = str_replace("&eacute;", "é", $word);
        $word = str_replace("&ecirc;", "ê", $word);
        $word = str_replace("&euml;", "ë", $word);
        $word = str_replace("&igrave;", "ì", $word);
        $word = str_replace("&iacute;", "í", $word);
        $word = str_replace("&icirc;", "î", $word);
        $word = str_replace("&iuml;", "ï", $word);
        $word = str_replace("&eth;", "ð", $word);
        $word = str_replace("&ntilde;", "ñ", $word);
        $word = str_replace("&ograve;", "ò", $word);
        $word = str_replace("&oacute;", "ó", $word);
        $word = str_replace("&ocirc;", "ô", $word);
        $word = str_replace("&otilde;", "õ", $word);
        $word = str_replace("&ouml;", "ö", $word);
        $word = str_replace("&divide;", "÷", $word);
        $word = str_replace("&oslash;", "ø", $word);
        $word = str_replace("&ugrave;", "ù", $word);
        $word = str_replace("&uacute;", "ú", $word);
        $word = str_replace("&ucirc;", "û", $word);
        $word = str_replace("&uuml;", "ü", $word);
        $word = str_replace("&yacute;", "ý", $word);
        $word = str_replace("&thorn;", "þ", $word);
        $word = str_replace("&yuml;", "ÿ", $word);
        $word = str_replace("&OElig;", "Œ", $word);
        $word = str_replace("&oelig;", "œ", $word);
        $word = str_replace("&Scaron;", "Š", $word);
        $word = str_replace("&scaron;", "š", $word);
        $word = str_replace("&Yuml;", "Ÿ", $word);
        $word = str_replace("&fnof;", "ƒ", $word);
        $word = str_replace("&circ;", "ˆ", $word);
        $word = str_replace("&tilde;", "˜", $word);
        $word = str_replace("&ndash;", "-", $word);
        $word = str_replace("&mdash;", "-", $word);
        $word = str_replace("&lsquo;", "`", $word);
        $word = str_replace("&rsquo;", "`", $word);
        $word = str_replace("&sbquo;", "‚", $word);
        $word = str_replace("&ldquo;", '"', $word);
        $word = str_replace("&rdquo;", '"', $word);
        $word = str_replace("&bdquo;", "„", $word);
        $word = str_replace("&lsaquo;", "‹", $word);
        $word = str_replace("&rsaquo;", "›", $word);
        $word = str_replace("&iexcl;", "¡", $word);
        $word = str_replace("&cent;", "¢", $word);
        $word = str_replace("&pound;", "£", $word);
        $word = str_replace("&curren;", "¤", $word);
        $word = str_replace("&yen;", "¥", $word);
        $word = str_replace("&euro;", "€", $word);
        $word = str_replace("&brvbar;", "¦", $word);
        $word = str_replace("&sect;", "§", $word);
        $word = str_replace("&uml;", "¨", $word);
        $word = str_replace("&copy;", "©", $word);
        $word = str_replace("&ordf;", "ª", $word);
        $word = str_replace("&laquo;", "«", $word);
        $word = str_replace("&not;", "¬", $word);
        $word = str_replace("&shy;", "­ ", $word);
        $word = str_replace("&reg;", "®", $word);
        $word = str_replace("&macr;", "¯", $word);
        $word = str_replace("&deg;", "°", $word);
        $word = str_replace("&plusmn;", "±", $word);
        $word = str_replace("&sup2;", "²", $word);
        $word = str_replace("&sup3;", "³", $word);
        $word = str_replace("&acute;", "´", $word);
        $word = str_replace("&micro;", "µ", $word);
        $word = str_replace("&para;", "¶", $word);
        $word = str_replace("&middot;", "·", $word);
        $word = str_replace("&cedil;", "¸", $word);
        $word = str_replace("&sup1;", "¹", $word);
        $word = str_replace("&ordm;", "º", $word);
        $word = str_replace("&raquo;", "»", $word);
        $word = str_replace("&frac14;", "¼", $word);
        $word = str_replace("&frac12;", "½", $word);
        $word = str_replace("&frac34;", "¾", $word);
        $word = str_replace("&iquest;", "¿", $word);
        $word = str_replace("&Alpha;", "Α", $word);
        $word = str_replace("&Beta;", "Β", $word);
        $word = str_replace("&Gamma;", "Γ", $word);
        $word = str_replace("&Delta;", "Δ", $word);
        $word = str_replace("&Epsilon;", "Ε", $word);
        $word = str_replace("&Zeta;", "Ζ", $word);
        $word = str_replace("&Eta;", "Η", $word);
        $word = str_replace("&Theta;", "Θ", $word);
        $word = str_replace("&Iota;", "Ι", $word);
        $word = str_replace("&Kappa;", "Κ", $word);
        $word = str_replace("&Lambda;", "Λ", $word);
        $word = str_replace("&Mu;", "Μ", $word);
        $word = str_replace("&Nu;", "Ν", $word);
        $word = str_replace("&Xi;", "Ξ", $word);
        $word = str_replace("&Omicron;", "Ο", $word);
        $word = str_replace("&Pi;", "Π", $word);
        $word = str_replace("&Rho;", "Ρ", $word);
        $word = str_replace("&Sigma;", "Σ", $word);
        $word = str_replace("&Tau;", "Τ", $word);
        $word = str_replace("&Upsilon;", "Υ", $word);
        $word = str_replace("&Phi;", "Φ", $word);
        $word = str_replace("&Chi;", "Χ", $word);
        $word = str_replace("&Psi;", "Ψ", $word);
        $word = str_replace("&Omega;", "Ω", $word);
        $word = str_replace("&alpha;", "α", $word);
        $word = str_replace("&beta;", "β", $word);
        $word = str_replace("&gamma;", "γ", $word);
        $word = str_replace("&delta;", "δ", $word);
        $word = str_replace("&epsilon;", "ε", $word);
        $word = str_replace("&zeta;", "ζ", $word);
        $word = str_replace("&eta;", "η", $word);
        $word = str_replace("&theta;", "θ", $word);
        $word = str_replace("&iota;", "ι", $word);
        $word = str_replace("&kappa;", "κ", $word);
        $word = str_replace("&lambda;", "λ", $word);
        $word = str_replace("&mu;", "μ", $word);
        $word = str_replace("&nu;", "ν", $word);
        $word = str_replace("&xi;", "ξ", $word);
        $word = str_replace("&omicron;", "ο", $word);
        $word = str_replace("&pi;", "π", $word);
        $word = str_replace("&rho;", "ρ", $word);
        $word = str_replace("&sigmaf;", "ς", $word);
        $word = str_replace("&sigma;", "σ", $word);
        $word = str_replace("&tau;", "τ", $word);
        $word = str_replace("&upsilon;", "υ", $word);
        $word = str_replace("&phi;", "φ", $word);
        $word = str_replace("&chi;", "χ", $word);
        $word = str_replace("&psi;", "ψ", $word);
        $word = str_replace("&omega;", "ω", $word);
        $word = str_replace("&thetasym;", "ϑ", $word);
        $word = str_replace("&upsih;", "ϒ", $word);
        $word = str_replace("&piv;", "ϖ", $word);
        $word = str_replace("&dagger;", "†", $word);
        $word = str_replace("&Dagger;", "‡", $word);
        $word = str_replace("&bull;", "•", $word);
        $word = str_replace("&hellip;", "…", $word);
        $word = str_replace("&permil;", "‰", $word);
        $word = str_replace("&prime;", "′", $word);
        $word = str_replace("&Prime;", "″", $word);
        $word = str_replace("&oline;", "‾", $word);
        $word = str_replace("&frasl;", "⁄", $word);
        $word = str_replace("&weierp;", "℘", $word);
        $word = str_replace("&image;", "ℑ", $word);
        $word = str_replace("&real;", "ℜ", $word);
        $word = str_replace("&trade;", "™", $word);
        $word = str_replace("&alefsym;", "ℵ", $word);
        $word = str_replace("&larr;", "←", $word);
        $word = str_replace("&uarr;", "↑", $word);
        $word = str_replace("&rarr;", "→", $word);
        $word = str_replace("&darr;", "↓", $word);
        $word = str_replace("&harr;", "↔", $word);
        $word = str_replace("&crarr;", "↵", $word);
        $word = str_replace("&lArr;", "⇐", $word);
        $word = str_replace("&uArr;", "⇑", $word);
        $word = str_replace("&rArr;", "⇒", $word);
        $word = str_replace("&dArr;", "⇓", $word);
        $word = str_replace("&hArr;", "⇔", $word);
        $word = str_replace("&forall;", "∀", $word);
        $word = str_replace("&part;", "∂", $word);
        $word = str_replace("&exist;", "∃", $word);
        $word = str_replace("&empty;", "∅", $word);
        $word = str_replace("&nabla;", "∇", $word);
        $word = str_replace("&isin;", "∈", $word);
        $word = str_replace("&notin;", "∉", $word);
        $word = str_replace("&ni;", "∋", $word);
        $word = str_replace("&prod;", "∏", $word);
        $word = str_replace("&sum;", "∑", $word);
        $word = str_replace("&minus;", "−", $word);
        $word = str_replace("&lowast;", "∗", $word);
        $word = str_replace("&radic;", "√", $word);
        $word = str_replace("&prop;", "∝", $word);
        $word = str_replace("&infin;", "∞", $word);
        $word = str_replace("&ang;", "∠", $word);
        $word = str_replace("&and;", "∧", $word);
        $word = str_replace("&or;", "∨", $word);
        $word = str_replace("&cap;", "∩", $word);
        $word = str_replace("&cup;", "∪", $word);
        $word = str_replace("&int;", "∫", $word);
        $word = str_replace("&there4;", "∴", $word);
        $word = str_replace("&sim;", "∼", $word);
        $word = str_replace("&cong;", "≅", $word);
        $word = str_replace("&asymp;", "≈", $word);
        $word = str_replace("&ne;", "≠", $word);
        $word = str_replace("&equiv;", "≡", $word);
        $word = str_replace("&le;", "≤", $word);
        $word = str_replace("&ge;", "≥", $word);
        $word = str_replace("&sub;", "⊂", $word);
        $word = str_replace("&sup;", "⊃", $word);
        $word = str_replace("&nsub;", "⊄", $word);
        $word = str_replace("&sube;", "⊆", $word);
        $word = str_replace("&supe;", "⊇", $word);
        $word = str_replace("&oplus;", "⊕", $word);
        $word = str_replace("&otimes;", "⊗", $word);
        $word = str_replace("&perp;", "⊥", $word);
        $word = str_replace("&sdot;", "⋅", $word);
        $word = str_replace("&lceil;", "⌈", $word);
        $word = str_replace("&rceil;", "⌉", $word);
        $word = str_replace("&lfloor;", "⌊", $word);
        $word = str_replace("&rfloor;", "⌋", $word);
        $word = str_replace("&lang;", "⟨", $word);
        $word = str_replace("&rang;", "⟩", $word);
        $word = str_replace("&loz;", "◊", $word);
        $word = str_replace("&spades;", "♠", $word);
        $word = str_replace("&clubs;", "♣", $word);
        $word = str_replace("&hearts;", "♥", $word);
        $word = str_replace("&diams;", "♦", $word);
        $word = str_replace("&ensp;", " ", $word);
        $word = str_replace("&emsp;", " ", $word);
        $word = str_replace("&thinsp;", " ", $word);
        $word = str_replace("&zwnj;", " ", $word);
        $word = str_replace("&zwj;", " ", $word);
        $word = str_replace("&lrm;", " ", $word);
        $word = str_replace("&rlm;", " ", $word);

        return $word;
    }

    // property add to campaign
    public function contactCampaign(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $campaign = new Campaign();
        $campaign->name = 'LETS BID - '.Carbon::now()->format('d-m-Y h:i');
        $campaign->campaign_status = 0;
        $campaign->user_id = $user_id;
        $campaign->save();

        foreach ($request->property_id as $property_id) {
            $trigger = 'trigger_'.$property_id;
            PropertyCampaign::where('id', $property_id)
                ->update([
                    'campaign_id' => $campaign->id,
                    'trigger' => $request->{$trigger},
                ])
            ;
        }
        Mail::to('bijodesigndirectuk@gmail.com')
            ->bcc('rohandesigndirect@gmail.com')
            ->queue(new ContactCampaign($campaign->name))
        ;

        $url = url('campaign-history');

        return redirect($url);
    }

    // function to bulksend and preview while bulk send
    public function bulkSendCampaign(Request $request)
    {
        $user_id    = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $post_type  = isset($request->post_type) ? $request->post_type : '';
        $properties = isset($request->property_campaign_triggers) ? $request->property_campaign_triggers : '';
        $print_type = 1;
        // for single pdf preview
        if ($post_type == "preview") {
            $property_data          = $request->data[0];
            $preview_send_response  = $this->printLetter($property_data, $print_type, $post_type);

            return $preview_send_response;
        }
        if ($post_type == "unlink") {
            $campaign_path          = public_path() . "/storage/generated-letters/";
            $filename               = basename($request->data);
            $generated_pdf_filepath = $campaign_path . $filename;
            if (file_exists($generated_pdf_filepath)) {
                unlink($generated_pdf_filepath);
            }
            return true;
        }
        // for bulk print
        if ($post_type == "bulk_send") { // for bulksend
            // fetch letter head
            /*if ($print_type === 1 || $print_type === 3) {
                $source_data = LetterHead::where('user_id', $user_id)->first();
                if (!$source_data) {
                    return -1;
                }
            }*/
            // add entry into campaign table
            $campaign                   = new Campaign();
            $campaign->name             = 'Bulk-Campaign-'.Carbon::now()->format('d-m-Y h:i');
            $campaign->campaign_status  = 0;
            $campaign->user_id          = $user_id;
            $campaign->save();
            foreach ($properties as $key => $value) {
                // update trigger in property table
                PropertyCampaign::where('id', $key)
                                    ->update([
                                                'campaign_id'   => $campaign->id,
                                                'trigger'       => $value,
                                            ]);

                $propertyCampaign       = PropertyCampaign::where('id', $key)->first();
                if ($propertyCampaign) {
                    // create or update entry in PropertyFlag table
                    $flag = PropertyFlag::updateOrCreate(
                                                            ['house_number_id' => $propertyCampaign->house_number_id, 'user_id' => $user_id],
                                                            [
                                                                'house_number_id'   => $propertyCampaign->house_number_id,
                                                                'user_id'           => $user_id,
                                                                'flag'              => 3
                                                            ]);
                    $letterHistory                  = new CampaignLetterHistory();
                    $letterHistory->house_number_id = $propertyCampaign->house_number_id;
                    $letterHistory->user_id         = $user_id;
                    $letterHistory->letter_type     = $value;
                    $letterHistory->save();
                }
            }
            $print_method               = $post_type;
            $bulk_send_response         = $this->printLetter($campaign->id, $print_type, $post_type);

            return back();
        }
    }

    public function allCampaign(Request $request)
    {
        $content = "<p>susmi &nbsp;Ipsum is simply dummy text of the printing and typesetting industry. Lorem Ipsum has been the industry's sta</p><p>&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; ndard dummy text ever since the 1500s, when an unknown printer took a galley of type and scrambled it to make a <strong>type specimen book. It has survived not only five centuries, but also the leap into electronic typesetting, remaining essentially unchanged. It was popularised in the 1960s with the release of Letraset sheets containing Lo</strong>rem&nbsp;</p><p>&nbsp;</p><p>&nbsp;</p><h2>Ipsum passages, and more recently with desktop<i> publishing software like Aldus Pa</i>geMaker including versions of Lorem Ipsum.</h2><p>&nbsp;</p><p>&nbsp;</p><p>&nbsp;</p><p>&nbsp;</p><p>Susmi</p>";

        $content = str_replace('&nbsp;', '', $content);

        // $content = preg_replace('/[^A-Za-z0-9\-]/', ' ',  filter_var($content, FILTER_SANITIZE_STRING));
        //    $content = preg_replace('/[[:^print:]]/', '', $content);

        //    $result = $this->WriteHTML($content);

        //    dd($result);
        $page_title = 'My Campaign';
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_outcode = $this->userOutcode();
        $lists = PropertyCampaign::join('house_number as h', 'property_campaigns.house_number_id', 'h.id')
            ->join('postcode as pc', 'pc.id', 'h.postcode_id')
            ->join('outcode as oc', 'oc.id', 'pc.outcode_id')
            ->select('property_campaigns.*', 'h.house_number', DB::raw('CONCAT(oc.outcode,pc.postcode) as pincode'), 'h.listed_date', 'h.id as h_id')
            ->where('property_campaigns.user_id', $user_id)
            ->whereIn('oc.id', $user_outcode)
            ->orderBy('property_campaigns.id', 'desc')
            ->get()
        ;

        if (count($lists) > 0) {
            $i = 0;
            foreach ($lists as $item) {
                $properties = PropertyHistory::select('property_history.*', 'a.name as agency')
                    ->join('property_house_number as phn', 'phn.property_history_id', 'property_history.id')
                    ->join('agency as a', 'a.id', 'property_history.agency_id')
                    ->where('phn.house_number_id', $item->house_number_id)
                    ->get()
                ;
                $full_address = $item->house_number.',<br/> ';
                $full_address .= (count($properties) > 0 && '' != $properties[0]->address) ? $properties[0]->address.',<br/>' : '';
                // $full_address .= $item->pincode;
                $agency = count($properties->pluck('agency')) > 0 ? implode(',', $properties->pluck('agency')->toArray()) : '';

                $campaign = Campaign::where('id', $item->campaign_id)
                    ->where('campaign_status', 1)
                    ->latest()
                    ->first()
                ;

                ++$i;
                $item->sl = $i;
                $item->address = $full_address;
                $item->agency = $agency;
                $item->trigger = Helper::selectTrigger($item, $properties);
                $item->listed_date = Carbon::parse($item->listed_date)->format('d/m/Y');
                $item->last_letter_sent = $campaign ? Carbon::parse($campaign->send_date)->format('d/m/Y') : '';
                $item->last_date_sent = '';
            }
        }
        $exists = Letter::where('user_id', $user_id)->where('letter_type', 1)->distinct('triggers')->pluck('triggers')->toArray();
        $triggers = $this->trigger;
        $campaign_merge = 1;
        $prob_class = 'add-un';
        $my_class = 'active';
        $pc = PropertyCampaign::withTrashed()->find(30);
        // dump($pc, preg_match_all('/\n/', $pc->edit_property_address));

        return view('campaign.list', compact('lists', 'triggers', 'campaign_merge', 'my_class', 'prob_class', 'exists'));
    }

    public function campaignHistory(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));

        $campaign = Campaign::where('user_id', $user->id)
            ->get()
        ;
        $letter = Letter::where('user_id', $user->id)->get();

        return view('campaign.history', compact('campaign', 'letter'));
    }

    public function campaignDetail(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));

        $campaign = Campaign::where('id', $id)
            ->with(
                'propertyCampaign',
                'propertyCampaign.houseNumber',
                'propertyCampaign.houseNumber.propertyHouseNumber',
                'propertyCampaign.houseNumber.propertyHouseNumber.propertyHistory',
                'propertyCampaign.houseNumber.propertyHouseNumber.propertyLetter',
                'propertyCampaign.houseNumber.propertyHouseNumber.propertyLetter.letter'
            )
            ->first()
        ;
        // dd($campaign->propertyCampaign[0]['houseNumber']['propertyHouseNumber'][0]['propertyHistory']['address']);die;

        return view('campaign.detail', compact('campaign'));
    }

    public function searchHistory(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $letter = Letter::where('user_id', $user->id)->get();
        $campaign = Campaign::where('user_id', $user->id)
            ->where('name', 'LIKE', '%'.$request->search.'%')
            ->get()
        ;

        return view('campaign.history', compact('campaign', 'letter'));
    }

    public function removeCampaign(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $campaign = PropertyCampaign::where('user_id', $user->id)->delete();
        $user_outcode = $this->userOutcode();
        $triggers = $this->trigger;
        $campaign_merge = 1;
        $prob_class = 'add-un';
        $my_class = 'active';
        $lists = PropertyCampaign::join('house_number as h', 'property_campaigns.house_number_id', 'h.id')
            ->join('postcode as pc', 'pc.id', 'h.postcode_id')
            ->join('outcode as oc', 'oc.id', 'pc.outcode_id')
            ->select('property_campaigns.*', 'h.house_number', DB::raw('CONCAT(oc.outcode,pc.postcode) as pincode'), 'h.listed_date', 'h.id as h_id')
            ->where('property_campaigns.user_id', $user->id)
            ->whereIn('oc.id', $user_outcode)
            ->orderBy('property_campaigns.id', 'desc')
            ->get()
    ;

        return view('campaign.list', compact('lists', 'triggers', 'campaign_merge', 'my_class', 'prob_class'));
    }

    public function removeCampaignId(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $delete = PropertyCampaign::where('id', $id)->delete();
        $callback = function ($query) {
            $query->where('campaign_status', '=', 1);
        };

        $campaign = PropertyCampaign::where('user_id', $user->id)
            ->with(
                'propertyHistory',
                'campaign',
                'user',
                'propertyHistory.agency',
                'propertyHistory.propertyLetter',
                'propertyHistory.propertyLetter.letter'
            )
            ->whereHas('campaign', $callback)
            ->with(['campaign' => $callback])
            ->get()
        ;
        $letter = Letter::where('user_id', $user->id)->get();

        return view('campaign.list', compact('campaign', 'letter'));
    }

    public function createLetter(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        if (!empty($request->file('file'))) {
            $path = $request->file('file')->store('public/letters');
        } else {
            $path = '';
        }

        Letter::create([
            'name' => $request->name,
            'user_id' => $user->id,
            'file' => $path,
            'trigger' => $request->trigger,
        ]);
        $letter = Letter::where('user_id', $user->id)->orderby('created_at', 'DESC')->get();

        return view('campaign.edit-letter', compact('letter'));
    }

    public function updateLetter(Request $request, $id)
    {
     
        $rules = [
            'name' => 'required',
            'trigger' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('msg_error', $validator->errors()->first());
        }
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        if (!empty($request->file('file'))) {
            $path = $request->file('file')->store('public/letters');
            Letter::where('id', $id)->update([
                'file' => $path,
            ]);
        }

        Letter::where('id', $id)->update([
            'name' => $request->name,
            'triggers' => $request->trigger,
            'content' => $request->content,
        ]);
        $letter = Letter::where('user_id', $user->id)->orderby('created_at', 'DESC')->get();

        return view('campaign.edit-letter', compact('letter'));
    }

    public function editLetter(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $letter = Letter::where('user_id', $user->id)->orderby('created_at', 'DESC')->get();

        return view('campaign.edit-letter', compact('letter'));
    }

    public function viewLetter(Request $request)
    {
        return view('campaign.view-letter');
    }

    public function manageLetter(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $letter = Letter::where('id', $id)->first();

        return view('campaign.manage-letter', compact('letter'));
    }

    public function deleteLetterId(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $delete = Letter::where('id', $id)->delete();
        $letter = Letter::where('user_id', $user->id)->orderby('created_at', 'DESC')->get();

        return view('campaign.edit-letter', compact('letter'));
    }

    // sanju-start
    public function update(Request $request, $id)
    {

        $campaign = Campaign::find($id);
        $campaign->name = $request->name;
        $campaign->save();

        $error = false;
        $message = 'Updated successfully';

        return compact('error', 'message');
    }

    public function show(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $campaign = Campaign::where('user_id', $user->id)->find($id);
        $page_title = 'Campaign History Details';

        if ($campaign) {
            $sendDate = '' != $campaign->sent_date ? Carbon::parse($campaign->sent_date)->format('d/m/Y') : '';

            $lists = PropertyCampaign::withTrashed()->join('house_number as h', 'property_campaigns.house_number_id', 'h.id')
                ->join('postcode as pc', 'pc.id', 'h.postcode_id')
                ->join('outcode as oc', 'oc.id', 'pc.outcode_id')
                ->select('property_campaigns.*', 'h.house_number', DB::raw('CONCAT(oc.outcode,pc.postcode) as pincode'), 'h.listed_date')
                ->where('property_campaigns.user_id', $user->id)
                ->where('property_campaigns.campaign_id', $campaign->id)
                ->get()
            ;

            if (count($lists) > 0) {
                $i = 0;
                foreach ($lists as $item) {
                    $properties = PropertyHistory::select('property_history.*', 'a.name as agency')
                        ->join('property_house_number as phn', 'phn.property_history_id', 'property_history.id')
                        ->join('agency as a', 'a.id', 'property_history.agency_id')
                        ->where('phn.house_number_id', $item->house_number_id)
                        ->get()
                    ;

                    $full_address = $item->house_number.',<br/> ';
                    $full_address .= (count($properties) > 0 && '' != $properties[0]->address) ? $properties[0]->address.',<br/>' : '';
                    // $full_address .= $item->pincode;
                    $agency = count($properties->pluck('agency')) > 0 ? implode(',', $properties->pluck('agency')->toArray()) : '';

                    // $campaign = Campaign::where('id', $item->campaign_id)
                    //     ->where('campaign_status', 1)
                    //     ->latest()
                    //     ->first();
                    $encrpt_id = Helper::encryptData($item->house_number_id);

                    $lastLetter = PropertyCampaign::withTrashed()
                        ->whereHas('campaign')
                        ->where('house_number_id', $item->house_number_id)
                        ->where('user_id', $user->id)
                        ->whereNotNull('campaign_id')
                        ->latest()->first();

                    ++$i;
                    $item->sl = $i;
                    $item->address = $full_address;
                    $item->property_url = route('propertyDetails', ['property_id' => $encrpt_id]);
                    $item->agency = $agency;
                    $item->trigger = $this->trigger[$item->trigger];
                    $item->listed_date = Carbon::parse($item->listed_date)->format('d/m/Y');
                    $item->last_letter_sent = $lastLetter ? $lastLetter->campaign->sent_date->format('d/m/Y') : '';
                }
            }

            return view('campaign.view', compact('campaign', 'sendDate', 'lists', 'page_title'));
        }

        return redirect()->route('campaign-history.index')->with('msg_error', 'Cannot find any campaign with this ID.');
    }

    // sanju-end

    //letter head
    public function letterHead(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $data = LetterHead::where('user_id', $user->id)->first();
        $mode='';
        if (!$data) {
            $data = '';
        }

        return view('campaign.letter_head', compact('data','mode'));
    }

    public function createLetterHead(Request $request)
    {
        $rules = [
            'letter_head_file' => 'required|mimes:pdf',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('msg_error', $validator->errors()->first());
        }

        if ($request->hasFile('letter_head_file')) {
            // Get filename with extension
            $filenameWithExt = $request->file('letter_head_file')->getClientOriginalName();
            // Get just filename
            $filename = pathinfo($filenameWithExt, PATHINFO_FILENAME);
            // Get just ext
            $extension = $request->file('letter_head_file')->getClientOriginalExtension();
            //Filename to store
            $fileNameToStore = time().'.'.$extension;
            // Upload Image
            $path = $request->file('letter_head_file')->storeAs('public/letter-heads', $fileNameToStore);
            $user = json_decode(Session::get(config('propai.authSessionKey')));
            $file = storage_path('app/public/letter-heads/'.$fileNameToStore);
            $filepdf = fopen($file, 'r');
            if ($filepdf) {
                $line_first = fgets($filepdf);
                preg_match_all('!\d+!', $line_first, $matches);
                $pdfversion = implode('.', $matches[0]);
                fclose($filepdf);
            } else {
                unlink($file);

                return Redirect::back()->with('datamsg_array', ['Something went wrong','red']);
            }
            $data = LetterHead::where('user_id', $user->id)->first();

            if (!$data) {
                $add = new LetterHead();
                $add->user_id = $user->id;
                $add->file = $fileNameToStore;
                $add->save();
                
                return Redirect::route('letterHead')->with('datamsg_array', ['Successfully Added','#25E9A5']);              
            }
                unlink(storage_path('app/public/letter-heads/'.$data['file']));
                $data = LetterHead::where('id', $data['id'])->update(['file' => $fileNameToStore,]);

                return Redirect::route('letterHead')->with('datamsg_array', ['Successfully Edited','#25E9A5']);              
            }            
        return Redirect::route('letterHead')->with('datamsg_array', ['Something went wrong','red']);        
    }

    public function letterHeadEdit()
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $data = LetterHead::where('user_id', $user->id)->first();
        $mode = 'edit';

        return view('campaign.letter_head', compact('data','mode'));
    }

    public function letterHeadDelete(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $data = LetterHead::where('user_id', $user->id)->first();
        if ($data != null && count($data->toArray()) > 1) {
            unlink(storage_path('app/public/letter-heads/'.$data['file']));
            LetterHead::where('id', $data['id'])->delete(); 
            return Redirect::route('letterHead')->with('datamsg_array', ['Successfully Deleted', '#25E9A5']);        
        } else {
            return Redirect::route('letterHead')->with('datamsg_array', ['Something went wrong', 'red']);           
        }
    }

    public function letterHeadPreview(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $data = LetterHead::where('user_id', $user->id)->first();
        if ($data != null && count($data->toArray()) > 1) {
            $filename = $data['file'];            
            $path = storage_path('app/public/letter-heads/'.$data['file']);

            return Response::make(file_get_contents($path), 200, [
                'Content-Type' => 'application/pdf',
                'Content-Disposition' => 'inline; filename="'.$filename.'"',
        ]);
        } else {
            return Redirect::route('letterHead')->with('datamsg_array', ['Something went wrong', 'red']);           
        }
    }
 
    //letter head

    public function WriteHTML($html, $pdf, $campaign)
    {
        //HTML parser
        $html = strip_tags($html, '<b><u><i><a><img><p><br><strong><em><font><tr><blockquote>'); //supprime tous les tags sauf ceux reconnus
        $html = str_replace("\n", ' ', $html); //remplace retour � la ligne par un espace
        $a = preg_split('/<(.*)>/U', $html, -1, PREG_SPLIT_DELIM_CAPTURE);

        //�clate la cha�ne avec les balises

        // $pdf = new \setasign\Fpdi\Fpdi('Portrait','mm',array(215.9,279.4), true, 'UTF-8', true);
        // $tpl = $pdf->importPage(1);
        // $pdf->AddPage();

        // Use the imported page as the template
        // $pdf->useTemplate($tpl);
        // $pdf->SetFont('Helvetica');
        foreach ($a as $i => $e) {
            if (0 == $i % 2) {
                //Text
                if ($this->HREF) {
                    $pdf->SetX(19.25);
                    $this->PutLink($this->HREF, $e, $pdf);
                } else {
                    // $pdf->SetX(19.25);
                    $pdf->Write(5, stripslashes($this->txtentities($e)));
                }
            } else {
                //Tag
                if ('/' == $e[0]) {
                    $this->CloseTag(strtoupper(substr($e, 1)), $pdf);
                } else {
                    //Extract attributes
                    $a2 = explode(' ', $e);
                    $tag = strtoupper(array_shift($a2));
                    $attr = [];
                    foreach ($a2 as $v) {
                        if (preg_match('/([^=]*)=["\']?([^"\']*)/', $v, $a3)) {
                            $attr[strtoupper($a3[1])] = $a3[2];
                        }
                    }
                    $this->OpenTag($tag, $attr, $pdf);
                }
            }
        }

        // $pdf->Output($campaign.'.pdf','I');
    }

    public function OpenTag($tag, $attr, $pdf)
    {
        //Opening tag
        switch ($tag) {
            case 'STRONG':
                $this->SetStyle('B', true, $pdf);

                break;

            case 'EM':
                $this->SetStyle('I', true, $pdf);

                break;

            case 'B':
            case 'I':
            case 'U':
                $this->SetStyle($tag, true, $pdf);

                break;

            case 'A':
                $this->HREF = $attr['HREF'];

                break;

            case 'IMG':
                if (isset($attr['SRC']) && (isset($attr['WIDTH']) || isset($attr['HEIGHT']))) {
                    if (!isset($attr['WIDTH'])) {
                        $attr['WIDTH'] = 0;
                    }
                    if (!isset($attr['HEIGHT'])) {
                        $attr['HEIGHT'] = 0;
                    }
                    $this->Image($attr['SRC'], $this->GetX(), $this->GetY(), px2mm($attr['WIDTH']), px2mm($attr['HEIGHT']));
                }

                break;

            case 'TR':
            case 'BLOCKQUOTE':
            case 'BR':
                $pdf->Ln(5);

                break;

            case 'P':
                $pdf->Ln(10);
                $pdf->SetX(19.25);

                break;

            case 'FONT':
                if (isset($attr['COLOR']) && '' != $attr['COLOR']) {
                    $coul = hex2dec($attr['COLOR']);
                    $pdf->SetTextColor($coul['R'], $coul['V'], $coul['B']);
                    $this->issetcolor = true;
                }
                if (isset($attr['FACE']) && in_array(strtolower($attr['FACE']), $this->fontlist)) {
                    $pdf->SetFont(strtolower($attr['FACE']));
                    $this->issetfont = true;
                }

                break;
        }
    }

    public function CloseTag($tag, $pdf)
    {
        //Closing tag
        if ('STRONG' == $tag) {
            $tag = 'B';
        }
        if ('EM' == $tag) {
            $tag = 'I';
        }
        if ('B' == $tag || 'I' == $tag || 'U' == $tag) {
            $this->SetStyle($tag, false, $pdf);
        }
        if ('A' == $tag) {
            $this->HREF = '';
        }
        if ('FONT' == $tag) {
            if (true == $this->issetcolor) {
                $pdf->SetTextColor(0);
            }
            if ($this->issetfont) {
                $pdf->SetFont('arial');
                $this->issetfont = false;
            }
        }
    }

    public function SetStyle($tag, $enable, $pdf)
    {
        //Modify style and select corresponding font
        $this->{$tag} += ($enable ? 1 : -1);
        $style = '';
        foreach (['B', 'I', 'U'] as $s) {
            if ($this->{$s} > 0) {
                $style .= $s;
            }
        }
        $pdf->SetFont('', $style);
    }

    public function PutLink($URL, $txt, $pdf)
    {
        //Put a hyperlink
        $pdf->SetTextColor(0, 0, 255);
        $this->SetStyle('U', true);
        $pdf->SetX(19.25);
        $this->Write(5, $txt, $URL);
        $this->SetStyle('U', false);
        $pdf->SetTextColor(0);
    }

    public function hex2dec($couleur = '#000000')
    {
        $R = substr($couleur, 1, 2);
        $rouge = hexdec($R);
        $V = substr($couleur, 3, 2);
        $vert = hexdec($V);
        $B = substr($couleur, 5, 2);
        $bleu = hexdec($B);
        $tbl_couleur = [];
        $tbl_couleur['R'] = $rouge;
        $tbl_couleur['V'] = $vert;
        $tbl_couleur['B'] = $bleu;

        return $tbl_couleur;
    }

    //conversion pixel -> millimeter at 72 dpi
    public function px2mm($px)
    {
        return $px * 25.4 / 72;
    }

    public function txtentities($html)
    {
        $trans = get_html_translation_table(HTML_ENTITIES);
        $trans = array_flip($trans);
        return strtr($html, $trans);
    }

    public function fetchPropertyAddress($house_number_id)
    {
        $res            = [];
        $propCampaign   = PropertyCampaign::where('house_number_id', $house_number_id)->first();
        if ($propCampaign) {
            $query              = $this->commonPropQuery();
            $property_details   = $query->where('house_number.id', $propCampaign->house_number_id)->first();
            if ($property_details) {
                $addr                   = $property_details->address;
                $hno                    = $property_details->house_number;
                $property               = [];
                $property['address']    = str_replace(' ,', ',', Helper::parseProbateAddress($hno.', '.$addr, true, true, ','));
                $property['hno']        = $propCampaign->house_number_id;
                $res[]                  = $property;
            }
        }
        return $res;
    }

    public function campaignEditAddress(Request $request)
    {
        $res = [];
        foreach ($request->data as $data) {
            $propCampaign = PropertyCampaign::where('id', $data[0])->first();

            if ($propCampaign) {
                $query = $this->commonPropQuery();
                $property_details = $query->where('house_number.id', $propCampaign->house_number_id)->first();

                if ($property_details) {
                    $addr = $property_details->address;
                    $hno = $property_details->house_number;
                    $property = [];
                    $property['address'] = str_replace(' ,', ',', Helper::parseProbateAddress($hno.' '.$addr, true, true, ','));
                    $property['hno'] = $propCampaign->house_number_id;
                    $property['id'] = $data[0];
                    $property['trigger'] = $data[1];
                    $res[] = $property;
                }
            }
        }

        return response()->json($res, 200);
    }

    public function campaignEditAddressSave(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if (1 == $request->print_type || 3 == $request->print_type) {
            $source_data = LetterHead::where('user_id', $user_id)->first();
            if (!$source_data) {
                return response()->json(['msg' => 'Please upload Letterhead and try again.'], 400);
            }
        }

        $campaign = new Campaign();
        $campaign->name = 'Campaign-'.Carbon::now()->format('d-m-Y h:i');
        $campaign->campaign_status = 0;
        $campaign->user_id = $user_id;
        $campaign->save();

        foreach ($request->pcid as $key => $pc) {
            $propCampaign = PropertyCampaign::where('id', $pc)->first();

            if ($propCampaign) {
                $propCampaign->campaign_id = $campaign->id;
                $propCampaign->trigger = $request->trigger[$key];
                $propCampaign->edit_property_address = $request->addr[$key];
                $propCampaign->save();

                $flag = PropertyFlag::updateOrCreate(
                    ['house_number_id' => $propCampaign->house_number_id, 'user_id' => $user_id],
                    ['house_number_id' => $propCampaign->house_number_id, 'user_id' => $user_id, 'flag' => 3]
                );

                $letterHistory = new CampaignLetterHistory();
                $letterHistory->house_number_id = $propCampaign->house_number_id;
                $letterHistory->user_id = $user_id;
                $letterHistory->letter_type = $request->trigger[$key];
                $letterHistory->save();
            }
        }

        return response()->json([
            'msg' => 'Campaign submitted successfully',
            'error' => false,
            'url' => url('campaigns/print/'.$campaign->id.'/'.$request->print_type),
        ], 200);
    }
}
