<?php

// <summary>
// File : CampaignController
// File Type : Controller
// Creator : Susmi Sidharthan
// Email: susmi@designdirect@gmail.com
// Company: Design Direct UK
// Description ; This is used as web controller for all estate agent campaign controller.
// </summary>

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Mail\ContactCampaign;
use App\Model\Campaign;
use App\Model\CampaignLetterHistory;
use App\Model\Letter;
use App\Model\LetterHead;
use App\Model\Property;
use App\Model\PropertyCampaign;
use App\Model\PropertyFlag;
use App\Model\PropertyHistory;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Session;
use Redirect;
use Validator;
use Yajra\DataTables\DataTables;

class CampaignController extends Controller
{
    public $status = [
        1 => 'New to Market',
        2 => 'For sale',
        3 => 'Sold',
        4 => 'Withdrawn',
        5 => 'Price drop',
        6 => 'Fallen Through',
    ];
    public $trigger = [
        0 => 'General Letters',
        1 => 'New to Market',
        2 => 'For sale',
        3 => 'Sold',
        4 => 'Withdrawn',
        5 => 'Price drop',
        6 => 'Fallen Through',
        7 => '8 Week',
        8 => '16 Week',
        9 => '22 Week',
    ];

    public function index()
    {
        $page = collect();
        $page->title = 'Campaign History';
        $page->link = url('campaign-history');
        $page_title = 'Campaign History';

        return view('campaign/histories', compact('page', 'page_title'));
    }

    public function lists(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $detail = Campaign::select(['name', 'sent_date', 'campaign_status', 'id', 'created_at'])
            ->where('user_id', $user_id)
            ->orderBy('campaigns.id', 'desc')
        ;

        return DataTables::of($detail)
            ->editColumn('sent_date', function ($detail) {
                return '' != $detail->sent_date ? Carbon::parse($detail->sent_date)->format('d/m/Y').Carbon::parse($detail->created_at)->format(' h:i a') : '';
            })
            ->editColumn('campaign_status', function ($detail) {
                return 1 == $detail->campaign_status ? 'Sent' : 'Sending';
            })
            ->addColumn('action', function ($detail) {
                $print_url = url('campaigns/print/'.$detail->id);
                $action = '';
                // $action .= '<a class="btn btn-primary btn-sm mr-2" target="_blank" href="' . $print_url . '">Print</a>';
                $action .= Helper::deleteButton($detail->id);
                $action .= '<button class="btn btn-success btn-sm mr-2 editCampaign" data-id="'.$detail->id.'" data-name="'.$detail->name.'">Edit</button>';
                $action .= '<a class="btn btn-info btn-sm mr-2" href=" '.route('campaign-history.show', $detail->id).'">View</a>';

                return $action;
            })
            ->removeColumn(['id', 'created_at'])
            ->escapeColumns([])
            ->make(false)
        ;
    }

    public function destroy($id): array
    {
        Campaign::destroy($id);
        $error = false;
        $message = 'Campaign deleted successfully';

        return compact('error', 'message');
    }

    //property add to campaign
    public function addToCampaignProperty(Request $request)
    {
        $rules = [
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }

        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        foreach ($request->property as $property_id) {
            $exist_data = PropertyCampaign::where('user_id', $user_id)->where('house_number_id', $property_id)->where('campaign_id', null)->first();

            if (!$exist_data) {
                $add = new PropertyCampaign();
                $add->house_number_id = $property_id;
                $add->user_id = $user_id;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully added to campaign...!');
    }

    //property add to campaign
    public function createCampaign(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $campaign = new Campaign();
        $campaign->name = 'Campaign-'.Carbon::now()->format('d-m-Y h:i');
        $campaign->campaign_status = 0;
        $campaign->user_id = $user_id;
        $campaign->save();
        foreach ($request->property_id as $property_id) {
            $trigger = 'trigger_'.$property_id;
            PropertyCampaign::where('id', $property_id)
                ->update([
                    'campaign_id' => $campaign->id,
                    'trigger' => $request->{$trigger},
                ])
            ;
        }

        return redirect('campaign-history');
    }

    //property add to campaign
    public function printCampaign(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $campaign = new Campaign();
        $campaign->name = 'Campaign-'.Carbon::now()->format('d-m-Y h:i');
        $campaign->campaign_status = 0;
        $campaign->user_id = $user_id;
        $campaign->save();
        foreach ($request->data as $data) {
            PropertyCampaign::where('id', $data[0])
                ->update([
                    'campaign_id' => $campaign->id,
                    'trigger' => $data[1],
                ])
            ;

            $propertyCampaign = PropertyCampaign::where('id', $data[0])->first();
            if ($propertyCampaign) {
                $flag = PropertyFlag::updateOrCreate(
                    ['house_number_id' => $propertyCampaign->house_number_id, 'user_id' => $user_id],
                    ['house_number_id' => $propertyCampaign->house_number_id, 'user_id' => $user_id, 'flag' => 3]
                );

                $letterHistory = new CampaignLetterHistory();
                $letterHistory->house_number_id = $propertyCampaign->house_number_id;
                $letterHistory->user_id = $user_id;
                $letterHistory->letter_type = $data[1];
                $letterHistory->save();
            }
        }

        $url = url('campaigns/print/'.$campaign->id);
        $error = false;
        $message = 'campaign submitted successfully';

        return compact('message', 'url', 'error');
    }

    public function printLetter($campaign_id)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $campaign = Campaign::find($campaign_id);
        $campaign->campaign_status = 1;
        $campaign->sent_date = Carbon::now();
        $campaign->save();
        $properties = PropertyCampaign::where('campaign_id', $campaign_id)
            ->get()
        ;
        if (count($properties) > 0) {
            foreach ($properties as $property) {
                ini_set('memory_limit', '-1');
                $property->address = PropertyHistory::join('property_house_number as phn', 'property_history.id', 'phn.property_history_id')
                    ->join('house_number as hn', 'hn.id', 'phn.house_number_id')
                    ->join('postcode as pc', 'pc.id', 'hn.postcode_id')
                    ->join('outcode as oc', 'oc.id', 'pc.outcode_id')
                    ->where('hn.id', $property->house_number_id)
                    ->select('hn.house_number', 'property_history.address', 'pc.postcode', 'oc.outcode')
                    ->first()
                ;
                $property->letter = Letter::where('triggers', $property->trigger)->where('user_id', $user_id)->first();
                PropertyCampaign::find($property->id)->delete();
            }

            // return view('campaign.print', compact('properties'));
            $pdf = App::make('dompdf.wrapper');
            $pdf->loadView('campaign.print', compact('properties'));

            return $pdf->download($campaign->name.'.pdf');
        }
    }

    //property add to campaign
    public function contactCampaign(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $campaign = new Campaign();
        $campaign->name = strtoupper(config('propai.appName')).' - '.Carbon::now()->format('d-m-Y h:i');
        $campaign->campaign_status = 0;
        $campaign->user_id = $user_id;
        $campaign->save();

        foreach ($request->property_id as $property_id) {
            $trigger = 'trigger_'.$property_id;
            PropertyCampaign::where('id', $property_id)
                ->update([
                    'campaign_id' => $campaign->id,
                    'trigger' => $request->{$trigger},
                ])
            ;
        }
        Mail::to('bijodesigndirectuk@gmail.com')
            ->bcc('rohandesigndirect@gmail.com')
            ->queue(new ContactCampaign($campaign->name))
        ;

        $url = url('campaign-history');

        return redirect($url);
    }

    public function allCampaign(Request $request)
    {
        $page_title = 'My Campaign';
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_outcode = $this->userOutcode();
        $lists = PropertyCampaign::join('house_number as h', 'property_campaigns.house_number_id', 'h.id')
            ->join('postcode as pc', 'pc.id', 'h.postcode_id')
            ->join('outcode as oc', 'oc.id', 'pc.outcode_id')
            ->select('property_campaigns.*', 'h.house_number', DB::raw('CONCAT(oc.outcode,pc.postcode) as pincode'), 'h.listed_date', 'h.id as h_id')
            ->where('property_campaigns.user_id', $user_id)
            ->whereIn('oc.id', $user_outcode)
            ->orderBy('property_campaigns.id', 'desc')
            ->get()
        ;

        if (count($lists) > 0) {
            $i = 0;
            foreach ($lists as $item) {
                $properties = PropertyHistory::select('property_history.*', 'a.name as agency')
                    ->join('property_house_number as phn', 'phn.property_history_id', 'property_history.id')
                    ->join('agency as a', 'a.id', 'property_history.agency_id')
                    ->where('phn.house_number_id', $item->house_number_id)
                    ->get()
                ;
                $full_address = $item->house_number.',<br/> ';
                $full_address .= (count($properties) > 0 && '' != $properties[0]->address) ? $properties[0]->address.',<br/>' : '';
                // $full_address .= $item->pincode;
                $agency = count($properties->pluck('agency')) > 0 ? implode(',', $properties->pluck('agency')->toArray()) : '';

                $campaign = Campaign::where('id', $item->campaign_id)
                    ->where('campaign_status', 1)
                    ->latest()
                    ->first()
                ;

                ++$i;
                $item->sl = $i;
                $item->address = $full_address;
                $item->agency = $agency;
                $item->trigger = Helper::selectTrigger($item, $properties);
                $item->listed_date = Carbon::parse($item->listed_date)->format('d/m/Y');
                $item->last_letter_sent = $campaign ? Carbon::parse($campaign->send_date)->format('d/m/Y') : '';
                $item->last_date_sent = '';
            }
        }

        return view('campaign.list', compact('lists', 'page_title'));
    }

    public function campaignHistory(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));

        $campaign = Campaign::where('user_id', $user->id)
            ->get()
        ;
        $letter = Letter::where('user_id', $user->id)->get();

        return view('campaign.history', compact('campaign', 'letter'));
    }

    public function campaignDetail(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));

        $campaign = Campaign::where('id', $id)
            ->with(
                'propertyCampaign',
                'propertyCampaign.houseNumber',
                'propertyCampaign.houseNumber.propertyHouseNumber',
                'propertyCampaign.houseNumber.propertyHouseNumber.propertyHistory',
                'propertyCampaign.houseNumber.propertyHouseNumber.propertyLetter',
                'propertyCampaign.houseNumber.propertyHouseNumber.propertyLetter.letter'
            )
            ->first()
        ;
        // dd($campaign->propertyCampaign[0]['houseNumber']['propertyHouseNumber'][0]['propertyHistory']['address']);die;

        return view('campaign.detail', compact('campaign'));
    }

    public function searchHistory(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $letter = Letter::where('user_id', $user->id)->get();
        $campaign = Campaign::where('user_id', $user->id)
            ->where('name', 'LIKE', '%'.$request->search.'%')
            ->get()
        ;

        return view('campaign.history', compact('campaign', 'letter'));
    }

    public function removeCampaign(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $campaign = PropertyCampaign::where('user_id', $user->id)->delete();

        return view('campaign.empty');
    }

    public function removeCampaignId(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $delete = PropertyCampaign::where('id', $id)->delete();
        $callback = function ($query) {
            $query->where('campaign_status', '=', 1);
        };

        $campaign = PropertyCampaign::where('user_id', $user->id)
            ->with(
                'propertyHistory',
                'campaign',
                'user',
                'propertyHistory.agency',
                'propertyHistory.propertyLetter',
                'propertyHistory.propertyLetter.letter'
            )
            ->whereHas('campaign', $callback)
            ->with(['campaign' => $callback])
            ->get()
        ;
        $letter = Letter::where('user_id', $user->id)->get();

        return view('campaign.list', compact('campaign', 'letter'));
    }

    public function createLetter(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        if (!empty($request->file('file'))) {
            $path = $request->file('file')->store('public/letters');
        } else {
            $path = '';
        }

        Letter::create([
            'name' => $request->name,
            'user_id' => $user->id,
            'file' => $path,
            'trigger' => $request->trigger,
        ]);
        $letter = Letter::where('user_id', $user->id)->orderby('created_at', 'DESC')->get();

        return view('campaign.edit-letter', compact('letter'));
    }

    public function updateLetter(Request $request, $id)
    {
        $rules = [
            'name' => 'required',
            'trigger' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('msg_error', $validator->errors()->first());
        }
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        if (!empty($request->file('file'))) {
            $path = $request->file('file')->store('public/letters');
            Letter::where('id', $id)->update([
                'file' => $path,
            ]);
        }

        Letter::where('id', $id)->update([
            'name' => $request->name,
            'triggers' => $request->trigger,
            'content' => $request->content,
        ]);
        $letter = Letter::where('user_id', $user->id)->orderby('created_at', 'DESC')->get();

        return view('campaign.edit-letter', compact('letter'));
    }

    public function editLetter(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $letter = Letter::where('user_id', $user->id)->orderby('created_at', 'DESC')->get();

        return view('campaign.edit-letter', compact('letter'));
    }

    public function viewLetter(Request $request)
    {
        return view('campaign.view-letter');
    }

    public function manageLetter(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $letter = Letter::where('id', $id)->first();

        return view('campaign.manage-letter', compact('letter'));
    }

    public function deleteLetterId(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $delete = Letter::where('id', $id)->delete();
        $letter = Letter::where('user_id', $user->id)->orderby('created_at', 'DESC')->get();

        return view('campaign.edit-letter', compact('letter'));
    }

    // sanju-start
    public function update(Request $request, $id)
    {
        $campaign = Campaign::find($id);
        $campaign->name = $request->name;
        $campaign->save();

        $error = false;
        $message = 'Updated successfully';

        return compact('error', 'message');
    }

    public function show(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $campaign = Campaign::where('user_id', $user->id)->find($id);
        $page_title = 'Campaign History Details';

        if ($campaign) {
            $sendDate = '' != $campaign->sent_date ? Carbon::parse($campaign->sent_date)->format('d/m/Y') : '';

            $lists = PropertyCampaign::withTrashed()->join('house_number as h', 'property_campaigns.house_number_id', 'h.id')
                ->join('postcode as pc', 'pc.id', 'h.postcode_id')
                ->join('outcode as oc', 'oc.id', 'pc.outcode_id')
                ->select('property_campaigns.*', 'h.house_number', DB::raw('CONCAT(oc.outcode,pc.postcode) as pincode'), 'h.listed_date')
                ->where('property_campaigns.user_id', $user->id)
                ->where('property_campaigns.campaign_id', $campaign->id)
                ->get()
            ;

            if (count($lists) > 0) {
                $i = 0;
                foreach ($lists as $item) {
                    $properties = PropertyHistory::select('property_history.*', 'a.name as agency')
                        ->join('property_house_number as phn', 'phn.property_history_id', 'property_history.id')
                        ->join('agency as a', 'a.id', 'property_history.agency_id')
                        ->where('phn.house_number_id', $item->house_number_id)
                        ->get()
                    ;

                    $full_address = $item->house_number.',<br/> ';
                    $full_address .= (count($properties) > 0 && '' != $properties[0]->address) ? $properties[0]->address.',<br/>' : '';
                    // $full_address .= $item->pincode;
                    $agency = count($properties->pluck('agency')) > 0 ? implode(',', $properties->pluck('agency')->toArray()) : '';

                    // $campaign = Campaign::where('id', $item->campaign_id)
                    //     ->where('campaign_status', 1)
                    //     ->latest()
                    //     ->first();
                    $encrpt_id = Helper::encryptData($item->house_number_id);

                    ++$i;
                    $item->sl = $i;
                    $item->address = $full_address;
                    $item->property_url = route('propertyDetails', ['property_id' => $encrpt_id]);
                    $item->agency = $agency;
                    $item->trigger = $this->trigger[$item->trigger];
                    $item->listed_date = Carbon::parse($item->listed_date)->format('d/m/Y');
                    $item->last_letter_sent = $campaign ? Carbon::parse($campaign->send_date)->format('d/m/Y') : '';
                }
            }

            return view('campaign.view', compact('campaign', 'sendDate', 'lists', 'page_title'));
        }

        return redirect()->route('campaign-history.index')->with('msg_error', 'Cannot find any campaign with this ID.');
    }

    // sanju-end

    //letter head
    public function letterHead(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $data = LetterHead::where('user_id', $user->id)->first();
        if (!$data) {
            $data = '';
        }

        return view('campaign.letter_head', compact('data'));
    }

    public function createLetterHead(Request $request)
    {
        $rules = [
            'letter_head_file' => 'required|mimes:pdf',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('msg_error', $validator->errors()->first());
        }

        if ($request->hasFile('letter_head_file')) {
            // Get filename with extension
            $filenameWithExt = $request->file('letter_head_file')->getClientOriginalName();
            // Get just filename
            $filename = pathinfo($filenameWithExt, PATHINFO_FILENAME);
            // Get just ext
            $extension = $request->file('letter_head_file')->getClientOriginalExtension();
            //Filename to store
            $fileNameToStore = time().'.'.$extension;
            // Upload Image
            $path = $request->file('letter_head_file')->storeAs('public/letter-heads', $fileNameToStore);
            $user = json_decode(Session::get(config('propai.authSessionKey')));

            $data = LetterHead::where('user_id', $user->id)->first();
            if (!$data) {
                $add = new LetterHead();
                $add->user_id = $user->id;
                $add->file = $fileNameToStore;
                $add->save();

                return Redirect::route('letterHead')->with('msg_success', 'Successfully Added.');
            }
            unlink(storage_path('app/public/letter-heads/'.$data['file']));

            LetterHead::where('id', $data['id'])
                ->update([
                    'file' => $fileNameToStore,
                ])
                ;

            return Redirect::route('letterHead')->with('msg_success', 'Successfully Updated.');
        }

        return Redirect::back()->with('msg_error', 'Something went wrong.');
    }

    public function letterHeadEdit()
    {
        $data = '';

        return view('campaign.letter_head', compact('data'));
    }

    public function letterHeadDelete(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));

        $data = LetterHead::where('user_id', $user->id)->first();
        unlink(storage_path('app/public/letter-heads/'.$data['file']));
        LetterHead::where('id', $data['id'])
            ->delete()
        ;

        return Redirect::route('letterHead')->with('msg_success', 'Successfully Deleted.');
    }

    //letter head
}
