<?php

namespace App\Http\Controllers;

use App\Model\PropertyHistory;
use App\Model\PropertyPriceHistory;
use App\Model\ProbateProperty;
use App\Model\ProbatePropertyCampaign;
use App\Model\UserOutcode;
use Illuminate\Routing\Controller as BaseController;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Session;

class Controller extends BaseController
{
    // use AuthorizesRequests, DispatchesJobs, ValidatesRequests;
    public function simpleReturn($status = '', $data = array(), $code = 200)
    {

        $out['status'] = $status;

        $out['data'] = $data;

        return response()->json($out, $code);

    }
    //probate properties
    public function commonProbateQuery($user_outcode)
    {
         return ProbateProperty::join('executor','executor.id','probate_property.executor_id')
        ->join("postcode", "postcode.id", "probate_property.postcode_id")
        ->join("outcode", "outcode.id", "postcode.outcode_id")
        ->leftjoin("probate_property_flag", "probate_property_flag.probate_id", "probate_property.id")
        ->whereIn('outcode.id', $user_outcode);
    }

    //common query
    public function commonPropQuery()
    {
        return PropertyHistory::join("property_house_number", "property_history.id", "property_house_number.property_history_id")
            ->join("house_number", "house_number.id", "property_house_number.house_number_id")
            ->join("postcode", "postcode.id", "house_number.postcode_id")
            ->join("outcode", "outcode.id", "postcode.outcode_id")
            ->leftjoin("agency", "agency.id", "property_history.agency_id")
            ->leftjoin("property_flag", "property_flag.house_number_id", "property_house_number.house_number_id")
            ->whereNull('property_house_number.deleted_at')
            ->whereNull('house_number.deleted_at')
            ->where(function ($query) {
            $query->where('property_history.unconfirmed', '<=', '1')
            ->orWhereNull('property_history.unconfirmed');
            })
            ->select('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', "house_number.latest_price", "house_number.latest_status", 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id', 'house_number.created_by')
            ->groupBy('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', "house_number.latest_price", "house_number.latest_status", 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id','agency.name');
    }

    public function commonFreshPropQuery()
    {
        // raw query method
        /*$prop_results = DB::select(DB::raw("SELECT property_house_number.house_number_id AS id 
                                            FROM `property_price_history` 
                                            JOIN property_history ON property_history.id = property_price_history.property_history_id
                                            JOIN property_house_number ON property_house_number.property_history_id = property_history.id
                                            WHERE (property_price_history.type LIKE '%Withdrawn%'
                                            OR  property_price_history.type LIKE '%STC%')
                                            -- OR  property_price_history.type LIKE '%sold%'
                                            -- OR  property_price_history.type LIKE '%Removed%'
                                            -- OR  property_price_history.type LIKE '%Price %'
                                            -- OR  property_price_history.type LIKE '%listed%'
                                            GROUP BY property_house_number.house_number_id"));
        $prop_house_number_id_array = [];
        foreach ($prop_results as $house_number_key => $house_number_value) {
            $prop_house_number_id_array[] = $house_number_value->id;
        }
        $prop_house_number_id_array = array_filter($prop_house_number_id_array);
        $prop_house_number_id_array = array_unique($prop_house_number_id_array);*/
        // query using models
        /*$prop_house_number_id_array = PropertyPriceHistory::select('property_house_number.house_number_id', 'property_price_history.created_at', 'property_price_history.type')
                                        ->join('property_history', 'property_history.id', 'property_price_history.property_history_id')
                                        ->join('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
                                        ->where(function ($query) {
                                            $query->where('property_price_history.type', 'like', '%Withdrawn%')
                                                ->orwhere('property_price_history.type', 'like', '%STC%');
                                                # ->orwhere('property_price_history.type', 'like', '%sold%');
                                                # ->orwhere('property_price_history.type', 'like', '%Removed%');
                                                # ->orwhere('property_price_history.type', 'like', '%Price%');
                                                # ->orwhere('property_price_history.type', 'like', '%listed%');
                                            })
                                        ->orderBy('property_price_history.created_at')
                                        ->groupBy('property_house_number.house_number_id', 'property_price_history.created_at', 'property_price_history.type');
                                        // ->first()
                                        // ->toArray();*/
        // new Property filtered Array
        $prop_results_1 = DB::select(DB::raw("SELECT parent_table_PPH.property_history_id as id
                                            FROM property_price_history as parent_table_PPH
                                            join property_house_number as parent_join_table_PHN ON parent_join_table_PHN.property_history_id = parent_table_PPH.property_history_id
                                            WHERE parent_table_PPH.id IN (SELECT MAX(child_table_PPH.id)
                                                                            FROM property_price_history as child_table_PPH
                                                                            GROUP BY child_table_PPH.property_history_id)
                                            AND parent_table_PPH.type NOT LIKE '%Withdrawn%'
                                            AND parent_table_PPH.type != 'STC'  
                                            ORDER BY `parent_table_PPH`.`property_history_id` ASC"));
        $prop_property_history_id_array = [];
        foreach ($prop_results_1 as $house_number_key => $house_number_value) {
            $prop_property_history_id_array[] = $house_number_value->id;
        }
        $prop_property_history_id_array = array_filter($prop_property_history_id_array);
        $prop_property_history_id_array = array_unique($prop_property_history_id_array);
        // dd($prop_property_history_id_array);

        /*$prop_results_2 = DB::select(DB::raw("SELECT parent_join_table_PHN.house_number_id as id
                                            FROM property_price_history as parent_table_PPH
                                            join property_house_number as parent_join_table_PHN ON parent_join_table_PHN.property_history_id = parent_table_PPH.property_history_id
                                            WHERE parent_table_PPH.id IN (SELECT MAX(child_table_PPH.id)
                                                                            FROM property_price_history as child_table_PPH
                                                                            GROUP BY child_table_PPH.property_history_id)
                                            AND parent_table_PPH.type NOT LIKE '%Withdrawn%'
                                            AND parent_table_PPH.type != 'STC'  
                                            ORDER BY `parent_table_PPH`.`property_history_id` ASC"));
        $prop_house_number_id_array = [];
        foreach ($prop_results_2 as $house_number_key => $house_number_value) {
            $prop_house_number_id_array[] = $house_number_value->id;
        }
        $prop_house_number_id_array = array_filter($prop_house_number_id_array);
        $prop_house_number_id_array = array_unique($prop_house_number_id_array);*/
        // dd($prop_house_number_id_array);

        return PropertyHistory::join("property_house_number", "property_history.id", "property_house_number.property_history_id")
            ->join("property_price_history", "property_price_history.property_history_id", "property_history.id")
            ->join("house_number", "house_number.id", "property_house_number.house_number_id")
            ->join("postcode", "postcode.id", "house_number.postcode_id")
            ->join("outcode", "outcode.id", "postcode.outcode_id")
            ->leftjoin("agency", "agency.id", "property_history.agency_id")
            ->leftjoin("property_flag", "property_flag.house_number_id", "property_house_number.house_number_id")
            ->whereNull('house_number.deleted_at')
            ->where(function ($query) {
            $query->where('property_history.unconfirmed', '<=', '1')
                ->orWhereNull('property_history.unconfirmed');
            })
            // ->whereNotIn('property_history.for_sale', [3, 6])
            ->whereIn('property_price_history.property_history_id', $prop_property_history_id_array)
            // ->whereIn('property_house_number.house_number_id', $prop_house_number_id_array)
            ->select('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', "house_number.latest_price", "house_number.latest_status", 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id', 'house_number.created_by')
            ->groupBy('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', "house_number.latest_price", "house_number.latest_status", 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id','agency.name');
    }
 
    // user outcode
    public function userOutcode()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $data = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_outcode.user_id', $user_id)->where('outcode.archive', 0)->pluck('user_outcode.outcode_id');
        return $data;
    }
    public function userOutcodes()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $data = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)
        ->where('user_outcode.user_id', $user_id)
        ->where('outcode.archive', 0)
        ->orderBy('outcode.outcode','asc')
        ->pluck('outcode.outcode')
        ->toArray();
        return array_unique($data);
    }

    //Send Mail
    public function propertyDataSendMail($mail_data, $template)
    {
        try {
            $data = Mail::send('mails.' . $template, $mail_data, function ($message) use ($mail_data) {
                $message->to($mail_data['_email'], $mail_data['_name'])->subject($mail_data['_subject']);
            });
        } catch (\Exception $e) {
            return 1;
            // Get error here
        }
    }

    public function houseNumberPortal($house_number, $platform = "")
    {

        $data = PropertyHistory::join("property_house_number", "property_history.id", "property_house_number.property_history_id")
            ->join("house_number", "house_number.id", "property_house_number.house_number_id")
            ->where("house_number.id", $house_number);
        if (($platform != -1 && $platform != 'platform' && $platform != '')) {
            $data = $data->where('property_history.portal', $platform);
        }
        $data = $data->select('property_history.portal_url', 'property_history.portal')
        // ->groupBy()
        ->get();

        return $data;

    }

}
