<?php

// <summary>
// File : PropertyController
// File Type : Controller
// Creator : Susmi Sidharthan
// Email: susmi@designdirect@gmail.com
// Company: Design Direct UK
// Description ; This is used as web controller for all estate agent Property controller.
// </summary>

namespace App\Http\Controllers\DataOperator;

use App\Helpers\Helper;
use App\Http\Controllers\Controller;
use App\Model\Agency;
use App\Model\DeoPropertyComment;
use App\Model\HouseNumber;
use App\Model\Outcode;
use App\Model\PostCode;
use App\Model\PropertyHistory;
use App\Model\PropertyHouseNumber;
use App\Model\PropertyPriceHistory;
use App\Model\PropertySaleHistory;
use App\Model\SubmittedProperty;
use App\Model\Withdrawn;
use App\Model\User;
use Carbon\Carbon;
use DB;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Session;
use Redirect;
use Validator;

class PropertyController extends Controller
{
    public function getProperty()
    {
        $page_title = 'All Properties';
        $filter = true;
        $outcode = Outcode::orderBy('outcode', 'asc')->get();
        $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');
        $deo_data =    json_decode(Session::get(config('propai.authSessionKey')));
        $deo_name = $deo_data->first_name.' '.$deo_data->last_name;
        // dd($deo_name);
        return view('data-operator.property', compact('page_title', 'filter', 'property', 'outcode','deo_name'));
    }

    public function propertyArray(Request $request)
    {
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
            ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
            ->where('property_house_number.house_number_id', null)
            ->where('property_house_number.property_history_id', null)
            ->where(function ($query) {
                $query->where('property_history.archive', '=', '0')
                    ->orWhereNull('property_history.archive')
                ;
            })
            ->Where('postcode.postcode', '<>', 'Unknown')
            ->Where('agency.name', '<>', 'Unknown')
            ->where('property_history.property_type', 'not like', '%flat%')
            ->where('property_history.property_type', 'not like', '%apartment%')
            ->where('property_history.property_type', 'not like', '%land%')
            ->where('property_history.property_type', 'not like', '%penthouse%')
            ->select('property_history.id', 'outcode.outcode', 'postcode.postcode', 'property_history.address', 'agency.name  AS agency_name', 'property_history.property_type', 'property_history.price', 'property_history.portal_url', 'property_history.latitude', 'property_history.longitude', 'property_history.portal', 'property_history.title')
        ;
        // ->groupBy("property_history.id", 'outcode.outcode', 'postcode.postcode', 'property_history.address', 'property_history.property_type', 'property_history.price', 'property_history.portal_url', 'property_history.latitude', 'property_history.longitude','property_history.portal','property_history.title');
        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('outcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('platform')) {
            if (('-1' != $request->platform && 'undefined' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'postcode',
            2 => 'address',
            3 => 'price',
            4 => 'agency',
            5 => 'type',
        ];

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy("listed_date", "DESC")->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                    ->orWhere('agency.name', 'like', '%'.$search.'%')
                    ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                    ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                    ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();
            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                    ->orWhere('agency.name', 'like', '%'.$search.'%')
                    ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                    ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                    ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ;
            })->orderBy("listed_date", "DESC")
                ->offset($start)
                ->limit($limit)
                ->get()
            ;

            // $totalFiltered = $query->Where('property_history.address', 'like', '%' . $search . '%')
            //     ->orWhere('agency.name', 'like', '%' . $search . '%')
            //     ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            //     ->count();
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                // dd($item);
                $encrpt_id = Helper::encryptData($item->id);
                //    $url = route('DEOpropertyDetails',['property_id'=>$encrpt_id]);
                // $map_url = "https://www.google.com/maps?q=$item->latitude,$item->longitude";

                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = "https://www.google.com/maps?q={$item->postcode}";
                }

                $nestedData['sl'] = $i;
                $nestedData['property_number'] = '<div class="input-group">
                    <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                    <div class="input-group-append">
                        <button data-id="'.$i.'"  type="button" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                    </div>
                </div>';
                $nestedData['not_found'] = "<a target='street_view' title='Property Number Can not Find' class='btn btn-danger btn-sm ml-2 archive' prop_id='{$item->id}' href='{$map_url}'><span class='svg-icon svg-icon-white svg-icon-2x'><svg  width='24px' height='24px' viewBox='0 0 24 24' version='1.1'>
                    <g stroke='none' stroke-width='2' fill='none' fill-rule='evenodd'>
                        <rect x='0' y='0' width='24' height='24'/>
                        <path d='M12,22 C6.4771525,22 2,17.5228475 2,12 C2,6.4771525 6.4771525,2 12,2 C17.5228475,2 22,6.4771525 22,12 C22,17.5228475 17.5228475,22 12,22 Z M12,20 C16.418278,20 20,16.418278 20,12 C20,7.581722 16.418278,4 12,4 C7.581722,4 4,7.581722 4,12 C4,16.418278 7.581722,20 12,20 Z M19.0710678,4.92893219 L19.0710678,4.92893219 C19.4615921,5.31945648 19.4615921,5.95262146 19.0710678,6.34314575 L6.34314575,19.0710678 C5.95262146,19.4615921 5.31945648,19.4615921 4.92893219,19.0710678 L4.92893219,19.0710678 C4.5384079,18.6805435 4.5384079,18.0473785 4.92893219,17.6568542 L17.6568542,4.92893219 C18.0473785,4.5384079 18.6805435,4.5384079 19.0710678,4.92893219 Z' fill='#000000' fill-rule='nonzero' opacity='0.3'/>
                    </g>
                </svg></span></a>";
                if (!empty($item->house_number)) {
                    $address2 = $item->address;
                } else {
                    $address2 = $item->address.',<br>  '.$item->postcode;
                }

                $nestedData['address'] = $address2.$request->post_code;
                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound; '.number_format($item->price);
                }

                // $nestedData['price'] = "&pound;" . number_format($item->price);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['comment'] = '<a href="javascript:void()" history-id='.$item->id.'  class="btn btn-primary font-weight-bolder btn-sm mr-3 addComment" title="Comment">
                <i class="flaticon-comment icon-lg"></i>
                </a>';

                //portal url

                if (0 == $item->portal) {
                    $urls = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove  btn-sm' title='Click here to see the listing'  href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $urls = "<a title='Click here to see the listing' target='zoopla' title='Click here to see the listing' class='btn btn-zoopla  btn-sm ml-2' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                $nestedData['url'] = $urls."<a target='street_view' title='Access Street View' class='btn btn-danger btn-sm ml-2' href='{$map_url}' style='display:none;'><i class='flaticon2-map icon-lg'></i></a>";

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    public function propertyDetails(Request $request)
    {
        $propert_id = Helper::decryptData($request->property_id);
        $page_title = 'Find House Number';
        $details = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->join('agency', 'agency.id', 'property_history.agency_id')
            ->where('property_history.id', $propert_id)
            ->select('property_history.id', 'property_history.address', 'property_history.latitude', 'property_history.longitude', 'property_history.bed_rooms', 'property_history.fallen_through', 'outcode.outcode', 'postcode.postcode', DB::raw("GROUP_CONCAT(DISTINCT agency.name SEPARATOR ', ') AS agency_name"), 'property_history.property_type', 'property_history.portal_url', 'property_history.archive')
            ->groupBy('property_history.id', 'property_history.address', 'property_history.latitude', 'property_history.longitude', 'property_history.bed_rooms', 'property_history.fallen_through', 'outcode.outcode', 'postcode.postcode', 'property_history.property_type', 'property_history.portal_url', 'property_history.archive')->first();
        // dd($details);
        return view('data-operator.property-details', compact('page_title', 'details', 'propert_id'));
    }

    //update house number
    public function postHouseNumber(Request $request)
    { 
        $rules = [
            'property_number' => 'required',
            'property' => 'required|unique:property_house_number,property_history_id',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        DB::beginTransaction();

        if($request->confirm_mode==3){
            $update = PropertyHistory::where('id', $request->property)
            ->update(['archive' => 1])
            ;
            if ($update) {
                DB::commit();

                return $this->simpleReturn('success', 'Successfully marked this house number not found...!');
            }else{
                DB::rollback();

                return $this->simpleReturn('error', 'Something went wrong...!');
            }
        }else{

            $details = PropertyHistory::where('id', $request->property)->first();

            $data = HouseNumber::join('postcode', 'postcode.id', 'house_number.postcode_id')
                ->join('outcode', 'outcode.id', 'postcode.outcode_id')
                ->where('house_number.house_number', $request->property_number)
                ->where('postcode.id', $details->postcode_id)
                ->select('house_number.id')
                ->first()
                ;

            if ($data) { //house number is existed
                $property_number = $data->id;
                $add = new PropertyHouseNumber();
                $add->property_history_id = $request->property;
                $add->house_number_id = $property_number;
                $add->created_by = $user_id;
                if ($add->save()) {
                    $update = PropertyHistory::where('id', $request->property)
                        ->update(['archive' => 0])
                        ;
                    if ($update) {
                        $add = new SubmittedProperty();
                        $add->property_history_id = $request->property;
                        $add->user_id = $user_id;
                        $add->type = 0;
                        $add->submitted_mode = $request->confirm_mode;
                        $add->submitted_date = date('Y-m-d');
                        if ($add->save()) {
                            $allPropertyHouseNumber = PropertyHouseNumber::where('house_number_id', $property_number);

                            $propertiesGet = $allPropertyHouseNumber->get();

                            //check all price history latest are STC or OFFER ,Same as zoopla
                            $data2 = [];
                            foreach ($propertiesGet as $KEY => $VAL) {
                                $data2[] = PropertyPriceHistory::where('property_history_id', $VAL->property_history_id)
                                    ->where('type', '<>', 'Withdrawn')->orderby('created_at', 'desc')->orderby('id', 'desc')->first();
                            }

                            if (count($data2) >= 1) {
                                $Data2 = [];
                                foreach ($data2 as $kk => $vv) {
                                    if ($vv) {
                                        $Data2[] = $vv['type'];
                                    }
                                }

                                if (1 == count(array_unique($Data2))) {
                                    if ('STC' == array_unique($Data2)[0]) {
                                        $status = 6;
                                        $newMarket = 0;
                                    // $stcCount = $stcCount + 1;
                                    } elseif ('Offer' == array_unique($Data2)[0]) {
                                        // print_r($data1);die;
                                        $status = 7;
                                        $newMarket = 0;
                                    // $offerCount = $offerCount + 1;
                                    } elseif ('First listed' == array_unique($Data2)[0]) {
                                        $newMarket = 1;
                                        $status = 1;
                                    } else {
                                        $newMarket = 0;
                                        $status = 1;
                                    }

                                    // Updating status in house nr table
                                    $houseNumberGet = HouseNumber::where('id', $property_number);
                                    $updateHouseNumberStatus = $houseNumberGet->update([
                                        'latest_status' => $status,
                                        'new_to_market' => $newMarket,
                                    ]);
                                }
                            }

                            //withdrawn
                            $data2 = [];
                            foreach ($propertiesGet as $KEY => $VAL) {
                                $data2[] = PropertyPriceHistory::where('property_history_id', $VAL->property_history_id)
                                    ->orderby('created_at', 'desc')->orderby('id', 'desc')->first();
                            }

                            if (count($data2) >= 1) {
                                $Data2 = [];
                                foreach ($data2 as $kk => $vv) {
                                    if ($vv) {
                                        $Data2[] = $vv['type'];
                                    }
                                }

                                if (1 == count(array_unique($Data2))) {
                                    if ('Withdrawn' == array_unique($Data2)[0]) {
                                        $status = 3;
                                        $newMarket = 0;
                                        // Updating status in house nr table
                                        HouseNumber::where('id', $property_number)->update([
                                            'latest_status' => 3,
                                            'withdrawn_date' => Carbon::now()->toDateString(),
                                            'price_change_diff' => null,
                                            'price_drop_date' => null,
                                            'sold_date' => null,
                                            'last_sold_price' => null,
                                            'sold_year' => null,
                                            'new_to_market' => 0,
                                        ]);
                                    }
                                }
                            }

                            $prop_his_id = PropertyHouseNumber::where('house_number_id', $property_number)->pluck('property_history_id');

                            $latest_price = PropertyPriceHistory::whereIn('property_history_id', $prop_his_id)
                                ->where('type', '<>', 'Withdrawn')->orderby('created_at', 'desc')->orderby('id', 'desc')->first();

                            // update price
                            HouseNumber::where('id', $property_number)->update([
                                'latest_price' => $latest_price->price,
                            ]);

                            DB::commit();

                            return Redirect::route('DEOVerifyProperty')->with('success', 'Verification successfully updated...!');
                        }
                        DB::rollback();

                        return $this->simpleReturn('error', 'Something went wrong...!');
                    }
                    DB::rollback();

                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
                DB::rollback();

                return $this->simpleReturn('error', 'Something went wrong...!');
            }
            $add = new HouseNumber();
            $add->house_number = $request->property_number;
            $add->listed_date = $details->listed_date;
            $add->sold_date = $details->sold_date;
            $add->price_drop_date = $details->price_drop_date;
            $add->withdrawn_date = $details->withdrawn_date;
            $add->postcode_id = $details->postcode_id;
            $add->latest_price = $details->price;
            $add->latest_status = $details->property_current_status;
            $add->price_change_diff = $details->price_drop;
            $add->latitude = $details->latitude;
            $add->longitude = $details->longitude;
            $add->created_by = $user_id;
            if ($add->save()) {
                $property_number = $add->id;

                $add = new PropertyHouseNumber();
                $add->property_history_id = $request->property;
                $add->house_number_id = $property_number;
                $add->created_by = $user_id;
                if ($add->save()) {
                    $update = PropertyHistory::where('id', $request->property)
                        ->update(['archive' => 0])
                            ;
                    if ($update) {
                        $add = new SubmittedProperty();
                        $add->property_history_id = $request->property;
                        $add->user_id = $user_id;
                        $add->type = 0;
                        $add->submitted_mode = $request->confirm_mode;
                        $add->submitted_date = date('Y-m-d');
                        if ($add->save()) {
                            DB::commit();

                            return $this->simpleReturn('success', 'House number successfully updated...!');
                        }
                        DB::rollback();

                        return $this->simpleReturn('error', 'Something went wrong...!');
                    }
                    DB::rollback();

                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
                DB::rollback();

                return $this->simpleReturn('error', 'Something went wrong...!');
            }

            DB::rollback();

            return $this->simpleReturn('error', 'Something went wrong...!');
        }
    }

    //move to archive
    public function moveArchive(Request $request)
    {
        $rules = [
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('error', $validator->errors()->first());
        }
        $update = PropertyHistory::where('id', $request->property)
            ->update(['archive' => 1])
            ;
        if ($update) {
            return $this->simpleReturn('success', 'Successfully marked this house number not found...!');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    public function getArchiveProperty(Request $request)
    {
        $page_title = 'Archive Properties';
        $filter = true;
        $outcode = Outcode::orderBy('outcode', 'asc')->get();

        $deo_data =    json_decode(Session::get(config('propai.authSessionKey')));
        $deo_name = $deo_data->first_name.' '.$deo_data->last_name;

        return view('data-operator.archive-property', compact('page_title', 'filter', 'outcode','deo_name'));
    }

    public function archiveArray(Request $request)
    {
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->join('agency', 'agency.id', 'property_history.agency_id')
            ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
            ->where('property_house_number.house_number_id', null)
            ->where('property_house_number.property_history_id', null)
            ->where('property_history.archive', 1)
            ->where('property_history.property_type', 'not like', '%flat%')
            ->where('property_history.property_type', 'not like', '%apartment%')
            ->where('property_history.property_type', 'not like', '%land%')
            ->where('property_history.property_type', 'not like', '%penthouse%')
            ->select('property_history.id', 'outcode.outcode', 'postcode.postcode', 'property_history.address', 'agency.name  AS agency_name', 'property_history.property_type', 'property_history.price', 'property_history.portal_url', 'property_history.latitude', 'property_history.longitude', 'property_history.title', 'property_history.portal')
        ;

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('outcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('platform')) {
            if (('-1' != $request->platform && 'undefined' != $request->platform)) {
                $query = $query->where('property_history.portal', $request->input('platform'));
            }
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            // 0 => 'outcode',
            0 => 'postcode',
            1 => 'address',
            2 => 'price',
            3 => 'agency',
            4 => 'type',
            5 => 'outcode',
        ];
        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy("property_history.listed_date", "DESC")->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                    ->orWhere('agency.name', 'like', '%'.$search.'%')
                    ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                    ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                    ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                    ->orWhere('agency.name', 'like', '%'.$search.'%')
                    ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                    ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                    ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ;
            })->orderBy("property_history.listed_date", "DESC")
                ->offset($start)
                ->limit($limit)
                ->get()
            ;

            // $totalFiltered = $query->Where('property_history.address', 'like', '%' . $search . '%')
            //     ->orWhere('agency.name', 'like', '%' . $search . '%')
            //     ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            //     ->count();
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                $url = route('DEOpropertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['sl'] = $i++;
                $nestedData['postcode'] = $item->postcode;
                // $map_url = "https://www.google.com/maps?q=$item->latitude,$item->longitude";
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = "https://www.google.com/maps?q={$item->postcode}";
                }

                ++$i;
                $nestedData['sl'] = $i++;
                $nestedData['property_number'] = '<div class="input-group">
                    <input value="" type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                    <div class="input-group-append">
                        <button data-id="'.$i.'"  type="button" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                    </div>
                </div>';
                if (!empty($item->house_number)) {
                    $address2 = $item->address;
                } else {
                    $address2 = $item->address.',<br> '.$item->postcode;
                }

                $nestedData['address'] = $address2;
                // $nestedData['price'] = "&pound;" . number_format($item->latest_price, 2);
                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound; '.number_format($item->price);
                }
                if ($item->title) {
                    $nestedData['type'] = $item->title.', <br> '.ucfirst($item->property_type);
                } else {
                    $nestedData['type'] = ucfirst($item->property_type);
                }

                $nestedData['comment'] = '<a href="javascript:void()" history-id='.$item->id.'  class="btn btn-primary font-weight-bolder btn-sm mr-3 addComment" title="Comment">
                <i class="flaticon-comment icon-lg"></i>
                </a>';
                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove  btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a><a target='street_view' title='Access Street View' class='btn btn-danger btn-sm ml-2' href='{$map_url}' style='display:none;'><i class='flaticon2-map icon-lg'></i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla  btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a><a target='street_view' title='Access Street View' class='btn btn-danger btn-sm ml-2' href='{$map_url}' style='display:none;'><i class='flaticon2-map icon-lg'></i></a>";
                }
                $nestedData['portal_url'] = $url;
                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    public function verifyProperty()
    {
        // $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        // dump($user_id);

        $page_title = 'Verify All Property';
        $search_keyword = session()->get('deo_verify_search');
        $search_outcode = session()->get('deo_verify_outcode');
        return view('data-operator.verify-property', compact('page_title','search_keyword','search_outcode'));
    }

    public function verifyPropertyArray(Request $request)
    {

        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $submitedproperty = SubmittedProperty::where('type', 1)->pluck('property_history_id')->toArray();

        // $query=PropertyHouseNumber::with(
        //     'houseNumber',
        //     'houseNumber.postCode',
        //     'houseNumber.postCode.outCode',
        //     'propertyHistory',
        //     'propertyHistory.agency'
        // )
        // ->whereNotIn('property_history_id',$submitedproperty);

        // $query = PropertyHistory::join("postcode", "postcode.id", "property_history.postcode_id")->join("outcode", "outcode.id", "postcode.outcode_id")
        //     ->leftjoin("agency", "agency.id", "property_history.agency_id")
        //     ->join("property_house_number", "property_house_number.property_history_id", "property_history.id")
        //     ->join("house_number", "property_house_number.house_number_id", "house_number.id")
        //     ->join('submitted_property', 'property_history.id', 'submitted_property.property_history_id')
        //     ->where('submitted_property.user_id', '=', $user_id)
        //     ->where(function ($query) {
        //         $query->where('property_history.archive', '=', '0')
        //             ->orWhereNull('property_history.archive');
        //     })
        //     // ->whereNotIn('property_history_id', $submitedproperty)
        //     ->select("property_history.id", 'outcode.outcode', 'postcode.postcode', 'property_history.address', 'agency.name  AS agency_name', 'property_history.property_type', 'property_history.price', 'house_number.house_number', 'house_number.listed_date', 'house_number.latest_price', 'property_history.title', 'house_number.created_by');


        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->join('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
        ->join('house_number', 'property_house_number.house_number_id', 'house_number.id')
        ->join('submitted_property', 'property_history.id', 'submitted_property.property_history_id')
        ->where('submitted_property.user_id', '=', $user_id)
        ->where(function ($query) {
            $query->where('property_history.archive', '=', '0')
                ->orWhereNull('property_history.archive')
            ;
        })
        ->has('submittedProperties', '=', 1)->where('submitted_property.type', '=', 0)
        // ->whereNotIn('property_history_id', $submitedproperty)
        ->select('property_history.id', 'outcode.outcode', 'postcode.postcode', 'property_history.address', 'agency.name  AS agency_name', 'property_history.property_type', 'property_history.price', 'house_number.house_number', 'house_number.listed_date', 'house_number.latest_price', 'property_history.title', 'house_number.created_by','property_history.portal')
        ;
        // ->groupBy("property_history.id", 'outcode.outcode', 'postcode.postcode', 'property_history.address', 'property_history.property_type', 'property_history.price','house_number.house_number','house_number.listed_date');

        if ($request->has('outcode') && $request->input('outcode') != '') {
            $query->where('outcode.outcode', $request->input('outcode'));
            session()->put('deo_verify_outcode',$request->input('outcode'));
        }else{
            session()->forget('deo_verify_outcode');
        }
        // Table's primary key
        $primaryKey = 'id';

        $columns = array(
            0 => 'outcode.outcode',
            1 => 'address',
            2 => 'price',
            3 => 'agency_name',
            4 => 'house_number.listed_date',
            5 => 'type',
        );

        if ($request->has('outcode') && $request->input('outcode') != '') {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = Outcode::
            join('postcode', 'postcode.outcode_id', 'outcode.id')->where('archive', 0)
            ->join('house_number', 'house_number.postcode_id', 'postcode.id')
            // ->orderBy('outcode.updated_at', 'desc')
            ->distinct('outcode.id')
            ->pluck('outcode.outcode');

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            $list = $query->orderBy("property_history.listed_date", "DESC")->offset($start)->limit($limit)->get()->unique('id');
            session()->forget('deo_verify_search');
        } else {

            $search = $request->input('search.value');

            session()->put('deo_verify_search',$search);
           

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%' . $search . '%')
                    ->orWhere('agency.name', 'like', '%' . $search . '%')
                    ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
                    ->orWhere('outcode.outcode', 'like', '%' . $search . '%')
                    ->orWhere('postcode.postcode', 'like', '%' . $search . '%')
                    ->orWhere('house_number.house_number', 'like', '%' . $search . '%');

            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%' . $search . '%')
                    ->orWhere('agency.name', 'like', '%' . $search . '%')
                    ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
                    ->orWhere('outcode.outcode', 'like', '%' . $search . '%')
                    ->orWhere('postcode.postcode', 'like', '%' . $search . '%')
                    ->orWhere('house_number.house_number', 'like', '%' . $search . '%');

            })->orderBy("property_history.listed_date", "DESC")
                ->offset($start)
                ->limit($limit)
                ->get()->unique('id');

           
        }

        $dataArr = array();
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                
                $house_number = $item->house_number;
                $latest_price = $item->latest_price;
                $listed_date = $item->listed_date;
                $address = $item->address;
                $property_type = ucfirst($item->property_type);
                $postcode = $item->postcode;
                $outcode = $item->outcode;

                $encrpt_id = Helper::encryptData($item->id);
                $url = route('DEOpropertyDetails', ['property_id' => $encrpt_id]);
                $i++;
                $nestedData['id'] = $item->id;
                $nestedData['sl'] = $i;
                if (!empty($item->house_number)) {
                    $address2 = $house_number . ',' . $address;
                } else {
                    $address2 = $house_number . ',' . $address . ',<br>  ' . $postcode;
                }
                if (!empty($item->created_by)) {

                    $address2 = $house_number . ',' . $address . ',<br>  ' . $postcode;
                }
                $nestedData['address'] = "<a href='$url'>$address2</a>";

                // $nestedData['price'] = "&pound;" . number_format($latest_price, 2);
                if ($item->title) {
                    $nestedData['type'] = $item->title . ', <br> ' . ucfirst($item->property_type);
                } else {
                    $nestedData['type'] = ucfirst($item->property_type);
                }
                $nestedData['agency'] = $item->agency_name;

                if ($item->listed_date) {
                    // $listed_date=date('Y-m-d',strtotime($item->listed_date));
                    // $today=date('Y-m-d');
                    // $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                    // .'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = "Hidden from public";
                }

                if ($latest_price == 0) {
                    $nestedData['price'] = "POA";
                } else {
                    $nestedData['price'] = "&pound; " . number_format($latest_price);
                }
                // $map_url = "https://www.google.com/maps?q=$item->latitude,$item->longitude";

                if($item->latitude!='' && $item->longitude!=''){
                    $map_url = "https://www.google.com/maps?q=$item->latitude,$item->longitude";
                }else{
                    $map_url = "https://www.google.com/maps?q=$item->postcode";
                }

                if ($item->portal == 0) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='$item->portal_url'><i class='icon-lg'>R</i></a><a target='street_view' title='Access Street View' class='btn btn-danger btn-sm ml-2' href='$map_url'><i class='flaticon2-map icon-lg'></i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='$item->portal_url'><i class='icon-lg'>Z</i></a><a target='street_view' title='Access Street View' class='btn btn-danger btn-sm ml-2' href='$map_url'><i class='flaticon2-map icon-lg'></i></a>";
                }

                // $nestedData['agency'] = '';
                $nestedData['type'] = $item->title . ' <br> ' . ucfirst($property_type);
                $nestedData['outcode'] = $outcode;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $dataArr,
            "outcode" => ($outcodes),
        );

        return json_encode($json_data);
    }

    public function verifyPropertyDetails(Request $request)
    {
        $page_title = 'Verify Property';
        $property_history = Helper::decryptData($request->property_id);

        // $query = $this->commonPropQuery();

        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
            ->join('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
            ->join('house_number', 'property_house_number.house_number_id', 'house_number.id')
            ->select('property_history.id', 'outcode.outcode', 'postcode.postcode', 'property_history.address', DB::raw("GROUP_CONCAT(DISTINCT agency.name SEPARATOR ', ') AS agency_name"), 'property_history.property_type', 'property_history.price', 'property_history.portal_url', 'property_history.latitude', 'property_history.longitude', 'property_history.portal', 'house_number.house_number', 'property_house_number.house_number_id', 'house_number.latest_price', 'property_history.listed_date')
            ->groupBy('property_history.id', 'outcode.outcode', 'postcode.postcode', 'property_history.address', 'property_history.property_type', 'property_history.price', 'property_history.portal_url', 'property_history.latitude', 'property_history.longitude', 'property_history.portal', 'house_number.house_number', 'property_house_number.house_number_id', 'house_number.latest_price', 'property_history.listed_date')
        ;

        $details = $query->where('property_history.id', $property_history)->first();

        // dd($details);
        return view('data-operator.verify-property-details', compact('details'));
    }

    public function submittedDetails(Request $request)
    {
        $page_title = 'Submitted Property';
        $property_history = Helper::decryptData($request->property_id);

        // $query = $this->commonPropQuery();

        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
            ->join('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
            ->join('house_number', 'property_house_number.house_number_id', 'house_number.id')
            ->select('property_history.id', 'outcode.outcode', 'postcode.postcode', 'property_history.address', DB::raw("GROUP_CONCAT(DISTINCT agency.name SEPARATOR ', ') AS agency_name"), 'property_history.property_type', 'property_history.price', 'property_history.portal_url', 'property_history.latitude', 'property_history.longitude', 'property_history.portal', 'house_number.house_number', 'property_house_number.house_number_id', 'house_number.latest_price', 'property_history.listed_date')
            ->groupBy('property_history.id', 'outcode.outcode', 'postcode.postcode', 'property_history.address', 'property_history.property_type', 'property_history.price', 'property_history.portal_url', 'property_history.latitude', 'property_history.longitude', 'property_history.portal', 'house_number.house_number', 'property_house_number.house_number_id', 'house_number.latest_price', 'property_history.listed_date')
        ;

        $details = $query->where('property_history.id', $property_history)->first();

        // dd($details);
        return view('data-operator.submitted-property-details', compact('details'));
    }

    //edit property
    public function editProperty(Request $request)
    {
        $rules = [
            'property' => 'required',
            'property_number' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('error', $validator->errors()->first());
        }
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        DB::beginTransaction();

        $details = PropertyHistory::where('id', $request->property)->first();

        $data = HouseNumber::join('postcode', 'postcode.id', 'house_number.postcode_id')
            ->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->where('house_number.house_number', $request->property_number)
            ->where('postcode.id', $details->postcode_id)
            ->select('house_number.id')
            ->first()
            ;

        $delete = PropertyHouseNumber::where('property_history_id', $request->property)->delete();
        if ($data) { //house number is existed
            $property_number = $data->id;
            $add = new PropertyHouseNumber();
            $add->property_history_id = $request->property;
            $add->house_number_id = $property_number;
            $add->created_by = $user_id;
            if ($add->save()) {
                $update = PropertyHistory::where('id', $request->property)
                    ->update(['archive' => 0])
                    ;
                if ($update) {
                    $add = new SubmittedProperty();
                    $add->property_history_id = $request->property;
                    $add->user_id = $user_id;
                    $add->type = 0;
                    $add->submitted_date = date('Y-m-d');
                    if ($add->save()) {
                        $allPropertyHouseNumber = PropertyHouseNumber::where('house_number_id', $property_number);

                        $propertiesGet = $allPropertyHouseNumber->get();

                        //check all price history latest are STC or OFFER ,Same as zoopla
                        $data2 = [];
                        foreach ($propertiesGet as $KEY => $VAL) {
                            $data2[] = PropertyPriceHistory::where('property_history_id', $VAL->property_history_id)
                                ->where('type', '<>', 'Withdrawn')->orderby('created_at', 'desc')->orderby('id', 'desc')->first();
                        }

                        if (count($data2) >= 1) {
                            $Data2 = [];
                            foreach ($data2 as $kk => $vv) {
                                if ($vv) {
                                    $Data2[] = $vv['type'];
                                }
                            }

                            if (1 == count(array_unique($Data2))) {
                                if ('STC' == array_unique($Data2)[0]) {
                                    $status = 6;
                                    $newMarket = 0;
                                // $stcCount = $stcCount + 1;
                                } elseif ('Offer' == array_unique($Data2)[0]) {
                                    // print_r($data1);die;
                                    $status = 7;
                                    $newMarket = 0;
                                // $offerCount = $offerCount + 1;
                                } elseif ('First listed' == array_unique($Data2)[0]) {
                                    $newMarket = 1;
                                    $status = 1;
                                } else {
                                    $newMarket = 0;
                                    $status = 1;
                                }

                                // Updating status in house nr table
                                $houseNumberGet = HouseNumber::where('id', $property_number);
                                $updateHouseNumberStatus = $houseNumberGet->update([
                                    'latest_status' => $status,
                                    'new_to_market' => $newMarket,
                                ]);
                            }
                        }

                        //withdrawn
                        $data2 = [];
                        foreach ($propertiesGet as $KEY => $VAL) {
                            $data2[] = PropertyPriceHistory::where('property_history_id', $VAL->property_history_id)
                                ->orderby('created_at', 'desc')->orderby('id', 'desc')->first();
                        }

                        if (count($data2) >= 1) {
                            $Data2 = [];
                            foreach ($data2 as $kk => $vv) {
                                if ($vv) {
                                    $Data2[] = $vv['type'];
                                }
                            }

                            if (1 == count(array_unique($Data2))) {
                                if ('Withdrawn' == array_unique($Data2)[0]) {
                                    $status = 3;
                                    $newMarket = 0;
                                    // Updating status in house nr table
                                    HouseNumber::where('id', $property_number)->update([
                                        'latest_status' => 3,
                                        'withdrawn_date' => Carbon::now()->toDateString(),
                                        'price_change_diff' => null,
                                        'price_drop_date' => null,
                                        'sold_date' => null,
                                        'last_sold_price' => null,
                                        'sold_year' => null,
                                        'new_to_market' => 0,
                                    ]);
                                }
                            }
                        }

                        $prop_his_id = PropertyHouseNumber::where('house_number_id', $property_number)->pluck('property_history_id');

                        $latest_price = PropertyPriceHistory::whereIn('property_history_id', $prop_his_id)
                            ->where('type', '<>', 'Withdrawn')->orderby('created_at', 'desc')->orderby('id', 'desc')->first();

                        // update price
                        HouseNumber::where('id', $property_number)->update([
                            'latest_price' => $latest_price->price,
                        ]);

                        DB::commit();

                        // return Redirect::route('DEOVerifyProperty')->with('success', 'Verification successfully updated...!');

                        return $this->simpleReturn("success","Verification successfully updated...!");
                    }
                    DB::rollback();

                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
                DB::rollback();

                return $this->simpleReturn('error', 'Something went wrong...!');
            }
            DB::rollback();

            return $this->simpleReturn('error', 'Something went wrong...!');
        }
        $add = new HouseNumber();
        $add->house_number = $request->property_number;
        $add->listed_date = $details->listed_date;
        $add->sold_date = $details->sold_date;
        $add->price_drop_date = $details->price_drop_date;
        $add->withdrawn_date = $details->withdrawn_date;
        $add->postcode_id = $details->postcode_id;
        $add->latest_price = $details->price;
        $add->latest_status = $details->property_current_status;
        $add->price_change_diff = $details->price_drop;
        $add->latitude = $details->latitude;
        $add->longitude = $details->longitude;
        $add->created_by = $user_id;
        if ($add->save()) {
            $property_number = $add->id;

            $add = new PropertyHouseNumber();
            $add->property_history_id = $request->property;
            $add->house_number_id = $property_number;
            $add->created_by = $user_id;
            if ($add->save()) {
                $update = PropertyHistory::where('id', $request->property)
                    ->update(['archive' => 0])
                        ;
                if ($update) {
                    $add = new SubmittedProperty();
                    $add->property_history_id = $request->property;
                    $add->user_id = $user_id;
                    $add->type = 0;
                    $add->submitted_date = date('Y-m-d');
                    if ($add->save()) {
                        DB::commit();

                        return $this->simpleReturn('success', 'Verification successfully updated...!');
                    }
                    DB::rollback();

                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
                DB::rollback();

                return $this->simpleReturn('error', 'Something went wrong...!');
            }
            DB::rollback();

            return $this->simpleReturn('error', 'Something went wrong...!');
        }

        DB::rollback();

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    //confirm property
    public function confirmProperty(Request $request)
    {
        $rules = [
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $add = new SubmittedProperty();
        $add->property_history_id = $request->property;
        $add->user_id = $user_id;
        $add->type = 1;
        $add->submitted_date = date('Y-m-d');
        if ($add->save()) {
            return $this->simpleReturn('success', 'Successfully marked this property verified');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    //Submitted property
    public function submittedProperty(Request $request)
    {
        $page_title = 'Submitted Properties';
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        if (3 == $user_type) {
            $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        } else {
            $user_id = $request->user_id;
        }
        $post_code = Postcode::get();
        $totalProp = true;
        session()->put('submitted_user',$user_id) ;
        $filter = true;
        $agency = Agency::get();
        $property = PropertyHistory::select('property_type', 'id')->get()->unique('property_type');

        $count = collect();
        $date = Carbon::now()->format('Y-m-d');
        $week = Carbon::now()->subWeek()->format('Y-m-d');
        $month = Carbon::now()->format('m');
        $year = Carbon::now()->format('Y');
        $count->today = SubmittedProperty::whereDate('submitted_date', $date)->where('user_id', $user_id)->count('property_history_id');
        $count->month = SubmittedProperty::whereMonth('submitted_date', $month)->whereYear('submitted_date', $year)->where('user_id', $user_id)->count('property_history_id');
        $count->year = SubmittedProperty::whereYear('submitted_date', $year)->where('user_id', $user_id)->count('property_history_id');
        $count->week = SubmittedProperty::whereBetween('submitted_date', [$week, $date])->where('user_id', $user_id)->count('property_history_id');

        return view('data-operator.submitted-property', compact('page_title', 'user_id', 'post_code', 'user_type', 'totalProp', 'filter', 'property', 'agency', 'count'));
    }

    public function submittedPropertyArray(Request $request)
    {
        $user_id = $request->userid;
       
        $query = SubmittedProperty::join('property_history', 'property_history.id', 'submitted_property.property_history_id')
            ->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
            ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
            ->join('postcode', 'postcode.id', 'house_number.postcode_id')
            ->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->join('agency', 'agency.id', 'property_history.agency_id')
            ->select('agency.name  AS agency_name', 'property_history.address', 'property_history.property_type', 'house_number.latest_price', 'house_number.created_by', 'house_number.latest_status', 'property_history.listed_date', 'house_number.house_number', 'postcode.postcode', 'outcode.outcode', 'submitted_property.type', 'submitted_property.submitted_date','submitted_property.submitted_mode','submitted_property.user_id','submitted_property.property_history_id')
        ;
        // ->groupBy('submitted_property.property_history_id', 'property_history.address', 'property_history.property_type', "house_number.latest_price", "house_number.latest_status", 'property_history.listed_date', 'house_number.house_number', 'postcode.postcode', 'outcode.outcode', 'submitted_property.type', 'submitted_property.submitted_date', 'property_house_number.house_number_id');

        $query = $query->where('submitted_property.user_id', $user_id);

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'undefined' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
            }
        }

        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('type')) {
            if ((0 != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'address',
            1 => 'latest_price',
            2 => 'property_history.listed_date',
            3 => 'class',
            4 => 'property_type',
            5 => 'submitted_date',
            6 => 'type',
        ];

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                    ->orWhere('agency.name', 'like', '%'.$search.'%')
                    ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                    ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                    ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();
            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                    ->orWhere('agency.name', 'like', '%'.$search.'%')
                    ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                    ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                    ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
                ->offset($start)
                ->limit($limit)
                ->get()
            ;

            // $totalFiltered = $query->Where('property_history.address', 'like', '%' . $search . '%')
            //     ->orWhere('agency.name', 'like', '%' . $search . '%')
            //     ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            //     ->count();
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                // $encrpt_id = Helper::encryptData($item->house_number_id);
                // $url = route('DEOpropertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.','.$item->address;
                } else {
                    $address2 = $item->house_number.','.$item->address.', <br> '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.','.$item->address.', <br> '.$item->postcode;
                }

                $encrpt_id = Helper::encryptData($item->property_history_id );
                $url = route('submittedDetails', ['property_id' => $encrpt_id]);

                $nestedData['address'] = "<a href='$url'>$address2</a>";

                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div>Number of Days : '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $class = '';

                $user_deo = User::where('id',$item->user_id)->first();
                $user_deo_name = mb_substr($user_deo['first_name'], 0, 3);
                // $user_deo_name = $user_deo['first_name'];
                if($item->submitted_mode==1){
                    $class = "C - ".$user_deo_name;
                }elseif($item->submitted_mode==2){
                    $class ="G - ".$user_deo_name;
                }
                $nestedData['class'] = $class;
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                $nestedData['submit_date'] = date('d M Y', strtotime($item->submitted_date));
                $nestedData['submit_type'] = $item->type;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    public function filterOperator(Request $request)
    {
        $count = collect();
        if (null == $request->date_from && null == $request->date_to) {
            $count->today = SubmittedProperty::getByDate($request, 'today');
            $count->month = SubmittedProperty::getByDate($request, 'month');
            $count->year = SubmittedProperty::getByDate($request, 'year');
            $count->week = SubmittedProperty::getByDate($request, 'week');
        } else {
            $count->range = SubmittedProperty::getByDate($request, 'range');
        }
        
        $user_id=session()->get('submitted_user');
        $count->founded_properties = SubmittedProperty::where('user_id',$user_id)->where('type',0)->count();
        $count->not_founded_properties = SubmittedProperty::where('user_id',$user_id)->where('type',5)->count();
        return view('admin.filter-operator', compact('count'));
    }

    public function addProperty(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));

        return view('data-operator.add-property');
    }

    public function addPropertyArray(Request $request)
    {
        $path = $request->file('file')->getRealPath();
        $dataArr = array_map('str_getcsv', file($path));

        if ('csv' == $request->file('file')->getClientOriginalExtension()) {
            $stock_head = [
                'address',
                'portal',
                'for_sale',
                'agency',
                'outcode',
                'postcode',
                'price_history_type',
                'price_history_date',
                'price_history_price',
                'price',
                'property_type',
                'bedrooms',
                'fallen_through',
                'latitude',
                'longitude',
                'portal_url',
                'listed_date',
                'description',
                'title',
                'pid',
                'house_number',
                'agent_address',
                'last_sale_date',
                'last_sale_price',
                'price_drop',
                'last_sold',
                'last_sold_price',
            ];

            //Caps to Lower headings
            $head_arr = [];
            foreach ($dataArr[0] as $h_title) {
                $head_arr[] = strtolower($h_title);
            }
            // print_r($head_arr);
            // die;
            //Check Head Columns
            if ($stock_head == $head_arr) {
                foreach ($dataArr as $key => $val) {
                    if ($key >= 1) {
                        if (Outcode::where('outcode', $val[4])->exists()) {
                            $outCode = Outcode::where('outcode', $val[4])->first();
                        } else {
                            $outcode = [
                                'outcode' => !empty($val[4]) ? $val[4] : null,
                            ];
                            $outCode = Outcode::create($outcode);
                        }
                        // print_r( $outCode );die;
                        if (PostCode::where('postcode', $val[5])->where('outcode_id', $outCode->id)->exists()) {
                            $postCode = PostCode::where('postcode', $val[5])->first();
                        } else {
                            $postcode = [
                                'outcode_id' => $outCode->id,
                                'postcode' => !empty($val[5]) ? $val[5] : null,
                            ];

                            $postCode = PostCode::create($postcode);
                        }
                        // if (Agency::where('name', $val[3])->exists()) {
                        //     $agency = Agency::where('name', $val[3])->first();
                        // } else {
                        if (Agency::where('name', $val[3])->exists()) {
                            $Agency = Agency::where('name', $val[3])->first();
                            Agency::where('id', $Agency->id)->update([
                                'address' => !empty($val[21]) ? $val[21] : null,
                            ]);
                            $agency = Agency::where('name', $val[3])->first();
                        } else {
                            $agency = Agency::create([
                                'name' => !empty($val[3]) ? $val[3] : null,
                                'address' => !empty($val[21]) ? $val[21] : null,
                            ]);
                        }

                        if (!empty($val[16])) {
                            $parse1 = Carbon::parse($val[16]);
                            $listedDate = $parse1->format('Y-m-d');
                        } else {
                            $listedDate = null;
                        }
                        if (!empty($val[7])) {
                            $parse2 = Carbon::parse($val[7]);
                            $priceHistDate = $parse2->format('Y-m-d');
                        } else {
                            $priceHistDate = null;
                        }
                        if (!empty($val[22])) {
                            $parse3 = Carbon::parse($val[22]);
                            $lastSaleDate = $parse3->format('Y-m-d');
                        } else {
                            $lastSaleDate = null;
                        }
                        if (!empty($val[25])) {
                            $parse4 = Carbon::parse($val[25]);
                            $lastSoldDate = $parse4->format('Y-m-d');
                        } else {                            
                            $lastSoldDate = null;
                        }
                        if (PropertyHistory::where('pid', $val[19])->exists()) {
                            PropertyHistory::where('pid', $val[19])->update([
                                'address' => $val[0],
                                'portal' => $val[1],
                                'for_sale' => $val[2],
                                'event_date' => Carbon::now()->toDateString(),
                                'price' => !empty($val[9]) ? $val[9] : 0,
                                'property_type' => $val[10],
                                'bed_rooms' => $val[11],
                                'fallen_through' => $val[12],
                                'latitude' => $val[13],
                                'longitude' => $val[14],
                                'portal_url' => !empty($val[15]) ? $val[15] : null,
                                'listed_date' => $listedDate,
                                'description' => !empty($val[17]) ? $val[17] : null,
                                'title' => !empty($val[18]) ? $val[18] : null,
                                // 'pid' => !empty($val[19])?$val[19]:NULL,
                                'agency_id' => $agency->id,
                            ]);
                            $property = PropertyHistory::where('pid', $val[19])->first();
                        } else {
                            $propertyHistory = [
                                'address' => $val[0],
                                'portal' => $val[1],
                                'for_sale' => $val[2],
                                'event_date' => Carbon::now()->toDateString(),
                                'price' => !empty($val[9]) ? $val[9] : 0,
                                'property_type' => $val[10],
                                'bed_rooms' => $val[11],
                                'fallen_through' => $val[12],
                                'latitude' => $val[13],
                                'longitude' => $val[14],
                                'portal_url' => !empty($val[15]) ? $val[15] : null,
                                'listed_date' => $listedDate,
                                'description' => !empty($val[17]) ? $val[17] : null,
                                'title' => !empty($val[18]) ? $val[18] : null,
                                'pid' => !empty($val[19]) ? $val[19] : null,
                                'agency_id' => $agency->id,
                            ];

                            $property = PropertyHistory::create($propertyHistory);
                        }

                        $priceHistory = PropertyPriceHistory::create([
                            'property_history_id' => $property->id,
                            'type' => !empty($val[6]) ? $val[6] : null,
                            'event_date' => $priceHistDate,
                            'price' => !empty($val[8]) ? $val[8] : 0,
                        ]);

                        if (null != $val[20]) {
                            if (HouseNumber::where('house_number', $val[20])->exists()) {
                                $housenumber = HouseNumber::where('house_number', $val[20])->first();
                                $houseNumber = HouseNumber::where('id', $housenumber->id)->update([
                                    'listed_date' => $listedDate,
                                    'latest_price' => !empty($val[9]) ? $val[9] : 0,
                                    'sold_date' => $lastSoldDate,
                                    'postcode_id' => $postCode->id,
                                ]);
                                $houseNumber = HouseNumber::where('house_number', $val[20])->first();
                            } else {
                                $houseNumber = HouseNumber::create([
                                    'house_number' => !empty($val[20]) ? $val[20] : null,
                                    'listed_date' => $listedDate,
                                    'latest_price' => !empty($val[9]) ? $val[9] : 0,
                                    'sold_date' => $lastSoldDate,
                                    'postcode_id' => $postCode->id,
                                ]);
                            }
                            $propertyHouseNumber = PropertyHouseNumber::create([
                                'house_number_id' => $houseNumber->id,
                                'property_history_id' => $property->id,
                            ]);
                        }
                        $saleHistory = PropertySaleHistory::create([
                            // 'house_number_id' => $houseNumber->id,
                            'property_history_id' => $property->id,
                            'price' => !empty($val[26]) ? $val[26] : 0,
                            'sale_date' => $lastSoldDate,
                        ]);
                    }
                }
            } else {
                echo 'Oops! Miss Matching Columns ;)';
            }
            // }
            echo 'Successfully Inserted ;)';
        } else {
            echo 'Oops! File Format not Supported;)';
        }
        // }
        // }
        // }

        return view('data-operator.add-property');
    }

    //post comment
    public function postComment(Request $request)
    {
        $rules = [
            'property' => 'required',
            'comment' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $data = DeoPropertyComment::where('property_history_id', $request->property)->first();
        if ($data) {
            $update = DeoPropertyComment::where('id', $data['id'])
                ->update(['comment' => $request->comment])
                ;
            if ($update) {
                return $this->simpleReturn('success', 'Successfully updated the comment');
            }
        } else {
            $add = new DeoPropertyComment();
            $add->user_id = $user_id;
            $add->property_history_id = $request->property;
            $add->comment = $request->comment;
            if ($add->save()) {
                return $this->simpleReturn('success', 'Successfully added the comment');
            }
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    //get comment
    public function getComment(Request $request)
    {
        $rules = [
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }
        $data = DeoPropertyComment::where('property_history_id', $request->property)->first();
        if ($data) {
            return $this->simpleReturn('success', $data['comment']);
        }

        return $this->simpleReturn('success', '');
    }

    public function deoPropertyWithoutPostcode(Request $request)
    {
        $page_title = 'Without Postcode Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;
        $without_data = true;

        return view('data-operator.property-without-postcode', compact('page_title', 'post_code', 'user_type', 'property', 'without_data'));
    }

    public function deoPropertyWithoutPostcodeArray(Request $request)
    {
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
            ->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->join('agency', 'agency.id', 'property_history.agency_id')
            ->where('postcode.postcode', 'Unknown')
            ->select('property_history.*', 'agency.name')
        ;

        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        // if ($request->has('outcode') && $request->input('outcode') != '') {
        //     $query->where('outcode.outcode', $request->input('outcode'));
        // }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                    ->orWhere('agency.name', 'like', '%'.$search.'%')
                    ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                    ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                    ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                    ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                    ->orWhere('agency.name', 'like', '%'.$search.'%')
                    ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                    ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                    ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                    ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
                ->offset($start)
                ->limit($limit)
                ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $nestedData['postcode'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="postcode" class="form-control add_postcode" placeholder="Postcode">
                <div class="input-group-append">
                    <button data-id="'.$i.'"  type="button"  title="Submit Postcode" class="btn btn-primary addPostcodeSubmit" prop_id="'.$item->id.'" postcode_id="'.$item->postcode_id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
            </div>';

                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                // $nestedData['outcode'] = $item->outcode;
                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('deoPropertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                // $nestedData['outcode'] = $item->outcode;
                // $nestedData['property_number'] = '<div class="input-group">
                //     <input type="text" id="inp_' . $i . '" name="property_number" class="form-control" placeholder="Property Number">
                //     <div class="input-group-append">
                //         <button data-id="' . $i . '"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="' . $item->id . '"><i class="flaticon-paper-plane icon-lg"></i></button>
                //     </div>
                // </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                // $nestedData['type'] = $totalFiltered;
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function deoPropertyWithoutAgent(Request $request)
    {
        $page_title = 'Without Agent Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        // $totalProp = true;

        $without_data = true;

        return view('data-operator.property-without-agent', compact('page_title', 'post_code', 'user_type', 'without_data'));
    }

    public function deoPropertyWithoutAgentArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
            ->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->join('agency', 'agency.id', 'property_history.agency_id')
            ->where('agency.name', 'Unknown')
            ->where('postcode.postcode', '<>', 'Unknown')
            ->select('property_history.*')
        ;

        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'agency',
            1 => 'outode',
            2 => 'address',
            3 => 'price',
            4 => 'property_history.listed_date',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        // if ($request->has('outcode') && $request->input('outcode') != '') {
        //     $query->where('outcode.outcode', $request->input('outcode'));
        // }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                    // ->orWhere('agency.name', 'like', '%' . $search . '%')
                    ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                    ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                    ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                    ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                    // ->orWhere('agency.name', 'like', '%' . $search . '%')
                    ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                    ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                    ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                    ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
                ->offset($start)
                ->limit($limit)
                ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;
                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                // $nestedData['agency'] = $item->name;
                $nestedData['action'] = '<button data-id="'.$item->id.'"  title="Add Agent" class="btn btn-danger btn-sm add_agent" type="button">Add Agent</button>';

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                    <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                    <div class="input-group-append">
                        <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                    </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                // $nestedData['type'] = $totalFiltered;
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function deoAddPostcode(Request $request)
    {
        $rules = [
            'postcode' => 'required',
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }

        if ($request->postcode_id) {
            $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

            DB::beginTransaction();
            $details = Postcode::where('id', $request->postcode_id)->first();

            $data = Postcode::where('outcode_id', $details->outcode_id)
                ->where('postcode', $request->postcode)
                ->first()
                ;

            if ($data) { //postcode is existed
                PropertyHistory::where('id', $request->property)->update([
                    'postcode_id' => $data->id,
                ]);

                DB::commit();

                return $this->simpleReturn('success', 'Postcode updated in this property...!');
            }

            $add = new Postcode();
            $add->postcode = $request->postcode;
            $add->outcode_id = $details->outcode_id;
            if ($add->save()) {
                PropertyHistory::where('id', $request->property)->update([
                    'postcode_id' => $add->id,
                ]);
                DB::commit();

                return $this->simpleReturn('success', 'Postcode updated in this property...!');
            }
            DB::rollback();

            return $this->simpleReturn('error', 'Something went wrong...!');
        }
        DB::rollback();

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    public function getAgent(Request $request)
    {
        $property = PropertyHistory::where('id', $request->property)->first();

        if (0 == $property->portal) {
            $agents = Agency::orderBy('id', 'desc')
                ->where('name', '<>', 'unknown')
                ->where('rightmove_url', '<>', '')
                ->get()
            ;
        } else {
            $agents = Agency::orderBy('id', 'desc')
                ->where('name', '<>', 'unknown')
                ->where('zoopla_url', '<>', '')
                ->get()
            ;
        }

        return $agents;
    }

    public function addPropertyAgent(Request $request)
    {
        $property = PropertyHistory::where('id', $request->property)->first();

        if (0 == $property->portal) { // rightmove
            if (false !== strpos($request->agent_url, 'zoopla')) {
                return $this->simpleReturn('error', 'Please add agent rightmove url.');
            }
        } else { //zoopla
            if (false !== strpos($request->agent_url, 'rightmove')) {
                return $this->simpleReturn('error', 'Please add agent zoopla url.');
            }
        }

        $rules = [
            'agent_name' => 'required',
            'agent_address' => 'required',
            'agent_postcode' => 'required',
            'agent_url' => 'required',
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }
        if (1 == $property->portal) {
            $count = Agency::where('name', $request->agent_name)
                ->where('address', $request->agent_address)
                ->where('postcode', $request->agent_postcode)
                ->where('house_no', $request->agent_house_number)
                ->where('zoopla_url', $request->agent_url)
                ->count()
                ;
        } else {
            $count = Agency::where('name', $request->agent_name)
                ->where('address', $request->agent_address)
                ->where('postcode', $request->agent_postcode)
                ->where('house_no', $request->agent_house_number)
                ->where('rightmove_url', $request->agent_url)
                ->count()
                ;
        }

        if (0 == $count) {
            $add = new Agency();
            $add->name = $request->agent_name;
            $add->address = $request->agent_address;
            $add->postcode = $request->agent_postcode;
            $add->house_no = $request->agent_house_number;
            if (1 == $property->portal) {
                $add->zoopla_url = $request->agent_url;
            } else {
                $add->rightmove_url = $request->agent_url;
            }
            if ($add->save()) {
                $update = PropertyHistory::where('id', $request->property)
                    ->update(['agency_id' => $add->id])
                    ;

                return $this->simpleReturn('success', 'Successfully added the agent');
            }

            return $this->simpleReturn('error', 'Something went wrong...!');
        }

        return $this->simpleReturn('error', 'This agent already existed, please select the agent.');
    }

    public function propertyAgentExist(Request $request)
    {
        $rules = [
            'agent' => 'required',
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }
        $update = PropertyHistory::where('id', $request->property)
            ->update(['agency_id' => $request->agent])
            ;
        if ($update) {
            return $this->simpleReturn('success', 'Successfully added the agent');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    public function unconfirmProperty()
    {
        $page_title = 'Unconfirm Property';
        $search_keyword = session()->get('deo_unconfirm_search');
        $search_outcode = session()->get('deo_unconfirm_outcode');
        return view('data-operator.unconfirm-property', compact('page_title','search_keyword','search_outcode'));
    }

    public function unconfirmPropertyArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        // $submitedproperty = SubmittedProperty::where('type', 1)->pluck('property_history_id')->toArray();

        // $query=PropertyHouseNumber::with(
        //     'houseNumber',
        //     'houseNumber.postCode',
        //     'houseNumber.postCode.outCode',
        //     'propertyHistory',
        //     'propertyHistory.agency'
        // )
        // ->whereNotIn('property_history_id',$submitedproperty);

        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
            ->join('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
            ->join('house_number', 'property_house_number.house_number_id', 'house_number.id')
            ->where('property_history.unconfirmed', 0)
            ->where(function ($query) {
                $query->where('property_history.archive', '=', '0')
                    ->orWhereNull('property_history.archive')
                ;
            })
            // ->whereNotIn('property_history_id', $submitedproperty)
            ->select('property_history.id', 'outcode.outcode', 'postcode.postcode', 'property_history.address', 'agency.name  AS agency_name', 'property_history.property_type', 'property_history.price', 'house_number.house_number', 'house_number.listed_date', 'house_number.latest_price', 'property_history.title', 'house_number.created_by')
        ;
        // ->groupBy("property_history.id", 'outcode.outcode', 'postcode.postcode', 'property_history.address', 'property_history.property_type', 'property_history.price','house_number.house_number','house_number.listed_date');

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }
        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode.outcode',
            1 => 'address',
            2 => 'price',
            3 => 'agency_name',
            4 => 'house_number.listed_date',
            5 => 'type',
        ];

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
            session()->put('deo_unconfirm_outcode',$request->input('outcode'));
        }else{
            session()->forget('deo_unconfirm_outcode');
        }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = Outcode::
            join('postcode', 'postcode.outcode_id', 'outcode.id')->where('archive', 0)
                ->join('house_number', 'house_number.postcode_id', 'postcode.id')
            // ->orderBy('outcode.updated_at', 'desc')
                ->distinct('outcode.id')
                ->pluck('outcode.outcode')
            ;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            $list = $query->orderBy("property_history.listed_date", "DESC")->offset($start)->limit($limit)->get();
            session()->forget('deo_unconfirm_search');
        } else {
            $search = $request->input('search.value');
            session()->put('deo_unconfirm_search',$search);
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                    ->orWhere('agency.name', 'like', '%'.$search.'%')
                    ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                    ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                    ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                    ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                    ->orWhere('agency.name', 'like', '%'.$search.'%')
                    ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                    ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                    ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                    ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ;
            })->orderBy("property_history.listed_date", "DESC")
                ->offset($start)
                ->limit($limit)
                ->get()
            ;
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                $house_number = $item->house_number;
                $latest_price = $item->latest_price;
                $listed_date = $item->listed_date;
                $address = $item->address;
                $property_type = ucfirst($item->property_type);
                $postcode = $item->postcode;
                $outcode = $item->outcode;

                $encrpt_id = Helper::encryptData($item->id);
                $url = route('DEOunconfirmPropertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->id;
                $nestedData['sl'] = $i;
                if (!empty($item->house_number)) {
                    $address2 = $house_number.','.$address;
                } else {
                    $address2 = $house_number.','.$address.',<br>  '.$postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $house_number.','.$address.',<br>  '.$postcode;
                }
                $nestedData['address'] = "<a href='{$url}'>{$address2}</a>";

                // $nestedData['price'] = "&pound;" . number_format($latest_price, 2);
                if ($item->title) {
                    $nestedData['type'] = $item->title.', <br> '.ucfirst($item->property_type);
                } else {
                    $nestedData['type'] = ucfirst($item->property_type);
                }
                $nestedData['agency'] = $item->agency_name;

                if ($item->listed_date) {
                    // $listed_date=date('Y-m-d',strtotime($item->listed_date));
                    // $today=date('Y-m-d');
                    // $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                // .'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                if (0 == $latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound; '.number_format($latest_price);
                }
                // $map_url = "https://www.google.com/maps?q=$item->latitude,$item->longitude";

                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = "https://www.google.com/maps?q={$item->postcode}";
                }

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a><a target='street_view' title='Access Street View' class='btn btn-danger btn-sm ml-2' href='{$map_url}' style='display:none;'><i class='flaticon2-map icon-lg'></i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a><a target='street_view' title='Access Street View' class='btn btn-danger btn-sm ml-2' href='{$map_url}' style='display:none;'><i class='flaticon2-map icon-lg'></i></a>";
                }

                // $nestedData['agency'] = '';
                $nestedData['type'] = $item->title.' <br> '.ucfirst($property_type);
                $nestedData['outcode'] = $outcode;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => ($outcodes),
        ];

        return json_encode($json_data);
    }

    //confirm property
    public function unConfirm_confirmProperty(Request $request)
    {
        $rules = [
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $add = new SubmittedProperty();
        $add->property_history_id = $request->property;
        $add->user_id = $user_id;
        $add->type = 3;
        $add->submitted_date = date('Y-m-d');
        if ($add->save()) {
            $update = PropertyHistory::where('id', $request->property)
                ->update(['confirmed_by' => 1])
                ;

            return $this->simpleReturn('success', 'Successfully marked this property confirmed');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    //wrong data property
    public function wrongProperty(Request $request)
    {
        $rules = [
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $add = new SubmittedProperty();
        $add->property_history_id = $request->property;
        $add->user_id = $user_id;
        $add->type = 4;
        $add->submitted_date = date('Y-m-d');
        if ($add->save()) {
            $update = PropertyHistory::where('id', $request->property)
                ->update(['confirmed_by' => 2])
            ;

            return $this->simpleReturn('success', 'Successfully marked this property wrong');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    public function unconfirmPropertyDetails(Request $request)
    {
        $page_title = 'Unconfirm Property';
        $property_history = Helper::decryptData($request->property_id);

        // $query = $this->commonPropQuery();

        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
            ->join('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
            ->join('house_number', 'property_house_number.house_number_id', 'house_number.id')
            ->select('property_history.id', 'outcode.outcode', 'postcode.postcode', 'property_history.address', DB::raw("GROUP_CONCAT(DISTINCT agency.name SEPARATOR ', ') AS agency_name"), 'property_history.property_type', 'property_history.price', 'property_history.portal_url', 'property_history.latitude', 'property_history.longitude', 'property_history.portal', 'house_number.house_number', 'property_house_number.house_number_id', 'house_number.latest_price', 'property_history.listed_date')
            ->groupBy('property_history.id', 'outcode.outcode', 'postcode.postcode', 'property_history.address', 'property_history.property_type', 'property_history.price', 'property_history.portal_url', 'property_history.latitude', 'property_history.longitude', 'property_history.portal', 'house_number.house_number', 'property_house_number.house_number_id', 'house_number.latest_price', 'property_history.listed_date')
        ;

        $details = $query->where('property_history.id', $property_history)->first();

        // dd($details);
        return view('data-operator.unconfirm-property-details', compact('details'));
    }

    public function notFoundProperty(Request $request){
        $rules = [
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $add = new SubmittedProperty();
        $add->property_history_id = $request->property;
        $add->user_id = $user_id;
        $add->type = 5;
        $add->submitted_date = date('Y-m-d');
        if ($add->save()) {
            PropertyHouseNumber::where("property_history_id",$request->property)
            ->where("created_by",$user_id)
            ->delete();
            return $this->simpleReturn('success', 'Successfully marked this property not found');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }
}
