<?php

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Http\Requests\LetterRequest;
use App\Model\Letter;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Str;
use Redirect;
use Yajra\DataTables\DataTables;

/**
 * Class HouseController.
 */
class LetterController extends Controller
{
    public $title = 'Letters';

    public $trigger = [
        0 => 'General Letters',
        1 => 'New to Market',
        2 => 'For sale',
        3 => 'Sold',
        4 => 'Withdrawn',
        5 => 'Price drop',
        6 => 'Fallen Through',
        7 => '8 Week',
        8 => '16 Week',
        9 => '22 Week',
    ];
    protected $viewPath = 'letters';
    protected $link = 'letters';

    /**
     * Display listing page.
     */
    public function index()
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page_title = 'Letter Management';
        $letter_merge = 1;
        $prob_class = "add-un";
        $my_class = "active";

        return view($this->viewPath.'/list', compact('page','letter_merge','prob_class','my_class'));
    }

    /**
     * Display a listing of the resource in datatable.
     *
     * @throws \Exception
     */
    public function lists(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $detail = Letter::where('user_id', $user_id)->where('letter_type',1)->select(['name', 'triggers', 'id']);
        if (isset($request->form)) {
            foreach ($request->form as $search) {
                if (null != $search['value'] && 'search_region' == $search['name']) {
                    $name = strtolower($search['value']);
                    $detail->where('name', $name);
                }
            }
        }
        
        return DataTables::of($detail)
            ->editColumn('triggers', function ($detail) {
                return $this->trigger[$detail->triggers];
            })
            ->addColumn('action', function ($detail) {
                $edit_url = url($this->link.'/'.$detail->id.'/edit');
                $action = '';
                $action .= Helper::editButton($edit_url);
                $action .= Helper::deleteButton($detail->id);

                return $action;
            })
            ->removeColumn(['id'])
            ->escapeColumns([])
            ->make(false)
        ;

       
    }

    /**
     * Show the form for create the specified resource.
     */
    public function create()
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->form_url = url($this->link);
        $page->form_method = 'POST';
        $triggers = $this->trigger;
        $letter = [];
        $page_title = 'Letter Management';

        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $exists = Letter::where('user_id', $user_id)->distinct('triggers')->where('letter_type',2)->pluck('triggers')->toArray();

        return view($this->viewPath.'/manage', compact('page', 'letter', 'triggers', 'page_title', 'exists'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     *
     * @return array
     */
    public function store(LetterRequest $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $letter = new Letter();
        $letter->user_id = $user_id;
        $this->saveDB($letter, $request);
        $url = url($this->link);
        $error = false;
        $message = Str::singular(Str::title($this->title)).' added successfully';
        // return compact('message', 'url', 'error');
        return redirect('letters')->with('msg_success', 'Letter added successfully.');
    }

    /**
     * @param $table
     * @param $request
     */
    public function saveDB($table, $request)
    {

        $table->name = $request->name;
        $table->triggers = $request->triggers;
        $table->content = $request->content;
        $table->save();
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param mixed $id
     */
    public function edit($id)
    {
        $authUser = json_decode(Session::get(config('propai.authSessionKey')));
        $user_id = $authUser->id;

        $letter = Letter::where('user_id', $user_id)->find($id);

        if ($letter) {
            $page = collect();
            $page->title = $this->title;
            $page->link = url($this->link);
            $page->form_url = url($this->link.'/'.$letter->id);
            $page->form_method = 'PUT';
            $triggers = $this->trigger;

            $exists = Letter::where('user_id', $user_id)->distinct('triggers')->pluck('triggers')->toArray();

            if (false !== ($key = array_search($letter->triggers, $exists))) {
                unset($exists[$key]);
            }

            return view($this->viewPath.'/manage', compact('page', 'letter', 'triggers', 'exists'));
        }

        return redirect('/letters')->with('msg_error', 'Cannot find any letter with this ID.');
    }

    /**
     * Update the specified resource in storage.
     *
     * @param $id
     *
     * @return array
     */
    public function update(LetterRequest $request, $id)
    {
      //dd($request);

        $letter = Letter::find($id);
        // if(isset($request->content) && $request->content != '')
        // {
        //    $request->content = str_replace('&nbsp;', ' ', $request->content);
        // }

        $this->saveDB($letter, $request);

        $url = url($this->link);
        $error = false;
        $message = Str::singular(Str::title($this->title)).' updated successfully';
        // return compact('error', 'message', 'url');
        return redirect('letters')->with('msg_success', 'Letter updated successfully.');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param $id
     */
    public function destroy($id): array
    {
        Letter::destroy($id);
        $error = false;
        $message = Str::singular(Str::title($this->title)).' deleted successfully';

        return compact('error', 'message');
    }


    public function test()
    {
        return view('letters.test');
    }
}
