<?php
namespace App\Http\Controllers;

use App\Model\Payment;
use App\Model\User;
use App\Model\Wallet;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Stripe\Stripe;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Session;
use Yajra\DataTables\DataTables;
use Carbon\Carbon;
use Illuminate\Support\Facades\URL;
use PDF;

class PaymentController extends Controller
{
    public function createIntent(Request $request)
    {
        $amount                 = round($request->amount?:0, 2);
        $user_id                = json_decode(Session::get(config('propai.authSessionKey')))->id; //Auth::user()->id;
        $platform               = $request->platform;
        if ($amount <= 0.30) {
            return response()->json(['success' => false, 'message' => 'Amount must be grater than 0.']);
        }
        if (env('APP_ENV') == 'live') {
            Stripe::setApiKey(config('services.stripe.secret'));
        } else {
            Stripe::setApiKey(config('services.stripe.test_secret'));
        }
        $pay_from_save_card     = 0;
        $save_card_for_future   = 0;
        if ($request->has('save_card')) {
            $save_card_for_future   = 1;
        } else if($request->has('pay_save_card')) {
            $pay_from_save_card     = 1;
        }
        $stripe_check           = 1;
        $payment_method         = '';
        $metadata               = [
                                        'user_id'   => $user_id,
                                        'amount'    => $amount,
                                        'platform'  => $platform
                                    ];
        $payment_data           = [
                                        'amount'        => $amount * 100,
                                        'currency'      => 'gbp',
                                        'metadata'      => $metadata,
                                        'description'   => 'LetsBid Wallet Payment'
                                    ];
        if ($save_card_for_future == 1) {
            $payment_data['setup_future_usage'] = 'off_session';
        } else if($stripe_check && $pay_from_save_card == 1) {
            $payment_data['customer']       = $stripe_payment->stripe_id;
            $payment_data['payment_method'] = $stripe_payment->payment_method;
            $payment_method                 = $stripe_payment->payment_method;
        }
        $intent                 = \Stripe\PaymentIntent::create($payment_data);         
        if ($intent) {
            $intent_key = $intent->client_secret;
            session(['payment_intent' => $intent->id]);
            return response()->json(['success' => true, 'intent_key' => $intent_key, 'payment_method' => $payment_method]);
        } else {
            return response()->json(['success' => false]);
        }
    }
    
    public function stripeWebhook(Request $request)
    {
        if (env('APP_ENV') == 'live') {
            Stripe::setApiKey(config('services.stripe.secret'));
        } else {
            Stripe::setApiKey(config('services.stripe.test_secret'));
        }
        $input              = @file_get_contents('php://input');
        Log::info($input);
        $event_json         = json_decode($input);
        $sig_header         = $_SERVER['HTTP_STRIPE_SIGNATURE'];
        $success            = true;
        if (env('APP_ENV') == 'live') {
            $endpoint_secret    = config('services.stripe.webhook');
        } else {
            $endpoint_secret    = config('services.stripe.test_webhook');
        }
        try {
            $event_json = \Stripe\Webhook::constructEvent($input, $sig_header, $endpoint_secret);
        } catch(\UnexpectedValueException $e) {
            // Invalid payload
            http_response_code(400);

            exit();
        } catch(\Stripe\Exception\SignatureVerificationException $e) {
            // Invalid signature
            http_response_code(400);

            exit();
        }
        $transaction_id     = $event_json->data->object->id;
        if ($event_json->type == 'payment_intent.succeeded') {
            $payment_status = 1;
        }
        $transaction_id     = $event_json->data->object->id;
        $amount             = $event_json->data->object->amount / 100;
        $user_id            = $event_json->data->object->metadata->user_id;
        //update wallet with $user_id and $amount. save $transaction_id in a table for future reference
        if ($payment_status == 1) {
            // insert data to DB
            $this->recordPaymentDetails([
                                            'transaction_id'    => $transaction_id,
                                            'user_id'           => $user_id,
                                            'action'            => 'credit',
                                            'action_amount'     => $amount,
                                        ]);
            return response()->json(['transaction_id' => $transaction_id, 'error' => false]);
        } else {
            $success = false;

            return response()->json(['error' => true]);
        }
    }
    public function fetchPaymentData()
    {
         $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
         $payment_table_array = Payment::where('agent_id', $user_id)->select(['payment_amount', 'transaction_id', 'created_at', 'payment_type', 'payment_status', 'invoice_id']);
        // dd($payment_table_array);
                return Datatables::of($payment_table_array)
                ->addIndexColumn()
                ->editColumn('payment_amount', function ($model) {
                    if($model->payment_type == 'credit'){
                        return "<span style='color:#1FD9A9'>£" . $model->payment_amount . "</span>";
                    } else {
                        return "<span style='color:#FF8989'>£" . $model->payment_amount . "</span>";
                    }
                 })
                ->editColumn('title', function ($model) {
                   return $model->transaction_id;
                 })
                ->editColumn('count_val', function ($model) {
                    $payment_transaction_id = $model->transaction_id;
                    $pay_trans_parts        = explode('-', $payment_transaction_id);
                    $pay_trans_parts_1[]    = explode('-', $payment_transaction_id);
                    $count_val              = isset($pay_trans_parts[2]) ? (int) $pay_trans_parts[2] : "-";
                    return $count_val;
                 })
                ->editColumn('date', function ($model) {
                   return Carbon::parse($model->created_at)->format('d/M/Y');
                 })
                ->editColumn('payment', function ($model) {
                    if($model->payment_type == 'credit'){
                        return "<i class='fa fa-fw fa-circle dot-cir'></i>" . $model->payment_type;
                    } else {
                        return "<i class='fa fa-fw fa-circle dot-cir dot-cir-1'></i>" . $model->payment_type;
                    }
                 })
                ->editColumn('status', function ($model) {
                    if($model->payment_status == 1){
                        return "<span class='label label-lg font-weight-bold label-light-dark label-inline' style='background-color:#0A8059;color:#fff;'>Success</span>";
                    } else {
                        return "<span class='label label-lg font-weight-bold label-light-dark label-inline' style='background-color:#B5A305;color:#fff;'>Pending</span>";
                    }
                 })
                ->editColumn('invoice', function ($model) {
                     if($model->payment_type == 'credit'){
                        return "<a href='" . URL::current() . "/fetch-invoice/" . $model->transaction_id . "' target='_blank'><span class='label label-lg font-weight-bold label-light-dark label-inline' style=' background-color:#1FD9A9; color:#fff;'> Get Invoice </span></a>";
                     } else {
                        return '';
                     }
                 })
                ->rawColumns(['slno','payment_amount','title','count_val','date','payment','status','invoice'])
                ->make(true);
    }
    public function fetchPaymentDetails($data = null)
    {
        $user_id = isset($data['user_id']) ? $data['user_id'] : '';
        if ($user_id === '') { // redirect to login page is user_id blank
            return redirect()->route('userLogin');
        } else {
            // initialise variables
            $current_user_total             = $current_user_used_total = $i = 0;
            $payment_fetch_data             = $fetch_payment_table_array = $count_val = [];
            $payment_table_array            = $this->fetchAgentPaymentDetails($user_id);
            foreach ($payment_table_array as $payment_amounts_row) {
                // $payment_status = ($payment_table_array[$i]['payment_status'] == 1) ? "Success" : "Pending";
                // $payment_transaction_id[$i] = $payment_table_array[$i]['transaction_id'];
                // $pay_trans_parts            = explode('-', $payment_transaction_id[$i]);
                // $pay_trans_parts_1[]        = explode('-', $payment_transaction_id[$i]);
                if ($payment_table_array[$i]['payment_type'] === "credit") {
                    $current_user_total         = $current_user_total + $payment_table_array[$i]['payment_amount'];
                    $count_val[$i]              = "-";
                } else if ($payment_table_array[$i]['payment_type'] === "debit") {
                    $current_user_total         = $current_user_total - $payment_table_array[$i]['payment_amount'];
                    $current_user_used_total    = $current_user_used_total + $payment_table_array[$i]['payment_amount'];
                    $count_val[$i]              = isset($pay_trans_parts[2]) ? (int) $pay_trans_parts[2] : "-";
                }
            //     $fetch_payment_table_array[$i] = [
            //                                         "slno"              => $i+1,
            //                                         "payment_amount"    => "£" . $payment_table_array[$i]['payment_amount'],
            //                                         "title"             => $payment_table_array[$i]['title'],
            //                                         "transaction_id"    => $payment_table_array[$i]['transaction_id'],
            //                                         "count_val"         => $count_val[$i],
            //                                         "created_at"        => Carbon::parse($payment_table_array[$i]['created_at'])->format('d/M/Y'),
            //                                         "payment_type"      => $payment_table_array[$i]['payment_type'],
            //                                         "payment_status"    => $payment_status,
            //                                         "invoice_id"        => $payment_table_array[$i]['invoice_id'],
            //                                     ];
                $i++;
            }
            $payment_fetch_data = [
                                        "current_user_total"        => "£" . $current_user_total,
                                        "current_user_used_total"   => "£" . $current_user_used_total,
                                        "payment_data_array"        => $fetch_payment_table_array,
                                    ];

            return $payment_fetch_data;
        }
    }

    public function recordPaymentDetails($data)
    {
        $user_id        = isset($data['user_id']) ? $data['user_id'] : '';
        $action         = isset($data['action']) ? $data['action'] : '';
        $action_amount  = isset($data['action_amount']) ? $data['action_amount'] : '';
        $campaign_id    = isset($data['campaign_id']) ? $data['campaign_id'] : '';
        $transaction_id = isset($data['transaction_id']) ? $data['transaction_id'] : '';
        if ($transaction_id != '' && $user_id != '' && $action != '' && $action_amount != '') {
            $current_user_total             = $current_user_used_total = $i = 0;
            $payment_table_array            = $this->fetchAgentPaymentDetails($user_id);
            if (count($payment_table_array) > 0) {
                foreach ($payment_table_array as $payment_amounts_row) {
                    $payment_status = ($payment_table_array[$i]['payment_status'] === 1) ? "Success" : "Pending";
                    if ($payment_table_array[$i]['payment_type'] === "credit") {
                        $current_user_total         = $current_user_total + $payment_table_array[$i]['payment_amount'];
                    } else if ($payment_table_array[$i]['payment_type'] === "debit") {
                        $current_user_total         = $current_user_total - $payment_table_array[$i]['payment_amount'];
                        $current_user_used_total    = $current_user_used_total + $payment_table_array[$i]['payment_amount'];
                    }
                    $i++;
                }
            }
            if ($action === "debit") {
                $current_user_total         = $current_user_total - $action_amount;
                $current_user_used_total    = $current_user_used_total + $action_amount;
                if ($campaign_id !== '') {
                    $payment_title = "Bulk-Mail Campaign No. $campaign_id";
                } else {
                    $payment_title = 'Bulk-Mail payment';
                }
            } else if ($action === "credit") {
                $current_user_total         = $current_user_total + $action_amount;
                $payment_title              = 'Top-up - ' . $action_amount;
            }
            $payment_insert = Payment::create([
                                                    'transaction_id'    => $transaction_id,
                                                    'title'             => $payment_title,
                                                    'agent_id'          => $user_id,
                                                    'payment_type'      => $action,
                                                    'payment_status'    => 1,
                                                    'payment_amount'    => $action_amount,
                                                    'current_total'     => $current_user_total,
                                                ]);
            // log payment insert
            Log::info($payment_insert);
            $wallet_insert  = Wallet::updateOrCreate(   ['agent_id'                 => $user_id], 
                                                        [
                                                            'current_total'         => $current_user_total,
                                                            'current_used_total'    => $current_user_used_total
                                                        ]);
            // log wallet insert
            Log::info($wallet_insert);

            return ['success' => true, 'message' => 'transaction recorded'];
        } else {
            return ['success' => false, 'message' => 'transaction params empty'];
        }
    }

    public function fetchAgentPaymentDetails($user_id)
    {
        $user_id    = isset($user_id) ? $user_id : '';
        $fetch_data = [];
        if ($user_id != '') {
            $payment_table_fetch    = Payment::where('agent_id', $user_id)
                                        // ->orderBy('updated_at', 'DESC')
                                        ->get()
                                        ->toArray();
            $fetch_data             = $payment_table_fetch;
        }
        return $fetch_data;
    }

    public function fetchAgentWalletBalance($user_id = null)
    {
        if ($user_id === null) {
            $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id; //Auth::user()->id;
        } else {
            $user_id = isset($user_id) ? $user_id : '';
        }
        $fetch_data = [];
        if ($user_id != '') {
            $wallet_table_fetch = Wallet::where('agent_id', $user_id)
                                    // ->orderBy('updated_at', 'DESC')
                                    ->get()
                                    ->toArray();
            if (count($wallet_table_fetch) > 0) {
                $fetch_data = [
                                    'current_total'         => $wallet_table_fetch[0]['current_total'],
                                    'current_used_total'    => $wallet_table_fetch[0]['current_used_total'],
                                ];
            } else {
                $fetch_data = [
                                    'current_total'         => 0,
                                    'current_used_total'    => 0,
                                ];
            }
        }
        return $fetch_data;
    }

    public function fetchPaymentInvoice($transaction_id = '')
    {
        // retreive all records from db
        if ($transaction_id !== '') {
            $data = Payment::where('transaction_id', $transaction_id)->get();
        } else {
            return false;
        }
        if ($data->count() === 1) {
            $data       = $data->toArray();
            $data       = $data[0];
            $user_data  = User::where('id', $data['agent_id'])->get()->toArray();
            $user_data  = $user_data[0];
            $file_date  = Carbon::now()->format('d_m_Y_g-i-s');
            $og_value   = $data['payment_amount'] / (1 + (config('stannp_api.vat_percentage')/100));
            $og_vat_val = $og_value * (config('stannp_api.vat_percentage')/100);
            $template   = [
                                "logo"              => "logo-letter-1.png",
                                "company_name"      => config('propai.company_name'),
                                "address"           => str_replace(',', '', config('propai.address')),
                                "phone"             => config('propai.phone'),
                                "email"             => config('propai.email'),
                                "website_address"   => config('propai.website_address'),
                                "foot_desc"         => "",
                                "head_desc"         => "",
                            ];
            $invoice    = [
                                "invoice_number"    => substr($data['transaction_id'], 0, 10),
                                "invoice_date"      => Carbon::now()->format('F j, Y g:i a'),
                                "title"             => $data['title'],
                                "user_name"         => $user_data['first_name'] . " " . $user_data['last_name'],
                                "user_address"      => $user_data['address'],
                                "user_contact_no"   => $user_data['contact_number'],
                                "description"       => 'Wallet top-up done on Lets Bid Property Ltd website for amount of £'.$data['payment_amount'],
                                "cost"              => $og_value,
                                "vat_cost"          => $og_vat_val,
                                "total_cost"        => $data['payment_amount'],
                                // "unit_price"        => $data['unit_price'],
                                "created_at"        => Carbon::parse($data['created_at'])->format('F j, Y g:i a'),
                                "discount"          => "",
                                "is_vat"            => 1,
                                "is_ship"           => 0,
                                "ship_amount"       => 0,
                                "status"            => 0,
                            ];
            $info       = [
                                "company_reg_no"    => config('propai.company_reg_no'),
                                "vat_reg_no"        => config('propai.vat_reg_no'),
                                "vat_percentage"    => config('stannp_api.vat_percentage'),
                            ];
            $pdf        = PDF::loadView('invoice/invoice_pdf', compact('data', 'template', 'invoice', 'info'));

            return $pdf->download("lets_bid_invoice_$file_date.pdf");
        } else {
            return false;
        }
    }

    public function confirmPaymentRecord($payment_id)
    {
         $payment_table_fetch = Payment::where('transaction_id', $payment_id)
                                    ->where('payment_status', 1)
                                    ->get()
                                    ->toArray();
        if (count($payment_table_fetch) > 0) {
            return true;
        } else {
            return false;
        }
    }
}