<?php

// <summary>
// File : CampaignController
// File Type : Controller
// Creator : Susmi Sidharthan, Sandeep Kumar
// Email: susmi@designdirect@gmail.com
// Company: Design Direct UK
// Description ; 1.This is used as web controller for all estate agent campaign controller.
// 2. function used to print letter.
// </summary>

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Mail\ContactCampaign;
use App\Model\Campaign;
use App\Model\ProbateCampaignLetterHistory;
use App\Model\Letter;
use App\Model\LetterHead;
use App\Model\ProbateProperty;
use App\Model\ProbatePropertyCampaign;
use App\Model\Property;
use App\Model\PropertyCampaign;
use App\Model\PropertyFlag;
use App\Model\PropertyHistory;
use App\Model\ProbatePropertyFlag;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Session;
use Redirect;
use Validator;
use Yajra\DataTables\DataTables;
use Crypt;

class ProbateCampaignController extends Controller
{
    public $status = [
        1 => 'New to Market',
        2 => 'For sale',
        3 => 'Sold',
        4 => 'Withdrawn',
        5 => 'Price drop',
        6 => 'Fallen Through',
    ];
    public $trigger = [
        0 => 'Probate 1',
        1 => 'Probate 2',
        2 => 'Probate 3',
        // 3 => 'Sold',
        // 4 => 'Withdrawn',
        // 5 => 'Price drop',
        // 6 => 'Fallen Through',
        // 7 => '8 Week',
        // 8 => '16 Week',
        // 9 => '22 Week',
    ];
    protected $link = 'probate-campaign-history';
    protected $B;
    protected $I;
    protected $U;
    protected $HREF;
    protected $fontList;
    protected $issetfont;
    protected $issetcolor;

    public function __construct($orientation = 'P', $unit = 'mm', $format = 'A4')
    {
        //Call parent constructor
        // parent::__construct($orientation,$unit,$format);
        //Initialization
        $this->B = 0;
        $this->I = 0;
        $this->U = 0;
        $this->HREF = '';
        $this->fontlist = ['arial', 'times', 'courier', 'helvetica', 'symbol'];
        $this->issetfont = false;
        $this->issetcolor = false;

        $pdf = new \setasign\Fpdi\Fpdi('Portrait', 'mm', [210, 297], true, 'UTF-8', true);

        $pdf->SetFont('Helvetica');
    }

    public function index()
    {
        $page = collect();
        $page->title = 'Campaign History';
        $page->link = url('probate-campaign-history');
        $page_title = 'Probate Campaign History';

        $cam_history_merge = 1;
        $my_class = 'add-un';
        $prob_class = 'active';
        // dd('d');
        return view('probate-campaign/histories', compact('page', 'cam_history_merge', 'my_class', 'prob_class'));
    }

    public function lists(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $detail = Campaign::where('user_id', $user_id)->where('type',2)->select(['name', 'sent_date', 'campaign_status', 'id'])->orderBy('campaigns.id', 'desc');
        return Datatables::of($detail)
                ->editColumn('name', function ($detail) {
                        return $detail->name;
                     })
                ->editColumn('sent_date', function ($detail) {
                        return '' != $detail->sent_date ? Carbon::parse($detail->sent_date)->format('d/m/Y') : '';
                     })  
                ->editColumn('campaign_status', function ($detail) {
                        return 1 == $detail->campaign_status ? 'Send' : 'Sending';
                     })  
                ->editColumn('action', function ($detail) {
                        $edit_url = url($this->link.'/'.$detail->id.'/edit');
                        $view_url = url($this->link.'/'.$detail->id.'/');
                        $action = '';
                        // $action .= Helper::editButton($edit_url);
                        $action .= '<button class="btn btn-primary btn-sm mr-2 editCampaign" data-id="'.$detail->id.'" data-name="'.$detail->name.'">Edit</button>';
                        $action .= Helper::deleteButton($detail->id);
                        $action .= Helper::viewButton($view_url);

                        return $action;
                    })
                ->rawColumns(['name','sent_date','campaign_status','action'])
                ->make(true);

        // $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        // $detail = Campaign::select(['name', 'sent_date', 'campaign_status', 'id'])
        //     ->where('type', 2)
        //     ->where('user_id', $user_id)
        //     ->orderBy('campaigns.id', 'desc')
        // ;

        // return DataTables::of($detail)
        //     ->editColumn('sent_date', function ($detail) {
        //         return '' != $detail->sent_date ? Carbon::parse($detail->sent_date)->format('d/m/Y') : '';
        //     })
        //     ->editColumn('campaign_status', function ($detail) {
        //         return 1 == $detail->campaign_status ? 'Send' : 'Sending';
        //     })
        //     ->addColumn('action', function ($detail) {
        //         $action = '';
        //         // $print_url = url('/campaign-history/'.$detail->id.'/print');
        //         //  $print_url = url('campaigns/print/'.$detail->id);
        //         // $action = '<a class="btn btn-primary btn-sm mr-2" target="_blank" href="'.$print_url.'">Print</a>';
        //         // $action .= Helper::deleteButton($detail->id);
        //         $action .= '<button class="btn btn-success btn-sm mr-2 editCampaign" data-id="'.$detail->id.'" data-name="'.$detail->name.'">Edit</button>';
        //         // $action .= '<a class="btn btn-info btn-sm mr-2" href=" '.route('probateCampaignHistoryView', $detail->id).'">View</a>';
        //         $edit_url = url($this->link.'/'.$detail->id.'/edit');
               
        //         // $action .= Helper::editButton($edit_url);
        //         $action .= Helper::deleteButton($detail->id);

        //         return $action;
        //     })
        //     ->removeColumn(['id'])
        //     ->escapeColumns([])
        //     ->make(false)
        // ;
    }

    public function destroy($id): array
    {
        $authUser = json_decode(Session::get(config('propai.authSessionKey')));
        $user_id = $authUser->id;
        $campaign = Campaign::where('user_id', $user_id)->find($id);
        if ($campaign) {
            Campaign::destroy($id);
            $error = false;
            $message = 'ProbatePropertyCampaign deleted successfully';

            return compact('error', 'message');
        }  
        $error = true;
        $message = 'Something went wrong!!';

        return compact('error', 'message');       
    }

    //property add to campaign
    public function addToCampaignProperty(Request $request)
    {
        $rules = [
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }

        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        foreach ($request->property as $property_id) {
            $exist_data = PropertyCampaign::where('user_id', $user_id)->where('house_number_id', $property_id)->where('campaign_id', null)->first();

            if (!$exist_data) {
                $add = new PropertyCampaign();
                $add->house_number_id = $property_id;
                $add->user_id = $user_id;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully added to campaign...!');
    }

    //property add to campaign
    public function createCampaign(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $campaign = new ProbatePropertyCampaign();
        $campaign->name = 'ProbatePropertyCampaign-'.Carbon::now()->format('d-m-Y h:i');
        $campaign->campaign_status = 0;
        $campaign->user_id = $user_id;
        $campaign->save();
        foreach ($request->property_id as $property_id) {
            $trigger = 'trigger_'.$property_id;
            PropertyCampaign::where('id', $property_id)
                ->update([
                    'campaign_id' => $campaign->id,
                    'trigger' => $request->{$trigger},
                ])
            ;
        }

        return redirect('campaign-history');
    }

    //property add to campaign
    public function printCampaign(Request $request)
    {
        // return $request->all();
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if (1 == $request->print_type || 3 == $request->print_type) {
            $source_data = LetterHead::where('user_id', $user_id)->first();
            if (!$source_data) {
                return -1;
            }
        }
        $campaign = new Campaign();
             $campaign->name = 'ProbatePropertyCampaign-'.Carbon::now()->format('d-m-Y h:i');
        $campaign->campaign_status = 0;
        $campaign->user_id = $user_id;
         $campaign->type = 2;
        $campaign->save();

        $cam_id = $campaign->id;

        foreach ($request->data as $data) {
            $ff = ProbatePropertyCampaign::where('id', $data[0])
                ->where('user_id', $user_id)
                ->update([
                    'campaign_id' => $cam_id,
                    'trigger' => $data[1],
                ])
            ;
            // return $ff;
            $propertyCampaign = ProbatePropertyCampaign::where('id', $data[0])->first();
            if ($propertyCampaign) {
                // $flag = ProbatePropertyFlag::updateOrCreate(
                //     ['probate_id' => $propertyCampaign['property_id'], 'user_id' => $user_id],
                //     ['probate_id' => $propertyCampaign['property_id'], 'user_id' => $user_id, 'flag' => 3]
                // );
                 $exist_data = ProbatePropertyFlag::where('user_id', $user_id)->where('probate_id', $propCampaign->property_id)->first();
                if ($exist_data) {
                    $update = ProbatePropertyFlag::find($exist_data['id']);
                    $update->fill(['flag' => 3])->save();
                }else{
                     $add = new ProbatePropertyFlag();
                     $add->probate_id = $propCampaign->property_id;
                     $add->user_id = $user_id;
                     $add->flag = 3;
                     $add->save();
                }

                $letterHistory = new ProbateCampaignLetterHistory();
                $letterHistory->property_id = $propertyCampaign->property_id;
                $letterHistory->user_id = $user_id;
                $letterHistory->letter_type = $data[1];
                $letterHistory->save();
            }
        }

        $url = url('probate-campaign/print/'.$campaign->id.'/'.$request->print_type);
        $error = false;
        $message = 'campaign submitted successfully';

        return compact('message', 'url', 'error');
    }

    public function printLetter($campaign_id, $print_type)
    {
        // dd($campaign_id, $print_type);
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $campaign = Campaign::find($campaign_id);
        $campaign->campaign_status = 1;
        $campaign->sent_date = Carbon::now();
        $campaign->type = 2;
        $campaign->save();

        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $properties = ProbatePropertyCampaign::where('campaign_id', $campaign_id)->where('user_id', $user_id)
            ->get()
        ;
  
        $source_data = LetterHead::where('user_id', $user->id)->first();

        $pdf = new \setasign\Fpdi\Fpdi('Portrait', 'mm', [210, 297], true, 'UTF-8', true);
        if (count($properties) > 0) {
            foreach ($properties as $property) {
                if (2 == $print_type) {
                    $source = public_path('letter-heads/blank_letter.pdf');
                } else {
                    $source = storage_path('app/public/letter-heads/'.$source_data['file']);
           

                  //sandeep edits - 26 Apr 2021

                    $filepdf = fopen($source,"r");
   if($filepdf) {
     $line_first = fgets($filepdf);
     fclose($filepdf);
   }
   else{
     echo "error opening the file.";
   }
    preg_match_all('!\d+!', $line_first, $matches);
     $pdfversion = implode('.', $matches[0]);  

  if($pdfversion > "1.4"){         
  }
  else{
    $pdfVersion = "1.7";   //The path that you want to save the new file to
$newFile = storage_path('app/public/letter-heads/1-'.$source_data['file']); //The path of the file that you want to convert
$currentFile = $source;
$imgPath = storage_path('app/public/letter-heads/');

////////////////////// LOCAL WORKING /////////////////////////
if(env('APP_ENV')=='local')
{
$gslocal = public_path('mygs/bin/gswin64c.exe');
$gsPath = '"'.$gslocal.'"';
$gsCmd = $gsPath.' -dSAFER -dBATCH -dNOPAUSE -dCompatibilityLevel='.$pdfVersion.' -sDEVICE=pdfwrite -sOutputFile='.$newFile .' '.$currentFile;     
}

/////////////////////// DEMO TESTING ////////////////////////
  if (env('APP_ENV') == 'demo' || env('APP_ENV') == 'live' ) 
{
  $gsCmd = 'gs -dSAFER -dBATCH -dNOPAUSE -dCompatibilityLevel='.$pdfVersion.' -sDEVICE=pdfwrite -sOutputFile='.$newFile .' '.$currentFile;
}
//Run it using PHP's exec function.
$result = exec($gsCmd);
$source = $newFile;
  }
     }


                $pagecount = $pdf->setSourceFile($source);

                // Reference the PDF you want to use (use relative path)

                // Import the first page from the PDF and add to dynamic PDF
                $tpl = $pdf->importPage(1);

                $pdf->AddPage();

                // Use the imported page as the template
                // $pdf->useTemplate($tpl);
                  $pdf ->useTemplate($tpl, null, null, 216, 280, FALSE);

                // Set the default font to use
                $pdf->SetFont('Helvetica');

                ini_set('memory_limit', '-1');
                $property->address = ProbateProperty::join('executor as exe', 'exe.id', 'probate_property.executor_id')
                    ->join('postcode as pc', 'pc.id', 'probate_property.postcode_id')
                    ->join('outcode as oc', 'oc.id', 'probate_property.outcode_id')
                    ->where('probate_property.id', $property->property_id)
                    ->select('probate_property.*', 'pc.postcode', 'oc.outcode', 'exe.address_1 as ex_address1', 'exe.address_2 as ex_address2', 'exe.first_name as ex_fname', 'exe.sur_name as ex_sname', 'exe.company_name as ex_company_name')
                    ->first()
                ;
                $property->letter = Letter::where('triggers', $property->trigger)->where('user_id', $user_id)->where('letter_type', 2)->first();
               ProbatePropertyCampaign::find($property->id)->delete();
                if (empty($property->address->ex_fname) && !empty($property->address->ex_company_name)) {
                    $companyname = explode(',', $property->address->ex_company_name);
                    $name = trim($companyname[0]);
                } else {
                    $name = $property->address->ex_fname.' '.$property->address->ex_sname;
                }

                $addr = $property->address->address_1;               
                $addr1 = $addr;
                $addr = str_replace($property->address->postcode, '', $addr);
                $addr = str_replace($property->address->outcode, '', $addr);
                $addr = rtrim($addr);
                $addr = rtrim($addr, ',');
                $addr = str_replace(', ', "\n", $addr);               

                $pdf->SetFontSize('10');
                // $pdf->SetXY(18.25, 50);
                // $pdf->Write(0, $name);

                $pdf->SetXY(18.25, 52);

                // $pdf->MultiCell(60, 5, ($addr)."\n".$property->address->postcode, 0, 1);
                $pdf->MultiCell(60, 4.5, $property->edit_property_address, 0, 1);

                $addrLines = preg_match_all('/\n/', $property->edit_property_address);

                $withoutFirstLine = preg_replace('/^.+\n/', '', $property->edit_property_address);

                $propertyAddress = "\n\nRef:\n".$withoutFirstLine;
               

                $offset = 80;
                $space = 5;
                if ($addrLines > 4) {
                    $offset += ($addrLines - 4) * $space;
                }

                if ($property->letter) {
                    $pdf->SetXY(18.25, $offset);
                    // $content = $property->letter->content;

                    // $content = str_replace('&nbsp;', '', $content);

                     $content = strip_tags($property->letter->content);

                         $content = $this->normalize($content);

               $content = iconv('UTF-8', 'windows-1252', $content);
               
                 // $content =  mb_convert_encoding($content,"HTML-ENTITIES","UTF-8");

                 // $content = str_replace('&nbsp;', ' ', $content);

                 // $content = str_replace('&rsquo;', "'", $content);

                 // $content = str_replace('&amp;', "&", $content);

                    // $content = preg_replace('/[[:^print:]]/', '', $content);

                    $pdf->SetMargins(18.25, 0, 18.25);

                    // $pdf->MultiCell(0, 6, $this->WriteHTML($content, $pdf, $campaign->name), 0, 1);
                    $pdf->MultiCell(0, 4.5, $content, 0, 1);
                }
                $tpl = $pdf->importPage(1);
                $pdf->AddPage();

                // Use the imported page as the template
                // $pdf->useTemplate($tpl);
                  $pdf ->useTemplate($tpl, null, null, 216, 280, FALSE);

                // Set the default font to use
                $pdf->SetFont('Helvetica');

                ini_set('memory_limit', '-1');

                if ($property->address->ex_address1) {
                    $addr = $property->address->ex_address1;
                } else {
                    $addr = trim($property->address->ex_company_name);
                    $name = '';
                } 

                $exAddress = $addr;
               
                $addr = $property->edit_exe_property_address . $propertyAddress;
          
                $pdf->SetFontSize('10');
                // $pdf->SetXY(18.25, 50);
                // $pdf->Write(0, $name);

                $pdf->SetXY(18.25, 52);

                // $pdf->MultiCell(60, 5, (Helper::parseProbateAddress($addr)), 0, 1);
                $pdf->MultiCell(60, 4.5, $addr, 0, 1);

                $addrLinesExe = preg_match_all('/\n/', $addr);

                $offset = 80;
                $space = 5;
                if ($addrLinesExe > 4) {

                    $offset += ($addrLinesExe - 4) * $space;
                }

                if ($property->letter) {
                    $pdf->SetXY(18.25, $offset);
                    // $content = $property->letter->content;

                    // $content = str_replace('&nbsp;', '', $content);

                 $content = strip_tags($property->letter->content);

                   $content = $this->normalize($content);

               $content = iconv('UTF-8', 'windows-1252', $content);
               
                 // $content =  mb_convert_encoding($content,"HTML-ENTITIES","UTF-8");

                 // $content = str_replace('&nbsp;', ' ', $content);

                 // $content = str_replace('&rsquo;', "'", $content);

                 // $content = str_replace('&amp;', "&", $content);

                    // $content = preg_replace('/[[:^print:]]/', '', $content);
                    $pdf->SetMargins(18.25, 0, 18.25);

                    // $pdf->MultiCell(0, 6, $this->WriteHTML($content, $pdf, $campaign->name), 0, 1);
                    $pdf->MultiCell(0, 4.5, $content, 0, 1);
                }
            }
            ob_get_clean();
            $pdf->Output($campaign->name.'.pdf', 'I');
        }

        // if (2 == $print_type) {
        //     if (count($properties) > 0) {
        //         foreach ($properties as $property) {
        //             ini_set('memory_limit', '-1');
        //             $property->address = ProbateProperty::join('executor as exe', 'exe.id', 'probate_property.executor_id')
        //                 ->join('postcode as pc', 'pc.id', 'probate_property.postcode_id')
        //                 ->join('outcode as oc', 'oc.id', 'probate_property.outcode_id')
        //                 ->where('probate_property.id', $property->property_id)
        //                 ->select('probate_property.*', 'pc.postcode', 'oc.outcode', 'exe.address_1 as ex_address1', 'exe.address_2 as ex_address2', 'exe.first_name as ex_fname', 'exe.sur_name as ex_sname', 'exe.company_name as ex_company_name')
        //                 ->first()
        //             ;
        //             $property->letter = Letter::where('triggers', $property->trigger)->where('user_id', $user_id)->where('letter_type', 2)->first();
        //             ProbatePropertyCampaign::find($property->id)->delete();
        //         }
        //         // return view('probate-campaign.print', compact('properties'));
        //         $pdf = App::make('dompdf.wrapper');
        //         $pdf->loadView('probate-campaign.print', compact('properties'));
        //         return $pdf->stream($campaign->name.'.pdf');
        //     }
        // } elseif (3 == $print_type) {
        //     $source_data = LetterHead::where('user_id', $user->id)->first();
        //     $pdf = new \setasign\Fpdi\Fpdi('Portrait', 'mm', [210, 297], true, 'UTF-8', true);
        //     if (count($properties) > 0) {
        //         foreach ($properties as $property) {
        //             $source = storage_path('app/public/letter-heads/'.$source_data['file']);
        //             $pagecount = $pdf->setSourceFile($source);
        //             $tpl = $pdf->importPage(1);
        //             $pdf->AddPage();
        //             $pdf->useTemplate($tpl);
        //             $pdf->SetFont('Helvetica');
        //             ini_set('memory_limit', '-1');
        //             $property->address = ProbateProperty::join('executor as exe', 'exe.id', 'probate_property.executor_id')
        //                 ->join('postcode as pc', 'pc.id', 'probate_property.postcode_id')
        //                 ->join('outcode as oc', 'oc.id', 'probate_property.outcode_id')
        //                 ->where('probate_property.id', $property->property_id)
        //                 ->select('probate_property.*', 'pc.postcode', 'oc.outcode', 'exe.address_1 as ex_address1', 'exe.address_2 as ex_address2', 'exe.first_name as ex_fname', 'exe.sur_name as ex_sname', 'exe.company_name as ex_company_name')
        //                 ->first()                    ;
        //             $property->letter = Letter::where('triggers', $property->trigger)->where('user_id', $user_id)->where('letter_type', 2)->first();
        //             ProbatePropertyCampaign::find($property->id)->delete();
        //             if (empty($property->address->ex_fname) && !empty($property->address->ex_company_name)) {
        //                 $companyname = explode(',', $property->address->ex_company_name);
        //                 $name = trim($companyname[0]);
        //             } else {
        //                 $name = $property->address->ex_fname.' '.$property->address->ex_sname;
        //             }
        //             $addr = $property->address->address_1;
        //             $addr = str_replace($property->address->postcode, '', $addr);
        //             $addr = str_replace($property->address->outcode, '', $addr);
        //             $addr = rtrim($addr);
        //             $addr = rtrim($addr, ',');
        //             $addr = str_replace(', ', "\n", $addr);
        //             $pdf->SetFontSize('10');
        //             $pdf->SetXY(18.25, 50);
        //             $pdf->Write(0, $name);
        //             $pdf->SetXY(18.25, 52);
        //             $pdf->MultiCell(60, 5, ($addr)."\n".$property->address->postcode, 0, 1);
        //             if ($property->letter) {
        //                 $pdf->SetXY(18.25, 98);
        //                 $content = $property->letter->content;
        //                 $content = str_replace('&nbsp;', '', $content);
        //                 // $content = preg_replace('/[[:^print:]]/', '', $content);
        //                 $pdf->SetMargins(18.25, 0, 18.25);
        //                 // $pdf->MultiCell(0, 6, $this->WriteHTML($content, $pdf, $campaign->name), 0, 1);
        //                 $pdf->MultiCell(0, 6,$content, 0, 1);
        //             }
        //             $tpl = $pdf->importPage(1);
        //             $pdf->AddPage();
        //             $pdf->useTemplate($tpl);
        //             $pdf->SetFont('Helvetica');
        //             ini_set('memory_limit', '-1');
        //             if ($property->address->ex_address1) {
        //                 $addr = $property->address->ex_address1;
        //             } else {
        //                 $addr = trim($property->address->ex_company_name);
        //                 $name = '';
        //             }

        //             $pdf->SetFontSize('10');
        //             $pdf->SetXY(18.25, 50);
        //             $pdf->Write(0, $name);
        //             $pdf->SetXY(18.25, 52);
        //             $pdf->MultiCell(60, 5, (Helper::parseProbateAddress($addr)), 0, 1);
        //             if ($property->letter) {
        //                 $pdf->SetXY(18.25, 90);
        //                 $content = $property->letter->content;
        //                 $content = str_replace('&nbsp;', '', $content);
        //                 // $content = preg_replace('/[[:^print:]]/', '', $content);
        //                 $pdf->SetMargins(18.25, 0, 18.25);
        //                 // $pdf->MultiCell(0, 6, $this->WriteHTML($content, $pdf, $campaign->name), 0, 1);
        //                 $pdf->MultiCell(0, 6, $content, 0, 1);
        //             }
        //         }
        //         ob_get_clean();
        //         $pdf->Output($campaign->name.'.pdf', 'I');
        //     }
        // }
    }
    

protected function normalize($word)
        {        
            $word = str_replace("&quot;",'"',$word);
            $word = str_replace("&amp;","&",$word);
            $word = str_replace("&lt;","<",$word);
            $word = str_replace("&gt;",">",$word);
            $word = str_replace("&nbsp;"," ",$word);
            $word = str_replace("&Agrave;","À",$word);
            $word = str_replace("&Aacute;","Á",$word);
            $word = str_replace("&Acirc;","Â",$word);
            $word = str_replace("&Atilde;","Ã",$word);
            $word = str_replace("&Auml;","Ä",$word);
            $word = str_replace("&Aring;","Å",$word);
            $word = str_replace("&AElig;","Æ",$word);
            $word = str_replace("&Ccedil;","Ç",$word);
            $word = str_replace("&Egrave;","È",$word);
            $word = str_replace("&Eacute;","É",$word);
            $word = str_replace("&Ecirc;","Ê",$word);
            $word = str_replace("&Euml;","Ë",$word);
            $word = str_replace("&Igrave;","Ì",$word);
            $word = str_replace("&Iacute;","Í",$word);
            $word = str_replace("&Icirc;","Î",$word);
            $word = str_replace("&Iuml;","Ï",$word);
            $word = str_replace("&ETH;","Ð",$word);
            $word = str_replace("&Ntilde;","Ñ",$word);
            $word = str_replace("&Ograve;","Ò",$word);
            $word = str_replace("&Oacute;","Ó",$word);
            $word = str_replace("&Ocirc;","Ô",$word);
            $word = str_replace("&Otilde;","Õ",$word);
            $word = str_replace("&Ouml;","Ö",$word);
            $word = str_replace("&times;","×",$word);
            $word = str_replace("&Oslash;","Ø",$word);
            $word = str_replace("&Ugrave;","Ù",$word);
            $word = str_replace("&Uacute;","Ú",$word);
            $word = str_replace("&Ucirc;","Û",$word);
            $word = str_replace("&Uuml;","Ü",$word);
            $word = str_replace("&Yacute;","Ý",$word);
            $word = str_replace("&THORN;","Þ",$word);
            $word = str_replace("&szlig;","ß",$word);
            $word = str_replace("&agrave;","à",$word);
            $word = str_replace("&aacute;","á",$word);
            $word = str_replace("&acirc;","â",$word);
            $word = str_replace("&atilde;","ã",$word);
            $word = str_replace("&auml;","ä",$word);
            $word = str_replace("&aring;","å",$word);
            $word = str_replace("&aelig;","æ",$word);
            $word = str_replace("&ccedil;","ç",$word);
            $word = str_replace("&egrave;","è",$word);
            $word = str_replace("&eacute;","é",$word);
            $word = str_replace("&ecirc;","ê",$word);
            $word = str_replace("&euml;","ë",$word);
            $word = str_replace("&igrave;","ì",$word);
            $word = str_replace("&iacute;","í",$word);
            $word = str_replace("&icirc;","î",$word);
            $word = str_replace("&iuml;","ï",$word);
            $word = str_replace("&eth;","ð",$word);
            $word = str_replace("&ntilde;","ñ",$word);
            $word = str_replace("&ograve;","ò",$word);
            $word = str_replace("&oacute;","ó",$word);
            $word = str_replace("&ocirc;","ô",$word);
            $word = str_replace("&otilde;","õ",$word);
            $word = str_replace("&ouml;","ö",$word);
            $word = str_replace("&divide;","÷",$word);
            $word = str_replace("&oslash;","ø",$word);
            $word = str_replace("&ugrave;","ù",$word);
            $word = str_replace("&uacute;","ú",$word);
            $word = str_replace("&ucirc;","û",$word);
            $word = str_replace("&uuml;","ü",$word);
            $word = str_replace("&yacute;","ý",$word);
            $word = str_replace("&thorn;","þ",$word);
            $word = str_replace("&yuml;","ÿ",$word);
            $word = str_replace("&OElig;","Œ",$word);
            $word = str_replace("&oelig;","œ",$word);
            $word = str_replace("&Scaron;","Š",$word);
            $word = str_replace("&scaron;","š",$word);
            $word = str_replace("&Yuml;","Ÿ",$word);
            $word = str_replace("&fnof;","ƒ",$word);
            $word = str_replace("&circ;","ˆ",$word);
            $word = str_replace("&tilde;","˜",$word);
            $word = str_replace("&ndash;","-",$word);
            $word = str_replace("&mdash;","-",$word);
            $word = str_replace("&lsquo;","`",$word);
            $word = str_replace("&rsquo;","`",$word);
            $word = str_replace("&sbquo;","‚",$word);
            $word = str_replace("&ldquo;",'"',$word);
            $word = str_replace("&rdquo;",'"',$word);
            $word = str_replace("&bdquo;","„",$word);
            $word = str_replace("&lsaquo;","‹",$word);
            $word = str_replace("&rsaquo;","›",$word);
            $word = str_replace("&iexcl;","¡",$word);
            $word = str_replace("&cent;","¢",$word);
            $word = str_replace("&pound;","£",$word);
            $word = str_replace("&curren;","¤",$word);
            $word = str_replace("&yen;","¥",$word);
            $word = str_replace("&euro;","€",$word);
            $word = str_replace("&brvbar;","¦",$word);
            $word = str_replace("&sect;","§",$word);
            $word = str_replace("&uml;","¨",$word);
            $word = str_replace("&copy;","©",$word);
            $word = str_replace("&ordf;","ª",$word);
            $word = str_replace("&laquo;","«",$word);
            $word = str_replace("&not;","¬",$word);
            $word = str_replace("&shy;","­ ",$word);
            $word = str_replace("&reg;","®",$word);
            $word = str_replace("&macr;","¯",$word);
            $word = str_replace("&deg;","°",$word);
            $word = str_replace("&plusmn;","±",$word);
            $word = str_replace("&sup2;","²",$word);
            $word = str_replace("&sup3;","³",$word);
            $word = str_replace("&acute;","´",$word);
            $word = str_replace("&micro;","µ",$word);
            $word = str_replace("&para;","¶",$word);
            $word = str_replace("&middot;","·",$word);
            $word = str_replace("&cedil;","¸",$word);
            $word = str_replace("&sup1;","¹",$word);
            $word = str_replace("&ordm;","º",$word);
            $word = str_replace("&raquo;","»",$word);
            $word = str_replace("&frac14;","¼",$word);
            $word = str_replace("&frac12;","½",$word);
            $word = str_replace("&frac34;","¾",$word);
            $word = str_replace("&iquest;","¿",$word);
            $word = str_replace("&Alpha;","Α",$word);
            $word = str_replace("&Beta;","Β",$word);
            $word = str_replace("&Gamma;","Γ",$word);
            $word = str_replace("&Delta;","Δ",$word);
            $word = str_replace("&Epsilon;","Ε",$word);
            $word = str_replace("&Zeta;","Ζ",$word);
            $word = str_replace("&Eta;","Η",$word);
            $word = str_replace("&Theta;","Θ",$word);
            $word = str_replace("&Iota;","Ι",$word);
            $word = str_replace("&Kappa;","Κ",$word);
            $word = str_replace("&Lambda;","Λ",$word);
            $word = str_replace("&Mu;","Μ",$word);
            $word = str_replace("&Nu;","Ν",$word);
            $word = str_replace("&Xi;","Ξ",$word);
            $word = str_replace("&Omicron;","Ο",$word);
            $word = str_replace("&Pi;","Π",$word);
            $word = str_replace("&Rho;","Ρ",$word);
            $word = str_replace("&Sigma;","Σ",$word);
            $word = str_replace("&Tau;","Τ",$word);
            $word = str_replace("&Upsilon;","Υ",$word);
            $word = str_replace("&Phi;","Φ",$word);
            $word = str_replace("&Chi;","Χ",$word);
            $word = str_replace("&Psi;","Ψ",$word);
            $word = str_replace("&Omega;","Ω",$word);
            $word = str_replace("&alpha;","α",$word);
            $word = str_replace("&beta;","β",$word);
            $word = str_replace("&gamma;","γ",$word);
            $word = str_replace("&delta;","δ",$word);
            $word = str_replace("&epsilon;","ε",$word);
            $word = str_replace("&zeta;","ζ",$word);
            $word = str_replace("&eta;","η",$word);
            $word = str_replace("&theta;","θ",$word);
            $word = str_replace("&iota;","ι",$word);
            $word = str_replace("&kappa;","κ",$word);
            $word = str_replace("&lambda;","λ",$word);
            $word = str_replace("&mu;","μ",$word);
            $word = str_replace("&nu;","ν",$word);
            $word = str_replace("&xi;","ξ",$word);
            $word = str_replace("&omicron;","ο",$word);
            $word = str_replace("&pi;","π",$word);
            $word = str_replace("&rho;","ρ",$word);
            $word = str_replace("&sigmaf;","ς",$word);
            $word = str_replace("&sigma;","σ",$word);
            $word = str_replace("&tau;","τ",$word);
            $word = str_replace("&upsilon;","υ",$word);
            $word = str_replace("&phi;","φ",$word);
            $word = str_replace("&chi;","χ",$word);
            $word = str_replace("&psi;","ψ",$word);
            $word = str_replace("&omega;","ω",$word);
            $word = str_replace("&thetasym;","ϑ",$word);
            $word = str_replace("&upsih;","ϒ",$word);
            $word = str_replace("&piv;","ϖ",$word);
            $word = str_replace("&dagger;","†",$word);
            $word = str_replace("&Dagger;","‡",$word);
            $word = str_replace("&bull;","•",$word);
            $word = str_replace("&hellip;","…",$word);
            $word = str_replace("&permil;","‰",$word);
            $word = str_replace("&prime;","′",$word);
            $word = str_replace("&Prime;","″",$word);
            $word = str_replace("&oline;","‾",$word);
            $word = str_replace("&frasl;","⁄",$word);
            $word = str_replace("&weierp;","℘",$word);
            $word = str_replace("&image;","ℑ",$word);
            $word = str_replace("&real;","ℜ",$word);
            $word = str_replace("&trade;","™",$word);
            $word = str_replace("&alefsym;","ℵ",$word);
            $word = str_replace("&larr;","←",$word);
            $word = str_replace("&uarr;","↑",$word);
            $word = str_replace("&rarr;","→",$word);
            $word = str_replace("&darr;","↓",$word);
            $word = str_replace("&harr;","↔",$word);
            $word = str_replace("&crarr;","↵",$word);
            $word = str_replace("&lArr;","⇐",$word);
            $word = str_replace("&uArr;","⇑",$word);
            $word = str_replace("&rArr;","⇒",$word);
            $word = str_replace("&dArr;","⇓",$word);
            $word = str_replace("&hArr;","⇔",$word);
            $word = str_replace("&forall;","∀",$word);
            $word = str_replace("&part;","∂",$word);
            $word = str_replace("&exist;","∃",$word);
            $word = str_replace("&empty;","∅",$word);
            $word = str_replace("&nabla;","∇",$word);
            $word = str_replace("&isin;","∈",$word);
            $word = str_replace("&notin;","∉",$word);
            $word = str_replace("&ni;","∋",$word);
            $word = str_replace("&prod;","∏",$word);
            $word = str_replace("&sum;","∑",$word);
            $word = str_replace("&minus;","−",$word);
            $word = str_replace("&lowast;","∗",$word);
            $word = str_replace("&radic;","√",$word);
            $word = str_replace("&prop;","∝",$word);
            $word = str_replace("&infin;","∞",$word);
            $word = str_replace("&ang;","∠",$word);
            $word = str_replace("&and;","∧",$word);
            $word = str_replace("&or;","∨",$word);
            $word = str_replace("&cap;","∩",$word);
            $word = str_replace("&cup;","∪",$word);
            $word = str_replace("&int;","∫",$word);
            $word = str_replace("&there4;","∴",$word);
            $word = str_replace("&sim;","∼",$word);
            $word = str_replace("&cong;","≅",$word);
            $word = str_replace("&asymp;","≈",$word);
            $word = str_replace("&ne;","≠",$word);
            $word = str_replace("&equiv;","≡",$word);
            $word = str_replace("&le;","≤",$word);
            $word = str_replace("&ge;","≥",$word);
            $word = str_replace("&sub;","⊂",$word);
            $word = str_replace("&sup;","⊃",$word);
            $word = str_replace("&nsub;","⊄",$word);
            $word = str_replace("&sube;","⊆",$word);
            $word = str_replace("&supe;","⊇",$word);
            $word = str_replace("&oplus;","⊕",$word);
            $word = str_replace("&otimes;","⊗",$word);
            $word = str_replace("&perp;","⊥",$word);
            $word = str_replace("&sdot;","⋅",$word);
            $word = str_replace("&lceil;","⌈",$word);
            $word = str_replace("&rceil;","⌉",$word);
            $word = str_replace("&lfloor;","⌊",$word);
            $word = str_replace("&rfloor;","⌋",$word);
            $word = str_replace("&lang;","⟨",$word);
            $word = str_replace("&rang;","⟩",$word);
            $word = str_replace("&loz;","◊",$word);
            $word = str_replace("&spades;","♠",$word);
            $word = str_replace("&clubs;","♣",$word);
            $word = str_replace("&hearts;","♥",$word);
            $word = str_replace("&diams;","♦",$word);
            $word = str_replace("&ensp;"," ",$word);
            $word = str_replace("&emsp;"," ",$word);
            $word = str_replace("&thinsp;"," ",$word);
            $word = str_replace("&zwnj;"," ",$word);
            $word = str_replace("&zwj;"," ",$word);
            $word = str_replace("&lrm;"," ",$word);
            $word = str_replace("&rlm;"," ",$word);

            return $word;
        }

    //property add to campaign
    public function contactCampaign(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $campaign = new Campaign();
        $campaign->name = 'LETS BID - '.Carbon::now()->format('d-m-Y h:i');
        $campaign->campaign_status = 0;
        $campaign->user_id = $user_id;
        $campaign->save();

        foreach ($request->property_id as $property_id) {
            $trigger = 'trigger_'.$property_id;
            PropertyCampaign::where('id', $property_id)
                ->update([
                    'campaign_id' => $campaign->id,
                    'trigger' => $request->{$trigger},
                ])
            ;
        }
        Mail::to('susmidesigndirect@gmail.com')
            // ->bcc('rohandesigndirect@gmail.com')
            ->queue(new ContactCampaign($campaign->name))
        ;

        $url = url('campaign-history');

        return redirect($url);
    }

    public function allCampaign(Request $request)
    {
        $page_title = 'My ProbatePropertyCampaign';
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_outcode = $this->userOutcode();
        $lists = ProbatePropertyCampaign::join('probate_property as h', 'probate_property_campaign.property_id', 'h.id')
            ->join('postcode as pc', 'pc.id', 'h.postcode_id')
            ->join('outcode as oc', 'oc.id', 'h.outcode_id')
            ->select('probate_property_campaign.*', 'h.id', DB::raw('CONCAT(oc.outcode,pc.postcode) as pincode'), 'h.publication_date', 'h.id as h_id', 'probate_property_campaign.id as pcid')
            ->where('probate_property_campaign.user_id', $user_id)
            ->whereIn('oc.id', $user_outcode)
            ->orderBy('probate_property_campaign.id', 'desc')
            ->get()
        ;

        // dd($lists);

        if (count($lists) > 0) {
            $i = 0;
            foreach ($lists as $item) {
                $properties = ProbateProperty::
                    where('probate_property.id', $item->h_id)
                        ->join('executor', 'executor.id', 'probate_property.executor_id')
                        ->select('probate_property.address_1', 'probate_property.address_2', 'executor.first_name', 'executor.company_name')
                        ->first()
                ;
                // $full_address = $item->house_number.',<br/> ';
                // $full_address .= (count($properties) > 0 && '' != $properties[0]->address) ? $properties[0]->address.',<br/>' : '';
                $full_address = $properties->address_1.' '.$properties->address_2;
                // $agency = count($properties->pluck('agency')) > 0 ? implode(',', $properties->pluck('agency')->toArray()) : '';

                // $campaign = ProbatePropertyCampaign::where('id', $item->campaign_id)
                // ->where('campaign_status', 1)
                // ->latest()
                // ->first()

                // dd($properties);

                if (empty($properties->first_name) && !empty($properties->company_name)) {
                    $companyname = explode(',', $properties->company_name);
                    $name = $companyname[0];
                } else {
                    $name = $properties->first_name.' '.$properties->sur_name;
                }
                $campaign = '';
                ++$i;
                $item->sl = $i;
                $item->address = $full_address;
                $item->agency = $name;
                $item->trigger = '';
                // $item->trigger = Helper::selectTrigger($item, $properties);
                $item->listed_date = Carbon::parse($item->publication_date)->format('d/m/Y');
                $item->last_letter_sent = $campaign ? Carbon::parse($campaign->send_date)->format('d/m/Y') : '';
                $item->last_date_sent = '';
            }
        }

  
        // return Datatables::of($item)
        //         ->editColumn('name', function ($model) {
        //            return $model->name;
        //          })
        //         ->editColumn('trigger', function ($model) {
        //             return$this->trigger[$model->triggers];                    
        //          })  
        //         ->editColumn('action', function ($model) {
        //                 $edit_url = url($this->link.'/'.$model->id.'/edit');
        //                 $action = '';
        //                 $action .= Helper::editButton($edit_url);
        //                 $action .= Helper::deleteButton($model->id);

        //         return $action;
        //   })

        //   ->rawColumns(['name','trigger','action'])
        //   ->make(true);


        $triggers = $this->trigger;
        $campaign_merge = 1;
        $prob_class = 'active';
        $my_class = 'add-un';

        return view('probate-campaign.list', compact('lists', 'triggers', 'campaign_merge', 'prob_class', 'my_class'));
    }


public function getProbateCampaignData()
{
    $session_array = $data_list = [];
    $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;  
    $user_outcode = $this->userOutcode();
    $lists = ProbatePropertyCampaign::join('probate_property as h', 'probate_property_campaign.property_id', 'h.id')
        ->join('postcode as pc', 'pc.id', 'h.postcode_id')
        ->join('outcode as oc', 'oc.id', 'h.outcode_id')
        ->select('probate_property_campaign.*', 'h.id', DB::raw('CONCAT(oc.outcode,pc.postcode) as pincode'), 'h.publication_date', 'h.id as h_id', 'probate_property_campaign.id as pcid')
        ->where('probate_property_campaign.user_id', $user_id)
        ->whereIn('oc.id', $user_outcode)
        ->orderBy('probate_property_campaign.id', 'desc')
        ->get();

    if (count($lists) > 0) {
        $i = 0;
        foreach ($lists as $item) {
            $properties = ProbateProperty::
                where('probate_property.id', $item->h_id)
                    ->join('executor', 'executor.id', 'probate_property.executor_id')
                    ->select('probate_property.address_1', 'probate_property.address_2', 'executor.first_name', 'executor.company_name')
                    ->first()
            ;
            $full_address = $properties->address_1.' '.$properties->address_2;
            if (empty($properties->first_name) && !empty($properties->company_name)) {
                $companyname = explode(',', $properties->company_name);
                $name = $companyname[0];
            } else {
                $name = $properties->first_name.' '.$properties->sur_name;
            }
            $campaign = '';
           ++$i;
            // $item[] = ['sl' => $i];
            
            $data_list['sl'] = $i;
            // $item->address = $full_address;
            // $item->agency = $name;
            // $item->trigger = '';
            // // $item->trigger = Helper::selectTrigger($item, $properties);
            // $item->listed_date = Carbon::parse($item->publication_date)->format('d/m/Y');
            // $item->last_letter_sent = $campaign ? Carbon::parse($campaign->send_date)->format('d/m/Y') : '';
            // $item->last_date_sent = '';
        }
    }
    $item = collect($data_list);
    dd($data_list);
    return Datatables::of($data_list)
        // ->editColumn('sl', function ($model) {
        //    return '<label class="checkbox checkbox-outline checkbox-outline-2x checkbox-primary"><input class="propertyCheck" type="checkbox" value="'.$model->id.'" name="property[]"><span></span> </label>';
        //     })
        ->editColumn('sl', function ($model) {
            // dd($model);
         return "s";
        })
        ->rawColumns(['sl'])
        ->make(true);
}


    public function campaignHistory(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));

        $campaign = ProbatePropertyCampaign::where('user_id', $user->id)
            ->get()
        ;
        $letter = Letter::where('user_id', $user->id)->get();

        return view('campaign.history', compact('campaign', 'letter'));
    }

    public function campaignDetail(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));

        $campaign = ProbatePropertyCampaign::where('id', $id)
            ->with(
                'propertyCampaign',
                'propertyCampaign.houseNumber',
                'propertyCampaign.houseNumber.propertyHouseNumber',
                'propertyCampaign.houseNumber.propertyHouseNumber.propertyHistory',
                'propertyCampaign.houseNumber.propertyHouseNumber.propertyLetter',
                'propertyCampaign.houseNumber.propertyHouseNumber.propertyLetter.letter'
            )
            ->first()
        ;
        // dd($campaign->propertyCampaign[0]['houseNumber']['propertyHouseNumber'][0]['propertyHistory']['address']);die;

        return view('campaign.detail', compact('campaign'));
    }

    public function searchHistory(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $letter = Letter::where('user_id', $user->id)->get();
        $campaign = ProbatePropertyCampaign::where('user_id', $user->id)
            ->where('name', 'LIKE', '%'.$request->search.'%')
            ->get()
        ;

        return view('campaign.history', compact('campaign', 'letter'));
    }

    // public function removeCampaign(Request $request)
    // {
    //     $user = json_decode(Session::get(config('propai.authSessionKey')));
    //     $campaign = ProbatePropertyCampaign::where('user_id', $user->id)->delete();

    //     $triggers = $this->trigger;
    //     $lists = [];

    //     $campaign_merge = 1;
    //     $prob_class = 'active';
    //     $my_class = 'add-un';

    //     return view('probate-campaign.list', compact('lists', 'triggers', 'campaign_merge', 'prob_class', 'my_class'));
    // }

    public function removeCampaign(Request $request)
    {
        $user           = json_decode(Session::get(config('propai.authSessionKey')));
        if (isset($request['del_prop']) && $request['del_prop'] != null) {
            $del_prop = $request['del_prop'];
            $campaign_prop_removed = '';
            if ($del_prop == 'all') {
                $campaign_prop_removed = ProbatePropertyCampaign::where('user_id', $user->id)->delete();
                
                return ["success" => true, "deleted_props" => $campaign_prop_removed];
            } else if (is_array($del_prop) && count($del_prop) > 0) {
                $campaign_prop_removed = ProbatePropertyCampaign::where('user_id', $user->id)->whereIn('id', $del_prop)->delete();
                
                return ["success" => true, "deleted_props" => $campaign_prop_removed];
            } else {
                return ["success" => false, "deleted_props" => null];
            }
        } else {
            return ["success" => false, "deleted_props" => null];
        }
    }

     public function removeCampaignId(Request $request,$id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));

        $delete = ProbatePropertyCampaign::where(['property_id' => $id,'user_id' => $user->id])->delete();
       
        if (!$delete) {
            return $this->simpleReturn('error', 'Something went wrong...!');
        }
        else
        {
            return $this->simpleReturn('success', 'Successfully removed from campaign...!');
        } 
    }

    /*public function removeCampaignId(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $delete = ProbatePropertyCampaign::where('id', $id)->delete();
        $callback = function ($query) {
            $query->where('campaign_status', '=', 1);
        };

        $campaign = ProbatePropertyCampaign::where('user_id', $user->id)
            ->with(
                'propertyHistory',
                'campaign',
                'user',
                'propertyHistory.agency',
                'propertyHistory.propertyLetter',
                'propertyHistory.propertyLetter.letter'
            )
            ->whereHas('campaign', $callback)
            ->with(['campaign' => $callback])
            ->get()
        ;
        $letter = Letter::where('user_id', $user->id)->get();

        return view('probate-campaign.list', compact('campaign', 'letter'));
    }*/

    public function createLetter(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        if (!empty($request->file('file'))) {
            $path = $request->file('file')->store('public/letters');
        } else {
            $path = '';
        }

        Letter::create([
            'name' => $request->name,
            'user_id' => $user->id,
            'file' => $path,
            'trigger' => $request->trigger,
        ]);

        $letter = Letter::where('user_id', $user->id)->orderby('created_at', 'DESC')->get();

        return view('campaign.edit-letter', compact('letter'));
    }

    public function updateLetter(Request $request, $id)
    {
        $rules = [
            'name' => 'required',
            'trigger' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('msg_error', $validator->errors()->first());
        }
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        if (!empty($request->file('file'))) {
            $path = $request->file('file')->store('public/letters');
            Letter::where('id', $id)->update([
                'file' => $path,
            ]);
        }

        Letter::where('id', $id)->update([
            'name' => $request->name,
            'triggers' => $request->trigger,
            'content' => $request->content,
        ]);
        $letter = Letter::where('user_id', $user->id)->orderby('created_at', 'DESC')->get();

        return view('campaign.edit-letter', compact('letter'));
    }

    public function editLetter(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $letter = Letter::where('user_id', $user->id)->orderby('created_at', 'DESC')->get();

        return view('campaign.edit-letter', compact('letter'));
    }

    public function viewLetter(Request $request)
    {
        return view('campaign.view-letter');
    }

    public function manageLetter(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $letter = Letter::where('id', $id)->first();

        return view('campaign.manage-letter', compact('letter'));
    }

    public function deleteLetterId(Request $request, $id)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $delete = Letter::where('id', $id)->delete();
        $letter = Letter::where('user_id', $user->id)->orderby('created_at', 'DESC')->get();

        return view('campaign.edit-letter', compact('letter'));
    }

    // sanju-start
    public function update(Request $request, $id)
    {
        $campaign = Campaign::find($id);
        $campaign->name = $request->name;
        $campaign->save();

        $error = false;
        $message = 'Updated successfully';

        return compact('error', 'message');
    }

    public function show(Request $request, $id)
    {
        $authUser = json_decode(Session::get(config('propai.authSessionKey')))->id;
        // $orgId = $authUser->organisation_id;

        $campaign = Campaign::
            where('user_id', $authUser)->find($id);

        if ($campaign) {
            $user = json_decode(Session::get(config('propai.authSessionKey')));
            $sendDate = '' != $campaign->sent_date ? Carbon::parse($campaign->sent_date)->format('d/m/Y') : '';

            $lists = ProbatePropertyCampaign::withTrashed()
                ->join('probate_property as h', 'probate_property_campaign.property_id', 'h.id')
                ->join('postcode as pc', 'pc.id', 'h.postcode_id')
                ->join('outcode as oc', 'oc.id', 'pc.outcode_id')
                ->join('executor as a', 'a.id', 'h.executor_id')
                ->select('probate_property_campaign.*', DB::raw('CONCAT(oc.outcode,pc.postcode) as pincode'), 'h.*', 'a.first_name as e_first', 'a.company_name as e_company', 'a.sur_name as e_sur')
                ->where('probate_property_campaign.campaign_id', $campaign->id)
                ->get()
            ;

            if (count($lists) > 0) {
                $i = 0;
                foreach ($lists as $item) {
                    $properties = ProbateProperty::select('probate_property.*', 'a.first_name as e_first', 'a.company_name as e_company', 'a.sur_name as e_sur', 'pc.postcode as post_code')
                        // ->join('property_house_number as phn', 'phn.property_history_id', 'property_history.id')
                        ->join('postcode as pc', 'pc.id', 'probate_property.postcode_id')
                        ->join('executor as a', 'a.id', 'probate_property.executor_id')
                        ->where('probate_property.id', $item->property_id)
                        ->get()
                    ;
                    // return $item;
                    $full_address = $item->address_1.' '.$item->address_2.' '.$item->town.' '.$item->postcode;
                    //  $full_address .= (count($properties) > 0 && '' != $properties[0]->address) ? $properties[0]->address.',<br/>' : '';
                    // $full_address .= $item->pincode;
                    // $agency = count($properties->pluck('agency')) > 0 ? implode(',', $properties->pluck('agency')->toArray()) : '';
                    //   $agency=$item->executor;
                    // $campaign = Campaign::where('id', $item->campaign_id)
                    //     ->where('campaign_status', 1)
                    //     ->latest()
                    //     ->first();
                    // $encrpt_id = Helper::encryptData($item->id);
                       $encrpt_id = Crypt::encrypt($item->id);

                    $lastLetter = ProbatePropertyCampaign::withTrashed()
                        ->whereHas('campaign')
                        ->where('property_id', $item->property_id)
                        ->where('user_id', $campaign->user_id)
                        ->whereNotNull('campaign_id')
                        ->latest()->first();
                    if (empty($item->e_first) && !empty($item->e_company)) {
                        $companyname = explode(',', $item->e_company);
                        $agency = $companyname[0];
                    } else {
                        $agency = $item->e_first.' '.$item->e_sur;
                    }
                    ++$i;
                    $item->sl = $i;
                    $item->address = $full_address;
                    $item->property_url = $encrpt_id;
                    $item->agency = $agency;
                    // $item->trigger = $this->trigger[$item->trigger];
                    $item->listed_date = Carbon::parse($item->publication_date)->format('d/m/Y');
                    $item->last_letter_sent = $lastLetter ? $lastLetter->campaign->sent_date->format('d/m/Y') : '';
                }
            }

            return view('probate-campaign.view', compact('campaign', 'sendDate', 'lists'));
        }

        return redirect()->route('probate-campaign-history.index')->with('msg_error', 'Cannot find any campaign with this ID.');
    }

    // sanju-end

    //letter head
    public function letterHead(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $data = LetterHead::where('user_id', $user->id)->first();
        if (!$data) {
            $data = '';
        }

        return view('campaign.letter_head', compact('data'));
    }

    public function createLetterHead(Request $request)
    {
        $rules = [
            'letter_head_file' => 'required|mimes:pdf',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('msg_error', $validator->errors()->first());
        }

        if ($request->hasFile('letter_head_file')) {
            // Get filename with extension
            $filenameWithExt = $request->file('letter_head_file')->getClientOriginalName();
            // Get just filename
            $filename = pathinfo($filenameWithExt, PATHINFO_FILENAME);
            // Get just ext
            $extension = $request->file('letter_head_file')->getClientOriginalExtension();
            //Filename to store
            $fileNameToStore = time().'.'.$extension;
            // Upload Image
            $path = $request->file('letter_head_file')->storeAs('public/letter-heads', $fileNameToStore);
            $user = json_decode(Session::get(config('propai.authSessionKey')));

            $file = storage_path('app/public/letter-heads/'.$fileNameToStore);

            $filepdf = fopen($file, 'r');

            if ($filepdf) {
                $line_first = fgets($filepdf);

                preg_match_all('!\d+!', $line_first, $matches);
                $pdfversion = implode('.', $matches[0]);
                fclose($filepdf);
            } else {
                unlink($file);

                return Redirect::back()->with('msg_error', 'Upload valid pdf file.');
            }

            // if (floatval($pdfversion) < 1.4) {
            //     unlink($file);

            //     return Redirect::back()->with('msg_error', 'Upload valid pdf file.');
            // }

            $data = LetterHead::where('user_id', $user->id)->first();
            if (!$data) {
                $add = new LetterHead();
                $add->user_id = $user->id;
                $add->file = $fileNameToStore;
                $add->save();

                return Redirect::route('letterHead')->with('msg_success', 'Successfully Added.');
            }
            unlink(storage_path('app/public/letter-heads/'.$data['file']));

            LetterHead::where('id', $data['id'])
                ->update([
                    'file' => $fileNameToStore,
                ])
                ;

            return Redirect::route('letterHead')->with('msg_success', 'Successfully Updated.');
        }

        return Redirect::back()->with('msg_error', 'Something went wrong.');
    }

    public function letterHeadEdit()
    {
        $data = '';

        return view('campaign.letter_head', compact('data'));
    }

    public function letterHeadDelete(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));

        $data = LetterHead::where('user_id', $user->id)->first();
        unlink(storage_path('app/public/letter-heads/'.$data['file']));
        LetterHead::where('id', $data['id'])
            ->delete()
        ;

        return Redirect::route('letterHead')->with('msg_success', 'Successfully Deleted.');
    }

    //letter head

    public function WriteHTML($html, $pdf, $campaign)
    {
        //HTML parser
        $html = strip_tags($html, '<b><u><i><a><img><p><br><strong><em><font><tr><blockquote>'); //supprime tous les tags sauf ceux reconnus
        $html = str_replace("\n", ' ', $html); //remplace retour � la ligne par un espace
        $a = preg_split('/<(.*)>/U', $html, -1, PREG_SPLIT_DELIM_CAPTURE);

        //�clate la cha�ne avec les balises

        // $pdf = new \setasign\Fpdi\Fpdi('Portrait','mm',array(215.9,279.4), true, 'UTF-8', true);
        // $tpl = $pdf->importPage(1);
        // $pdf->AddPage();

        // Use the imported page as the template
        // $pdf->useTemplate($tpl);
        // $pdf->SetFont('Helvetica');
        foreach ($a as $i => $e) {
            if (0 == $i % 2) {
                //Text
                if ($this->HREF) {
                    $pdf->SetX(18.25);
                    $this->PutLink($this->HREF, $e, $pdf);
                } else {
                    // $pdf->SetX(18.25);
                    $pdf->Write(5, stripslashes($this->txtentities($e)));
                }
            } else {
                //Tag
                if ('/' == $e[0]) {
                    $this->CloseTag(strtoupper(substr($e, 1)), $pdf);
                } else {
                    //Extract attributes
                    $a2 = explode(' ', $e);
                    $tag = strtoupper(array_shift($a2));
                    $attr = [];
                    foreach ($a2 as $v) {
                        if (preg_match('/([^=]*)=["\']?([^"\']*)/', $v, $a3)) {
                            $attr[strtoupper($a3[1])] = $a3[2];
                        }
                    }
                    $this->OpenTag($tag, $attr, $pdf);
                }
            }
        }

        // $pdf->Output($campaign.'.pdf','I');
    }

    public function OpenTag($tag, $attr, $pdf)
    {
        //Opening tag
        switch ($tag) {
            case 'STRONG':
                $this->SetStyle('B', true, $pdf);

                break;

            case 'EM':
                $this->SetStyle('I', true, $pdf);

                break;

            case 'B':
            case 'I':
            case 'U':
                $this->SetStyle($tag, true, $pdf);

                break;

            case 'A':
                $this->HREF = $attr['HREF'];

                break;

            case 'IMG':
                if (isset($attr['SRC']) && (isset($attr['WIDTH']) || isset($attr['HEIGHT']))) {
                    if (!isset($attr['WIDTH'])) {
                        $attr['WIDTH'] = 0;
                    }
                    if (!isset($attr['HEIGHT'])) {
                        $attr['HEIGHT'] = 0;
                    }
                    $this->Image($attr['SRC'], $this->GetX(), $this->GetY(), px2mm($attr['WIDTH']), px2mm($attr['HEIGHT']));
                }

                break;

            case 'TR':
            case 'BLOCKQUOTE':
            case 'BR':
                $pdf->Ln(5);

                break;

            case 'P':
                $pdf->Ln(10);
                $pdf->SetX(18.25);

                break;

            case 'FONT':
                if (isset($attr['COLOR']) && '' != $attr['COLOR']) {
                    $coul = hex2dec($attr['COLOR']);
                    $pdf->SetTextColor($coul['R'], $coul['V'], $coul['B']);
                    $this->issetcolor = true;
                }
                if (isset($attr['FACE']) && in_array(strtolower($attr['FACE']), $this->fontlist)) {
                    $pdf->SetFont(strtolower($attr['FACE']));
                    $this->issetfont = true;
                }

                break;
        }
    }

    public function CloseTag($tag, $pdf)
    {
        //Closing tag
        if ('STRONG' == $tag) {
            $tag = 'B';
        }
        if ('EM' == $tag) {
            $tag = 'I';
        }
        if ('B' == $tag || 'I' == $tag || 'U' == $tag) {
            $this->SetStyle($tag, false, $pdf);
        }
        if ('A' == $tag) {
            $this->HREF = '';
        }
        if ('FONT' == $tag) {
            if (true == $this->issetcolor) {
                $pdf->SetTextColor(0);
            }
            if ($this->issetfont) {
                $pdf->SetFont('arial');
                $this->issetfont = false;
            }
        }
    }

    public function SetStyle($tag, $enable, $pdf)
    {
        //Modify style and select corresponding font
        $this->{$tag} += ($enable ? 1 : -1);
        $style = '';
        foreach (['B', 'I', 'U'] as $s) {
            if ($this->{$s} > 0) {
                $style .= $s;
            }
        }
        $pdf->SetFont('', $style);
    }

    public function PutLink($URL, $txt, $pdf)
    {
        //Put a hyperlink
        $pdf->SetTextColor(0, 0, 255);
        $this->SetStyle('U', true);
        $pdf->SetX(18.25);
        $this->Write(5, $txt, $URL);
        $this->SetStyle('U', false);
        $pdf->SetTextColor(0);
    }

    public function hex2dec($couleur = '#000000')
    {
        $R = substr($couleur, 1, 2);
        $rouge = hexdec($R);
        $V = substr($couleur, 3, 2);
        $vert = hexdec($V);
        $B = substr($couleur, 5, 2);
        $bleu = hexdec($B);
        $tbl_couleur = [];
        $tbl_couleur['R'] = $rouge;
        $tbl_couleur['V'] = $vert;
        $tbl_couleur['B'] = $bleu;

        return $tbl_couleur;
    }

    //conversion pixel -> millimeter at 72 dpi
    public function px2mm($px)
    {
        return $px * 25.4 / 72;
    }

    public function txtentities($html)
    {
        $trans = get_html_translation_table(HTML_ENTITIES);
        $trans = array_flip($trans);

        return strtr($html, $trans);
    }

    public function campaignEditAddress(Request $request)
    {
        $res = [];
        foreach ($request->data as $data) {
            $propCampaign = ProbatePropertyCampaign::where('id', $data[0])->first();

            if ($propCampaign) {
                $property_details = ProbateProperty::join('executor as exe', 'exe.id', 'probate_property.executor_id')
                    ->join('postcode as pc', 'pc.id', 'probate_property.postcode_id')
                    ->join('outcode as oc', 'oc.id', 'probate_property.outcode_id')
                    ->where('probate_property.id', $propCampaign->property_id)
                    ->select('probate_property.*', 'pc.postcode', 'oc.outcode', 'exe.address_1 as ex_address1', 'exe.address_2 as ex_address2', 'exe.first_name as ex_fname', 'exe.sur_name as ex_sname', 'exe.company_name as ex_company_name', 'exe.postcode as ex_postcode')
                    ->first()
                ;

                if ($property_details) {
                    if (empty($property_details->ex_fname) && !empty($property_details->ex_company_name)) {
                        $companyname = explode(',', $property_details->ex_company_name);
                        $name = trim($companyname[0]);
                    } else {
                        $name = $property_details->ex_fname.' '.$property_details->ex_sname;
                    }

                    $addr = $property_details->address_1;
                    $addr = str_replace($property_details->postcode, '', $addr);
                    $addr = str_replace($property_details->outcode, '', $addr);
                    $addr = rtrim($addr);
                    $addr = rtrim($addr, ',');
                    // $addr = str_replace(', ', "\n", $addr);

                    $property = [];
                    $property['address'] = str_replace(' ,', ',', Helper::parseProbateAddress($name.', '.$addr.', '.$property_details->postcode, true, true, ','));
                    $property['name'] = $name;

                    if ($property_details->ex_address1) {
                        $addr = $name.', '.$property_details->ex_address1.', '.$property_details->ex_address2.', '.$property_details->ex_postcode;
                    } else {
                        $addr = trim($property_details->ex_company_name);
                    }

                    $property['exe_address'] = str_replace(' ,', ',', Helper::parseProbateAddress($addr, true, true, ','));
                    $property['pcid'] = $propCampaign->property_id;
                    $property['id'] = $data[0];
                    $property['trigger'] = $data[1];
                    $property['property_details'] = $property_details;

                    $res[] = $property;
                }
            }
        }

        return response()->json($res, 200);
    }

    public function campaignEditAddressSave(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if (1 == $request->print_type || 3 == $request->print_type) {
            $source_data = LetterHead::where('user_id', $user_id)->first();
            if (!$source_data) {
                return response()->json(['msg' => 'Please upload Letterhead and try again.'], 400);
            }
        }

        $campaign = new Campaign();
        $campaign->name = 'ProbatePropertyCampaign-'.Carbon::now()->format('d-m-Y h:i');
        $campaign->campaign_status = 0;
        $campaign->user_id = $user_id;
        $campaign->type = 2;
        $campaign->save();

        foreach ($request->pcid as $key => $pc) {
            $propCampaign = ProbatePropertyCampaign::where('id', $pc)->first();

            if ($propCampaign) {
                $propCampaign->campaign_id = $campaign->id;
                $propCampaign->trigger = $request->trigger[$key];
                $propCampaign->edit_property_address = $request->addr[$key];
                $propCampaign->edit_exe_property_address = $request->exe_addr[$key];
                $propCampaign->save();

                // $flag = ProbatePropertyFlag::updateOrCreate(
                //     ['probate_id' => $propCampaign->property_id, 'user_id' => $user_id],
                //     ['probate_id' => $propCampaign->property_id, 'user_id' => $user_id, 'flag' => 3]
                // );
                
                $exist_data = ProbatePropertyFlag::where('user_id', $user_id)->where('probate_id', $propCampaign->property_id)->first();
                if ($exist_data) {
                    $update = ProbatePropertyFlag::find($exist_data['id']);
                    $update->fill(['flag' => 3])->save();
                }else{
                     $add = new ProbatePropertyFlag();
                     $add->probate_id = $propCampaign->property_id;
                     $add->user_id = $user_id;
                     $add->flag = 3;
                     $add->save();
                }

                $letterHistory = new ProbateCampaignLetterHistory();
                $letterHistory->property_id = $propCampaign->property_id;
                $letterHistory->user_id = $user_id;
                $letterHistory->letter_type = $request->trigger[$key];
                $letterHistory->save();
            }
        }

        return response()->json([
            'msg' => 'Campaign submitted successfully',
            'error' => false,
            'url' => url('probate-campaign/print/'.$campaign->id.'/'.$request->print_type),
        ], 200);
    }
}
