<?php

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Http\Requests\LetterRequest;
use App\Model\Letter;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Str;
use Redirect;
use Yajra\DataTables\DataTables;
use Crypt;

/**
 * Class HouseController.
 */
class ProbateLetterController extends Controller
{
    public $title = 'Letters';

    public $trigger = [
        0 => 'Probate 1',
        1 => 'Probate 2',
        2 => 'Probate 3',
        // 3 => 'Sold',
        // 4 => 'Withdrawn',
        // 5 => 'Price drop',
        // 6 => 'Fallen Through',
        // 7 => '8 Week',
        // 8 => '16 Week',
        // 9 => '22 Week',
    ];
    protected $viewPath = 'probate_letters';
    protected $link = 'probate-letters';

    /**
     * Display listing page.
     */
    public function index()
    {
        
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page_title = 'Letter Management';

        $letter_merge = 1;
        $prob_class = "active";
        $my_class = "add-un";
        return view($this->viewPath.'/list', compact('page', 'letter_merge','prob_class','my_class'));
    }

    /**
     * Display a listing of the resource in datatable.
     *
     * @throws \Exception
     */
    public function lists(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $data = Letter::where('user_id', $user_id)->where('letter_type',2)->select(['name', 'triggers', 'id']);

        return Datatables::of($data)
                ->editColumn('name', function ($model) {
                   return $model->name;
                 })
                ->editColumn('trigger', function ($model) {
                    return$this->trigger[$model->triggers];                    
                 })  
                ->editColumn('action', function ($model) {
                        $edit_url = url($this->link.'/'.$model->id.'/edit');
                        $action = '';
                        $action .= Helper::editButton($edit_url);
                        $action .= Helper::deleteButton($model->id);

                return $action;
          })

          ->rawColumns(['name','trigger','action'])
          ->make(true);
        // if (isset($request->form)) {
        //     foreach ($request->form as $search) {
        //         if (null != $search['value'] && 'search_region' == $search['name']) {
        //             $name = strtolower($search['value']);
        //             $detail->where('name', $name);
        //         }
        //     }
        // }

        // return DataTables::of($detail)
        //     ->editColumn('triggers', function ($detail) {
        //         return $this->trigger[$detail->triggers];
        //     })
        //     ->addColumn('action', function ($detail) {
        //         $edit_url = url($this->link.'/'.$detail->id.'/edit');
        //         $action = '';
        //         $action .= Helper::editButton($edit_url);
        //         $action .= Helper::deleteButton($detail->id);

        //         return $action;
        //     })
        //     ->removeColumn(['id'])
        //     ->escapeColumns([])
        //     ->make(false)
        // ;
    }

    /**
     * Show the form for create the specified resource.
     */
    public function create()
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->form_url = url($this->link);
        $page->form_method = 'POST';
        $triggers = $this->trigger;
        $letter = [];
        $page_title = 'Letter Management';

        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $exists = Letter::where('user_id', $user_id)->where('letter_type',2)->distinct('triggers')->pluck('triggers')->toArray();

        return view($this->viewPath.'/manage', compact('page', 'letter', 'triggers', 'page_title', 'exists'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     *
     * @return array
     */
    public function store(LetterRequest $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $letter = new Letter();
        $letter->user_id = $user_id;
         if(isset($request->content) && $request->content != '')
        {

           $cc = str_replace('<br />', '&nbsp;</span></p>'."\r\n".'<p class="MsoNormal" style="margin-bottom: 7.5pt; line-height: 19.2pt; background: white;" data-mce-style="margin-bottom: 7.5pt; line-height: 19.2pt; background: white;"><span lang="EN-US" style="font-size: 12.0pt; font-family: "Calibri Light",sans-serif; mso-fareast-font-family: "Times New Roman"; color: #222222; mso-ansi-language: EN-US; mso-fareast-language: EN-IN;" data-mce-style="font-size: 12.0pt; font-family: "Calibri Light",sans-serif; mso-fareast-font-family: "Times New Roman"; color: #222222; mso-ansi-language: EN-US; mso-fareast-language: EN-IN;">', $request->content);
        }
       // $this->saveDB($letter, $request);
        $letter->name = $request->name;
        $letter->triggers = $request->triggers;
        $letter->letter_type = 2;
        $letter->content = $cc;
        $letter->save();
        $url = url($this->link);
        $error = false;
        $message = Str::singular(Str::title($this->title)).' added successfully';
        // return compact('message', 'url', 'error');
        return redirect('probate-letters')->with('msg_success', 'Letter added successfully.');
    }

    /**
     * @param $table
     * @param $request
     */
    public function saveDB($table, $request)
    {
        $table->name = $request->name;
        $table->triggers = $request->triggers;
        $table->content = $request->content;
        $table->letter_type = 2;

        $table->save();
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param mixed $id
     */
    public function edit($id)
    {
        $authUser = json_decode(Session::get(config('propai.authSessionKey')));
        $user_id = $authUser->id;

        $letter = Letter::where('user_id', $user_id)->find($id);

        if ($letter) {
            $page = collect();
            $page->title = $this->title;
            $page->link = url($this->link);
            $page->form_url = url($this->link.'/'.$letter->id);
            $page->form_method = 'PUT';
            $triggers = $this->trigger;

            $exists = Letter::where('user_id', $user_id)->where('letter_type',2)->distinct('triggers')->pluck('triggers')->toArray();

            if (false !== ($key = array_search($letter->triggers, $exists))) {
                unset($exists[$key]);
            }

            return view($this->viewPath.'/manage', compact('page', 'letter', 'triggers', 'exists'));
        }

        return redirect('/probate-letters')->with('msg_error', 'Cannot find any letter with this ID.');
    }

    /**
     * Update the specified resource in storage.
     *
     * @param $id
     *
     * @return array
     */
    public function update(LetterRequest $request, $id)
    {
        $letter = Letter::find($id);
         if(isset($request->content) && $request->content != '')
        {

           $cc = str_replace('<br />', '&nbsp;</span></p>'."\r\n".'<p class="MsoNormal" style="margin-bottom: 7.5pt; line-height: 19.2pt; background: white;" data-mce-style="margin-bottom: 7.5pt; line-height: 19.2pt; background: white;"><span lang="EN-US" style="font-size: 12.0pt; font-family: "Calibri Light",sans-serif; mso-fareast-font-family: "Times New Roman"; color: #222222; mso-ansi-language: EN-US; mso-fareast-language: EN-IN;" data-mce-style="font-size: 12.0pt; font-family: "Calibri Light",sans-serif; mso-fareast-font-family: "Times New Roman"; color: #222222; mso-ansi-language: EN-US; mso-fareast-language: EN-IN;">', $request->content);
        }
      // dd($request);
       // $this->saveDB($letter, $request);

        $letter->name = $request->name;
        $letter->triggers = $request->triggers;
        $letter->letter_type = 2;
        $letter->content = $cc;
        $letter->save();

        $url = url($this->link);
        $error = false;
        $message = Str::singular(Str::title($this->title)).' updated successfully';
        // return compact('error', 'message', 'url');
        return redirect('probate-letters')->with('msg_success', 'Letter updated successfully.');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param $id
     */
    public function destroy($id): array
    {
        $authUser = json_decode(Session::get(config('propai.authSessionKey')));
        $user_id = $authUser->id;

        $letter = Letter::where('user_id', $user_id)->find($id);

        if ($letter) {
            Letter::destroy($id);
            $error = false;
            $message = Str::singular(Str::title($this->title)).' deleted successfully';
            $color='#25E9A5';

            return compact('error', 'message', 'color');
        }
            $error = true;
            $message = 'Something went wrong!!';
            $color='red';

            return compact('error', 'message', 'color');            
    }
}
