<?php

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Model\UserValuation;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Session;
use Redirect;
use Validator;

class ValuationController extends Controller
{
    public function import(Request $request)
    {
        $user = json_decode(Session::get(config('propai.authSessionKey')));
        $page_title = "Import Valuation";

        return view('valuation.import', compact('page_title'));
    }
    public function importCreate(Request $request)
    {
        $rules = array(
            'name' => 'required',
            'address' => 'required',
            'phone_number' => ['required', 'regex:/^(((\+44\s?\d{4}|\(?0\d{4}\)?)\s?\d{3}\s?\d{3})|((\+44\s?\d{3}|\(?0\d{3}\)?)\s?\d{3}\s?\d{4})|((\+44\s?\d{2}|\(?0\d{2}\)?)\s?\d{4}\s?\d{4}))(\s?\#(\d{4}|\d{3}))?$/'],
            'valuation_date' => 'required'
            
        );
        $validator = validator::make($request->all(), $rules);
        if ($validator->fails()) {
            return Redirect::back()->with('msg_error', $validator->errors()->first())->withInput();
        } else {

            $user = json_decode(Session::get(config('propai.authSessionKey')));
            $path = '';
            if (!empty($request->file('file'))) {
                $path = $request->file('file')->store('public/valuations');
            }
            UserValuation::create([

                'user_id' => $user->id,
                'file' => $path,
                'name' => $request->name,
                'address' => $request->address,
                'phone_number' => $request->phone_number,
                'valuation_date' => date('Y-m-d', strtotime($request->valuation_date)),
            ]);
            return Redirect::route('recentValuation')->with('msg_success', "Valuation has been created successfully...!");
        }

    }

    public function recentValuation()
    {
        $page_title = "Recent Valuation";

        // dd($query->get());
        return view('valuation.recent_valuation', compact('page_title'));
    }

    public function allValuationArray(Request $request)
    {

        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $query = UserValuation::select('id', 'name', 'address', 'phone_number', 'valuation_date', 'file')->where('user_id', $user_id);

        $primaryKey = 'id';

        $columns = array(
            0 => 'id',
            1 => 'name',
            2 => 'address',
            3 => 'phone_number',
            4 => 'valuation_date',
            5 => 'action',

        );
        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;
        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {

            $search = $request->input('search.value');

            $list = $query->Where('user_valuations.name', 'like', '%' . $search . '%')
                ->orderBy($order, $dir)
                ->offset($start)
                ->limit($limit)
                ->get();

            $totalFiltered = $query->Where('user_valuations.name', 'like', '%' . $search . '%')
                ->count();

        }
        $dataArr = array();
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                //$url = route('agentDetails',['agent_id'=>$encrpt_id]);
                $i++;
                $nestedData['id'] = $item->id;
                $nestedData['sl'] = $i;
                $nestedData['name'] = $item->name;
                $nestedData['address'] = $item->address;
                $nestedData['phone_number'] = $item->phone_number;
                $nestedData['valuation_date'] = date('d/m/Y', strtotime($item->valuation_date));

                $nestedData['file'] =
                    "<a href='$item->file' target='preview' >Preview</a>";

                $nestedData['action'] = $item->id;

                $dataArr[] = $nestedData;
            }
        }
        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $dataArr,
        );

        return json_encode($json_data);
    }

    public function editValuation($id)
    {
        $page_title = "Edit Valuation";
        if ($id) {
            $valuation = UserValuation::find($id);

            if ($valuation) {
                return view('valuation.valuation_edit', compact('page_title', 'valuation'));
            }
            return $this->simpleReturn('error', 'Something Went Wrong...!');
        }
        return $this->simpleReturn('error', 'Something Went Wrong...!');
    }

    public function updateValuation(Request $request, $id)
    {
        $rules = array(
            'name' => 'required',
            'address' => 'required',
            'phone_number' => ['required', 'regex:/^(((\+44\s?\d{4}|\(?0\d{4}\)?)\s?\d{3}\s?\d{3})|((\+44\s?\d{3}|\(?0\d{3}\)?)\s?\d{3}\s?\d{4})|((\+44\s?\d{2}|\(?0\d{2}\)?)\s?\d{4}\s?\d{4}))(\s?\#(\d{4}|\d{3}))?$/'],
            'valuation_date' => 'required'
        );

        $validator = validator::make($request->all(), $rules);
        if ($validator->fails()) {
            return Redirect::back()->with('msg_error', $validator->errors()->first())->withInput();
        }
        $path = '';
        if (!empty($request->file('file'))) {
            $path = $request->file('file')->store('public/valuations');
        }
        if ($id) {
            $valuation = UserValuation::find($id);
            if ($valuation) {
                $valuation->name = $request->name;
                $valuation->address = $request->address;
                $valuation->phone_number = $request->phone_number;
                $valuation->file = $path;
                $valuation->valuation_date = date('Y-m-d', strtotime($request->valuation_date));
                if ($valuation->save()) {
                    $valuation = $valuation->id;

                    return Redirect::route('recentValuation')->with('msg_success', "Valuation has been updated successfully...!");
                } else {
                    return $this->simpleReturn('error', 'Something Went Wrong...!');
                }
            }
            return $this->simpleReturn('error', 'Something Went Wrong...!');
        }
        return $this->simpleReturn('error', 'Something Went Wrong...!');

    }

    public function deleteValuation(Request $request)
    {
        foreach ($request->property as $id) {

            $exist_data = UserValuation::where('id', $id)->first();

            if ($exist_data) {
                $exist_data->delete();
            } else { //insert
                return $this->simpleReturn('error', 'Something went wrong...!');
            }
        }
        return $this->simpleReturn('success', 'Successfully valuation deleted...!');
    }

}
