<?php

namespace App\Model;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class SubmittedProperty extends Model
{
    use SoftDeletes;
    protected $table = 'submitted_property';

    public static function getByDate($request, $criteria)
    {
        $user_id = $request->userid ?? '';
        return self::when($user_id, function ($query, $user_id) {
            return $query->where('user_id', $user_id);
        })
            ->when($criteria == 'today', function ($query) {
                $today = Carbon::now()->format('Y-m-d');
                return $query->whereDate('submitted_date', $today);
            })
            ->when($criteria == 'month', function ($query) {
                $month = Carbon::now()->format('m');
                $year = Carbon::now()->format('Y');
                return $query->whereMonth('submitted_date', $month)
                    ->whereYear('submitted_date', $year);
            })
            ->when($criteria == 'year', function ($query) {
                $year = Carbon::now()->format('Y');
                return $query->whereYear('submitted_date', $year);
            })
            ->when($criteria == 'week', function ($query) {
                $start_date = Carbon::now()->format('Y-m-d');
                $end_date = Carbon::now()->subWeek()->format('Y-m-d');
                return $query->whereBetween('submitted_date', [$start_date, $end_date]);
            })
            ->when($criteria == 'range' && $request->date_from, function ($query) use ($request) {
                $start_date = Carbon::createFromFormat('d-m-Y', $request->date_from)->format('Y-m-d');
                if ($request->date_to != null) {
                    $end_date = Carbon::createFromFormat('d-m-Y', $request->date_to)->format('Y-m-d');
                } else {
                    $end_date = Carbon::now()->format('Y-m-d');
                }
                return $query->whereBetween('submitted_date', [$start_date, $end_date]);
            })
            ->count('property_history_id');
    }

//    }    public static function getByDate($request, $criteria)
//    {
//        $user_id = $request->userid ?? '';
//        $post_code = $request->post_code ?? '';
//        $platform = $request->platform ?? '';
//        $agency = $request->agency ?? '';
//        $type = $request->type ?? '';
//        $query = SubmittedProperty::join('property_history', 'property_history.id', 'submitted_property.property_history_id')
//            ->join("property_house_number", "property_history.id", "property_house_number.property_history_id")
//            ->join("house_number", "house_number.id", "property_house_number.house_number_id")
//            ->join("postcode", "postcode.id", "house_number.postcode_id")
//            ->join("outcode", "outcode.id", "postcode.outcode_id")
//            ->join("agency", "agency.id", "property_history.agency_id")
//            ->select(DB::raw("GROUP_CONCAT(DISTINCT agency.name SEPARATOR ', ') AS agency_name"), 'property_history.address', 'property_history.property_type', "house_number.latest_price", "house_number.latest_status", 'property_history.listed_date', 'house_number.house_number', 'postcode.postcode', 'outcode.outcode', 'submitted_property.type', 'submitted_property.submitted_date')
//            ->groupBy('submitted_property.property_history_id', 'property_history.address', 'property_history.property_type', "house_number.latest_price", "house_number.latest_status", 'property_history.listed_date', 'house_number.house_number', 'postcode.postcode', 'outcode.outcode', 'submitted_property.type', 'submitted_property.submitted_date', 'property_house_number.house_number_id');
//
//        $data = self::join('property_history as ph', 'ph.id', 'submitted_property.property_history_id')
//            ->join("property_house_number as phn", "ph.id", "phn.property_history_id")
//            ->join("house_number as hn", "hn.id", "phn.house_number_id")
//            ->join("postcode as p", "p.id", "hn.postcode_id")
//            ->join("outcode as o", "o.id", "p.outcode_id")
//            ->join("agency as a", "a.id", "ph.agency_id")
//            ->when($criteria == 'today', function ($query) {
//                $today = Carbon::now()->format('Y-m-d');
//                return $query->whereDate('submitted_date', $today);
//            })
//            ->when($criteria == 'month', function ($query) {
//                $month = Carbon::now()->format('m');
//                $year = Carbon::now()->format('Y');
//                return $query->whereMonth('submitted_date', $month)
//                    ->whereYear('submitted_date', $year);
//            })
//            ->when($criteria == 'year', function ($query) {
//                $year = Carbon::now()->format('Y');
//                return $query->whereYear('submitted_date', $year);
//            })
//            ->when($criteria == 'week', function ($query) {
//                $start_date = Carbon::now()->format('Y-m-d');
//                $end_date = Carbon::now()->subWeek()->format('Y-m-d');
//                return $query->whereBetween('submitted_date', [$start_date, $end_date]);
//            })
//            ->when($criteria == 'range' && $request->date_from, function ($query) use ($request) {
//                $start_date = Carbon::createFromFormat('d-m-Y', $request->date_from)->format('Y-m-d');
//                if ($request->date_to != null) {
//                    $end_date = Carbon::createFromFormat('d-m-Y', $request->date_to)->format('Y-m-d');
//                } else {
//                    $end_date = Carbon::now()->format('Y-m-d');
//                }
//                return $query->whereBetween('submitted_date', [$start_date, $end_date]);
//            })
//            ->when($user_id, function ($query, $user_id) {
//                return $query->where('submitted_property.user_id', $user_id);
//            })
//            ->when($post_code, function ($query, $post_code) {
//                return $query->where('p.id', $post_code);
//            })
//            ->when($platform, function ($query, $platform) {
//                return $query->where('ph.portal', $platform);
//            })
//            ->when($agency, function ($query, $agency) {
//                return $query->where('a.id', $agency);
//            })
//            ->when($type, function ($query, $type) {
//                return $query->where('ph.property_type', $type);
//            })
//            ->groupBy('submitted_property.property_history_id')
//            ->get('submitted_property.property_history_id');
//        return count($data);
//    }


    public static function getByToday($request)
    {
        $user_id = $request->userid;
        return self::when($user_id, function ($query, $user_id) {
            return $query->where('user_id', $user_id);
        })
            ->whereDate('submitted_date', $request->start_date)
            ->count('property_history_id');
    }

    public static function getByMonth($request)
    {
        $month = Carbon::now()->format('m');
        $year = Carbon::now()->format('Y');

        $user_id = $request->userid;
        return self::when($user_id, function ($query, $user_id) {
            return $query->where('user_id', $user_id);
        })
            ->whereMonth('submitted_date', $month)
            ->whereYear('submitted_date', $year)
            ->count('property_history_id');
    }

    public static function getByYear($request)
    {
        $year = Carbon::now()->format('Y');

        $user_id = $request->userid;
        return self::when($user_id, function ($query, $user_id) {
            return $query->where('user_id', $user_id);
        })
            ->whereYear('submitted_date', $year)
            ->count('property_history_id');
    }

    public static function getByRange($request)
    {
        $user_id = $request->userid;
        return self::when($user_id, function ($query, $user_id) {
            return $query->where('user_id', $user_id);
        })
            ->whereBetween('submitted_date', [$request->start_date, $request->end_date])
            ->count('property_history_id');
    }
}
