<?php

// <summary>
// File : PropertyController
// File Type : Controller
// Creator : Susmi Sidharthan
// Email: susmi@designdirect@gmail.com
// Company: Design Direct UK
// Description ; This is used as web controller for all estate agent Property controller.
// </summary>

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Model\Agency;
use App\Model\CampaignLetterHistory;
use App\Model\HouseNumber;
use App\Model\Outcode;
// use Symfony\Component\HttpFoundation\StreamedResponse;
use App\Model\ProbateProperty;
use App\Model\PostCode;
use App\Model\PropertyCampaign;
use App\Model\PropertyComment;
use App\Model\PropertyFlag;
use App\Model\PropertyHistory;
use App\Model\PropertyLetter;
use App\Model\PropertyPriceHistory;
use App\Model\UserNotification;
use App\Model\UserOutcode;
use App\Model\SoldHistory;
use App\Model\User;
use App\Model\School;
use App\Model\SchoolCoordinates;
use Carbon\Carbon;
use GuzzleHttp\Client;
use Yajra\Datatables\Datatables;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\DB;
use Redirect;
use Validator;
use Crypt;


class PropertyController extends Controller
{
    public $trigger = [
        0 => 'General Letters',
        1 => 'New to Market',
        2 => 'For sale',
        3 => 'Sold',
        4 => 'Withdrawn',
        5 => 'Price drop',
        6 => 'Fallen Through',
        7 => '8 Week',
        8 => '16 Week',
        9 => '22 Week',
    ];

    //All property
    public function allProperty(Request $request)
    {
        $user_id            = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title         = 'Property Listings';
        $active             = 2;
        $user_type          = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $post_code          = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')
        ->where('user_id', $user_id)
        ->join('postcode', 'outcode.id', 'postcode.outcode_id')
        ->where('user_outcode.status', 1)
        ->where('outcode.archive', 0)
        ->select('postcode.*')
        ->orderBy('postcode.postcode', 'asc')
        ->get();
        // $property           = PropertyHistory::select('property_type', 'id')->get()->unique('property_type');
        // $property           = $this->commonPropQuery()->select('property_type')->get()->unique('property_type');
        $property           = $this->commonFreshPropQuery()->select('property_history.property_type')->get()->unique('property_type');
        $prop_count         = count($property);
        $agency             = Agency::get();
        $userOutcodes       = $this->userOutcodes();
        $campaign           = 1;
        $filter             = true;
        $enable_export      = User::find($user_id)->enable_export;
        $weeks              = '';
        $search_data        = '';
        $filter_title       = 'All Property Listings(Without Filters)';
        // Filter based on weeks
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                if ($weeks == 1) {
                    $filter_title = 'Property Filtered in 0-4 Weeks';
                } elseif ($weeks == 2) {
                    $filter_title = 'Property Filtered in 4-8 Weeks';
                } elseif ($weeks == 3) {
                    $filter_title = 'Property Filtered in 8-12 Weeks';
                } elseif ($weeks == 4) {
                    $filter_title = 'Property Filtered in 12-16 Weeks';
                } elseif ($weeks == 5) {
                    $filter_title = 'Property Filtered in 16 Weeks or More';
                }
            }
            $search_data        = json_decode(Session::get('search_data'));
            $search_data->weeks = $weeks;
        }

        return view('all-property', compact('page_title', 'filter_title', 'property', 'prop_count', 'post_code', 'userOutcodes', 'user_type', 'active', 'campaign', 'agency', 'filter','search_data','enable_export'));
    }

    public function allPropertyArray(Request $request)
    {
        $sesstion_array =[];
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $user_outcode = $this->userOutcode();
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                $query = $this->commonFreshPropQuery();
                // $query = $query->whereNotIn('property_history.for_sale', [3, 6]);
                // $query = $query->whereNotIn('property_price_history.type', ['Withdrawn', 'STC']);
            }
        } else {
            $query = $this->commonPropQuery();
        }
        $query = $query->whereIn('outcode.id', $user_outcode);
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
            $sesstion_array['outcode'] = $request->input('outcode');
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
                $sesstion_array['post_code'] = $request->input('post_code');

            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
                $sesstion_array['type'] = $request->input('type');
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
                $query->where('property_flag.user_id', $user_id);
                $sesstion_array['flag'] = $request->input('flag');
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
                $sesstion_array['agency'] = $request->input('agency');
            }
        }
        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
                $sesstion_array['platform'] = $request->input('platform');
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('weeks')) {
            $sesstion_array['weeks'] = $request->input('weeks');
            $weeks = $request->weeks;
            $dt = Carbon::now();

            if (in_array($weeks, [1, 2, 3, 4])) {
                if (1 == $weeks) {
                    $start = $dt->copy()->subWeeks(4)->addDays(1)->startOfDay();
                    $end = $dt->copy()->endOfDay();
                } elseif (2 == $weeks) {
                    $start = $dt->copy()->subWeeks(8)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(4)->endOfDay();
                } elseif (3 == $weeks) {
                    $start = $dt->copy()->subWeeks(12)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(8)->endOfDay();
                } elseif (4 == $weeks) {
                    $start = $dt->copy()->subWeeks(16)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(12)->endOfDay();
                }
                $query->whereBetween('house_number.listed_date', [$start, $end]);
            } elseif (5 == $weeks) {
                $query->whereDate('house_number.listed_date', '<=', $dt->copy()->subWeeks(16)->endOfDay());
            }
        }
        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode.outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'agency.name',
            6 => 'property_type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');
            $sesstion_array['search'] = $search;
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%');

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }

            })->orderBy($order, $dir)->get()->unique('house_number_id');
            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }
            });
        }

        $dataArr = [];


        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                //   $urls = $r_count.$z_count;
                //   $urls = '<div class=" d-flex"> <a class="btn btn-primary  btn-xs  text-center"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">3</span>
                //    <a class="btn btn-success  btn-xs" href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">3</span></div>
                //   ';
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                // if (!empty($item->house_number)) {
                $address2 = $item->house_number.', '.$item->address;
                // } else {
                // $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                // if (!empty($item->created_by)) {
                //     $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $st = '';
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                // $nestedData['type'] = $request->outcodeFilter.'--'. $request->input('search.value');
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }
        
        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        $session_data =  json_encode( $sesstion_array);
        Session::put('search_data', $session_data );

        return json_encode($json_data);
    }

    //property flag change
    public function flagChange(Request $request)
    {
        // return $request->all();

        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ('' != $exist_data) { //update
                // $update = PropertyFlag::where('id',$exist_data['id'])
                //     ->update(['flag' => $request->flag]);
                $update = PropertyFlag::find($exist_data['id']);
                $update->fill(['flag' => $request->flag])->save();

                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    //property detail page
    public function propertyDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode.');
            }
        }

        $prop = true;
        $house_number_id = Helper::decryptData($request->property_id);
        $query = $this->commonPropQuery();
        $property_details = $query->where('house_number.id', $house_number_id)->first();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $page_title = $property_details->address;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
        ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('house_number_id', $house_number_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
        ->orderBy('property_price_history.event_date', 'desc')->orderBy('property_price_history.id', 'desc')->get();
        // dd($listing_activity);
        $ambiguity_checker_response = $this->propertyAmbiguityChecker(['data_type' => 'listing_activity_object', 'data' => $listing_activity]);

        $comments = Helper::propertyComment($house_number_id);

        $letter_list = PropertyLetter::join('house_number', 'property_letters.house_number_id', 'house_number.id')
        ->join('letters', 'letters.id', 'property_letters.letter_id')
        ->where('property_letters.house_number_id', $house_number_id)
        ->select('property_letters.*', 'letters.name as letter_name')
        ->get()
        ;

        $current_flag = Helper::propertyFlag($house_number_id);

        $portal_url = $this->houseNumberPortal($house_number_id);
        $campaign_history = PropertyCampaign::where('house_number_id', $house_number_id)
        ->whereHas('campaign')
        ->with('campaign')
        ->withTrashed()
        ->where('user_id', $user_id)
        ->orderBy('id', 'desc')
        ->get()
        ;
        $trigger = $this->trigger;

        return view('property-details', compact('property_details', 'listing_activity', 'ambiguity_checker_response', 'comments', 'letter_list', 'house_number_id', 'prop', 'current_flag', 'portal_url', 'house_number_id', 'campaign_history', 'trigger'));
    }

    //property history details
    public function propertyHistoryDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode');
            }
        }

        $prop = true;
        $property_history_id = Helper::decryptData($request->property_id);

        // dd($property_history_id);

        $property_details = PropertyHistory::join('postcode', 'property_history.postcode_id', 'postcode.id')
        ->where('property_history.id', $property_history_id)
        ->select('property_history.*', 'postcode.postcode as p_code')
        ->first()
        ;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->where('property_history.id', $property_history_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id')
        ->orderBy('property_price_history.event_date', 'desc')->get();

        // dd($listing_activity);

        // $comments = Helper::propertyComment($house_number_id);
        $comments = [];

        // $letter_list = PropertyLetter::join('house_number','property_letters.house_number_id','house_number.id')
        //     ->join('letters','letters.id','property_letters.letter_id')
        //     ->where('property_letters.house_number_id',$house_number_id)
        //     ->select('property_letters.*','letters.name as letter_name')
        //     ->get();

        $letter_list = [];

        $portal_url[0] = ['portal' => $property_details->portal, 'portal_url' => $property_details->portal_url];

        return view('property-history-details', compact('property_details', 'listing_activity', 'comments', 'letter_list', 'prop', 'portal_url'));
    }

    // Check if ambiguous data is provided by the system
    public function propertyAmbiguityChecker($data)
    {
        $data_type = isset($data['data_type']) ? $data['data_type'] : '';
        if ($data_type === 'listing_activity_object') {
            $listing_activity   = isset($data['data']) ? $data['data'] : '';
        } else if ($data_type === 'house_number_id') {
            $house_number_id    = isset($data['data']) ? $data['data'] : '';
            if ($house_number_id != '') {
                $listing_activity   = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
                ->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
                ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
                ->join('agency', 'agency.id', 'property_history.agency_id')
                ->where('house_number_id', $house_number_id)
                ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
                ->orderBy('property_price_history.event_date', 'desc')
                ->orderBy('property_price_history.id', 'desc')
                ->get();
            }
        }
        if (!empty($listing_activity) && count($listing_activity) > 0) {
            // Convert listing fetch to array
            $listing_activity_array = $listing_activity->toArray();
            // prepare portal to agent array
            $list_activity_portal_array         = array_column($listing_activity_array, 'portal');
            $list_activity_agent_array          = array_column($listing_activity_array, 'name');
            $list_activity_portal_agent_array   = [];
            $i = 0;
            // Prepare an event date array out of listing activity with correct position keys
            foreach ($listing_activity_array as $list_key => $list_value) {
                if (isset($listing_activity_array[$i]['event_date']) && $listing_activity_array[$i]['event_date'] != '') {
                    $list_activity_array[$i] = $listing_activity_array[$i]['event_date'];
                }
                $list_activity_portal_agent_array[] = $list_activity_portal_array[$i]." - ".$list_activity_agent_array[$i];
                $i++;
            }
            $unique_list_activity               = array_unique($list_activity_portal_agent_array);
            $unique_agent_activity_count        = count($unique_list_activity);
            // Preparing activity_listing_position_array for comparing in pattern
            $temp_listing_activity_array        = $list_activity_array;
            $count_list_activity_array          = count($list_activity_array);
            $count_activity                     = array_count_values($list_activity_array);
            $total_count_activity               = count($count_activity);
            $cycle_count                        = 0;
            $activity_listing_position_array    = [];
            foreach ($count_activity as $count_activity_key => $count_activity_value) {
                if ($count_activity_value > 1) {
                    $cycle_count += $count_activity_value;
                }
            }
            $temp_cycle_count                   = $cycle_count;
            if ($cycle_count > 0) {
                while ((count($temp_listing_activity_array) >= $cycle_count) && $cycle_count !== 0) {
                    foreach ($count_activity as $count_activity_key => $count_activity_value) {
                        if ($count_activity_value > 1) {
                            $activity_listing_position = '';
                            $activity_listing_position = array_search($count_activity_key, $temp_listing_activity_array);
                            if ($activity_listing_position !== false) {
                                $activity_listing_position_array[$activity_listing_position] = $count_activity_key;
                                unset($temp_listing_activity_array[$activity_listing_position]);
                            }
                        }
                    }
                    $cycle_count--;
                }
                // Sort in DESC order of activity date
                ksort($activity_listing_position_array);
            }
            // ambiguous listing checks
            $ambiguous_listing_flag             = false;
            $type_unfavourable_array            = ['Withdrawn', 'STC', 'Sold'];
            $first_listing_activity_array       = array_key_first($listing_activity_array);
            // $type_unfavourable_array            = ['Withdrawn'];
            if (!empty($activity_listing_position_array)) {
                $total_activity_listing_position_array = count($activity_listing_position_array);
                foreach ($activity_listing_position_array as $alpa_key => $alpa_value) {
                    $next_activity_listing = next($activity_listing_position_array);
                    // pattern checking for ambiguous data provided for a listing through system
                    // Pattern 1
                    if (isset($listing_activity_array[$alpa_key]['type'])
                        && isset($listing_activity_array[$alpa_key+1]['type'])
                        && ($listing_activity_array[$alpa_key]['type'] != $listing_activity_array[$alpa_key+1]['type']) // check if event type is different
                        && ($next_activity_listing !== false) // check if next item available
                        && (in_array($listing_activity_array[$alpa_key]['type'], $type_unfavourable_array)) // check if type is not offer or price related
                        // && ($listing_activity_array[$alpa_key]['portal'] === $listing_activity_array[$alpa_key+1]['portal']) // check if same portal provides the data
                        && ($listing_activity_array[$alpa_key]['event_date'] === $listing_activity_array[$alpa_key+1]['event_date'])) // check if event date is same
                        // && ($listing_activity_array[$alpa_key]['name'] !== $listing_activity_array[$alpa_key+1]['name'])) // check if portal agent is different
                    {
                        $ambiguous_listing_flag = true;
                    }
                }
                // Pattern 2
                if (isset($listing_activity_array[$first_listing_activity_array]['type'])
                    && (in_array($listing_activity_array[$first_listing_activity_array]['type'], $type_unfavourable_array)) // check if first listing type is unfavourable
                    && ($unique_agent_activity_count > 1)) // check if first listing type is unfavourable
                {
                    $ambiguous_listing_flag = true;
                }
            }
            // debuging unit for ambiguous_listing_flag listing data
            // $list_activity_array            = array_column($listing_activity_array, 'event_date');
            // $list_activity_array_filtered   = array_filter($list_activity_array);
            // print("<pre>".print_r($listing_activity_array,true)."</pre>");
            /*print("<pre>".print_r($list_activity_array,true)."</pre>");
            echo "total activity count : $count_list_activity_array";
            print("<pre>".print_r($count_activity,true)."</pre>");
            echo "unique activity count : $total_count_activity<br>";
            print("<pre>".print_r($temp_listing_activity_array,true)."</pre>");
            echo "cycle count : $temp_cycle_count<br>";
            echo "cycle count now: $cycle_count<br>";
            print("<pre>".print_r($activity_listing_position_array,true)."</pre>");*/
            // echo $ambiguous_listing_flag;
            // dd('stopped');
            return $ambiguous_listing_flag;
        }
    }

    //comment delete
    public function deletePropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }

        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $update = PropertyComment::where('id', $request->comment)->delete();
        if ($update) {
            return $this->simpleReturn('success', 'Successfully comment deleted..!');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    //add property comment
    public function addPropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('error', $validator->errors()->first());
        }
        
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $add = new PropertyComment();
        $add->house_number_id = $request->property;
        $add->comment = $request->comment;
        $add->user_id = $user_id;
        if ($add->save()) {
            return Redirect::back()->with('success', 'Successfully comment deleted..!');
        }

        return Redirect::back()->with('error', 'Something went wrong...!');
    }

    //search property
    public function searchProperty(Request $request)
    {
        $search_keyword = $request->search;
        $page_title = 'Addresses or Agencies That Contain: '.$search_keyword;
        session()->put('search_keyword', $search_keyword);

        return view('search-result', compact('page_title'));
    }

    //search property array
    public function searchPropertyArray(Request $request)
    {
        $search_keyword = session()->get('search_keyword');
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        // ->Where('property_history.address', 'like', '%' . $search_keyword . '%')
        // ->orWhere('agency.name', 'like', '%' . $search_keyword . '%')
        // ->orWhere('postcode.postcode', 'like', '%' . $search_keyword . '%');
        // ->orWhere('property_history.property_type', 'like', '%' . $search_keyword . '%');

        $query = $query->where(function ($q) use ($search_keyword) {
            $q->Where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('property_history.property_type', 'like', '%'.$search_keyword.'%')
            ->orWhere('outcode.outcode', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
        });

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'agency',
            4 => 'listed_date',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //  $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)
            ->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()->unique('house_number_id');
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }

                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                //  $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    //new to market
    public function newToMarket()
    {
        $page_title = 'Weekly New to Market';
        $campaign = 1;

        return view('new_to_market', compact('page_title', 'campaign'));
    }

    public function newToMarketArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        $new_date_from = date('Y-m-d', strtotime('-7 days'));
        $new_date_to = date('Y-m-d');

        $query = $query->whereBetween('house_number.listed_date', [$new_date_from, $new_date_to]);

        // dd($items->where('house_number.id',1)->get());
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'price',
            4 => 'listed_date',
            5 => 'agency.name',
            6 => 'type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                $nestedData['outcode'] = $item->outcode;

                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }
                // $listed_date=date('Y-m-d',strtotime($item->listed_date));
                // $today=date('Y-m-d');
                // $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                // $nestedData['listed_date'] = date('d M Y',strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }
                // $nestedData['week'] = $item->week;
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //withdrawn
    public function withdrawn()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Withdrawn';

        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;
        $userOutcodes = $this->userOutcodes();

        return view('withdrawn-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function withdrawnArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        if (1 == $user_type) { //admin
            $data = HouseNumber::get();
        } elseif (2 == $user_type) { //agent
            $user_outcode = $this->userOutcode();
            $outcodes = $this->userOutcodes();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.latest_status', 3);
            // dd($items->where('house_number.id',1)->get());
            if ($request->has('outcode') && '' != $request->input('outcode')) {
                $query->where('outcode.outcode', $request->input('outcode'));
            }

            if ($request->has('post_code')) {
                if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                    $query->where('postcode.id', $request->input('post_code'));
                }
            }
            // dd($request->all());
            if ($request->has('type')) {
                if (('0' != $request->type && 'undefined' != $request->type)) {
                    $query->where('property_history.property_type', $request->input('type'));
                }
            }
            if ($request->has('flag')) {
                if ((0 != $request->flag && 'undefined' != $request->flag)) {
                    $query->where('property_flag.flag', $request->input('flag'));
                }
            }
            if ($request->has('agency')) {
                if ((0 != $request->agency && 'undefined' != $request->agency)) {
                    $query->where('agency.id', $request->input('agency'));
                }
            }
            if ($request->has('platform')) {
                if ((-1 != $request->platform && 'platform' != $request->platform)) {
                    $query->where('property_history.portal', $request->input('platform'));
                }
            }
            if ($request->has('date_from') && $request->has('date_to')) {
                if (('' != $request->date_from && '' != $request->date_to)) {
                    $date_from = date('Y-m-d', strtotime($request->date_from));
                    $date_to = date('Y-m-d', strtotime($request->date_to));
                    $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
                }
            }
        } else { //data entry operator
            $data = HouseNumber::get();
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'withdrawn_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $withdrwan_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item->house_number_id)
                ->where('property_price_history.type', 'Withdrawn')
                ->orderBy('property_price_history.event_date', 'desc')
                ->select('property_price_history.event_date')
                ->first()
                ;

                // $nestedData['withdrawn_date'] = date('d M Y', strtotime( $withdrwan_date->event_date
                $nestedData['withdrawn_date'] = date('d M Y', strtotime($item->withdrawn_date));

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function recentlySold()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Sold - Under Offer';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('recently-sold-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function recentlySoldArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        //print_r($userOutcodes); exit;

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode)
        ->where('house_number.latest_status', '!=', 3)
        ;

        $query = $query->where(function ($q) {
            $q->Where('property_history.sold_stc', 1)
            ->orWhere('property_history.under_offer', 1)
            ;
        });

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            // 5 =>'property_price_history.event_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');

        if (empty($request->input('search.value'))) {
            // $data = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
            // $dd = $query->orderBy($order,$dir)->get()->unique('house_number_id')->toArray();
            // $list = array_slice($data, $start, $limit);
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            //  $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';

                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item['house_number_id']);

                $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item['house_number_id'])
                ->where('property_price_history.type', 'STC')
                ->orderBy('property_price_history.id', 'desc')
                ->limit(1)
                ->select('property_price_history.event_date', 'property_price_history.sold_year')
                ->first()
                ;

                if (!$sold_date) {
                    $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                    ->where('property_house_number.house_number_id', $item['house_number_id'])
                    ->where('property_price_history.type', 'Offer')
                    ->orderBy('property_price_history.id', 'desc')
                    ->limit(1)
                    ->select('property_price_history.event_date', 'property_price_history.sold_year')
                    ->first()
                    ;
                }

                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;

                if ($sold_date) {
                    if ($sold_date->event_date) {
                        $so_date = date('d M Y', strtotime($sold_date->event_date));
                    } else {
                        $so_date = $sold_date->sold_year;
                    }
                } else {
                    $so_date = '';
                }

                $nestedData['sold_date'] = $so_date;

                if ($item['listed_date']) {
                    $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['id'] = $item['house_number_id'];
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item['outcode'];
                if (null == $item['created_by']) {
                    $st = 'By System';
                    $color = '';
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }

                if (!empty($item['house_number'])) {
                    $address2 = $item['house_number'].', '.$item['address'];
                } else {
                    $address2 = $item['house_number'].', '.$item['address'].',<br>  '.$item['postcode'];
                }
                if (!empty($item['created_by'])) {
                    $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                // $nestedData['price'] = "&pound; ".number_format($item['latest_price']);
                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                // $nestedData['sold_date'] =  date('d M Y',strtotime($item['sold_date']));
                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item['title'].' <br> '.ucfirst($item['property_type']);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function priceDrop()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Recent Price Drops        ';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $agency = Agency::get();
        $filter = true;
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('price-drop-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'agency', 'filter', 'totalProp'));
    }

    //price drop
    public function priceDropArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        // if($user_type==1){//admin
        //     $data = HouseNumber::get();

        // }elseif($user_type==2){//agent

        $user_outcode = $this->userOutcode();

        // $query = $this->commonPropQuery();
        // $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.price_change_diff', '<', 0);

        $query = HouseNumber::join('property_house_number', 'property_house_number.house_number_id', 'house_number.id')
        ->join('property_price_history', 'property_price_history.property_history_id', 'property_house_number.property_history_id')
        ->join('property_history', 'property_history.id', 'property_house_number.property_history_id')
        ->join('postcode', 'postcode.id', 'house_number.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_flag', 'property_flag.house_number_id', 'property_house_number.house_number_id')
        ->whereNull('property_house_number.deleted_at')
        ->whereIn('outcode.id', $user_outcode)
        ->Where('property_price_history.type', 'like', '%Price reduced by%')
        ->select('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', 'house_number.latest_price', 'house_number.latest_status', 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id', 'house_number.created_by', 'property_price_history.type', 'property_price_history.event_date')
        ->orderBy('property_price_history.event_date', 'desc')
        ;
        // ->groupBy('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', "house_number.latest_price", "house_number.latest_status", 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id,'property_price_history.type'');

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // dd($items->where('house_number.id',1)->get());

        // }else{//data entry operator
        //     $data = HouseNumber::get();
        // }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'price_change_diff',
            5 => 'house_number.listed_date',
            6 => 'price_drop_date',
            7 => 'agency.name',
            8 => 'property_type',
            9 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        $outcodes = $this->userOutcodes();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //$query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                // $address3=empty($item->title)?$address2:$address1;
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['price_drop'] = '-'.substr($item->type, 16);
                // '&pound;'.number_format($item->price_change_diff);
                $listed_date = date('Y-m-d', strtotime($item->listed_date));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);

                if ($item->listed_date) {
                    $dt = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $dt = 'Hidden from public';
                }

                $nestedData['listed_date'] = $dt;
                $nestedData['price_drop_date'] = date('d M Y', strtotime($item->event_date));
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //export property data
    public function exportPropertyData(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        $file_name = '';

        if ('all_property' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);
                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'All-property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('new_to_market' == $request->mode) {
            $user_outcode = $this->userOutcode();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.new_to_market', 1);
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'New-to-market'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('recent_sold' == $request->mode) {
            $query = $query->where('house_number.latest_status', '!=', 3);

            $data_list = $query->where(function ($q) {
                $q->Where('property_history.sold_stc', 1)->orWhere('property_history.under_offer', 1);
            })->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Recentl-sold'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('hot_list' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
        } elseif ('withdrawn' == $request->mode) {
            $data_list = $query->where('house_number.latest_status', 2)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Withdrawn Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['withdrawn_date'] = date('d-m-Y', strtotime($item->withdrawn_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Withdrawn-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('price_drop' == $request->mode) {
            $data_list = $query->where('house_number.price_change_diff', '<', 0)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['price_drop'] = number_format($item->price_change_diff);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['price_drop_date'] = date('d-m-Y', strtotime($item->price_drop_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Price-drop-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('all_type_property' == $request->mode) {
            $user_outcode = $this->userOutcode();
            $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
            $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();
            $list1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
            ->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
            ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
            ->where('property_house_number.house_number_id', null)
            ->where('property_house_number.property_history_id', null)
            ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title')->get()->toArray();

            $list2 = $this->commonPropQuery();
            $list2 = $list2->whereIn('outcode.id', $user_outcode)->get()->unique('house_number_id')->toArray();
            $data_list = array_merge($list1, $list2);

            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['price'] = number_format($item['latest_price']);
                $arr['listed_date'] = date('d-m-Y', strtotime($item['listed_date']));
                if ($item['house_number_id']) {
                    $arr['address'] = $item['house_number'].', '.$item['address'].', '.$item['postcode'];
                    $arr['agency'] = Helper::agencyName($item['house_number_id']);
                } else {
                    $arr['address'] = $item['address'].', '.$item['postcode'];
                    $arr['agency'] = $item['name'];
                }
                $arr['type'] = ucfirst($item['property_type']);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'all-property-data'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } else {
            $search_keyword = session()->get('search_keyword');
            $query = $query->where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
            $data_list = $query->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        }

        $url = url('storage/property-downloads/'.$file_name);
        $data['url'] = $url;
        $data['name'] = $file_name;

        return $data;
    }

    //Hotlist
    public function hotList()
    {
        $page_title = 'Hotlist';

        return view('hotlist', compact('page_title'));
    }

    public function hotListArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'listed_date',
            4 => 'agency',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;

            // $totalFiltered =$query->Where('property_history.address', 'like', '%' . $search . '%')
            // ->orWhere('agency.name', 'like', '%' . $search . '%')
            // ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            // ->count();
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                $nestedData['agency'] = $item->agency_name;
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['comments'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    public function filterByPostcode(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $post_code = $request->postCode;
        if ($post_code) {
        }

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ($exist_data) { //update
                $update = PropertyFlag::where('id', $exist_data['id'])
                ->update(['flag' => $request->flag])
                ;
                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    public function propertyTrigger($tab)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Property';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $active = 2;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $enable_export = User::find($user_id)->enable_export;
        $userOutcodes = $this->userOutcodes();

        if (1 == $tab) {
            $filter = true;
            $active = 1;
            $agency = Agency::get();
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');

            return view('property-all-type', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'agency', 'property', 'filter','enable_export'));
        }
        if (2 == $tab) {
            $campaign = 1;
            $active = 2;
            $filter = true;
            $agency = Agency::get();

            return view('all-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'active', 'user_type', 'filter', 'agency', 'campaign','enable_export'));
        }
        if (3 == $tab) {
            $active = 3;
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');
            $filter = true;
            $post_code = PostCode::orderBy('postcode', 'asc')->get()->pluck('postcode', 'id');
            return view('common.property.property-without-address', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'property', 'filter','enable_export'));
        }

        return view('all-property', compact('page_title', 'post_code', 'userOutcodes', 'active', 'user_type','enable_export'));
    }

    public function allTypeProperty()
    {
        $page_title = 'Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $active = 1;

        return view('admin.property-all-type', compact('page_title', 'post_code', 'user_type', 'active'));
    }

    public function allTypePropertyArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
        $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();

        $query1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
        ->where('property_house_number.house_number_id', null)
        ->where('property_house_number.property_history_id', null)
        ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title');

        // ->get()->toArray();

        $query2 = $this->commonPropQuery();
        $query2 = $query2->whereIn('outcode.id', $user_outcode);

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query1->where('outcode.outcode', $request->input('outcode'));
            $query2->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query1->where('postcode.id', $request->input('post_code'));
                $query2->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('platform')) {
            if (('-1' != $request->platform && 'undefined' != $request->platform)) {
                $query1->where('property_history.portal', $request->input('platform'));
                $query2->where('property_history.portal', $request->input('platform'));
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query1->whereBetween('property_history.listed_date', [$date_from, $date_to]);
                $query2->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query1->where('property_history.property_type', $request->input('type'));
                $query2->where('property_history.property_type', $request->input('type'));
            }
        }

        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query2->where('property_flag.flag', $request->input('flag'));
            }
        }

        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query1->where('agency.id', $request->input('agency'));
                $query2->where('agency.id', $request->input('agency'));
            }
        }
        // ->get()->unique('house_number_id')->toArray();

        // $all_data = array_merge($all, $withoutHouseNumber);

        // Table's primary key
        $primaryKey = 'id';
        // $outcodes = $this->userOutcodes();

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'latest_price',
            3 => 'listed_date',
            4 => 'agency_name',
            5 => 'property_type',
            6 => 'flag',
        ];

        $q_data1 = $query1->get()->toArray();
        $q_data2 = $query2->get()->unique('house_number_id')->toArray();
        $all_data = array_merge($q_data1, $q_data2);
        $totalData = count($all_data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        if (empty($request->input('search.value'))) {
            $list1 = $query1->orderBy($order, $dir)->get()->toArray();
            $list2 = $query2->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

            $data_val = array_merge($list1, $list2); //echo "<pre>"; print_r($data_val); exit;
        } else {
            $search = $request->input('search.value');

            $list1 = $query1->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->toArray()
            ;

            $list2 = $query2->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->unique('house_number_id')
            ->toArray()
            ;

            $data_val = array_merge($list1, $list2);

            // $totalFiltered = $query->where(function($q) use ($search) {
            //     $q->Where('property_history.address', 'like', '%' . $search . '%')
            //         ->orWhere('agency.name', 'like', '%' . $search . '%')
            //     ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            //     ->orWhere('outcode.outcode', 'like', '%' . $search . '%')
            //     ->orWhere('postcode.postcode', 'like', '%' . $search . '%');

            // })->count();
            $totalFiltered = count($data_val);
        }

        $dataArr = [];
        $outcodes = $this->userOutcodes();

        if (!empty($data_val)) {
            $data_val = array_slice($data_val, $start, $limit); // return the first five elements

            $i = 0;
            foreach ($data_val as $item) {
                $nestedData['outcode'] = $item['outcode'];
                $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                if ($item['listed_date']) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                // $nestedData['price'] = "&pound;" . number_format($item['latest_price']);

                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                $nestedData['type'] = ucfirst($item['title']).'<br> '.ucfirst($item['property_type']);

                if ($item['house_number_id']) {
                    //portal url
                    $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                    $platform = '';
                    $r = 0;
                    $z = 0;
                    $r_count = '';
                    $z_count = '';
                    if (count($portal_url)) {
                        $platform = '<div class=" d-flex">';
                        foreach ($portal_url as $url) {
                            // $r_count="";$z_count="";
                            if (0 == $url->portal) {
                                ++$r;
                            } else {
                                ++$z;
                            }
                        }
                        if ($r > 0) {
                            $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                        }
                        if ($z > 0) {
                            $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                        }
                        $platform .= $r_count.$z_count;
                        $platform .= '</div>';
                    }
                    $encrpt_id = Helper::encryptData($item['house_number_id']);
                    $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                    ++$i;
                    $nestedData['id'] = $item['house_number_id'];
                    $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                    if (!empty($item['house_number'])) {
                        $address2 = $item['house_number'].' , '.$item['address'];
                    } else {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    if (!empty($item['created_by'])) {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    // $address3=empty($item['title'])?$address2:$address1;
                    if (null == $item['created_by']) {
                        $st = 'By System';
                        $color = '';
                    } else {
                        $st = 'By Manual';
                        $color="style='color:#edf124'";
                    }
                    $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                    // $nestedData['price'] = "&pound; ".number_format($item['latest_price']) ;

                    $nestedData['type'] = $item['title'].'<br> '.ucfirst($item['property_type']);
                    $nestedData['portal_url'] = $platform;
                    $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                    $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
                    $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();
                    $dataArr[] = $nestedData;
                } else {


                    $encrpt_id = Helper::encryptData($item['id']);
                    $nestedData['id'] = '';

                    $url = route('propertyHistoryDetails', ['property_id' => $encrpt_id]);

                    $nestedData['property_number'] = '<div class="input-group">
                    <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                    <div class="input-group-append">
                    <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item['id'].'"><i class="flaticon-paper-plane icon-lg"></i></button>
                    </div>
                    </div>';

                    $nestedData['address'] = "<a  href='{$url}'>{$item['address']},  {$item['postcode']}</a>";

                    $nestedData['agency'] = $item['name'];

                    if (0 == $item['portal']) {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">1</span></div>';
                    } else {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">1</span></div>';
                    }

                    $nestedData['flag'] = 0;
                    $nestedData['letter_count'] = 0;
                    ++$i;

                    $dataArr[] = $nestedData;
                }
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutPostcode(Request $request)
    {
        $page_title = 'Without Postcode Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;
        $without_data = true;

        return view('property-without-postcode', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutPostcodeArray(Request $request)
    {
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('postcode.postcode', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;
        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;
                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                // $nestedData['type'] = $totalFiltered;
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutAgent(Request $request)
    {
        $page_title = 'Without Agent Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;

        $without_data = true;

        return view('property-without-agent', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutAgentArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('agency.name', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;

        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;

                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function getProbateData()
    {
     //$user_outcode = $this->userOutcode();
        if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
            $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
            $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
        }
        $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
        $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
        $user_outcode = $selOutcodeCollection;

        $query = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
        ->join("postcode", "postcode.id", "probate_property.postcode_id")
        ->join("outcode", "outcode.id", "postcode.outcode_id")
        ->leftjoin("probate_property_flag", "probate_property_flag.probate_id", "probate_property.id")
        ->whereIn('outcode.id', $user_outcode)
        ->where('probate_property.status',1)
        ->whereDate('publication_date', '>', Carbon::now()->subDays(530));
        $data = $query
        ->select('postcode.postcode','outcode.outcode','probate_property.address_1','probate_property.address_2','probate_property.firstname','probate_property.surname','executor.company_name','executor.first_name','executor.sur_name','probate_property.publication_date','probate_property.claim_date','probate_property.id')
        ->get();


    return Datatables::of($data)

  ->editColumn('outcode', function ($model) {
   return $model->postcode;
 })
  ->editColumn('address', function ($model) {
$pid = Crypt::encrypt($model->id);
    return '<a href="' . url('property-detail?pid=' . $pid ) . '" title="View">'.$model->address_1.' '.$model->address_2.'</a>';
 })  
      ->editColumn('executer', function ($model) {
    $executor = $model->company_name ? $model->company_name : $model->first_name." ".$model->sur_name;
   return $executor;
  })
  ->editColumn('publication_date', function ($model) {
    $publicationDate =  date('d-m-Y', strtotime($model->publication_date));  
   return $publicationDate;
 })
   ->editColumn('claim_date', function ($model) {
    $claim_date =  date('d-m-Y', strtotime($model->claim_date));  
   return $claim_date;
 })
  

->editColumn('created_date', function ($model) {
    return 1;
})


  ->editColumn('action', function ($model) {
    $idG = Crypt::encrypt($model->id);
    // $user = \App\User::find($model->user_id);
    // Auth::loginUsingId($user->id, true);

            return '<a href="' . url('boat-for-sale/' . $model->slug ) . '" type="" target="_blank" class="act-sp" title="View"><i class="fas fa-file"></i></a>  <a href="' . url('edit-boat-info/' . $idG) . '" type="" target="_blank" class="act-sp" title="Login as user to edit"><i class="fas fa-edit"></i></a> ';
        })

        ->rawColumns(['s#','name','address','status','expiry','action'])
        ->make(true);
    }

    public function propertyDetail(Request $request)
    {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::where('status',1)->findOrFail($propId); 
            if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
                $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
                $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
            }
            $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
            $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
            $user_outcode = $selOutcodeCollection;
        // $probateProperties = ProbateProperty::where('status',1)->orderBy('id', 'DESC')->take(10)->get(['address_1','latitude','longitude','id'])->toArray();
            $probateProperties = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
            ->join("postcode", "postcode.id", "probate_property.postcode_id")
            ->join("outcode", "outcode.id", "postcode.outcode_id")
            ->whereIn('outcode.id', $user_outcode)
            ->where('probate_property.status',1)->orderBy('probate_property.id', 'DESC')->take(10)->get(['probate_property.address_1','probate_property.latitude','probate_property.longitude','probate_property.id'])->toArray();
            return view('admin.probate-property-detail')->with(['probateProperty' => $probateProperty,'probateProperties' => $probateProperties]);
        }
    }
    public function getProbatePopupData(Request $request) {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            //$probateProperty = ProbateProperty::where('status',1)->findOrFail($propId);
            $probateProperty = ProbateProperty::join('executor', 'probate_property.executor_id','executor.id')
                ->join("postcode", "postcode.id", "probate_property.postcode_id")
                ->join("outcode", "outcode.id", "postcode.outcode_id")
                ->leftJoin("property_sale_history", "property_sale_history.property_history_id", "probate_property.id")
                ->select('probate_property.*', 'executor.company_name as executor_comapnyname', 'executor.first_name','executor.sur_name','executor.address_1 as executor_address1','executor.address_2 as executor_address2','executor.town as executor_town','executor.postcode as executor_postcode', 'executor.phone as executor_phone','executor.email as executor_email', 'postcode.postcode as postcode_postcode','property_sale_history.price as property_sale_price','property_sale_history.sale_date as property_sale_date','property_sale_history.sold_year as property_sold_year')
                ->findOrFail($propId);
            $str = $probateProperty->address_1;
            $addr1 = explode(" ",$str);
            $postcode = $probateProperty->postcode_postcode;
            $soldHistory = SoldHistory::where('house_no', $addr1[0])->where('postcode',$postcode)->get();
            $returnHTML = view('admin.popup-content',['probateProperty'=>$probateProperty, 'soldHistory' => $soldHistory])->render();

            return response()->json( array('success' => true, 'html'=>$returnHTML) );
        }
    }

    public function getProbateTooltipData(Request $request) {
        if ($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::join('outcode','probate_property.outcode_id','outcode.id')               ->where('status',1)
                               ->findOrFail($propId); 
            $returnHTML = view('admin.tooltip-content',['probateProperty'=>$probateProperty])->render(); 

            return response()->json(array('success' => true, 'html'=>$returnHTML));
        }
    }

    public function getSchoolTooltipData(Request $request) {
        $sch_latitude = $request->latitude;
        $sch_longitude = $request->longitude;
        $prop_latitude = $request->prop_latitude;
        $prop_longitude = $request->prop_longitude;
        $radius = 1.5;
        $schools = School::selectRaw("*,
        ( 3956 * acos( cos( radians(?) ) *
        cos( radians( latitude ) )
        * cos( radians( longitude ) - radians(?)
        ) + sin( radians(?) ) *
        sin( radians( latitude ) ) )
        ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
        ->where([
        ['latitude',$sch_latitude],
        ['longitude',$sch_longitude]
        ])
        ->having('distance', '<=', $radius)->first();

        $returnHTML = view('admin.school-content',['schoolAddress'=>$schools])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getEpcDetails(Request $request) {
        $full_address = explode("|",$request->address);
        $address = trim(trim($full_address[0]," "),",");
        $outcode = trim($full_address[1]," ");
        $epc_url = 'https://epc.opendatacommunities.org/api/v1/domestic/search?postcode='.$outcode.'&address='.$address;
        $epc_client = new Client();
        $epc_results = $epc_client->get($epc_url, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Authorization' => 'Basic c2hhcnU2MzFAZ21haWwuY29tOmQwZjE1Y2ZjNDlmNDlkYzQ2NWQzZjMyYjdlZDMzYjU5OTk3Y2YzYmY='
            ]
        ]);

        $epc_status = $epc_results->getStatusCode();
        $epc_contents = json_decode($epc_results->getBody(), true);
        $epc_details = $epc_contents['rows'];
        $allEpc['data'] = $epc_details[0];
        $returnHTML = view('admin.epc-content',['epcDetails'=>$allEpc ])->render(); 
        return response()->json( array('success' => true, 'html'=>$returnHTML));
    }

    public function getSchoolFullDetails(Request $request) {
        $latitude = $request->sch_latitude;
        $longitude = $request->sch_longitude;

        $schoolDetails = School::where([
            ['latitude',$latitude],
            ['longitude',$longitude]
        ])->first();

        $returnHTML = view('admin.school-full-details',['schoolDetails'=>$schoolDetails])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function schoolCoordUpdate(Request $request) {
        $schools_count = SchoolList::count();
        $count = 0;
        DB::table('schools')->orderBy('urn')->where('latitude', null)->chunk(100, function ($list) {
            foreach ($list as $school) {
                $count =+ 1;
                $urn = $school->urn;
                $school_name = $school->schname;
                $school_latitude = $school->latitude;
                $school_longitude = $school->longitude;
                if ($school_latitude  == null && $school_longitude == null) {
                    $school_address = $school->schname.', '.$school->street.', '.$school->town.', '.$school->postcode;
                    $google_autocomplete = "https://maps.googleapis.com/maps/api/place/autocomplete/json?input=".$school_address."&key=".config('map.apiKey');
                    $autocomplete_client = new Client();
                    $autocomplete_result = $autocomplete_client->get($google_autocomplete, [
                        'verify' => false,
                        'headers' => [
                            'Accept' => 'application/json',
                        ]
                    ]);
                    $autocomplete_status = $autocomplete_result->getStatusCode();
                    $autocomplete_response = json_decode($autocomplete_result->getBody(), true);
                    foreach ($autocomplete_response['predictions'] as $autocomplete_res) {
                        $google_placeId = "https://maps.googleapis.com/maps/api/place/details/json?place_id=".$autocomplete_res['place_id']."&key=".config('map.apiKey');
                        $placeId_client = new Client();
                        $placeId_result = $placeId_client->get($google_placeId, [
                            'verify' => false,
                            'headers' => [
                                'Accept' => 'application/json',
                            ]
                        ]);
                        $placeId_status = $placeId_result->getStatusCode();
                        $placeId_response = json_decode($placeId_result->getBody(), true);
                        $school_location = $placeId_response['result']['geometry']['location'];
                        $lat = $school_location['latitude'];
                        $lng = $school_location['longitude'];
                        School::where("urn",$urn)
                            ->update(
                                array(
                                    "latitude" => $lat,
                                    "longitude" => $lng,
                                )
                            );
                    }
                }
            }
        });
        if ($schools_count == $count) {
            return response()->json(array('success' => true, 'message' => 'Finished'));
        } else {
            return response()->json(array('success' => false, 'message' => 'Error'));
        }
    }

    public function getSchoolList(Request $request) {
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $sc_phase[] = $request->phase;       
        $radius = 1.5;
        $schools = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)->where('rating','!=',null);

                // if (in_array("Primary", $sc_phase))
                // {
                //    $schools = $schools->orWhere('phase_of_education', 'Primary'); 
                // }
                //  if (in_array("Secondary", $sc_phase))
                // {
                //     dd($sc_phase);
                //    $schools = $schools->orWhere('phase_of_education', 'Secondary'); 
                // }

                 $schools =  $schools->get();
         
        return response()->json( array('success' => true, 'html'=>$schools));
    }

    public function nearestSchoolList(Request $request) {
        $post_string = explode(' ', $request->postcode);
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $radius = 1.5;
        $schools_list = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)
                    ->orderBy('distance', 'ASC')
                    ->get();
        
        $returnHTML = view('admin.nearest-school-list',['schoolsList'=>$schools_list])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPhotos(Request $request) {
        $returnHTML = view('admin.photos-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }
    
    public function getFloorPlan(Request $request) {
        $returnHTML = view('admin.floorPlan-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPlanning(Request $request) {
        $returnHTML = view('admin.planning-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getSoldInStreet(Request $request) {
        $postcode = $request->postcode;
        $soldHistory = SoldHistory::select('sold_date','sold_price','tenure','house_no','addr2','addr5','addr6')->where('postcode',$postcode)->get();    
        $result = array();
         foreach ($soldHistory as $sold) {
         $result[$sold['house_no']][] = $sold;
         }
             dd($result);
        $returnHTML = view('admin.soldInStreet-content',['soldHistory'=>$soldHistory])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getMarketData(Request $request) {
        $returnHTML = view('admin.marketData-content')->render(); 
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getOfcom(Request $request) {
        $postcode = $request->postcode;
        $ofcomApi = "https://api-proxy.ofcom.org.uk/broadband/coverage/".str_replace(' ', '', $postcode);
        $ofcom_client = new Client();
        $ofcom_results = $ofcom_client->get($ofcomApi, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Ocp-Apim-Subscription-Key' => config('map.ofcomKey'),
            ]
        ]);
        $ofcom_status = $ofcom_results->getStatusCode();
        $ofcom_contents = json_decode($ofcom_results->getBody(), true);
        $ofcom_result = [];
        foreach ($ofcom_contents['Availability'] as $ofcom_results) {
            $ofcom_result['MaxBbPredictedDown'] = $ofcom_results['MaxBbPredictedDown'];
            $ofcom_result['MaxBbPredictedUp'] = $ofcom_results['MaxBbPredictedUp'];
            $ofcom_result['MaxSfbbPredictedDown'] = $ofcom_results['MaxSfbbPredictedDown'];
            $ofcom_result['MaxSfbbPredictedUp'] = $ofcom_results['MaxSfbbPredictedUp'];
            $ofcom_result['MaxUfbbPredictedDown'] = $ofcom_results['MaxUfbbPredictedDown'];
            $ofcom_result['MaxUfbbPredictedUp'] = $ofcom_results['MaxUfbbPredictedUp'];
            $ofcom_result['MaxPredictedDown'] = $ofcom_results['MaxPredictedDown'];
            $ofcom_result['MaxPredictedUp'] = $ofcom_results['MaxPredictedUp'];
            $ofcom_res['data'] = $ofcom_result;
        }
        $returnHTML = view('admin.ofcom-content',['ofcomDetails'=>$ofcom_res['data']])->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function documents(Request $request) {
        $returnHTML = view('admin.documents')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }


     //All property
    public function allProperty(Request $request)
    {
        $user_id            = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title         = 'Property Listings';
        $active             = 2;
        $user_type          = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $post_code          = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')
        ->where('user_id', $user_id)
        ->join('postcode', 'outcode.id', 'postcode.outcode_id')
        ->where('user_outcode.status', 1)
        ->where('outcode.archive', 0)
        ->select('postcode.*')
        ->orderBy('postcode.postcode', 'asc')
        ->get();
        // $property           = PropertyHistory::select('property_type', 'id')->get()->unique('property_type');
        // $property           = $this->commonPropQuery()->select('property_type')->get()->unique('property_type');
        $property           = $this->commonFreshPropQuery()->select('property_history.property_type')->get()->unique('property_type');
        $prop_count         = count($property);
        $agency             = Agency::get();
        $userOutcodes       = $this->userOutcodes();
        $campaign           = 1;
        $filter             = true;
        $enable_export      = User::find($user_id)->enable_export;
        $weeks              = '';
        $search_data        = '';
        $filter_title       = 'All Property Listings(Without Filters)';
        // Filter based on weeks
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                if ($weeks == 1) {
                    $filter_title = 'Property Filtered in 0-4 Weeks';
                } elseif ($weeks == 2) {
                    $filter_title = 'Property Filtered in 4-8 Weeks';
                } elseif ($weeks == 3) {
                    $filter_title = 'Property Filtered in 8-12 Weeks';
                } elseif ($weeks == 4) {
                    $filter_title = 'Property Filtered in 12-16 Weeks';
                } elseif ($weeks == 5) {
                    $filter_title = 'Property Filtered in 16 Weeks or More';
                }
            }
            $search_data        = json_decode(Session::get('search_data'));
            $search_data->weeks = $weeks;
        }

        return view('all-property', compact('page_title', 'filter_title', 'property', 'prop_count', 'post_code', 'userOutcodes', 'user_type', 'active', 'campaign', 'agency', 'filter','search_data','enable_export'));
    }

    public function allPropertyArray(Request $request)
    {
        $sesstion_array =[];
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $user_outcode = $this->userOutcode();
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                $query = $this->commonFreshPropQuery();
                // $query = $query->whereNotIn('property_history.for_sale', [3, 6]);
                // $query = $query->whereNotIn('property_price_history.type', ['Withdrawn', 'STC']);
            }
        } else {
            $query = $this->commonPropQuery();
        }
        $query = $query->whereIn('outcode.id', $user_outcode);
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
            $sesstion_array['outcode'] = $request->input('outcode');
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
                $sesstion_array['post_code'] = $request->input('post_code');

            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
                $sesstion_array['type'] = $request->input('type');
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
                $query->where('property_flag.user_id', $user_id);
                $sesstion_array['flag'] = $request->input('flag');
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
                $sesstion_array['agency'] = $request->input('agency');
            }
        }
        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
                $sesstion_array['platform'] = $request->input('platform');
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('weeks')) {
            $sesstion_array['weeks'] = $request->input('weeks');
            $weeks = $request->weeks;
            $dt = Carbon::now();

            if (in_array($weeks, [1, 2, 3, 4])) {
                if (1 == $weeks) {
                    $start = $dt->copy()->subWeeks(4)->addDays(1)->startOfDay();
                    $end = $dt->copy()->endOfDay();
                } elseif (2 == $weeks) {
                    $start = $dt->copy()->subWeeks(8)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(4)->endOfDay();
                } elseif (3 == $weeks) {
                    $start = $dt->copy()->subWeeks(12)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(8)->endOfDay();
                } elseif (4 == $weeks) {
                    $start = $dt->copy()->subWeeks(16)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(12)->endOfDay();
                }
                $query->whereBetween('house_number.listed_date', [$start, $end]);
            } elseif (5 == $weeks) {
                $query->whereDate('house_number.listed_date', '<=', $dt->copy()->subWeeks(16)->endOfDay());
            }
        }
        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode.outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'agency.name',
            6 => 'property_type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');
            $sesstion_array['search'] = $search;
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%');

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }

            })->orderBy($order, $dir)->get()->unique('house_number_id');
            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }
            });
        }

        $dataArr = [];


        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                //   $urls = $r_count.$z_count;
                //   $urls = '<div class=" d-flex"> <a class="btn btn-primary  btn-xs  text-center"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">3</span>
                //    <a class="btn btn-success  btn-xs" href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">3</span></div>
                //   ';
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                // if (!empty($item->house_number)) {
                $address2 = $item->house_number.', '.$item->address;
                // } else {
                // $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                // if (!empty($item->created_by)) {
                //     $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $st = '';
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                // $nestedData['type'] = $request->outcodeFilter.'--'. $request->input('search.value');
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }
        
        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        $session_data =  json_encode( $sesstion_array);
        Session::put('search_data', $session_data );

        return json_encode($json_data);
    }

    //property flag change
    public function flagChange(Request $request)
    {
        // return $request->all();

        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ('' != $exist_data) { //update
                // $update = PropertyFlag::where('id',$exist_data['id'])
                //     ->update(['flag' => $request->flag]);
                $update = PropertyFlag::find($exist_data['id']);
                $update->fill(['flag' => $request->flag])->save();

                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    //property detail page
    public function propertyDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode.');
            }
        }

        $prop = true;
        $house_number_id = Helper::decryptData($request->property_id);
        $query = $this->commonPropQuery();
        $property_details = $query->where('house_number.id', $house_number_id)->first();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $page_title = $property_details->address;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
        ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('house_number_id', $house_number_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
        ->orderBy('property_price_history.event_date', 'desc')->orderBy('property_price_history.id', 'desc')->get();
        // dd($listing_activity);
        $ambiguity_checker_response = $this->propertyAmbiguityChecker(['data_type' => 'listing_activity_object', 'data' => $listing_activity]);

        $comments = Helper::propertyComment($house_number_id);

        $letter_list = PropertyLetter::join('house_number', 'property_letters.house_number_id', 'house_number.id')
        ->join('letters', 'letters.id', 'property_letters.letter_id')
        ->where('property_letters.house_number_id', $house_number_id)
        ->select('property_letters.*', 'letters.name as letter_name')
        ->get()
        ;

        $current_flag = Helper::propertyFlag($house_number_id);

        $portal_url = $this->houseNumberPortal($house_number_id);
        $campaign_history = PropertyCampaign::where('house_number_id', $house_number_id)
        ->whereHas('campaign')
        ->with('campaign')
        ->withTrashed()
        ->where('user_id', $user_id)
        ->orderBy('id', 'desc')
        ->get()
        ;
        $trigger = $this->trigger;

        return view('property-details', compact('property_details', 'listing_activity', 'ambiguity_checker_response', 'comments', 'letter_list', 'house_number_id', 'prop', 'current_flag', 'portal_url', 'house_number_id', 'campaign_history', 'trigger'));
    }

    //property history details
    public function propertyHistoryDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode');
            }
        }

        $prop = true;
        $property_history_id = Helper::decryptData($request->property_id);

        // dd($property_history_id);

        $property_details = PropertyHistory::join('postcode', 'property_history.postcode_id', 'postcode.id')
        ->where('property_history.id', $property_history_id)
        ->select('property_history.*', 'postcode.postcode as p_code')
        ->first()
        ;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->where('property_history.id', $property_history_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id')
        ->orderBy('property_price_history.event_date', 'desc')->get();

        // dd($listing_activity);

        // $comments = Helper::propertyComment($house_number_id);
        $comments = [];

        // $letter_list = PropertyLetter::join('house_number','property_letters.house_number_id','house_number.id')
        //     ->join('letters','letters.id','property_letters.letter_id')
        //     ->where('property_letters.house_number_id',$house_number_id)
        //     ->select('property_letters.*','letters.name as letter_name')
        //     ->get();

        $letter_list = [];

        $portal_url[0] = ['portal' => $property_details->portal, 'portal_url' => $property_details->portal_url];

        return view('property-history-details', compact('property_details', 'listing_activity', 'comments', 'letter_list', 'prop', 'portal_url'));
    }

    // Check if ambiguous data is provided by the system
    public function propertyAmbiguityChecker($data)
    {
        $data_type = isset($data['data_type']) ? $data['data_type'] : '';
        if ($data_type === 'listing_activity_object') {
            $listing_activity   = isset($data['data']) ? $data['data'] : '';
        } else if ($data_type === 'house_number_id') {
            $house_number_id    = isset($data['data']) ? $data['data'] : '';
            if ($house_number_id != '') {
                $listing_activity   = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
                ->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
                ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
                ->join('agency', 'agency.id', 'property_history.agency_id')
                ->where('house_number_id', $house_number_id)
                ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
                ->orderBy('property_price_history.event_date', 'desc')
                ->orderBy('property_price_history.id', 'desc')
                ->get();
            }
        }
        if (!empty($listing_activity) && count($listing_activity) > 0) {
            // Convert listing fetch to array
            $listing_activity_array = $listing_activity->toArray();
            // prepare portal to agent array
            $list_activity_portal_array         = array_column($listing_activity_array, 'portal');
            $list_activity_agent_array          = array_column($listing_activity_array, 'name');
            $list_activity_portal_agent_array   = [];
            $i = 0;
            // Prepare an event date array out of listing activity with correct position keys
            foreach ($listing_activity_array as $list_key => $list_value) {
                if (isset($listing_activity_array[$i]['event_date']) && $listing_activity_array[$i]['event_date'] != '') {
                    $list_activity_array[$i] = $listing_activity_array[$i]['event_date'];
                }
                $list_activity_portal_agent_array[] = $list_activity_portal_array[$i]." - ".$list_activity_agent_array[$i];
                $i++;
            }
            $unique_list_activity               = array_unique($list_activity_portal_agent_array);
            $unique_agent_activity_count        = count($unique_list_activity);
            // Preparing activity_listing_position_array for comparing in pattern
            $temp_listing_activity_array        = $list_activity_array;
            $count_list_activity_array          = count($list_activity_array);
            $count_activity                     = array_count_values($list_activity_array);
            $total_count_activity               = count($count_activity);
            $cycle_count                        = 0;
            $activity_listing_position_array    = [];
            foreach ($count_activity as $count_activity_key => $count_activity_value) {
                if ($count_activity_value > 1) {
                    $cycle_count += $count_activity_value;
                }
            }
            $temp_cycle_count                   = $cycle_count;
            if ($cycle_count > 0) {
                while ((count($temp_listing_activity_array) >= $cycle_count) && $cycle_count !== 0) {
                    foreach ($count_activity as $count_activity_key => $count_activity_value) {
                        if ($count_activity_value > 1) {
                            $activity_listing_position = '';
                            $activity_listing_position = array_search($count_activity_key, $temp_listing_activity_array);
                            if ($activity_listing_position !== false) {
                                $activity_listing_position_array[$activity_listing_position] = $count_activity_key;
                                unset($temp_listing_activity_array[$activity_listing_position]);
                            }
                        }
                    }
                    $cycle_count--;
                }
                // Sort in DESC order of activity date
                ksort($activity_listing_position_array);
            }
            // ambiguous listing checks
            $ambiguous_listing_flag             = false;
            $type_unfavourable_array            = ['Withdrawn', 'STC', 'Sold'];
            $first_listing_activity_array       = array_key_first($listing_activity_array);
            // $type_unfavourable_array            = ['Withdrawn'];
            if (!empty($activity_listing_position_array)) {
                $total_activity_listing_position_array = count($activity_listing_position_array);
                foreach ($activity_listing_position_array as $alpa_key => $alpa_value) {
                    $next_activity_listing = next($activity_listing_position_array);
                    // pattern checking for ambiguous data provided for a listing through system
                    // Pattern 1
                    if (isset($listing_activity_array[$alpa_key]['type'])
                        && isset($listing_activity_array[$alpa_key+1]['type'])
                        && ($listing_activity_array[$alpa_key]['type'] != $listing_activity_array[$alpa_key+1]['type']) // check if event type is different
                        && ($next_activity_listing !== false) // check if next item available
                        && (in_array($listing_activity_array[$alpa_key]['type'], $type_unfavourable_array)) // check if type is not offer or price related
                        // && ($listing_activity_array[$alpa_key]['portal'] === $listing_activity_array[$alpa_key+1]['portal']) // check if same portal provides the data
                        && ($listing_activity_array[$alpa_key]['event_date'] === $listing_activity_array[$alpa_key+1]['event_date'])) // check if event date is same
                        // && ($listing_activity_array[$alpa_key]['name'] !== $listing_activity_array[$alpa_key+1]['name'])) // check if portal agent is different
                    {
                        $ambiguous_listing_flag = true;
                    }
                }
                // Pattern 2
                if (isset($listing_activity_array[$first_listing_activity_array]['type'])
                    && (in_array($listing_activity_array[$first_listing_activity_array]['type'], $type_unfavourable_array)) // check if first listing type is unfavourable
                    && ($unique_agent_activity_count > 1)) // check if first listing type is unfavourable
                {
                    $ambiguous_listing_flag = true;
                }
            }
            // debuging unit for ambiguous_listing_flag listing data
            // $list_activity_array            = array_column($listing_activity_array, 'event_date');
            // $list_activity_array_filtered   = array_filter($list_activity_array);
            // print("<pre>".print_r($listing_activity_array,true)."</pre>");
            /*print("<pre>".print_r($list_activity_array,true)."</pre>");
            echo "total activity count : $count_list_activity_array";
            print("<pre>".print_r($count_activity,true)."</pre>");
            echo "unique activity count : $total_count_activity<br>";
            print("<pre>".print_r($temp_listing_activity_array,true)."</pre>");
            echo "cycle count : $temp_cycle_count<br>";
            echo "cycle count now: $cycle_count<br>";
            print("<pre>".print_r($activity_listing_position_array,true)."</pre>");*/
            // echo $ambiguous_listing_flag;
            // dd('stopped');
            return $ambiguous_listing_flag;
        }
    }

    //comment delete
    public function deletePropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }

        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $update = PropertyComment::where('id', $request->comment)->delete();
        if ($update) {
            return $this->simpleReturn('success', 'Successfully comment deleted..!');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    //add property comment
    public function addPropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('error', $validator->errors()->first());
        }
        
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $add = new PropertyComment();
        $add->house_number_id = $request->property;
        $add->comment = $request->comment;
        $add->user_id = $user_id;
        if ($add->save()) {
            return Redirect::back()->with('success', 'Successfully comment deleted..!');
        }

        return Redirect::back()->with('error', 'Something went wrong...!');
    }

    //search property
    public function searchProperty(Request $request)
    {
        $search_keyword = $request->search;
        $page_title = 'Addresses or Agencies That Contain: '.$search_keyword;
        session()->put('search_keyword', $search_keyword);

        return view('search-result', compact('page_title'));
    }

    //search property array
    public function searchPropertyArray(Request $request)
    {
        $search_keyword = session()->get('search_keyword');
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        // ->Where('property_history.address', 'like', '%' . $search_keyword . '%')
        // ->orWhere('agency.name', 'like', '%' . $search_keyword . '%')
        // ->orWhere('postcode.postcode', 'like', '%' . $search_keyword . '%');
        // ->orWhere('property_history.property_type', 'like', '%' . $search_keyword . '%');

        $query = $query->where(function ($q) use ($search_keyword) {
            $q->Where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('property_history.property_type', 'like', '%'.$search_keyword.'%')
            ->orWhere('outcode.outcode', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
        });

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'agency',
            4 => 'listed_date',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //  $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)
            ->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()->unique('house_number_id');
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }

                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                //  $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    //new to market
    public function newToMarket()
    {
        $page_title = 'Weekly New to Market';
        $campaign = 1;

        return view('new_to_market', compact('page_title', 'campaign'));
    }

    public function newToMarketArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        $new_date_from = date('Y-m-d', strtotime('-7 days'));
        $new_date_to = date('Y-m-d');

        $query = $query->whereBetween('house_number.listed_date', [$new_date_from, $new_date_to]);

        // dd($items->where('house_number.id',1)->get());
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'price',
            4 => 'listed_date',
            5 => 'agency.name',
            6 => 'type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                $nestedData['outcode'] = $item->outcode;

                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }
                // $listed_date=date('Y-m-d',strtotime($item->listed_date));
                // $today=date('Y-m-d');
                // $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                // $nestedData['listed_date'] = date('d M Y',strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }
                // $nestedData['week'] = $item->week;
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //withdrawn
    public function withdrawn()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Withdrawn';

        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;
        $userOutcodes = $this->userOutcodes();

        return view('withdrawn-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function withdrawnArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        if (1 == $user_type) { //admin
            $data = HouseNumber::get();
        } elseif (2 == $user_type) { //agent
            $user_outcode = $this->userOutcode();
            $outcodes = $this->userOutcodes();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.latest_status', 3);
            // dd($items->where('house_number.id',1)->get());
            if ($request->has('outcode') && '' != $request->input('outcode')) {
                $query->where('outcode.outcode', $request->input('outcode'));
            }

            if ($request->has('post_code')) {
                if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                    $query->where('postcode.id', $request->input('post_code'));
                }
            }
            // dd($request->all());
            if ($request->has('type')) {
                if (('0' != $request->type && 'undefined' != $request->type)) {
                    $query->where('property_history.property_type', $request->input('type'));
                }
            }
            if ($request->has('flag')) {
                if ((0 != $request->flag && 'undefined' != $request->flag)) {
                    $query->where('property_flag.flag', $request->input('flag'));
                }
            }
            if ($request->has('agency')) {
                if ((0 != $request->agency && 'undefined' != $request->agency)) {
                    $query->where('agency.id', $request->input('agency'));
                }
            }
            if ($request->has('platform')) {
                if ((-1 != $request->platform && 'platform' != $request->platform)) {
                    $query->where('property_history.portal', $request->input('platform'));
                }
            }
            if ($request->has('date_from') && $request->has('date_to')) {
                if (('' != $request->date_from && '' != $request->date_to)) {
                    $date_from = date('Y-m-d', strtotime($request->date_from));
                    $date_to = date('Y-m-d', strtotime($request->date_to));
                    $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
                }
            }
        } else { //data entry operator
            $data = HouseNumber::get();
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'withdrawn_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $withdrwan_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item->house_number_id)
                ->where('property_price_history.type', 'Withdrawn')
                ->orderBy('property_price_history.event_date', 'desc')
                ->select('property_price_history.event_date')
                ->first()
                ;

                // $nestedData['withdrawn_date'] = date('d M Y', strtotime( $withdrwan_date->event_date
                $nestedData['withdrawn_date'] = date('d M Y', strtotime($item->withdrawn_date));

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function recentlySold()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Sold - Under Offer';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('recently-sold-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function recentlySoldArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        //print_r($userOutcodes); exit;

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode)
        ->where('house_number.latest_status', '!=', 3)
        ;

        $query = $query->where(function ($q) {
            $q->Where('property_history.sold_stc', 1)
            ->orWhere('property_history.under_offer', 1)
            ;
        });

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            // 5 =>'property_price_history.event_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');

        if (empty($request->input('search.value'))) {
            // $data = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
            // $dd = $query->orderBy($order,$dir)->get()->unique('house_number_id')->toArray();
            // $list = array_slice($data, $start, $limit);
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            //  $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';

                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item['house_number_id']);

                $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item['house_number_id'])
                ->where('property_price_history.type', 'STC')
                ->orderBy('property_price_history.id', 'desc')
                ->limit(1)
                ->select('property_price_history.event_date', 'property_price_history.sold_year')
                ->first()
                ;

                if (!$sold_date) {
                    $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                    ->where('property_house_number.house_number_id', $item['house_number_id'])
                    ->where('property_price_history.type', 'Offer')
                    ->orderBy('property_price_history.id', 'desc')
                    ->limit(1)
                    ->select('property_price_history.event_date', 'property_price_history.sold_year')
                    ->first()
                    ;
                }

                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;

                if ($sold_date) {
                    if ($sold_date->event_date) {
                        $so_date = date('d M Y', strtotime($sold_date->event_date));
                    } else {
                        $so_date = $sold_date->sold_year;
                    }
                } else {
                    $so_date = '';
                }

                $nestedData['sold_date'] = $so_date;

                if ($item['listed_date']) {
                    $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['id'] = $item['house_number_id'];
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item['outcode'];
                if (null == $item['created_by']) {
                    $st = 'By System';
                    $color = '';
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }

                if (!empty($item['house_number'])) {
                    $address2 = $item['house_number'].', '.$item['address'];
                } else {
                    $address2 = $item['house_number'].', '.$item['address'].',<br>  '.$item['postcode'];
                }
                if (!empty($item['created_by'])) {
                    $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                // $nestedData['price'] = "&pound; ".number_format($item['latest_price']);
                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                // $nestedData['sold_date'] =  date('d M Y',strtotime($item['sold_date']));
                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item['title'].' <br> '.ucfirst($item['property_type']);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function priceDrop()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Recent Price Drops        ';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $agency = Agency::get();
        $filter = true;
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('price-drop-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'agency', 'filter', 'totalProp'));
    }

    //price drop
    public function priceDropArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        // if($user_type==1){//admin
        //     $data = HouseNumber::get();

        // }elseif($user_type==2){//agent

        $user_outcode = $this->userOutcode();

        // $query = $this->commonPropQuery();
        // $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.price_change_diff', '<', 0);

        $query = HouseNumber::join('property_house_number', 'property_house_number.house_number_id', 'house_number.id')
        ->join('property_price_history', 'property_price_history.property_history_id', 'property_house_number.property_history_id')
        ->join('property_history', 'property_history.id', 'property_house_number.property_history_id')
        ->join('postcode', 'postcode.id', 'house_number.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_flag', 'property_flag.house_number_id', 'property_house_number.house_number_id')
        ->whereNull('property_house_number.deleted_at')
        ->whereIn('outcode.id', $user_outcode)
        ->Where('property_price_history.type', 'like', '%Price reduced by%')
        ->select('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', 'house_number.latest_price', 'house_number.latest_status', 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id', 'house_number.created_by', 'property_price_history.type', 'property_price_history.event_date')
        ->orderBy('property_price_history.event_date', 'desc')
        ;
        // ->groupBy('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', "house_number.latest_price", "house_number.latest_status", 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id,'property_price_history.type'');

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // dd($items->where('house_number.id',1)->get());

        // }else{//data entry operator
        //     $data = HouseNumber::get();
        // }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'price_change_diff',
            5 => 'house_number.listed_date',
            6 => 'price_drop_date',
            7 => 'agency.name',
            8 => 'property_type',
            9 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        $outcodes = $this->userOutcodes();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //$query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                // $address3=empty($item->title)?$address2:$address1;
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['price_drop'] = '-'.substr($item->type, 16);
                // '&pound;'.number_format($item->price_change_diff);
                $listed_date = date('Y-m-d', strtotime($item->listed_date));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);

                if ($item->listed_date) {
                    $dt = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $dt = 'Hidden from public';
                }

                $nestedData['listed_date'] = $dt;
                $nestedData['price_drop_date'] = date('d M Y', strtotime($item->event_date));
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //export property data
    public function exportPropertyData(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        $file_name = '';

        if ('all_property' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);
                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'All-property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('new_to_market' == $request->mode) {
            $user_outcode = $this->userOutcode();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.new_to_market', 1);
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'New-to-market'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('recent_sold' == $request->mode) {
            $query = $query->where('house_number.latest_status', '!=', 3);

            $data_list = $query->where(function ($q) {
                $q->Where('property_history.sold_stc', 1)->orWhere('property_history.under_offer', 1);
            })->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Recentl-sold'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('hot_list' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
        } elseif ('withdrawn' == $request->mode) {
            $data_list = $query->where('house_number.latest_status', 2)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Withdrawn Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['withdrawn_date'] = date('d-m-Y', strtotime($item->withdrawn_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Withdrawn-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('price_drop' == $request->mode) {
            $data_list = $query->where('house_number.price_change_diff', '<', 0)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['price_drop'] = number_format($item->price_change_diff);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['price_drop_date'] = date('d-m-Y', strtotime($item->price_drop_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Price-drop-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('all_type_property' == $request->mode) {
            $user_outcode = $this->userOutcode();
            $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
            $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();
            $list1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
            ->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
            ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
            ->where('property_house_number.house_number_id', null)
            ->where('property_house_number.property_history_id', null)
            ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title')->get()->toArray();

            $list2 = $this->commonPropQuery();
            $list2 = $list2->whereIn('outcode.id', $user_outcode)->get()->unique('house_number_id')->toArray();
            $data_list = array_merge($list1, $list2);

            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['price'] = number_format($item['latest_price']);
                $arr['listed_date'] = date('d-m-Y', strtotime($item['listed_date']));
                if ($item['house_number_id']) {
                    $arr['address'] = $item['house_number'].', '.$item['address'].', '.$item['postcode'];
                    $arr['agency'] = Helper::agencyName($item['house_number_id']);
                } else {
                    $arr['address'] = $item['address'].', '.$item['postcode'];
                    $arr['agency'] = $item['name'];
                }
                $arr['type'] = ucfirst($item['property_type']);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'all-property-data'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } else {
            $search_keyword = session()->get('search_keyword');
            $query = $query->where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
            $data_list = $query->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        }

        $url = url('storage/property-downloads/'.$file_name);
        $data['url'] = $url;
        $data['name'] = $file_name;

        return $data;
    }

    //Hotlist
    public function hotList()
    {
        $page_title = 'Hotlist';

        return view('hotlist', compact('page_title'));
    }

    public function hotListArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'listed_date',
            4 => 'agency',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;

            // $totalFiltered =$query->Where('property_history.address', 'like', '%' . $search . '%')
            // ->orWhere('agency.name', 'like', '%' . $search . '%')
            // ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            // ->count();
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                $nestedData['agency'] = $item->agency_name;
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['comments'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    public function filterByPostcode(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $post_code = $request->postCode;
        if ($post_code) {
        }

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ($exist_data) { //update
                $update = PropertyFlag::where('id', $exist_data['id'])
                ->update(['flag' => $request->flag])
                ;
                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    public function propertyTrigger($tab)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Property';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $active = 2;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $enable_export = User::find($user_id)->enable_export;
        $userOutcodes = $this->userOutcodes();

        if (1 == $tab) {
            $filter = true;
            $active = 1;
            $agency = Agency::get();
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');

            return view('property-all-type', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'agency', 'property', 'filter','enable_export'));
        }
        if (2 == $tab) {
            $campaign = 1;
            $active = 2;
            $filter = true;
            $agency = Agency::get();

            return view('all-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'active', 'user_type', 'filter', 'agency', 'campaign','enable_export'));
        }
        if (3 == $tab) {
            $active = 3;
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');
            $filter = true;
            $post_code = PostCode::orderBy('postcode', 'asc')->get()->pluck('postcode', 'id');
            return view('common.property.property-without-address', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'property', 'filter','enable_export'));
        }

        return view('all-property', compact('page_title', 'post_code', 'userOutcodes', 'active', 'user_type','enable_export'));
    }

    public function allTypeProperty()
    {
        $page_title = 'Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $active = 1;

        return view('admin.property-all-type', compact('page_title', 'post_code', 'user_type', 'active'));
    }

    public function allTypePropertyArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
        $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();

        $query1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
        ->where('property_house_number.house_number_id', null)
        ->where('property_house_number.property_history_id', null)
        ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title');

        // ->get()->toArray();

        $query2 = $this->commonPropQuery();
        $query2 = $query2->whereIn('outcode.id', $user_outcode);

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query1->where('outcode.outcode', $request->input('outcode'));
            $query2->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query1->where('postcode.id', $request->input('post_code'));
                $query2->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('platform')) {
            if (('-1' != $request->platform && 'undefined' != $request->platform)) {
                $query1->where('property_history.portal', $request->input('platform'));
                $query2->where('property_history.portal', $request->input('platform'));
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query1->whereBetween('property_history.listed_date', [$date_from, $date_to]);
                $query2->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query1->where('property_history.property_type', $request->input('type'));
                $query2->where('property_history.property_type', $request->input('type'));
            }
        }

        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query2->where('property_flag.flag', $request->input('flag'));
            }
        }

        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query1->where('agency.id', $request->input('agency'));
                $query2->where('agency.id', $request->input('agency'));
            }
        }
        // ->get()->unique('house_number_id')->toArray();

        // $all_data = array_merge($all, $withoutHouseNumber);

        // Table's primary key
        $primaryKey = 'id';
        // $outcodes = $this->userOutcodes();

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'latest_price',
            3 => 'listed_date',
            4 => 'agency_name',
            5 => 'property_type',
            6 => 'flag',
        ];

        $q_data1 = $query1->get()->toArray();
        $q_data2 = $query2->get()->unique('house_number_id')->toArray();
        $all_data = array_merge($q_data1, $q_data2);
        $totalData = count($all_data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        if (empty($request->input('search.value'))) {
            $list1 = $query1->orderBy($order, $dir)->get()->toArray();
            $list2 = $query2->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

            $data_val = array_merge($list1, $list2); //echo "<pre>"; print_r($data_val); exit;
        } else {
            $search = $request->input('search.value');

            $list1 = $query1->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->toArray()
            ;

            $list2 = $query2->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->unique('house_number_id')
            ->toArray()
            ;

            $data_val = array_merge($list1, $list2);

            // $totalFiltered = $query->where(function($q) use ($search) {
            //     $q->Where('property_history.address', 'like', '%' . $search . '%')
            //         ->orWhere('agency.name', 'like', '%' . $search . '%')
            //     ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            //     ->orWhere('outcode.outcode', 'like', '%' . $search . '%')
            //     ->orWhere('postcode.postcode', 'like', '%' . $search . '%');

            // })->count();
            $totalFiltered = count($data_val);
        }

        $dataArr = [];
        $outcodes = $this->userOutcodes();

        if (!empty($data_val)) {
            $data_val = array_slice($data_val, $start, $limit); // return the first five elements

            $i = 0;
            foreach ($data_val as $item) {
                $nestedData['outcode'] = $item['outcode'];
                $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                if ($item['listed_date']) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                // $nestedData['price'] = "&pound;" . number_format($item['latest_price']);

                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                $nestedData['type'] = ucfirst($item['title']).'<br> '.ucfirst($item['property_type']);

                if ($item['house_number_id']) {
                    //portal url
                    $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                    $platform = '';
                    $r = 0;
                    $z = 0;
                    $r_count = '';
                    $z_count = '';
                    if (count($portal_url)) {
                        $platform = '<div class=" d-flex">';
                        foreach ($portal_url as $url) {
                            // $r_count="";$z_count="";
                            if (0 == $url->portal) {
                                ++$r;
                            } else {
                                ++$z;
                            }
                        }
                        if ($r > 0) {
                            $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                        }
                        if ($z > 0) {
                            $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                        }
                        $platform .= $r_count.$z_count;
                        $platform .= '</div>';
                    }
                    $encrpt_id = Helper::encryptData($item['house_number_id']);
                    $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                    ++$i;
                    $nestedData['id'] = $item['house_number_id'];
                    $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                    if (!empty($item['house_number'])) {
                        $address2 = $item['house_number'].' , '.$item['address'];
                    } else {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    if (!empty($item['created_by'])) {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    // $address3=empty($item['title'])?$address2:$address1;
                    if (null == $item['created_by']) {
                        $st = 'By System';
                        $color = '';
                    } else {
                        $st = 'By Manual';
                        $color="style='color:#edf124'";
                    }
                    $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                    // $nestedData['price'] = "&pound; ".number_format($item['latest_price']) ;

                    $nestedData['type'] = $item['title'].'<br> '.ucfirst($item['property_type']);
                    $nestedData['portal_url'] = $platform;
                    $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                    $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
                    $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();
                    $dataArr[] = $nestedData;
                } else {


                    $encrpt_id = Helper::encryptData($item['id']);
                    $nestedData['id'] = '';

                    $url = route('propertyHistoryDetails', ['property_id' => $encrpt_id]);

                    $nestedData['property_number'] = '<div class="input-group">
                    <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                    <div class="input-group-append">
                    <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item['id'].'"><i class="flaticon-paper-plane icon-lg"></i></button>
                    </div>
                    </div>';

                    $nestedData['address'] = "<a  href='{$url}'>{$item['address']},  {$item['postcode']}</a>";

                    $nestedData['agency'] = $item['name'];

                    if (0 == $item['portal']) {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">1</span></div>';
                    } else {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">1</span></div>';
                    }

                    $nestedData['flag'] = 0;
                    $nestedData['letter_count'] = 0;
                    ++$i;

                    $dataArr[] = $nestedData;
                }
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutPostcode(Request $request)
    {
        $page_title = 'Without Postcode Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;
        $without_data = true;

        return view('property-without-postcode', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutPostcodeArray(Request $request)
    {
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('postcode.postcode', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;
        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;
                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                // $nestedData['type'] = $totalFiltered;
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutAgent(Request $request)
    {
        $page_title = 'Without Agent Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;

        $without_data = true;

        return view('property-without-agent', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutAgentArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('agency.name', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;

        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;

                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function getProbateData()
    {
     //$user_outcode = $this->userOutcode();
        if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
            $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
            $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
        }
        $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
        $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
        $user_outcode = $selOutcodeCollection;

        $query = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
        ->join("postcode", "postcode.id", "probate_property.postcode_id")
        ->join("outcode", "outcode.id", "postcode.outcode_id")
        ->leftjoin("probate_property_flag", "probate_property_flag.probate_id", "probate_property.id")
        ->whereIn('outcode.id', $user_outcode)
        ->where('probate_property.status',1)
        ->whereDate('publication_date', '>', Carbon::now()->subDays(530));
        $data = $query
        ->select('postcode.postcode','outcode.outcode','probate_property.address_1','probate_property.address_2','probate_property.firstname','probate_property.surname','executor.company_name','executor.first_name','executor.sur_name','probate_property.publication_date','probate_property.claim_date','probate_property.id')
        ->get();


    return Datatables::of($data)

  ->editColumn('outcode', function ($model) {
   return $model->postcode;
 })
  ->editColumn('address', function ($model) {
$pid = Crypt::encrypt($model->id);
    return '<a href="' . url('property-detail?pid=' . $pid ) . '" title="View">'.$model->address_1.' '.$model->address_2.'</a>';
 })  
      ->editColumn('executer', function ($model) {
    $executor = $model->company_name ? $model->company_name : $model->first_name." ".$model->sur_name;
   return $executor;
  })
  ->editColumn('publication_date', function ($model) {
    $publicationDate =  date('d-m-Y', strtotime($model->publication_date));  
   return $publicationDate;
 })
   ->editColumn('claim_date', function ($model) {
    $claim_date =  date('d-m-Y', strtotime($model->claim_date));  
   return $claim_date;
 })
  

->editColumn('created_date', function ($model) {
    return 1;
})


  ->editColumn('action', function ($model) {
    $idG = Crypt::encrypt($model->id);
    // $user = \App\User::find($model->user_id);
    // Auth::loginUsingId($user->id, true);

            return '<a href="' . url('boat-for-sale/' . $model->slug ) . '" type="" target="_blank" class="act-sp" title="View"><i class="fas fa-file"></i></a>  <a href="' . url('edit-boat-info/' . $idG) . '" type="" target="_blank" class="act-sp" title="Login as user to edit"><i class="fas fa-edit"></i></a> ';
        })

        ->rawColumns(['s#','name','address','status','expiry','action'])
        ->make(true);
    }

    public function propertyDetail(Request $request)
    {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::where('status',1)->findOrFail($propId); 
            if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
                $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
                $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
            }
            $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
            $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
            $user_outcode = $selOutcodeCollection;
        // $probateProperties = ProbateProperty::where('status',1)->orderBy('id', 'DESC')->take(10)->get(['address_1','latitude','longitude','id'])->toArray();
            $probateProperties = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
            ->join("postcode", "postcode.id", "probate_property.postcode_id")
            ->join("outcode", "outcode.id", "postcode.outcode_id")
            ->whereIn('outcode.id', $user_outcode)
            ->where('probate_property.status',1)->orderBy('probate_property.id', 'DESC')->take(10)->get(['probate_property.address_1','probate_property.latitude','probate_property.longitude','probate_property.id'])->toArray();
            return view('admin.probate-property-detail')->with(['probateProperty' => $probateProperty,'probateProperties' => $probateProperties]);
        }
    }
    public function getProbatePopupData(Request $request) {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            //$probateProperty = ProbateProperty::where('status',1)->findOrFail($propId);
            $probateProperty = ProbateProperty::join('executor', 'probate_property.executor_id','executor.id')
                ->join("postcode", "postcode.id", "probate_property.postcode_id")
                ->join("outcode", "outcode.id", "postcode.outcode_id")
                ->leftJoin("property_sale_history", "property_sale_history.property_history_id", "probate_property.id")
                ->select('probate_property.*', 'executor.company_name as executor_comapnyname', 'executor.first_name','executor.sur_name','executor.address_1 as executor_address1','executor.address_2 as executor_address2','executor.town as executor_town','executor.postcode as executor_postcode', 'executor.phone as executor_phone','executor.email as executor_email', 'postcode.postcode as postcode_postcode','property_sale_history.price as property_sale_price','property_sale_history.sale_date as property_sale_date','property_sale_history.sold_year as property_sold_year')
                ->findOrFail($propId);
            $str = $probateProperty->address_1;
            $addr1 = explode(" ",$str);
            $postcode = $probateProperty->postcode_postcode;
            $soldHistory = SoldHistory::where('house_no', $addr1[0])->where('postcode',$postcode)->get();
            $returnHTML = view('admin.popup-content',['probateProperty'=>$probateProperty, 'soldHistory' => $soldHistory])->render();

            return response()->json( array('success' => true, 'html'=>$returnHTML) );
        }
    }

    public function getProbateTooltipData(Request $request) {
        if ($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::join('outcode','probate_property.outcode_id','outcode.id')               ->where('status',1)
                               ->findOrFail($propId); 
            $returnHTML = view('admin.tooltip-content',['probateProperty'=>$probateProperty])->render(); 

            return response()->json(array('success' => true, 'html'=>$returnHTML));
        }
    }

    public function getSchoolTooltipData(Request $request) {
        $sch_latitude = $request->latitude;
        $sch_longitude = $request->longitude;
        $prop_latitude = $request->prop_latitude;
        $prop_longitude = $request->prop_longitude;
        $radius = 1.5;
        $schools = School::selectRaw("*,
        ( 3956 * acos( cos( radians(?) ) *
        cos( radians( latitude ) )
        * cos( radians( longitude ) - radians(?)
        ) + sin( radians(?) ) *
        sin( radians( latitude ) ) )
        ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
        ->where([
        ['latitude',$sch_latitude],
        ['longitude',$sch_longitude]
        ])
        ->having('distance', '<=', $radius)->first();

        $returnHTML = view('admin.school-content',['schoolAddress'=>$schools])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getEpcDetails(Request $request) {
        $full_address = explode("|",$request->address);
        $address = trim(trim($full_address[0]," "),",");
        $outcode = trim($full_address[1]," ");
        $epc_url = 'https://epc.opendatacommunities.org/api/v1/domestic/search?postcode='.$outcode.'&address='.$address;
        $epc_client = new Client();
        $epc_results = $epc_client->get($epc_url, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Authorization' => 'Basic c2hhcnU2MzFAZ21haWwuY29tOmQwZjE1Y2ZjNDlmNDlkYzQ2NWQzZjMyYjdlZDMzYjU5OTk3Y2YzYmY='
            ]
        ]);

        $epc_status = $epc_results->getStatusCode();
        $epc_contents = json_decode($epc_results->getBody(), true);
        $epc_details = $epc_contents['rows'];
        $allEpc['data'] = $epc_details[0];
        $returnHTML = view('admin.epc-content',['epcDetails'=>$allEpc ])->render(); 
        return response()->json( array('success' => true, 'html'=>$returnHTML));
    }

    public function getSchoolFullDetails(Request $request) {
        $latitude = $request->sch_latitude;
        $longitude = $request->sch_longitude;

        $schoolDetails = School::where([
            ['latitude',$latitude],
            ['longitude',$longitude]
        ])->first();

        $returnHTML = view('admin.school-full-details',['schoolDetails'=>$schoolDetails])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function schoolCoordUpdate(Request $request) {
        $schools_count = SchoolList::count();
        $count = 0;
        DB::table('schools')->orderBy('urn')->where('latitude', null)->chunk(100, function ($list) {
            foreach ($list as $school) {
                $count =+ 1;
                $urn = $school->urn;
                $school_name = $school->schname;
                $school_latitude = $school->latitude;
                $school_longitude = $school->longitude;
                if ($school_latitude  == null && $school_longitude == null) {
                    $school_address = $school->schname.', '.$school->street.', '.$school->town.', '.$school->postcode;
                    $google_autocomplete = "https://maps.googleapis.com/maps/api/place/autocomplete/json?input=".$school_address."&key=".config('map.apiKey');
                    $autocomplete_client = new Client();
                    $autocomplete_result = $autocomplete_client->get($google_autocomplete, [
                        'verify' => false,
                        'headers' => [
                            'Accept' => 'application/json',
                        ]
                    ]);
                    $autocomplete_status = $autocomplete_result->getStatusCode();
                    $autocomplete_response = json_decode($autocomplete_result->getBody(), true);
                    foreach ($autocomplete_response['predictions'] as $autocomplete_res) {
                        $google_placeId = "https://maps.googleapis.com/maps/api/place/details/json?place_id=".$autocomplete_res['place_id']."&key=".config('map.apiKey');
                        $placeId_client = new Client();
                        $placeId_result = $placeId_client->get($google_placeId, [
                            'verify' => false,
                            'headers' => [
                                'Accept' => 'application/json',
                            ]
                        ]);
                        $placeId_status = $placeId_result->getStatusCode();
                        $placeId_response = json_decode($placeId_result->getBody(), true);
                        $school_location = $placeId_response['result']['geometry']['location'];
                        $lat = $school_location['latitude'];
                        $lng = $school_location['longitude'];
                        School::where("urn",$urn)
                            ->update(
                                array(
                                    "latitude" => $lat,
                                    "longitude" => $lng,
                                )
                            );
                    }
                }
            }
        });
        if ($schools_count == $count) {
            return response()->json(array('success' => true, 'message' => 'Finished'));
        } else {
            return response()->json(array('success' => false, 'message' => 'Error'));
        }
    }

    public function getSchoolList(Request $request) {
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $sc_phase[] = $request->phase;       
        $radius = 1.5;
        $schools = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)->where('rating','!=',null);

                // if (in_array("Primary", $sc_phase))
                // {
                //    $schools = $schools->orWhere('phase_of_education', 'Primary'); 
                // }
                //  if (in_array("Secondary", $sc_phase))
                // {
                //     dd($sc_phase);
                //    $schools = $schools->orWhere('phase_of_education', 'Secondary'); 
                // }

                 $schools =  $schools->get();
         
        return response()->json( array('success' => true, 'html'=>$schools));
    }

    public function nearestSchoolList(Request $request) {
        $post_string = explode(' ', $request->postcode);
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $radius = 1.5;
        $schools_list = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)
                    ->orderBy('distance', 'ASC')
                    ->get();
        
        $returnHTML = view('admin.nearest-school-list',['schoolsList'=>$schools_list])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPhotos(Request $request) {
        $returnHTML = view('admin.photos-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }
    
    public function getFloorPlan(Request $request) {
        $returnHTML = view('admin.floorPlan-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPlanning(Request $request) {
        $returnHTML = view('admin.planning-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getSoldInStreet(Request $request) {
        $postcode = $request->postcode;
        $soldHistory = SoldHistory::select('sold_date','sold_price','tenure','house_no','addr2','addr5','addr6')->where('postcode',$postcode)->get();    
        $result = array();
         foreach ($soldHistory as $sold) {
         $result[$sold['house_no']][] = $sold;
         }
             dd($result);
        $returnHTML = view('admin.soldInStreet-content',['soldHistory'=>$soldHistory])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getMarketData(Request $request) {
        $returnHTML = view('admin.marketData-content')->render(); 
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getOfcom(Request $request) {
        $postcode = $request->postcode;
        $ofcomApi = "https://api-proxy.ofcom.org.uk/broadband/coverage/".str_replace(' ', '', $postcode);
        $ofcom_client = new Client();
        $ofcom_results = $ofcom_client->get($ofcomApi, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Ocp-Apim-Subscription-Key' => config('map.ofcomKey'),
            ]
        ]);
        $ofcom_status = $ofcom_results->getStatusCode();
        $ofcom_contents = json_decode($ofcom_results->getBody(), true);
        $ofcom_result = [];
        foreach ($ofcom_contents['Availability'] as $ofcom_results) {
            $ofcom_result['MaxBbPredictedDown'] = $ofcom_results['MaxBbPredictedDown'];
            $ofcom_result['MaxBbPredictedUp'] = $ofcom_results['MaxBbPredictedUp'];
            $ofcom_result['MaxSfbbPredictedDown'] = $ofcom_results['MaxSfbbPredictedDown'];
            $ofcom_result['MaxSfbbPredictedUp'] = $ofcom_results['MaxSfbbPredictedUp'];
            $ofcom_result['MaxUfbbPredictedDown'] = $ofcom_results['MaxUfbbPredictedDown'];
            $ofcom_result['MaxUfbbPredictedUp'] = $ofcom_results['MaxUfbbPredictedUp'];
            $ofcom_result['MaxPredictedDown'] = $ofcom_results['MaxPredictedDown'];
            $ofcom_result['MaxPredictedUp'] = $ofcom_results['MaxPredictedUp'];
            $ofcom_res['data'] = $ofcom_result;
        }
        $returnHTML = view('admin.ofcom-content',['ofcomDetails'=>$ofcom_res['data']])->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function documents(Request $request) {
        $returnHTML = view('admin.documents')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

     //All property
    public function allProperty(Request $request)
    {
        $user_id            = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title         = 'Property Listings';
        $active             = 2;
        $user_type          = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $post_code          = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')
        ->where('user_id', $user_id)
        ->join('postcode', 'outcode.id', 'postcode.outcode_id')
        ->where('user_outcode.status', 1)
        ->where('outcode.archive', 0)
        ->select('postcode.*')
        ->orderBy('postcode.postcode', 'asc')
        ->get();
        // $property           = PropertyHistory::select('property_type', 'id')->get()->unique('property_type');
        // $property           = $this->commonPropQuery()->select('property_type')->get()->unique('property_type');
        $property           = $this->commonFreshPropQuery()->select('property_history.property_type')->get()->unique('property_type');
        $prop_count         = count($property);
        $agency             = Agency::get();
        $userOutcodes       = $this->userOutcodes();
        $campaign           = 1;
        $filter             = true;
        $enable_export      = User::find($user_id)->enable_export;
        $weeks              = '';
        $search_data        = '';
        $filter_title       = 'All Property Listings(Without Filters)';
        // Filter based on weeks
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                if ($weeks == 1) {
                    $filter_title = 'Property Filtered in 0-4 Weeks';
                } elseif ($weeks == 2) {
                    $filter_title = 'Property Filtered in 4-8 Weeks';
                } elseif ($weeks == 3) {
                    $filter_title = 'Property Filtered in 8-12 Weeks';
                } elseif ($weeks == 4) {
                    $filter_title = 'Property Filtered in 12-16 Weeks';
                } elseif ($weeks == 5) {
                    $filter_title = 'Property Filtered in 16 Weeks or More';
                }
            }
            $search_data        = json_decode(Session::get('search_data'));
            $search_data->weeks = $weeks;
        }

        return view('all-property', compact('page_title', 'filter_title', 'property', 'prop_count', 'post_code', 'userOutcodes', 'user_type', 'active', 'campaign', 'agency', 'filter','search_data','enable_export'));
    }

    public function allPropertyArray(Request $request)
    {
        $sesstion_array =[];
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $user_outcode = $this->userOutcode();
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                $query = $this->commonFreshPropQuery();
                // $query = $query->whereNotIn('property_history.for_sale', [3, 6]);
                // $query = $query->whereNotIn('property_price_history.type', ['Withdrawn', 'STC']);
            }
        } else {
            $query = $this->commonPropQuery();
        }
        $query = $query->whereIn('outcode.id', $user_outcode);
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
            $sesstion_array['outcode'] = $request->input('outcode');
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
                $sesstion_array['post_code'] = $request->input('post_code');

            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
                $sesstion_array['type'] = $request->input('type');
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
                $query->where('property_flag.user_id', $user_id);
                $sesstion_array['flag'] = $request->input('flag');
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
                $sesstion_array['agency'] = $request->input('agency');
            }
        }
        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
                $sesstion_array['platform'] = $request->input('platform');
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('weeks')) {
            $sesstion_array['weeks'] = $request->input('weeks');
            $weeks = $request->weeks;
            $dt = Carbon::now();

            if (in_array($weeks, [1, 2, 3, 4])) {
                if (1 == $weeks) {
                    $start = $dt->copy()->subWeeks(4)->addDays(1)->startOfDay();
                    $end = $dt->copy()->endOfDay();
                } elseif (2 == $weeks) {
                    $start = $dt->copy()->subWeeks(8)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(4)->endOfDay();
                } elseif (3 == $weeks) {
                    $start = $dt->copy()->subWeeks(12)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(8)->endOfDay();
                } elseif (4 == $weeks) {
                    $start = $dt->copy()->subWeeks(16)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(12)->endOfDay();
                }
                $query->whereBetween('house_number.listed_date', [$start, $end]);
            } elseif (5 == $weeks) {
                $query->whereDate('house_number.listed_date', '<=', $dt->copy()->subWeeks(16)->endOfDay());
            }
        }
        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode.outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'agency.name',
            6 => 'property_type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');
            $sesstion_array['search'] = $search;
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%');

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }

            })->orderBy($order, $dir)->get()->unique('house_number_id');
            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }
            });
        }

        $dataArr = [];


        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                //   $urls = $r_count.$z_count;
                //   $urls = '<div class=" d-flex"> <a class="btn btn-primary  btn-xs  text-center"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">3</span>
                //    <a class="btn btn-success  btn-xs" href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">3</span></div>
                //   ';
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                // if (!empty($item->house_number)) {
                $address2 = $item->house_number.', '.$item->address;
                // } else {
                // $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                // if (!empty($item->created_by)) {
                //     $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $st = '';
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                // $nestedData['type'] = $request->outcodeFilter.'--'. $request->input('search.value');
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }
        
        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        $session_data =  json_encode( $sesstion_array);
        Session::put('search_data', $session_data );

        return json_encode($json_data);
    }

    //property flag change
    public function flagChange(Request $request)
    {
        // return $request->all();

        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ('' != $exist_data) { //update
                // $update = PropertyFlag::where('id',$exist_data['id'])
                //     ->update(['flag' => $request->flag]);
                $update = PropertyFlag::find($exist_data['id']);
                $update->fill(['flag' => $request->flag])->save();

                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    //property detail page
    public function propertyDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode.');
            }
        }

        $prop = true;
        $house_number_id = Helper::decryptData($request->property_id);
        $query = $this->commonPropQuery();
        $property_details = $query->where('house_number.id', $house_number_id)->first();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $page_title = $property_details->address;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
        ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('house_number_id', $house_number_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
        ->orderBy('property_price_history.event_date', 'desc')->orderBy('property_price_history.id', 'desc')->get();
        // dd($listing_activity);
        $ambiguity_checker_response = $this->propertyAmbiguityChecker(['data_type' => 'listing_activity_object', 'data' => $listing_activity]);

        $comments = Helper::propertyComment($house_number_id);

        $letter_list = PropertyLetter::join('house_number', 'property_letters.house_number_id', 'house_number.id')
        ->join('letters', 'letters.id', 'property_letters.letter_id')
        ->where('property_letters.house_number_id', $house_number_id)
        ->select('property_letters.*', 'letters.name as letter_name')
        ->get()
        ;

        $current_flag = Helper::propertyFlag($house_number_id);

        $portal_url = $this->houseNumberPortal($house_number_id);
        $campaign_history = PropertyCampaign::where('house_number_id', $house_number_id)
        ->whereHas('campaign')
        ->with('campaign')
        ->withTrashed()
        ->where('user_id', $user_id)
        ->orderBy('id', 'desc')
        ->get()
        ;
        $trigger = $this->trigger;

        return view('property-details', compact('property_details', 'listing_activity', 'ambiguity_checker_response', 'comments', 'letter_list', 'house_number_id', 'prop', 'current_flag', 'portal_url', 'house_number_id', 'campaign_history', 'trigger'));
    }

    //property history details
    public function propertyHistoryDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode');
            }
        }

        $prop = true;
        $property_history_id = Helper::decryptData($request->property_id);

        // dd($property_history_id);

        $property_details = PropertyHistory::join('postcode', 'property_history.postcode_id', 'postcode.id')
        ->where('property_history.id', $property_history_id)
        ->select('property_history.*', 'postcode.postcode as p_code')
        ->first()
        ;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->where('property_history.id', $property_history_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id')
        ->orderBy('property_price_history.event_date', 'desc')->get();

        // dd($listing_activity);

        // $comments = Helper::propertyComment($house_number_id);
        $comments = [];

        // $letter_list = PropertyLetter::join('house_number','property_letters.house_number_id','house_number.id')
        //     ->join('letters','letters.id','property_letters.letter_id')
        //     ->where('property_letters.house_number_id',$house_number_id)
        //     ->select('property_letters.*','letters.name as letter_name')
        //     ->get();

        $letter_list = [];

        $portal_url[0] = ['portal' => $property_details->portal, 'portal_url' => $property_details->portal_url];

        return view('property-history-details', compact('property_details', 'listing_activity', 'comments', 'letter_list', 'prop', 'portal_url'));
    }

    // Check if ambiguous data is provided by the system
    public function propertyAmbiguityChecker($data)
    {
        $data_type = isset($data['data_type']) ? $data['data_type'] : '';
        if ($data_type === 'listing_activity_object') {
            $listing_activity   = isset($data['data']) ? $data['data'] : '';
        } else if ($data_type === 'house_number_id') {
            $house_number_id    = isset($data['data']) ? $data['data'] : '';
            if ($house_number_id != '') {
                $listing_activity   = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
                ->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
                ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
                ->join('agency', 'agency.id', 'property_history.agency_id')
                ->where('house_number_id', $house_number_id)
                ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
                ->orderBy('property_price_history.event_date', 'desc')
                ->orderBy('property_price_history.id', 'desc')
                ->get();
            }
        }
        if (!empty($listing_activity) && count($listing_activity) > 0) {
            // Convert listing fetch to array
            $listing_activity_array = $listing_activity->toArray();
            // prepare portal to agent array
            $list_activity_portal_array         = array_column($listing_activity_array, 'portal');
            $list_activity_agent_array          = array_column($listing_activity_array, 'name');
            $list_activity_portal_agent_array   = [];
            $i = 0;
            // Prepare an event date array out of listing activity with correct position keys
            foreach ($listing_activity_array as $list_key => $list_value) {
                if (isset($listing_activity_array[$i]['event_date']) && $listing_activity_array[$i]['event_date'] != '') {
                    $list_activity_array[$i] = $listing_activity_array[$i]['event_date'];
                }
                $list_activity_portal_agent_array[] = $list_activity_portal_array[$i]." - ".$list_activity_agent_array[$i];
                $i++;
            }
            $unique_list_activity               = array_unique($list_activity_portal_agent_array);
            $unique_agent_activity_count        = count($unique_list_activity);
            // Preparing activity_listing_position_array for comparing in pattern
            $temp_listing_activity_array        = $list_activity_array;
            $count_list_activity_array          = count($list_activity_array);
            $count_activity                     = array_count_values($list_activity_array);
            $total_count_activity               = count($count_activity);
            $cycle_count                        = 0;
            $activity_listing_position_array    = [];
            foreach ($count_activity as $count_activity_key => $count_activity_value) {
                if ($count_activity_value > 1) {
                    $cycle_count += $count_activity_value;
                }
            }
            $temp_cycle_count                   = $cycle_count;
            if ($cycle_count > 0) {
                while ((count($temp_listing_activity_array) >= $cycle_count) && $cycle_count !== 0) {
                    foreach ($count_activity as $count_activity_key => $count_activity_value) {
                        if ($count_activity_value > 1) {
                            $activity_listing_position = '';
                            $activity_listing_position = array_search($count_activity_key, $temp_listing_activity_array);
                            if ($activity_listing_position !== false) {
                                $activity_listing_position_array[$activity_listing_position] = $count_activity_key;
                                unset($temp_listing_activity_array[$activity_listing_position]);
                            }
                        }
                    }
                    $cycle_count--;
                }
                // Sort in DESC order of activity date
                ksort($activity_listing_position_array);
            }
            // ambiguous listing checks
            $ambiguous_listing_flag             = false;
            $type_unfavourable_array            = ['Withdrawn', 'STC', 'Sold'];
            $first_listing_activity_array       = array_key_first($listing_activity_array);
            // $type_unfavourable_array            = ['Withdrawn'];
            if (!empty($activity_listing_position_array)) {
                $total_activity_listing_position_array = count($activity_listing_position_array);
                foreach ($activity_listing_position_array as $alpa_key => $alpa_value) {
                    $next_activity_listing = next($activity_listing_position_array);
                    // pattern checking for ambiguous data provided for a listing through system
                    // Pattern 1
                    if (isset($listing_activity_array[$alpa_key]['type'])
                        && isset($listing_activity_array[$alpa_key+1]['type'])
                        && ($listing_activity_array[$alpa_key]['type'] != $listing_activity_array[$alpa_key+1]['type']) // check if event type is different
                        && ($next_activity_listing !== false) // check if next item available
                        && (in_array($listing_activity_array[$alpa_key]['type'], $type_unfavourable_array)) // check if type is not offer or price related
                        // && ($listing_activity_array[$alpa_key]['portal'] === $listing_activity_array[$alpa_key+1]['portal']) // check if same portal provides the data
                        && ($listing_activity_array[$alpa_key]['event_date'] === $listing_activity_array[$alpa_key+1]['event_date'])) // check if event date is same
                        // && ($listing_activity_array[$alpa_key]['name'] !== $listing_activity_array[$alpa_key+1]['name'])) // check if portal agent is different
                    {
                        $ambiguous_listing_flag = true;
                    }
                }
                // Pattern 2
                if (isset($listing_activity_array[$first_listing_activity_array]['type'])
                    && (in_array($listing_activity_array[$first_listing_activity_array]['type'], $type_unfavourable_array)) // check if first listing type is unfavourable
                    && ($unique_agent_activity_count > 1)) // check if first listing type is unfavourable
                {
                    $ambiguous_listing_flag = true;
                }
            }
            // debuging unit for ambiguous_listing_flag listing data
            // $list_activity_array            = array_column($listing_activity_array, 'event_date');
            // $list_activity_array_filtered   = array_filter($list_activity_array);
            // print("<pre>".print_r($listing_activity_array,true)."</pre>");
            /*print("<pre>".print_r($list_activity_array,true)."</pre>");
            echo "total activity count : $count_list_activity_array";
            print("<pre>".print_r($count_activity,true)."</pre>");
            echo "unique activity count : $total_count_activity<br>";
            print("<pre>".print_r($temp_listing_activity_array,true)."</pre>");
            echo "cycle count : $temp_cycle_count<br>";
            echo "cycle count now: $cycle_count<br>";
            print("<pre>".print_r($activity_listing_position_array,true)."</pre>");*/
            // echo $ambiguous_listing_flag;
            // dd('stopped');
            return $ambiguous_listing_flag;
        }
    }

    //comment delete
    public function deletePropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }

        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $update = PropertyComment::where('id', $request->comment)->delete();
        if ($update) {
            return $this->simpleReturn('success', 'Successfully comment deleted..!');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    //add property comment
    public function addPropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('error', $validator->errors()->first());
        }
        
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $add = new PropertyComment();
        $add->house_number_id = $request->property;
        $add->comment = $request->comment;
        $add->user_id = $user_id;
        if ($add->save()) {
            return Redirect::back()->with('success', 'Successfully comment deleted..!');
        }

        return Redirect::back()->with('error', 'Something went wrong...!');
    }

    //search property
    public function searchProperty(Request $request)
    {
        $search_keyword = $request->search;
        $page_title = 'Addresses or Agencies That Contain: '.$search_keyword;
        session()->put('search_keyword', $search_keyword);

        return view('search-result', compact('page_title'));
    }

    //search property array
    public function searchPropertyArray(Request $request)
    {
        $search_keyword = session()->get('search_keyword');
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        // ->Where('property_history.address', 'like', '%' . $search_keyword . '%')
        // ->orWhere('agency.name', 'like', '%' . $search_keyword . '%')
        // ->orWhere('postcode.postcode', 'like', '%' . $search_keyword . '%');
        // ->orWhere('property_history.property_type', 'like', '%' . $search_keyword . '%');

        $query = $query->where(function ($q) use ($search_keyword) {
            $q->Where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('property_history.property_type', 'like', '%'.$search_keyword.'%')
            ->orWhere('outcode.outcode', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
        });

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'agency',
            4 => 'listed_date',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //  $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)
            ->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()->unique('house_number_id');
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }

                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                //  $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    //new to market
    public function newToMarket()
    {
        $page_title = 'Weekly New to Market';
        $campaign = 1;

        return view('new_to_market', compact('page_title', 'campaign'));
    }

    public function newToMarketArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        $new_date_from = date('Y-m-d', strtotime('-7 days'));
        $new_date_to = date('Y-m-d');

        $query = $query->whereBetween('house_number.listed_date', [$new_date_from, $new_date_to]);

        // dd($items->where('house_number.id',1)->get());
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'price',
            4 => 'listed_date',
            5 => 'agency.name',
            6 => 'type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                $nestedData['outcode'] = $item->outcode;

                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }
                // $listed_date=date('Y-m-d',strtotime($item->listed_date));
                // $today=date('Y-m-d');
                // $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                // $nestedData['listed_date'] = date('d M Y',strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }
                // $nestedData['week'] = $item->week;
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //withdrawn
    public function withdrawn()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Withdrawn';

        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;
        $userOutcodes = $this->userOutcodes();

        return view('withdrawn-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function withdrawnArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        if (1 == $user_type) { //admin
            $data = HouseNumber::get();
        } elseif (2 == $user_type) { //agent
            $user_outcode = $this->userOutcode();
            $outcodes = $this->userOutcodes();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.latest_status', 3);
            // dd($items->where('house_number.id',1)->get());
            if ($request->has('outcode') && '' != $request->input('outcode')) {
                $query->where('outcode.outcode', $request->input('outcode'));
            }

            if ($request->has('post_code')) {
                if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                    $query->where('postcode.id', $request->input('post_code'));
                }
            }
            // dd($request->all());
            if ($request->has('type')) {
                if (('0' != $request->type && 'undefined' != $request->type)) {
                    $query->where('property_history.property_type', $request->input('type'));
                }
            }
            if ($request->has('flag')) {
                if ((0 != $request->flag && 'undefined' != $request->flag)) {
                    $query->where('property_flag.flag', $request->input('flag'));
                }
            }
            if ($request->has('agency')) {
                if ((0 != $request->agency && 'undefined' != $request->agency)) {
                    $query->where('agency.id', $request->input('agency'));
                }
            }
            if ($request->has('platform')) {
                if ((-1 != $request->platform && 'platform' != $request->platform)) {
                    $query->where('property_history.portal', $request->input('platform'));
                }
            }
            if ($request->has('date_from') && $request->has('date_to')) {
                if (('' != $request->date_from && '' != $request->date_to)) {
                    $date_from = date('Y-m-d', strtotime($request->date_from));
                    $date_to = date('Y-m-d', strtotime($request->date_to));
                    $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
                }
            }
        } else { //data entry operator
            $data = HouseNumber::get();
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'withdrawn_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $withdrwan_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item->house_number_id)
                ->where('property_price_history.type', 'Withdrawn')
                ->orderBy('property_price_history.event_date', 'desc')
                ->select('property_price_history.event_date')
                ->first()
                ;

                // $nestedData['withdrawn_date'] = date('d M Y', strtotime( $withdrwan_date->event_date
                $nestedData['withdrawn_date'] = date('d M Y', strtotime($item->withdrawn_date));

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function recentlySold()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Sold - Under Offer';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('recently-sold-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function recentlySoldArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        //print_r($userOutcodes); exit;

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode)
        ->where('house_number.latest_status', '!=', 3)
        ;

        $query = $query->where(function ($q) {
            $q->Where('property_history.sold_stc', 1)
            ->orWhere('property_history.under_offer', 1)
            ;
        });

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            // 5 =>'property_price_history.event_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');

        if (empty($request->input('search.value'))) {
            // $data = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
            // $dd = $query->orderBy($order,$dir)->get()->unique('house_number_id')->toArray();
            // $list = array_slice($data, $start, $limit);
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            //  $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';

                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item['house_number_id']);

                $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item['house_number_id'])
                ->where('property_price_history.type', 'STC')
                ->orderBy('property_price_history.id', 'desc')
                ->limit(1)
                ->select('property_price_history.event_date', 'property_price_history.sold_year')
                ->first()
                ;

                if (!$sold_date) {
                    $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                    ->where('property_house_number.house_number_id', $item['house_number_id'])
                    ->where('property_price_history.type', 'Offer')
                    ->orderBy('property_price_history.id', 'desc')
                    ->limit(1)
                    ->select('property_price_history.event_date', 'property_price_history.sold_year')
                    ->first()
                    ;
                }

                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;

                if ($sold_date) {
                    if ($sold_date->event_date) {
                        $so_date = date('d M Y', strtotime($sold_date->event_date));
                    } else {
                        $so_date = $sold_date->sold_year;
                    }
                } else {
                    $so_date = '';
                }

                $nestedData['sold_date'] = $so_date;

                if ($item['listed_date']) {
                    $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['id'] = $item['house_number_id'];
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item['outcode'];
                if (null == $item['created_by']) {
                    $st = 'By System';
                    $color = '';
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }

                if (!empty($item['house_number'])) {
                    $address2 = $item['house_number'].', '.$item['address'];
                } else {
                    $address2 = $item['house_number'].', '.$item['address'].',<br>  '.$item['postcode'];
                }
                if (!empty($item['created_by'])) {
                    $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                // $nestedData['price'] = "&pound; ".number_format($item['latest_price']);
                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                // $nestedData['sold_date'] =  date('d M Y',strtotime($item['sold_date']));
                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item['title'].' <br> '.ucfirst($item['property_type']);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function priceDrop()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Recent Price Drops        ';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $agency = Agency::get();
        $filter = true;
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('price-drop-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'agency', 'filter', 'totalProp'));
    }

    //price drop
    public function priceDropArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        // if($user_type==1){//admin
        //     $data = HouseNumber::get();

        // }elseif($user_type==2){//agent

        $user_outcode = $this->userOutcode();

        // $query = $this->commonPropQuery();
        // $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.price_change_diff', '<', 0);

        $query = HouseNumber::join('property_house_number', 'property_house_number.house_number_id', 'house_number.id')
        ->join('property_price_history', 'property_price_history.property_history_id', 'property_house_number.property_history_id')
        ->join('property_history', 'property_history.id', 'property_house_number.property_history_id')
        ->join('postcode', 'postcode.id', 'house_number.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_flag', 'property_flag.house_number_id', 'property_house_number.house_number_id')
        ->whereNull('property_house_number.deleted_at')
        ->whereIn('outcode.id', $user_outcode)
        ->Where('property_price_history.type', 'like', '%Price reduced by%')
        ->select('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', 'house_number.latest_price', 'house_number.latest_status', 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id', 'house_number.created_by', 'property_price_history.type', 'property_price_history.event_date')
        ->orderBy('property_price_history.event_date', 'desc')
        ;
        // ->groupBy('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', "house_number.latest_price", "house_number.latest_status", 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id,'property_price_history.type'');

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // dd($items->where('house_number.id',1)->get());

        // }else{//data entry operator
        //     $data = HouseNumber::get();
        // }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'price_change_diff',
            5 => 'house_number.listed_date',
            6 => 'price_drop_date',
            7 => 'agency.name',
            8 => 'property_type',
            9 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        $outcodes = $this->userOutcodes();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //$query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                // $address3=empty($item->title)?$address2:$address1;
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['price_drop'] = '-'.substr($item->type, 16);
                // '&pound;'.number_format($item->price_change_diff);
                $listed_date = date('Y-m-d', strtotime($item->listed_date));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);

                if ($item->listed_date) {
                    $dt = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $dt = 'Hidden from public';
                }

                $nestedData['listed_date'] = $dt;
                $nestedData['price_drop_date'] = date('d M Y', strtotime($item->event_date));
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //export property data
    public function exportPropertyData(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        $file_name = '';

        if ('all_property' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);
                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'All-property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('new_to_market' == $request->mode) {
            $user_outcode = $this->userOutcode();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.new_to_market', 1);
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'New-to-market'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('recent_sold' == $request->mode) {
            $query = $query->where('house_number.latest_status', '!=', 3);

            $data_list = $query->where(function ($q) {
                $q->Where('property_history.sold_stc', 1)->orWhere('property_history.under_offer', 1);
            })->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Recentl-sold'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('hot_list' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
        } elseif ('withdrawn' == $request->mode) {
            $data_list = $query->where('house_number.latest_status', 2)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Withdrawn Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['withdrawn_date'] = date('d-m-Y', strtotime($item->withdrawn_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Withdrawn-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('price_drop' == $request->mode) {
            $data_list = $query->where('house_number.price_change_diff', '<', 0)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['price_drop'] = number_format($item->price_change_diff);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['price_drop_date'] = date('d-m-Y', strtotime($item->price_drop_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Price-drop-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('all_type_property' == $request->mode) {
            $user_outcode = $this->userOutcode();
            $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
            $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();
            $list1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
            ->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
            ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
            ->where('property_house_number.house_number_id', null)
            ->where('property_house_number.property_history_id', null)
            ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title')->get()->toArray();

            $list2 = $this->commonPropQuery();
            $list2 = $list2->whereIn('outcode.id', $user_outcode)->get()->unique('house_number_id')->toArray();
            $data_list = array_merge($list1, $list2);

            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['price'] = number_format($item['latest_price']);
                $arr['listed_date'] = date('d-m-Y', strtotime($item['listed_date']));
                if ($item['house_number_id']) {
                    $arr['address'] = $item['house_number'].', '.$item['address'].', '.$item['postcode'];
                    $arr['agency'] = Helper::agencyName($item['house_number_id']);
                } else {
                    $arr['address'] = $item['address'].', '.$item['postcode'];
                    $arr['agency'] = $item['name'];
                }
                $arr['type'] = ucfirst($item['property_type']);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'all-property-data'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } else {
            $search_keyword = session()->get('search_keyword');
            $query = $query->where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
            $data_list = $query->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        }

        $url = url('storage/property-downloads/'.$file_name);
        $data['url'] = $url;
        $data['name'] = $file_name;

        return $data;
    }

    //Hotlist
    public function hotList()
    {
        $page_title = 'Hotlist';

        return view('hotlist', compact('page_title'));
    }

    public function hotListArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'listed_date',
            4 => 'agency',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;

            // $totalFiltered =$query->Where('property_history.address', 'like', '%' . $search . '%')
            // ->orWhere('agency.name', 'like', '%' . $search . '%')
            // ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            // ->count();
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                $nestedData['agency'] = $item->agency_name;
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['comments'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    public function filterByPostcode(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $post_code = $request->postCode;
        if ($post_code) {
        }

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ($exist_data) { //update
                $update = PropertyFlag::where('id', $exist_data['id'])
                ->update(['flag' => $request->flag])
                ;
                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    public function propertyTrigger($tab)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Property';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $active = 2;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $enable_export = User::find($user_id)->enable_export;
        $userOutcodes = $this->userOutcodes();

        if (1 == $tab) {
            $filter = true;
            $active = 1;
            $agency = Agency::get();
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');

            return view('property-all-type', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'agency', 'property', 'filter','enable_export'));
        }
        if (2 == $tab) {
            $campaign = 1;
            $active = 2;
            $filter = true;
            $agency = Agency::get();

            return view('all-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'active', 'user_type', 'filter', 'agency', 'campaign','enable_export'));
        }
        if (3 == $tab) {
            $active = 3;
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');
            $filter = true;
            $post_code = PostCode::orderBy('postcode', 'asc')->get()->pluck('postcode', 'id');
            return view('common.property.property-without-address', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'property', 'filter','enable_export'));
        }

        return view('all-property', compact('page_title', 'post_code', 'userOutcodes', 'active', 'user_type','enable_export'));
    }

    public function allTypeProperty()
    {
        $page_title = 'Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $active = 1;

        return view('admin.property-all-type', compact('page_title', 'post_code', 'user_type', 'active'));
    }

    public function allTypePropertyArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
        $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();

        $query1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
        ->where('property_house_number.house_number_id', null)
        ->where('property_house_number.property_history_id', null)
        ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title');

        // ->get()->toArray();

        $query2 = $this->commonPropQuery();
        $query2 = $query2->whereIn('outcode.id', $user_outcode);

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query1->where('outcode.outcode', $request->input('outcode'));
            $query2->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query1->where('postcode.id', $request->input('post_code'));
                $query2->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('platform')) {
            if (('-1' != $request->platform && 'undefined' != $request->platform)) {
                $query1->where('property_history.portal', $request->input('platform'));
                $query2->where('property_history.portal', $request->input('platform'));
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query1->whereBetween('property_history.listed_date', [$date_from, $date_to]);
                $query2->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query1->where('property_history.property_type', $request->input('type'));
                $query2->where('property_history.property_type', $request->input('type'));
            }
        }

        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query2->where('property_flag.flag', $request->input('flag'));
            }
        }

        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query1->where('agency.id', $request->input('agency'));
                $query2->where('agency.id', $request->input('agency'));
            }
        }
        // ->get()->unique('house_number_id')->toArray();

        // $all_data = array_merge($all, $withoutHouseNumber);

        // Table's primary key
        $primaryKey = 'id';
        // $outcodes = $this->userOutcodes();

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'latest_price',
            3 => 'listed_date',
            4 => 'agency_name',
            5 => 'property_type',
            6 => 'flag',
        ];

        $q_data1 = $query1->get()->toArray();
        $q_data2 = $query2->get()->unique('house_number_id')->toArray();
        $all_data = array_merge($q_data1, $q_data2);
        $totalData = count($all_data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        if (empty($request->input('search.value'))) {
            $list1 = $query1->orderBy($order, $dir)->get()->toArray();
            $list2 = $query2->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

            $data_val = array_merge($list1, $list2); //echo "<pre>"; print_r($data_val); exit;
        } else {
            $search = $request->input('search.value');

            $list1 = $query1->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->toArray()
            ;

            $list2 = $query2->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->unique('house_number_id')
            ->toArray()
            ;

            $data_val = array_merge($list1, $list2);

            // $totalFiltered = $query->where(function($q) use ($search) {
            //     $q->Where('property_history.address', 'like', '%' . $search . '%')
            //         ->orWhere('agency.name', 'like', '%' . $search . '%')
            //     ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            //     ->orWhere('outcode.outcode', 'like', '%' . $search . '%')
            //     ->orWhere('postcode.postcode', 'like', '%' . $search . '%');

            // })->count();
            $totalFiltered = count($data_val);
        }

        $dataArr = [];
        $outcodes = $this->userOutcodes();

        if (!empty($data_val)) {
            $data_val = array_slice($data_val, $start, $limit); // return the first five elements

            $i = 0;
            foreach ($data_val as $item) {
                $nestedData['outcode'] = $item['outcode'];
                $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                if ($item['listed_date']) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                // $nestedData['price'] = "&pound;" . number_format($item['latest_price']);

                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                $nestedData['type'] = ucfirst($item['title']).'<br> '.ucfirst($item['property_type']);

                if ($item['house_number_id']) {
                    //portal url
                    $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                    $platform = '';
                    $r = 0;
                    $z = 0;
                    $r_count = '';
                    $z_count = '';
                    if (count($portal_url)) {
                        $platform = '<div class=" d-flex">';
                        foreach ($portal_url as $url) {
                            // $r_count="";$z_count="";
                            if (0 == $url->portal) {
                                ++$r;
                            } else {
                                ++$z;
                            }
                        }
                        if ($r > 0) {
                            $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                        }
                        if ($z > 0) {
                            $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                        }
                        $platform .= $r_count.$z_count;
                        $platform .= '</div>';
                    }
                    $encrpt_id = Helper::encryptData($item['house_number_id']);
                    $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                    ++$i;
                    $nestedData['id'] = $item['house_number_id'];
                    $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                    if (!empty($item['house_number'])) {
                        $address2 = $item['house_number'].' , '.$item['address'];
                    } else {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    if (!empty($item['created_by'])) {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    // $address3=empty($item['title'])?$address2:$address1;
                    if (null == $item['created_by']) {
                        $st = 'By System';
                        $color = '';
                    } else {
                        $st = 'By Manual';
                        $color="style='color:#edf124'";
                    }
                    $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                    // $nestedData['price'] = "&pound; ".number_format($item['latest_price']) ;

                    $nestedData['type'] = $item['title'].'<br> '.ucfirst($item['property_type']);
                    $nestedData['portal_url'] = $platform;
                    $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                    $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
                    $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();
                    $dataArr[] = $nestedData;
                } else {


                    $encrpt_id = Helper::encryptData($item['id']);
                    $nestedData['id'] = '';

                    $url = route('propertyHistoryDetails', ['property_id' => $encrpt_id]);

                    $nestedData['property_number'] = '<div class="input-group">
                    <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                    <div class="input-group-append">
                    <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item['id'].'"><i class="flaticon-paper-plane icon-lg"></i></button>
                    </div>
                    </div>';

                    $nestedData['address'] = "<a  href='{$url}'>{$item['address']},  {$item['postcode']}</a>";

                    $nestedData['agency'] = $item['name'];

                    if (0 == $item['portal']) {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">1</span></div>';
                    } else {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">1</span></div>';
                    }

                    $nestedData['flag'] = 0;
                    $nestedData['letter_count'] = 0;
                    ++$i;

                    $dataArr[] = $nestedData;
                }
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutPostcode(Request $request)
    {
        $page_title = 'Without Postcode Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;
        $without_data = true;

        return view('property-without-postcode', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutPostcodeArray(Request $request)
    {
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('postcode.postcode', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;
        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;
                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                // $nestedData['type'] = $totalFiltered;
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutAgent(Request $request)
    {
        $page_title = 'Without Agent Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;

        $without_data = true;

        return view('property-without-agent', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutAgentArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('agency.name', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;

        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;

                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function getProbateData()
    {
     //$user_outcode = $this->userOutcode();
        if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
            $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
            $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
        }
        $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
        $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
        $user_outcode = $selOutcodeCollection;

        $query = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
        ->join("postcode", "postcode.id", "probate_property.postcode_id")
        ->join("outcode", "outcode.id", "postcode.outcode_id")
        ->leftjoin("probate_property_flag", "probate_property_flag.probate_id", "probate_property.id")
        ->whereIn('outcode.id', $user_outcode)
        ->where('probate_property.status',1)
        ->whereDate('publication_date', '>', Carbon::now()->subDays(530));
        $data = $query
        ->select('postcode.postcode','outcode.outcode','probate_property.address_1','probate_property.address_2','probate_property.firstname','probate_property.surname','executor.company_name','executor.first_name','executor.sur_name','probate_property.publication_date','probate_property.claim_date','probate_property.id')
        ->get();


    return Datatables::of($data)

  ->editColumn('outcode', function ($model) {
   return $model->postcode;
 })
  ->editColumn('address', function ($model) {
$pid = Crypt::encrypt($model->id);
    return '<a href="' . url('property-detail?pid=' . $pid ) . '" title="View">'.$model->address_1.' '.$model->address_2.'</a>';
 })  
      ->editColumn('executer', function ($model) {
    $executor = $model->company_name ? $model->company_name : $model->first_name." ".$model->sur_name;
   return $executor;
  })
  ->editColumn('publication_date', function ($model) {
    $publicationDate =  date('d-m-Y', strtotime($model->publication_date));  
   return $publicationDate;
 })
   ->editColumn('claim_date', function ($model) {
    $claim_date =  date('d-m-Y', strtotime($model->claim_date));  
   return $claim_date;
 })
  

->editColumn('created_date', function ($model) {
    return 1;
})


  ->editColumn('action', function ($model) {
    $idG = Crypt::encrypt($model->id);
    // $user = \App\User::find($model->user_id);
    // Auth::loginUsingId($user->id, true);

            return '<a href="' . url('boat-for-sale/' . $model->slug ) . '" type="" target="_blank" class="act-sp" title="View"><i class="fas fa-file"></i></a>  <a href="' . url('edit-boat-info/' . $idG) . '" type="" target="_blank" class="act-sp" title="Login as user to edit"><i class="fas fa-edit"></i></a> ';
        })

        ->rawColumns(['s#','name','address','status','expiry','action'])
        ->make(true);
    }

    public function propertyDetail(Request $request)
    {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::where('status',1)->findOrFail($propId); 
            if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
                $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
                $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
            }
            $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
            $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
            $user_outcode = $selOutcodeCollection;
        // $probateProperties = ProbateProperty::where('status',1)->orderBy('id', 'DESC')->take(10)->get(['address_1','latitude','longitude','id'])->toArray();
            $probateProperties = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
            ->join("postcode", "postcode.id", "probate_property.postcode_id")
            ->join("outcode", "outcode.id", "postcode.outcode_id")
            ->whereIn('outcode.id', $user_outcode)
            ->where('probate_property.status',1)->orderBy('probate_property.id', 'DESC')->take(10)->get(['probate_property.address_1','probate_property.latitude','probate_property.longitude','probate_property.id'])->toArray();
            return view('admin.probate-property-detail')->with(['probateProperty' => $probateProperty,'probateProperties' => $probateProperties]);
        }
    }
    public function getProbatePopupData(Request $request) {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            //$probateProperty = ProbateProperty::where('status',1)->findOrFail($propId);
            $probateProperty = ProbateProperty::join('executor', 'probate_property.executor_id','executor.id')
                ->join("postcode", "postcode.id", "probate_property.postcode_id")
                ->join("outcode", "outcode.id", "postcode.outcode_id")
                ->leftJoin("property_sale_history", "property_sale_history.property_history_id", "probate_property.id")
                ->select('probate_property.*', 'executor.company_name as executor_comapnyname', 'executor.first_name','executor.sur_name','executor.address_1 as executor_address1','executor.address_2 as executor_address2','executor.town as executor_town','executor.postcode as executor_postcode', 'executor.phone as executor_phone','executor.email as executor_email', 'postcode.postcode as postcode_postcode','property_sale_history.price as property_sale_price','property_sale_history.sale_date as property_sale_date','property_sale_history.sold_year as property_sold_year')
                ->findOrFail($propId);
            $str = $probateProperty->address_1;
            $addr1 = explode(" ",$str);
            $postcode = $probateProperty->postcode_postcode;
            $soldHistory = SoldHistory::where('house_no', $addr1[0])->where('postcode',$postcode)->get();
            $returnHTML = view('admin.popup-content',['probateProperty'=>$probateProperty, 'soldHistory' => $soldHistory])->render();

            return response()->json( array('success' => true, 'html'=>$returnHTML) );
        }
    }

    public function getProbateTooltipData(Request $request) {
        if ($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::join('outcode','probate_property.outcode_id','outcode.id')               ->where('status',1)
                               ->findOrFail($propId); 
            $returnHTML = view('admin.tooltip-content',['probateProperty'=>$probateProperty])->render(); 

            return response()->json(array('success' => true, 'html'=>$returnHTML));
        }
    }

    public function getSchoolTooltipData(Request $request) {
        $sch_latitude = $request->latitude;
        $sch_longitude = $request->longitude;
        $prop_latitude = $request->prop_latitude;
        $prop_longitude = $request->prop_longitude;
        $radius = 1.5;
        $schools = School::selectRaw("*,
        ( 3956 * acos( cos( radians(?) ) *
        cos( radians( latitude ) )
        * cos( radians( longitude ) - radians(?)
        ) + sin( radians(?) ) *
        sin( radians( latitude ) ) )
        ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
        ->where([
        ['latitude',$sch_latitude],
        ['longitude',$sch_longitude]
        ])
        ->having('distance', '<=', $radius)->first();

        $returnHTML = view('admin.school-content',['schoolAddress'=>$schools])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getEpcDetails(Request $request) {
        $full_address = explode("|",$request->address);
        $address = trim(trim($full_address[0]," "),",");
        $outcode = trim($full_address[1]," ");
        $epc_url = 'https://epc.opendatacommunities.org/api/v1/domestic/search?postcode='.$outcode.'&address='.$address;
        $epc_client = new Client();
        $epc_results = $epc_client->get($epc_url, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Authorization' => 'Basic c2hhcnU2MzFAZ21haWwuY29tOmQwZjE1Y2ZjNDlmNDlkYzQ2NWQzZjMyYjdlZDMzYjU5OTk3Y2YzYmY='
            ]
        ]);

        $epc_status = $epc_results->getStatusCode();
        $epc_contents = json_decode($epc_results->getBody(), true);
        $epc_details = $epc_contents['rows'];
        $allEpc['data'] = $epc_details[0];
        $returnHTML = view('admin.epc-content',['epcDetails'=>$allEpc ])->render(); 
        return response()->json( array('success' => true, 'html'=>$returnHTML));
    }

    public function getSchoolFullDetails(Request $request) {
        $latitude = $request->sch_latitude;
        $longitude = $request->sch_longitude;

        $schoolDetails = School::where([
            ['latitude',$latitude],
            ['longitude',$longitude]
        ])->first();

        $returnHTML = view('admin.school-full-details',['schoolDetails'=>$schoolDetails])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function schoolCoordUpdate(Request $request) {
        $schools_count = SchoolList::count();
        $count = 0;
        DB::table('schools')->orderBy('urn')->where('latitude', null)->chunk(100, function ($list) {
            foreach ($list as $school) {
                $count =+ 1;
                $urn = $school->urn;
                $school_name = $school->schname;
                $school_latitude = $school->latitude;
                $school_longitude = $school->longitude;
                if ($school_latitude  == null && $school_longitude == null) {
                    $school_address = $school->schname.', '.$school->street.', '.$school->town.', '.$school->postcode;
                    $google_autocomplete = "https://maps.googleapis.com/maps/api/place/autocomplete/json?input=".$school_address."&key=".config('map.apiKey');
                    $autocomplete_client = new Client();
                    $autocomplete_result = $autocomplete_client->get($google_autocomplete, [
                        'verify' => false,
                        'headers' => [
                            'Accept' => 'application/json',
                        ]
                    ]);
                    $autocomplete_status = $autocomplete_result->getStatusCode();
                    $autocomplete_response = json_decode($autocomplete_result->getBody(), true);
                    foreach ($autocomplete_response['predictions'] as $autocomplete_res) {
                        $google_placeId = "https://maps.googleapis.com/maps/api/place/details/json?place_id=".$autocomplete_res['place_id']."&key=".config('map.apiKey');
                        $placeId_client = new Client();
                        $placeId_result = $placeId_client->get($google_placeId, [
                            'verify' => false,
                            'headers' => [
                                'Accept' => 'application/json',
                            ]
                        ]);
                        $placeId_status = $placeId_result->getStatusCode();
                        $placeId_response = json_decode($placeId_result->getBody(), true);
                        $school_location = $placeId_response['result']['geometry']['location'];
                        $lat = $school_location['latitude'];
                        $lng = $school_location['longitude'];
                        School::where("urn",$urn)
                            ->update(
                                array(
                                    "latitude" => $lat,
                                    "longitude" => $lng,
                                )
                            );
                    }
                }
            }
        });
        if ($schools_count == $count) {
            return response()->json(array('success' => true, 'message' => 'Finished'));
        } else {
            return response()->json(array('success' => false, 'message' => 'Error'));
        }
    }

    public function getSchoolList(Request $request) {
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $sc_phase[] = $request->phase;       
        $radius = 1.5;
        $schools = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)->where('rating','!=',null);

                // if (in_array("Primary", $sc_phase))
                // {
                //    $schools = $schools->orWhere('phase_of_education', 'Primary'); 
                // }
                //  if (in_array("Secondary", $sc_phase))
                // {
                //     dd($sc_phase);
                //    $schools = $schools->orWhere('phase_of_education', 'Secondary'); 
                // }

                 $schools =  $schools->get();
         
        return response()->json( array('success' => true, 'html'=>$schools));
    }

    public function nearestSchoolList(Request $request) {
        $post_string = explode(' ', $request->postcode);
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $radius = 1.5;
        $schools_list = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)
                    ->orderBy('distance', 'ASC')
                    ->get();
        
        $returnHTML = view('admin.nearest-school-list',['schoolsList'=>$schools_list])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPhotos(Request $request) {
        $returnHTML = view('admin.photos-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }
    
    public function getFloorPlan(Request $request) {
        $returnHTML = view('admin.floorPlan-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPlanning(Request $request) {
        $returnHTML = view('admin.planning-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getSoldInStreet(Request $request) {
        $postcode = $request->postcode;
        $soldHistory = SoldHistory::select('sold_date','sold_price','tenure','house_no','addr2','addr5','addr6')->where('postcode',$postcode)->get();    
        $result = array();
         foreach ($soldHistory as $sold) {
         $result[$sold['house_no']][] = $sold;
         }
             dd($result);
        $returnHTML = view('admin.soldInStreet-content',['soldHistory'=>$soldHistory])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getMarketData(Request $request) {
        $returnHTML = view('admin.marketData-content')->render(); 
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getOfcom(Request $request) {
        $postcode = $request->postcode;
        $ofcomApi = "https://api-proxy.ofcom.org.uk/broadband/coverage/".str_replace(' ', '', $postcode);
        $ofcom_client = new Client();
        $ofcom_results = $ofcom_client->get($ofcomApi, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Ocp-Apim-Subscription-Key' => config('map.ofcomKey'),
            ]
        ]);
        $ofcom_status = $ofcom_results->getStatusCode();
        $ofcom_contents = json_decode($ofcom_results->getBody(), true);
        $ofcom_result = [];
        foreach ($ofcom_contents['Availability'] as $ofcom_results) {
            $ofcom_result['MaxBbPredictedDown'] = $ofcom_results['MaxBbPredictedDown'];
            $ofcom_result['MaxBbPredictedUp'] = $ofcom_results['MaxBbPredictedUp'];
            $ofcom_result['MaxSfbbPredictedDown'] = $ofcom_results['MaxSfbbPredictedDown'];
            $ofcom_result['MaxSfbbPredictedUp'] = $ofcom_results['MaxSfbbPredictedUp'];
            $ofcom_result['MaxUfbbPredictedDown'] = $ofcom_results['MaxUfbbPredictedDown'];
            $ofcom_result['MaxUfbbPredictedUp'] = $ofcom_results['MaxUfbbPredictedUp'];
            $ofcom_result['MaxPredictedDown'] = $ofcom_results['MaxPredictedDown'];
            $ofcom_result['MaxPredictedUp'] = $ofcom_results['MaxPredictedUp'];
            $ofcom_res['data'] = $ofcom_result;
        }
        $returnHTML = view('admin.ofcom-content',['ofcomDetails'=>$ofcom_res['data']])->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function documents(Request $request) {
        $returnHTML = view('admin.documents')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }


    public function allProperty(Request $request)
    {
        $user_id            = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title         = 'Property Listings';
        $active             = 2;
        $user_type          = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $post_code          = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')
        ->where('user_id', $user_id)
        ->join('postcode', 'outcode.id', 'postcode.outcode_id')
        ->where('user_outcode.status', 1)
        ->where('outcode.archive', 0)
        ->select('postcode.*')
        ->orderBy('postcode.postcode', 'asc')
        ->get();
        // $property           = PropertyHistory::select('property_type', 'id')->get()->unique('property_type');
        // $property           = $this->commonPropQuery()->select('property_type')->get()->unique('property_type');
        $property           = $this->commonFreshPropQuery()->select('property_history.property_type')->get()->unique('property_type');
        $prop_count         = count($property);
        $agency             = Agency::get();
        $userOutcodes       = $this->userOutcodes();
        $campaign           = 1;
        $filter             = true;
        $enable_export      = User::find($user_id)->enable_export;
        $weeks              = '';
        $search_data        = '';
        $filter_title       = 'All Property Listings(Without Filters)';
        // Filter based on weeks
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                if ($weeks == 1) {
                    $filter_title = 'Property Filtered in 0-4 Weeks';
                } elseif ($weeks == 2) {
                    $filter_title = 'Property Filtered in 4-8 Weeks';
                } elseif ($weeks == 3) {
                    $filter_title = 'Property Filtered in 8-12 Weeks';
                } elseif ($weeks == 4) {
                    $filter_title = 'Property Filtered in 12-16 Weeks';
                } elseif ($weeks == 5) {
                    $filter_title = 'Property Filtered in 16 Weeks or More';
                }
            }
            $search_data        = json_decode(Session::get('search_data'));
            $search_data->weeks = $weeks;
        }

        return view('all-property', compact('page_title', 'filter_title', 'property', 'prop_count', 'post_code', 'userOutcodes', 'user_type', 'active', 'campaign', 'agency', 'filter','search_data','enable_export'));
    }

    public function allPropertyArray(Request $request)
    {
        $sesstion_array =[];
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $user_outcode = $this->userOutcode();
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                $query = $this->commonFreshPropQuery();
                // $query = $query->whereNotIn('property_history.for_sale', [3, 6]);
                // $query = $query->whereNotIn('property_price_history.type', ['Withdrawn', 'STC']);
            }
        } else {
            $query = $this->commonPropQuery();
        }
        $query = $query->whereIn('outcode.id', $user_outcode);
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
            $sesstion_array['outcode'] = $request->input('outcode');
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
                $sesstion_array['post_code'] = $request->input('post_code');

            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
                $sesstion_array['type'] = $request->input('type');
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
                $query->where('property_flag.user_id', $user_id);
                $sesstion_array['flag'] = $request->input('flag');
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
                $sesstion_array['agency'] = $request->input('agency');
            }
        }
        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
                $sesstion_array['platform'] = $request->input('platform');
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('weeks')) {
            $sesstion_array['weeks'] = $request->input('weeks');
            $weeks = $request->weeks;
            $dt = Carbon::now();

            if (in_array($weeks, [1, 2, 3, 4])) {
                if (1 == $weeks) {
                    $start = $dt->copy()->subWeeks(4)->addDays(1)->startOfDay();
                    $end = $dt->copy()->endOfDay();
                } elseif (2 == $weeks) {
                    $start = $dt->copy()->subWeeks(8)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(4)->endOfDay();
                } elseif (3 == $weeks) {
                    $start = $dt->copy()->subWeeks(12)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(8)->endOfDay();
                } elseif (4 == $weeks) {
                    $start = $dt->copy()->subWeeks(16)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(12)->endOfDay();
                }
                $query->whereBetween('house_number.listed_date', [$start, $end]);
            } elseif (5 == $weeks) {
                $query->whereDate('house_number.listed_date', '<=', $dt->copy()->subWeeks(16)->endOfDay());
            }
        }
        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode.outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'agency.name',
            6 => 'property_type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');
            $sesstion_array['search'] = $search;
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%');

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }

            })->orderBy($order, $dir)->get()->unique('house_number_id');
            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }
            });
        }

        $dataArr = [];


        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                //   $urls = $r_count.$z_count;
                //   $urls = '<div class=" d-flex"> <a class="btn btn-primary  btn-xs  text-center"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">3</span>
                //    <a class="btn btn-success  btn-xs" href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">3</span></div>
                //   ';
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                // if (!empty($item->house_number)) {
                $address2 = $item->house_number.', '.$item->address;
                // } else {
                // $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                // if (!empty($item->created_by)) {
                //     $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $st = '';
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                // $nestedData['type'] = $request->outcodeFilter.'--'. $request->input('search.value');
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }
        
        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        $session_data =  json_encode( $sesstion_array);
        Session::put('search_data', $session_data );

        return json_encode($json_data);
    }

    //property flag change
    public function flagChange(Request $request)
    {
        // return $request->all();

        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ('' != $exist_data) { //update
                // $update = PropertyFlag::where('id',$exist_data['id'])
                //     ->update(['flag' => $request->flag]);
                $update = PropertyFlag::find($exist_data['id']);
                $update->fill(['flag' => $request->flag])->save();

                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    //property detail page
    public function propertyDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode.');
            }
        }

        $prop = true;
        $house_number_id = Helper::decryptData($request->property_id);
        $query = $this->commonPropQuery();
        $property_details = $query->where('house_number.id', $house_number_id)->first();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $page_title = $property_details->address;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
        ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('house_number_id', $house_number_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
        ->orderBy('property_price_history.event_date', 'desc')->orderBy('property_price_history.id', 'desc')->get();
        // dd($listing_activity);
        $ambiguity_checker_response = $this->propertyAmbiguityChecker(['data_type' => 'listing_activity_object', 'data' => $listing_activity]);

        $comments = Helper::propertyComment($house_number_id);

        $letter_list = PropertyLetter::join('house_number', 'property_letters.house_number_id', 'house_number.id')
        ->join('letters', 'letters.id', 'property_letters.letter_id')
        ->where('property_letters.house_number_id', $house_number_id)
        ->select('property_letters.*', 'letters.name as letter_name')
        ->get()
        ;

        $current_flag = Helper::propertyFlag($house_number_id);

        $portal_url = $this->houseNumberPortal($house_number_id);
        $campaign_history = PropertyCampaign::where('house_number_id', $house_number_id)
        ->whereHas('campaign')
        ->with('campaign')
        ->withTrashed()
        ->where('user_id', $user_id)
        ->orderBy('id', 'desc')
        ->get()
        ;
        $trigger = $this->trigger;

        return view('property-details', compact('property_details', 'listing_activity', 'ambiguity_checker_response', 'comments', 'letter_list', 'house_number_id', 'prop', 'current_flag', 'portal_url', 'house_number_id', 'campaign_history', 'trigger'));
    }

    //property history details
    public function propertyHistoryDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode');
            }
        }

        $prop = true;
        $property_history_id = Helper::decryptData($request->property_id);

        // dd($property_history_id);

        $property_details = PropertyHistory::join('postcode', 'property_history.postcode_id', 'postcode.id')
        ->where('property_history.id', $property_history_id)
        ->select('property_history.*', 'postcode.postcode as p_code')
        ->first()
        ;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->where('property_history.id', $property_history_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id')
        ->orderBy('property_price_history.event_date', 'desc')->get();

        // dd($listing_activity);

        // $comments = Helper::propertyComment($house_number_id);
        $comments = [];

        // $letter_list = PropertyLetter::join('house_number','property_letters.house_number_id','house_number.id')
        //     ->join('letters','letters.id','property_letters.letter_id')
        //     ->where('property_letters.house_number_id',$house_number_id)
        //     ->select('property_letters.*','letters.name as letter_name')
        //     ->get();

        $letter_list = [];

        $portal_url[0] = ['portal' => $property_details->portal, 'portal_url' => $property_details->portal_url];

        return view('property-history-details', compact('property_details', 'listing_activity', 'comments', 'letter_list', 'prop', 'portal_url'));
    }

    // Check if ambiguous data is provided by the system
    public function propertyAmbiguityChecker($data)
    {
        $data_type = isset($data['data_type']) ? $data['data_type'] : '';
        if ($data_type === 'listing_activity_object') {
            $listing_activity   = isset($data['data']) ? $data['data'] : '';
        } else if ($data_type === 'house_number_id') {
            $house_number_id    = isset($data['data']) ? $data['data'] : '';
            if ($house_number_id != '') {
                $listing_activity   = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
                ->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
                ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
                ->join('agency', 'agency.id', 'property_history.agency_id')
                ->where('house_number_id', $house_number_id)
                ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
                ->orderBy('property_price_history.event_date', 'desc')
                ->orderBy('property_price_history.id', 'desc')
                ->get();
            }
        }
        if (!empty($listing_activity) && count($listing_activity) > 0) {
            // Convert listing fetch to array
            $listing_activity_array = $listing_activity->toArray();
            // prepare portal to agent array
            $list_activity_portal_array         = array_column($listing_activity_array, 'portal');
            $list_activity_agent_array          = array_column($listing_activity_array, 'name');
            $list_activity_portal_agent_array   = [];
            $i = 0;
            // Prepare an event date array out of listing activity with correct position keys
            foreach ($listing_activity_array as $list_key => $list_value) {
                if (isset($listing_activity_array[$i]['event_date']) && $listing_activity_array[$i]['event_date'] != '') {
                    $list_activity_array[$i] = $listing_activity_array[$i]['event_date'];
                }
                $list_activity_portal_agent_array[] = $list_activity_portal_array[$i]." - ".$list_activity_agent_array[$i];
                $i++;
            }
            $unique_list_activity               = array_unique($list_activity_portal_agent_array);
            $unique_agent_activity_count        = count($unique_list_activity);
            // Preparing activity_listing_position_array for comparing in pattern
            $temp_listing_activity_array        = $list_activity_array;
            $count_list_activity_array          = count($list_activity_array);
            $count_activity                     = array_count_values($list_activity_array);
            $total_count_activity               = count($count_activity);
            $cycle_count                        = 0;
            $activity_listing_position_array    = [];
            foreach ($count_activity as $count_activity_key => $count_activity_value) {
                if ($count_activity_value > 1) {
                    $cycle_count += $count_activity_value;
                }
            }
            $temp_cycle_count                   = $cycle_count;
            if ($cycle_count > 0) {
                while ((count($temp_listing_activity_array) >= $cycle_count) && $cycle_count !== 0) {
                    foreach ($count_activity as $count_activity_key => $count_activity_value) {
                        if ($count_activity_value > 1) {
                            $activity_listing_position = '';
                            $activity_listing_position = array_search($count_activity_key, $temp_listing_activity_array);
                            if ($activity_listing_position !== false) {
                                $activity_listing_position_array[$activity_listing_position] = $count_activity_key;
                                unset($temp_listing_activity_array[$activity_listing_position]);
                            }
                        }
                    }
                    $cycle_count--;
                }
                // Sort in DESC order of activity date
                ksort($activity_listing_position_array);
            }
            // ambiguous listing checks
            $ambiguous_listing_flag             = false;
            $type_unfavourable_array            = ['Withdrawn', 'STC', 'Sold'];
            $first_listing_activity_array       = array_key_first($listing_activity_array);
            // $type_unfavourable_array            = ['Withdrawn'];
            if (!empty($activity_listing_position_array)) {
                $total_activity_listing_position_array = count($activity_listing_position_array);
                foreach ($activity_listing_position_array as $alpa_key => $alpa_value) {
                    $next_activity_listing = next($activity_listing_position_array);
                    // pattern checking for ambiguous data provided for a listing through system
                    // Pattern 1
                    if (isset($listing_activity_array[$alpa_key]['type'])
                        && isset($listing_activity_array[$alpa_key+1]['type'])
                        && ($listing_activity_array[$alpa_key]['type'] != $listing_activity_array[$alpa_key+1]['type']) // check if event type is different
                        && ($next_activity_listing !== false) // check if next item available
                        && (in_array($listing_activity_array[$alpa_key]['type'], $type_unfavourable_array)) // check if type is not offer or price related
                        // && ($listing_activity_array[$alpa_key]['portal'] === $listing_activity_array[$alpa_key+1]['portal']) // check if same portal provides the data
                        && ($listing_activity_array[$alpa_key]['event_date'] === $listing_activity_array[$alpa_key+1]['event_date'])) // check if event date is same
                        // && ($listing_activity_array[$alpa_key]['name'] !== $listing_activity_array[$alpa_key+1]['name'])) // check if portal agent is different
                    {
                        $ambiguous_listing_flag = true;
                    }
                }
                // Pattern 2
                if (isset($listing_activity_array[$first_listing_activity_array]['type'])
                    && (in_array($listing_activity_array[$first_listing_activity_array]['type'], $type_unfavourable_array)) // check if first listing type is unfavourable
                    && ($unique_agent_activity_count > 1)) // check if first listing type is unfavourable
                {
                    $ambiguous_listing_flag = true;
                }
            }
            // debuging unit for ambiguous_listing_flag listing data
            // $list_activity_array            = array_column($listing_activity_array, 'event_date');
            // $list_activity_array_filtered   = array_filter($list_activity_array);
            // print("<pre>".print_r($listing_activity_array,true)."</pre>");
            /*print("<pre>".print_r($list_activity_array,true)."</pre>");
            echo "total activity count : $count_list_activity_array";
            print("<pre>".print_r($count_activity,true)."</pre>");
            echo "unique activity count : $total_count_activity<br>";
            print("<pre>".print_r($temp_listing_activity_array,true)."</pre>");
            echo "cycle count : $temp_cycle_count<br>";
            echo "cycle count now: $cycle_count<br>";
            print("<pre>".print_r($activity_listing_position_array,true)."</pre>");*/
            // echo $ambiguous_listing_flag;
            // dd('stopped');
            return $ambiguous_listing_flag;
        }
    }

    //comment delete
    public function deletePropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }

        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $update = PropertyComment::where('id', $request->comment)->delete();
        if ($update) {
            return $this->simpleReturn('success', 'Successfully comment deleted..!');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    //add property comment
    public function addPropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('error', $validator->errors()->first());
        }
        
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $add = new PropertyComment();
        $add->house_number_id = $request->property;
        $add->comment = $request->comment;
        $add->user_id = $user_id;
        if ($add->save()) {
            return Redirect::back()->with('success', 'Successfully comment deleted..!');
        }

        return Redirect::back()->with('error', 'Something went wrong...!');
    }

    //search property
    public function searchProperty(Request $request)
    {
        $search_keyword = $request->search;
        $page_title = 'Addresses or Agencies That Contain: '.$search_keyword;
        session()->put('search_keyword', $search_keyword);

        return view('search-result', compact('page_title'));
    }

    //search property array
    public function searchPropertyArray(Request $request)
    {
        $search_keyword = session()->get('search_keyword');
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        // ->Where('property_history.address', 'like', '%' . $search_keyword . '%')
        // ->orWhere('agency.name', 'like', '%' . $search_keyword . '%')
        // ->orWhere('postcode.postcode', 'like', '%' . $search_keyword . '%');
        // ->orWhere('property_history.property_type', 'like', '%' . $search_keyword . '%');

        $query = $query->where(function ($q) use ($search_keyword) {
            $q->Where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('property_history.property_type', 'like', '%'.$search_keyword.'%')
            ->orWhere('outcode.outcode', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
        });

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'agency',
            4 => 'listed_date',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //  $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)
            ->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()->unique('house_number_id');
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }

                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                //  $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    //new to market
    public function newToMarket()
    {
        $page_title = 'Weekly New to Market';
        $campaign = 1;

        return view('new_to_market', compact('page_title', 'campaign'));
    }

    public function newToMarketArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        $new_date_from = date('Y-m-d', strtotime('-7 days'));
        $new_date_to = date('Y-m-d');

        $query = $query->whereBetween('house_number.listed_date', [$new_date_from, $new_date_to]);

        // dd($items->where('house_number.id',1)->get());
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'price',
            4 => 'listed_date',
            5 => 'agency.name',
            6 => 'type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                $nestedData['outcode'] = $item->outcode;

                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }
                // $listed_date=date('Y-m-d',strtotime($item->listed_date));
                // $today=date('Y-m-d');
                // $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                // $nestedData['listed_date'] = date('d M Y',strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }
                // $nestedData['week'] = $item->week;
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //withdrawn
    public function withdrawn()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Withdrawn';

        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;
        $userOutcodes = $this->userOutcodes();

        return view('withdrawn-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function withdrawnArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        if (1 == $user_type) { //admin
            $data = HouseNumber::get();
        } elseif (2 == $user_type) { //agent
            $user_outcode = $this->userOutcode();
            $outcodes = $this->userOutcodes();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.latest_status', 3);
            // dd($items->where('house_number.id',1)->get());
            if ($request->has('outcode') && '' != $request->input('outcode')) {
                $query->where('outcode.outcode', $request->input('outcode'));
            }

            if ($request->has('post_code')) {
                if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                    $query->where('postcode.id', $request->input('post_code'));
                }
            }
            // dd($request->all());
            if ($request->has('type')) {
                if (('0' != $request->type && 'undefined' != $request->type)) {
                    $query->where('property_history.property_type', $request->input('type'));
                }
            }
            if ($request->has('flag')) {
                if ((0 != $request->flag && 'undefined' != $request->flag)) {
                    $query->where('property_flag.flag', $request->input('flag'));
                }
            }
            if ($request->has('agency')) {
                if ((0 != $request->agency && 'undefined' != $request->agency)) {
                    $query->where('agency.id', $request->input('agency'));
                }
            }
            if ($request->has('platform')) {
                if ((-1 != $request->platform && 'platform' != $request->platform)) {
                    $query->where('property_history.portal', $request->input('platform'));
                }
            }
            if ($request->has('date_from') && $request->has('date_to')) {
                if (('' != $request->date_from && '' != $request->date_to)) {
                    $date_from = date('Y-m-d', strtotime($request->date_from));
                    $date_to = date('Y-m-d', strtotime($request->date_to));
                    $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
                }
            }
        } else { //data entry operator
            $data = HouseNumber::get();
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'withdrawn_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $withdrwan_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item->house_number_id)
                ->where('property_price_history.type', 'Withdrawn')
                ->orderBy('property_price_history.event_date', 'desc')
                ->select('property_price_history.event_date')
                ->first()
                ;

                // $nestedData['withdrawn_date'] = date('d M Y', strtotime( $withdrwan_date->event_date
                $nestedData['withdrawn_date'] = date('d M Y', strtotime($item->withdrawn_date));

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function recentlySold()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Sold - Under Offer';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('recently-sold-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function recentlySoldArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        //print_r($userOutcodes); exit;

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode)
        ->where('house_number.latest_status', '!=', 3)
        ;

        $query = $query->where(function ($q) {
            $q->Where('property_history.sold_stc', 1)
            ->orWhere('property_history.under_offer', 1)
            ;
        });

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            // 5 =>'property_price_history.event_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');

        if (empty($request->input('search.value'))) {
            // $data = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
            // $dd = $query->orderBy($order,$dir)->get()->unique('house_number_id')->toArray();
            // $list = array_slice($data, $start, $limit);
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            //  $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';

                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item['house_number_id']);

                $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item['house_number_id'])
                ->where('property_price_history.type', 'STC')
                ->orderBy('property_price_history.id', 'desc')
                ->limit(1)
                ->select('property_price_history.event_date', 'property_price_history.sold_year')
                ->first()
                ;

                if (!$sold_date) {
                    $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                    ->where('property_house_number.house_number_id', $item['house_number_id'])
                    ->where('property_price_history.type', 'Offer')
                    ->orderBy('property_price_history.id', 'desc')
                    ->limit(1)
                    ->select('property_price_history.event_date', 'property_price_history.sold_year')
                    ->first()
                    ;
                }

                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;

                if ($sold_date) {
                    if ($sold_date->event_date) {
                        $so_date = date('d M Y', strtotime($sold_date->event_date));
                    } else {
                        $so_date = $sold_date->sold_year;
                    }
                } else {
                    $so_date = '';
                }

                $nestedData['sold_date'] = $so_date;

                if ($item['listed_date']) {
                    $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['id'] = $item['house_number_id'];
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item['outcode'];
                if (null == $item['created_by']) {
                    $st = 'By System';
                    $color = '';
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }

                if (!empty($item['house_number'])) {
                    $address2 = $item['house_number'].', '.$item['address'];
                } else {
                    $address2 = $item['house_number'].', '.$item['address'].',<br>  '.$item['postcode'];
                }
                if (!empty($item['created_by'])) {
                    $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                // $nestedData['price'] = "&pound; ".number_format($item['latest_price']);
                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                // $nestedData['sold_date'] =  date('d M Y',strtotime($item['sold_date']));
                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item['title'].' <br> '.ucfirst($item['property_type']);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function priceDrop()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Recent Price Drops        ';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $agency = Agency::get();
        $filter = true;
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('price-drop-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'agency', 'filter', 'totalProp'));
    }

    //price drop
    public function priceDropArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        // if($user_type==1){//admin
        //     $data = HouseNumber::get();

        // }elseif($user_type==2){//agent

        $user_outcode = $this->userOutcode();

        // $query = $this->commonPropQuery();
        // $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.price_change_diff', '<', 0);

        $query = HouseNumber::join('property_house_number', 'property_house_number.house_number_id', 'house_number.id')
        ->join('property_price_history', 'property_price_history.property_history_id', 'property_house_number.property_history_id')
        ->join('property_history', 'property_history.id', 'property_house_number.property_history_id')
        ->join('postcode', 'postcode.id', 'house_number.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_flag', 'property_flag.house_number_id', 'property_house_number.house_number_id')
        ->whereNull('property_house_number.deleted_at')
        ->whereIn('outcode.id', $user_outcode)
        ->Where('property_price_history.type', 'like', '%Price reduced by%')
        ->select('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', 'house_number.latest_price', 'house_number.latest_status', 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id', 'house_number.created_by', 'property_price_history.type', 'property_price_history.event_date')
        ->orderBy('property_price_history.event_date', 'desc')
        ;
        // ->groupBy('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', "house_number.latest_price", "house_number.latest_status", 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id,'property_price_history.type'');

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // dd($items->where('house_number.id',1)->get());

        // }else{//data entry operator
        //     $data = HouseNumber::get();
        // }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'price_change_diff',
            5 => 'house_number.listed_date',
            6 => 'price_drop_date',
            7 => 'agency.name',
            8 => 'property_type',
            9 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        $outcodes = $this->userOutcodes();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //$query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                // $address3=empty($item->title)?$address2:$address1;
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['price_drop'] = '-'.substr($item->type, 16);
                // '&pound;'.number_format($item->price_change_diff);
                $listed_date = date('Y-m-d', strtotime($item->listed_date));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);

                if ($item->listed_date) {
                    $dt = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $dt = 'Hidden from public';
                }

                $nestedData['listed_date'] = $dt;
                $nestedData['price_drop_date'] = date('d M Y', strtotime($item->event_date));
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //export property data
    public function exportPropertyData(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        $file_name = '';

        if ('all_property' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);
                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'All-property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('new_to_market' == $request->mode) {
            $user_outcode = $this->userOutcode();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.new_to_market', 1);
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'New-to-market'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('recent_sold' == $request->mode) {
            $query = $query->where('house_number.latest_status', '!=', 3);

            $data_list = $query->where(function ($q) {
                $q->Where('property_history.sold_stc', 1)->orWhere('property_history.under_offer', 1);
            })->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Recentl-sold'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('hot_list' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
        } elseif ('withdrawn' == $request->mode) {
            $data_list = $query->where('house_number.latest_status', 2)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Withdrawn Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['withdrawn_date'] = date('d-m-Y', strtotime($item->withdrawn_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Withdrawn-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('price_drop' == $request->mode) {
            $data_list = $query->where('house_number.price_change_diff', '<', 0)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['price_drop'] = number_format($item->price_change_diff);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['price_drop_date'] = date('d-m-Y', strtotime($item->price_drop_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Price-drop-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('all_type_property' == $request->mode) {
            $user_outcode = $this->userOutcode();
            $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
            $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();
            $list1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
            ->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
            ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
            ->where('property_house_number.house_number_id', null)
            ->where('property_house_number.property_history_id', null)
            ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title')->get()->toArray();

            $list2 = $this->commonPropQuery();
            $list2 = $list2->whereIn('outcode.id', $user_outcode)->get()->unique('house_number_id')->toArray();
            $data_list = array_merge($list1, $list2);

            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['price'] = number_format($item['latest_price']);
                $arr['listed_date'] = date('d-m-Y', strtotime($item['listed_date']));
                if ($item['house_number_id']) {
                    $arr['address'] = $item['house_number'].', '.$item['address'].', '.$item['postcode'];
                    $arr['agency'] = Helper::agencyName($item['house_number_id']);
                } else {
                    $arr['address'] = $item['address'].', '.$item['postcode'];
                    $arr['agency'] = $item['name'];
                }
                $arr['type'] = ucfirst($item['property_type']);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'all-property-data'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } else {
            $search_keyword = session()->get('search_keyword');
            $query = $query->where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
            $data_list = $query->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        }

        $url = url('storage/property-downloads/'.$file_name);
        $data['url'] = $url;
        $data['name'] = $file_name;

        return $data;
    }

    //Hotlist
    public function hotList()
    {
        $page_title = 'Hotlist';

        return view('hotlist', compact('page_title'));
    }

    public function hotListArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'listed_date',
            4 => 'agency',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;

            // $totalFiltered =$query->Where('property_history.address', 'like', '%' . $search . '%')
            // ->orWhere('agency.name', 'like', '%' . $search . '%')
            // ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            // ->count();
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                $nestedData['agency'] = $item->agency_name;
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['comments'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    public function filterByPostcode(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $post_code = $request->postCode;
        if ($post_code) {
        }

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ($exist_data) { //update
                $update = PropertyFlag::where('id', $exist_data['id'])
                ->update(['flag' => $request->flag])
                ;
                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    public function propertyTrigger($tab)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Property';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $active = 2;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $enable_export = User::find($user_id)->enable_export;
        $userOutcodes = $this->userOutcodes();

        if (1 == $tab) {
            $filter = true;
            $active = 1;
            $agency = Agency::get();
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');

            return view('property-all-type', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'agency', 'property', 'filter','enable_export'));
        }
        if (2 == $tab) {
            $campaign = 1;
            $active = 2;
            $filter = true;
            $agency = Agency::get();

            return view('all-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'active', 'user_type', 'filter', 'agency', 'campaign','enable_export'));
        }
        if (3 == $tab) {
            $active = 3;
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');
            $filter = true;
            $post_code = PostCode::orderBy('postcode', 'asc')->get()->pluck('postcode', 'id');
            return view('common.property.property-without-address', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'property', 'filter','enable_export'));
        }

        return view('all-property', compact('page_title', 'post_code', 'userOutcodes', 'active', 'user_type','enable_export'));
    }

    public function allTypeProperty()
    {
        $page_title = 'Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $active = 1;

        return view('admin.property-all-type', compact('page_title', 'post_code', 'user_type', 'active'));
    }

    public function allTypePropertyArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
        $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();

        $query1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
        ->where('property_house_number.house_number_id', null)
        ->where('property_house_number.property_history_id', null)
        ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title');

        // ->get()->toArray();

        $query2 = $this->commonPropQuery();
        $query2 = $query2->whereIn('outcode.id', $user_outcode);

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query1->where('outcode.outcode', $request->input('outcode'));
            $query2->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query1->where('postcode.id', $request->input('post_code'));
                $query2->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('platform')) {
            if (('-1' != $request->platform && 'undefined' != $request->platform)) {
                $query1->where('property_history.portal', $request->input('platform'));
                $query2->where('property_history.portal', $request->input('platform'));
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query1->whereBetween('property_history.listed_date', [$date_from, $date_to]);
                $query2->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query1->where('property_history.property_type', $request->input('type'));
                $query2->where('property_history.property_type', $request->input('type'));
            }
        }

        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query2->where('property_flag.flag', $request->input('flag'));
            }
        }

        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query1->where('agency.id', $request->input('agency'));
                $query2->where('agency.id', $request->input('agency'));
            }
        }
        // ->get()->unique('house_number_id')->toArray();

        // $all_data = array_merge($all, $withoutHouseNumber);

        // Table's primary key
        $primaryKey = 'id';
        // $outcodes = $this->userOutcodes();

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'latest_price',
            3 => 'listed_date',
            4 => 'agency_name',
            5 => 'property_type',
            6 => 'flag',
        ];

        $q_data1 = $query1->get()->toArray();
        $q_data2 = $query2->get()->unique('house_number_id')->toArray();
        $all_data = array_merge($q_data1, $q_data2);
        $totalData = count($all_data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        if (empty($request->input('search.value'))) {
            $list1 = $query1->orderBy($order, $dir)->get()->toArray();
            $list2 = $query2->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

            $data_val = array_merge($list1, $list2); //echo "<pre>"; print_r($data_val); exit;
        } else {
            $search = $request->input('search.value');

            $list1 = $query1->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->toArray()
            ;

            $list2 = $query2->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->unique('house_number_id')
            ->toArray()
            ;

            $data_val = array_merge($list1, $list2);

            // $totalFiltered = $query->where(function($q) use ($search) {
            //     $q->Where('property_history.address', 'like', '%' . $search . '%')
            //         ->orWhere('agency.name', 'like', '%' . $search . '%')
            //     ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            //     ->orWhere('outcode.outcode', 'like', '%' . $search . '%')
            //     ->orWhere('postcode.postcode', 'like', '%' . $search . '%');

            // })->count();
            $totalFiltered = count($data_val);
        }

        $dataArr = [];
        $outcodes = $this->userOutcodes();

        if (!empty($data_val)) {
            $data_val = array_slice($data_val, $start, $limit); // return the first five elements

            $i = 0;
            foreach ($data_val as $item) {
                $nestedData['outcode'] = $item['outcode'];
                $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                if ($item['listed_date']) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                // $nestedData['price'] = "&pound;" . number_format($item['latest_price']);

                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                $nestedData['type'] = ucfirst($item['title']).'<br> '.ucfirst($item['property_type']);

                if ($item['house_number_id']) {
                    //portal url
                    $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                    $platform = '';
                    $r = 0;
                    $z = 0;
                    $r_count = '';
                    $z_count = '';
                    if (count($portal_url)) {
                        $platform = '<div class=" d-flex">';
                        foreach ($portal_url as $url) {
                            // $r_count="";$z_count="";
                            if (0 == $url->portal) {
                                ++$r;
                            } else {
                                ++$z;
                            }
                        }
                        if ($r > 0) {
                            $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                        }
                        if ($z > 0) {
                            $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                        }
                        $platform .= $r_count.$z_count;
                        $platform .= '</div>';
                    }
                    $encrpt_id = Helper::encryptData($item['house_number_id']);
                    $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                    ++$i;
                    $nestedData['id'] = $item['house_number_id'];
                    $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                    if (!empty($item['house_number'])) {
                        $address2 = $item['house_number'].' , '.$item['address'];
                    } else {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    if (!empty($item['created_by'])) {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    // $address3=empty($item['title'])?$address2:$address1;
                    if (null == $item['created_by']) {
                        $st = 'By System';
                        $color = '';
                    } else {
                        $st = 'By Manual';
                        $color="style='color:#edf124'";
                    }
                    $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                    // $nestedData['price'] = "&pound; ".number_format($item['latest_price']) ;

                    $nestedData['type'] = $item['title'].'<br> '.ucfirst($item['property_type']);
                    $nestedData['portal_url'] = $platform;
                    $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                    $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
                    $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();
                    $dataArr[] = $nestedData;
                } else {


                    $encrpt_id = Helper::encryptData($item['id']);
                    $nestedData['id'] = '';

                    $url = route('propertyHistoryDetails', ['property_id' => $encrpt_id]);

                    $nestedData['property_number'] = '<div class="input-group">
                    <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                    <div class="input-group-append">
                    <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item['id'].'"><i class="flaticon-paper-plane icon-lg"></i></button>
                    </div>
                    </div>';

                    $nestedData['address'] = "<a  href='{$url}'>{$item['address']},  {$item['postcode']}</a>";

                    $nestedData['agency'] = $item['name'];

                    if (0 == $item['portal']) {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">1</span></div>';
                    } else {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">1</span></div>';
                    }

                    $nestedData['flag'] = 0;
                    $nestedData['letter_count'] = 0;
                    ++$i;

                    $dataArr[] = $nestedData;
                }
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutPostcode(Request $request)
    {
        $page_title = 'Without Postcode Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;
        $without_data = true;

        return view('property-without-postcode', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutPostcodeArray(Request $request)
    {
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('postcode.postcode', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;
        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;
                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                // $nestedData['type'] = $totalFiltered;
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutAgent(Request $request)
    {
        $page_title = 'Without Agent Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;

        $without_data = true;

        return view('property-without-agent', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutAgentArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('agency.name', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;

        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;

                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function getProbateData()
    {
     //$user_outcode = $this->userOutcode();
        if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
            $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
            $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
        }
        $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
        $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
        $user_outcode = $selOutcodeCollection;

        $query = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
        ->join("postcode", "postcode.id", "probate_property.postcode_id")
        ->join("outcode", "outcode.id", "postcode.outcode_id")
        ->leftjoin("probate_property_flag", "probate_property_flag.probate_id", "probate_property.id")
        ->whereIn('outcode.id', $user_outcode)
        ->where('probate_property.status',1)
        ->whereDate('publication_date', '>', Carbon::now()->subDays(530));
        $data = $query
        ->select('postcode.postcode','outcode.outcode','probate_property.address_1','probate_property.address_2','probate_property.firstname','probate_property.surname','executor.company_name','executor.first_name','executor.sur_name','probate_property.publication_date','probate_property.claim_date','probate_property.id')
        ->get();


    return Datatables::of($data)

  ->editColumn('outcode', function ($model) {
   return $model->postcode;
 })
  ->editColumn('address', function ($model) {
$pid = Crypt::encrypt($model->id);
    return '<a href="' . url('property-detail?pid=' . $pid ) . '" title="View">'.$model->address_1.' '.$model->address_2.'</a>';
 })  
      ->editColumn('executer', function ($model) {
    $executor = $model->company_name ? $model->company_name : $model->first_name." ".$model->sur_name;
   return $executor;
  })
  ->editColumn('publication_date', function ($model) {
    $publicationDate =  date('d-m-Y', strtotime($model->publication_date));  
   return $publicationDate;
 })
   ->editColumn('claim_date', function ($model) {
    $claim_date =  date('d-m-Y', strtotime($model->claim_date));  
   return $claim_date;
 })
  

->editColumn('created_date', function ($model) {
    return 1;
})


  ->editColumn('action', function ($model) {
    $idG = Crypt::encrypt($model->id);
    // $user = \App\User::find($model->user_id);
    // Auth::loginUsingId($user->id, true);

            return '<a href="' . url('boat-for-sale/' . $model->slug ) . '" type="" target="_blank" class="act-sp" title="View"><i class="fas fa-file"></i></a>  <a href="' . url('edit-boat-info/' . $idG) . '" type="" target="_blank" class="act-sp" title="Login as user to edit"><i class="fas fa-edit"></i></a> ';
        })

        ->rawColumns(['s#','name','address','status','expiry','action'])
        ->make(true);
    }

    public function propertyDetail(Request $request)
    {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::where('status',1)->findOrFail($propId); 
            if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
                $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
                $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
            }
            $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
            $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
            $user_outcode = $selOutcodeCollection;
        // $probateProperties = ProbateProperty::where('status',1)->orderBy('id', 'DESC')->take(10)->get(['address_1','latitude','longitude','id'])->toArray();
            $probateProperties = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
            ->join("postcode", "postcode.id", "probate_property.postcode_id")
            ->join("outcode", "outcode.id", "postcode.outcode_id")
            ->whereIn('outcode.id', $user_outcode)
            ->where('probate_property.status',1)->orderBy('probate_property.id', 'DESC')->take(10)->get(['probate_property.address_1','probate_property.latitude','probate_property.longitude','probate_property.id'])->toArray();
            return view('admin.probate-property-detail')->with(['probateProperty' => $probateProperty,'probateProperties' => $probateProperties]);
        }
    }
    public function getProbatePopupData(Request $request) {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            //$probateProperty = ProbateProperty::where('status',1)->findOrFail($propId);
            $probateProperty = ProbateProperty::join('executor', 'probate_property.executor_id','executor.id')
                ->join("postcode", "postcode.id", "probate_property.postcode_id")
                ->join("outcode", "outcode.id", "postcode.outcode_id")
                ->leftJoin("property_sale_history", "property_sale_history.property_history_id", "probate_property.id")
                ->select('probate_property.*', 'executor.company_name as executor_comapnyname', 'executor.first_name','executor.sur_name','executor.address_1 as executor_address1','executor.address_2 as executor_address2','executor.town as executor_town','executor.postcode as executor_postcode', 'executor.phone as executor_phone','executor.email as executor_email', 'postcode.postcode as postcode_postcode','property_sale_history.price as property_sale_price','property_sale_history.sale_date as property_sale_date','property_sale_history.sold_year as property_sold_year')
                ->findOrFail($propId);
            $str = $probateProperty->address_1;
            $addr1 = explode(" ",$str);
            $postcode = $probateProperty->postcode_postcode;
            $soldHistory = SoldHistory::where('house_no', $addr1[0])->where('postcode',$postcode)->get();
            $returnHTML = view('admin.popup-content',['probateProperty'=>$probateProperty, 'soldHistory' => $soldHistory])->render();

            return response()->json( array('success' => true, 'html'=>$returnHTML) );
        }
    }

    public function getProbateTooltipData(Request $request) {
        if ($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::join('outcode','probate_property.outcode_id','outcode.id')               ->where('status',1)
                               ->findOrFail($propId); 
            $returnHTML = view('admin.tooltip-content',['probateProperty'=>$probateProperty])->render(); 

            return response()->json(array('success' => true, 'html'=>$returnHTML));
        }
    }

    public function getSchoolTooltipData(Request $request) {
        $sch_latitude = $request->latitude;
        $sch_longitude = $request->longitude;
        $prop_latitude = $request->prop_latitude;
        $prop_longitude = $request->prop_longitude;
        $radius = 1.5;
        $schools = School::selectRaw("*,
        ( 3956 * acos( cos( radians(?) ) *
        cos( radians( latitude ) )
        * cos( radians( longitude ) - radians(?)
        ) + sin( radians(?) ) *
        sin( radians( latitude ) ) )
        ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
        ->where([
        ['latitude',$sch_latitude],
        ['longitude',$sch_longitude]
        ])
        ->having('distance', '<=', $radius)->first();

        $returnHTML = view('admin.school-content',['schoolAddress'=>$schools])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getEpcDetails(Request $request) {
        $full_address = explode("|",$request->address);
        $address = trim(trim($full_address[0]," "),",");
        $outcode = trim($full_address[1]," ");
        $epc_url = 'https://epc.opendatacommunities.org/api/v1/domestic/search?postcode='.$outcode.'&address='.$address;
        $epc_client = new Client();
        $epc_results = $epc_client->get($epc_url, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Authorization' => 'Basic c2hhcnU2MzFAZ21haWwuY29tOmQwZjE1Y2ZjNDlmNDlkYzQ2NWQzZjMyYjdlZDMzYjU5OTk3Y2YzYmY='
            ]
        ]);

        $epc_status = $epc_results->getStatusCode();
        $epc_contents = json_decode($epc_results->getBody(), true);
        $epc_details = $epc_contents['rows'];
        $allEpc['data'] = $epc_details[0];
        $returnHTML = view('admin.epc-content',['epcDetails'=>$allEpc ])->render(); 
        return response()->json( array('success' => true, 'html'=>$returnHTML));
    }

    public function getSchoolFullDetails(Request $request) {
        $latitude = $request->sch_latitude;
        $longitude = $request->sch_longitude;

        $schoolDetails = School::where([
            ['latitude',$latitude],
            ['longitude',$longitude]
        ])->first();

        $returnHTML = view('admin.school-full-details',['schoolDetails'=>$schoolDetails])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function schoolCoordUpdate(Request $request) {
        $schools_count = SchoolList::count();
        $count = 0;
        DB::table('schools')->orderBy('urn')->where('latitude', null)->chunk(100, function ($list) {
            foreach ($list as $school) {
                $count =+ 1;
                $urn = $school->urn;
                $school_name = $school->schname;
                $school_latitude = $school->latitude;
                $school_longitude = $school->longitude;
                if ($school_latitude  == null && $school_longitude == null) {
                    $school_address = $school->schname.', '.$school->street.', '.$school->town.', '.$school->postcode;
                    $google_autocomplete = "https://maps.googleapis.com/maps/api/place/autocomplete/json?input=".$school_address."&key=".config('map.apiKey');
                    $autocomplete_client = new Client();
                    $autocomplete_result = $autocomplete_client->get($google_autocomplete, [
                        'verify' => false,
                        'headers' => [
                            'Accept' => 'application/json',
                        ]
                    ]);
                    $autocomplete_status = $autocomplete_result->getStatusCode();
                    $autocomplete_response = json_decode($autocomplete_result->getBody(), true);
                    foreach ($autocomplete_response['predictions'] as $autocomplete_res) {
                        $google_placeId = "https://maps.googleapis.com/maps/api/place/details/json?place_id=".$autocomplete_res['place_id']."&key=".config('map.apiKey');
                        $placeId_client = new Client();
                        $placeId_result = $placeId_client->get($google_placeId, [
                            'verify' => false,
                            'headers' => [
                                'Accept' => 'application/json',
                            ]
                        ]);
                        $placeId_status = $placeId_result->getStatusCode();
                        $placeId_response = json_decode($placeId_result->getBody(), true);
                        $school_location = $placeId_response['result']['geometry']['location'];
                        $lat = $school_location['latitude'];
                        $lng = $school_location['longitude'];
                        School::where("urn",$urn)
                            ->update(
                                array(
                                    "latitude" => $lat,
                                    "longitude" => $lng,
                                )
                            );
                    }
                }
            }
        });
        if ($schools_count == $count) {
            return response()->json(array('success' => true, 'message' => 'Finished'));
        } else {
            return response()->json(array('success' => false, 'message' => 'Error'));
        }
    }

    public function getSchoolList(Request $request) {
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $sc_phase[] = $request->phase;       
        $radius = 1.5;
        $schools = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)->where('rating','!=',null);

                // if (in_array("Primary", $sc_phase))
                // {
                //    $schools = $schools->orWhere('phase_of_education', 'Primary'); 
                // }
                //  if (in_array("Secondary", $sc_phase))
                // {
                //     dd($sc_phase);
                //    $schools = $schools->orWhere('phase_of_education', 'Secondary'); 
                // }

                 $schools =  $schools->get();
         
        return response()->json( array('success' => true, 'html'=>$schools));
    }

    public function nearestSchoolList(Request $request) {
        $post_string = explode(' ', $request->postcode);
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $radius = 1.5;
        $schools_list = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)
                    ->orderBy('distance', 'ASC')
                    ->get();
        
        $returnHTML = view('admin.nearest-school-list',['schoolsList'=>$schools_list])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPhotos(Request $request) {
        $returnHTML = view('admin.photos-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }
    
    public function getFloorPlan(Request $request) {
        $returnHTML = view('admin.floorPlan-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPlanning(Request $request) {
        $returnHTML = view('admin.planning-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getSoldInStreet(Request $request) {
        $postcode = $request->postcode;
        $soldHistory = SoldHistory::select('sold_date','sold_price','tenure','house_no','addr2','addr5','addr6')->where('postcode',$postcode)->get();    
        $result = array();
         foreach ($soldHistory as $sold) {
         $result[$sold['house_no']][] = $sold;
         }
             dd($result);
        $returnHTML = view('admin.soldInStreet-content',['soldHistory'=>$soldHistory])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getMarketData(Request $request) {
        $returnHTML = view('admin.marketData-content')->render(); 
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getOfcom(Request $request) {
        $postcode = $request->postcode;
        $ofcomApi = "https://api-proxy.ofcom.org.uk/broadband/coverage/".str_replace(' ', '', $postcode);
        $ofcom_client = new Client();
        $ofcom_results = $ofcom_client->get($ofcomApi, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Ocp-Apim-Subscription-Key' => config('map.ofcomKey'),
            ]
        ]);
        $ofcom_status = $ofcom_results->getStatusCode();
        $ofcom_contents = json_decode($ofcom_results->getBody(), true);
        $ofcom_result = [];
        foreach ($ofcom_contents['Availability'] as $ofcom_results) {
            $ofcom_result['MaxBbPredictedDown'] = $ofcom_results['MaxBbPredictedDown'];
            $ofcom_result['MaxBbPredictedUp'] = $ofcom_results['MaxBbPredictedUp'];
            $ofcom_result['MaxSfbbPredictedDown'] = $ofcom_results['MaxSfbbPredictedDown'];
            $ofcom_result['MaxSfbbPredictedUp'] = $ofcom_results['MaxSfbbPredictedUp'];
            $ofcom_result['MaxUfbbPredictedDown'] = $ofcom_results['MaxUfbbPredictedDown'];
            $ofcom_result['MaxUfbbPredictedUp'] = $ofcom_results['MaxUfbbPredictedUp'];
            $ofcom_result['MaxPredictedDown'] = $ofcom_results['MaxPredictedDown'];
            $ofcom_result['MaxPredictedUp'] = $ofcom_results['MaxPredictedUp'];
            $ofcom_res['data'] = $ofcom_result;
        }
        $returnHTML = view('admin.ofcom-content',['ofcomDetails'=>$ofcom_res['data']])->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function documents(Request $request) {
        $returnHTML = view('admin.documents')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }


     //All property
    public function allProperty(Request $request)
    {
        $user_id            = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title         = 'Property Listings';
        $active             = 2;
        $user_type          = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $post_code          = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')
        ->where('user_id', $user_id)
        ->join('postcode', 'outcode.id', 'postcode.outcode_id')
        ->where('user_outcode.status', 1)
        ->where('outcode.archive', 0)
        ->select('postcode.*')
        ->orderBy('postcode.postcode', 'asc')
        ->get();
        // $property           = PropertyHistory::select('property_type', 'id')->get()->unique('property_type');
        // $property           = $this->commonPropQuery()->select('property_type')->get()->unique('property_type');
        $property           = $this->commonFreshPropQuery()->select('property_history.property_type')->get()->unique('property_type');
        $prop_count         = count($property);
        $agency             = Agency::get();
        $userOutcodes       = $this->userOutcodes();
        $campaign           = 1;
        $filter             = true;
        $enable_export      = User::find($user_id)->enable_export;
        $weeks              = '';
        $search_data        = '';
        $filter_title       = 'All Property Listings(Without Filters)';
        // Filter based on weeks
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                if ($weeks == 1) {
                    $filter_title = 'Property Filtered in 0-4 Weeks';
                } elseif ($weeks == 2) {
                    $filter_title = 'Property Filtered in 4-8 Weeks';
                } elseif ($weeks == 3) {
                    $filter_title = 'Property Filtered in 8-12 Weeks';
                } elseif ($weeks == 4) {
                    $filter_title = 'Property Filtered in 12-16 Weeks';
                } elseif ($weeks == 5) {
                    $filter_title = 'Property Filtered in 16 Weeks or More';
                }
            }
            $search_data        = json_decode(Session::get('search_data'));
            $search_data->weeks = $weeks;
        }

        return view('all-property', compact('page_title', 'filter_title', 'property', 'prop_count', 'post_code', 'userOutcodes', 'user_type', 'active', 'campaign', 'agency', 'filter','search_data','enable_export'));
    }

    public function allPropertyArray(Request $request)
    {
        $sesstion_array =[];
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $user_outcode = $this->userOutcode();
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                $query = $this->commonFreshPropQuery();
                // $query = $query->whereNotIn('property_history.for_sale', [3, 6]);
                // $query = $query->whereNotIn('property_price_history.type', ['Withdrawn', 'STC']);
            }
        } else {
            $query = $this->commonPropQuery();
        }
        $query = $query->whereIn('outcode.id', $user_outcode);
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
            $sesstion_array['outcode'] = $request->input('outcode');
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
                $sesstion_array['post_code'] = $request->input('post_code');

            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
                $sesstion_array['type'] = $request->input('type');
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
                $query->where('property_flag.user_id', $user_id);
                $sesstion_array['flag'] = $request->input('flag');
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
                $sesstion_array['agency'] = $request->input('agency');
            }
        }
        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
                $sesstion_array['platform'] = $request->input('platform');
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('weeks')) {
            $sesstion_array['weeks'] = $request->input('weeks');
            $weeks = $request->weeks;
            $dt = Carbon::now();

            if (in_array($weeks, [1, 2, 3, 4])) {
                if (1 == $weeks) {
                    $start = $dt->copy()->subWeeks(4)->addDays(1)->startOfDay();
                    $end = $dt->copy()->endOfDay();
                } elseif (2 == $weeks) {
                    $start = $dt->copy()->subWeeks(8)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(4)->endOfDay();
                } elseif (3 == $weeks) {
                    $start = $dt->copy()->subWeeks(12)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(8)->endOfDay();
                } elseif (4 == $weeks) {
                    $start = $dt->copy()->subWeeks(16)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(12)->endOfDay();
                }
                $query->whereBetween('house_number.listed_date', [$start, $end]);
            } elseif (5 == $weeks) {
                $query->whereDate('house_number.listed_date', '<=', $dt->copy()->subWeeks(16)->endOfDay());
            }
        }
        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode.outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'agency.name',
            6 => 'property_type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');
            $sesstion_array['search'] = $search;
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%');

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }

            })->orderBy($order, $dir)->get()->unique('house_number_id');
            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }
            });
        }

        $dataArr = [];


        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                //   $urls = $r_count.$z_count;
                //   $urls = '<div class=" d-flex"> <a class="btn btn-primary  btn-xs  text-center"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">3</span>
                //    <a class="btn btn-success  btn-xs" href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">3</span></div>
                //   ';
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                // if (!empty($item->house_number)) {
                $address2 = $item->house_number.', '.$item->address;
                // } else {
                // $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                // if (!empty($item->created_by)) {
                //     $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $st = '';
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                // $nestedData['type'] = $request->outcodeFilter.'--'. $request->input('search.value');
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }
        
        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        $session_data =  json_encode( $sesstion_array);
        Session::put('search_data', $session_data );

        return json_encode($json_data);
    }

    //property flag change
    public function flagChange(Request $request)
    {
        // return $request->all();

        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ('' != $exist_data) { //update
                // $update = PropertyFlag::where('id',$exist_data['id'])
                //     ->update(['flag' => $request->flag]);
                $update = PropertyFlag::find($exist_data['id']);
                $update->fill(['flag' => $request->flag])->save();

                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    //property detail page
    public function propertyDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode.');
            }
        }

        $prop = true;
        $house_number_id = Helper::decryptData($request->property_id);
        $query = $this->commonPropQuery();
        $property_details = $query->where('house_number.id', $house_number_id)->first();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $page_title = $property_details->address;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
        ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('house_number_id', $house_number_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
        ->orderBy('property_price_history.event_date', 'desc')->orderBy('property_price_history.id', 'desc')->get();
        // dd($listing_activity);
        $ambiguity_checker_response = $this->propertyAmbiguityChecker(['data_type' => 'listing_activity_object', 'data' => $listing_activity]);

        $comments = Helper::propertyComment($house_number_id);

        $letter_list = PropertyLetter::join('house_number', 'property_letters.house_number_id', 'house_number.id')
        ->join('letters', 'letters.id', 'property_letters.letter_id')
        ->where('property_letters.house_number_id', $house_number_id)
        ->select('property_letters.*', 'letters.name as letter_name')
        ->get()
        ;

        $current_flag = Helper::propertyFlag($house_number_id);

        $portal_url = $this->houseNumberPortal($house_number_id);
        $campaign_history = PropertyCampaign::where('house_number_id', $house_number_id)
        ->whereHas('campaign')
        ->with('campaign')
        ->withTrashed()
        ->where('user_id', $user_id)
        ->orderBy('id', 'desc')
        ->get()
        ;
        $trigger = $this->trigger;

        return view('property-details', compact('property_details', 'listing_activity', 'ambiguity_checker_response', 'comments', 'letter_list', 'house_number_id', 'prop', 'current_flag', 'portal_url', 'house_number_id', 'campaign_history', 'trigger'));
    }

    //property history details
    public function propertyHistoryDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode');
            }
        }

        $prop = true;
        $property_history_id = Helper::decryptData($request->property_id);

        // dd($property_history_id);

        $property_details = PropertyHistory::join('postcode', 'property_history.postcode_id', 'postcode.id')
        ->where('property_history.id', $property_history_id)
        ->select('property_history.*', 'postcode.postcode as p_code')
        ->first()
        ;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->where('property_history.id', $property_history_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id')
        ->orderBy('property_price_history.event_date', 'desc')->get();

        // dd($listing_activity);

        // $comments = Helper::propertyComment($house_number_id);
        $comments = [];

        // $letter_list = PropertyLetter::join('house_number','property_letters.house_number_id','house_number.id')
        //     ->join('letters','letters.id','property_letters.letter_id')
        //     ->where('property_letters.house_number_id',$house_number_id)
        //     ->select('property_letters.*','letters.name as letter_name')
        //     ->get();

        $letter_list = [];

        $portal_url[0] = ['portal' => $property_details->portal, 'portal_url' => $property_details->portal_url];

        return view('property-history-details', compact('property_details', 'listing_activity', 'comments', 'letter_list', 'prop', 'portal_url'));
    }

    // Check if ambiguous data is provided by the system
    public function propertyAmbiguityChecker($data)
    {
        $data_type = isset($data['data_type']) ? $data['data_type'] : '';
        if ($data_type === 'listing_activity_object') {
            $listing_activity   = isset($data['data']) ? $data['data'] : '';
        } else if ($data_type === 'house_number_id') {
            $house_number_id    = isset($data['data']) ? $data['data'] : '';
            if ($house_number_id != '') {
                $listing_activity   = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
                ->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
                ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
                ->join('agency', 'agency.id', 'property_history.agency_id')
                ->where('house_number_id', $house_number_id)
                ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
                ->orderBy('property_price_history.event_date', 'desc')
                ->orderBy('property_price_history.id', 'desc')
                ->get();
            }
        }
        if (!empty($listing_activity) && count($listing_activity) > 0) {
            // Convert listing fetch to array
            $listing_activity_array = $listing_activity->toArray();
            // prepare portal to agent array
            $list_activity_portal_array         = array_column($listing_activity_array, 'portal');
            $list_activity_agent_array          = array_column($listing_activity_array, 'name');
            $list_activity_portal_agent_array   = [];
            $i = 0;
            // Prepare an event date array out of listing activity with correct position keys
            foreach ($listing_activity_array as $list_key => $list_value) {
                if (isset($listing_activity_array[$i]['event_date']) && $listing_activity_array[$i]['event_date'] != '') {
                    $list_activity_array[$i] = $listing_activity_array[$i]['event_date'];
                }
                $list_activity_portal_agent_array[] = $list_activity_portal_array[$i]." - ".$list_activity_agent_array[$i];
                $i++;
            }
            $unique_list_activity               = array_unique($list_activity_portal_agent_array);
            $unique_agent_activity_count        = count($unique_list_activity);
            // Preparing activity_listing_position_array for comparing in pattern
            $temp_listing_activity_array        = $list_activity_array;
            $count_list_activity_array          = count($list_activity_array);
            $count_activity                     = array_count_values($list_activity_array);
            $total_count_activity               = count($count_activity);
            $cycle_count                        = 0;
            $activity_listing_position_array    = [];
            foreach ($count_activity as $count_activity_key => $count_activity_value) {
                if ($count_activity_value > 1) {
                    $cycle_count += $count_activity_value;
                }
            }
            $temp_cycle_count                   = $cycle_count;
            if ($cycle_count > 0) {
                while ((count($temp_listing_activity_array) >= $cycle_count) && $cycle_count !== 0) {
                    foreach ($count_activity as $count_activity_key => $count_activity_value) {
                        if ($count_activity_value > 1) {
                            $activity_listing_position = '';
                            $activity_listing_position = array_search($count_activity_key, $temp_listing_activity_array);
                            if ($activity_listing_position !== false) {
                                $activity_listing_position_array[$activity_listing_position] = $count_activity_key;
                                unset($temp_listing_activity_array[$activity_listing_position]);
                            }
                        }
                    }
                    $cycle_count--;
                }
                // Sort in DESC order of activity date
                ksort($activity_listing_position_array);
            }
            // ambiguous listing checks
            $ambiguous_listing_flag             = false;
            $type_unfavourable_array            = ['Withdrawn', 'STC', 'Sold'];
            $first_listing_activity_array       = array_key_first($listing_activity_array);
            // $type_unfavourable_array            = ['Withdrawn'];
            if (!empty($activity_listing_position_array)) {
                $total_activity_listing_position_array = count($activity_listing_position_array);
                foreach ($activity_listing_position_array as $alpa_key => $alpa_value) {
                    $next_activity_listing = next($activity_listing_position_array);
                    // pattern checking for ambiguous data provided for a listing through system
                    // Pattern 1
                    if (isset($listing_activity_array[$alpa_key]['type'])
                        && isset($listing_activity_array[$alpa_key+1]['type'])
                        && ($listing_activity_array[$alpa_key]['type'] != $listing_activity_array[$alpa_key+1]['type']) // check if event type is different
                        && ($next_activity_listing !== false) // check if next item available
                        && (in_array($listing_activity_array[$alpa_key]['type'], $type_unfavourable_array)) // check if type is not offer or price related
                        // && ($listing_activity_array[$alpa_key]['portal'] === $listing_activity_array[$alpa_key+1]['portal']) // check if same portal provides the data
                        && ($listing_activity_array[$alpa_key]['event_date'] === $listing_activity_array[$alpa_key+1]['event_date'])) // check if event date is same
                        // && ($listing_activity_array[$alpa_key]['name'] !== $listing_activity_array[$alpa_key+1]['name'])) // check if portal agent is different
                    {
                        $ambiguous_listing_flag = true;
                    }
                }
                // Pattern 2
                if (isset($listing_activity_array[$first_listing_activity_array]['type'])
                    && (in_array($listing_activity_array[$first_listing_activity_array]['type'], $type_unfavourable_array)) // check if first listing type is unfavourable
                    && ($unique_agent_activity_count > 1)) // check if first listing type is unfavourable
                {
                    $ambiguous_listing_flag = true;
                }
            }
            // debuging unit for ambiguous_listing_flag listing data
            // $list_activity_array            = array_column($listing_activity_array, 'event_date');
            // $list_activity_array_filtered   = array_filter($list_activity_array);
            // print("<pre>".print_r($listing_activity_array,true)."</pre>");
            /*print("<pre>".print_r($list_activity_array,true)."</pre>");
            echo "total activity count : $count_list_activity_array";
            print("<pre>".print_r($count_activity,true)."</pre>");
            echo "unique activity count : $total_count_activity<br>";
            print("<pre>".print_r($temp_listing_activity_array,true)."</pre>");
            echo "cycle count : $temp_cycle_count<br>";
            echo "cycle count now: $cycle_count<br>";
            print("<pre>".print_r($activity_listing_position_array,true)."</pre>");*/
            // echo $ambiguous_listing_flag;
            // dd('stopped');
            return $ambiguous_listing_flag;
        }
    }

    //comment delete
    public function deletePropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }

        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $update = PropertyComment::where('id', $request->comment)->delete();
        if ($update) {
            return $this->simpleReturn('success', 'Successfully comment deleted..!');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    //add property comment
    public function addPropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('error', $validator->errors()->first());
        }
        
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $add = new PropertyComment();
        $add->house_number_id = $request->property;
        $add->comment = $request->comment;
        $add->user_id = $user_id;
        if ($add->save()) {
            return Redirect::back()->with('success', 'Successfully comment deleted..!');
        }

        return Redirect::back()->with('error', 'Something went wrong...!');
    }

    //search property
    public function searchProperty(Request $request)
    {
        $search_keyword = $request->search;
        $page_title = 'Addresses or Agencies That Contain: '.$search_keyword;
        session()->put('search_keyword', $search_keyword);

        return view('search-result', compact('page_title'));
    }

    //search property array
    public function searchPropertyArray(Request $request)
    {
        $search_keyword = session()->get('search_keyword');
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        // ->Where('property_history.address', 'like', '%' . $search_keyword . '%')
        // ->orWhere('agency.name', 'like', '%' . $search_keyword . '%')
        // ->orWhere('postcode.postcode', 'like', '%' . $search_keyword . '%');
        // ->orWhere('property_history.property_type', 'like', '%' . $search_keyword . '%');

        $query = $query->where(function ($q) use ($search_keyword) {
            $q->Where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('property_history.property_type', 'like', '%'.$search_keyword.'%')
            ->orWhere('outcode.outcode', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
        });

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'agency',
            4 => 'listed_date',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //  $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)
            ->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()->unique('house_number_id');
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }

                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                //  $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    //new to market
    public function newToMarket()
    {
        $page_title = 'Weekly New to Market';
        $campaign = 1;

        return view('new_to_market', compact('page_title', 'campaign'));
    }

    public function newToMarketArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        $new_date_from = date('Y-m-d', strtotime('-7 days'));
        $new_date_to = date('Y-m-d');

        $query = $query->whereBetween('house_number.listed_date', [$new_date_from, $new_date_to]);

        // dd($items->where('house_number.id',1)->get());
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'price',
            4 => 'listed_date',
            5 => 'agency.name',
            6 => 'type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                $nestedData['outcode'] = $item->outcode;

                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }
                // $listed_date=date('Y-m-d',strtotime($item->listed_date));
                // $today=date('Y-m-d');
                // $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                // $nestedData['listed_date'] = date('d M Y',strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }
                // $nestedData['week'] = $item->week;
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //withdrawn
    public function withdrawn()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Withdrawn';

        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;
        $userOutcodes = $this->userOutcodes();

        return view('withdrawn-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function withdrawnArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        if (1 == $user_type) { //admin
            $data = HouseNumber::get();
        } elseif (2 == $user_type) { //agent
            $user_outcode = $this->userOutcode();
            $outcodes = $this->userOutcodes();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.latest_status', 3);
            // dd($items->where('house_number.id',1)->get());
            if ($request->has('outcode') && '' != $request->input('outcode')) {
                $query->where('outcode.outcode', $request->input('outcode'));
            }

            if ($request->has('post_code')) {
                if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                    $query->where('postcode.id', $request->input('post_code'));
                }
            }
            // dd($request->all());
            if ($request->has('type')) {
                if (('0' != $request->type && 'undefined' != $request->type)) {
                    $query->where('property_history.property_type', $request->input('type'));
                }
            }
            if ($request->has('flag')) {
                if ((0 != $request->flag && 'undefined' != $request->flag)) {
                    $query->where('property_flag.flag', $request->input('flag'));
                }
            }
            if ($request->has('agency')) {
                if ((0 != $request->agency && 'undefined' != $request->agency)) {
                    $query->where('agency.id', $request->input('agency'));
                }
            }
            if ($request->has('platform')) {
                if ((-1 != $request->platform && 'platform' != $request->platform)) {
                    $query->where('property_history.portal', $request->input('platform'));
                }
            }
            if ($request->has('date_from') && $request->has('date_to')) {
                if (('' != $request->date_from && '' != $request->date_to)) {
                    $date_from = date('Y-m-d', strtotime($request->date_from));
                    $date_to = date('Y-m-d', strtotime($request->date_to));
                    $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
                }
            }
        } else { //data entry operator
            $data = HouseNumber::get();
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'withdrawn_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $withdrwan_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item->house_number_id)
                ->where('property_price_history.type', 'Withdrawn')
                ->orderBy('property_price_history.event_date', 'desc')
                ->select('property_price_history.event_date')
                ->first()
                ;

                // $nestedData['withdrawn_date'] = date('d M Y', strtotime( $withdrwan_date->event_date
                $nestedData['withdrawn_date'] = date('d M Y', strtotime($item->withdrawn_date));

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function recentlySold()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Sold - Under Offer';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('recently-sold-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function recentlySoldArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        //print_r($userOutcodes); exit;

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode)
        ->where('house_number.latest_status', '!=', 3)
        ;

        $query = $query->where(function ($q) {
            $q->Where('property_history.sold_stc', 1)
            ->orWhere('property_history.under_offer', 1)
            ;
        });

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            // 5 =>'property_price_history.event_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');

        if (empty($request->input('search.value'))) {
            // $data = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
            // $dd = $query->orderBy($order,$dir)->get()->unique('house_number_id')->toArray();
            // $list = array_slice($data, $start, $limit);
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            //  $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';

                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item['house_number_id']);

                $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item['house_number_id'])
                ->where('property_price_history.type', 'STC')
                ->orderBy('property_price_history.id', 'desc')
                ->limit(1)
                ->select('property_price_history.event_date', 'property_price_history.sold_year')
                ->first()
                ;

                if (!$sold_date) {
                    $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                    ->where('property_house_number.house_number_id', $item['house_number_id'])
                    ->where('property_price_history.type', 'Offer')
                    ->orderBy('property_price_history.id', 'desc')
                    ->limit(1)
                    ->select('property_price_history.event_date', 'property_price_history.sold_year')
                    ->first()
                    ;
                }

                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;

                if ($sold_date) {
                    if ($sold_date->event_date) {
                        $so_date = date('d M Y', strtotime($sold_date->event_date));
                    } else {
                        $so_date = $sold_date->sold_year;
                    }
                } else {
                    $so_date = '';
                }

                $nestedData['sold_date'] = $so_date;

                if ($item['listed_date']) {
                    $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['id'] = $item['house_number_id'];
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item['outcode'];
                if (null == $item['created_by']) {
                    $st = 'By System';
                    $color = '';
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }

                if (!empty($item['house_number'])) {
                    $address2 = $item['house_number'].', '.$item['address'];
                } else {
                    $address2 = $item['house_number'].', '.$item['address'].',<br>  '.$item['postcode'];
                }
                if (!empty($item['created_by'])) {
                    $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                // $nestedData['price'] = "&pound; ".number_format($item['latest_price']);
                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                // $nestedData['sold_date'] =  date('d M Y',strtotime($item['sold_date']));
                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item['title'].' <br> '.ucfirst($item['property_type']);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function priceDrop()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Recent Price Drops        ';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $agency = Agency::get();
        $filter = true;
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('price-drop-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'agency', 'filter', 'totalProp'));
    }

    //price drop
    public function priceDropArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        // if($user_type==1){//admin
        //     $data = HouseNumber::get();

        // }elseif($user_type==2){//agent

        $user_outcode = $this->userOutcode();

        // $query = $this->commonPropQuery();
        // $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.price_change_diff', '<', 0);

        $query = HouseNumber::join('property_house_number', 'property_house_number.house_number_id', 'house_number.id')
        ->join('property_price_history', 'property_price_history.property_history_id', 'property_house_number.property_history_id')
        ->join('property_history', 'property_history.id', 'property_house_number.property_history_id')
        ->join('postcode', 'postcode.id', 'house_number.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_flag', 'property_flag.house_number_id', 'property_house_number.house_number_id')
        ->whereNull('property_house_number.deleted_at')
        ->whereIn('outcode.id', $user_outcode)
        ->Where('property_price_history.type', 'like', '%Price reduced by%')
        ->select('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', 'house_number.latest_price', 'house_number.latest_status', 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id', 'house_number.created_by', 'property_price_history.type', 'property_price_history.event_date')
        ->orderBy('property_price_history.event_date', 'desc')
        ;
        // ->groupBy('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', "house_number.latest_price", "house_number.latest_status", 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id,'property_price_history.type'');

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // dd($items->where('house_number.id',1)->get());

        // }else{//data entry operator
        //     $data = HouseNumber::get();
        // }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'price_change_diff',
            5 => 'house_number.listed_date',
            6 => 'price_drop_date',
            7 => 'agency.name',
            8 => 'property_type',
            9 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        $outcodes = $this->userOutcodes();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //$query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                // $address3=empty($item->title)?$address2:$address1;
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['price_drop'] = '-'.substr($item->type, 16);
                // '&pound;'.number_format($item->price_change_diff);
                $listed_date = date('Y-m-d', strtotime($item->listed_date));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);

                if ($item->listed_date) {
                    $dt = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $dt = 'Hidden from public';
                }

                $nestedData['listed_date'] = $dt;
                $nestedData['price_drop_date'] = date('d M Y', strtotime($item->event_date));
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //export property data
    public function exportPropertyData(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        $file_name = '';

        if ('all_property' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);
                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'All-property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('new_to_market' == $request->mode) {
            $user_outcode = $this->userOutcode();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.new_to_market', 1);
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'New-to-market'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('recent_sold' == $request->mode) {
            $query = $query->where('house_number.latest_status', '!=', 3);

            $data_list = $query->where(function ($q) {
                $q->Where('property_history.sold_stc', 1)->orWhere('property_history.under_offer', 1);
            })->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Recentl-sold'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('hot_list' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
        } elseif ('withdrawn' == $request->mode) {
            $data_list = $query->where('house_number.latest_status', 2)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Withdrawn Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['withdrawn_date'] = date('d-m-Y', strtotime($item->withdrawn_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Withdrawn-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('price_drop' == $request->mode) {
            $data_list = $query->where('house_number.price_change_diff', '<', 0)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['price_drop'] = number_format($item->price_change_diff);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['price_drop_date'] = date('d-m-Y', strtotime($item->price_drop_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Price-drop-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('all_type_property' == $request->mode) {
            $user_outcode = $this->userOutcode();
            $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
            $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();
            $list1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
            ->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
            ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
            ->where('property_house_number.house_number_id', null)
            ->where('property_house_number.property_history_id', null)
            ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title')->get()->toArray();

            $list2 = $this->commonPropQuery();
            $list2 = $list2->whereIn('outcode.id', $user_outcode)->get()->unique('house_number_id')->toArray();
            $data_list = array_merge($list1, $list2);

            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['price'] = number_format($item['latest_price']);
                $arr['listed_date'] = date('d-m-Y', strtotime($item['listed_date']));
                if ($item['house_number_id']) {
                    $arr['address'] = $item['house_number'].', '.$item['address'].', '.$item['postcode'];
                    $arr['agency'] = Helper::agencyName($item['house_number_id']);
                } else {
                    $arr['address'] = $item['address'].', '.$item['postcode'];
                    $arr['agency'] = $item['name'];
                }
                $arr['type'] = ucfirst($item['property_type']);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'all-property-data'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } else {
            $search_keyword = session()->get('search_keyword');
            $query = $query->where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
            $data_list = $query->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        }

        $url = url('storage/property-downloads/'.$file_name);
        $data['url'] = $url;
        $data['name'] = $file_name;

        return $data;
    }

    //Hotlist
    public function hotList()
    {
        $page_title = 'Hotlist';

        return view('hotlist', compact('page_title'));
    }

    public function hotListArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'listed_date',
            4 => 'agency',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;

            // $totalFiltered =$query->Where('property_history.address', 'like', '%' . $search . '%')
            // ->orWhere('agency.name', 'like', '%' . $search . '%')
            // ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            // ->count();
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                $nestedData['agency'] = $item->agency_name;
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['comments'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    public function filterByPostcode(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $post_code = $request->postCode;
        if ($post_code) {
        }

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ($exist_data) { //update
                $update = PropertyFlag::where('id', $exist_data['id'])
                ->update(['flag' => $request->flag])
                ;
                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    public function propertyTrigger($tab)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Property';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $active = 2;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $enable_export = User::find($user_id)->enable_export;
        $userOutcodes = $this->userOutcodes();

        if (1 == $tab) {
            $filter = true;
            $active = 1;
            $agency = Agency::get();
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');

            return view('property-all-type', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'agency', 'property', 'filter','enable_export'));
        }
        if (2 == $tab) {
            $campaign = 1;
            $active = 2;
            $filter = true;
            $agency = Agency::get();

            return view('all-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'active', 'user_type', 'filter', 'agency', 'campaign','enable_export'));
        }
        if (3 == $tab) {
            $active = 3;
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');
            $filter = true;
            $post_code = PostCode::orderBy('postcode', 'asc')->get()->pluck('postcode', 'id');
            return view('common.property.property-without-address', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'property', 'filter','enable_export'));
        }

        return view('all-property', compact('page_title', 'post_code', 'userOutcodes', 'active', 'user_type','enable_export'));
    }

    public function allTypeProperty()
    {
        $page_title = 'Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $active = 1;

        return view('admin.property-all-type', compact('page_title', 'post_code', 'user_type', 'active'));
    }

    public function allTypePropertyArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
        $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();

        $query1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
        ->where('property_house_number.house_number_id', null)
        ->where('property_house_number.property_history_id', null)
        ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title');

        // ->get()->toArray();

        $query2 = $this->commonPropQuery();
        $query2 = $query2->whereIn('outcode.id', $user_outcode);

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query1->where('outcode.outcode', $request->input('outcode'));
            $query2->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query1->where('postcode.id', $request->input('post_code'));
                $query2->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('platform')) {
            if (('-1' != $request->platform && 'undefined' != $request->platform)) {
                $query1->where('property_history.portal', $request->input('platform'));
                $query2->where('property_history.portal', $request->input('platform'));
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query1->whereBetween('property_history.listed_date', [$date_from, $date_to]);
                $query2->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query1->where('property_history.property_type', $request->input('type'));
                $query2->where('property_history.property_type', $request->input('type'));
            }
        }

        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query2->where('property_flag.flag', $request->input('flag'));
            }
        }

        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query1->where('agency.id', $request->input('agency'));
                $query2->where('agency.id', $request->input('agency'));
            }
        }
        // ->get()->unique('house_number_id')->toArray();

        // $all_data = array_merge($all, $withoutHouseNumber);

        // Table's primary key
        $primaryKey = 'id';
        // $outcodes = $this->userOutcodes();

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'latest_price',
            3 => 'listed_date',
            4 => 'agency_name',
            5 => 'property_type',
            6 => 'flag',
        ];

        $q_data1 = $query1->get()->toArray();
        $q_data2 = $query2->get()->unique('house_number_id')->toArray();
        $all_data = array_merge($q_data1, $q_data2);
        $totalData = count($all_data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        if (empty($request->input('search.value'))) {
            $list1 = $query1->orderBy($order, $dir)->get()->toArray();
            $list2 = $query2->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

            $data_val = array_merge($list1, $list2); //echo "<pre>"; print_r($data_val); exit;
        } else {
            $search = $request->input('search.value');

            $list1 = $query1->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->toArray()
            ;

            $list2 = $query2->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->unique('house_number_id')
            ->toArray()
            ;

            $data_val = array_merge($list1, $list2);

            // $totalFiltered = $query->where(function($q) use ($search) {
            //     $q->Where('property_history.address', 'like', '%' . $search . '%')
            //         ->orWhere('agency.name', 'like', '%' . $search . '%')
            //     ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            //     ->orWhere('outcode.outcode', 'like', '%' . $search . '%')
            //     ->orWhere('postcode.postcode', 'like', '%' . $search . '%');

            // })->count();
            $totalFiltered = count($data_val);
        }

        $dataArr = [];
        $outcodes = $this->userOutcodes();

        if (!empty($data_val)) {
            $data_val = array_slice($data_val, $start, $limit); // return the first five elements

            $i = 0;
            foreach ($data_val as $item) {
                $nestedData['outcode'] = $item['outcode'];
                $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                if ($item['listed_date']) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                // $nestedData['price'] = "&pound;" . number_format($item['latest_price']);

                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                $nestedData['type'] = ucfirst($item['title']).'<br> '.ucfirst($item['property_type']);

                if ($item['house_number_id']) {
                    //portal url
                    $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                    $platform = '';
                    $r = 0;
                    $z = 0;
                    $r_count = '';
                    $z_count = '';
                    if (count($portal_url)) {
                        $platform = '<div class=" d-flex">';
                        foreach ($portal_url as $url) {
                            // $r_count="";$z_count="";
                            if (0 == $url->portal) {
                                ++$r;
                            } else {
                                ++$z;
                            }
                        }
                        if ($r > 0) {
                            $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                        }
                        if ($z > 0) {
                            $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                        }
                        $platform .= $r_count.$z_count;
                        $platform .= '</div>';
                    }
                    $encrpt_id = Helper::encryptData($item['house_number_id']);
                    $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                    ++$i;
                    $nestedData['id'] = $item['house_number_id'];
                    $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                    if (!empty($item['house_number'])) {
                        $address2 = $item['house_number'].' , '.$item['address'];
                    } else {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    if (!empty($item['created_by'])) {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    // $address3=empty($item['title'])?$address2:$address1;
                    if (null == $item['created_by']) {
                        $st = 'By System';
                        $color = '';
                    } else {
                        $st = 'By Manual';
                        $color="style='color:#edf124'";
                    }
                    $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                    // $nestedData['price'] = "&pound; ".number_format($item['latest_price']) ;

                    $nestedData['type'] = $item['title'].'<br> '.ucfirst($item['property_type']);
                    $nestedData['portal_url'] = $platform;
                    $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                    $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
                    $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();
                    $dataArr[] = $nestedData;
                } else {


                    $encrpt_id = Helper::encryptData($item['id']);
                    $nestedData['id'] = '';

                    $url = route('propertyHistoryDetails', ['property_id' => $encrpt_id]);

                    $nestedData['property_number'] = '<div class="input-group">
                    <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                    <div class="input-group-append">
                    <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item['id'].'"><i class="flaticon-paper-plane icon-lg"></i></button>
                    </div>
                    </div>';

                    $nestedData['address'] = "<a  href='{$url}'>{$item['address']},  {$item['postcode']}</a>";

                    $nestedData['agency'] = $item['name'];

                    if (0 == $item['portal']) {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">1</span></div>';
                    } else {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">1</span></div>';
                    }

                    $nestedData['flag'] = 0;
                    $nestedData['letter_count'] = 0;
                    ++$i;

                    $dataArr[] = $nestedData;
                }
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutPostcode(Request $request)
    {
        $page_title = 'Without Postcode Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;
        $without_data = true;

        return view('property-without-postcode', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutPostcodeArray(Request $request)
    {
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('postcode.postcode', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;
        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;
                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                // $nestedData['type'] = $totalFiltered;
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutAgent(Request $request)
    {
        $page_title = 'Without Agent Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;

        $without_data = true;

        return view('property-without-agent', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutAgentArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('agency.name', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;

        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;

                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function getProbateData()
    {
     //$user_outcode = $this->userOutcode();
        if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
            $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
            $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
        }
        $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
        $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
        $user_outcode = $selOutcodeCollection;

        $query = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
        ->join("postcode", "postcode.id", "probate_property.postcode_id")
        ->join("outcode", "outcode.id", "postcode.outcode_id")
        ->leftjoin("probate_property_flag", "probate_property_flag.probate_id", "probate_property.id")
        ->whereIn('outcode.id', $user_outcode)
        ->where('probate_property.status',1)
        ->whereDate('publication_date', '>', Carbon::now()->subDays(530));
        $data = $query
        ->select('postcode.postcode','outcode.outcode','probate_property.address_1','probate_property.address_2','probate_property.firstname','probate_property.surname','executor.company_name','executor.first_name','executor.sur_name','probate_property.publication_date','probate_property.claim_date','probate_property.id')
        ->get();


    return Datatables::of($data)

  ->editColumn('outcode', function ($model) {
   return $model->postcode;
 })
  ->editColumn('address', function ($model) {
$pid = Crypt::encrypt($model->id);
    return '<a href="' . url('property-detail?pid=' . $pid ) . '" title="View">'.$model->address_1.' '.$model->address_2.'</a>';
 })  
      ->editColumn('executer', function ($model) {
    $executor = $model->company_name ? $model->company_name : $model->first_name." ".$model->sur_name;
   return $executor;
  })
  ->editColumn('publication_date', function ($model) {
    $publicationDate =  date('d-m-Y', strtotime($model->publication_date));  
   return $publicationDate;
 })
   ->editColumn('claim_date', function ($model) {
    $claim_date =  date('d-m-Y', strtotime($model->claim_date));  
   return $claim_date;
 })
  

->editColumn('created_date', function ($model) {
    return 1;
})


  ->editColumn('action', function ($model) {
    $idG = Crypt::encrypt($model->id);
    // $user = \App\User::find($model->user_id);
    // Auth::loginUsingId($user->id, true);

            return '<a href="' . url('boat-for-sale/' . $model->slug ) . '" type="" target="_blank" class="act-sp" title="View"><i class="fas fa-file"></i></a>  <a href="' . url('edit-boat-info/' . $idG) . '" type="" target="_blank" class="act-sp" title="Login as user to edit"><i class="fas fa-edit"></i></a> ';
        })

        ->rawColumns(['s#','name','address','status','expiry','action'])
        ->make(true);
    }

    public function propertyDetail(Request $request)
    {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::where('status',1)->findOrFail($propId); 
            if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
                $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
                $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
            }
            $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
            $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
            $user_outcode = $selOutcodeCollection;
        // $probateProperties = ProbateProperty::where('status',1)->orderBy('id', 'DESC')->take(10)->get(['address_1','latitude','longitude','id'])->toArray();
            $probateProperties = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
            ->join("postcode", "postcode.id", "probate_property.postcode_id")
            ->join("outcode", "outcode.id", "postcode.outcode_id")
            ->whereIn('outcode.id', $user_outcode)
            ->where('probate_property.status',1)->orderBy('probate_property.id', 'DESC')->take(10)->get(['probate_property.address_1','probate_property.latitude','probate_property.longitude','probate_property.id'])->toArray();
            return view('admin.probate-property-detail')->with(['probateProperty' => $probateProperty,'probateProperties' => $probateProperties]);
        }
    }
    public function getProbatePopupData(Request $request) {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            //$probateProperty = ProbateProperty::where('status',1)->findOrFail($propId);
            $probateProperty = ProbateProperty::join('executor', 'probate_property.executor_id','executor.id')
                ->join("postcode", "postcode.id", "probate_property.postcode_id")
                ->join("outcode", "outcode.id", "postcode.outcode_id")
                ->leftJoin("property_sale_history", "property_sale_history.property_history_id", "probate_property.id")
                ->select('probate_property.*', 'executor.company_name as executor_comapnyname', 'executor.first_name','executor.sur_name','executor.address_1 as executor_address1','executor.address_2 as executor_address2','executor.town as executor_town','executor.postcode as executor_postcode', 'executor.phone as executor_phone','executor.email as executor_email', 'postcode.postcode as postcode_postcode','property_sale_history.price as property_sale_price','property_sale_history.sale_date as property_sale_date','property_sale_history.sold_year as property_sold_year')
                ->findOrFail($propId);
            $str = $probateProperty->address_1;
            $addr1 = explode(" ",$str);
            $postcode = $probateProperty->postcode_postcode;
            $soldHistory = SoldHistory::where('house_no', $addr1[0])->where('postcode',$postcode)->get();
            $returnHTML = view('admin.popup-content',['probateProperty'=>$probateProperty, 'soldHistory' => $soldHistory])->render();

            return response()->json( array('success' => true, 'html'=>$returnHTML) );
        }
    }

    public function getProbateTooltipData(Request $request) {
        if ($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::join('outcode','probate_property.outcode_id','outcode.id')               ->where('status',1)
                               ->findOrFail($propId); 
            $returnHTML = view('admin.tooltip-content',['probateProperty'=>$probateProperty])->render(); 

            return response()->json(array('success' => true, 'html'=>$returnHTML));
        }
    }

    public function getSchoolTooltipData(Request $request) {
        $sch_latitude = $request->latitude;
        $sch_longitude = $request->longitude;
        $prop_latitude = $request->prop_latitude;
        $prop_longitude = $request->prop_longitude;
        $radius = 1.5;
        $schools = School::selectRaw("*,
        ( 3956 * acos( cos( radians(?) ) *
        cos( radians( latitude ) )
        * cos( radians( longitude ) - radians(?)
        ) + sin( radians(?) ) *
        sin( radians( latitude ) ) )
        ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
        ->where([
        ['latitude',$sch_latitude],
        ['longitude',$sch_longitude]
        ])
        ->having('distance', '<=', $radius)->first();

        $returnHTML = view('admin.school-content',['schoolAddress'=>$schools])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getEpcDetails(Request $request) {
        $full_address = explode("|",$request->address);
        $address = trim(trim($full_address[0]," "),",");
        $outcode = trim($full_address[1]," ");
        $epc_url = 'https://epc.opendatacommunities.org/api/v1/domestic/search?postcode='.$outcode.'&address='.$address;
        $epc_client = new Client();
        $epc_results = $epc_client->get($epc_url, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Authorization' => 'Basic c2hhcnU2MzFAZ21haWwuY29tOmQwZjE1Y2ZjNDlmNDlkYzQ2NWQzZjMyYjdlZDMzYjU5OTk3Y2YzYmY='
            ]
        ]);

        $epc_status = $epc_results->getStatusCode();
        $epc_contents = json_decode($epc_results->getBody(), true);
        $epc_details = $epc_contents['rows'];
        $allEpc['data'] = $epc_details[0];
        $returnHTML = view('admin.epc-content',['epcDetails'=>$allEpc ])->render(); 
        return response()->json( array('success' => true, 'html'=>$returnHTML));
    }

    public function getSchoolFullDetails(Request $request) {
        $latitude = $request->sch_latitude;
        $longitude = $request->sch_longitude;

        $schoolDetails = School::where([
            ['latitude',$latitude],
            ['longitude',$longitude]
        ])->first();

        $returnHTML = view('admin.school-full-details',['schoolDetails'=>$schoolDetails])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function schoolCoordUpdate(Request $request) {
        $schools_count = SchoolList::count();
        $count = 0;
        DB::table('schools')->orderBy('urn')->where('latitude', null)->chunk(100, function ($list) {
            foreach ($list as $school) {
                $count =+ 1;
                $urn = $school->urn;
                $school_name = $school->schname;
                $school_latitude = $school->latitude;
                $school_longitude = $school->longitude;
                if ($school_latitude  == null && $school_longitude == null) {
                    $school_address = $school->schname.', '.$school->street.', '.$school->town.', '.$school->postcode;
                    $google_autocomplete = "https://maps.googleapis.com/maps/api/place/autocomplete/json?input=".$school_address."&key=".config('map.apiKey');
                    $autocomplete_client = new Client();
                    $autocomplete_result = $autocomplete_client->get($google_autocomplete, [
                        'verify' => false,
                        'headers' => [
                            'Accept' => 'application/json',
                        ]
                    ]);
                    $autocomplete_status = $autocomplete_result->getStatusCode();
                    $autocomplete_response = json_decode($autocomplete_result->getBody(), true);
                    foreach ($autocomplete_response['predictions'] as $autocomplete_res) {
                        $google_placeId = "https://maps.googleapis.com/maps/api/place/details/json?place_id=".$autocomplete_res['place_id']."&key=".config('map.apiKey');
                        $placeId_client = new Client();
                        $placeId_result = $placeId_client->get($google_placeId, [
                            'verify' => false,
                            'headers' => [
                                'Accept' => 'application/json',
                            ]
                        ]);
                        $placeId_status = $placeId_result->getStatusCode();
                        $placeId_response = json_decode($placeId_result->getBody(), true);
                        $school_location = $placeId_response['result']['geometry']['location'];
                        $lat = $school_location['latitude'];
                        $lng = $school_location['longitude'];
                        School::where("urn",$urn)
                            ->update(
                                array(
                                    "latitude" => $lat,
                                    "longitude" => $lng,
                                )
                            );
                    }
                }
            }
        });
        if ($schools_count == $count) {
            return response()->json(array('success' => true, 'message' => 'Finished'));
        } else {
            return response()->json(array('success' => false, 'message' => 'Error'));
        }
    }

    public function getSchoolList(Request $request) {
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $sc_phase[] = $request->phase;       
        $radius = 1.5;
        $schools = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)->where('rating','!=',null);

                // if (in_array("Primary", $sc_phase))
                // {
                //    $schools = $schools->orWhere('phase_of_education', 'Primary'); 
                // }
                //  if (in_array("Secondary", $sc_phase))
                // {
                //     dd($sc_phase);
                //    $schools = $schools->orWhere('phase_of_education', 'Secondary'); 
                // }

                 $schools =  $schools->get();
         
        return response()->json( array('success' => true, 'html'=>$schools));
    }

    public function nearestSchoolList(Request $request) {
        $post_string = explode(' ', $request->postcode);
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $radius = 1.5;
        $schools_list = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)
                    ->orderBy('distance', 'ASC')
                    ->get();
        
        $returnHTML = view('admin.nearest-school-list',['schoolsList'=>$schools_list])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPhotos(Request $request) {
        $returnHTML = view('admin.photos-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }
    
    public function getFloorPlan(Request $request) {
        $returnHTML = view('admin.floorPlan-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPlanning(Request $request) {
        $returnHTML = view('admin.planning-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getSoldInStreet(Request $request) {
        $postcode = $request->postcode;
        $soldHistory = SoldHistory::select('sold_date','sold_price','tenure','house_no','addr2','addr5','addr6')->where('postcode',$postcode)->get();    
        $result = array();
         foreach ($soldHistory as $sold) {
         $result[$sold['house_no']][] = $sold;
         }
             dd($result);
        $returnHTML = view('admin.soldInStreet-content',['soldHistory'=>$soldHistory])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getMarketData(Request $request) {
        $returnHTML = view('admin.marketData-content')->render(); 
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getOfcom(Request $request) {
        $postcode = $request->postcode;
        $ofcomApi = "https://api-proxy.ofcom.org.uk/broadband/coverage/".str_replace(' ', '', $postcode);
        $ofcom_client = new Client();
        $ofcom_results = $ofcom_client->get($ofcomApi, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Ocp-Apim-Subscription-Key' => config('map.ofcomKey'),
            ]
        ]);
        $ofcom_status = $ofcom_results->getStatusCode();
        $ofcom_contents = json_decode($ofcom_results->getBody(), true);
        $ofcom_result = [];
        foreach ($ofcom_contents['Availability'] as $ofcom_results) {
            $ofcom_result['MaxBbPredictedDown'] = $ofcom_results['MaxBbPredictedDown'];
            $ofcom_result['MaxBbPredictedUp'] = $ofcom_results['MaxBbPredictedUp'];
            $ofcom_result['MaxSfbbPredictedDown'] = $ofcom_results['MaxSfbbPredictedDown'];
            $ofcom_result['MaxSfbbPredictedUp'] = $ofcom_results['MaxSfbbPredictedUp'];
            $ofcom_result['MaxUfbbPredictedDown'] = $ofcom_results['MaxUfbbPredictedDown'];
            $ofcom_result['MaxUfbbPredictedUp'] = $ofcom_results['MaxUfbbPredictedUp'];
            $ofcom_result['MaxPredictedDown'] = $ofcom_results['MaxPredictedDown'];
            $ofcom_result['MaxPredictedUp'] = $ofcom_results['MaxPredictedUp'];
            $ofcom_res['data'] = $ofcom_result;
        }
        $returnHTML = view('admin.ofcom-content',['ofcomDetails'=>$ofcom_res['data']])->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function documents(Request $request) {
        $returnHTML = view('admin.documents')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

     //All property
    public function allProperty(Request $request)
    {
        $user_id            = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title         = 'Property Listings';
        $active             = 2;
        $user_type          = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $post_code          = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')
        ->where('user_id', $user_id)
        ->join('postcode', 'outcode.id', 'postcode.outcode_id')
        ->where('user_outcode.status', 1)
        ->where('outcode.archive', 0)
        ->select('postcode.*')
        ->orderBy('postcode.postcode', 'asc')
        ->get();
        // $property           = PropertyHistory::select('property_type', 'id')->get()->unique('property_type');
        // $property           = $this->commonPropQuery()->select('property_type')->get()->unique('property_type');
        $property           = $this->commonFreshPropQuery()->select('property_history.property_type')->get()->unique('property_type');
        $prop_count         = count($property);
        $agency             = Agency::get();
        $userOutcodes       = $this->userOutcodes();
        $campaign           = 1;
        $filter             = true;
        $enable_export      = User::find($user_id)->enable_export;
        $weeks              = '';
        $search_data        = '';
        $filter_title       = 'All Property Listings(Without Filters)';
        // Filter based on weeks
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                if ($weeks == 1) {
                    $filter_title = 'Property Filtered in 0-4 Weeks';
                } elseif ($weeks == 2) {
                    $filter_title = 'Property Filtered in 4-8 Weeks';
                } elseif ($weeks == 3) {
                    $filter_title = 'Property Filtered in 8-12 Weeks';
                } elseif ($weeks == 4) {
                    $filter_title = 'Property Filtered in 12-16 Weeks';
                } elseif ($weeks == 5) {
                    $filter_title = 'Property Filtered in 16 Weeks or More';
                }
            }
            $search_data        = json_decode(Session::get('search_data'));
            $search_data->weeks = $weeks;
        }

        return view('all-property', compact('page_title', 'filter_title', 'property', 'prop_count', 'post_code', 'userOutcodes', 'user_type', 'active', 'campaign', 'agency', 'filter','search_data','enable_export'));
    }

    public function allPropertyArray(Request $request)
    {
        $sesstion_array =[];
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $user_outcode = $this->userOutcode();
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                $query = $this->commonFreshPropQuery();
                // $query = $query->whereNotIn('property_history.for_sale', [3, 6]);
                // $query = $query->whereNotIn('property_price_history.type', ['Withdrawn', 'STC']);
            }
        } else {
            $query = $this->commonPropQuery();
        }
        $query = $query->whereIn('outcode.id', $user_outcode);
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
            $sesstion_array['outcode'] = $request->input('outcode');
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
                $sesstion_array['post_code'] = $request->input('post_code');

            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
                $sesstion_array['type'] = $request->input('type');
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
                $query->where('property_flag.user_id', $user_id);
                $sesstion_array['flag'] = $request->input('flag');
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
                $sesstion_array['agency'] = $request->input('agency');
            }
        }
        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
                $sesstion_array['platform'] = $request->input('platform');
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('weeks')) {
            $sesstion_array['weeks'] = $request->input('weeks');
            $weeks = $request->weeks;
            $dt = Carbon::now();

            if (in_array($weeks, [1, 2, 3, 4])) {
                if (1 == $weeks) {
                    $start = $dt->copy()->subWeeks(4)->addDays(1)->startOfDay();
                    $end = $dt->copy()->endOfDay();
                } elseif (2 == $weeks) {
                    $start = $dt->copy()->subWeeks(8)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(4)->endOfDay();
                } elseif (3 == $weeks) {
                    $start = $dt->copy()->subWeeks(12)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(8)->endOfDay();
                } elseif (4 == $weeks) {
                    $start = $dt->copy()->subWeeks(16)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(12)->endOfDay();
                }
                $query->whereBetween('house_number.listed_date', [$start, $end]);
            } elseif (5 == $weeks) {
                $query->whereDate('house_number.listed_date', '<=', $dt->copy()->subWeeks(16)->endOfDay());
            }
        }
        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode.outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'agency.name',
            6 => 'property_type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');
            $sesstion_array['search'] = $search;
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%');

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }

            })->orderBy($order, $dir)->get()->unique('house_number_id');
            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }
            });
        }

        $dataArr = [];


        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                //   $urls = $r_count.$z_count;
                //   $urls = '<div class=" d-flex"> <a class="btn btn-primary  btn-xs  text-center"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">3</span>
                //    <a class="btn btn-success  btn-xs" href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">3</span></div>
                //   ';
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                // if (!empty($item->house_number)) {
                $address2 = $item->house_number.', '.$item->address;
                // } else {
                // $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                // if (!empty($item->created_by)) {
                //     $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $st = '';
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                // $nestedData['type'] = $request->outcodeFilter.'--'. $request->input('search.value');
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }
        
        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        $session_data =  json_encode( $sesstion_array);
        Session::put('search_data', $session_data );

        return json_encode($json_data);
    }

    //property flag change
    public function flagChange(Request $request)
    {
        // return $request->all();

        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ('' != $exist_data) { //update
                // $update = PropertyFlag::where('id',$exist_data['id'])
                //     ->update(['flag' => $request->flag]);
                $update = PropertyFlag::find($exist_data['id']);
                $update->fill(['flag' => $request->flag])->save();

                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    //property detail page
    public function propertyDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode.');
            }
        }

        $prop = true;
        $house_number_id = Helper::decryptData($request->property_id);
        $query = $this->commonPropQuery();
        $property_details = $query->where('house_number.id', $house_number_id)->first();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $page_title = $property_details->address;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
        ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('house_number_id', $house_number_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
        ->orderBy('property_price_history.event_date', 'desc')->orderBy('property_price_history.id', 'desc')->get();
        // dd($listing_activity);
        $ambiguity_checker_response = $this->propertyAmbiguityChecker(['data_type' => 'listing_activity_object', 'data' => $listing_activity]);

        $comments = Helper::propertyComment($house_number_id);

        $letter_list = PropertyLetter::join('house_number', 'property_letters.house_number_id', 'house_number.id')
        ->join('letters', 'letters.id', 'property_letters.letter_id')
        ->where('property_letters.house_number_id', $house_number_id)
        ->select('property_letters.*', 'letters.name as letter_name')
        ->get()
        ;

        $current_flag = Helper::propertyFlag($house_number_id);

        $portal_url = $this->houseNumberPortal($house_number_id);
        $campaign_history = PropertyCampaign::where('house_number_id', $house_number_id)
        ->whereHas('campaign')
        ->with('campaign')
        ->withTrashed()
        ->where('user_id', $user_id)
        ->orderBy('id', 'desc')
        ->get()
        ;
        $trigger = $this->trigger;

        return view('property-details', compact('property_details', 'listing_activity', 'ambiguity_checker_response', 'comments', 'letter_list', 'house_number_id', 'prop', 'current_flag', 'portal_url', 'house_number_id', 'campaign_history', 'trigger'));
    }

    //property history details
    public function propertyHistoryDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode');
            }
        }

        $prop = true;
        $property_history_id = Helper::decryptData($request->property_id);

        // dd($property_history_id);

        $property_details = PropertyHistory::join('postcode', 'property_history.postcode_id', 'postcode.id')
        ->where('property_history.id', $property_history_id)
        ->select('property_history.*', 'postcode.postcode as p_code')
        ->first()
        ;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->where('property_history.id', $property_history_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id')
        ->orderBy('property_price_history.event_date', 'desc')->get();

        // dd($listing_activity);

        // $comments = Helper::propertyComment($house_number_id);
        $comments = [];

        // $letter_list = PropertyLetter::join('house_number','property_letters.house_number_id','house_number.id')
        //     ->join('letters','letters.id','property_letters.letter_id')
        //     ->where('property_letters.house_number_id',$house_number_id)
        //     ->select('property_letters.*','letters.name as letter_name')
        //     ->get();

        $letter_list = [];

        $portal_url[0] = ['portal' => $property_details->portal, 'portal_url' => $property_details->portal_url];

        return view('property-history-details', compact('property_details', 'listing_activity', 'comments', 'letter_list', 'prop', 'portal_url'));
    }

    // Check if ambiguous data is provided by the system
    public function propertyAmbiguityChecker($data)
    {
        $data_type = isset($data['data_type']) ? $data['data_type'] : '';
        if ($data_type === 'listing_activity_object') {
            $listing_activity   = isset($data['data']) ? $data['data'] : '';
        } else if ($data_type === 'house_number_id') {
            $house_number_id    = isset($data['data']) ? $data['data'] : '';
            if ($house_number_id != '') {
                $listing_activity   = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
                ->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
                ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
                ->join('agency', 'agency.id', 'property_history.agency_id')
                ->where('house_number_id', $house_number_id)
                ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
                ->orderBy('property_price_history.event_date', 'desc')
                ->orderBy('property_price_history.id', 'desc')
                ->get();
            }
        }
        if (!empty($listing_activity) && count($listing_activity) > 0) {
            // Convert listing fetch to array
            $listing_activity_array = $listing_activity->toArray();
            // prepare portal to agent array
            $list_activity_portal_array         = array_column($listing_activity_array, 'portal');
            $list_activity_agent_array          = array_column($listing_activity_array, 'name');
            $list_activity_portal_agent_array   = [];
            $i = 0;
            // Prepare an event date array out of listing activity with correct position keys
            foreach ($listing_activity_array as $list_key => $list_value) {
                if (isset($listing_activity_array[$i]['event_date']) && $listing_activity_array[$i]['event_date'] != '') {
                    $list_activity_array[$i] = $listing_activity_array[$i]['event_date'];
                }
                $list_activity_portal_agent_array[] = $list_activity_portal_array[$i]." - ".$list_activity_agent_array[$i];
                $i++;
            }
            $unique_list_activity               = array_unique($list_activity_portal_agent_array);
            $unique_agent_activity_count        = count($unique_list_activity);
            // Preparing activity_listing_position_array for comparing in pattern
            $temp_listing_activity_array        = $list_activity_array;
            $count_list_activity_array          = count($list_activity_array);
            $count_activity                     = array_count_values($list_activity_array);
            $total_count_activity               = count($count_activity);
            $cycle_count                        = 0;
            $activity_listing_position_array    = [];
            foreach ($count_activity as $count_activity_key => $count_activity_value) {
                if ($count_activity_value > 1) {
                    $cycle_count += $count_activity_value;
                }
            }
            $temp_cycle_count                   = $cycle_count;
            if ($cycle_count > 0) {
                while ((count($temp_listing_activity_array) >= $cycle_count) && $cycle_count !== 0) {
                    foreach ($count_activity as $count_activity_key => $count_activity_value) {
                        if ($count_activity_value > 1) {
                            $activity_listing_position = '';
                            $activity_listing_position = array_search($count_activity_key, $temp_listing_activity_array);
                            if ($activity_listing_position !== false) {
                                $activity_listing_position_array[$activity_listing_position] = $count_activity_key;
                                unset($temp_listing_activity_array[$activity_listing_position]);
                            }
                        }
                    }
                    $cycle_count--;
                }
                // Sort in DESC order of activity date
                ksort($activity_listing_position_array);
            }
            // ambiguous listing checks
            $ambiguous_listing_flag             = false;
            $type_unfavourable_array            = ['Withdrawn', 'STC', 'Sold'];
            $first_listing_activity_array       = array_key_first($listing_activity_array);
            // $type_unfavourable_array            = ['Withdrawn'];
            if (!empty($activity_listing_position_array)) {
                $total_activity_listing_position_array = count($activity_listing_position_array);
                foreach ($activity_listing_position_array as $alpa_key => $alpa_value) {
                    $next_activity_listing = next($activity_listing_position_array);
                    // pattern checking for ambiguous data provided for a listing through system
                    // Pattern 1
                    if (isset($listing_activity_array[$alpa_key]['type'])
                        && isset($listing_activity_array[$alpa_key+1]['type'])
                        && ($listing_activity_array[$alpa_key]['type'] != $listing_activity_array[$alpa_key+1]['type']) // check if event type is different
                        && ($next_activity_listing !== false) // check if next item available
                        && (in_array($listing_activity_array[$alpa_key]['type'], $type_unfavourable_array)) // check if type is not offer or price related
                        // && ($listing_activity_array[$alpa_key]['portal'] === $listing_activity_array[$alpa_key+1]['portal']) // check if same portal provides the data
                        && ($listing_activity_array[$alpa_key]['event_date'] === $listing_activity_array[$alpa_key+1]['event_date'])) // check if event date is same
                        // && ($listing_activity_array[$alpa_key]['name'] !== $listing_activity_array[$alpa_key+1]['name'])) // check if portal agent is different
                    {
                        $ambiguous_listing_flag = true;
                    }
                }
                // Pattern 2
                if (isset($listing_activity_array[$first_listing_activity_array]['type'])
                    && (in_array($listing_activity_array[$first_listing_activity_array]['type'], $type_unfavourable_array)) // check if first listing type is unfavourable
                    && ($unique_agent_activity_count > 1)) // check if first listing type is unfavourable
                {
                    $ambiguous_listing_flag = true;
                }
            }
            // debuging unit for ambiguous_listing_flag listing data
            // $list_activity_array            = array_column($listing_activity_array, 'event_date');
            // $list_activity_array_filtered   = array_filter($list_activity_array);
            // print("<pre>".print_r($listing_activity_array,true)."</pre>");
            /*print("<pre>".print_r($list_activity_array,true)."</pre>");
            echo "total activity count : $count_list_activity_array";
            print("<pre>".print_r($count_activity,true)."</pre>");
            echo "unique activity count : $total_count_activity<br>";
            print("<pre>".print_r($temp_listing_activity_array,true)."</pre>");
            echo "cycle count : $temp_cycle_count<br>";
            echo "cycle count now: $cycle_count<br>";
            print("<pre>".print_r($activity_listing_position_array,true)."</pre>");*/
            // echo $ambiguous_listing_flag;
            // dd('stopped');
            return $ambiguous_listing_flag;
        }
    }

    //comment delete
    public function deletePropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }

        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $update = PropertyComment::where('id', $request->comment)->delete();
        if ($update) {
            return $this->simpleReturn('success', 'Successfully comment deleted..!');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    //add property comment
    public function addPropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('error', $validator->errors()->first());
        }
        
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $add = new PropertyComment();
        $add->house_number_id = $request->property;
        $add->comment = $request->comment;
        $add->user_id = $user_id;
        if ($add->save()) {
            return Redirect::back()->with('success', 'Successfully comment deleted..!');
        }

        return Redirect::back()->with('error', 'Something went wrong...!');
    }

    //search property
    public function searchProperty(Request $request)
    {
        $search_keyword = $request->search;
        $page_title = 'Addresses or Agencies That Contain: '.$search_keyword;
        session()->put('search_keyword', $search_keyword);

        return view('search-result', compact('page_title'));
    }

    //search property array
    public function searchPropertyArray(Request $request)
    {
        $search_keyword = session()->get('search_keyword');
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        // ->Where('property_history.address', 'like', '%' . $search_keyword . '%')
        // ->orWhere('agency.name', 'like', '%' . $search_keyword . '%')
        // ->orWhere('postcode.postcode', 'like', '%' . $search_keyword . '%');
        // ->orWhere('property_history.property_type', 'like', '%' . $search_keyword . '%');

        $query = $query->where(function ($q) use ($search_keyword) {
            $q->Where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('property_history.property_type', 'like', '%'.$search_keyword.'%')
            ->orWhere('outcode.outcode', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
        });

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'agency',
            4 => 'listed_date',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //  $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)
            ->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()->unique('house_number_id');
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }

                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                //  $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    //new to market
    public function newToMarket()
    {
        $page_title = 'Weekly New to Market';
        $campaign = 1;

        return view('new_to_market', compact('page_title', 'campaign'));
    }

    public function newToMarketArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        $new_date_from = date('Y-m-d', strtotime('-7 days'));
        $new_date_to = date('Y-m-d');

        $query = $query->whereBetween('house_number.listed_date', [$new_date_from, $new_date_to]);

        // dd($items->where('house_number.id',1)->get());
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'price',
            4 => 'listed_date',
            5 => 'agency.name',
            6 => 'type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                $nestedData['outcode'] = $item->outcode;

                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }
                // $listed_date=date('Y-m-d',strtotime($item->listed_date));
                // $today=date('Y-m-d');
                // $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                // $nestedData['listed_date'] = date('d M Y',strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }
                // $nestedData['week'] = $item->week;
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //withdrawn
    public function withdrawn()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Withdrawn';

        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;
        $userOutcodes = $this->userOutcodes();

        return view('withdrawn-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function withdrawnArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        if (1 == $user_type) { //admin
            $data = HouseNumber::get();
        } elseif (2 == $user_type) { //agent
            $user_outcode = $this->userOutcode();
            $outcodes = $this->userOutcodes();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.latest_status', 3);
            // dd($items->where('house_number.id',1)->get());
            if ($request->has('outcode') && '' != $request->input('outcode')) {
                $query->where('outcode.outcode', $request->input('outcode'));
            }

            if ($request->has('post_code')) {
                if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                    $query->where('postcode.id', $request->input('post_code'));
                }
            }
            // dd($request->all());
            if ($request->has('type')) {
                if (('0' != $request->type && 'undefined' != $request->type)) {
                    $query->where('property_history.property_type', $request->input('type'));
                }
            }
            if ($request->has('flag')) {
                if ((0 != $request->flag && 'undefined' != $request->flag)) {
                    $query->where('property_flag.flag', $request->input('flag'));
                }
            }
            if ($request->has('agency')) {
                if ((0 != $request->agency && 'undefined' != $request->agency)) {
                    $query->where('agency.id', $request->input('agency'));
                }
            }
            if ($request->has('platform')) {
                if ((-1 != $request->platform && 'platform' != $request->platform)) {
                    $query->where('property_history.portal', $request->input('platform'));
                }
            }
            if ($request->has('date_from') && $request->has('date_to')) {
                if (('' != $request->date_from && '' != $request->date_to)) {
                    $date_from = date('Y-m-d', strtotime($request->date_from));
                    $date_to = date('Y-m-d', strtotime($request->date_to));
                    $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
                }
            }
        } else { //data entry operator
            $data = HouseNumber::get();
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'withdrawn_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $withdrwan_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item->house_number_id)
                ->where('property_price_history.type', 'Withdrawn')
                ->orderBy('property_price_history.event_date', 'desc')
                ->select('property_price_history.event_date')
                ->first()
                ;

                // $nestedData['withdrawn_date'] = date('d M Y', strtotime( $withdrwan_date->event_date
                $nestedData['withdrawn_date'] = date('d M Y', strtotime($item->withdrawn_date));

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function recentlySold()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Sold - Under Offer';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('recently-sold-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function recentlySoldArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        //print_r($userOutcodes); exit;

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode)
        ->where('house_number.latest_status', '!=', 3)
        ;

        $query = $query->where(function ($q) {
            $q->Where('property_history.sold_stc', 1)
            ->orWhere('property_history.under_offer', 1)
            ;
        });

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            // 5 =>'property_price_history.event_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');

        if (empty($request->input('search.value'))) {
            // $data = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
            // $dd = $query->orderBy($order,$dir)->get()->unique('house_number_id')->toArray();
            // $list = array_slice($data, $start, $limit);
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            //  $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';

                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item['house_number_id']);

                $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item['house_number_id'])
                ->where('property_price_history.type', 'STC')
                ->orderBy('property_price_history.id', 'desc')
                ->limit(1)
                ->select('property_price_history.event_date', 'property_price_history.sold_year')
                ->first()
                ;

                if (!$sold_date) {
                    $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                    ->where('property_house_number.house_number_id', $item['house_number_id'])
                    ->where('property_price_history.type', 'Offer')
                    ->orderBy('property_price_history.id', 'desc')
                    ->limit(1)
                    ->select('property_price_history.event_date', 'property_price_history.sold_year')
                    ->first()
                    ;
                }

                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;

                if ($sold_date) {
                    if ($sold_date->event_date) {
                        $so_date = date('d M Y', strtotime($sold_date->event_date));
                    } else {
                        $so_date = $sold_date->sold_year;
                    }
                } else {
                    $so_date = '';
                }

                $nestedData['sold_date'] = $so_date;

                if ($item['listed_date']) {
                    $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['id'] = $item['house_number_id'];
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item['outcode'];
                if (null == $item['created_by']) {
                    $st = 'By System';
                    $color = '';
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }

                if (!empty($item['house_number'])) {
                    $address2 = $item['house_number'].', '.$item['address'];
                } else {
                    $address2 = $item['house_number'].', '.$item['address'].',<br>  '.$item['postcode'];
                }
                if (!empty($item['created_by'])) {
                    $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                // $nestedData['price'] = "&pound; ".number_format($item['latest_price']);
                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                // $nestedData['sold_date'] =  date('d M Y',strtotime($item['sold_date']));
                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item['title'].' <br> '.ucfirst($item['property_type']);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function priceDrop()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Recent Price Drops        ';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $agency = Agency::get();
        $filter = true;
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('price-drop-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'agency', 'filter', 'totalProp'));
    }

    //price drop
    public function priceDropArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        // if($user_type==1){//admin
        //     $data = HouseNumber::get();

        // }elseif($user_type==2){//agent

        $user_outcode = $this->userOutcode();

        // $query = $this->commonPropQuery();
        // $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.price_change_diff', '<', 0);

        $query = HouseNumber::join('property_house_number', 'property_house_number.house_number_id', 'house_number.id')
        ->join('property_price_history', 'property_price_history.property_history_id', 'property_house_number.property_history_id')
        ->join('property_history', 'property_history.id', 'property_house_number.property_history_id')
        ->join('postcode', 'postcode.id', 'house_number.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_flag', 'property_flag.house_number_id', 'property_house_number.house_number_id')
        ->whereNull('property_house_number.deleted_at')
        ->whereIn('outcode.id', $user_outcode)
        ->Where('property_price_history.type', 'like', '%Price reduced by%')
        ->select('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', 'house_number.latest_price', 'house_number.latest_status', 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id', 'house_number.created_by', 'property_price_history.type', 'property_price_history.event_date')
        ->orderBy('property_price_history.event_date', 'desc')
        ;
        // ->groupBy('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', "house_number.latest_price", "house_number.latest_status", 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id,'property_price_history.type'');

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // dd($items->where('house_number.id',1)->get());

        // }else{//data entry operator
        //     $data = HouseNumber::get();
        // }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'price_change_diff',
            5 => 'house_number.listed_date',
            6 => 'price_drop_date',
            7 => 'agency.name',
            8 => 'property_type',
            9 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        $outcodes = $this->userOutcodes();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //$query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                // $address3=empty($item->title)?$address2:$address1;
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['price_drop'] = '-'.substr($item->type, 16);
                // '&pound;'.number_format($item->price_change_diff);
                $listed_date = date('Y-m-d', strtotime($item->listed_date));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);

                if ($item->listed_date) {
                    $dt = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $dt = 'Hidden from public';
                }

                $nestedData['listed_date'] = $dt;
                $nestedData['price_drop_date'] = date('d M Y', strtotime($item->event_date));
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //export property data
    public function exportPropertyData(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        $file_name = '';

        if ('all_property' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);
                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'All-property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('new_to_market' == $request->mode) {
            $user_outcode = $this->userOutcode();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.new_to_market', 1);
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'New-to-market'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('recent_sold' == $request->mode) {
            $query = $query->where('house_number.latest_status', '!=', 3);

            $data_list = $query->where(function ($q) {
                $q->Where('property_history.sold_stc', 1)->orWhere('property_history.under_offer', 1);
            })->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Recentl-sold'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('hot_list' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
        } elseif ('withdrawn' == $request->mode) {
            $data_list = $query->where('house_number.latest_status', 2)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Withdrawn Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['withdrawn_date'] = date('d-m-Y', strtotime($item->withdrawn_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Withdrawn-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('price_drop' == $request->mode) {
            $data_list = $query->where('house_number.price_change_diff', '<', 0)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['price_drop'] = number_format($item->price_change_diff);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['price_drop_date'] = date('d-m-Y', strtotime($item->price_drop_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Price-drop-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('all_type_property' == $request->mode) {
            $user_outcode = $this->userOutcode();
            $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
            $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();
            $list1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
            ->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
            ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
            ->where('property_house_number.house_number_id', null)
            ->where('property_house_number.property_history_id', null)
            ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title')->get()->toArray();

            $list2 = $this->commonPropQuery();
            $list2 = $list2->whereIn('outcode.id', $user_outcode)->get()->unique('house_number_id')->toArray();
            $data_list = array_merge($list1, $list2);

            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['price'] = number_format($item['latest_price']);
                $arr['listed_date'] = date('d-m-Y', strtotime($item['listed_date']));
                if ($item['house_number_id']) {
                    $arr['address'] = $item['house_number'].', '.$item['address'].', '.$item['postcode'];
                    $arr['agency'] = Helper::agencyName($item['house_number_id']);
                } else {
                    $arr['address'] = $item['address'].', '.$item['postcode'];
                    $arr['agency'] = $item['name'];
                }
                $arr['type'] = ucfirst($item['property_type']);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'all-property-data'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } else {
            $search_keyword = session()->get('search_keyword');
            $query = $query->where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
            $data_list = $query->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        }

        $url = url('storage/property-downloads/'.$file_name);
        $data['url'] = $url;
        $data['name'] = $file_name;

        return $data;
    }

    //Hotlist
    public function hotList()
    {
        $page_title = 'Hotlist';

        return view('hotlist', compact('page_title'));
    }

    public function hotListArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'listed_date',
            4 => 'agency',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;

            // $totalFiltered =$query->Where('property_history.address', 'like', '%' . $search . '%')
            // ->orWhere('agency.name', 'like', '%' . $search . '%')
            // ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            // ->count();
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                $nestedData['agency'] = $item->agency_name;
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['comments'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    public function filterByPostcode(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $post_code = $request->postCode;
        if ($post_code) {
        }

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ($exist_data) { //update
                $update = PropertyFlag::where('id', $exist_data['id'])
                ->update(['flag' => $request->flag])
                ;
                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    public function propertyTrigger($tab)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Property';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $active = 2;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $enable_export = User::find($user_id)->enable_export;
        $userOutcodes = $this->userOutcodes();

        if (1 == $tab) {
            $filter = true;
            $active = 1;
            $agency = Agency::get();
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');

            return view('property-all-type', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'agency', 'property', 'filter','enable_export'));
        }
        if (2 == $tab) {
            $campaign = 1;
            $active = 2;
            $filter = true;
            $agency = Agency::get();

            return view('all-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'active', 'user_type', 'filter', 'agency', 'campaign','enable_export'));
        }
        if (3 == $tab) {
            $active = 3;
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');
            $filter = true;
            $post_code = PostCode::orderBy('postcode', 'asc')->get()->pluck('postcode', 'id');
            return view('common.property.property-without-address', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'property', 'filter','enable_export'));
        }

        return view('all-property', compact('page_title', 'post_code', 'userOutcodes', 'active', 'user_type','enable_export'));
    }

    public function allTypeProperty()
    {
        $page_title = 'Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $active = 1;

        return view('admin.property-all-type', compact('page_title', 'post_code', 'user_type', 'active'));
    }

    public function allTypePropertyArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
        $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();

        $query1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
        ->where('property_house_number.house_number_id', null)
        ->where('property_house_number.property_history_id', null)
        ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title');

        // ->get()->toArray();

        $query2 = $this->commonPropQuery();
        $query2 = $query2->whereIn('outcode.id', $user_outcode);

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query1->where('outcode.outcode', $request->input('outcode'));
            $query2->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query1->where('postcode.id', $request->input('post_code'));
                $query2->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('platform')) {
            if (('-1' != $request->platform && 'undefined' != $request->platform)) {
                $query1->where('property_history.portal', $request->input('platform'));
                $query2->where('property_history.portal', $request->input('platform'));
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query1->whereBetween('property_history.listed_date', [$date_from, $date_to]);
                $query2->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query1->where('property_history.property_type', $request->input('type'));
                $query2->where('property_history.property_type', $request->input('type'));
            }
        }

        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query2->where('property_flag.flag', $request->input('flag'));
            }
        }

        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query1->where('agency.id', $request->input('agency'));
                $query2->where('agency.id', $request->input('agency'));
            }
        }
        // ->get()->unique('house_number_id')->toArray();

        // $all_data = array_merge($all, $withoutHouseNumber);

        // Table's primary key
        $primaryKey = 'id';
        // $outcodes = $this->userOutcodes();

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'latest_price',
            3 => 'listed_date',
            4 => 'agency_name',
            5 => 'property_type',
            6 => 'flag',
        ];

        $q_data1 = $query1->get()->toArray();
        $q_data2 = $query2->get()->unique('house_number_id')->toArray();
        $all_data = array_merge($q_data1, $q_data2);
        $totalData = count($all_data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        if (empty($request->input('search.value'))) {
            $list1 = $query1->orderBy($order, $dir)->get()->toArray();
            $list2 = $query2->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

            $data_val = array_merge($list1, $list2); //echo "<pre>"; print_r($data_val); exit;
        } else {
            $search = $request->input('search.value');

            $list1 = $query1->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->toArray()
            ;

            $list2 = $query2->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->unique('house_number_id')
            ->toArray()
            ;

            $data_val = array_merge($list1, $list2);

            // $totalFiltered = $query->where(function($q) use ($search) {
            //     $q->Where('property_history.address', 'like', '%' . $search . '%')
            //         ->orWhere('agency.name', 'like', '%' . $search . '%')
            //     ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            //     ->orWhere('outcode.outcode', 'like', '%' . $search . '%')
            //     ->orWhere('postcode.postcode', 'like', '%' . $search . '%');

            // })->count();
            $totalFiltered = count($data_val);
        }

        $dataArr = [];
        $outcodes = $this->userOutcodes();

        if (!empty($data_val)) {
            $data_val = array_slice($data_val, $start, $limit); // return the first five elements

            $i = 0;
            foreach ($data_val as $item) {
                $nestedData['outcode'] = $item['outcode'];
                $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                if ($item['listed_date']) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                // $nestedData['price'] = "&pound;" . number_format($item['latest_price']);

                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                $nestedData['type'] = ucfirst($item['title']).'<br> '.ucfirst($item['property_type']);

                if ($item['house_number_id']) {
                    //portal url
                    $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                    $platform = '';
                    $r = 0;
                    $z = 0;
                    $r_count = '';
                    $z_count = '';
                    if (count($portal_url)) {
                        $platform = '<div class=" d-flex">';
                        foreach ($portal_url as $url) {
                            // $r_count="";$z_count="";
                            if (0 == $url->portal) {
                                ++$r;
                            } else {
                                ++$z;
                            }
                        }
                        if ($r > 0) {
                            $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                        }
                        if ($z > 0) {
                            $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                        }
                        $platform .= $r_count.$z_count;
                        $platform .= '</div>';
                    }
                    $encrpt_id = Helper::encryptData($item['house_number_id']);
                    $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                    ++$i;
                    $nestedData['id'] = $item['house_number_id'];
                    $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                    if (!empty($item['house_number'])) {
                        $address2 = $item['house_number'].' , '.$item['address'];
                    } else {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    if (!empty($item['created_by'])) {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    // $address3=empty($item['title'])?$address2:$address1;
                    if (null == $item['created_by']) {
                        $st = 'By System';
                        $color = '';
                    } else {
                        $st = 'By Manual';
                        $color="style='color:#edf124'";
                    }
                    $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                    // $nestedData['price'] = "&pound; ".number_format($item['latest_price']) ;

                    $nestedData['type'] = $item['title'].'<br> '.ucfirst($item['property_type']);
                    $nestedData['portal_url'] = $platform;
                    $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                    $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
                    $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();
                    $dataArr[] = $nestedData;
                } else {


                    $encrpt_id = Helper::encryptData($item['id']);
                    $nestedData['id'] = '';

                    $url = route('propertyHistoryDetails', ['property_id' => $encrpt_id]);

                    $nestedData['property_number'] = '<div class="input-group">
                    <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                    <div class="input-group-append">
                    <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item['id'].'"><i class="flaticon-paper-plane icon-lg"></i></button>
                    </div>
                    </div>';

                    $nestedData['address'] = "<a  href='{$url}'>{$item['address']},  {$item['postcode']}</a>";

                    $nestedData['agency'] = $item['name'];

                    if (0 == $item['portal']) {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">1</span></div>';
                    } else {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">1</span></div>';
                    }

                    $nestedData['flag'] = 0;
                    $nestedData['letter_count'] = 0;
                    ++$i;

                    $dataArr[] = $nestedData;
                }
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutPostcode(Request $request)
    {
        $page_title = 'Without Postcode Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;
        $without_data = true;

        return view('property-without-postcode', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutPostcodeArray(Request $request)
    {
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('postcode.postcode', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;
        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;
                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                // $nestedData['type'] = $totalFiltered;
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutAgent(Request $request)
    {
        $page_title = 'Without Agent Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;

        $without_data = true;

        return view('property-without-agent', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutAgentArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('agency.name', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;

        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;

                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function getProbateData()
    {
     //$user_outcode = $this->userOutcode();
        if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
            $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
            $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
        }
        $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
        $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
        $user_outcode = $selOutcodeCollection;

        $query = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
        ->join("postcode", "postcode.id", "probate_property.postcode_id")
        ->join("outcode", "outcode.id", "postcode.outcode_id")
        ->leftjoin("probate_property_flag", "probate_property_flag.probate_id", "probate_property.id")
        ->whereIn('outcode.id', $user_outcode)
        ->where('probate_property.status',1)
        ->whereDate('publication_date', '>', Carbon::now()->subDays(530));
        $data = $query
        ->select('postcode.postcode','outcode.outcode','probate_property.address_1','probate_property.address_2','probate_property.firstname','probate_property.surname','executor.company_name','executor.first_name','executor.sur_name','probate_property.publication_date','probate_property.claim_date','probate_property.id')
        ->get();


    return Datatables::of($data)

  ->editColumn('outcode', function ($model) {
   return $model->postcode;
 })
  ->editColumn('address', function ($model) {
$pid = Crypt::encrypt($model->id);
    return '<a href="' . url('property-detail?pid=' . $pid ) . '" title="View">'.$model->address_1.' '.$model->address_2.'</a>';
 })  
      ->editColumn('executer', function ($model) {
    $executor = $model->company_name ? $model->company_name : $model->first_name." ".$model->sur_name;
   return $executor;
  })
  ->editColumn('publication_date', function ($model) {
    $publicationDate =  date('d-m-Y', strtotime($model->publication_date));  
   return $publicationDate;
 })
   ->editColumn('claim_date', function ($model) {
    $claim_date =  date('d-m-Y', strtotime($model->claim_date));  
   return $claim_date;
 })
  

->editColumn('created_date', function ($model) {
    return 1;
})


  ->editColumn('action', function ($model) {
    $idG = Crypt::encrypt($model->id);
    // $user = \App\User::find($model->user_id);
    // Auth::loginUsingId($user->id, true);

            return '<a href="' . url('boat-for-sale/' . $model->slug ) . '" type="" target="_blank" class="act-sp" title="View"><i class="fas fa-file"></i></a>  <a href="' . url('edit-boat-info/' . $idG) . '" type="" target="_blank" class="act-sp" title="Login as user to edit"><i class="fas fa-edit"></i></a> ';
        })

        ->rawColumns(['s#','name','address','status','expiry','action'])
        ->make(true);
    }

    public function propertyDetail(Request $request)
    {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::where('status',1)->findOrFail($propId); 
            if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
                $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
                $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
            }
            $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
            $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
            $user_outcode = $selOutcodeCollection;
        // $probateProperties = ProbateProperty::where('status',1)->orderBy('id', 'DESC')->take(10)->get(['address_1','latitude','longitude','id'])->toArray();
            $probateProperties = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
            ->join("postcode", "postcode.id", "probate_property.postcode_id")
            ->join("outcode", "outcode.id", "postcode.outcode_id")
            ->whereIn('outcode.id', $user_outcode)
            ->where('probate_property.status',1)->orderBy('probate_property.id', 'DESC')->take(10)->get(['probate_property.address_1','probate_property.latitude','probate_property.longitude','probate_property.id'])->toArray();
            return view('admin.probate-property-detail')->with(['probateProperty' => $probateProperty,'probateProperties' => $probateProperties]);
        }
    }
    public function getProbatePopupData(Request $request) {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            //$probateProperty = ProbateProperty::where('status',1)->findOrFail($propId);
            $probateProperty = ProbateProperty::join('executor', 'probate_property.executor_id','executor.id')
                ->join("postcode", "postcode.id", "probate_property.postcode_id")
                ->join("outcode", "outcode.id", "postcode.outcode_id")
                ->leftJoin("property_sale_history", "property_sale_history.property_history_id", "probate_property.id")
                ->select('probate_property.*', 'executor.company_name as executor_comapnyname', 'executor.first_name','executor.sur_name','executor.address_1 as executor_address1','executor.address_2 as executor_address2','executor.town as executor_town','executor.postcode as executor_postcode', 'executor.phone as executor_phone','executor.email as executor_email', 'postcode.postcode as postcode_postcode','property_sale_history.price as property_sale_price','property_sale_history.sale_date as property_sale_date','property_sale_history.sold_year as property_sold_year')
                ->findOrFail($propId);
            $str = $probateProperty->address_1;
            $addr1 = explode(" ",$str);
            $postcode = $probateProperty->postcode_postcode;
            $soldHistory = SoldHistory::where('house_no', $addr1[0])->where('postcode',$postcode)->get();
            $returnHTML = view('admin.popup-content',['probateProperty'=>$probateProperty, 'soldHistory' => $soldHistory])->render();

            return response()->json( array('success' => true, 'html'=>$returnHTML) );
        }
    }

    public function getProbateTooltipData(Request $request) {
        if ($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::join('outcode','probate_property.outcode_id','outcode.id')               ->where('status',1)
                               ->findOrFail($propId); 
            $returnHTML = view('admin.tooltip-content',['probateProperty'=>$probateProperty])->render(); 

            return response()->json(array('success' => true, 'html'=>$returnHTML));
        }
    }

    public function getSchoolTooltipData(Request $request) {
        $sch_latitude = $request->latitude;
        $sch_longitude = $request->longitude;
        $prop_latitude = $request->prop_latitude;
        $prop_longitude = $request->prop_longitude;
        $radius = 1.5;
        $schools = School::selectRaw("*,
        ( 3956 * acos( cos( radians(?) ) *
        cos( radians( latitude ) )
        * cos( radians( longitude ) - radians(?)
        ) + sin( radians(?) ) *
        sin( radians( latitude ) ) )
        ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
        ->where([
        ['latitude',$sch_latitude],
        ['longitude',$sch_longitude]
        ])
        ->having('distance', '<=', $radius)->first();

        $returnHTML = view('admin.school-content',['schoolAddress'=>$schools])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getEpcDetails(Request $request) {
        $full_address = explode("|",$request->address);
        $address = trim(trim($full_address[0]," "),",");
        $outcode = trim($full_address[1]," ");
        $epc_url = 'https://epc.opendatacommunities.org/api/v1/domestic/search?postcode='.$outcode.'&address='.$address;
        $epc_client = new Client();
        $epc_results = $epc_client->get($epc_url, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Authorization' => 'Basic c2hhcnU2MzFAZ21haWwuY29tOmQwZjE1Y2ZjNDlmNDlkYzQ2NWQzZjMyYjdlZDMzYjU5OTk3Y2YzYmY='
            ]
        ]);

        $epc_status = $epc_results->getStatusCode();
        $epc_contents = json_decode($epc_results->getBody(), true);
        $epc_details = $epc_contents['rows'];
        $allEpc['data'] = $epc_details[0];
        $returnHTML = view('admin.epc-content',['epcDetails'=>$allEpc ])->render(); 
        return response()->json( array('success' => true, 'html'=>$returnHTML));
    }

    public function getSchoolFullDetails(Request $request) {
        $latitude = $request->sch_latitude;
        $longitude = $request->sch_longitude;

        $schoolDetails = School::where([
            ['latitude',$latitude],
            ['longitude',$longitude]
        ])->first();

        $returnHTML = view('admin.school-full-details',['schoolDetails'=>$schoolDetails])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function schoolCoordUpdate(Request $request) {
        $schools_count = SchoolList::count();
        $count = 0;
        DB::table('schools')->orderBy('urn')->where('latitude', null)->chunk(100, function ($list) {
            foreach ($list as $school) {
                $count =+ 1;
                $urn = $school->urn;
                $school_name = $school->schname;
                $school_latitude = $school->latitude;
                $school_longitude = $school->longitude;
                if ($school_latitude  == null && $school_longitude == null) {
                    $school_address = $school->schname.', '.$school->street.', '.$school->town.', '.$school->postcode;
                    $google_autocomplete = "https://maps.googleapis.com/maps/api/place/autocomplete/json?input=".$school_address."&key=".config('map.apiKey');
                    $autocomplete_client = new Client();
                    $autocomplete_result = $autocomplete_client->get($google_autocomplete, [
                        'verify' => false,
                        'headers' => [
                            'Accept' => 'application/json',
                        ]
                    ]);
                    $autocomplete_status = $autocomplete_result->getStatusCode();
                    $autocomplete_response = json_decode($autocomplete_result->getBody(), true);
                    foreach ($autocomplete_response['predictions'] as $autocomplete_res) {
                        $google_placeId = "https://maps.googleapis.com/maps/api/place/details/json?place_id=".$autocomplete_res['place_id']."&key=".config('map.apiKey');
                        $placeId_client = new Client();
                        $placeId_result = $placeId_client->get($google_placeId, [
                            'verify' => false,
                            'headers' => [
                                'Accept' => 'application/json',
                            ]
                        ]);
                        $placeId_status = $placeId_result->getStatusCode();
                        $placeId_response = json_decode($placeId_result->getBody(), true);
                        $school_location = $placeId_response['result']['geometry']['location'];
                        $lat = $school_location['latitude'];
                        $lng = $school_location['longitude'];
                        School::where("urn",$urn)
                            ->update(
                                array(
                                    "latitude" => $lat,
                                    "longitude" => $lng,
                                )
                            );
                    }
                }
            }
        });
        if ($schools_count == $count) {
            return response()->json(array('success' => true, 'message' => 'Finished'));
        } else {
            return response()->json(array('success' => false, 'message' => 'Error'));
        }
    }

    public function getSchoolList(Request $request) {
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $sc_phase[] = $request->phase;       
        $radius = 1.5;
        $schools = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)->where('rating','!=',null);

                // if (in_array("Primary", $sc_phase))
                // {
                //    $schools = $schools->orWhere('phase_of_education', 'Primary'); 
                // }
                //  if (in_array("Secondary", $sc_phase))
                // {
                //     dd($sc_phase);
                //    $schools = $schools->orWhere('phase_of_education', 'Secondary'); 
                // }

                 $schools =  $schools->get();
         
        return response()->json( array('success' => true, 'html'=>$schools));
    }

    public function nearestSchoolList(Request $request) {
        $post_string = explode(' ', $request->postcode);
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $radius = 1.5;
        $schools_list = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)
                    ->orderBy('distance', 'ASC')
                    ->get();
        
        $returnHTML = view('admin.nearest-school-list',['schoolsList'=>$schools_list])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPhotos(Request $request) {
        $returnHTML = view('admin.photos-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }
    
    public function getFloorPlan(Request $request) {
        $returnHTML = view('admin.floorPlan-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPlanning(Request $request) {
        $returnHTML = view('admin.planning-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getSoldInStreet(Request $request) {
        $postcode = $request->postcode;
        $soldHistory = SoldHistory::select('sold_date','sold_price','tenure','house_no','addr2','addr5','addr6')->where('postcode',$postcode)->get();    
        $result = array();
         foreach ($soldHistory as $sold) {
         $result[$sold['house_no']][] = $sold;
         }
             dd($result);
        $returnHTML = view('admin.soldInStreet-content',['soldHistory'=>$soldHistory])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getMarketData(Request $request) {
        $returnHTML = view('admin.marketData-content')->render(); 
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getOfcom(Request $request) {
        $postcode = $request->postcode;
        $ofcomApi = "https://api-proxy.ofcom.org.uk/broadband/coverage/".str_replace(' ', '', $postcode);
        $ofcom_client = new Client();
        $ofcom_results = $ofcom_client->get($ofcomApi, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Ocp-Apim-Subscription-Key' => config('map.ofcomKey'),
            ]
        ]);
        $ofcom_status = $ofcom_results->getStatusCode();
        $ofcom_contents = json_decode($ofcom_results->getBody(), true);
        $ofcom_result = [];
        foreach ($ofcom_contents['Availability'] as $ofcom_results) {
            $ofcom_result['MaxBbPredictedDown'] = $ofcom_results['MaxBbPredictedDown'];
            $ofcom_result['MaxBbPredictedUp'] = $ofcom_results['MaxBbPredictedUp'];
            $ofcom_result['MaxSfbbPredictedDown'] = $ofcom_results['MaxSfbbPredictedDown'];
            $ofcom_result['MaxSfbbPredictedUp'] = $ofcom_results['MaxSfbbPredictedUp'];
            $ofcom_result['MaxUfbbPredictedDown'] = $ofcom_results['MaxUfbbPredictedDown'];
            $ofcom_result['MaxUfbbPredictedUp'] = $ofcom_results['MaxUfbbPredictedUp'];
            $ofcom_result['MaxPredictedDown'] = $ofcom_results['MaxPredictedDown'];
            $ofcom_result['MaxPredictedUp'] = $ofcom_results['MaxPredictedUp'];
            $ofcom_res['data'] = $ofcom_result;
        }
        $returnHTML = view('admin.ofcom-content',['ofcomDetails'=>$ofcom_res['data']])->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function documents(Request $request) {
        $returnHTML = view('admin.documents')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function allProperty(Request $request)
    {
        $user_id            = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title         = 'Property Listings';
        $active             = 2;
        $user_type          = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $post_code          = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')
        ->where('user_id', $user_id)
        ->join('postcode', 'outcode.id', 'postcode.outcode_id')
        ->where('user_outcode.status', 1)
        ->where('outcode.archive', 0)
        ->select('postcode.*')
        ->orderBy('postcode.postcode', 'asc')
        ->get();
        // $property           = PropertyHistory::select('property_type', 'id')->get()->unique('property_type');
        // $property           = $this->commonPropQuery()->select('property_type')->get()->unique('property_type');
        $property           = $this->commonFreshPropQuery()->select('property_history.property_type')->get()->unique('property_type');
        $prop_count         = count($property);
        $agency             = Agency::get();
        $userOutcodes       = $this->userOutcodes();
        $campaign           = 1;
        $filter             = true;
        $enable_export      = User::find($user_id)->enable_export;
        $weeks              = '';
        $search_data        = '';
        $filter_title       = 'All Property Listings(Without Filters)';
        // Filter based on weeks
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                if ($weeks == 1) {
                    $filter_title = 'Property Filtered in 0-4 Weeks';
                } elseif ($weeks == 2) {
                    $filter_title = 'Property Filtered in 4-8 Weeks';
                } elseif ($weeks == 3) {
                    $filter_title = 'Property Filtered in 8-12 Weeks';
                } elseif ($weeks == 4) {
                    $filter_title = 'Property Filtered in 12-16 Weeks';
                } elseif ($weeks == 5) {
                    $filter_title = 'Property Filtered in 16 Weeks or More';
                }
            }
            $search_data        = json_decode(Session::get('search_data'));
            $search_data->weeks = $weeks;
        }

        return view('all-property', compact('page_title', 'filter_title', 'property', 'prop_count', 'post_code', 'userOutcodes', 'user_type', 'active', 'campaign', 'agency', 'filter','search_data','enable_export'));
    }

    public function allPropertyArray(Request $request)
    {
        $sesstion_array =[];
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $user_outcode = $this->userOutcode();
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                $query = $this->commonFreshPropQuery();
                // $query = $query->whereNotIn('property_history.for_sale', [3, 6]);
                // $query = $query->whereNotIn('property_price_history.type', ['Withdrawn', 'STC']);
            }
        } else {
            $query = $this->commonPropQuery();
        }
        $query = $query->whereIn('outcode.id', $user_outcode);
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
            $sesstion_array['outcode'] = $request->input('outcode');
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
                $sesstion_array['post_code'] = $request->input('post_code');

            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
                $sesstion_array['type'] = $request->input('type');
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
                $query->where('property_flag.user_id', $user_id);
                $sesstion_array['flag'] = $request->input('flag');
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
                $sesstion_array['agency'] = $request->input('agency');
            }
        }
        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
                $sesstion_array['platform'] = $request->input('platform');
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('weeks')) {
            $sesstion_array['weeks'] = $request->input('weeks');
            $weeks = $request->weeks;
            $dt = Carbon::now();

            if (in_array($weeks, [1, 2, 3, 4])) {
                if (1 == $weeks) {
                    $start = $dt->copy()->subWeeks(4)->addDays(1)->startOfDay();
                    $end = $dt->copy()->endOfDay();
                } elseif (2 == $weeks) {
                    $start = $dt->copy()->subWeeks(8)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(4)->endOfDay();
                } elseif (3 == $weeks) {
                    $start = $dt->copy()->subWeeks(12)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(8)->endOfDay();
                } elseif (4 == $weeks) {
                    $start = $dt->copy()->subWeeks(16)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(12)->endOfDay();
                }
                $query->whereBetween('house_number.listed_date', [$start, $end]);
            } elseif (5 == $weeks) {
                $query->whereDate('house_number.listed_date', '<=', $dt->copy()->subWeeks(16)->endOfDay());
            }
        }
        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode.outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'agency.name',
            6 => 'property_type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');
            $sesstion_array['search'] = $search;
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%');

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }

            })->orderBy($order, $dir)->get()->unique('house_number_id');
            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }
            });
        }

        $dataArr = [];


        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                //   $urls = $r_count.$z_count;
                //   $urls = '<div class=" d-flex"> <a class="btn btn-primary  btn-xs  text-center"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">3</span>
                //    <a class="btn btn-success  btn-xs" href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">3</span></div>
                //   ';
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                // if (!empty($item->house_number)) {
                $address2 = $item->house_number.', '.$item->address;
                // } else {
                // $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                // if (!empty($item->created_by)) {
                //     $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $st = '';
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                // $nestedData['type'] = $request->outcodeFilter.'--'. $request->input('search.value');
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }
        
        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        $session_data =  json_encode( $sesstion_array);
        Session::put('search_data', $session_data );

        return json_encode($json_data);
    }

    //property flag change
    public function flagChange(Request $request)
    {
        // return $request->all();

        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ('' != $exist_data) { //update
                // $update = PropertyFlag::where('id',$exist_data['id'])
                //     ->update(['flag' => $request->flag]);
                $update = PropertyFlag::find($exist_data['id']);
                $update->fill(['flag' => $request->flag])->save();

                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    //property detail page
    public function propertyDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode.');
            }
        }

        $prop = true;
        $house_number_id = Helper::decryptData($request->property_id);
        $query = $this->commonPropQuery();
        $property_details = $query->where('house_number.id', $house_number_id)->first();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $page_title = $property_details->address;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
        ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('house_number_id', $house_number_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
        ->orderBy('property_price_history.event_date', 'desc')->orderBy('property_price_history.id', 'desc')->get();
        // dd($listing_activity);
        $ambiguity_checker_response = $this->propertyAmbiguityChecker(['data_type' => 'listing_activity_object', 'data' => $listing_activity]);

        $comments = Helper::propertyComment($house_number_id);

        $letter_list = PropertyLetter::join('house_number', 'property_letters.house_number_id', 'house_number.id')
        ->join('letters', 'letters.id', 'property_letters.letter_id')
        ->where('property_letters.house_number_id', $house_number_id)
        ->select('property_letters.*', 'letters.name as letter_name')
        ->get()
        ;

        $current_flag = Helper::propertyFlag($house_number_id);

        $portal_url = $this->houseNumberPortal($house_number_id);
        $campaign_history = PropertyCampaign::where('house_number_id', $house_number_id)
        ->whereHas('campaign')
        ->with('campaign')
        ->withTrashed()
        ->where('user_id', $user_id)
        ->orderBy('id', 'desc')
        ->get()
        ;
        $trigger = $this->trigger;

        return view('property-details', compact('property_details', 'listing_activity', 'ambiguity_checker_response', 'comments', 'letter_list', 'house_number_id', 'prop', 'current_flag', 'portal_url', 'house_number_id', 'campaign_history', 'trigger'));
    }

    //property history details
    public function propertyHistoryDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode');
            }
        }

        $prop = true;
        $property_history_id = Helper::decryptData($request->property_id);

        // dd($property_history_id);

        $property_details = PropertyHistory::join('postcode', 'property_history.postcode_id', 'postcode.id')
        ->where('property_history.id', $property_history_id)
        ->select('property_history.*', 'postcode.postcode as p_code')
        ->first()
        ;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->where('property_history.id', $property_history_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id')
        ->orderBy('property_price_history.event_date', 'desc')->get();

        // dd($listing_activity);

        // $comments = Helper::propertyComment($house_number_id);
        $comments = [];

        // $letter_list = PropertyLetter::join('house_number','property_letters.house_number_id','house_number.id')
        //     ->join('letters','letters.id','property_letters.letter_id')
        //     ->where('property_letters.house_number_id',$house_number_id)
        //     ->select('property_letters.*','letters.name as letter_name')
        //     ->get();

        $letter_list = [];

        $portal_url[0] = ['portal' => $property_details->portal, 'portal_url' => $property_details->portal_url];

        return view('property-history-details', compact('property_details', 'listing_activity', 'comments', 'letter_list', 'prop', 'portal_url'));
    }

    // Check if ambiguous data is provided by the system
    public function propertyAmbiguityChecker($data)
    {
        $data_type = isset($data['data_type']) ? $data['data_type'] : '';
        if ($data_type === 'listing_activity_object') {
            $listing_activity   = isset($data['data']) ? $data['data'] : '';
        } else if ($data_type === 'house_number_id') {
            $house_number_id    = isset($data['data']) ? $data['data'] : '';
            if ($house_number_id != '') {
                $listing_activity   = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
                ->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
                ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
                ->join('agency', 'agency.id', 'property_history.agency_id')
                ->where('house_number_id', $house_number_id)
                ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
                ->orderBy('property_price_history.event_date', 'desc')
                ->orderBy('property_price_history.id', 'desc')
                ->get();
            }
        }
        if (!empty($listing_activity) && count($listing_activity) > 0) {
            // Convert listing fetch to array
            $listing_activity_array = $listing_activity->toArray();
            // prepare portal to agent array
            $list_activity_portal_array         = array_column($listing_activity_array, 'portal');
            $list_activity_agent_array          = array_column($listing_activity_array, 'name');
            $list_activity_portal_agent_array   = [];
            $i = 0;
            // Prepare an event date array out of listing activity with correct position keys
            foreach ($listing_activity_array as $list_key => $list_value) {
                if (isset($listing_activity_array[$i]['event_date']) && $listing_activity_array[$i]['event_date'] != '') {
                    $list_activity_array[$i] = $listing_activity_array[$i]['event_date'];
                }
                $list_activity_portal_agent_array[] = $list_activity_portal_array[$i]." - ".$list_activity_agent_array[$i];
                $i++;
            }
            $unique_list_activity               = array_unique($list_activity_portal_agent_array);
            $unique_agent_activity_count        = count($unique_list_activity);
            // Preparing activity_listing_position_array for comparing in pattern
            $temp_listing_activity_array        = $list_activity_array;
            $count_list_activity_array          = count($list_activity_array);
            $count_activity                     = array_count_values($list_activity_array);
            $total_count_activity               = count($count_activity);
            $cycle_count                        = 0;
            $activity_listing_position_array    = [];
            foreach ($count_activity as $count_activity_key => $count_activity_value) {
                if ($count_activity_value > 1) {
                    $cycle_count += $count_activity_value;
                }
            }
            $temp_cycle_count                   = $cycle_count;
            if ($cycle_count > 0) {
                while ((count($temp_listing_activity_array) >= $cycle_count) && $cycle_count !== 0) {
                    foreach ($count_activity as $count_activity_key => $count_activity_value) {
                        if ($count_activity_value > 1) {
                            $activity_listing_position = '';
                            $activity_listing_position = array_search($count_activity_key, $temp_listing_activity_array);
                            if ($activity_listing_position !== false) {
                                $activity_listing_position_array[$activity_listing_position] = $count_activity_key;
                                unset($temp_listing_activity_array[$activity_listing_position]);
                            }
                        }
                    }
                    $cycle_count--;
                }
                // Sort in DESC order of activity date
                ksort($activity_listing_position_array);
            }
            // ambiguous listing checks
            $ambiguous_listing_flag             = false;
            $type_unfavourable_array            = ['Withdrawn', 'STC', 'Sold'];
            $first_listing_activity_array       = array_key_first($listing_activity_array);
            // $type_unfavourable_array            = ['Withdrawn'];
            if (!empty($activity_listing_position_array)) {
                $total_activity_listing_position_array = count($activity_listing_position_array);
                foreach ($activity_listing_position_array as $alpa_key => $alpa_value) {
                    $next_activity_listing = next($activity_listing_position_array);
                    // pattern checking for ambiguous data provided for a listing through system
                    // Pattern 1
                    if (isset($listing_activity_array[$alpa_key]['type'])
                        && isset($listing_activity_array[$alpa_key+1]['type'])
                        && ($listing_activity_array[$alpa_key]['type'] != $listing_activity_array[$alpa_key+1]['type']) // check if event type is different
                        && ($next_activity_listing !== false) // check if next item available
                        && (in_array($listing_activity_array[$alpa_key]['type'], $type_unfavourable_array)) // check if type is not offer or price related
                        // && ($listing_activity_array[$alpa_key]['portal'] === $listing_activity_array[$alpa_key+1]['portal']) // check if same portal provides the data
                        && ($listing_activity_array[$alpa_key]['event_date'] === $listing_activity_array[$alpa_key+1]['event_date'])) // check if event date is same
                        // && ($listing_activity_array[$alpa_key]['name'] !== $listing_activity_array[$alpa_key+1]['name'])) // check if portal agent is different
                    {
                        $ambiguous_listing_flag = true;
                    }
                }
                // Pattern 2
                if (isset($listing_activity_array[$first_listing_activity_array]['type'])
                    && (in_array($listing_activity_array[$first_listing_activity_array]['type'], $type_unfavourable_array)) // check if first listing type is unfavourable
                    && ($unique_agent_activity_count > 1)) // check if first listing type is unfavourable
                {
                    $ambiguous_listing_flag = true;
                }
            }
            // debuging unit for ambiguous_listing_flag listing data
            // $list_activity_array            = array_column($listing_activity_array, 'event_date');
            // $list_activity_array_filtered   = array_filter($list_activity_array);
            // print("<pre>".print_r($listing_activity_array,true)."</pre>");
            /*print("<pre>".print_r($list_activity_array,true)."</pre>");
            echo "total activity count : $count_list_activity_array";
            print("<pre>".print_r($count_activity,true)."</pre>");
            echo "unique activity count : $total_count_activity<br>";
            print("<pre>".print_r($temp_listing_activity_array,true)."</pre>");
            echo "cycle count : $temp_cycle_count<br>";
            echo "cycle count now: $cycle_count<br>";
            print("<pre>".print_r($activity_listing_position_array,true)."</pre>");*/
            // echo $ambiguous_listing_flag;
            // dd('stopped');
            return $ambiguous_listing_flag;
        }
    }

    //comment delete
    public function deletePropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }

        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $update = PropertyComment::where('id', $request->comment)->delete();
        if ($update) {
            return $this->simpleReturn('success', 'Successfully comment deleted..!');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    //add property comment
    public function addPropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('error', $validator->errors()->first());
        }
        
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $add = new PropertyComment();
        $add->house_number_id = $request->property;
        $add->comment = $request->comment;
        $add->user_id = $user_id;
        if ($add->save()) {
            return Redirect::back()->with('success', 'Successfully comment deleted..!');
        }

        return Redirect::back()->with('error', 'Something went wrong...!');
    }

    //search property
    public function searchProperty(Request $request)
    {
        $search_keyword = $request->search;
        $page_title = 'Addresses or Agencies That Contain: '.$search_keyword;
        session()->put('search_keyword', $search_keyword);

        return view('search-result', compact('page_title'));
    }

    //search property array
    public function searchPropertyArray(Request $request)
    {
        $search_keyword = session()->get('search_keyword');
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        // ->Where('property_history.address', 'like', '%' . $search_keyword . '%')
        // ->orWhere('agency.name', 'like', '%' . $search_keyword . '%')
        // ->orWhere('postcode.postcode', 'like', '%' . $search_keyword . '%');
        // ->orWhere('property_history.property_type', 'like', '%' . $search_keyword . '%');

        $query = $query->where(function ($q) use ($search_keyword) {
            $q->Where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('property_history.property_type', 'like', '%'.$search_keyword.'%')
            ->orWhere('outcode.outcode', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
        });

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'agency',
            4 => 'listed_date',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //  $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)
            ->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()->unique('house_number_id');
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }

                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                //  $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    //new to market
    public function newToMarket()
    {
        $page_title = 'Weekly New to Market';
        $campaign = 1;

        return view('new_to_market', compact('page_title', 'campaign'));
    }

    public function newToMarketArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        $new_date_from = date('Y-m-d', strtotime('-7 days'));
        $new_date_to = date('Y-m-d');

        $query = $query->whereBetween('house_number.listed_date', [$new_date_from, $new_date_to]);

        // dd($items->where('house_number.id',1)->get());
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'price',
            4 => 'listed_date',
            5 => 'agency.name',
            6 => 'type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                $nestedData['outcode'] = $item->outcode;

                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }
                // $listed_date=date('Y-m-d',strtotime($item->listed_date));
                // $today=date('Y-m-d');
                // $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                // $nestedData['listed_date'] = date('d M Y',strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }
                // $nestedData['week'] = $item->week;
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //withdrawn
    public function withdrawn()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Withdrawn';

        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;
        $userOutcodes = $this->userOutcodes();

        return view('withdrawn-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function withdrawnArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        if (1 == $user_type) { //admin
            $data = HouseNumber::get();
        } elseif (2 == $user_type) { //agent
            $user_outcode = $this->userOutcode();
            $outcodes = $this->userOutcodes();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.latest_status', 3);
            // dd($items->where('house_number.id',1)->get());
            if ($request->has('outcode') && '' != $request->input('outcode')) {
                $query->where('outcode.outcode', $request->input('outcode'));
            }

            if ($request->has('post_code')) {
                if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                    $query->where('postcode.id', $request->input('post_code'));
                }
            }
            // dd($request->all());
            if ($request->has('type')) {
                if (('0' != $request->type && 'undefined' != $request->type)) {
                    $query->where('property_history.property_type', $request->input('type'));
                }
            }
            if ($request->has('flag')) {
                if ((0 != $request->flag && 'undefined' != $request->flag)) {
                    $query->where('property_flag.flag', $request->input('flag'));
                }
            }
            if ($request->has('agency')) {
                if ((0 != $request->agency && 'undefined' != $request->agency)) {
                    $query->where('agency.id', $request->input('agency'));
                }
            }
            if ($request->has('platform')) {
                if ((-1 != $request->platform && 'platform' != $request->platform)) {
                    $query->where('property_history.portal', $request->input('platform'));
                }
            }
            if ($request->has('date_from') && $request->has('date_to')) {
                if (('' != $request->date_from && '' != $request->date_to)) {
                    $date_from = date('Y-m-d', strtotime($request->date_from));
                    $date_to = date('Y-m-d', strtotime($request->date_to));
                    $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
                }
            }
        } else { //data entry operator
            $data = HouseNumber::get();
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'withdrawn_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $withdrwan_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item->house_number_id)
                ->where('property_price_history.type', 'Withdrawn')
                ->orderBy('property_price_history.event_date', 'desc')
                ->select('property_price_history.event_date')
                ->first()
                ;

                // $nestedData['withdrawn_date'] = date('d M Y', strtotime( $withdrwan_date->event_date
                $nestedData['withdrawn_date'] = date('d M Y', strtotime($item->withdrawn_date));

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function recentlySold()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Sold - Under Offer';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('recently-sold-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function recentlySoldArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        //print_r($userOutcodes); exit;

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode)
        ->where('house_number.latest_status', '!=', 3)
        ;

        $query = $query->where(function ($q) {
            $q->Where('property_history.sold_stc', 1)
            ->orWhere('property_history.under_offer', 1)
            ;
        });

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            // 5 =>'property_price_history.event_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');

        if (empty($request->input('search.value'))) {
            // $data = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
            // $dd = $query->orderBy($order,$dir)->get()->unique('house_number_id')->toArray();
            // $list = array_slice($data, $start, $limit);
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            //  $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';

                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item['house_number_id']);

                $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item['house_number_id'])
                ->where('property_price_history.type', 'STC')
                ->orderBy('property_price_history.id', 'desc')
                ->limit(1)
                ->select('property_price_history.event_date', 'property_price_history.sold_year')
                ->first()
                ;

                if (!$sold_date) {
                    $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                    ->where('property_house_number.house_number_id', $item['house_number_id'])
                    ->where('property_price_history.type', 'Offer')
                    ->orderBy('property_price_history.id', 'desc')
                    ->limit(1)
                    ->select('property_price_history.event_date', 'property_price_history.sold_year')
                    ->first()
                    ;
                }

                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;

                if ($sold_date) {
                    if ($sold_date->event_date) {
                        $so_date = date('d M Y', strtotime($sold_date->event_date));
                    } else {
                        $so_date = $sold_date->sold_year;
                    }
                } else {
                    $so_date = '';
                }

                $nestedData['sold_date'] = $so_date;

                if ($item['listed_date']) {
                    $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['id'] = $item['house_number_id'];
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item['outcode'];
                if (null == $item['created_by']) {
                    $st = 'By System';
                    $color = '';
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }

                if (!empty($item['house_number'])) {
                    $address2 = $item['house_number'].', '.$item['address'];
                } else {
                    $address2 = $item['house_number'].', '.$item['address'].',<br>  '.$item['postcode'];
                }
                if (!empty($item['created_by'])) {
                    $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                // $nestedData['price'] = "&pound; ".number_format($item['latest_price']);
                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                // $nestedData['sold_date'] =  date('d M Y',strtotime($item['sold_date']));
                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item['title'].' <br> '.ucfirst($item['property_type']);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function priceDrop()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Recent Price Drops        ';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $agency = Agency::get();
        $filter = true;
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('price-drop-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'agency', 'filter', 'totalProp'));
    }

    //price drop
    public function priceDropArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        // if($user_type==1){//admin
        //     $data = HouseNumber::get();

        // }elseif($user_type==2){//agent

        $user_outcode = $this->userOutcode();

        // $query = $this->commonPropQuery();
        // $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.price_change_diff', '<', 0);

        $query = HouseNumber::join('property_house_number', 'property_house_number.house_number_id', 'house_number.id')
        ->join('property_price_history', 'property_price_history.property_history_id', 'property_house_number.property_history_id')
        ->join('property_history', 'property_history.id', 'property_house_number.property_history_id')
        ->join('postcode', 'postcode.id', 'house_number.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_flag', 'property_flag.house_number_id', 'property_house_number.house_number_id')
        ->whereNull('property_house_number.deleted_at')
        ->whereIn('outcode.id', $user_outcode)
        ->Where('property_price_history.type', 'like', '%Price reduced by%')
        ->select('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', 'house_number.latest_price', 'house_number.latest_status', 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id', 'house_number.created_by', 'property_price_history.type', 'property_price_history.event_date')
        ->orderBy('property_price_history.event_date', 'desc')
        ;
        // ->groupBy('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', "house_number.latest_price", "house_number.latest_status", 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id,'property_price_history.type'');

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // dd($items->where('house_number.id',1)->get());

        // }else{//data entry operator
        //     $data = HouseNumber::get();
        // }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'price_change_diff',
            5 => 'house_number.listed_date',
            6 => 'price_drop_date',
            7 => 'agency.name',
            8 => 'property_type',
            9 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        $outcodes = $this->userOutcodes();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //$query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                // $address3=empty($item->title)?$address2:$address1;
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['price_drop'] = '-'.substr($item->type, 16);
                // '&pound;'.number_format($item->price_change_diff);
                $listed_date = date('Y-m-d', strtotime($item->listed_date));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);

                if ($item->listed_date) {
                    $dt = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $dt = 'Hidden from public';
                }

                $nestedData['listed_date'] = $dt;
                $nestedData['price_drop_date'] = date('d M Y', strtotime($item->event_date));
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //export property data
    public function exportPropertyData(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        $file_name = '';

        if ('all_property' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);
                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'All-property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('new_to_market' == $request->mode) {
            $user_outcode = $this->userOutcode();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.new_to_market', 1);
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'New-to-market'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('recent_sold' == $request->mode) {
            $query = $query->where('house_number.latest_status', '!=', 3);

            $data_list = $query->where(function ($q) {
                $q->Where('property_history.sold_stc', 1)->orWhere('property_history.under_offer', 1);
            })->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Recentl-sold'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('hot_list' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
        } elseif ('withdrawn' == $request->mode) {
            $data_list = $query->where('house_number.latest_status', 2)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Withdrawn Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['withdrawn_date'] = date('d-m-Y', strtotime($item->withdrawn_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Withdrawn-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('price_drop' == $request->mode) {
            $data_list = $query->where('house_number.price_change_diff', '<', 0)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['price_drop'] = number_format($item->price_change_diff);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['price_drop_date'] = date('d-m-Y', strtotime($item->price_drop_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Price-drop-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('all_type_property' == $request->mode) {
            $user_outcode = $this->userOutcode();
            $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
            $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();
            $list1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
            ->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
            ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
            ->where('property_house_number.house_number_id', null)
            ->where('property_house_number.property_history_id', null)
            ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title')->get()->toArray();

            $list2 = $this->commonPropQuery();
            $list2 = $list2->whereIn('outcode.id', $user_outcode)->get()->unique('house_number_id')->toArray();
            $data_list = array_merge($list1, $list2);

            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['price'] = number_format($item['latest_price']);
                $arr['listed_date'] = date('d-m-Y', strtotime($item['listed_date']));
                if ($item['house_number_id']) {
                    $arr['address'] = $item['house_number'].', '.$item['address'].', '.$item['postcode'];
                    $arr['agency'] = Helper::agencyName($item['house_number_id']);
                } else {
                    $arr['address'] = $item['address'].', '.$item['postcode'];
                    $arr['agency'] = $item['name'];
                }
                $arr['type'] = ucfirst($item['property_type']);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'all-property-data'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } else {
            $search_keyword = session()->get('search_keyword');
            $query = $query->where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
            $data_list = $query->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        }

        $url = url('storage/property-downloads/'.$file_name);
        $data['url'] = $url;
        $data['name'] = $file_name;

        return $data;
    }

    //Hotlist
    public function hotList()
    {
        $page_title = 'Hotlist';

        return view('hotlist', compact('page_title'));
    }

    public function hotListArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'listed_date',
            4 => 'agency',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;

            // $totalFiltered =$query->Where('property_history.address', 'like', '%' . $search . '%')
            // ->orWhere('agency.name', 'like', '%' . $search . '%')
            // ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            // ->count();
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                $nestedData['agency'] = $item->agency_name;
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['comments'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    public function filterByPostcode(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $post_code = $request->postCode;
        if ($post_code) {
        }

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ($exist_data) { //update
                $update = PropertyFlag::where('id', $exist_data['id'])
                ->update(['flag' => $request->flag])
                ;
                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    public function propertyTrigger($tab)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Property';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $active = 2;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $enable_export = User::find($user_id)->enable_export;
        $userOutcodes = $this->userOutcodes();

        if (1 == $tab) {
            $filter = true;
            $active = 1;
            $agency = Agency::get();
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');

            return view('property-all-type', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'agency', 'property', 'filter','enable_export'));
        }
        if (2 == $tab) {
            $campaign = 1;
            $active = 2;
            $filter = true;
            $agency = Agency::get();

            return view('all-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'active', 'user_type', 'filter', 'agency', 'campaign','enable_export'));
        }
        if (3 == $tab) {
            $active = 3;
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');
            $filter = true;
            $post_code = PostCode::orderBy('postcode', 'asc')->get()->pluck('postcode', 'id');
            return view('common.property.property-without-address', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'property', 'filter','enable_export'));
        }

        return view('all-property', compact('page_title', 'post_code', 'userOutcodes', 'active', 'user_type','enable_export'));
    }

    public function allTypeProperty()
    {
        $page_title = 'Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $active = 1;

        return view('admin.property-all-type', compact('page_title', 'post_code', 'user_type', 'active'));
    }

    public function allTypePropertyArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
        $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();

        $query1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
        ->where('property_house_number.house_number_id', null)
        ->where('property_house_number.property_history_id', null)
        ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title');

        // ->get()->toArray();

        $query2 = $this->commonPropQuery();
        $query2 = $query2->whereIn('outcode.id', $user_outcode);

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query1->where('outcode.outcode', $request->input('outcode'));
            $query2->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query1->where('postcode.id', $request->input('post_code'));
                $query2->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('platform')) {
            if (('-1' != $request->platform && 'undefined' != $request->platform)) {
                $query1->where('property_history.portal', $request->input('platform'));
                $query2->where('property_history.portal', $request->input('platform'));
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query1->whereBetween('property_history.listed_date', [$date_from, $date_to]);
                $query2->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query1->where('property_history.property_type', $request->input('type'));
                $query2->where('property_history.property_type', $request->input('type'));
            }
        }

        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query2->where('property_flag.flag', $request->input('flag'));
            }
        }

        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query1->where('agency.id', $request->input('agency'));
                $query2->where('agency.id', $request->input('agency'));
            }
        }
        // ->get()->unique('house_number_id')->toArray();

        // $all_data = array_merge($all, $withoutHouseNumber);

        // Table's primary key
        $primaryKey = 'id';
        // $outcodes = $this->userOutcodes();

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'latest_price',
            3 => 'listed_date',
            4 => 'agency_name',
            5 => 'property_type',
            6 => 'flag',
        ];

        $q_data1 = $query1->get()->toArray();
        $q_data2 = $query2->get()->unique('house_number_id')->toArray();
        $all_data = array_merge($q_data1, $q_data2);
        $totalData = count($all_data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        if (empty($request->input('search.value'))) {
            $list1 = $query1->orderBy($order, $dir)->get()->toArray();
            $list2 = $query2->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

            $data_val = array_merge($list1, $list2); //echo "<pre>"; print_r($data_val); exit;
        } else {
            $search = $request->input('search.value');

            $list1 = $query1->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->toArray()
            ;

            $list2 = $query2->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->unique('house_number_id')
            ->toArray()
            ;

            $data_val = array_merge($list1, $list2);

            // $totalFiltered = $query->where(function($q) use ($search) {
            //     $q->Where('property_history.address', 'like', '%' . $search . '%')
            //         ->orWhere('agency.name', 'like', '%' . $search . '%')
            //     ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            //     ->orWhere('outcode.outcode', 'like', '%' . $search . '%')
            //     ->orWhere('postcode.postcode', 'like', '%' . $search . '%');

            // })->count();
            $totalFiltered = count($data_val);
        }

        $dataArr = [];
        $outcodes = $this->userOutcodes();

        if (!empty($data_val)) {
            $data_val = array_slice($data_val, $start, $limit); // return the first five elements

            $i = 0;
            foreach ($data_val as $item) {
                $nestedData['outcode'] = $item['outcode'];
                $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                if ($item['listed_date']) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                // $nestedData['price'] = "&pound;" . number_format($item['latest_price']);

                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                $nestedData['type'] = ucfirst($item['title']).'<br> '.ucfirst($item['property_type']);

                if ($item['house_number_id']) {
                    //portal url
                    $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                    $platform = '';
                    $r = 0;
                    $z = 0;
                    $r_count = '';
                    $z_count = '';
                    if (count($portal_url)) {
                        $platform = '<div class=" d-flex">';
                        foreach ($portal_url as $url) {
                            // $r_count="";$z_count="";
                            if (0 == $url->portal) {
                                ++$r;
                            } else {
                                ++$z;
                            }
                        }
                        if ($r > 0) {
                            $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                        }
                        if ($z > 0) {
                            $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                        }
                        $platform .= $r_count.$z_count;
                        $platform .= '</div>';
                    }
                    $encrpt_id = Helper::encryptData($item['house_number_id']);
                    $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                    ++$i;
                    $nestedData['id'] = $item['house_number_id'];
                    $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                    if (!empty($item['house_number'])) {
                        $address2 = $item['house_number'].' , '.$item['address'];
                    } else {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    if (!empty($item['created_by'])) {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    // $address3=empty($item['title'])?$address2:$address1;
                    if (null == $item['created_by']) {
                        $st = 'By System';
                        $color = '';
                    } else {
                        $st = 'By Manual';
                        $color="style='color:#edf124'";
                    }
                    $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                    // $nestedData['price'] = "&pound; ".number_format($item['latest_price']) ;

                    $nestedData['type'] = $item['title'].'<br> '.ucfirst($item['property_type']);
                    $nestedData['portal_url'] = $platform;
                    $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                    $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
                    $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();
                    $dataArr[] = $nestedData;
                } else {


                    $encrpt_id = Helper::encryptData($item['id']);
                    $nestedData['id'] = '';

                    $url = route('propertyHistoryDetails', ['property_id' => $encrpt_id]);

                    $nestedData['property_number'] = '<div class="input-group">
                    <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                    <div class="input-group-append">
                    <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item['id'].'"><i class="flaticon-paper-plane icon-lg"></i></button>
                    </div>
                    </div>';

                    $nestedData['address'] = "<a  href='{$url}'>{$item['address']},  {$item['postcode']}</a>";

                    $nestedData['agency'] = $item['name'];

                    if (0 == $item['portal']) {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">1</span></div>';
                    } else {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">1</span></div>';
                    }

                    $nestedData['flag'] = 0;
                    $nestedData['letter_count'] = 0;
                    ++$i;

                    $dataArr[] = $nestedData;
                }
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutPostcode(Request $request)
    {
        $page_title = 'Without Postcode Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;
        $without_data = true;

        return view('property-without-postcode', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutPostcodeArray(Request $request)
    {
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('postcode.postcode', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;
        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;
                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                // $nestedData['type'] = $totalFiltered;
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutAgent(Request $request)
    {
        $page_title = 'Without Agent Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;

        $without_data = true;

        return view('property-without-agent', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutAgentArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('agency.name', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;

        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;

                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function getProbateData()
    {
     //$user_outcode = $this->userOutcode();
        if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
            $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
            $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
        }
        $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
        $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
        $user_outcode = $selOutcodeCollection;

        $query = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
        ->join("postcode", "postcode.id", "probate_property.postcode_id")
        ->join("outcode", "outcode.id", "postcode.outcode_id")
        ->leftjoin("probate_property_flag", "probate_property_flag.probate_id", "probate_property.id")
        ->whereIn('outcode.id', $user_outcode)
        ->where('probate_property.status',1)
        ->whereDate('publication_date', '>', Carbon::now()->subDays(530));
        $data = $query
        ->select('postcode.postcode','outcode.outcode','probate_property.address_1','probate_property.address_2','probate_property.firstname','probate_property.surname','executor.company_name','executor.first_name','executor.sur_name','probate_property.publication_date','probate_property.claim_date','probate_property.id')
        ->get();


    return Datatables::of($data)

  ->editColumn('outcode', function ($model) {
   return $model->postcode;
 })
  ->editColumn('address', function ($model) {
$pid = Crypt::encrypt($model->id);
    return '<a href="' . url('property-detail?pid=' . $pid ) . '" title="View">'.$model->address_1.' '.$model->address_2.'</a>';
 })  
      ->editColumn('executer', function ($model) {
    $executor = $model->company_name ? $model->company_name : $model->first_name." ".$model->sur_name;
   return $executor;
  })
  ->editColumn('publication_date', function ($model) {
    $publicationDate =  date('d-m-Y', strtotime($model->publication_date));  
   return $publicationDate;
 })
   ->editColumn('claim_date', function ($model) {
    $claim_date =  date('d-m-Y', strtotime($model->claim_date));  
   return $claim_date;
 })
  

->editColumn('created_date', function ($model) {
    return 1;
})


  ->editColumn('action', function ($model) {
    $idG = Crypt::encrypt($model->id);
    // $user = \App\User::find($model->user_id);
    // Auth::loginUsingId($user->id, true);

            return '<a href="' . url('boat-for-sale/' . $model->slug ) . '" type="" target="_blank" class="act-sp" title="View"><i class="fas fa-file"></i></a>  <a href="' . url('edit-boat-info/' . $idG) . '" type="" target="_blank" class="act-sp" title="Login as user to edit"><i class="fas fa-edit"></i></a> ';
        })

        ->rawColumns(['s#','name','address','status','expiry','action'])
        ->make(true);
    }

    public function propertyDetail(Request $request)
    {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::where('status',1)->findOrFail($propId); 
            if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
                $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
                $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
            }
            $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
            $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
            $user_outcode = $selOutcodeCollection;
        // $probateProperties = ProbateProperty::where('status',1)->orderBy('id', 'DESC')->take(10)->get(['address_1','latitude','longitude','id'])->toArray();
            $probateProperties = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
            ->join("postcode", "postcode.id", "probate_property.postcode_id")
            ->join("outcode", "outcode.id", "postcode.outcode_id")
            ->whereIn('outcode.id', $user_outcode)
            ->where('probate_property.status',1)->orderBy('probate_property.id', 'DESC')->take(10)->get(['probate_property.address_1','probate_property.latitude','probate_property.longitude','probate_property.id'])->toArray();
            return view('admin.probate-property-detail')->with(['probateProperty' => $probateProperty,'probateProperties' => $probateProperties]);
        }
    }
    public function getProbatePopupData(Request $request) {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            //$probateProperty = ProbateProperty::where('status',1)->findOrFail($propId);
            $probateProperty = ProbateProperty::join('executor', 'probate_property.executor_id','executor.id')
                ->join("postcode", "postcode.id", "probate_property.postcode_id")
                ->join("outcode", "outcode.id", "postcode.outcode_id")
                ->leftJoin("property_sale_history", "property_sale_history.property_history_id", "probate_property.id")
                ->select('probate_property.*', 'executor.company_name as executor_comapnyname', 'executor.first_name','executor.sur_name','executor.address_1 as executor_address1','executor.address_2 as executor_address2','executor.town as executor_town','executor.postcode as executor_postcode', 'executor.phone as executor_phone','executor.email as executor_email', 'postcode.postcode as postcode_postcode','property_sale_history.price as property_sale_price','property_sale_history.sale_date as property_sale_date','property_sale_history.sold_year as property_sold_year')
                ->findOrFail($propId);
            $str = $probateProperty->address_1;
            $addr1 = explode(" ",$str);
            $postcode = $probateProperty->postcode_postcode;
            $soldHistory = SoldHistory::where('house_no', $addr1[0])->where('postcode',$postcode)->get();
            $returnHTML = view('admin.popup-content',['probateProperty'=>$probateProperty, 'soldHistory' => $soldHistory])->render();

            return response()->json( array('success' => true, 'html'=>$returnHTML) );
        }
    }

    public function getProbateTooltipData(Request $request) {
        if ($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::join('outcode','probate_property.outcode_id','outcode.id')               ->where('status',1)
                               ->findOrFail($propId); 
            $returnHTML = view('admin.tooltip-content',['probateProperty'=>$probateProperty])->render(); 

            return response()->json(array('success' => true, 'html'=>$returnHTML));
        }
    }

    public function getSchoolTooltipData(Request $request) {
        $sch_latitude = $request->latitude;
        $sch_longitude = $request->longitude;
        $prop_latitude = $request->prop_latitude;
        $prop_longitude = $request->prop_longitude;
        $radius = 1.5;
        $schools = School::selectRaw("*,
        ( 3956 * acos( cos( radians(?) ) *
        cos( radians( latitude ) )
        * cos( radians( longitude ) - radians(?)
        ) + sin( radians(?) ) *
        sin( radians( latitude ) ) )
        ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
        ->where([
        ['latitude',$sch_latitude],
        ['longitude',$sch_longitude]
        ])
        ->having('distance', '<=', $radius)->first();

        $returnHTML = view('admin.school-content',['schoolAddress'=>$schools])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getEpcDetails(Request $request) {
        $full_address = explode("|",$request->address);
        $address = trim(trim($full_address[0]," "),",");
        $outcode = trim($full_address[1]," ");
        $epc_url = 'https://epc.opendatacommunities.org/api/v1/domestic/search?postcode='.$outcode.'&address='.$address;
        $epc_client = new Client();
        $epc_results = $epc_client->get($epc_url, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Authorization' => 'Basic c2hhcnU2MzFAZ21haWwuY29tOmQwZjE1Y2ZjNDlmNDlkYzQ2NWQzZjMyYjdlZDMzYjU5OTk3Y2YzYmY='
            ]
        ]);

        $epc_status = $epc_results->getStatusCode();
        $epc_contents = json_decode($epc_results->getBody(), true);
        $epc_details = $epc_contents['rows'];
        $allEpc['data'] = $epc_details[0];
        $returnHTML = view('admin.epc-content',['epcDetails'=>$allEpc ])->render(); 
        return response()->json( array('success' => true, 'html'=>$returnHTML));
    }

    public function getSchoolFullDetails(Request $request) {
        $latitude = $request->sch_latitude;
        $longitude = $request->sch_longitude;

        $schoolDetails = School::where([
            ['latitude',$latitude],
            ['longitude',$longitude]
        ])->first();

        $returnHTML = view('admin.school-full-details',['schoolDetails'=>$schoolDetails])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function schoolCoordUpdate(Request $request) {
        $schools_count = SchoolList::count();
        $count = 0;
        DB::table('schools')->orderBy('urn')->where('latitude', null)->chunk(100, function ($list) {
            foreach ($list as $school) {
                $count =+ 1;
                $urn = $school->urn;
                $school_name = $school->schname;
                $school_latitude = $school->latitude;
                $school_longitude = $school->longitude;
                if ($school_latitude  == null && $school_longitude == null) {
                    $school_address = $school->schname.', '.$school->street.', '.$school->town.', '.$school->postcode;
                    $google_autocomplete = "https://maps.googleapis.com/maps/api/place/autocomplete/json?input=".$school_address."&key=".config('map.apiKey');
                    $autocomplete_client = new Client();
                    $autocomplete_result = $autocomplete_client->get($google_autocomplete, [
                        'verify' => false,
                        'headers' => [
                            'Accept' => 'application/json',
                        ]
                    ]);
                    $autocomplete_status = $autocomplete_result->getStatusCode();
                    $autocomplete_response = json_decode($autocomplete_result->getBody(), true);
                    foreach ($autocomplete_response['predictions'] as $autocomplete_res) {
                        $google_placeId = "https://maps.googleapis.com/maps/api/place/details/json?place_id=".$autocomplete_res['place_id']."&key=".config('map.apiKey');
                        $placeId_client = new Client();
                        $placeId_result = $placeId_client->get($google_placeId, [
                            'verify' => false,
                            'headers' => [
                                'Accept' => 'application/json',
                            ]
                        ]);
                        $placeId_status = $placeId_result->getStatusCode();
                        $placeId_response = json_decode($placeId_result->getBody(), true);
                        $school_location = $placeId_response['result']['geometry']['location'];
                        $lat = $school_location['latitude'];
                        $lng = $school_location['longitude'];
                        School::where("urn",$urn)
                            ->update(
                                array(
                                    "latitude" => $lat,
                                    "longitude" => $lng,
                                )
                            );
                    }
                }
            }
        });
        if ($schools_count == $count) {
            return response()->json(array('success' => true, 'message' => 'Finished'));
        } else {
            return response()->json(array('success' => false, 'message' => 'Error'));
        }
    }

    public function getSchoolList(Request $request) {
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $sc_phase[] = $request->phase;       
        $radius = 1.5;
        $schools = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)->where('rating','!=',null);

                // if (in_array("Primary", $sc_phase))
                // {
                //    $schools = $schools->orWhere('phase_of_education', 'Primary'); 
                // }
                //  if (in_array("Secondary", $sc_phase))
                // {
                //     dd($sc_phase);
                //    $schools = $schools->orWhere('phase_of_education', 'Secondary'); 
                // }

                 $schools =  $schools->get();
         
        return response()->json( array('success' => true, 'html'=>$schools));
    }

    public function nearestSchoolList(Request $request) {
        $post_string = explode(' ', $request->postcode);
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $radius = 1.5;
        $schools_list = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)
                    ->orderBy('distance', 'ASC')
                    ->get();
        
        $returnHTML = view('admin.nearest-school-list',['schoolsList'=>$schools_list])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPhotos(Request $request) {
        $returnHTML = view('admin.photos-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }
    
    public function getFloorPlan(Request $request) {
        $returnHTML = view('admin.floorPlan-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPlanning(Request $request) {
        $returnHTML = view('admin.planning-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getSoldInStreet(Request $request) {
        $postcode = $request->postcode;
        $soldHistory = SoldHistory::select('sold_date','sold_price','tenure','house_no','addr2','addr5','addr6')->where('postcode',$postcode)->get();    
        $result = array();
         foreach ($soldHistory as $sold) {
         $result[$sold['house_no']][] = $sold;
         }
             dd($result);
        $returnHTML = view('admin.soldInStreet-content',['soldHistory'=>$soldHistory])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getMarketData(Request $request) {
        $returnHTML = view('admin.marketData-content')->render(); 
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getOfcom(Request $request) {
        $postcode = $request->postcode;
        $ofcomApi = "https://api-proxy.ofcom.org.uk/broadband/coverage/".str_replace(' ', '', $postcode);
        $ofcom_client = new Client();
        $ofcom_results = $ofcom_client->get($ofcomApi, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Ocp-Apim-Subscription-Key' => config('map.ofcomKey'),
            ]
        ]);
        $ofcom_status = $ofcom_results->getStatusCode();
        $ofcom_contents = json_decode($ofcom_results->getBody(), true);
        $ofcom_result = [];
        foreach ($ofcom_contents['Availability'] as $ofcom_results) {
            $ofcom_result['MaxBbPredictedDown'] = $ofcom_results['MaxBbPredictedDown'];
            $ofcom_result['MaxBbPredictedUp'] = $ofcom_results['MaxBbPredictedUp'];
            $ofcom_result['MaxSfbbPredictedDown'] = $ofcom_results['MaxSfbbPredictedDown'];
            $ofcom_result['MaxSfbbPredictedUp'] = $ofcom_results['MaxSfbbPredictedUp'];
            $ofcom_result['MaxUfbbPredictedDown'] = $ofcom_results['MaxUfbbPredictedDown'];
            $ofcom_result['MaxUfbbPredictedUp'] = $ofcom_results['MaxUfbbPredictedUp'];
            $ofcom_result['MaxPredictedDown'] = $ofcom_results['MaxPredictedDown'];
            $ofcom_result['MaxPredictedUp'] = $ofcom_results['MaxPredictedUp'];
            $ofcom_res['data'] = $ofcom_result;
        }
        $returnHTML = view('admin.ofcom-content',['ofcomDetails'=>$ofcom_res['data']])->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function documents(Request $request) {
        $returnHTML = view('admin.documents')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }


     //All property
    public function allProperty(Request $request)
    {
        $user_id            = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title         = 'Property Listings';
        $active             = 2;
        $user_type          = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $post_code          = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')
        ->where('user_id', $user_id)
        ->join('postcode', 'outcode.id', 'postcode.outcode_id')
        ->where('user_outcode.status', 1)
        ->where('outcode.archive', 0)
        ->select('postcode.*')
        ->orderBy('postcode.postcode', 'asc')
        ->get();
        // $property           = PropertyHistory::select('property_type', 'id')->get()->unique('property_type');
        // $property           = $this->commonPropQuery()->select('property_type')->get()->unique('property_type');
        $property           = $this->commonFreshPropQuery()->select('property_history.property_type')->get()->unique('property_type');
        $prop_count         = count($property);
        $agency             = Agency::get();
        $userOutcodes       = $this->userOutcodes();
        $campaign           = 1;
        $filter             = true;
        $enable_export      = User::find($user_id)->enable_export;
        $weeks              = '';
        $search_data        = '';
        $filter_title       = 'All Property Listings(Without Filters)';
        // Filter based on weeks
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                if ($weeks == 1) {
                    $filter_title = 'Property Filtered in 0-4 Weeks';
                } elseif ($weeks == 2) {
                    $filter_title = 'Property Filtered in 4-8 Weeks';
                } elseif ($weeks == 3) {
                    $filter_title = 'Property Filtered in 8-12 Weeks';
                } elseif ($weeks == 4) {
                    $filter_title = 'Property Filtered in 12-16 Weeks';
                } elseif ($weeks == 5) {
                    $filter_title = 'Property Filtered in 16 Weeks or More';
                }
            }
            $search_data        = json_decode(Session::get('search_data'));
            $search_data->weeks = $weeks;
        }

        return view('all-property', compact('page_title', 'filter_title', 'property', 'prop_count', 'post_code', 'userOutcodes', 'user_type', 'active', 'campaign', 'agency', 'filter','search_data','enable_export'));
    }

    public function allPropertyArray(Request $request)
    {
        $sesstion_array =[];
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $user_outcode = $this->userOutcode();
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                $query = $this->commonFreshPropQuery();
                // $query = $query->whereNotIn('property_history.for_sale', [3, 6]);
                // $query = $query->whereNotIn('property_price_history.type', ['Withdrawn', 'STC']);
            }
        } else {
            $query = $this->commonPropQuery();
        }
        $query = $query->whereIn('outcode.id', $user_outcode);
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
            $sesstion_array['outcode'] = $request->input('outcode');
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
                $sesstion_array['post_code'] = $request->input('post_code');

            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
                $sesstion_array['type'] = $request->input('type');
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
                $query->where('property_flag.user_id', $user_id);
                $sesstion_array['flag'] = $request->input('flag');
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
                $sesstion_array['agency'] = $request->input('agency');
            }
        }
        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
                $sesstion_array['platform'] = $request->input('platform');
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('weeks')) {
            $sesstion_array['weeks'] = $request->input('weeks');
            $weeks = $request->weeks;
            $dt = Carbon::now();

            if (in_array($weeks, [1, 2, 3, 4])) {
                if (1 == $weeks) {
                    $start = $dt->copy()->subWeeks(4)->addDays(1)->startOfDay();
                    $end = $dt->copy()->endOfDay();
                } elseif (2 == $weeks) {
                    $start = $dt->copy()->subWeeks(8)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(4)->endOfDay();
                } elseif (3 == $weeks) {
                    $start = $dt->copy()->subWeeks(12)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(8)->endOfDay();
                } elseif (4 == $weeks) {
                    $start = $dt->copy()->subWeeks(16)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(12)->endOfDay();
                }
                $query->whereBetween('house_number.listed_date', [$start, $end]);
            } elseif (5 == $weeks) {
                $query->whereDate('house_number.listed_date', '<=', $dt->copy()->subWeeks(16)->endOfDay());
            }
        }
        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode.outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'agency.name',
            6 => 'property_type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');
            $sesstion_array['search'] = $search;
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%');

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }

            })->orderBy($order, $dir)->get()->unique('house_number_id');
            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }
            });
        }

        $dataArr = [];


        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                //   $urls = $r_count.$z_count;
                //   $urls = '<div class=" d-flex"> <a class="btn btn-primary  btn-xs  text-center"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">3</span>
                //    <a class="btn btn-success  btn-xs" href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">3</span></div>
                //   ';
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                // if (!empty($item->house_number)) {
                $address2 = $item->house_number.', '.$item->address;
                // } else {
                // $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                // if (!empty($item->created_by)) {
                //     $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $st = '';
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                // $nestedData['type'] = $request->outcodeFilter.'--'. $request->input('search.value');
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }
        
        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        $session_data =  json_encode( $sesstion_array);
        Session::put('search_data', $session_data );

        return json_encode($json_data);
    }

    //property flag change
    public function flagChange(Request $request)
    {
        // return $request->all();

        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ('' != $exist_data) { //update
                // $update = PropertyFlag::where('id',$exist_data['id'])
                //     ->update(['flag' => $request->flag]);
                $update = PropertyFlag::find($exist_data['id']);
                $update->fill(['flag' => $request->flag])->save();

                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    //property detail page
    public function propertyDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode.');
            }
        }

        $prop = true;
        $house_number_id = Helper::decryptData($request->property_id);
        $query = $this->commonPropQuery();
        $property_details = $query->where('house_number.id', $house_number_id)->first();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $page_title = $property_details->address;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
        ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('house_number_id', $house_number_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
        ->orderBy('property_price_history.event_date', 'desc')->orderBy('property_price_history.id', 'desc')->get();
        // dd($listing_activity);
        $ambiguity_checker_response = $this->propertyAmbiguityChecker(['data_type' => 'listing_activity_object', 'data' => $listing_activity]);

        $comments = Helper::propertyComment($house_number_id);

        $letter_list = PropertyLetter::join('house_number', 'property_letters.house_number_id', 'house_number.id')
        ->join('letters', 'letters.id', 'property_letters.letter_id')
        ->where('property_letters.house_number_id', $house_number_id)
        ->select('property_letters.*', 'letters.name as letter_name')
        ->get()
        ;

        $current_flag = Helper::propertyFlag($house_number_id);

        $portal_url = $this->houseNumberPortal($house_number_id);
        $campaign_history = PropertyCampaign::where('house_number_id', $house_number_id)
        ->whereHas('campaign')
        ->with('campaign')
        ->withTrashed()
        ->where('user_id', $user_id)
        ->orderBy('id', 'desc')
        ->get()
        ;
        $trigger = $this->trigger;

        return view('property-details', compact('property_details', 'listing_activity', 'ambiguity_checker_response', 'comments', 'letter_list', 'house_number_id', 'prop', 'current_flag', 'portal_url', 'house_number_id', 'campaign_history', 'trigger'));
    }

    //property history details
    public function propertyHistoryDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode');
            }
        }

        $prop = true;
        $property_history_id = Helper::decryptData($request->property_id);

        // dd($property_history_id);

        $property_details = PropertyHistory::join('postcode', 'property_history.postcode_id', 'postcode.id')
        ->where('property_history.id', $property_history_id)
        ->select('property_history.*', 'postcode.postcode as p_code')
        ->first()
        ;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->where('property_history.id', $property_history_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id')
        ->orderBy('property_price_history.event_date', 'desc')->get();

        // dd($listing_activity);

        // $comments = Helper::propertyComment($house_number_id);
        $comments = [];

        // $letter_list = PropertyLetter::join('house_number','property_letters.house_number_id','house_number.id')
        //     ->join('letters','letters.id','property_letters.letter_id')
        //     ->where('property_letters.house_number_id',$house_number_id)
        //     ->select('property_letters.*','letters.name as letter_name')
        //     ->get();

        $letter_list = [];

        $portal_url[0] = ['portal' => $property_details->portal, 'portal_url' => $property_details->portal_url];

        return view('property-history-details', compact('property_details', 'listing_activity', 'comments', 'letter_list', 'prop', 'portal_url'));
    }

    // Check if ambiguous data is provided by the system
    public function propertyAmbiguityChecker($data)
    {
        $data_type = isset($data['data_type']) ? $data['data_type'] : '';
        if ($data_type === 'listing_activity_object') {
            $listing_activity   = isset($data['data']) ? $data['data'] : '';
        } else if ($data_type === 'house_number_id') {
            $house_number_id    = isset($data['data']) ? $data['data'] : '';
            if ($house_number_id != '') {
                $listing_activity   = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
                ->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
                ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
                ->join('agency', 'agency.id', 'property_history.agency_id')
                ->where('house_number_id', $house_number_id)
                ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
                ->orderBy('property_price_history.event_date', 'desc')
                ->orderBy('property_price_history.id', 'desc')
                ->get();
            }
        }
        if (!empty($listing_activity) && count($listing_activity) > 0) {
            // Convert listing fetch to array
            $listing_activity_array = $listing_activity->toArray();
            // prepare portal to agent array
            $list_activity_portal_array         = array_column($listing_activity_array, 'portal');
            $list_activity_agent_array          = array_column($listing_activity_array, 'name');
            $list_activity_portal_agent_array   = [];
            $i = 0;
            // Prepare an event date array out of listing activity with correct position keys
            foreach ($listing_activity_array as $list_key => $list_value) {
                if (isset($listing_activity_array[$i]['event_date']) && $listing_activity_array[$i]['event_date'] != '') {
                    $list_activity_array[$i] = $listing_activity_array[$i]['event_date'];
                }
                $list_activity_portal_agent_array[] = $list_activity_portal_array[$i]." - ".$list_activity_agent_array[$i];
                $i++;
            }
            $unique_list_activity               = array_unique($list_activity_portal_agent_array);
            $unique_agent_activity_count        = count($unique_list_activity);
            // Preparing activity_listing_position_array for comparing in pattern
            $temp_listing_activity_array        = $list_activity_array;
            $count_list_activity_array          = count($list_activity_array);
            $count_activity                     = array_count_values($list_activity_array);
            $total_count_activity               = count($count_activity);
            $cycle_count                        = 0;
            $activity_listing_position_array    = [];
            foreach ($count_activity as $count_activity_key => $count_activity_value) {
                if ($count_activity_value > 1) {
                    $cycle_count += $count_activity_value;
                }
            }
            $temp_cycle_count                   = $cycle_count;
            if ($cycle_count > 0) {
                while ((count($temp_listing_activity_array) >= $cycle_count) && $cycle_count !== 0) {
                    foreach ($count_activity as $count_activity_key => $count_activity_value) {
                        if ($count_activity_value > 1) {
                            $activity_listing_position = '';
                            $activity_listing_position = array_search($count_activity_key, $temp_listing_activity_array);
                            if ($activity_listing_position !== false) {
                                $activity_listing_position_array[$activity_listing_position] = $count_activity_key;
                                unset($temp_listing_activity_array[$activity_listing_position]);
                            }
                        }
                    }
                    $cycle_count--;
                }
                // Sort in DESC order of activity date
                ksort($activity_listing_position_array);
            }
            // ambiguous listing checks
            $ambiguous_listing_flag             = false;
            $type_unfavourable_array            = ['Withdrawn', 'STC', 'Sold'];
            $first_listing_activity_array       = array_key_first($listing_activity_array);
            // $type_unfavourable_array            = ['Withdrawn'];
            if (!empty($activity_listing_position_array)) {
                $total_activity_listing_position_array = count($activity_listing_position_array);
                foreach ($activity_listing_position_array as $alpa_key => $alpa_value) {
                    $next_activity_listing = next($activity_listing_position_array);
                    // pattern checking for ambiguous data provided for a listing through system
                    // Pattern 1
                    if (isset($listing_activity_array[$alpa_key]['type'])
                        && isset($listing_activity_array[$alpa_key+1]['type'])
                        && ($listing_activity_array[$alpa_key]['type'] != $listing_activity_array[$alpa_key+1]['type']) // check if event type is different
                        && ($next_activity_listing !== false) // check if next item available
                        && (in_array($listing_activity_array[$alpa_key]['type'], $type_unfavourable_array)) // check if type is not offer or price related
                        // && ($listing_activity_array[$alpa_key]['portal'] === $listing_activity_array[$alpa_key+1]['portal']) // check if same portal provides the data
                        && ($listing_activity_array[$alpa_key]['event_date'] === $listing_activity_array[$alpa_key+1]['event_date'])) // check if event date is same
                        // && ($listing_activity_array[$alpa_key]['name'] !== $listing_activity_array[$alpa_key+1]['name'])) // check if portal agent is different
                    {
                        $ambiguous_listing_flag = true;
                    }
                }
                // Pattern 2
                if (isset($listing_activity_array[$first_listing_activity_array]['type'])
                    && (in_array($listing_activity_array[$first_listing_activity_array]['type'], $type_unfavourable_array)) // check if first listing type is unfavourable
                    && ($unique_agent_activity_count > 1)) // check if first listing type is unfavourable
                {
                    $ambiguous_listing_flag = true;
                }
            }
            // debuging unit for ambiguous_listing_flag listing data
            // $list_activity_array            = array_column($listing_activity_array, 'event_date');
            // $list_activity_array_filtered   = array_filter($list_activity_array);
            // print("<pre>".print_r($listing_activity_array,true)."</pre>");
            /*print("<pre>".print_r($list_activity_array,true)."</pre>");
            echo "total activity count : $count_list_activity_array";
            print("<pre>".print_r($count_activity,true)."</pre>");
            echo "unique activity count : $total_count_activity<br>";
            print("<pre>".print_r($temp_listing_activity_array,true)."</pre>");
            echo "cycle count : $temp_cycle_count<br>";
            echo "cycle count now: $cycle_count<br>";
            print("<pre>".print_r($activity_listing_position_array,true)."</pre>");*/
            // echo $ambiguous_listing_flag;
            // dd('stopped');
            return $ambiguous_listing_flag;
        }
    }

    //comment delete
    public function deletePropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }

        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $update = PropertyComment::where('id', $request->comment)->delete();
        if ($update) {
            return $this->simpleReturn('success', 'Successfully comment deleted..!');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    //add property comment
    public function addPropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('error', $validator->errors()->first());
        }
        
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $add = new PropertyComment();
        $add->house_number_id = $request->property;
        $add->comment = $request->comment;
        $add->user_id = $user_id;
        if ($add->save()) {
            return Redirect::back()->with('success', 'Successfully comment deleted..!');
        }

        return Redirect::back()->with('error', 'Something went wrong...!');
    }

    //search property
    public function searchProperty(Request $request)
    {
        $search_keyword = $request->search;
        $page_title = 'Addresses or Agencies That Contain: '.$search_keyword;
        session()->put('search_keyword', $search_keyword);

        return view('search-result', compact('page_title'));
    }

    //search property array
    public function searchPropertyArray(Request $request)
    {
        $search_keyword = session()->get('search_keyword');
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        // ->Where('property_history.address', 'like', '%' . $search_keyword . '%')
        // ->orWhere('agency.name', 'like', '%' . $search_keyword . '%')
        // ->orWhere('postcode.postcode', 'like', '%' . $search_keyword . '%');
        // ->orWhere('property_history.property_type', 'like', '%' . $search_keyword . '%');

        $query = $query->where(function ($q) use ($search_keyword) {
            $q->Where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('property_history.property_type', 'like', '%'.$search_keyword.'%')
            ->orWhere('outcode.outcode', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
        });

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'agency',
            4 => 'listed_date',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //  $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)
            ->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()->unique('house_number_id');
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }

                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                //  $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    //new to market
    public function newToMarket()
    {
        $page_title = 'Weekly New to Market';
        $campaign = 1;

        return view('new_to_market', compact('page_title', 'campaign'));
    }

    public function newToMarketArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        $new_date_from = date('Y-m-d', strtotime('-7 days'));
        $new_date_to = date('Y-m-d');

        $query = $query->whereBetween('house_number.listed_date', [$new_date_from, $new_date_to]);

        // dd($items->where('house_number.id',1)->get());
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'price',
            4 => 'listed_date',
            5 => 'agency.name',
            6 => 'type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                $nestedData['outcode'] = $item->outcode;

                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }
                // $listed_date=date('Y-m-d',strtotime($item->listed_date));
                // $today=date('Y-m-d');
                // $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                // $nestedData['listed_date'] = date('d M Y',strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }
                // $nestedData['week'] = $item->week;
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //withdrawn
    public function withdrawn()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Withdrawn';

        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;
        $userOutcodes = $this->userOutcodes();

        return view('withdrawn-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function withdrawnArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        if (1 == $user_type) { //admin
            $data = HouseNumber::get();
        } elseif (2 == $user_type) { //agent
            $user_outcode = $this->userOutcode();
            $outcodes = $this->userOutcodes();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.latest_status', 3);
            // dd($items->where('house_number.id',1)->get());
            if ($request->has('outcode') && '' != $request->input('outcode')) {
                $query->where('outcode.outcode', $request->input('outcode'));
            }

            if ($request->has('post_code')) {
                if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                    $query->where('postcode.id', $request->input('post_code'));
                }
            }
            // dd($request->all());
            if ($request->has('type')) {
                if (('0' != $request->type && 'undefined' != $request->type)) {
                    $query->where('property_history.property_type', $request->input('type'));
                }
            }
            if ($request->has('flag')) {
                if ((0 != $request->flag && 'undefined' != $request->flag)) {
                    $query->where('property_flag.flag', $request->input('flag'));
                }
            }
            if ($request->has('agency')) {
                if ((0 != $request->agency && 'undefined' != $request->agency)) {
                    $query->where('agency.id', $request->input('agency'));
                }
            }
            if ($request->has('platform')) {
                if ((-1 != $request->platform && 'platform' != $request->platform)) {
                    $query->where('property_history.portal', $request->input('platform'));
                }
            }
            if ($request->has('date_from') && $request->has('date_to')) {
                if (('' != $request->date_from && '' != $request->date_to)) {
                    $date_from = date('Y-m-d', strtotime($request->date_from));
                    $date_to = date('Y-m-d', strtotime($request->date_to));
                    $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
                }
            }
        } else { //data entry operator
            $data = HouseNumber::get();
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'withdrawn_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $withdrwan_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item->house_number_id)
                ->where('property_price_history.type', 'Withdrawn')
                ->orderBy('property_price_history.event_date', 'desc')
                ->select('property_price_history.event_date')
                ->first()
                ;

                // $nestedData['withdrawn_date'] = date('d M Y', strtotime( $withdrwan_date->event_date
                $nestedData['withdrawn_date'] = date('d M Y', strtotime($item->withdrawn_date));

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function recentlySold()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Sold - Under Offer';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('recently-sold-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function recentlySoldArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        //print_r($userOutcodes); exit;

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode)
        ->where('house_number.latest_status', '!=', 3)
        ;

        $query = $query->where(function ($q) {
            $q->Where('property_history.sold_stc', 1)
            ->orWhere('property_history.under_offer', 1)
            ;
        });

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            // 5 =>'property_price_history.event_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');

        if (empty($request->input('search.value'))) {
            // $data = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
            // $dd = $query->orderBy($order,$dir)->get()->unique('house_number_id')->toArray();
            // $list = array_slice($data, $start, $limit);
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            //  $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';

                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item['house_number_id']);

                $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item['house_number_id'])
                ->where('property_price_history.type', 'STC')
                ->orderBy('property_price_history.id', 'desc')
                ->limit(1)
                ->select('property_price_history.event_date', 'property_price_history.sold_year')
                ->first()
                ;

                if (!$sold_date) {
                    $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                    ->where('property_house_number.house_number_id', $item['house_number_id'])
                    ->where('property_price_history.type', 'Offer')
                    ->orderBy('property_price_history.id', 'desc')
                    ->limit(1)
                    ->select('property_price_history.event_date', 'property_price_history.sold_year')
                    ->first()
                    ;
                }

                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;

                if ($sold_date) {
                    if ($sold_date->event_date) {
                        $so_date = date('d M Y', strtotime($sold_date->event_date));
                    } else {
                        $so_date = $sold_date->sold_year;
                    }
                } else {
                    $so_date = '';
                }

                $nestedData['sold_date'] = $so_date;

                if ($item['listed_date']) {
                    $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['id'] = $item['house_number_id'];
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item['outcode'];
                if (null == $item['created_by']) {
                    $st = 'By System';
                    $color = '';
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }

                if (!empty($item['house_number'])) {
                    $address2 = $item['house_number'].', '.$item['address'];
                } else {
                    $address2 = $item['house_number'].', '.$item['address'].',<br>  '.$item['postcode'];
                }
                if (!empty($item['created_by'])) {
                    $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                // $nestedData['price'] = "&pound; ".number_format($item['latest_price']);
                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                // $nestedData['sold_date'] =  date('d M Y',strtotime($item['sold_date']));
                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item['title'].' <br> '.ucfirst($item['property_type']);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function priceDrop()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Recent Price Drops        ';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $agency = Agency::get();
        $filter = true;
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('price-drop-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'agency', 'filter', 'totalProp'));
    }

    //price drop
    public function priceDropArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        // if($user_type==1){//admin
        //     $data = HouseNumber::get();

        // }elseif($user_type==2){//agent

        $user_outcode = $this->userOutcode();

        // $query = $this->commonPropQuery();
        // $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.price_change_diff', '<', 0);

        $query = HouseNumber::join('property_house_number', 'property_house_number.house_number_id', 'house_number.id')
        ->join('property_price_history', 'property_price_history.property_history_id', 'property_house_number.property_history_id')
        ->join('property_history', 'property_history.id', 'property_house_number.property_history_id')
        ->join('postcode', 'postcode.id', 'house_number.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_flag', 'property_flag.house_number_id', 'property_house_number.house_number_id')
        ->whereNull('property_house_number.deleted_at')
        ->whereIn('outcode.id', $user_outcode)
        ->Where('property_price_history.type', 'like', '%Price reduced by%')
        ->select('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', 'house_number.latest_price', 'house_number.latest_status', 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id', 'house_number.created_by', 'property_price_history.type', 'property_price_history.event_date')
        ->orderBy('property_price_history.event_date', 'desc')
        ;
        // ->groupBy('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', "house_number.latest_price", "house_number.latest_status", 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id,'property_price_history.type'');

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // dd($items->where('house_number.id',1)->get());

        // }else{//data entry operator
        //     $data = HouseNumber::get();
        // }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'price_change_diff',
            5 => 'house_number.listed_date',
            6 => 'price_drop_date',
            7 => 'agency.name',
            8 => 'property_type',
            9 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        $outcodes = $this->userOutcodes();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //$query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                // $address3=empty($item->title)?$address2:$address1;
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['price_drop'] = '-'.substr($item->type, 16);
                // '&pound;'.number_format($item->price_change_diff);
                $listed_date = date('Y-m-d', strtotime($item->listed_date));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);

                if ($item->listed_date) {
                    $dt = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $dt = 'Hidden from public';
                }

                $nestedData['listed_date'] = $dt;
                $nestedData['price_drop_date'] = date('d M Y', strtotime($item->event_date));
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //export property data
    public function exportPropertyData(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        $file_name = '';

        if ('all_property' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);
                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'All-property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('new_to_market' == $request->mode) {
            $user_outcode = $this->userOutcode();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.new_to_market', 1);
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'New-to-market'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('recent_sold' == $request->mode) {
            $query = $query->where('house_number.latest_status', '!=', 3);

            $data_list = $query->where(function ($q) {
                $q->Where('property_history.sold_stc', 1)->orWhere('property_history.under_offer', 1);
            })->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Recentl-sold'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('hot_list' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
        } elseif ('withdrawn' == $request->mode) {
            $data_list = $query->where('house_number.latest_status', 2)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Withdrawn Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['withdrawn_date'] = date('d-m-Y', strtotime($item->withdrawn_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Withdrawn-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('price_drop' == $request->mode) {
            $data_list = $query->where('house_number.price_change_diff', '<', 0)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['price_drop'] = number_format($item->price_change_diff);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['price_drop_date'] = date('d-m-Y', strtotime($item->price_drop_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Price-drop-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('all_type_property' == $request->mode) {
            $user_outcode = $this->userOutcode();
            $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
            $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();
            $list1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
            ->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
            ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
            ->where('property_house_number.house_number_id', null)
            ->where('property_house_number.property_history_id', null)
            ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title')->get()->toArray();

            $list2 = $this->commonPropQuery();
            $list2 = $list2->whereIn('outcode.id', $user_outcode)->get()->unique('house_number_id')->toArray();
            $data_list = array_merge($list1, $list2);

            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['price'] = number_format($item['latest_price']);
                $arr['listed_date'] = date('d-m-Y', strtotime($item['listed_date']));
                if ($item['house_number_id']) {
                    $arr['address'] = $item['house_number'].', '.$item['address'].', '.$item['postcode'];
                    $arr['agency'] = Helper::agencyName($item['house_number_id']);
                } else {
                    $arr['address'] = $item['address'].', '.$item['postcode'];
                    $arr['agency'] = $item['name'];
                }
                $arr['type'] = ucfirst($item['property_type']);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'all-property-data'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } else {
            $search_keyword = session()->get('search_keyword');
            $query = $query->where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
            $data_list = $query->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        }

        $url = url('storage/property-downloads/'.$file_name);
        $data['url'] = $url;
        $data['name'] = $file_name;

        return $data;
    }

    //Hotlist
    public function hotList()
    {
        $page_title = 'Hotlist';

        return view('hotlist', compact('page_title'));
    }

    public function hotListArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'listed_date',
            4 => 'agency',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;

            // $totalFiltered =$query->Where('property_history.address', 'like', '%' . $search . '%')
            // ->orWhere('agency.name', 'like', '%' . $search . '%')
            // ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            // ->count();
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                $nestedData['agency'] = $item->agency_name;
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['comments'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    public function filterByPostcode(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $post_code = $request->postCode;
        if ($post_code) {
        }

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ($exist_data) { //update
                $update = PropertyFlag::where('id', $exist_data['id'])
                ->update(['flag' => $request->flag])
                ;
                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    public function propertyTrigger($tab)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Property';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $active = 2;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $enable_export = User::find($user_id)->enable_export;
        $userOutcodes = $this->userOutcodes();

        if (1 == $tab) {
            $filter = true;
            $active = 1;
            $agency = Agency::get();
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');

            return view('property-all-type', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'agency', 'property', 'filter','enable_export'));
        }
        if (2 == $tab) {
            $campaign = 1;
            $active = 2;
            $filter = true;
            $agency = Agency::get();

            return view('all-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'active', 'user_type', 'filter', 'agency', 'campaign','enable_export'));
        }
        if (3 == $tab) {
            $active = 3;
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');
            $filter = true;
            $post_code = PostCode::orderBy('postcode', 'asc')->get()->pluck('postcode', 'id');
            return view('common.property.property-without-address', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'property', 'filter','enable_export'));
        }

        return view('all-property', compact('page_title', 'post_code', 'userOutcodes', 'active', 'user_type','enable_export'));
    }

    public function allTypeProperty()
    {
        $page_title = 'Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $active = 1;

        return view('admin.property-all-type', compact('page_title', 'post_code', 'user_type', 'active'));
    }

    public function allTypePropertyArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
        $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();

        $query1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
        ->where('property_house_number.house_number_id', null)
        ->where('property_house_number.property_history_id', null)
        ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title');

        // ->get()->toArray();

        $query2 = $this->commonPropQuery();
        $query2 = $query2->whereIn('outcode.id', $user_outcode);

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query1->where('outcode.outcode', $request->input('outcode'));
            $query2->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query1->where('postcode.id', $request->input('post_code'));
                $query2->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('platform')) {
            if (('-1' != $request->platform && 'undefined' != $request->platform)) {
                $query1->where('property_history.portal', $request->input('platform'));
                $query2->where('property_history.portal', $request->input('platform'));
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query1->whereBetween('property_history.listed_date', [$date_from, $date_to]);
                $query2->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query1->where('property_history.property_type', $request->input('type'));
                $query2->where('property_history.property_type', $request->input('type'));
            }
        }

        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query2->where('property_flag.flag', $request->input('flag'));
            }
        }

        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query1->where('agency.id', $request->input('agency'));
                $query2->where('agency.id', $request->input('agency'));
            }
        }
        // ->get()->unique('house_number_id')->toArray();

        // $all_data = array_merge($all, $withoutHouseNumber);

        // Table's primary key
        $primaryKey = 'id';
        // $outcodes = $this->userOutcodes();

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'latest_price',
            3 => 'listed_date',
            4 => 'agency_name',
            5 => 'property_type',
            6 => 'flag',
        ];

        $q_data1 = $query1->get()->toArray();
        $q_data2 = $query2->get()->unique('house_number_id')->toArray();
        $all_data = array_merge($q_data1, $q_data2);
        $totalData = count($all_data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        if (empty($request->input('search.value'))) {
            $list1 = $query1->orderBy($order, $dir)->get()->toArray();
            $list2 = $query2->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

            $data_val = array_merge($list1, $list2); //echo "<pre>"; print_r($data_val); exit;
        } else {
            $search = $request->input('search.value');

            $list1 = $query1->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->toArray()
            ;

            $list2 = $query2->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->unique('house_number_id')
            ->toArray()
            ;

            $data_val = array_merge($list1, $list2);

            // $totalFiltered = $query->where(function($q) use ($search) {
            //     $q->Where('property_history.address', 'like', '%' . $search . '%')
            //         ->orWhere('agency.name', 'like', '%' . $search . '%')
            //     ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            //     ->orWhere('outcode.outcode', 'like', '%' . $search . '%')
            //     ->orWhere('postcode.postcode', 'like', '%' . $search . '%');

            // })->count();
            $totalFiltered = count($data_val);
        }

        $dataArr = [];
        $outcodes = $this->userOutcodes();

        if (!empty($data_val)) {
            $data_val = array_slice($data_val, $start, $limit); // return the first five elements

            $i = 0;
            foreach ($data_val as $item) {
                $nestedData['outcode'] = $item['outcode'];
                $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                if ($item['listed_date']) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                // $nestedData['price'] = "&pound;" . number_format($item['latest_price']);

                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                $nestedData['type'] = ucfirst($item['title']).'<br> '.ucfirst($item['property_type']);

                if ($item['house_number_id']) {
                    //portal url
                    $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                    $platform = '';
                    $r = 0;
                    $z = 0;
                    $r_count = '';
                    $z_count = '';
                    if (count($portal_url)) {
                        $platform = '<div class=" d-flex">';
                        foreach ($portal_url as $url) {
                            // $r_count="";$z_count="";
                            if (0 == $url->portal) {
                                ++$r;
                            } else {
                                ++$z;
                            }
                        }
                        if ($r > 0) {
                            $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                        }
                        if ($z > 0) {
                            $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                        }
                        $platform .= $r_count.$z_count;
                        $platform .= '</div>';
                    }
                    $encrpt_id = Helper::encryptData($item['house_number_id']);
                    $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                    ++$i;
                    $nestedData['id'] = $item['house_number_id'];
                    $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                    if (!empty($item['house_number'])) {
                        $address2 = $item['house_number'].' , '.$item['address'];
                    } else {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    if (!empty($item['created_by'])) {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    // $address3=empty($item['title'])?$address2:$address1;
                    if (null == $item['created_by']) {
                        $st = 'By System';
                        $color = '';
                    } else {
                        $st = 'By Manual';
                        $color="style='color:#edf124'";
                    }
                    $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                    // $nestedData['price'] = "&pound; ".number_format($item['latest_price']) ;

                    $nestedData['type'] = $item['title'].'<br> '.ucfirst($item['property_type']);
                    $nestedData['portal_url'] = $platform;
                    $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                    $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
                    $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();
                    $dataArr[] = $nestedData;
                } else {


                    $encrpt_id = Helper::encryptData($item['id']);
                    $nestedData['id'] = '';

                    $url = route('propertyHistoryDetails', ['property_id' => $encrpt_id]);

                    $nestedData['property_number'] = '<div class="input-group">
                    <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                    <div class="input-group-append">
                    <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item['id'].'"><i class="flaticon-paper-plane icon-lg"></i></button>
                    </div>
                    </div>';

                    $nestedData['address'] = "<a  href='{$url}'>{$item['address']},  {$item['postcode']}</a>";

                    $nestedData['agency'] = $item['name'];

                    if (0 == $item['portal']) {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">1</span></div>';
                    } else {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">1</span></div>';
                    }

                    $nestedData['flag'] = 0;
                    $nestedData['letter_count'] = 0;
                    ++$i;

                    $dataArr[] = $nestedData;
                }
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutPostcode(Request $request)
    {
        $page_title = 'Without Postcode Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;
        $without_data = true;

        return view('property-without-postcode', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutPostcodeArray(Request $request)
    {
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('postcode.postcode', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;
        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;
                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                // $nestedData['type'] = $totalFiltered;
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutAgent(Request $request)
    {
        $page_title = 'Without Agent Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;

        $without_data = true;

        return view('property-without-agent', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutAgentArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('agency.name', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;

        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;

                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function getProbateData()
    {
     //$user_outcode = $this->userOutcode();
        if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
            $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
            $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
        }
        $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
        $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
        $user_outcode = $selOutcodeCollection;

        $query = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
        ->join("postcode", "postcode.id", "probate_property.postcode_id")
        ->join("outcode", "outcode.id", "postcode.outcode_id")
        ->leftjoin("probate_property_flag", "probate_property_flag.probate_id", "probate_property.id")
        ->whereIn('outcode.id', $user_outcode)
        ->where('probate_property.status',1)
        ->whereDate('publication_date', '>', Carbon::now()->subDays(530));
        $data = $query
        ->select('postcode.postcode','outcode.outcode','probate_property.address_1','probate_property.address_2','probate_property.firstname','probate_property.surname','executor.company_name','executor.first_name','executor.sur_name','probate_property.publication_date','probate_property.claim_date','probate_property.id')
        ->get();


    return Datatables::of($data)

  ->editColumn('outcode', function ($model) {
   return $model->postcode;
 })
  ->editColumn('address', function ($model) {
$pid = Crypt::encrypt($model->id);
    return '<a href="' . url('property-detail?pid=' . $pid ) . '" title="View">'.$model->address_1.' '.$model->address_2.'</a>';
 })  
      ->editColumn('executer', function ($model) {
    $executor = $model->company_name ? $model->company_name : $model->first_name." ".$model->sur_name;
   return $executor;
  })
  ->editColumn('publication_date', function ($model) {
    $publicationDate =  date('d-m-Y', strtotime($model->publication_date));  
   return $publicationDate;
 })
   ->editColumn('claim_date', function ($model) {
    $claim_date =  date('d-m-Y', strtotime($model->claim_date));  
   return $claim_date;
 })
  

->editColumn('created_date', function ($model) {
    return 1;
})


  ->editColumn('action', function ($model) {
    $idG = Crypt::encrypt($model->id);
    // $user = \App\User::find($model->user_id);
    // Auth::loginUsingId($user->id, true);

            return '<a href="' . url('boat-for-sale/' . $model->slug ) . '" type="" target="_blank" class="act-sp" title="View"><i class="fas fa-file"></i></a>  <a href="' . url('edit-boat-info/' . $idG) . '" type="" target="_blank" class="act-sp" title="Login as user to edit"><i class="fas fa-edit"></i></a> ';
        })

        ->rawColumns(['s#','name','address','status','expiry','action'])
        ->make(true);
    }

    public function propertyDetail(Request $request)
    {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::where('status',1)->findOrFail($propId); 
            if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
                $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
                $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
            }
            $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
            $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
            $user_outcode = $selOutcodeCollection;
        // $probateProperties = ProbateProperty::where('status',1)->orderBy('id', 'DESC')->take(10)->get(['address_1','latitude','longitude','id'])->toArray();
            $probateProperties = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
            ->join("postcode", "postcode.id", "probate_property.postcode_id")
            ->join("outcode", "outcode.id", "postcode.outcode_id")
            ->whereIn('outcode.id', $user_outcode)
            ->where('probate_property.status',1)->orderBy('probate_property.id', 'DESC')->take(10)->get(['probate_property.address_1','probate_property.latitude','probate_property.longitude','probate_property.id'])->toArray();
            return view('admin.probate-property-detail')->with(['probateProperty' => $probateProperty,'probateProperties' => $probateProperties]);
        }
    }
    public function getProbatePopupData(Request $request) {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            //$probateProperty = ProbateProperty::where('status',1)->findOrFail($propId);
            $probateProperty = ProbateProperty::join('executor', 'probate_property.executor_id','executor.id')
                ->join("postcode", "postcode.id", "probate_property.postcode_id")
                ->join("outcode", "outcode.id", "postcode.outcode_id")
                ->leftJoin("property_sale_history", "property_sale_history.property_history_id", "probate_property.id")
                ->select('probate_property.*', 'executor.company_name as executor_comapnyname', 'executor.first_name','executor.sur_name','executor.address_1 as executor_address1','executor.address_2 as executor_address2','executor.town as executor_town','executor.postcode as executor_postcode', 'executor.phone as executor_phone','executor.email as executor_email', 'postcode.postcode as postcode_postcode','property_sale_history.price as property_sale_price','property_sale_history.sale_date as property_sale_date','property_sale_history.sold_year as property_sold_year')
                ->findOrFail($propId);
            $str = $probateProperty->address_1;
            $addr1 = explode(" ",$str);
            $postcode = $probateProperty->postcode_postcode;
            $soldHistory = SoldHistory::where('house_no', $addr1[0])->where('postcode',$postcode)->get();
            $returnHTML = view('admin.popup-content',['probateProperty'=>$probateProperty, 'soldHistory' => $soldHistory])->render();

            return response()->json( array('success' => true, 'html'=>$returnHTML) );
        }
    }

    public function getProbateTooltipData(Request $request) {
        if ($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::join('outcode','probate_property.outcode_id','outcode.id')               ->where('status',1)
                               ->findOrFail($propId); 
            $returnHTML = view('admin.tooltip-content',['probateProperty'=>$probateProperty])->render(); 

            return response()->json(array('success' => true, 'html'=>$returnHTML));
        }
    }

    public function getSchoolTooltipData(Request $request) {
        $sch_latitude = $request->latitude;
        $sch_longitude = $request->longitude;
        $prop_latitude = $request->prop_latitude;
        $prop_longitude = $request->prop_longitude;
        $radius = 1.5;
        $schools = School::selectRaw("*,
        ( 3956 * acos( cos( radians(?) ) *
        cos( radians( latitude ) )
        * cos( radians( longitude ) - radians(?)
        ) + sin( radians(?) ) *
        sin( radians( latitude ) ) )
        ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
        ->where([
        ['latitude',$sch_latitude],
        ['longitude',$sch_longitude]
        ])
        ->having('distance', '<=', $radius)->first();

        $returnHTML = view('admin.school-content',['schoolAddress'=>$schools])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getEpcDetails(Request $request) {
        $full_address = explode("|",$request->address);
        $address = trim(trim($full_address[0]," "),",");
        $outcode = trim($full_address[1]," ");
        $epc_url = 'https://epc.opendatacommunities.org/api/v1/domestic/search?postcode='.$outcode.'&address='.$address;
        $epc_client = new Client();
        $epc_results = $epc_client->get($epc_url, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Authorization' => 'Basic c2hhcnU2MzFAZ21haWwuY29tOmQwZjE1Y2ZjNDlmNDlkYzQ2NWQzZjMyYjdlZDMzYjU5OTk3Y2YzYmY='
            ]
        ]);

        $epc_status = $epc_results->getStatusCode();
        $epc_contents = json_decode($epc_results->getBody(), true);
        $epc_details = $epc_contents['rows'];
        $allEpc['data'] = $epc_details[0];
        $returnHTML = view('admin.epc-content',['epcDetails'=>$allEpc ])->render(); 
        return response()->json( array('success' => true, 'html'=>$returnHTML));
    }

    public function getSchoolFullDetails(Request $request) {
        $latitude = $request->sch_latitude;
        $longitude = $request->sch_longitude;

        $schoolDetails = School::where([
            ['latitude',$latitude],
            ['longitude',$longitude]
        ])->first();

        $returnHTML = view('admin.school-full-details',['schoolDetails'=>$schoolDetails])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function schoolCoordUpdate(Request $request) {
        $schools_count = SchoolList::count();
        $count = 0;
        DB::table('schools')->orderBy('urn')->where('latitude', null)->chunk(100, function ($list) {
            foreach ($list as $school) {
                $count =+ 1;
                $urn = $school->urn;
                $school_name = $school->schname;
                $school_latitude = $school->latitude;
                $school_longitude = $school->longitude;
                if ($school_latitude  == null && $school_longitude == null) {
                    $school_address = $school->schname.', '.$school->street.', '.$school->town.', '.$school->postcode;
                    $google_autocomplete = "https://maps.googleapis.com/maps/api/place/autocomplete/json?input=".$school_address."&key=".config('map.apiKey');
                    $autocomplete_client = new Client();
                    $autocomplete_result = $autocomplete_client->get($google_autocomplete, [
                        'verify' => false,
                        'headers' => [
                            'Accept' => 'application/json',
                        ]
                    ]);
                    $autocomplete_status = $autocomplete_result->getStatusCode();
                    $autocomplete_response = json_decode($autocomplete_result->getBody(), true);
                    foreach ($autocomplete_response['predictions'] as $autocomplete_res) {
                        $google_placeId = "https://maps.googleapis.com/maps/api/place/details/json?place_id=".$autocomplete_res['place_id']."&key=".config('map.apiKey');
                        $placeId_client = new Client();
                        $placeId_result = $placeId_client->get($google_placeId, [
                            'verify' => false,
                            'headers' => [
                                'Accept' => 'application/json',
                            ]
                        ]);
                        $placeId_status = $placeId_result->getStatusCode();
                        $placeId_response = json_decode($placeId_result->getBody(), true);
                        $school_location = $placeId_response['result']['geometry']['location'];
                        $lat = $school_location['latitude'];
                        $lng = $school_location['longitude'];
                        School::where("urn",$urn)
                            ->update(
                                array(
                                    "latitude" => $lat,
                                    "longitude" => $lng,
                                )
                            );
                    }
                }
            }
        });
        if ($schools_count == $count) {
            return response()->json(array('success' => true, 'message' => 'Finished'));
        } else {
            return response()->json(array('success' => false, 'message' => 'Error'));
        }
    }

    public function getSchoolList(Request $request) {
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $sc_phase[] = $request->phase;       
        $radius = 1.5;
        $schools = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)->where('rating','!=',null);

                // if (in_array("Primary", $sc_phase))
                // {
                //    $schools = $schools->orWhere('phase_of_education', 'Primary'); 
                // }
                //  if (in_array("Secondary", $sc_phase))
                // {
                //     dd($sc_phase);
                //    $schools = $schools->orWhere('phase_of_education', 'Secondary'); 
                // }

                 $schools =  $schools->get();
         
        return response()->json( array('success' => true, 'html'=>$schools));
    }

    public function nearestSchoolList(Request $request) {
        $post_string = explode(' ', $request->postcode);
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $radius = 1.5;
        $schools_list = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)
                    ->orderBy('distance', 'ASC')
                    ->get();
        
        $returnHTML = view('admin.nearest-school-list',['schoolsList'=>$schools_list])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPhotos(Request $request) {
        $returnHTML = view('admin.photos-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }
    
    public function getFloorPlan(Request $request) {
        $returnHTML = view('admin.floorPlan-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPlanning(Request $request) {
        $returnHTML = view('admin.planning-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getSoldInStreet(Request $request) {
        $postcode = $request->postcode;
        $soldHistory = SoldHistory::select('sold_date','sold_price','tenure','house_no','addr2','addr5','addr6')->where('postcode',$postcode)->get();    
        $result = array();
         foreach ($soldHistory as $sold) {
         $result[$sold['house_no']][] = $sold;
         }
             dd($result);
        $returnHTML = view('admin.soldInStreet-content',['soldHistory'=>$soldHistory])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getMarketData(Request $request) {
        $returnHTML = view('admin.marketData-content')->render(); 
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getOfcom(Request $request) {
        $postcode = $request->postcode;
        $ofcomApi = "https://api-proxy.ofcom.org.uk/broadband/coverage/".str_replace(' ', '', $postcode);
        $ofcom_client = new Client();
        $ofcom_results = $ofcom_client->get($ofcomApi, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Ocp-Apim-Subscription-Key' => config('map.ofcomKey'),
            ]
        ]);
        $ofcom_status = $ofcom_results->getStatusCode();
        $ofcom_contents = json_decode($ofcom_results->getBody(), true);
        $ofcom_result = [];
        foreach ($ofcom_contents['Availability'] as $ofcom_results) {
            $ofcom_result['MaxBbPredictedDown'] = $ofcom_results['MaxBbPredictedDown'];
            $ofcom_result['MaxBbPredictedUp'] = $ofcom_results['MaxBbPredictedUp'];
            $ofcom_result['MaxSfbbPredictedDown'] = $ofcom_results['MaxSfbbPredictedDown'];
            $ofcom_result['MaxSfbbPredictedUp'] = $ofcom_results['MaxSfbbPredictedUp'];
            $ofcom_result['MaxUfbbPredictedDown'] = $ofcom_results['MaxUfbbPredictedDown'];
            $ofcom_result['MaxUfbbPredictedUp'] = $ofcom_results['MaxUfbbPredictedUp'];
            $ofcom_result['MaxPredictedDown'] = $ofcom_results['MaxPredictedDown'];
            $ofcom_result['MaxPredictedUp'] = $ofcom_results['MaxPredictedUp'];
            $ofcom_res['data'] = $ofcom_result;
        }
        $returnHTML = view('admin.ofcom-content',['ofcomDetails'=>$ofcom_res['data']])->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function documents(Request $request) {
        $returnHTML = view('admin.documents')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

     //All property
    public function allProperty(Request $request)
    {
        $user_id            = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title         = 'Property Listings';
        $active             = 2;
        $user_type          = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $post_code          = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')
        ->where('user_id', $user_id)
        ->join('postcode', 'outcode.id', 'postcode.outcode_id')
        ->where('user_outcode.status', 1)
        ->where('outcode.archive', 0)
        ->select('postcode.*')
        ->orderBy('postcode.postcode', 'asc')
        ->get();
        // $property           = PropertyHistory::select('property_type', 'id')->get()->unique('property_type');
        // $property           = $this->commonPropQuery()->select('property_type')->get()->unique('property_type');
        $property           = $this->commonFreshPropQuery()->select('property_history.property_type')->get()->unique('property_type');
        $prop_count         = count($property);
        $agency             = Agency::get();
        $userOutcodes       = $this->userOutcodes();
        $campaign           = 1;
        $filter             = true;
        $enable_export      = User::find($user_id)->enable_export;
        $weeks              = '';
        $search_data        = '';
        $filter_title       = 'All Property Listings(Without Filters)';
        // Filter based on weeks
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                if ($weeks == 1) {
                    $filter_title = 'Property Filtered in 0-4 Weeks';
                } elseif ($weeks == 2) {
                    $filter_title = 'Property Filtered in 4-8 Weeks';
                } elseif ($weeks == 3) {
                    $filter_title = 'Property Filtered in 8-12 Weeks';
                } elseif ($weeks == 4) {
                    $filter_title = 'Property Filtered in 12-16 Weeks';
                } elseif ($weeks == 5) {
                    $filter_title = 'Property Filtered in 16 Weeks or More';
                }
            }
            $search_data        = json_decode(Session::get('search_data'));
            $search_data->weeks = $weeks;
        }

        return view('all-property', compact('page_title', 'filter_title', 'property', 'prop_count', 'post_code', 'userOutcodes', 'user_type', 'active', 'campaign', 'agency', 'filter','search_data','enable_export'));
    }

    public function allPropertyArray(Request $request)
    {
        $sesstion_array =[];
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $user_outcode = $this->userOutcode();
        if ($request->has('weeks') && isset($request->weeks)) {
            $weeks = isset($request->weeks) ? $request->weeks : '';
            if (in_array($weeks, [1, 2, 3, 4, 5])) {
                $query = $this->commonFreshPropQuery();
                // $query = $query->whereNotIn('property_history.for_sale', [3, 6]);
                // $query = $query->whereNotIn('property_price_history.type', ['Withdrawn', 'STC']);
            }
        } else {
            $query = $this->commonPropQuery();
        }
        $query = $query->whereIn('outcode.id', $user_outcode);
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
            $sesstion_array['outcode'] = $request->input('outcode');
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
                $sesstion_array['post_code'] = $request->input('post_code');

            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
                $sesstion_array['type'] = $request->input('type');
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
                $query->where('property_flag.user_id', $user_id);
                $sesstion_array['flag'] = $request->input('flag');
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
                $sesstion_array['agency'] = $request->input('agency');
            }
        }
        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
                $sesstion_array['platform'] = $request->input('platform');
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('weeks')) {
            $sesstion_array['weeks'] = $request->input('weeks');
            $weeks = $request->weeks;
            $dt = Carbon::now();

            if (in_array($weeks, [1, 2, 3, 4])) {
                if (1 == $weeks) {
                    $start = $dt->copy()->subWeeks(4)->addDays(1)->startOfDay();
                    $end = $dt->copy()->endOfDay();
                } elseif (2 == $weeks) {
                    $start = $dt->copy()->subWeeks(8)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(4)->endOfDay();
                } elseif (3 == $weeks) {
                    $start = $dt->copy()->subWeeks(12)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(8)->endOfDay();
                } elseif (4 == $weeks) {
                    $start = $dt->copy()->subWeeks(16)->addDays(1)->startOfDay();
                    $end = $dt->copy()->subWeeks(12)->endOfDay();
                }
                $query->whereBetween('house_number.listed_date', [$start, $end]);
            } elseif (5 == $weeks) {
                $query->whereDate('house_number.listed_date', '<=', $dt->copy()->subWeeks(16)->endOfDay());
            }
        }
        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode.outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'agency.name',
            6 => 'property_type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');
            $sesstion_array['search'] = $search;
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%');

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }

            })->orderBy($order, $dir)->get()->unique('house_number_id');
            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;

                $searchWithHNo = explode(',',$search);
                if(@$searchWithHNo[1]!=''){
                    $q->orWhere('house_number.house_number', 'like', '%'.$searchWithHNo[0].'%')
                    ->where('property_history.address', 'like', '%'.$searchWithHNo[1].'%');

                }
            });
        }

        $dataArr = [];


        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                //   $urls = $r_count.$z_count;
                //   $urls = '<div class=" d-flex"> <a class="btn btn-primary  btn-xs  text-center"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">3</span>
                //    <a class="btn btn-success  btn-xs" href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">3</span></div>
                //   ';
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                // if (!empty($item->house_number)) {
                $address2 = $item->house_number.', '.$item->address;
                // } else {
                // $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                // if (!empty($item->created_by)) {
                //     $address2 = $item->address.',<br>  '.$item->postcode;
                // }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $st = '';
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                // $nestedData['type'] = $request->outcodeFilter.'--'. $request->input('search.value');
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }
        
        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        $session_data =  json_encode( $sesstion_array);
        Session::put('search_data', $session_data );

        return json_encode($json_data);
    }

    //property flag change
    public function flagChange(Request $request)
    {
        // return $request->all();

        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ('' != $exist_data) { //update
                // $update = PropertyFlag::where('id',$exist_data['id'])
                //     ->update(['flag' => $request->flag]);
                $update = PropertyFlag::find($exist_data['id']);
                $update->fill(['flag' => $request->flag])->save();

                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    //property detail page
    public function propertyDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode.');
            }
        }

        $prop = true;
        $house_number_id = Helper::decryptData($request->property_id);
        $query = $this->commonPropQuery();
        $property_details = $query->where('house_number.id', $house_number_id)->first();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $page_title = $property_details->address;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
        ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('house_number_id', $house_number_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
        ->orderBy('property_price_history.event_date', 'desc')->orderBy('property_price_history.id', 'desc')->get();
        // dd($listing_activity);
        $ambiguity_checker_response = $this->propertyAmbiguityChecker(['data_type' => 'listing_activity_object', 'data' => $listing_activity]);

        $comments = Helper::propertyComment($house_number_id);

        $letter_list = PropertyLetter::join('house_number', 'property_letters.house_number_id', 'house_number.id')
        ->join('letters', 'letters.id', 'property_letters.letter_id')
        ->where('property_letters.house_number_id', $house_number_id)
        ->select('property_letters.*', 'letters.name as letter_name')
        ->get()
        ;

        $current_flag = Helper::propertyFlag($house_number_id);

        $portal_url = $this->houseNumberPortal($house_number_id);
        $campaign_history = PropertyCampaign::where('house_number_id', $house_number_id)
        ->whereHas('campaign')
        ->with('campaign')
        ->withTrashed()
        ->where('user_id', $user_id)
        ->orderBy('id', 'desc')
        ->get()
        ;
        $trigger = $this->trigger;

        return view('property-details', compact('property_details', 'listing_activity', 'ambiguity_checker_response', 'comments', 'letter_list', 'house_number_id', 'prop', 'current_flag', 'portal_url', 'house_number_id', 'campaign_history', 'trigger'));
    }

    //property history details
    public function propertyHistoryDetails(Request $request)
    {
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        if ($request->notification) {
            $outcode = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->first();

            $delete = UserNotification::where('user_notification.user_id', $user_id)->
            where('user_notification.id', $request->notification)->delete();

            $exist_code = Useroutcode::where('user_id', $user_id)->where('status', 1)->
            where('outcode_id', $outcode['outcode_id'])->first();
            if (!$exist_code) {
                return Redirect::route('userHome')->with('msg_error', 'You have not subscribe this postcode');
            }
        }

        $prop = true;
        $property_history_id = Helper::decryptData($request->property_id);

        // dd($property_history_id);

        $property_details = PropertyHistory::join('postcode', 'property_history.postcode_id', 'postcode.id')
        ->where('property_history.id', $property_history_id)
        ->select('property_history.*', 'postcode.postcode as p_code')
        ->first()
        ;

        $listing_activity = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->where('property_history.id', $property_history_id)
        ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id')
        ->orderBy('property_price_history.event_date', 'desc')->get();

        // dd($listing_activity);

        // $comments = Helper::propertyComment($house_number_id);
        $comments = [];

        // $letter_list = PropertyLetter::join('house_number','property_letters.house_number_id','house_number.id')
        //     ->join('letters','letters.id','property_letters.letter_id')
        //     ->where('property_letters.house_number_id',$house_number_id)
        //     ->select('property_letters.*','letters.name as letter_name')
        //     ->get();

        $letter_list = [];

        $portal_url[0] = ['portal' => $property_details->portal, 'portal_url' => $property_details->portal_url];

        return view('property-history-details', compact('property_details', 'listing_activity', 'comments', 'letter_list', 'prop', 'portal_url'));
    }

    // Check if ambiguous data is provided by the system
    public function propertyAmbiguityChecker($data)
    {
        $data_type = isset($data['data_type']) ? $data['data_type'] : '';
        if ($data_type === 'listing_activity_object') {
            $listing_activity   = isset($data['data']) ? $data['data'] : '';
        } else if ($data_type === 'house_number_id') {
            $house_number_id    = isset($data['data']) ? $data['data'] : '';
            if ($house_number_id != '') {
                $listing_activity   = PropertyPriceHistory::join('property_history', 'property_history.id', 'property_price_history.property_history_id')
                ->join('property_house_number', 'property_history.id', 'property_house_number.property_history_id')
                ->join('house_number', 'house_number.id', 'property_house_number.house_number_id')
                ->join('agency', 'agency.id', 'property_history.agency_id')
                ->where('house_number_id', $house_number_id)
                ->select('property_price_history.*', 'agency.name', 'property_history.portal', 'property_history.agency_id', 'property_history.id')
                ->orderBy('property_price_history.event_date', 'desc')
                ->orderBy('property_price_history.id', 'desc')
                ->get();
            }
        }
        if (!empty($listing_activity) && count($listing_activity) > 0) {
            // Convert listing fetch to array
            $listing_activity_array = $listing_activity->toArray();
            // prepare portal to agent array
            $list_activity_portal_array         = array_column($listing_activity_array, 'portal');
            $list_activity_agent_array          = array_column($listing_activity_array, 'name');
            $list_activity_portal_agent_array   = [];
            $i = 0;
            // Prepare an event date array out of listing activity with correct position keys
            foreach ($listing_activity_array as $list_key => $list_value) {
                if (isset($listing_activity_array[$i]['event_date']) && $listing_activity_array[$i]['event_date'] != '') {
                    $list_activity_array[$i] = $listing_activity_array[$i]['event_date'];
                }
                $list_activity_portal_agent_array[] = $list_activity_portal_array[$i]." - ".$list_activity_agent_array[$i];
                $i++;
            }
            $unique_list_activity               = array_unique($list_activity_portal_agent_array);
            $unique_agent_activity_count        = count($unique_list_activity);
            // Preparing activity_listing_position_array for comparing in pattern
            $temp_listing_activity_array        = $list_activity_array;
            $count_list_activity_array          = count($list_activity_array);
            $count_activity                     = array_count_values($list_activity_array);
            $total_count_activity               = count($count_activity);
            $cycle_count                        = 0;
            $activity_listing_position_array    = [];
            foreach ($count_activity as $count_activity_key => $count_activity_value) {
                if ($count_activity_value > 1) {
                    $cycle_count += $count_activity_value;
                }
            }
            $temp_cycle_count                   = $cycle_count;
            if ($cycle_count > 0) {
                while ((count($temp_listing_activity_array) >= $cycle_count) && $cycle_count !== 0) {
                    foreach ($count_activity as $count_activity_key => $count_activity_value) {
                        if ($count_activity_value > 1) {
                            $activity_listing_position = '';
                            $activity_listing_position = array_search($count_activity_key, $temp_listing_activity_array);
                            if ($activity_listing_position !== false) {
                                $activity_listing_position_array[$activity_listing_position] = $count_activity_key;
                                unset($temp_listing_activity_array[$activity_listing_position]);
                            }
                        }
                    }
                    $cycle_count--;
                }
                // Sort in DESC order of activity date
                ksort($activity_listing_position_array);
            }
            // ambiguous listing checks
            $ambiguous_listing_flag             = false;
            $type_unfavourable_array            = ['Withdrawn', 'STC', 'Sold'];
            $first_listing_activity_array       = array_key_first($listing_activity_array);
            // $type_unfavourable_array            = ['Withdrawn'];
            if (!empty($activity_listing_position_array)) {
                $total_activity_listing_position_array = count($activity_listing_position_array);
                foreach ($activity_listing_position_array as $alpa_key => $alpa_value) {
                    $next_activity_listing = next($activity_listing_position_array);
                    // pattern checking for ambiguous data provided for a listing through system
                    // Pattern 1
                    if (isset($listing_activity_array[$alpa_key]['type'])
                        && isset($listing_activity_array[$alpa_key+1]['type'])
                        && ($listing_activity_array[$alpa_key]['type'] != $listing_activity_array[$alpa_key+1]['type']) // check if event type is different
                        && ($next_activity_listing !== false) // check if next item available
                        && (in_array($listing_activity_array[$alpa_key]['type'], $type_unfavourable_array)) // check if type is not offer or price related
                        // && ($listing_activity_array[$alpa_key]['portal'] === $listing_activity_array[$alpa_key+1]['portal']) // check if same portal provides the data
                        && ($listing_activity_array[$alpa_key]['event_date'] === $listing_activity_array[$alpa_key+1]['event_date'])) // check if event date is same
                        // && ($listing_activity_array[$alpa_key]['name'] !== $listing_activity_array[$alpa_key+1]['name'])) // check if portal agent is different
                    {
                        $ambiguous_listing_flag = true;
                    }
                }
                // Pattern 2
                if (isset($listing_activity_array[$first_listing_activity_array]['type'])
                    && (in_array($listing_activity_array[$first_listing_activity_array]['type'], $type_unfavourable_array)) // check if first listing type is unfavourable
                    && ($unique_agent_activity_count > 1)) // check if first listing type is unfavourable
                {
                    $ambiguous_listing_flag = true;
                }
            }
            // debuging unit for ambiguous_listing_flag listing data
            // $list_activity_array            = array_column($listing_activity_array, 'event_date');
            // $list_activity_array_filtered   = array_filter($list_activity_array);
            // print("<pre>".print_r($listing_activity_array,true)."</pre>");
            /*print("<pre>".print_r($list_activity_array,true)."</pre>");
            echo "total activity count : $count_list_activity_array";
            print("<pre>".print_r($count_activity,true)."</pre>");
            echo "unique activity count : $total_count_activity<br>";
            print("<pre>".print_r($temp_listing_activity_array,true)."</pre>");
            echo "cycle count : $temp_cycle_count<br>";
            echo "cycle count now: $cycle_count<br>";
            print("<pre>".print_r($activity_listing_position_array,true)."</pre>");*/
            // echo $ambiguous_listing_flag;
            // dd('stopped');
            return $ambiguous_listing_flag;
        }
    }

    //comment delete
    public function deletePropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return $this->simpleReturn('error', $validator->errors()->first());
        }

        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $update = PropertyComment::where('id', $request->comment)->delete();
        if ($update) {
            return $this->simpleReturn('success', 'Successfully comment deleted..!');
        }

        return $this->simpleReturn('error', 'Something went wrong...!');
    }

    //add property comment
    public function addPropertyComment(Request $request)
    {
        $rules = [
            'comment' => 'required',
            'property' => 'required',
        ];

        $validator = validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return Redirect::back()->with('error', $validator->errors()->first());
        }
        
        $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $add = new PropertyComment();
        $add->house_number_id = $request->property;
        $add->comment = $request->comment;
        $add->user_id = $user_id;
        if ($add->save()) {
            return Redirect::back()->with('success', 'Successfully comment deleted..!');
        }

        return Redirect::back()->with('error', 'Something went wrong...!');
    }

    //search property
    public function searchProperty(Request $request)
    {
        $search_keyword = $request->search;
        $page_title = 'Addresses or Agencies That Contain: '.$search_keyword;
        session()->put('search_keyword', $search_keyword);

        return view('search-result', compact('page_title'));
    }

    //search property array
    public function searchPropertyArray(Request $request)
    {
        $search_keyword = session()->get('search_keyword');
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        // ->Where('property_history.address', 'like', '%' . $search_keyword . '%')
        // ->orWhere('agency.name', 'like', '%' . $search_keyword . '%')
        // ->orWhere('postcode.postcode', 'like', '%' . $search_keyword . '%');
        // ->orWhere('property_history.property_type', 'like', '%' . $search_keyword . '%');

        $query = $query->where(function ($q) use ($search_keyword) {
            $q->Where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('property_history.property_type', 'like', '%'.$search_keyword.'%')
            ->orWhere('outcode.outcode', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
        });

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'agency',
            4 => 'listed_date',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //  $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)
            ->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');
            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()->unique('house_number_id');
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }

                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                //  $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    //new to market
    public function newToMarket()
    {
        $page_title = 'Weekly New to Market';
        $campaign = 1;

        return view('new_to_market', compact('page_title', 'campaign'));
    }

    public function newToMarketArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        $new_date_from = date('Y-m-d', strtotime('-7 days'));
        $new_date_to = date('Y-m-d');

        $query = $query->whereBetween('house_number.listed_date', [$new_date_from, $new_date_to]);

        // dd($items->where('house_number.id',1)->get());
        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'price',
            4 => 'listed_date',
            5 => 'agency.name',
            6 => 'type',
            7 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['outcode'] = $item->outcode;
                $nestedData['outcode'] = $item->outcode;

                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }
                // $listed_date=date('Y-m-d',strtotime($item->listed_date));
                // $today=date('Y-m-d');
                // $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                // $nestedData['listed_date'] = date('d M Y',strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }
                // $nestedData['week'] = $item->week;
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //withdrawn
    public function withdrawn()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Withdrawn';

        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;
        $userOutcodes = $this->userOutcodes();

        return view('withdrawn-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function withdrawnArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        if (1 == $user_type) { //admin
            $data = HouseNumber::get();
        } elseif (2 == $user_type) { //agent
            $user_outcode = $this->userOutcode();
            $outcodes = $this->userOutcodes();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.latest_status', 3);
            // dd($items->where('house_number.id',1)->get());
            if ($request->has('outcode') && '' != $request->input('outcode')) {
                $query->where('outcode.outcode', $request->input('outcode'));
            }

            if ($request->has('post_code')) {
                if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                    $query->where('postcode.id', $request->input('post_code'));
                }
            }
            // dd($request->all());
            if ($request->has('type')) {
                if (('0' != $request->type && 'undefined' != $request->type)) {
                    $query->where('property_history.property_type', $request->input('type'));
                }
            }
            if ($request->has('flag')) {
                if ((0 != $request->flag && 'undefined' != $request->flag)) {
                    $query->where('property_flag.flag', $request->input('flag'));
                }
            }
            if ($request->has('agency')) {
                if ((0 != $request->agency && 'undefined' != $request->agency)) {
                    $query->where('agency.id', $request->input('agency'));
                }
            }
            if ($request->has('platform')) {
                if ((-1 != $request->platform && 'platform' != $request->platform)) {
                    $query->where('property_history.portal', $request->input('platform'));
                }
            }
            if ($request->has('date_from') && $request->has('date_to')) {
                if (('' != $request->date_from && '' != $request->date_to)) {
                    $date_from = date('Y-m-d', strtotime($request->date_from));
                    $date_to = date('Y-m-d', strtotime($request->date_to));
                    $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
                }
            }
        } else { //data entry operator
            $data = HouseNumber::get();
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            5 => 'withdrawn_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                // $nestedData['price'] = "&pound; ".number_format($item->latest_price);
                if (0 == $item->latest_price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $withdrwan_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item->house_number_id)
                ->where('property_price_history.type', 'Withdrawn')
                ->orderBy('property_price_history.event_date', 'desc')
                ->select('property_price_history.event_date')
                ->first()
                ;

                // $nestedData['withdrawn_date'] = date('d M Y', strtotime( $withdrwan_date->event_date
                $nestedData['withdrawn_date'] = date('d M Y', strtotime($item->withdrawn_date));

                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function recentlySold()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Sold - Under Offer';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $filter = true;
        $agency = Agency::get();
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('recently-sold-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'filter', 'agency', 'totalProp'));
    }

    public function recentlySoldArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        //print_r($userOutcodes); exit;

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode)
        ->where('house_number.latest_status', '!=', 3)
        ;

        $query = $query->where(function ($q) {
            $q->Where('property_history.sold_stc', 1)
            ->orWhere('property_history.under_offer', 1)
            ;
        });

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'house_number.listed_date',
            // 5 =>'property_price_history.event_date',
            6 => 'agency.name',
            7 => 'property_type',
            8 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        // $outcodes = $query->pluck('outcode.outcode','outcode.id')->toArray();
        $outcodes = $this->userOutcodes();

        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        // $query = $query->orderBy('house_number.listed_date','desc');

        if (empty($request->input('search.value'))) {
            // $data = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
            // $dd = $query->orderBy($order,$dir)->get()->unique('house_number_id')->toArray();
            // $list = array_slice($data, $start, $limit);
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            //  $arr = json_decode(json_encode($list));
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';

                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item['house_number_id']);

                $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                ->where('property_house_number.house_number_id', $item['house_number_id'])
                ->where('property_price_history.type', 'STC')
                ->orderBy('property_price_history.id', 'desc')
                ->limit(1)
                ->select('property_price_history.event_date', 'property_price_history.sold_year')
                ->first()
                ;

                if (!$sold_date) {
                    $sold_date = PropertyPriceHistory::join('property_house_number', 'property_house_number.property_history_id', 'property_price_history.property_history_id')
                    ->where('property_house_number.house_number_id', $item['house_number_id'])
                    ->where('property_price_history.type', 'Offer')
                    ->orderBy('property_price_history.id', 'desc')
                    ->limit(1)
                    ->select('property_price_history.event_date', 'property_price_history.sold_year')
                    ->first()
                    ;
                }

                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;

                if ($sold_date) {
                    if ($sold_date->event_date) {
                        $so_date = date('d M Y', strtotime($sold_date->event_date));
                    } else {
                        $so_date = $sold_date->sold_year;
                    }
                } else {
                    $so_date = '';
                }

                $nestedData['sold_date'] = $so_date;

                if ($item['listed_date']) {
                    $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['id'] = $item['house_number_id'];
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item['outcode'];
                if (null == $item['created_by']) {
                    $st = 'By System';
                    $color = '';
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }

                if (!empty($item['house_number'])) {
                    $address2 = $item['house_number'].', '.$item['address'];
                } else {
                    $address2 = $item['house_number'].', '.$item['address'].',<br>  '.$item['postcode'];
                }
                if (!empty($item['created_by'])) {
                    $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";

                // $nestedData['price'] = "&pound; ".number_format($item['latest_price']);
                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                // $nestedData['sold_date'] =  date('d M Y',strtotime($item['sold_date']));
                $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                $nestedData['type'] = $item['title'].' <br> '.ucfirst($item['property_type']);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //recently sold
    public function priceDrop()
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Recent Price Drops        ';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $campaign = 1;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $agency = Agency::get();
        $filter = true;
        $totalProp = true;

        $userOutcodes = $this->userOutcodes();

        return view('price-drop-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'campaign', 'agency', 'filter', 'totalProp'));
    }

    //price drop
    public function priceDropArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        // if($user_type==1){//admin
        //     $data = HouseNumber::get();

        // }elseif($user_type==2){//agent

        $user_outcode = $this->userOutcode();

        // $query = $this->commonPropQuery();
        // $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.price_change_diff', '<', 0);

        $query = HouseNumber::join('property_house_number', 'property_house_number.house_number_id', 'house_number.id')
        ->join('property_price_history', 'property_price_history.property_history_id', 'property_house_number.property_history_id')
        ->join('property_history', 'property_history.id', 'property_house_number.property_history_id')
        ->join('postcode', 'postcode.id', 'house_number.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_flag', 'property_flag.house_number_id', 'property_house_number.house_number_id')
        ->whereNull('property_house_number.deleted_at')
        ->whereIn('outcode.id', $user_outcode)
        ->Where('property_price_history.type', 'like', '%Price reduced by%')
        ->select('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', 'house_number.latest_price', 'house_number.latest_status', 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id', 'house_number.created_by', 'property_price_history.type', 'property_price_history.event_date')
        ->orderBy('property_price_history.event_date', 'desc')
        ;
        // ->groupBy('property_house_number.house_number_id', 'property_history.address', 'property_history.property_type', "house_number.latest_price", "house_number.latest_status", 'property_history.bed_rooms', 'property_history.fallen_through', 'property_history.latitude', 'property_history.longitude', 'house_number.listed_date', 'house_number.house_number', 'postcode.postcode', 'house_number.withdrawn_date', 'house_number.price_drop_date', 'house_number.price_change_diff', 'house_number.sold_date', 'outcode.outcode', 'house_number.created_by', 'property_history.portal', 'house_number.last_sold_price', 'house_number.last_sold_date', 'property_history.title', 'property_flag.flag', 'outcode.id,'property_price_history.type'');

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query->where('property_history.property_type', $request->input('type'));
            }
        }
        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query->where('property_flag.flag', $request->input('flag'));
            }
        }
        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query->where('agency.id', $request->input('agency'));
            }
        }

        if ($request->has('platform')) {
            if ((-1 != $request->platform && 'platform' != $request->platform)) {
                $query->where('property_history.portal', $request->input('platform'));
            }
        }

        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }

        // dd($items->where('house_number.id',1)->get());

        // }else{//data entry operator
        //     $data = HouseNumber::get();
        // }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'outcode',
            2 => 'address',
            3 => 'latest_price',
            4 => 'price_change_diff',
            5 => 'house_number.listed_date',
            6 => 'price_drop_date',
            7 => 'agency.name',
            8 => 'property_type',
            9 => 'flag',
        ];

        $data = $query->get()->unique('house_number_id');
        $totalData = count($data);
        $totalFiltered = $totalData;
        $outcodes = $this->userOutcodes();

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        //$query = $query->orderBy('house_number.listed_date','desc');
        if (empty($request->input('search.value'))) {
            // $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get()->unique('house_number_id');
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get()->unique('house_number_id');

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            });
        }

        $dataArr = [];
        $list = $query->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();
        if (!empty($list)) {
            $list = array_slice($list, $start, $limit);
            $arr = json_decode(json_encode($list));

            $i = 0;
            foreach ($arr as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id, $request->platform);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }

                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                // $address3=empty($item->title)?$address2:$address1;
                if (null == $item->created_by) {
                    $st = 'By System';
                    $color="";
                } else {
                    $st = 'By Manual';
                    $color="style='color:#edf124'";
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['price_drop'] = '-'.substr($item->type, 16);
                // '&pound;'.number_format($item->price_change_diff);
                $listed_date = date('Y-m-d', strtotime($item->listed_date));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);

                if ($item->listed_date) {
                    $dt = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $dt = 'Hidden from public';
                }

                $nestedData['listed_date'] = $dt;
                $nestedData['price_drop_date'] = date('d M Y', strtotime($item->event_date));
                $nestedData['agency'] = Helper::agencyName($item->house_number_id);
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['portal_url'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    //export property data
    public function exportPropertyData(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);
        $file_name = '';

        if ('all_property' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);
                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'All-property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('new_to_market' == $request->mode) {
            $user_outcode = $this->userOutcode();

            $query = $this->commonPropQuery();
            $query = $query->whereIn('outcode.id', $user_outcode)->where('house_number.new_to_market', 1);
            $data_list = $query->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'New-to-market'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('recent_sold' == $request->mode) {
            $query = $query->where('house_number.latest_status', '!=', 3);

            $data_list = $query->where(function ($q) {
                $q->Where('property_history.sold_stc', 1)->orWhere('property_history.under_offer', 1);
            })->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Recentl-sold'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('hot_list' == $request->mode) {
            $data_list = $query->get()->unique('house_number_id');
        } elseif ('withdrawn' == $request->mode) {
            $data_list = $query->where('house_number.latest_status', 2)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,List Date,Withdrawn Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['withdrawn_date'] = date('d-m-Y', strtotime($item->withdrawn_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Withdrawn-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('price_drop' == $request->mode) {
            $data_list = $query->where('house_number.price_change_diff', '<', 0)->get()->unique('house_number_id');
            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['price_drop'] = number_format($item->price_change_diff);
                $arr['listed_date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['price_drop_date'] = date('d-m-Y', strtotime($item->price_drop_date));
                $arr['agency'] = Helper::agencyName($item->house_number_id);
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Price-drop-property'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } elseif ('all_type_property' == $request->mode) {
            $user_outcode = $this->userOutcode();
            $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
            $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();
            $list1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
            ->join('outcode', 'outcode.id', 'postcode.outcode_id')
            ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
            ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
            ->where('property_house_number.house_number_id', null)
            ->where('property_house_number.property_history_id', null)
            ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title')->get()->toArray();

            $list2 = $this->commonPropQuery();
            $list2 = $list2->whereIn('outcode.id', $user_outcode)->get()->unique('house_number_id')->toArray();
            $data_list = array_merge($list1, $list2);

            $data_item = 'Sl,Address,Price,Price Drop,List Date,Price Drop Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item['house_number_id'])) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['price'] = number_format($item['latest_price']);
                $arr['listed_date'] = date('d-m-Y', strtotime($item['listed_date']));
                if ($item['house_number_id']) {
                    $arr['address'] = $item['house_number'].', '.$item['address'].', '.$item['postcode'];
                    $arr['agency'] = Helper::agencyName($item['house_number_id']);
                } else {
                    $arr['address'] = $item['address'].', '.$item['postcode'];
                    $arr['agency'] = $item['name'];
                }
                $arr['type'] = ucfirst($item['property_type']);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'all-property-data'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        } else {
            $search_keyword = session()->get('search_keyword');
            $query = $query->where('property_history.address', 'like', '%'.$search_keyword.'%')
            ->orWhere('agency.name', 'like', '%'.$search_keyword.'%')
            ->orWhere('postcode.postcode', 'like', '%'.$search_keyword.'%')
            ;
            $data_list = $query->get()->unique('house_number_id');

            $data_item = 'Sl,Address,Price,List Date,Agency,Type,Flag';
            $i = 0;
            foreach ($data_list as $item) {
                ++$i;
                $arr = [];
                if (1 == Helper::propertyFlag($item->id)) {
                    $flag = 'Contacted';
                } elseif (2 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Do Not Contact';
                } elseif (3 == Helper::propertyFlag($item->house_number_id)) {
                    $flag = 'Letter Sent';
                } else {
                    $flag = 'No Status';
                }

                $arr['sl'] = $i;
                $arr['address'] = $item->house_number.', '.$item->address.', '.$item->postcode;
                $arr['price'] = number_format($item->latest_price);
                $arr['date'] = date('d-m-Y', strtotime($item->listed_date));
                $arr['agency'] = $item->agency_name;
                $arr['type'] = ucfirst($item->property_type);

                $arr['flag'] = $flag;

                $items = implode('","', $arr);
                $data_item .= PHP_EOL.'"'.$items.'"';
            }
            $file_name = 'Property-'.date('dmYHis').'.csv';
            Storage::disk('property-downloads')->put($file_name, $data_item);
        }

        $url = url('storage/property-downloads/'.$file_name);
        $data['url'] = $url;
        $data['name'] = $file_name;

        return $data;
    }

    //Hotlist
    public function hotList()
    {
        $page_title = 'Hotlist';

        return view('hotlist', compact('page_title'));
    }

    public function hotListArray(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();

        $query = $this->commonPropQuery();
        $query = $query->whereIn('outcode.id', $user_outcode);

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'house_number_id',
            1 => 'address',
            2 => 'price',
            3 => 'listed_date',
            4 => 'agency',
            5 => 'type',
            6 => 'flag',
        ];

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;

            // $totalFiltered =$query->Where('property_history.address', 'like', '%' . $search . '%')
            // ->orWhere('agency.name', 'like', '%' . $search . '%')
            // ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            // ->count();
        }

        $dataArr = [];
        if (!empty($list)) {
            $i = 0;
            foreach ($list as $item) {
                //portal url
                $portal_url = $this->houseNumberPortal($item->house_number_id);
                $platform = '';
                $r = 0;
                $z = 0;
                $r_count = '';
                $z_count = '';
                if (count($portal_url)) {
                    $platform = '<div class=" d-flex">';
                    foreach ($portal_url as $url) {
                        // $r_count="";$z_count="";
                        if (0 == $url->portal) {
                            ++$r;
                        } else {
                            ++$z;
                        }
                    }
                    if ($r > 0) {
                        $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                    }
                    if ($z > 0) {
                        $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                    }
                    $platform .= $r_count.$z_count;
                    $platform .= '</div>';
                }
                $encrpt_id = Helper::encryptData($item->house_number_id);
                $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                ++$i;
                $nestedData['id'] = $item->house_number_id;
                $nestedData['sl'] = $i;
                $nestedData['outcode'] = $item->outcode;
                if (!empty($item->house_number)) {
                    $address2 = $item->house_number.', '.$item->address;
                } else {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                if (!empty($item->created_by)) {
                    $address2 = $item->house_number.', '.$item->address.',<br>  '.$item->postcode;
                }
                $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                $nestedData['price'] = '&pound;'.number_format($item->latest_price);
                $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date));
                $nestedData['agency'] = $item->agency_name;
                $nestedData['type'] = $item->title.'<br> '.ucfirst($item->property_type);
                $nestedData['comments'] = $platform;
                $nestedData['flag'] = Helper::propertyFlag($item->house_number_id);
                $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item->house_number_id)->where('user_id', $user_id)->count();

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
        ];

        return json_encode($json_data);
    }

    public function filterByPostcode(Request $request)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;

        $post_code = $request->postCode;
        if ($post_code) {
        }

        foreach ($request->property as $house_number_id) {
            $exist_data = PropertyFlag::where('user_id', $user_id)->where('house_number_id', $house_number_id)->first();

            if ($exist_data) { //update
                $update = PropertyFlag::where('id', $exist_data['id'])
                ->update(['flag' => $request->flag])
                ;
                if (!$update) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            } else { //insert
                $add = new PropertyFlag();
                $add->house_number_id = $house_number_id;
                $add->user_id = $user_id;
                $add->flag = $request->flag;
                if (!$add->save()) {
                    return $this->simpleReturn('error', 'Something went wrong...!');
                }
            }
        }

        return $this->simpleReturn('success', 'Successfully updated the flag...!');
    }

    public function propertyTrigger($tab)
    {
        $user_id = $id = json_decode(Session::get(config('propai.authSessionKey')))->id;
        $page_title = 'Property';
        $post_code = UserOutcode::join('outcode', 'outcode.id', 'user_outcode.outcode_id')->where('user_outcode.status', 1)->where('user_id', $user_id)->join('postcode', 'outcode.id', 'postcode.outcode_id')->where('outcode.archive', 0)->select('postcode.*')->orderBy('postcode.postcode', 'asc')->get();
        $active = 2;
        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $enable_export = User::find($user_id)->enable_export;
        $userOutcodes = $this->userOutcodes();

        if (1 == $tab) {
            $filter = true;
            $active = 1;
            $agency = Agency::get();
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');

            return view('property-all-type', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'agency', 'property', 'filter','enable_export'));
        }
        if (2 == $tab) {
            $campaign = 1;
            $active = 2;
            $filter = true;
            $agency = Agency::get();

            return view('all-property', compact('page_title', 'property', 'post_code', 'userOutcodes', 'active', 'user_type', 'filter', 'agency', 'campaign','enable_export'));
        }
        if (3 == $tab) {
            $active = 3;
            $property = PropertyHistory::
            select('property_type', 'id')->get()->unique('property_type');
            $filter = true;
            $post_code = PostCode::orderBy('postcode', 'asc')->get()->pluck('postcode', 'id');
            return view('common.property.property-without-address', compact('page_title', 'post_code', 'userOutcodes', 'user_type', 'active', 'property', 'filter','enable_export'));
        }

        return view('all-property', compact('page_title', 'post_code', 'userOutcodes', 'active', 'user_type','enable_export'));
    }

    public function allTypeProperty()
    {
        $page_title = 'Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        $active = 1;

        return view('admin.property-all-type', compact('page_title', 'post_code', 'user_type', 'active'));
    }

    public function allTypePropertyArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $outcode = OutCode::whereIn('id', $user_outcode)->get(['id'])->toArray();
        $postcode = PostCode::whereIn('outcode_id', $outcode)->get(['id'])->toArray();

        $query1 = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->leftjoin('agency', 'agency.id', 'property_history.agency_id')
        ->leftjoin('property_house_number', 'property_house_number.property_history_id', 'property_history.id')
        ->where('property_house_number.house_number_id', null)
        ->where('property_house_number.property_history_id', null)
        ->whereIn('postcode_id', $postcode)->doesnthave('propertyHouseNumber')->select('outcode.outcode', 'property_history.address', 'property_history.price as latest_price', 'property_history.listed_date', 'agency.name', 'property_history.property_type', 'property_history.portal', 'property_house_number.house_number_id', 'property_history.id', 'postcode.postcode', 'property_history.title');

        // ->get()->toArray();

        $query2 = $this->commonPropQuery();
        $query2 = $query2->whereIn('outcode.id', $user_outcode);

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query1->where('outcode.outcode', $request->input('outcode'));
            $query2->where('outcode.outcode', $request->input('outcode'));
        }

        if ($request->has('post_code')) {
            if ((0 != $request->post_code && 'undefined' != $request->post_code)) {
                $query1->where('postcode.id', $request->input('post_code'));
                $query2->where('postcode.id', $request->input('post_code'));
            }
        }
        if ($request->has('platform')) {
            if (('-1' != $request->platform && 'undefined' != $request->platform)) {
                $query1->where('property_history.portal', $request->input('platform'));
                $query2->where('property_history.portal', $request->input('platform'));
            }
        }
        if ($request->has('date_from') && $request->has('date_to')) {
            if (('' != $request->date_from && '' != $request->date_to)) {
                $date_from = date('Y-m-d', strtotime($request->date_from));
                $date_to = date('Y-m-d', strtotime($request->date_to));
                $query1->whereBetween('property_history.listed_date', [$date_from, $date_to]);
                $query2->whereBetween('house_number.listed_date', [$date_from, $date_to]);
            }
        }
        if ($request->has('type')) {
            if (('0' != $request->type && 'undefined' != $request->type)) {
                $query1->where('property_history.property_type', $request->input('type'));
                $query2->where('property_history.property_type', $request->input('type'));
            }
        }

        if ($request->has('flag')) {
            if ((0 != $request->flag && 'undefined' != $request->flag)) {
                $query2->where('property_flag.flag', $request->input('flag'));
            }
        }

        if ($request->has('agency')) {
            if ((0 != $request->agency && 'undefined' != $request->agency)) {
                $query1->where('agency.id', $request->input('agency'));
                $query2->where('agency.id', $request->input('agency'));
            }
        }
        // ->get()->unique('house_number_id')->toArray();

        // $all_data = array_merge($all, $withoutHouseNumber);

        // Table's primary key
        $primaryKey = 'id';
        // $outcodes = $this->userOutcodes();

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'latest_price',
            3 => 'listed_date',
            4 => 'agency_name',
            5 => 'property_type',
            6 => 'flag',
        ];

        $q_data1 = $query1->get()->toArray();
        $q_data2 = $query2->get()->unique('house_number_id')->toArray();
        $all_data = array_merge($q_data1, $q_data2);
        $totalData = count($all_data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        if (empty($request->input('search.value'))) {
            $list1 = $query1->orderBy($order, $dir)->get()->toArray();
            $list2 = $query2->orderBy($order, $dir)->get()->unique('house_number_id')->toArray();

            $data_val = array_merge($list1, $list2); //echo "<pre>"; print_r($data_val); exit;
        } else {
            $search = $request->input('search.value');

            $list1 = $query1->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->toArray()
            ;

            $list2 = $query2->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('house_number.house_number', 'like', '%'.$search.'%')
                ->orWhere('house_number.latest_price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->get()
            ->unique('house_number_id')
            ->toArray()
            ;

            $data_val = array_merge($list1, $list2);

            // $totalFiltered = $query->where(function($q) use ($search) {
            //     $q->Where('property_history.address', 'like', '%' . $search . '%')
            //         ->orWhere('agency.name', 'like', '%' . $search . '%')
            //     ->orWhere('property_history.property_type', 'like', '%' . $search . '%')
            //     ->orWhere('outcode.outcode', 'like', '%' . $search . '%')
            //     ->orWhere('postcode.postcode', 'like', '%' . $search . '%');

            // })->count();
            $totalFiltered = count($data_val);
        }

        $dataArr = [];
        $outcodes = $this->userOutcodes();

        if (!empty($data_val)) {
            $data_val = array_slice($data_val, $start, $limit); // return the first five elements

            $i = 0;
            foreach ($data_val as $item) {
                $nestedData['outcode'] = $item['outcode'];
                $listed_date = date('Y-m-d', strtotime($item['listed_date']));
                $today = date('Y-m-d');
                $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                if ($item['listed_date']) {
                    $nestedData['listed_date'] = date('d M Y', strtotime($item['listed_date'])).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                // $nestedData['price'] = "&pound;" . number_format($item['latest_price']);

                if (0 == $item['latest_price']) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item['latest_price']);
                }

                $nestedData['type'] = ucfirst($item['title']).'<br> '.ucfirst($item['property_type']);

                if ($item['house_number_id']) {
                    //portal url
                    $portal_url = $this->houseNumberPortal($item['house_number_id'], $request->platform);
                    $platform = '';
                    $r = 0;
                    $z = 0;
                    $r_count = '';
                    $z_count = '';
                    if (count($portal_url)) {
                        $platform = '<div class=" d-flex">';
                        foreach ($portal_url as $url) {
                            // $r_count="";$z_count="";
                            if (0 == $url->portal) {
                                ++$r;
                            } else {
                                ++$z;
                            }
                        }
                        if ($r > 0) {
                            $r_count = '<a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">'.$r.'</span>';
                        }
                        if ($z > 0) {
                            $z_count = '<a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">'.$z.'</span>';
                        }
                        $platform .= $r_count.$z_count;
                        $platform .= '</div>';
                    }
                    $encrpt_id = Helper::encryptData($item['house_number_id']);
                    $url = route('propertyDetails', ['property_id' => $encrpt_id]);
                    ++$i;
                    $nestedData['id'] = $item['house_number_id'];
                    $nestedData['agency'] = Helper::agencyName($item['house_number_id']);
                    if (!empty($item['house_number'])) {
                        $address2 = $item['house_number'].' , '.$item['address'];
                    } else {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    if (!empty($item['created_by'])) {
                        $address2 = $item['house_number'].' , '.$item['address'].',<br>  '.$item['postcode'];
                    }
                    // $address3=empty($item['title'])?$address2:$address1;
                    if (null == $item['created_by']) {
                        $st = 'By System';
                        $color = '';
                    } else {
                        $st = 'By Manual';
                        $color="style='color:#edf124'";
                    }
                    $nestedData['address'] = "<a {$color} href='{$url}'>{$address2}</a>";
                    // $nestedData['price'] = "&pound; ".number_format($item['latest_price']) ;

                    $nestedData['type'] = $item['title'].'<br> '.ucfirst($item['property_type']);
                    $nestedData['portal_url'] = $platform;
                    $nestedData['flag'] = Helper::propertyFlag($item['house_number_id']);
                    $user_id = json_decode(Session::get(config('propai.authSessionKey')))->id;
                    $nestedData['letter_count'] = CampaignLetterHistory::where('house_number_id', $item['house_number_id'])->where('user_id', $user_id)->count();
                    $dataArr[] = $nestedData;
                } else {


                    $encrpt_id = Helper::encryptData($item['id']);
                    $nestedData['id'] = '';

                    $url = route('propertyHistoryDetails', ['property_id' => $encrpt_id]);

                    $nestedData['property_number'] = '<div class="input-group">
                    <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                    <div class="input-group-append">
                    <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item['id'].'"><i class="flaticon-paper-plane icon-lg"></i></button>
                    </div>
                    </div>';

                    $nestedData['address'] = "<a  href='{$url}'>{$item['address']},  {$item['postcode']}</a>";

                    $nestedData['agency'] = $item['name'];

                    if (0 == $item['portal']) {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-rightmove  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">R</i></a><span class="counter counter-lg">1</span></div>';
                    } else {
                        $nestedData['portal_url'] = '<div class=" d-flex"><a class="btn btn-zoopla  btn-xs  text-center btn-static"  href="javascript:void(0)"><i class="icon-md">Z</i></a><span class="counter counter-lg">1</span></div>';
                    }

                    $nestedData['flag'] = 0;
                    $nestedData['letter_count'] = 0;
                    ++$i;

                    $dataArr[] = $nestedData;
                }
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutPostcode(Request $request)
    {
        $page_title = 'Without Postcode Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;
        $without_data = true;

        return view('property-without-postcode', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutPostcodeArray(Request $request)
    {
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $user_outcode = $this->userOutcode();
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('postcode.postcode', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;
        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;
                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                // $nestedData['type'] = $totalFiltered;
                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function propertyWithoutAgent(Request $request)
    {
        $page_title = 'Without Agent Property';
        $post_code = PostCode::orderBy('postcode.postcode', 'asc')->get();
        $user_type = $id = json_decode(Session::get(config('propai.authSessionKey')))->user_type;

        $property = PropertyHistory::
        select('property_type', 'id')->get()->unique('property_type');
        // $totalProp = true;

        $without_data = true;

        return view('property-without-agent', compact('post_code', 'user_type', 'property', 'without_data'));
    }

    public function propertyWithoutAgentArray(Request $request)
    {
        $user_outcode = $this->userOutcode();
        $user_type = json_decode(Session::get(config('propai.authSessionKey')))->user_type;
        // $query = $this->commonPropQuery();
        $query = PropertyHistory::join('postcode', 'postcode.id', 'property_history.postcode_id')
        ->join('outcode', 'outcode.id', 'postcode.outcode_id')
        ->join('agency', 'agency.id', 'property_history.agency_id')
        ->where('agency.name', 'Unknown')
        ->select('property_history.*', 'agency.name', 'outcode.outcode')
        ;

        if (2 == $user_type) {
            $query = $query->whereIn('outcode.id', $user_outcode);
        }

        // Table's primary key
        $primaryKey = 'id';

        $columns = [
            0 => 'outcode',
            1 => 'address',
            2 => 'price',
            3 => 'property_history.listed_date',
            4 => 'agency',
            5 => 'type',
        ];

        // if ($user_type == 2) {
        // $user_outcode = $this->userOutcode();
        $outcodes = $this->userOutcodes();
        // $query = $query->whereIn('outcode.id', $user_outcode);
        // }

        if ($request->has('outcode') && '' != $request->input('outcode')) {
            $query->where('outcode.outcode', $request->input('outcode'));
        }

        // if ($request->has('platform')) {
        //     if (($request->platform != '-1' && $request->platform != 'undefined')) {
        //         $query->where('property_history.portal', $request->input('platform'));
        //     }
        // }
        // if ($request->has('date_from') && $request->has('date_to')) {
        //     if (($request->date_from != '' && $request->date_to != '')) {

        //         $date_from = date('Y-m-d', strtotime($request->date_from));
        //         $date_to = date('Y-m-d', strtotime($request->date_to));
        //         $query->whereBetween('property_history.listed_date', [$date_from, $date_to]);
        //     }
        // }
        // if ($request->has('type')) {
        //     if (($request->type != '0' && $request->type != 'undefined')) {
        //         $query->where('property_history.property_type', $request->input('type'));
        //     }
        // }

        $data = $query->get();
        $totalData = count($data);
        $totalFiltered = $totalData;

        $limit = $request->length;
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (empty($request->input('search.value'))) {
            $list = $query->orderBy($order, $dir)->offset($start)->limit($limit)->get();
        } else {
            $search = $request->input('search.value');

            $totalFiltered = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)->get();

            $totalFiltered = count($totalFiltered);

            $list = $query->where(function ($q) use ($search) {
                $q->Where('property_history.address', 'like', '%'.$search.'%')
                ->orWhere('agency.name', 'like', '%'.$search.'%')
                ->orWhere('property_history.property_type', 'like', '%'.$search.'%')
                ->orWhere('outcode.outcode', 'like', '%'.$search.'%')
                ->orWhere('postcode.postcode', 'like', '%'.$search.'%')
                ->orWhere('property_history.price', 'like', '%'.$search.'%')
                ;
            })->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get()
            ;
        }

        // dd($list);

        $dataArr = [];
        if (!empty($list)) {
            $i = 1;
            foreach ($list as $item) {
                $encrpt_id = Helper::encryptData($item->id);
                if ('' != $item->latitude && '' != $item->longitude) {
                    $map_url = "https://www.google.com/maps?q={$item->latitude},{$item->longitude}";
                } else {
                    $map_url = '';
                }

                $nestedData['outcode'] = $item->outcode;

                if (0 == $item->price) {
                    $nestedData['price'] = 'POA';
                } else {
                    $nestedData['price'] = '&pound;'.number_format($item->price);
                }

                if ($item->listed_date) {
                    $listed_date = date('Y-m-d', strtotime($item->listed_date));
                    $today = date('Y-m-d');
                    $days = (strtotime($today) - strtotime($listed_date)) / (60 * 60 * 24);
                    $nestedData['listed_date'] = date('d M Y', strtotime($item->listed_date)).'<div style="width: 82px;">Days - '.$days.'</div>';
                } else {
                    $nestedData['listed_date'] = 'Hidden from public';
                }

                $nestedData['agency'] = $item->name;

                // $url = route('propertyWithoutPostcodeDetails', ['property_id' => $encrpt_id]);

                $nestedData['outcode'] = $item->outcode;
                $nestedData['property_number'] = '<div class="input-group">
                <input type="text" id="inp_'.$i.'" name="property_number" class="form-control" placeholder="Property Number">
                <div class="input-group-append">
                <button data-id="'.$i.'"  type="button"  title="Submit Property Number" class="btn btn-primary addPropNumber" prop_id="'.$item->id.'"><i class="flaticon-paper-plane icon-lg"></i></button>
                </div>
                </div>';

                $address2 = $item->address;

                $nestedData['address'] = $address2;

                $nestedData['type'] = $item->title.' <br> '.ucfirst($item->property_type);

                if (0 == $item->portal) {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='rightmove' class='btn btn-rightmove btn-sm' href='{$item->portal_url}'><i class='icon-lg'>R</i></a>";
                } else {
                    $nestedData['url'] = "<a title='Click here to see the listing' target='zoopla' class='btn btn-zoopla btn-sm' href='{$item->portal_url}'><i class='icon-lg'>Z</i></a>";
                }

                ++$i;

                $dataArr[] = $nestedData;
            }
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => intval($totalFiltered),
            'data' => $dataArr,
            'outcode' => $outcodes,
            'query' => $query->get(),
        ];

        return json_encode($json_data);
    }

    public function getProbateData()
    {
     //$user_outcode = $this->userOutcode();
        if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
            $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
            $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
        }
        $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
        $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
        $user_outcode = $selOutcodeCollection;

        $query = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
        ->join("postcode", "postcode.id", "probate_property.postcode_id")
        ->join("outcode", "outcode.id", "postcode.outcode_id")
        ->leftjoin("probate_property_flag", "probate_property_flag.probate_id", "probate_property.id")
        ->whereIn('outcode.id', $user_outcode)
        ->where('probate_property.status',1)
        ->whereDate('publication_date', '>', Carbon::now()->subDays(530));
        $data = $query
        ->select('postcode.postcode','outcode.outcode','probate_property.address_1','probate_property.address_2','probate_property.firstname','probate_property.surname','executor.company_name','executor.first_name','executor.sur_name','probate_property.publication_date','probate_property.claim_date','probate_property.id')
        ->get();


    return Datatables::of($data)

  ->editColumn('outcode', function ($model) {
   return $model->postcode;
 })
  ->editColumn('address', function ($model) {
$pid = Crypt::encrypt($model->id);
    return '<a href="' . url('property-detail?pid=' . $pid ) . '" title="View">'.$model->address_1.' '.$model->address_2.'</a>';
 })  
      ->editColumn('executer', function ($model) {
    $executor = $model->company_name ? $model->company_name : $model->first_name." ".$model->sur_name;
   return $executor;
  })
  ->editColumn('publication_date', function ($model) {
    $publicationDate =  date('d-m-Y', strtotime($model->publication_date));  
   return $publicationDate;
 })
   ->editColumn('claim_date', function ($model) {
    $claim_date =  date('d-m-Y', strtotime($model->claim_date));  
   return $claim_date;
 })
  

->editColumn('created_date', function ($model) {
    return 1;
})


  ->editColumn('action', function ($model) {
    $idG = Crypt::encrypt($model->id);
    // $user = \App\User::find($model->user_id);
    // Auth::loginUsingId($user->id, true);

            return '<a href="' . url('boat-for-sale/' . $model->slug ) . '" type="" target="_blank" class="act-sp" title="View"><i class="fas fa-file"></i></a>  <a href="' . url('edit-boat-info/' . $idG) . '" type="" target="_blank" class="act-sp" title="Login as user to edit"><i class="fas fa-edit"></i></a> ';
        })

        ->rawColumns(['s#','name','address','status','expiry','action'])
        ->make(true);
    }

    public function propertyDetail(Request $request)
    {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::where('status',1)->findOrFail($propId); 
            if (SESSION::has(config('propai.authSessionKey')) && json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode) {
                $selOutcode = json_decode(SESSION::get(config('propai.authSessionKey')))->user_selected_outcode;         
                $selectedOutcode = $selOutcode->outcode;
            //dd($selectedOutcode);
            }
            $selOutcodeId = $selectedOutcode && $selectedOutcode =='HA1' ? 4258 : 3682;      

       // $user_outcode = $this->userOutcode();
            $selOutcodeCollection = collect((object) [0 => $selOutcodeId]);
            $user_outcode = $selOutcodeCollection;
        // $probateProperties = ProbateProperty::where('status',1)->orderBy('id', 'DESC')->take(10)->get(['address_1','latitude','longitude','id'])->toArray();
            $probateProperties = ProbateProperty::join('executor','executor.id','probate_property.executor_id')
            ->join("postcode", "postcode.id", "probate_property.postcode_id")
            ->join("outcode", "outcode.id", "postcode.outcode_id")
            ->whereIn('outcode.id', $user_outcode)
            ->where('probate_property.status',1)->orderBy('probate_property.id', 'DESC')->take(10)->get(['probate_property.address_1','probate_property.latitude','probate_property.longitude','probate_property.id'])->toArray();
            return view('admin.probate-property-detail')->with(['probateProperty' => $probateProperty,'probateProperties' => $probateProperties]);
        }
    }
    public function getProbatePopupData(Request $request) {
        if($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            //$probateProperty = ProbateProperty::where('status',1)->findOrFail($propId);
            $probateProperty = ProbateProperty::join('executor', 'probate_property.executor_id','executor.id')
                ->join("postcode", "postcode.id", "probate_property.postcode_id")
                ->join("outcode", "outcode.id", "postcode.outcode_id")
                ->leftJoin("property_sale_history", "property_sale_history.property_history_id", "probate_property.id")
                ->select('probate_property.*', 'executor.company_name as executor_comapnyname', 'executor.first_name','executor.sur_name','executor.address_1 as executor_address1','executor.address_2 as executor_address2','executor.town as executor_town','executor.postcode as executor_postcode', 'executor.phone as executor_phone','executor.email as executor_email', 'postcode.postcode as postcode_postcode','property_sale_history.price as property_sale_price','property_sale_history.sale_date as property_sale_date','property_sale_history.sold_year as property_sold_year')
                ->findOrFail($propId);
            $str = $probateProperty->address_1;
            $addr1 = explode(" ",$str);
            $postcode = $probateProperty->postcode_postcode;
            $soldHistory = SoldHistory::where('house_no', $addr1[0])->where('postcode',$postcode)->get();
            $returnHTML = view('admin.popup-content',['probateProperty'=>$probateProperty, 'soldHistory' => $soldHistory])->render();

            return response()->json( array('success' => true, 'html'=>$returnHTML) );
        }
    }

    public function getProbateTooltipData(Request $request) {
        if ($request->get('pid')) {
            $propId = Crypt::decrypt($request->get('pid')); 
            $probateProperty = ProbateProperty::join('outcode','probate_property.outcode_id','outcode.id')               ->where('status',1)
                               ->findOrFail($propId); 
            $returnHTML = view('admin.tooltip-content',['probateProperty'=>$probateProperty])->render(); 

            return response()->json(array('success' => true, 'html'=>$returnHTML));
        }
    }

    public function getSchoolTooltipData(Request $request) {
        $sch_latitude = $request->latitude;
        $sch_longitude = $request->longitude;
        $prop_latitude = $request->prop_latitude;
        $prop_longitude = $request->prop_longitude;
        $radius = 1.5;
        $schools = School::selectRaw("*,
        ( 3956 * acos( cos( radians(?) ) *
        cos( radians( latitude ) )
        * cos( radians( longitude ) - radians(?)
        ) + sin( radians(?) ) *
        sin( radians( latitude ) ) )
        ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
        ->where([
        ['latitude',$sch_latitude],
        ['longitude',$sch_longitude]
        ])
        ->having('distance', '<=', $radius)->first();

        $returnHTML = view('admin.school-content',['schoolAddress'=>$schools])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getEpcDetails(Request $request) {
        $full_address = explode("|",$request->address);
        $address = trim(trim($full_address[0]," "),",");
        $outcode = trim($full_address[1]," ");
        $epc_url = 'https://epc.opendatacommunities.org/api/v1/domestic/search?postcode='.$outcode.'&address='.$address;
        $epc_client = new Client();
        $epc_results = $epc_client->get($epc_url, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Authorization' => 'Basic c2hhcnU2MzFAZ21haWwuY29tOmQwZjE1Y2ZjNDlmNDlkYzQ2NWQzZjMyYjdlZDMzYjU5OTk3Y2YzYmY='
            ]
        ]);

        $epc_status = $epc_results->getStatusCode();
        $epc_contents = json_decode($epc_results->getBody(), true);
        $epc_details = $epc_contents['rows'];
        $allEpc['data'] = $epc_details[0];
        $returnHTML = view('admin.epc-content',['epcDetails'=>$allEpc ])->render(); 
        return response()->json( array('success' => true, 'html'=>$returnHTML));
    }

    public function getSchoolFullDetails(Request $request) {
        $latitude = $request->sch_latitude;
        $longitude = $request->sch_longitude;

        $schoolDetails = School::where([
            ['latitude',$latitude],
            ['longitude',$longitude]
        ])->first();

        $returnHTML = view('admin.school-full-details',['schoolDetails'=>$schoolDetails])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function schoolCoordUpdate(Request $request) {
        $schools_count = SchoolList::count();
        $count = 0;
        DB::table('schools')->orderBy('urn')->where('latitude', null)->chunk(100, function ($list) {
            foreach ($list as $school) {
                $count =+ 1;
                $urn = $school->urn;
                $school_name = $school->schname;
                $school_latitude = $school->latitude;
                $school_longitude = $school->longitude;
                if ($school_latitude  == null && $school_longitude == null) {
                    $school_address = $school->schname.', '.$school->street.', '.$school->town.', '.$school->postcode;
                    $google_autocomplete = "https://maps.googleapis.com/maps/api/place/autocomplete/json?input=".$school_address."&key=".config('map.apiKey');
                    $autocomplete_client = new Client();
                    $autocomplete_result = $autocomplete_client->get($google_autocomplete, [
                        'verify' => false,
                        'headers' => [
                            'Accept' => 'application/json',
                        ]
                    ]);
                    $autocomplete_status = $autocomplete_result->getStatusCode();
                    $autocomplete_response = json_decode($autocomplete_result->getBody(), true);
                    foreach ($autocomplete_response['predictions'] as $autocomplete_res) {
                        $google_placeId = "https://maps.googleapis.com/maps/api/place/details/json?place_id=".$autocomplete_res['place_id']."&key=".config('map.apiKey');
                        $placeId_client = new Client();
                        $placeId_result = $placeId_client->get($google_placeId, [
                            'verify' => false,
                            'headers' => [
                                'Accept' => 'application/json',
                            ]
                        ]);
                        $placeId_status = $placeId_result->getStatusCode();
                        $placeId_response = json_decode($placeId_result->getBody(), true);
                        $school_location = $placeId_response['result']['geometry']['location'];
                        $lat = $school_location['latitude'];
                        $lng = $school_location['longitude'];
                        School::where("urn",$urn)
                            ->update(
                                array(
                                    "latitude" => $lat,
                                    "longitude" => $lng,
                                )
                            );
                    }
                }
            }
        });
        if ($schools_count == $count) {
            return response()->json(array('success' => true, 'message' => 'Finished'));
        } else {
            return response()->json(array('success' => false, 'message' => 'Error'));
        }
    }

    public function getSchoolList(Request $request) {
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $sc_phase[] = $request->phase;       
        $radius = 1.5;
        $schools = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)->where('rating','!=',null);

                // if (in_array("Primary", $sc_phase))
                // {
                //    $schools = $schools->orWhere('phase_of_education', 'Primary'); 
                // }
                //  if (in_array("Secondary", $sc_phase))
                // {
                //     dd($sc_phase);
                //    $schools = $schools->orWhere('phase_of_education', 'Secondary'); 
                // }

                 $schools =  $schools->get();
         
        return response()->json( array('success' => true, 'html'=>$schools));
    }

    public function nearestSchoolList(Request $request) {
        $post_string = explode(' ', $request->postcode);
        $prop_latitude = $request->latitude;
        $prop_longitude = $request->longitude;
        $radius = 1.5;
        $schools_list = School::selectRaw("*, 
                    ( 3956 * acos( cos( radians(?) ) *
                    cos( radians( latitude ) )
                    * cos( radians( longitude ) - radians(?)
                    ) + sin( radians(?) ) *
                    sin( radians( latitude ) ) )
                    ) AS distance", [$prop_latitude, $prop_longitude, $prop_latitude])
                    ->having('distance', '<=', $radius)
                    ->orderBy('distance', 'ASC')
                    ->get();
        
        $returnHTML = view('admin.nearest-school-list',['schoolsList'=>$schools_list])->render();

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPhotos(Request $request) {
        $returnHTML = view('admin.photos-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }
    
    public function getFloorPlan(Request $request) {
        $returnHTML = view('admin.floorPlan-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getPlanning(Request $request) {
        $returnHTML = view('admin.planning-content')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getSoldInStreet(Request $request) {
        $postcode = $request->postcode;
        $soldHistory = SoldHistory::select('sold_date','sold_price','tenure','house_no','addr2','addr5','addr6')->where('postcode',$postcode)->get();    
        $result = array();
         foreach ($soldHistory as $sold) {
         $result[$sold['house_no']][] = $sold;
         }
             dd($result);
        $returnHTML = view('admin.soldInStreet-content',['soldHistory'=>$soldHistory])->render(); 

        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getMarketData(Request $request) {
        $returnHTML = view('admin.marketData-content')->render(); 
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function getOfcom(Request $request) {
        $postcode = $request->postcode;
        $ofcomApi = "https://api-proxy.ofcom.org.uk/broadband/coverage/".str_replace(' ', '', $postcode);
        $ofcom_client = new Client();
        $ofcom_results = $ofcom_client->get($ofcomApi, [
            'verify' => false,
            'headers' => [
                'Accept' => 'application/json',
                'Ocp-Apim-Subscription-Key' => config('map.ofcomKey'),
            ]
        ]);
        $ofcom_status = $ofcom_results->getStatusCode();
        $ofcom_contents = json_decode($ofcom_results->getBody(), true);
        $ofcom_result = [];
        foreach ($ofcom_contents['Availability'] as $ofcom_results) {
            $ofcom_result['MaxBbPredictedDown'] = $ofcom_results['MaxBbPredictedDown'];
            $ofcom_result['MaxBbPredictedUp'] = $ofcom_results['MaxBbPredictedUp'];
            $ofcom_result['MaxSfbbPredictedDown'] = $ofcom_results['MaxSfbbPredictedDown'];
            $ofcom_result['MaxSfbbPredictedUp'] = $ofcom_results['MaxSfbbPredictedUp'];
            $ofcom_result['MaxUfbbPredictedDown'] = $ofcom_results['MaxUfbbPredictedDown'];
            $ofcom_result['MaxUfbbPredictedUp'] = $ofcom_results['MaxUfbbPredictedUp'];
            $ofcom_result['MaxPredictedDown'] = $ofcom_results['MaxPredictedDown'];
            $ofcom_result['MaxPredictedUp'] = $ofcom_results['MaxPredictedUp'];
            $ofcom_res['data'] = $ofcom_result;
        }
        $returnHTML = view('admin.ofcom-content',['ofcomDetails'=>$ofcom_res['data']])->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }

    public function documents(Request $request) {
        $returnHTML = view('admin.documents')->render();
        return response()->json(array('success' => true, 'html'=>$returnHTML));
    }
}